@@ -1,5 +1,5 @@
 /*
- * $Id: common.h 14064 2007-07-07 04:13:57Z cbiere $
+ * $Id: common.h 16301 2009-03-29 15:31:34Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -42,12 +42,13 @@
  * Constants
  */
 
+#define GTA_PRODUCT_NAME "gtk-gnutella"	/**< Normally "gtk-gnutella" */
 #define GTA_VERSION 0				/**< major version */
 #define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 4			/**< patch level or teeny version */
+#define GTA_PATCHLEVEL 6			/**< patch level or teeny version */
 #define GTA_REVISION "stable"		/**< unstable, beta, stable */
 #define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
-#define GTA_RELEASE "2007-07-07"	/**< ISO 8601 format YYYY-MM-DD */
+#define GTA_RELEASE "2009-03-29"	/**< ISO 8601 format YYYY-MM-DD */
 #define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
 
 #if defined(USE_GTK1)
@@ -397,6 +398,10 @@ typedef void (*GCallback) (void);
 #define OFF_T_MAX MAX_INT_VAL(off_t)
 #endif /* OFF_T_MAX */
 
+#ifndef SIZE_MAX
+#define SIZE_MAX MAX_INT_VAL(size_t)
+#endif
+
 /*
  * Other common macros.
  */
@@ -463,7 +468,7 @@ G_STMT_START {			\
  *
  * void my_memcpy(void *dst, const void *src, size_t n) NON_NULL_PARAM((1, 2));
  */
-#if HAVE_GCC(3, 3)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 3) 
 #define NON_NULL_PARAM(x) __attribute__((__nonnull__ x))
 #else /* GCC < 3.3 */
 #define NON_NULL_PARAM(x)
@@ -473,7 +478,7 @@ G_STMT_START {			\
  * This is the same G_GNUC_FORMAT() but for function pointers. Older versions
  * of GCC do not allow function attributes for function pointers.
  */
-#if HAVE_GCC(3, 0)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 0)
 #define PRINTF_FUNC_PTR(x, y) __attribute__((format(__printf__, (x), (y))))
 #else /* GCC < 3.0 */
 #define PRINTF_FUNC_PTR(x, y)
@@ -481,7 +486,7 @@ G_STMT_START {			\
 
 /* Functions using this attribute cause a warning if the returned
  * value is not used. */
-#if HAVE_GCC(3, 4)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 4)
 #define WARN_UNUSED_RESULT __attribute__((__warn_unused_result__))
 #else /* GCC < 3.4 */
 #define WARN_UNUSED_RESULT
@@ -489,7 +494,7 @@ G_STMT_START {			\
 
 /* Instructs the compiler to emit code for this function even if it is
  * or seems to be unused. */
-#if HAVE_GCC(3, 1)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 1)
 #define KEEP_FUNCTION __attribute__((__used__))
 #else /* GCC < 3.1 || !GCC */
 #define KEEP_FUNCTION
@@ -506,7 +511,7 @@ G_STMT_START {			\
 /* Functions using this attribute cause a warning if the variable
  * argument list does not contain a NULL pointer. */
 #ifndef G_GNUC_NULL_TERMINATED
-#if HAVE_GCC(4, 0)
+#if defined(HASATTRIBUTE) && HAVE_GCC(4, 0)
 #define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
 #else	/* GCC < 4 */
 #define G_GNUC_NULL_TERMINATED
@@ -527,20 +532,20 @@ G_STMT_START {			\
 #endif /* !G_LIKELY */
 
 #ifndef G_GNUC_MALLOC
-#if HAVE_GCC(3, 0)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 0)
 #define G_GNUC_MALLOC __attribute__((__malloc__))
 #else
 #define G_GNUC_MALLOC
 #endif	/* GCC >= 3.0 */
 #endif	/* G_GNUC_MALLOC */
 
-#if HAVE_GCC(3, 1)
+#if defined(HASATTRIBUTE) && HAVE_GCC(3, 1)
 #define ALWAYS_INLINE __attribute__((__always_inline__))
 #else
 #define ALWAYS_INLINE
 #endif	/* GCC >= 3.1 */
 
-#ifdef HAS_REGPARM
+#if defined(HASATTRIBUTE) && defined(HAS_REGPARM)
 #define REGPARM(n)	__attribute__((__regparm__((n))))
 #else
 #define REGPARM(n)
@@ -585,7 +590,7 @@ G_STMT_START {			\
 
 #define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
-#define GTA_BUILD					"$Revision: 14064 $"
+#define GTA_BUILD					"$Revision: 16301 $"
 
 /* The next two defines came from huge.h --- Emile */
 #define SHA1_BASE16_SIZE 	40		/**< 160 bits in base16 representation */
@@ -667,15 +672,10 @@ ngettext_(const gchar *msg1, const gchar *msg2, gulong n)
  */
 #define RCSID(x) \
 static KEEP_FUNCTION inline const char *	\
-get_rcsid(void)	\
+get_rcsid_string(void)	\
 {	\
 	static const char rcsid[] = "@(#) " x;	\
-	const char *s = rcsid;	\
-	while (*s != '\0') {	\
-		if (*s++ == '$')	\
-			break;	\
-	}	\
-	return s;	\
+	return rcsid;	\
 }
 
 #include "casts.h"
