@@ -1,5 +1,5 @@
 /*
- * $Id: bh_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: bh_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: bh_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
 #include "bh_upload.h"
 #include "share.h"
@@ -50,12 +50,12 @@ RCSID("$Id: bh_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
 #include "guid.h"
 #include "version.h"
 
-#include "if/core/hosts.h"
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
 #include "lib/array.h"
 #include "lib/header.h"
+#include "lib/gnet_host.h"
 #include "lib/misc.h"
 #include "lib/url.h"
 #include "lib/walloc.h"
@@ -94,11 +94,11 @@ enum bh_type {
 struct browse_host_upload {
 	struct special_upload special;	/**< vtable, MUST be first field */
 	enum bh_type type;		/**< Type of data to send back */
-	gint flags;				/**< Opening flags */
+	int flags;				/**< Opening flags */
 	txdrv_t *tx;			/**< The transmission stack */
-	gchar *w_buf;			/**< Used for dynamically wallocated buffer */
+	char *w_buf;			/**< Used for dynamically wallocated buffer */
 	size_t w_buf_size;		/**< Size of the wallocated buffer */
-	const gchar *b_data;	/**< Current data block */
+	const char *b_data;	/**< Current data block */
 	size_t b_offset;		/**< Offset in data block */
 	size_t b_size;			/**< Size of the data block */
 	guint file_index;		/**< Current file index (iterator) */
@@ -126,12 +126,13 @@ cast_to_browse_host_upload(struct special_upload *p)
  * @return The amount of bytes copied. Use this to advance ``dest''.
  */
 static inline size_t
-browse_host_read_data(struct browse_host_upload *bh, gchar *dest, size_t *size)
+browse_host_read_data(struct browse_host_upload *bh, char *dest, size_t *size)
 {
 	size_t len;
 
 	g_assert(NULL != size);
-	g_assert((ssize_t) bh->b_offset >= 0 && bh->b_offset <= bh->b_size);
+	g_assert(size_is_non_negative(bh->b_offset));
+	g_assert(bh->b_offset <= bh->b_size);
 	g_assert(bh->b_data != NULL);
 	g_assert(*size <= INT_MAX);
 
@@ -152,7 +153,7 @@ static inline void
 browse_host_next_state(struct browse_host_upload *bh, enum bh_state state)
 {
 	g_assert(NULL != bh);
-	g_assert((gint) state >= 0 && state < NUM_BH_STATES);
+	g_assert((int) state >= 0 && state < NUM_BH_STATES);
 	bh->w_buf = NULL;
 	bh->w_buf_size = 0;
 	bh->b_data = NULL;
@@ -180,22 +181,22 @@ static ssize_t
 browse_host_read_html(struct special_upload *ctx,
 	gpointer const dest, size_t size)
 {
-	static const gchar header[] =
+	static const char header[] =
 		"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\r\n"
 		"<html>\r\n"
 		"<head>\r\n"
 		"<title>Browse Host</title>\r\n"
 		"</head>\r\n"
 		"<body>\r\n";
-	static const gchar trailer[] = "</ul>\r\n</body>\r\n</html>\r\n";
+	static const char trailer[] = "</ul>\r\n</body>\r\n</html>\r\n";
 	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
-	gchar *p = dest;
+	char *p = dest;
 
 	g_assert(NULL != bh);
 	g_assert(NULL != dest);
 	g_assert(size <= INT_MAX);
 
-	g_assert((gint) bh->state >= 0 && bh->state < NUM_BH_STATES);
+	g_assert((int) bh->state >= 0 && bh->state < NUM_BH_STATES);
 	g_assert(bh->b_size <= INT_MAX);
 	g_assert(bh->b_offset <= bh->b_size);
 
@@ -214,7 +215,7 @@ browse_host_read_html(struct special_upload *ctx,
 		case BH_STATE_LIBRARY_INFO:
 			if (!bh->b_data) {
 				bh->w_buf_size = w_concat_strings(&bh->w_buf,
-					"<h1>gtk-gnutella</h1>\r\n"
+					"<h1>" GTA_PRODUCT_NAME "</h1>\r\n"
 					"<h3>", version_get_string(),
 				   	" sharing ",
 					uint64_to_string(shared_files_scanned()),
@@ -257,7 +258,7 @@ browse_host_read_html(struct special_upload *ctx,
 				const shared_file_t *sf;
 
 				bh->file_index++;
-				sf = shared_file(bh->file_index);
+				sf = shared_file_sorted(bh->file_index);
 				if (!sf) {
 				   	if (bh->file_index > shared_files_scanned())
 						browse_host_next_state(bh, BH_STATE_TRAILER);
@@ -265,14 +266,14 @@ browse_host_read_html(struct special_upload *ctx,
 				} else if (SHARE_REBUILDING == sf) {
 					browse_host_next_state(bh, BH_STATE_REBUILDING);
 				} else {
-					const gchar * const name_nfc = shared_file_name_nfc(sf);
+					const char * const name_nfc = shared_file_name_nfc(sf);
 					const filesize_t file_size = shared_file_size(sf);
 					size_t html_size;
-					gchar *html_name;
+					char *html_name;
 
 					{
-						const gchar *dir;
-						gchar *name;
+						const char *dir;
+						char *name;
 						
 						dir = shared_file_relative_path(sf);
 						if (dir) {
@@ -301,7 +302,7 @@ browse_host_read_html(struct special_upload *ctx,
 							"]</li>\r\n",
 							(void *) 0);
 					} else {
-						gchar *escaped;
+						char *escaped;
 
 						escaped = url_escape(name_nfc);
 						bh->w_buf_size = w_concat_strings(&bh->w_buf,
@@ -332,7 +333,7 @@ browse_host_read_html(struct special_upload *ctx,
 
 		case BH_STATE_REBUILDING:
 			if (!bh->b_data) {
-				static const gchar msg[] =
+				static const char msg[] =
 					"<li>"
 						"<b>"
 							"The library is currently being rebuild. Please, "
@@ -392,7 +393,7 @@ browse_host_read_qhits(struct special_upload *ctx,
 {
 	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 	size_t remain = size;
-	gchar *p = dest;
+	char *p = dest;
 
 	/*
 	 * If we have no hit pending that we can send, build some more.
@@ -400,7 +401,7 @@ browse_host_read_qhits(struct special_upload *ctx,
 
 	if (NULL == bh->hits) {
 		GSList *files = NULL;
-		gint i;
+		int i;
 
 		for (i = 0; i < BH_SCAN_AHEAD; i++) {
 			const shared_file_t *sf;
@@ -408,7 +409,7 @@ browse_host_read_qhits(struct special_upload *ctx,
 			do {
 				/* Skip holes in indices */
 				bh->file_index++;
-				sf = shared_file(bh->file_index);
+				sf = shared_file_sorted(bh->file_index);
 			} while (NULL == sf && bh->file_index <= shared_files_scanned());
 
 			if (SHARE_REBUILDING == sf || NULL == sf)
@@ -427,7 +428,7 @@ browse_host_read_qhits(struct special_upload *ctx,
 		files = g_slist_reverse(files);			/* Preserve order */
 
 		qhit_build_results(files, i, BH_MAX_QHIT_SIZE,
-			browse_host_record_hit, bh, blank_guid, FALSE, &zero_array);
+			browse_host_record_hit, bh, &blank_guid, FALSE, &zero_array);
 
 		g_assert(bh->hits != NULL);		/* At least 1 hit enqueued */
 
@@ -442,7 +443,7 @@ browse_host_read_qhits(struct special_upload *ctx,
 
 	while (remain > 0 && NULL != bh->hits) {
 		pmsg_t *mb = bh->hits->data;
-		gint r;
+		int r;
 
 		r = pmsg_read(mb, p, remain);
 		p += r;
@@ -571,7 +572,7 @@ browse_host_open(
 	const struct tx_deflate_cb *deflate_cb,
 	const struct tx_link_cb *link_cb,
 	struct wrap_io *wio,
-	gint flags)
+	int flags)
 {
 	struct browse_host_upload *bh;
 
@@ -601,7 +602,7 @@ browse_host_open(
 
 		args.cb = link_cb;
 		args.wio = wio;
-		args.bws = BSCHED_BWS_OUT;
+		args.bws = bsched_out_select_by_addr(gnet_host_get_addr(host));
 
 		bh->tx = tx_make(owner, host, tx_link_get_ops(), &args);
 	}
