@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 15111 2008-02-28 23:17:05Z cbiere $
+ * $Id: bitzi.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -36,6 +36,13 @@
  * @note
  * The code requires libxml to parse the XML responses.
  *
+ * @bug FIXME: Due to the way the XML file is loaded and stored,
+ *      old tickets are not removed until they expire. If a
+ *      ticket is refreshed it is appended to the XML file, but
+ *      the old ticket is still at a lower offset, so that we
+ *      load the old ticket on startup but discard it when we
+ *      reach the newer ticket.
+ *
  * @author Alex Bennee <alex@bennee.com>
  * @date 2004
  */
@@ -68,12 +75,12 @@
  *
  */
 
-static const gchar bitzi_url_fmt[] = "http://ticket.bitzi.com/rdf/urn:sha1:%s";
+static const char bitzi_url_fmt[] = "http://ticket.bitzi.com/rdf/urn:sha1:%s";
 
 typedef struct {
 	const struct sha1 *sha1;			/**< binary SHA-1, atom */
 	filesize_t filesize;
-	gchar bitzi_url[SHA1_BASE32_SIZE + sizeof bitzi_url_fmt]; /**< request URL */
+	char bitzi_url[SHA1_BASE32_SIZE + sizeof bitzi_url_fmt]; /**< request URL */
 
 	/*
 	 * xml related bits
@@ -116,20 +123,21 @@ static void process_meta_data(bitzi_request_t * req);
 /* cache functions */
 static gboolean bitzi_cache_add(bitzi_data_t * data);
 static void bitzi_cache_remove(bitzi_data_t * data);
+static void bitzi_cache_remove_by_sha1(const struct sha1 *);
 static void bitzi_cache_clean(void);
 
 /* Get rid of the obnoxious (xmlChar *) */
-static inline gchar *
-xml_get_string(xmlNode *node, const gchar *id)
+static inline char *
+xml_get_string(xmlNode *node, const char *id)
 {
-	return (gchar *) xmlGetProp(node, (const xmlChar *) id);
+	return (char *) xmlGetProp(node, (const xmlChar *) id);
 }
 
 /**
  * Use this to free strings returned by xml_get_string().
  */
 static inline void
-xml_free_null(gchar **ptr)
+xml_free_null(char **ptr)
 {
 	g_assert(ptr);
 	if (*ptr) {
@@ -139,16 +147,16 @@ xml_free_null(gchar **ptr)
 }
 
 static inline const xmlChar *
-string_to_xmlChar(const gchar *s)
+string_to_xmlChar(const char *s)
 {
 	/* If we were pedantic, we'd verify that ``s'' is UTF-8 encoded */
 	return (const xmlChar *) s;
 }
 
-static inline const gchar *
+static inline const char *
 xmlChar_to_gchar(const xmlChar *s)
 {
-	return (const gchar *) s;
+	return (const char *) s;
 }
 
 /********************************************************************
@@ -201,9 +209,9 @@ bitzi_destroy(bitzi_data_t *data)
  * the parsing of the document tree and processes the ticket.
  */
 static void
-bitzi_host_data_ind(struct http_async *unused_handle, gchar *data, gint len)
+bitzi_host_data_ind(struct http_async *unused_handle, char *data, int len)
 {
-	gint result;
+	int result;
 
 	(void) unused_handle;
 
@@ -274,12 +282,13 @@ bitzi_host_error_ind(struct http_async *handle,
  */
 
 struct efj_t {
-	const gchar *string;
+	const char *string;
 	bitzi_fj_t judgement;
 };
 
 static const struct efj_t enum_fj_table[] = {
 	{ "Unknown",				BITZI_FJ_UNKNOWN },
+	{ "Unrated",				BITZI_FJ_UNRATED },
 	{ "Bitzi lookup failure",	BITZI_FJ_FAILURE },
 	{ "Filesize mismatch",		BITZI_FJ_WRONG_FILESIZE },
 	{ "Dangerous/Misleading",	BITZI_FJ_DANGEROUS_MISLEADING },
@@ -300,7 +309,7 @@ static const struct efj_t enum_fj_table[] = {
 static void
 process_rdf_description(xmlNode *node, bitzi_data_t *data)
 {
-	gchar *value;
+	char *value;
 
 	/*
 	 * We extract the urn:sha1 from the ticket as we may be processing
@@ -369,6 +378,8 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 				break;
 			}
 		}
+	} else {
+		data->judgement = BITZI_FJ_UNRATED;
 	}
 	xml_free_null(&value);
 
@@ -378,7 +389,7 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 
 	value = STRTRACK(xml_get_string(node, "fileLength"));
 	if (value) {
-		gint error;
+		int error;
 		data->size = parse_uint64(value, NULL, 10, &error);
 	}
 	xml_free_null(&value);
@@ -398,12 +409,12 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 		atom_str_change(&data->mime_type, value);
 
 		if (is_strcaseprefix(value, "video")) {
-			gchar *bitrate = STRTRACK(xml_get_string(node, "videoBitrate"));
-			gchar *fps = STRTRACK(xml_get_string(node, "videoFPS"));
-			gchar *height = STRTRACK(xml_get_string(node, "videoHeight"));
-			gchar *width = STRTRACK(xml_get_string(node, "videoWidth"));
+			char *bitrate = STRTRACK(xml_get_string(node, "videoBitrate"));
+			char *fps = STRTRACK(xml_get_string(node, "videoFPS"));
+			char *height = STRTRACK(xml_get_string(node, "videoHeight"));
+			char *width = STRTRACK(xml_get_string(node, "videoWidth"));
 			gboolean has_res = width && height;
-			gchar desc[256];
+			char desc[256];
 				
 			/*
 			 * format the mime details
@@ -430,15 +441,15 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 			xml_free_null(&height);
 			xml_free_null(&width);
 		} else if (is_strcaseprefix(value, "audio")) {
-			gchar *channels = STRTRACK(xml_get_string(node, "audioChannels"));
-			gchar *duration = STRTRACK(xml_get_string(node, "duration"));
-			gchar *kbps = STRTRACK(xml_get_string(node, "audioBitrate"));
-			gchar *srate = STRTRACK(xml_get_string(node, "audioSamplerate"));
+			char *channels = STRTRACK(xml_get_string(node, "audioChannels"));
+			char *duration = STRTRACK(xml_get_string(node, "duration"));
+			char *kbps = STRTRACK(xml_get_string(node, "audioBitrate"));
+			char *srate = STRTRACK(xml_get_string(node, "audioSamplerate"));
 			guint32 seconds;
-			gchar desc[256];
+			char desc[256];
 
 			if (duration) {
-				gint error;
+				int error;
 				seconds = parse_uint32(duration, NULL, 10, &error) / 1000;
 			} else {
 				seconds = 0;
@@ -468,7 +479,7 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 		xmlAttr *cur_attr;
 
 		for (cur_attr = node->properties; cur_attr; cur_attr = cur_attr->next) {
-			const gchar *name;
+			const char *name;
 		   
 			name = xmlChar_to_gchar(cur_attr->name);
 			value = STRTRACK(xml_get_string(node, name));
@@ -590,6 +601,7 @@ process_meta_data(bitzi_request_t *request)
 		bitzi_destroy(data);
 		goto finish;
 	}
+	bitzi_cache_remove_by_sha1(data->sha1);
 
 	if (request->sha1 && !sha1_eq(data->sha1, request->sha1)) {
 		if (GNET_PROPERTY(bitzi_debug))  {
@@ -717,7 +729,7 @@ do_metadata_query(bitzi_request_t *req)
  * Add the data entry to the cache and in expiry sorted date order to
  * the linked list.
  */
-static gint
+static int
 bitzi_date_compare(gconstpointer p, gconstpointer q)
 {
 	const bitzi_data_t *a = p, *b = q;
@@ -844,16 +856,30 @@ bitzi_has_cached_ticket(const struct sha1 *sha1)
 	return NULL != bitzi_query_cache_by_sha1(sha1);
 }
 
+static void
+bitzi_cache_remove_by_sha1(const struct sha1 *sha1)
+{
+	bitzi_data_t *data = bitzi_query_cache_by_sha1(sha1);
+	if (data) {
+		bitzi_cache_remove(data);
+	}
+}
+
 /**
  * A GUI/Bitzi API passes a pointer to the search type (currently only
  * urn:sha1), a pointer to a callback function and a user data
  * pointer.
  *
+ * @param sha1 The SHA-1 of the file.
+ * @param filesize The expected filesize.
+ * @param refresh If TRUE a fresh ticket is requested, otherwise a
+ *                cached ticket is used.
+ *
  * If no query succeds then the call back is never made, however we
  * should always get some sort of data back from the service.
  */
 bitzi_data_t *
-bitzi_query_by_sha1(const struct sha1 *sha1, filesize_t filesize)
+bitzi_query_by_sha1(const struct sha1 *sha1, filesize_t filesize, gboolean refresh)
 {
 	bitzi_data_t *data;
 	bitzi_request_t	*request;
@@ -865,7 +891,13 @@ bitzi_query_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 		if (GNET_PROPERTY(bitzi_debug)) {
 			g_message("bitzi_query_by_sha1: result already in cache");
 		}
+		if (refresh) {
+			bitzi_cache_remove(data);
+			data = NULL;
+		}
+	}
 
+	if (data) {
 		if (filesize && data->size != filesize) {
 			bitzi_failure(sha1, filesize,
 				data->size ? BITZI_FJ_WRONG_FILESIZE : BITZI_FJ_UNKNOWN);
@@ -887,7 +919,6 @@ bitzi_query_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 		request->filesize = filesize;
 		len = gm_snprintf(request->bitzi_url, sizeof request->bitzi_url,
 				bitzi_url_fmt, sha1_base32(sha1));
-		g_assert(len < sizeof request->bitzi_url);
 
 		bitzi_rq = g_slist_append(bitzi_rq, request);
 		if (GNET_PROPERTY(bitzi_debug)) {
@@ -908,16 +939,16 @@ bitzi_ticket_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 {
 	bitzi_data_t *data;
 	
-	data = bitzi_query_by_sha1(sha1, filesize);
-	return data ? data->ticket : NULL;
+	data = bitzi_query_cache_by_sha1(sha1);
+	return data && (!filesize || data->size == filesize) ? data->ticket : NULL;
 }
 
 static void
 bitzi_load_cache(void)
 {
 	FILE *old_data;
-	gint ticket_count = 0;
-	gchar *path, *oldpath;
+	int ticket_count = 0;
+	char *path, *oldpath;
 
 	/*
 	 * Rename the old file , overwritting stuff if we have to
@@ -954,7 +985,7 @@ bitzi_load_cache(void)
 	} else {
 		bitzi_request_t *request = NULL;
 		gboolean truncated = FALSE;
-		gchar tmp[1024];
+		char tmp[1024];
 
 		for (;;) {
 			static const char xml_prefix[] = "<?xml";
