@@ -1,5 +1,5 @@
 /*
- * $Id: bogons.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: bogons.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,13 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: bogons.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: bogons.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "bogons.h"
 #include "settings.h"
 
+#include "lib/ascii.h"
 #include "lib/file.h"
-#include "lib/misc.h"
 #include "lib/glib-missing.h"
 #include "lib/iprange.h"
 #include "lib/walloc.h"
@@ -52,10 +52,10 @@ RCSID("$Id: bogons.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static const gchar bogon[] = "bogon";
+static const char bogon[] = "bogon";
 
-static const gchar bogons_file[] = "bogons.txt";
-static const gchar bogons_what[] = "Bogus IP addresses";
+static const char bogons_file[] = "bogons.txt";
+static const char bogons_what[] = "Bogus IP addresses";
 
 static struct iprange_db *bogons_db; /**< The database of bogus CIDR ranges */
 
@@ -64,14 +64,14 @@ static struct iprange_db *bogons_db; /**< The database of bogus CIDR ranges */
  *
  * @returns the amount of entries loaded.
  */
-static gint
+static int
 bogons_load(FILE *f)
 {
-	gchar line[1024];
-	gchar *p;
+	char line[1024];
+	char *p;
 	guint32 ip, netmask;
 	int linenum = 0;
-	gint bits;
+	int bits;
 	iprange_err_t error;
 
 	bogons_db = iprange_new();
@@ -134,11 +134,11 @@ bogons_load(FILE *f)
  * addresses changed.
  */
 static void
-bogons_changed(const gchar *filename, gpointer unused_udata)
+bogons_changed(const char *filename, gpointer unused_udata)
 {
 	FILE *f;
-	gchar buf[80];
-	gint count;
+	char buf[80];
+	int count;
 
 	(void) unused_udata;
 
@@ -170,8 +170,8 @@ static void
 bogons_retrieve(void)
 {
 	FILE *f;
-	gint idx;
-	gchar *filename;
+	int idx;
+	char *filename;
 #ifndef OFFICIAL_BUILD
 	static file_path_t fp[3];
 #else
