@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: bsched.c 16129 2009-03-12 06:47:48Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bsched.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: bsched.c 16129 2009-03-12 06:47:48Z rmanfredi $")
 
 #include "bsched.h"
 #include "inet.h"
@@ -109,32 +109,33 @@ enum bsched_magic {
 
 struct bsched {
 	enum bsched_magic magic;
-	tm_t last_period;				/**< Last time we ran our period */
-	GList *sources;					/**< List of bio_source_t */
-	GSList *stealers;				/**< List of bsched_t stealing bw */
-	gchar *name;					/**< Name, for tracing purposes */
-	gint count;						/**< Amount of sources */
-	gint type;						/**< Scheduling type */
-	gint flags;						/**< Processing flags */
-	gint period;					/**< Fixed scheduling period, in ms */
-	gint min_period;				/**< Minimal period without correction */
-	gint max_period;				/**< Maximal period without correction */
-	gint period_ema;				/**< EMA of period, in ms */
-	gint bw_per_second;				/**< Configure bandwidth in bytes/sec */
-	gint bw_max;					/**< Max bandwidth per period */
-	gint bw_actual;					/**< Bandwidth used so far in period */
-	gint bw_last_period;			/**< Bandwidth used last period */
-	gint bw_last_capped;			/**< Bandwidth capped last period */
-	gint bw_slot;					/**< Basic per-source bandwidth lot */
-	gint bw_ema;					/**< EMA of bandwidth really used */
-	gint bw_stolen;					/**< Amount we stole this period */
-	gint bw_stolen_ema;				/**< EMA of stolen bandwidth */
-	gint bw_delta;					/**< Running diff of actual vs. theoric */
-	gint bw_unwritten;				/**< Data that we could not write */
-	gint bw_capped;					/**< Bandwidth we refused to sources */
-	gint last_used;					/**< Nb of active sources last period */
-	gint current_used;				/**< Nb of active sources this period */
-	gboolean looped;				/**< True when looped once over sources */
+	tm_t last_period;			/**< Last time we ran our period */
+	GList *sources;				/**< List of bio_source_t */
+	GSList *stealers;			/**< List of bsched_t stealing bw */
+	char *name;					/**< Name, for tracing purposes */
+	int count;					/**< Amount of sources */
+	int type;					/**< Scheduling type */
+	int flags;					/**< Processing flags */
+	int period;					/**< Fixed scheduling period, in ms */
+	int min_period;				/**< Minimal period without correction */
+	int max_period;				/**< Maximal period without correction */
+	int period_ema;				/**< EMA of period, in ms */
+	int bw_per_second;			/**< Configure bandwidth in bytes/sec */
+	int bw_max;					/**< Max bandwidth per period */
+	int bw_actual;				/**< Bandwidth used so far in period */
+	int bw_last_period;			/**< Bandwidth used last period */
+	int bw_last_capped;			/**< Bandwidth capped last period */
+	int bw_slot;				/**< Basic per-source bandwidth lot */
+	int bw_ema;					/**< EMA of bandwidth really used */
+	int bw_stolen;				/**< Amount we stole this period */
+	int bw_stolen_ema;			/**< EMA of stolen bandwidth */
+	int bw_delta;				/**< Running diff of actual vs. theoric */
+	int bw_unwritten;			/**< Data that we could not write */
+	int bw_capped;				/**< Bandwidth we refused to sources */
+	int last_used;				/**< Nb of active sources last period */
+	int current_used;			/**< Nb of active sources this period */
+	int bw_urgent;				/**< Urgent b/w required in stealing */
+	gboolean looped;			/**< True when looped once over sources */
 };
 
 /*
@@ -146,19 +147,20 @@ static bsched_t *bws_set[NUM_BSCHED_BWS];
 static GSList *bws_list = NULL;
 static GSList *bws_out_list = NULL;
 static GSList *bws_in_list = NULL;
-static gint bws_out_ema = 0;
-static gint bws_in_ema = 0;
+static int bws_out_ema = 0;
+static int bws_in_ema = 0;
 
 #define BW_SLOT_MIN		64	 /**< Minimum bandwidth/slot for realloc */
 
 #define BW_OUT_UP_MIN	8192 /**< Minimum out bandwidth for becoming ultra */
+#define BW_OUT_DHT_MIN	8192 /**< Minimum out bandwidth for joining the DHT */
 #define BW_OUT_GNET_MIN	128	 /**< Minimum out bandwidth per Gnet connection */
 #define BW_OUT_LEAF_MIN	32	 /**< Minimum out bandwidth per leaf connection */
 
 #define BW_TCP_MSG		40	 /**< Smallest size of a TCP message */
 #define BW_UDP_MSG		28	 /**< Minimal IP+UDP overhead for a UDP message */
 
-#define BW_UDP_OVERSIZE	512	 /**< Allow that many bytes over available b/w */
+#define BW_UDP_OVERSIZE	1024 /**< Allow that many bytes over available b/w */
 
 static inline void
 bsched_check(const bsched_t * const bs)
@@ -184,8 +186,8 @@ bio_check(const bio_source_t * const bio)
  * @param `period' is the scheduling period in ms.
  */
 static bsched_t *
-bsched_make(const gchar *name, gint type, guint32 mode,
-	gint bandwidth, gint period)
+bsched_make(const char *name, int type, guint32 mode,
+	int bandwidth, int period)
 {
 	bsched_t *bs;
 
@@ -210,7 +212,7 @@ bsched_make(const gchar *name, gint type, guint32 mode,
 	bs->max_period = period << 1;		/* 200% of nominal period */
 	bs->period_ema = period;
 	bs->bw_per_second = bandwidth;
-	bs->bw_max = (gint) (bandwidth / 1000.0 * period);
+	bs->bw_max = (int) (bandwidth / 1000.0 * period);
 
 	return bs;
 }
@@ -296,6 +298,26 @@ bsched_avg_pct(bsched_bws_t bws)
 	return bsched_avg_bps(bws) * 100 / (1 + bsched_bw_per_second(bws));
 }
 
+int
+bsched_urgent(bsched_bws_t bws)
+{
+	const bsched_t *bs = bsched_get(bws);
+	return bs->bw_urgent;
+}
+
+/**
+ * Record urgent bandwidth stealing needs that should be fulfilled to
+ * prevent any TX flow control situation.
+ */
+void
+bsched_set_urgent(bsched_bws_t bws, int amount)
+{
+	bsched_t *bs;
+
+	bs = bsched_get(bws);
+	bs->bw_urgent = amount;
+}
+
 /**
  * Add `stealer' as a bandwidth stealer for underused bandwidth in `bs'.
  * Both must be either reading or writing schedulers.
@@ -393,6 +415,10 @@ bsched_config_steal_gnet(void)
 	bsched_add_stealer(BSCHED_BWS_GOUT_UDP, BSCHED_BWS_GOUT);
 }
 
+/*
+ * This is called BEFORE settings_init(), bandwidth values are the default
+ * values, not the configured ones.
+ */
 void
 bsched_early_init(void)
 {
@@ -415,11 +441,18 @@ bsched_early_init(void)
 		BS_T_STREAM, BS_F_READ, GNET_PROPERTY(bw_gnet_in) / 2, 1000);
 
 	bws_set[BSCHED_BWS_GIN_UDP] = bsched_make("G UDP in",
-		BS_T_STREAM, BS_F_READ, GNET_PROPERTY(bw_gnet_in) / 2, 1000);
+		BS_T_STREAM, BS_F_READ, 0, 1000);
 
 	bws_set[BSCHED_BWS_GLIN] = bsched_make("GL in",
 		BS_T_STREAM, BS_F_READ, GNET_PROPERTY(bw_gnet_lin), 1000);
 
+	bws_set[BSCHED_BWS_LOOPBACK_OUT] = bsched_make("loopback out",
+		BS_T_STREAM, BS_F_WRITE, 0, 1000);
+	bws_set[BSCHED_BWS_LOOPBACK_IN] = bsched_make("loopback in",
+		BS_T_STREAM, BS_F_READ, 0, 1000);
+
+	bws_list = g_slist_prepend(bws_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_IN));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_GLIN));
 	bws_list = g_slist_prepend(bws_list, 
@@ -428,6 +461,8 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_GIN_UDP));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_IN));
+	bws_list = g_slist_prepend(bws_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_OUT));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_GLOUT));
 	bws_list = g_slist_prepend(bws_list, 
@@ -437,6 +472,8 @@ bsched_early_init(void)
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_OUT));
 
+	bws_out_list = g_slist_prepend(bws_in_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_IN));
 	bws_in_list = g_slist_prepend(bws_in_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_GLIN));
 	bws_in_list = g_slist_prepend(bws_in_list, 
@@ -446,6 +483,8 @@ bsched_early_init(void)
 	bws_in_list = g_slist_prepend(bws_in_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_IN));
 
+	bws_out_list = g_slist_prepend(bws_out_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_OUT));
 	bws_out_list = g_slist_prepend(bws_out_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_GLOUT));
 	bws_out_list = g_slist_prepend(bws_out_list, 
@@ -456,12 +495,30 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_OUT));
 }
 
+/**
+ * Configure schedules with actual limits set by the user.
+ */
+static void
+bsched_correct_init(void)
+{
+	bsched_set_bandwidth(BSCHED_BWS_OUT, GNET_PROPERTY(bw_http_out));
+	bsched_set_bandwidth(BSCHED_BWS_GOUT, GNET_PROPERTY(bw_gnet_out) / 2);
+	bsched_set_bandwidth(BSCHED_BWS_GOUT_UDP, GNET_PROPERTY(bw_gnet_out) / 2);
+	bsched_set_bandwidth(BSCHED_BWS_GLOUT, GNET_PROPERTY(bw_gnet_lout));
+	bsched_set_bandwidth(BSCHED_BWS_IN, GNET_PROPERTY(bw_http_in));
+	bsched_set_bandwidth(BSCHED_BWS_GIN, GNET_PROPERTY(bw_gnet_in));
+	bsched_set_bandwidth(BSCHED_BWS_GIN_UDP, 0);
+	bsched_set_bandwidth(BSCHED_BWS_GLIN, GNET_PROPERTY(bw_gnet_lin));
+}
+
 /**
  * Initialize global bandwidth schedulers.
  */
 void
 bsched_init(void)
 {
+	bsched_correct_init();	/* Fix badnwidth using user-configured values */
+
 	/*
 	 * We always steal bandwidth between TCP and UDP gnet, since we
 	 * forcefully split the allocated bandwidth evenly between the
@@ -772,8 +829,8 @@ bsched_begin_timeslice(bsched_t *bs)
 {
 	GList *iter;
 	GList *last = NULL;
-	gdouble norm_factor;
-	gint count;
+	double norm_factor;
+	int count;
 
 	bsched_check(bs);
 
@@ -860,7 +917,7 @@ bsched_begin_timeslice(bsched_t *bs)
 	 */
 
 	if (bs->count) {
-		gint dividor = bs->count;
+		int dividor = bs->count;
 		if (bs->last_used > 0 && bs->last_used < bs->count)
 			dividor = bs->last_used + 1;
 		bs->bw_slot = (bs->bw_max + bs->bw_last_capped) / dividor;
@@ -1010,7 +1067,7 @@ bsched_source_remove(bio_source_t *bio)
  * On-the-fly changing of the allowed bandwidth.
  */
 void
-bsched_set_bandwidth(bsched_bws_t bws, gint bandwidth)
+bsched_set_bandwidth(bsched_bws_t bws, int bandwidth)
 {
 	bsched_t *bs;
 
@@ -1019,8 +1076,12 @@ bsched_set_bandwidth(bsched_bws_t bws, gint bandwidth)
 	g_assert(bandwidth >= 0);
 	g_assert(bandwidth <= BS_BW_MAX);	/* Signed, and multiplied by 1000 */
 
+	if (GNET_PROPERTY(dbg))
+		g_message("bsched_set_bandwidth: using %.2f KiB/s for %s",
+			(double) bandwidth / bs->period, bs->name);
+
 	bs->bw_per_second = bandwidth;
-	bs->bw_max = (gint) (bandwidth / 1000.0 * bs->period);
+	bs->bw_max = (int) (bandwidth / 1000.0 * bs->period);
 
 	/*
 	 * If `bandwidth' is 0, then we're disabling bandwidth scheduling and
@@ -1052,12 +1113,12 @@ bsched_set_bandwidth(bsched_bws_t bws, gint bandwidth)
  *
  * @returns the bandwidth available for a given source.
  */
-static gint
-bw_available(bio_source_t *bio, gint len)
+static int
+bw_available(bio_source_t *bio, int len)
 {
 	bsched_t *bs;
-	gint available;
-	gint result;
+	int available;
+	int result;
 	gboolean capped = FALSE;
 	gboolean used;
 	gboolean active;
@@ -1139,7 +1200,7 @@ bw_available(bio_source_t *bio, gint len)
 		available > BW_SLOT_MIN &&
 		active
 	) {
-		gint slot = available / bs->count;
+		int slot = available / bs->count;
 
 		/*
 		 * It's not worth redistributing less than BW_SLOT_MIN bytes per slot.
@@ -1204,11 +1265,11 @@ bw_available(bio_source_t *bio, gint len)
 		result < len && available > 0 && bs->looped &&
 		(!used || bs->bw_last_capped > 0)
 	) {
-		gint adj = len - result;
-		gint nominal;
+		int adj = len - result;
+		int nominal;
 
 		if (bs->bw_last_capped > 0 && bs->bw_last_period < bs->bw_max) {
-			gint distribute = MAX(bs->bw_last_capped, available);
+			int distribute = MAX(bs->bw_last_capped, available);
 
 			/*
 			 * We have capped bandwidth last period, yet we consumed less
@@ -1278,7 +1339,7 @@ bw_available(bio_source_t *bio, gint len)
  * @param `requested' is the amount of bytes requested for the I/O.
  */
 static void
-bsched_bw_update(bsched_t *bs, gint used, gint requested)
+bsched_bw_update(bsched_t *bs, int used, int requested)
 {
 	bsched_check(bs);		/* Ensure I/O source was in alive scheduler */
 	g_assert(used <= requested);
@@ -1344,7 +1405,7 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bio_write(wio=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	r = bio->wio->write(bio->wio, data, amount);
 
@@ -1358,7 +1419,7 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 
 	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("wio->write(fd=%d, len=%d) returned -1 with errno = 0, "
-			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
+			"assuming EAGAIN", bio->wio->fd(bio->wio), (int) len);
 		errno = VAL_EAGAIN;
 	}
 
@@ -1377,13 +1438,13 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
  * errno set to EAGAIN.
  */
 ssize_t
-bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
+bio_writev(bio_source_t *bio, struct iovec *iov, int iovcnt)
 {
 	size_t available;
 	ssize_t r;
 	size_t len;
 	struct iovec *siov;
-	gint slen = -1;			/* Avoid "may be used uninitialized" warning */
+	int slen = -1;			/* Avoid "may be used uninitialized" warning */
 
 	bio_check(bio);
 	g_assert(bio->flags & BIO_F_WRITE);
@@ -1456,7 +1517,7 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bio_writev(fd=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	if (iovcnt > MAX_IOV_COUNT)
 		r = safe_writev(bio->wio, iov, iovcnt);
@@ -1473,7 +1534,7 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 
 	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("writev(fd=%d, len=%d) returned -1 with errno = 0, "
-			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
+			"assuming EAGAIN", bio->wio->fd(bio->wio), (int) len);
 		errno = VAL_EAGAIN;
 	}
 
@@ -1532,7 +1593,7 @@ bio_sendto(bio_source_t *bio, const gnet_host_t *to,
 
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bio_sendto(wio=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	g_assert(bio->wio != NULL);
 	g_assert(bio->wio->sendto != NULL);
@@ -1548,7 +1609,7 @@ bio_sendto(bio_source_t *bio, const gnet_host_t *to,
 
 	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("wio->sendto(fd=%d, len=%d) returned -1 with errno = 0, "
-			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
+			"assuming EAGAIN", bio->wio->fd(bio->wio), (int) len);
 		errno = VAL_EAGAIN;
 	}
 
@@ -1609,7 +1670,7 @@ signal_handler(int signo)
  * bandwidth constraints.
  */
 ssize_t
-bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
+bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd, off_t *offset,
 	size_t len)
 {
 #if !defined(USE_MMAP) && !defined(HAS_SENDFILE)
@@ -1637,7 +1698,7 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
 	volatile size_t amount;
 	size_t available;
 	ssize_t r;
-	gint out_fd;
+	int out_fd;
 	off_t start;
 
 	g_assert(ctx);
@@ -1669,13 +1730,13 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
 
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bsched_write(fd=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 #ifdef USE_MMAP
 	{
 		static gboolean first_call = TRUE;
-		const gchar *data;
-		gint n;
+		const char *data;
+		int n;
 
 		if (first_call) {
 			first_call = FALSE;
@@ -1715,7 +1776,7 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
 
 			if (
 				map_len < min_map_size &&
-				map_start + min_map_size > map_start
+				MAX_INT_VAL(off_t) - map_start >= min_map_size
 			) {
 				map_len = min_map_size;
 			}
@@ -1832,7 +1893,7 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
 		g_warning("FIXED SENDFILE returned offset: "
 			"was set to %s instead of %s (%d byte%s written)",
 			uint64_to_string(*offset), uint64_to_string2(start + r),
-			(gint) r, r == 1 ? "" : "s");
+			(int) r, r == 1 ? "" : "s");
 		*offset = start + r;
 	} else if ((ssize_t) -1 == r) {
 		*offset = start;	/* Paranoid: in case sendfile() touched it */
@@ -1881,7 +1942,7 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
 	amount = len > available ? available : len;
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bsched_read(fd=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	r = bio->wio->read(bio->wio, data, amount);
 	if (r > 0) {
@@ -1902,13 +1963,13 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
  * errno set to EAGAIN.
  */
 ssize_t
-bio_readv(bio_source_t *bio, struct iovec *iov, gint iovcnt)
+bio_readv(bio_source_t *bio, struct iovec *iov, int iovcnt)
 {
 	size_t available;
 	ssize_t r;
 	size_t len;
 	struct iovec *siov;
-	gint slen = -1;			/* Avoid "may be used uninitialized" warning */
+	int slen = -1;			/* Avoid "may be used uninitialized" warning */
 
 	bio_check(bio);
 	g_assert(bio->flags & BIO_F_READ);
@@ -1981,7 +2042,7 @@ bio_readv(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 
 	if (GNET_PROPERTY(dbg) > 7)
 		printf("bio_readv(fd=%d, len=%d) available=%d\n",
-			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	if (iovcnt > MAX_IOV_COUNT)
 		r = safe_readv(bio->wio, iov, iovcnt);
@@ -1998,7 +2059,7 @@ bio_readv(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 
 	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("readv(fd=%d, len=%d) returned -1 with errno = 0, "
-			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
+			"assuming EAGAIN", bio->wio->fd(bio->wio), (int) len);
 		errno = VAL_EAGAIN;
 	}
 
@@ -2079,9 +2140,9 @@ bws_read(bsched_bws_t bws, wrap_io_t *wio, gpointer data, size_t len)
  * Account for read data from UDP.
  */
 void
-bws_udp_count_read(gint len)
+bws_udp_count_read(int len)
 {
-	gint count = BW_UDP_MSG + len;
+	int count = BW_UDP_MSG + len;
 	bsched_t *bs;
 
 	bs = bsched_get(BSCHED_BWS_GIN_UDP);
@@ -2093,9 +2154,9 @@ bws_udp_count_read(gint len)
  * Account for written data to UDP.
  */
 void
-bws_udp_count_written(gint len)
+bws_udp_count_written(int len)
 {
-	gint count = BW_UDP_MSG + len;
+	int count = BW_UDP_MSG + len;
 	bsched_t *bs;
 
 	bs = bsched_get(BSCHED_BWS_GOUT_UDP);
@@ -2272,7 +2333,7 @@ bws_can_connect(enum socket_type type)
 {
 	bsched_t *bsout = bs_socket(SOCK_CONN_OUTGOING, type);
 	bsched_t *bsin = bs_socket(SOCK_CONN_INCOMING, type);
-	gint available;
+	int available;
 
 	if (bsout != NULL && (bsout->flags & BS_F_ENABLED)) {
 		if (bsout->flags & BS_F_NOBW)				/* No more bandwidth */
@@ -2312,13 +2373,13 @@ static void
 bsched_heartbeat(bsched_t *bs, tm_t *tv)
 {
 	GList *iter;
-	gint delay;
-	gint overused;
-	gint theoric;
-	gint correction;
-	gint last_bw_max;
-	gint last_capped;
-	gint last_used;
+	int delay;
+	int overused;
+	int theoric;
+	int correction;
+	int last_bw_max;
+	int last_capped;
+	int last_used;
 
 	bsched_check(bs);
 
@@ -2326,13 +2387,13 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 	 * How much time elapsed since last call?
 	 */
 
-	delay = (gint) ((tv->tv_sec - bs->last_period.tv_sec) * 1000 +
+	delay = (int) ((tv->tv_sec - bs->last_period.tv_sec) * 1000 +
 		(tv->tv_usec - bs->last_period.tv_usec) / 1000);
 
 	if (GNET_PROPERTY(dbg) > 9)
 		printf("[%s] tv = %d,%d  bs = %d,%d, delay = %d\n",
-			bs->name, (gint) tv->tv_sec, (gint) tv->tv_usec,
-			(gint) bs->last_period.tv_sec, (gint) bs->last_period.tv_usec,
+			bs->name, (int) tv->tv_sec, (int) tv->tv_usec,
+			(int) bs->last_period.tv_sec, (int) bs->last_period.tv_usec,
 			delay);
 
 	/*
@@ -2399,13 +2460,13 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 	last_bw_max = bs->bw_max;
 	last_capped = bs->bw_capped;
 
-	theoric = (gint) (bs->bw_per_second / 1000.0 * delay);
+	theoric = (int) (bs->bw_per_second / 1000.0 * delay);
 	overused = bs->bw_actual - theoric;
 	bs->bw_delta += overused;
 
 	overused -= bs->bw_stolen;		/* Correct for computations below */
 
-	bs->bw_max = (gint) (bs->bw_per_second / 1000.0 * bs->period_ema);
+	bs->bw_max = (int) (bs->bw_per_second / 1000.0 * bs->period_ema);
 
 	/*
 	 * We correct the bandwidth for the next slot.
@@ -2508,10 +2569,10 @@ bsched_stealbeat(bsched_t *bs)
 {
 	GSList *l;
 	GSList *all_used = NULL;		/* List of bsched_t that used all b/w */
-	gint all_used_count = 0;		/* Amount of bsched_t that used all b/w */
+	int all_used_count = 0;			/* Amount of bsched_t that used all b/w */
 	guint all_bw_count = 0;			/* Sum of configured bandwidth */
-	gint steal_count = 0;
-	gint underused;
+	int steal_count = 0;
+	int underused;
 
 	bsched_check(bs);
 	g_assert(bs->bw_actual == 0);	/* Heartbeat step must have been done */
@@ -2558,7 +2619,7 @@ bsched_stealbeat(bsched_t *bs)
 	 */
 
 	if (bs->flags & BS_F_WRITE) {
-		gint half_contribution = bs->count ? bs->bw_max / (2 * bs->count) : 0;
+		int half_contribution = bs->count ? bs->bw_max / (2 * bs->count) : 0;
 		GList *bl;
 
 		for (bl = bs->sources; bl && underused > 0; bl = g_list_next(bl)) {
@@ -2587,6 +2648,29 @@ bsched_stealbeat(bsched_t *bs)
 			all_used_count++;
 			all_bw_count += xbs->bw_max;
 		}
+
+		/*
+		 * If a stealer has an urgent need for bandwidth, distribute as
+		 * much as we can to it before considering others.  If more than
+		 * one stealer has urgent need, the order in which it is distributed
+		 * is the reverse order in which stealers were added (i.e. the last
+		 * addition is viewed as more prioritary).
+		 */
+
+		if (xbs->bw_urgent > 0) {
+			int amount = MIN(underused, xbs->bw_urgent);
+			xbs->bw_stolen += amount;
+			xbs->bw_urgent -= amount;
+			underused -= amount;
+
+			if (GNET_PROPERTY(dbg) > 4)
+				printf("b/w sched \"%s\" urgently giving %d bytes to \"%s\""
+					" (still wants %d bytes urgently)\n",
+					bs->name, amount, xbs->name, xbs->bw_urgent);
+
+			if (underused <= 0)			/* Nothing left to redistribute */
+				return;
+		}
 	}
 
 	g_assert(steal_count > 0);
@@ -2609,21 +2693,21 @@ bsched_stealbeat(bsched_t *bs)
 	} else {
 		for (l = all_used; l; l = g_slist_next(l)) {
 			bsched_t *xbs = l->data;
-			gdouble amount;
+			double amount;
 
 			if (xbs->bw_max == 0)
 				continue;
 
-			amount = (gdouble) underused * (gdouble) xbs->bw_max / all_bw_count;
+			amount = (double) underused * (double) xbs->bw_max / all_bw_count;
 
-			if ((gdouble) xbs->bw_stolen + amount > (gdouble) BS_BW_MAX)
+			if ((double) xbs->bw_stolen + amount > (double) BS_BW_MAX)
 				xbs->bw_stolen = BS_BW_MAX;
 			else
-				xbs->bw_stolen += (gint) amount;
+				xbs->bw_stolen += (int) amount;
 
 			if (GNET_PROPERTY(dbg) > 4)
 				printf("b/w sched \"%s\" giving %d bytes to \"%s\"\n",
-					bs->name, (gint) amount, xbs->name);
+					bs->name, (int) amount, xbs->name);
 		}
 		g_slist_free(all_used);
 	}
@@ -2637,8 +2721,8 @@ bsched_timer(void)
 {
 	tm_t tv;
 	GSList *l;
-	gint out_used = 0;
-	gint in_used = 0;
+	int out_used = 0;
+	int in_used = 0;
 	gboolean read_data = FALSE;
 
 	tm_now(&tv);
@@ -2678,7 +2762,7 @@ bsched_timer(void)
 	for (l = bws_out_list; l; l = g_slist_next(l)) {
 		bsched_bws_t bws = GPOINTER_TO_UINT(l->data);
 		bsched_t *bs = bsched_get(bws);
-		out_used += (gint) (bs->bw_last_period * 1000.0 / bs->period_ema);
+		out_used += (int) (bs->bw_last_period * 1000.0 / bs->period_ema);
 	}
 
 	bws_out_ema += (out_used >> 6) - (bws_out_ema >> 6);	/* Slow EMA */
@@ -2690,7 +2774,7 @@ bsched_timer(void)
 		bsched_bws_t bws = GPOINTER_TO_UINT(l->data);
 		bsched_t *bs = bsched_get(bws);
 
-		in_used += (gint) (bs->bw_last_period * 1000.0 / bs->period_ema);
+		in_used += (int) (bs->bw_last_period * 1000.0 / bs->period_ema);
 
 		if (bs->flags & BS_F_DATA_READ) {
 			read_data = TRUE;
@@ -2715,7 +2799,7 @@ bsched_timer(void)
 }
 
 static gboolean
-true_expr(const gchar *expr)
+true_expr(const char *expr)
 {
 	if (GNET_PROPERTY(dbg) > 0) {
 		g_message("%s", expr);
@@ -2793,4 +2877,19 @@ bsched_enough_up_bandwidth(void)
 	return TRUE;
 }
 
+
+bsched_bws_t
+bsched_out_select_by_addr(const host_addr_t addr)
+{
+	return host_addr_is_loopback(addr)
+			? BSCHED_BWS_LOOPBACK_OUT : BSCHED_BWS_OUT;
+}
+
+bsched_bws_t
+bsched_in_select_by_addr(const host_addr_t addr)
+{
+	return host_addr_is_loopback(addr)
+			? BSCHED_BWS_LOOPBACK_IN : BSCHED_BWS_IN;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
