@@ -1,5 +1,5 @@
 /*
- * $Id: clock.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: clock.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: clock.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: clock.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #include "clock.h"
 
@@ -60,7 +60,7 @@ RCSID("$Id: clock.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
 struct used_val {
 	host_addr_t addr;			/**< The IP address */
 	cevent_t *cq_ev;			/**< Scheduled cleanup event */
-	gint precision;				/**< The precision used for the last update */
+	int precision;				/**< The precision used for the last update */
 };
 
 static GHashTable *used;		/**< Records the IP address used */
@@ -86,7 +86,7 @@ static GHashTable *used;		/**< Records the IP address used */
  * local time is only used when the host is not running NTP.  Otherwise,
  * we compute the skew just for the fun of it.
  */
-static struct statx *datapoints;
+static statx_t *datapoints;
 
 /**
  * Dispose of the value from the `used' table.
@@ -122,7 +122,7 @@ val_destroy(cqueue_t *unused_cq, gpointer obj)
  * Create a value for the `used' table.
  */
 static struct used_val *
-val_create(const host_addr_t addr, gint precision)
+val_create(const host_addr_t addr, int precision)
 {
 	struct used_val *v = walloc(sizeof *v);
 
@@ -138,7 +138,7 @@ val_create(const host_addr_t addr, gint precision)
  * expiration timeout.
  */
 static void
-val_reused(struct used_val *v, gint precision)
+val_reused(struct used_val *v, int precision)
 {
 	v->precision = precision;
 	cq_resched(callout_queue, v->cq_ev, REUSE_DELAY * 1000);
@@ -181,14 +181,14 @@ clock_close(void)
 static void
 clock_adjust(void)
 {
-	gint n;
-	gdouble avg;
-	gdouble sdev;
-	gdouble min;
-	gdouble max;
-	gint i;
+	int n;
+	double avg;
+	double sdev;
+	double min;
+	double max;
+	int i;
 	guint32 new_skew;
-	gint k;
+	int k;
 
 	/*
 	 * Compute average and standard deviation using all the data points.
@@ -203,7 +203,7 @@ clock_adjust(void)
 	 */
 
 	for (k = 0; k < CLEAN_STEPS; k++) {
-		gdouble *value = statx_data(datapoints);
+		double *value = statx_data(datapoints);
 
 		if (GNET_PROPERTY(dbg) > 1)
 			printf("CLOCK before #%d: n=%d avg=%.2f sdev=%.2f\n",
@@ -220,13 +220,13 @@ clock_adjust(void)
 		max = avg + sdev;
 
 		for (i = 0; i < n; i++) {
-			gdouble v = value[i];
+			double v = value[i];
 			if (v < min || v > max)
 				continue;
 			statx_add(datapoints, v);
 		}
 
-		g_free(value);
+		G_FREE_NULL(value);
 
 		/*
 		 * Recompute the new average using the "sound" points we kept.
@@ -276,7 +276,7 @@ clock_adjust(void)
  * REUSE_DELAY seconds.
  */
 void
-clock_update(time_t update, gint precision, const host_addr_t addr)
+clock_update(time_t update, int precision, const host_addr_t addr)
 {
 	time_t now;
 	gint32 delta;
@@ -305,8 +305,8 @@ clock_update(time_t update, gint precision, const host_addr_t addr)
 	now = tm_time();
 	delta = delta_time(update, (now + (gint32) GNET_PROPERTY(clock_skew)));
 
-	statx_add(datapoints, (gdouble) (delta + precision));
-	statx_add(datapoints, (gdouble) (delta - precision));
+	statx_add(datapoints, (double) (delta + precision));
+	statx_add(datapoints, (double) (delta - precision));
 
 	if (GNET_PROPERTY(dbg) > 1)
 		printf("CLOCK skew=%d delta=%d +/-%d [%s] (n=%d avg=%.2f sdev=%.2f)\n",
