@@ -1,7 +1,7 @@
 /*
- * $Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $
+ * $Id: dmesh.c 16283 2009-03-28 21:03:49Z cbiere $
  *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -30,12 +30,12 @@
  * Download mesh.
  *
  * @author Raphael Manfredi
- * @date 2002-2007
+ * @date 2002-2009
  */
 
 #include "common.h"
 
-RCSID("$Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $")
+RCSID("$Id: dmesh.c 16283 2009-03-28 21:03:49Z cbiere $")
 
 #include "gnutella.h"
 #include "downloads.h"
@@ -49,10 +49,11 @@ RCSID("$Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $")
 #include "fileinfo.h"
 #include "settings.h"
 #include "hosts.h"
-#include "g2_cache.h"
+#include "ipp_cache.h"
 
 #include "if/gnet_property_priv.h"
 
+#include "lib/ascii.h"
 #include "lib/atoms.h"
 #include "lib/base32.h"
 #include "lib/cq.h"
@@ -62,6 +63,7 @@ RCSID("$Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $")
 #include "lib/glib-missing.h"
 #include "lib/hashlist.h"
 #include "lib/header.h"
+#include "lib/strtok.h"
 #include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/urn.h"
@@ -81,8 +83,8 @@ static GHashTable *mesh = NULL;
 struct dmesh {				/**< A download mesh bucket */
 	list_t *entries;		/**< The download mesh entries, dmesh_entry data */
 	GHashTable *by_host;	/**< Entries indexed by host */
-	time_t last_update;		/**< Timestamp of last insertion in the mesh */
-	const struct sha1 *sha1;	/**< For tracing, the SHA1 of this mesh */
+	time_t last_update;		/**< Timestamp of last insert/expire in the mesh */
+	const struct sha1 *sha1;	/**< The SHA1 of this mesh */
 };
 
 struct dmesh_entry {
@@ -93,14 +95,15 @@ struct dmesh_entry {
 	gboolean good;			/**< Whether marked as being a good entry */
 };
 
-#define MAX_LIFETIME	86400		/**< 1 day */
+#define MAX_LIFETIME	43200		/**< half a day */
+#define MAX_LIBLIFETIME	3600		/**< 1 hour for shared/seeded files */
 #define MAX_ENTRIES		256			/**< Max amount of entries kept per SHA1 */
 
 #define MIN_PFSP_SIZE	524288		/**< 512K, min size for PFSP advertising */
 #define MIN_PFSP_PCT	10			/**< 10%, min available data for PFSP */
 #define MIN_BAD_REPORT	2			/**< Don't ban before that many X-Nalt */
 
-static const gchar dmesh_file[] = "dmesh";
+static const char dmesh_file[] = "dmesh";
 
 /**
  * If we get a "bad" URL into the mesh ("bad" = gives 404 or other error when
@@ -122,7 +125,7 @@ struct dmesh_banned {
 	dmesh_urlinfo_t *info;	/**< The banned URL (same as key) */
 	cevent_t *cq_ev;		/**< Scheduled callout event */
 	const struct sha1 *sha1;/**< The SHA1, if any */
-	time_t ctime;			/**< Last time we saw this banned URL */
+	time_t created;			/**< Last time we saw this banned URL */
 };
 
 typedef void (*dmesh_add_cb)(
@@ -135,11 +138,11 @@ static GHashTable *ban_mesh_by_sha1 = NULL;
 
 #define BAN_LIFETIME	7200		/**< 2 hours */
 
-static const gchar dmesh_ban_file[] = "dmesh_ban";
+static const char dmesh_ban_file[] = "dmesh_ban";
 
 static void dmesh_retrieve(void);
 static void dmesh_ban_retrieve(void);
-static gchar *dmesh_urlinfo_to_string(const dmesh_urlinfo_t *info);
+static char *dmesh_urlinfo_to_string(const dmesh_urlinfo_t *info);
 
 /**
  * Hash a URL info.
@@ -161,7 +164,7 @@ urlinfo_hash(gconstpointer key)
 /**
  * Test equality of two URL infos.
  */
-static gint
+static int
 urlinfo_eq(gconstpointer a, gconstpointer b)
 {
 	const dmesh_urlinfo_t *ia = a, *ib = b;
@@ -213,10 +216,10 @@ dmesh_entry_free(struct dmesh_entry *dme)
 static void
 dmesh_fill_info(dmesh_urlinfo_t *info,
 	const struct sha1 *sha1, const host_addr_t addr,
-	guint16 port, guint idx, const gchar *name)
+	guint16 port, guint idx, const char *name)
 {
-	static const gchar urnsha1[] = "urn:sha1:";
-	static gchar urn[SHA1_BASE32_SIZE + sizeof urnsha1];
+	static const char urnsha1[] = "urn:sha1:";
+	static char urn[SHA1_BASE32_SIZE + sizeof urnsha1];
 
 	info->addr = addr;
 	info->port = port;
@@ -336,7 +339,7 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
 
 		dmb = walloc(sizeof *dmb);
 		dmb->info = ui;
-		dmb->ctime = stamp;
+		dmb->created = stamp;
 		dmb->cq_ev = cq_insert(callout_queue,
 			lifetime * 1000, dmesh_ban_expire, dmb);
 		dmb->sha1 = NULL;
@@ -370,8 +373,8 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
 					atom_sha1_get(sha1), by_addr);
 		}
 	}
-	else if (delta_time(dmb->ctime, stamp) < 0) {
-		dmb->ctime = stamp;
+	else if (delta_time(dmb->created, stamp) < 0) {
+		dmb->created = stamp;
 		cq_resched(callout_queue, dmb->cq_ev, lifetime * 1000);
 	}
 }
@@ -407,7 +410,7 @@ dmesh_is_banned(const dmesh_urlinfo_t *info)
  *** Mesh URL parsing.
  ***/
 
-static const gchar * const parse_errstr[] = {
+static const char * const parse_errstr[] = {
 	"OK",									/**< DMESH_URL_OK */
 	"HTTP parsing error",					/**< DMESH_URL_HTTP_PARSER */
 	"File prefix neither /uri-res nor /get",/**< DMESH_URL_BAD_FILE_PREFIX */
@@ -420,14 +423,14 @@ static const gchar * const parse_errstr[] = {
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *
+const char *
 dmesh_url_strerror(dmesh_url_error_t errnum)
 {
-	if ((gint) errnum < 0 || errnum >= G_N_ELEMENTS(parse_errstr))
+	if (UNSIGNED(errnum) >= G_N_ELEMENTS(parse_errstr))
 		return "Invalid error code";
 
 	if (errnum == DMESH_URL_HTTP_PARSER) {
-		static gchar http_error_str[128];
+		static char http_error_str[128];
 
 		concat_strings(http_error_str, sizeof http_error_str,
 			parse_errstr[errnum], ": ", http_url_strerror(http_url_errno),
@@ -445,12 +448,12 @@ dmesh_url_strerror(dmesh_url_error_t errnum)
  * The variable `dmesh_url_errno' is set accordingly.
  */
 gboolean
-dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
+dmesh_url_parse(const char *url, dmesh_urlinfo_t *info)
 {
 	host_addr_t addr;
 	guint16 port;
 	guint idx;
-	const gchar *endptr, *file, *host = NULL, *path = NULL;
+	const char *endptr, *file, *host = NULL, *path = NULL;
 
 	if (!http_url_parse(url, &port, &host, &path)) {
 		dmesh_url_errno = DMESH_URL_HTTP_PARSER;
@@ -471,7 +474,7 @@ dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
 	 */
 
 	if (NULL != (endptr = is_strprefix(path, "/get/"))) {
-		gint error;
+		int error;
 
 		idx = parse_uint32(endptr, &endptr, 10, &error);
 		if (!error && URN_INDEX == idx) {
@@ -517,7 +520,7 @@ dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
 	 */
 
 	if (idx != URN_INDEX) {
-		gchar *unescaped = url_unescape(deconstify_gchar(file), FALSE);
+		char *unescaped = url_unescape(deconstify_gchar(file), FALSE);
 		if (!unescaped) {
 			dmesh_url_errno = DMESH_URL_BAD_ENCODING;
 			return FALSE;
@@ -658,16 +661,47 @@ dm_remove(struct dmesh *dm, const host_addr_t addr, guint16 port)
 }
 
 /**
- * Expire entries older than `agemax' in a given mesh bucket `dm'.
+ * Is the SHA1 that of a finished file (either shared in the library or
+ * seeded after completion)?
+ */
+static gboolean
+sha1_of_finished_file(const struct sha1 *sha1)
+{
+	const struct shared_file *sf = shared_file_by_sha1(sha1);
+
+	return sf && sf != SHARE_REBUILDING && shared_file_is_finished(sf);
+}
+
+/**
+ * Compute suitable life time for mesh entries.
+ *
+ * Complete files have no download HTTP transactions, hence the alt-locs
+ * we get are only from uploaders.  To keep only the ones that are fresh-enough, 
+ * reduce the lifetime of each entry.
+ */
+static glong
+dm_lifetime(const struct dmesh *dm)
+{
+	g_assert(dm);
+	g_assert(dm->sha1);
+
+	return sha1_of_finished_file(dm->sha1) ? MAX_LIBLIFETIME : MAX_LIFETIME;
+}
+
+/**
+ * Expire entries deemed too old in a given mesh bucket `dm'.
  */
 static void
-dm_expire(struct dmesh *dm, glong agemax)
+dm_expire(struct dmesh *dm)
 {
 	GSList *expired = NULL;
 	GSList *sl;
 	time_t now = tm_time();
+	glong agemax;
 	list_iter_t *iter;
 
+	agemax = dm_lifetime(dm);
+
 	iter = list_iter_before_head(dm->entries);
 
 	while (list_iter_has_next(iter)) {
@@ -684,10 +718,10 @@ dm_expire(struct dmesh *dm, glong agemax)
 		 */
 
 		if (GNET_PROPERTY(dmesh_debug) > 4)
-			g_message("MESH %s: EXPIRED \"%s\", age=%d",
+			g_message("MESH %s: EXPIRED \"%s\", age=%u",
 				sha1_base32(dm->sha1),
 				dmesh_urlinfo_to_string(&dme->url),
-				(gint) delta_time(now, dme->stamp));
+				(unsigned) delta_time(now, dme->stamp));
 
 		expired = g_slist_prepend(expired, dme);
 	}
@@ -701,6 +735,8 @@ dm_expire(struct dmesh *dm, glong agemax)
 	}
 
 	g_slist_free(expired);
+
+	dm->last_update = tm_time();
 }
 
 /**
@@ -732,7 +768,7 @@ dmesh_dispose(const struct sha1 *sha1)
  */
 gboolean
 dmesh_remove(const struct sha1 *sha1, const host_addr_t addr, guint16 port,
-	guint idx, const gchar *name)
+	guint idx, const char *name)
 {
 	struct dmesh *dm;
 	dmesh_urlinfo_t info;
@@ -771,7 +807,7 @@ dmesh_remove(const struct sha1 *sha1, const host_addr_t addr, guint16 port,
  *
  * @return the number of dmesh entries
  */
-gint
+int
 dmesh_count(const struct sha1 *sha1)
 {
 	struct dmesh *dm;
@@ -803,9 +839,9 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 	host_addr_t addr = info->addr;
 	guint16 port = info->port;
 	guint idx = info->idx;
-	const gchar *name = info->name;
+	const char *name = info->name;
 	struct packed_host packed;
-	const gchar *reason = NULL;
+	const char *reason = NULL;
 
 	g_return_val_if_fail(sha1, FALSE);
 
@@ -818,7 +854,8 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 	}
 
 	/*
-	 * Reject if this is for our host, or if the host is a private/hostile IP.
+	 * Reject if this is for our host, or if the host is a private/hostile IP
+	 * or if the IP:port point back to one of our recent addresses.
 	 */
 
 	if (is_my_address_and_port(addr, port)) {
@@ -826,6 +863,11 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 		goto rejected;
 	}
 
+	if (local_addr_cache_lookup(addr, port)) {
+		reason = "recent own address and port";
+		goto rejected;
+	}
+
 	if (!host_is_valid(addr, port)) {
 		reason = "address/port not valid";
 		goto rejected;
@@ -860,7 +902,7 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 		dm = dm_alloc(sha1);
 		gm_hash_table_insert_const(mesh, atom_sha1_get(sha1), dm);
 	} else {
-		dm_expire(dm, MAX_LIFETIME);
+		dm_expire(dm);
 	}
 
 	/*
@@ -944,10 +986,10 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 
 rejected:
 	if (GNET_PROPERTY(dmesh_debug) > 4)
-		g_message("MESH %s: rejecting \"%s\", stamp=%u age=%d: %s",
+		g_message("MESH %s: rejecting \"%s\", stamp=%u age=%u: %s",
 			sha1_base32(sha1),
 			dmesh_urlinfo_to_string(info), (guint) stamp,
-			(gint) delta_time(now, stamp),
+			(unsigned) delta_time(now, stamp),
 			reason);
 
 	return FALSE;
@@ -958,7 +1000,7 @@ rejected:
  */
 gboolean
 dmesh_add(const struct sha1 *sha1, const host_addr_t addr,
-	guint16 port, guint idx, const gchar *name, time_t stamp)
+	guint16 port, guint idx, const char *name, time_t stamp)
 {
 	dmesh_urlinfo_t info;
 
@@ -998,7 +1040,7 @@ dmesh_add_good_alternate(const struct sha1 *sha1,
 void
 dmesh_add_alternates(const struct sha1 *sha1, const gnet_host_vec_t *alt)
 {
-	gint i;
+	int i;
 
 	for (i = gnet_host_vec_count(alt) - 1; i >= 0; i--) {
 		struct gnutella_host host;
@@ -1164,12 +1206,12 @@ retry:
  * contains a "," character.
  */
 static size_t
-dmesh_urlinfo(const dmesh_urlinfo_t *info, gchar *buf,
+dmesh_urlinfo(const dmesh_urlinfo_t *info, char *buf,
 	size_t len, gboolean *quoting)
 {
 	size_t rw;
 	size_t maxslen = len - 1;			/* Account for trailing NUL */
-	const gchar *host;
+	const char *host;
 
 	g_assert(len > 0);
 	g_assert(len <= INT_MAX);
@@ -1194,7 +1236,7 @@ dmesh_urlinfo(const dmesh_urlinfo_t *info, gchar *buf,
 		 */
 
 		if (rw < maxslen) {
-			gint re = url_escape_into(info->name, &buf[rw], len - rw);
+			int re = url_escape_into(info->name, &buf[rw], len - rw);
 
 			if (re < 0)
 				return (size_t) -1;
@@ -1220,10 +1262,10 @@ dmesh_urlinfo(const dmesh_urlinfo_t *info, gchar *buf,
 /**
  * Format the `info' URL and return pointer to static string.
  */
-static gchar *
+static char *
 dmesh_urlinfo_to_string(const dmesh_urlinfo_t *info)
 {
-	static gchar urlstr[1024];
+	static char urlstr[1024];
 
 	(void) dmesh_urlinfo(info, urlstr, sizeof urlstr, NULL);
 
@@ -1239,10 +1281,10 @@ dmesh_urlinfo_to_string(const dmesh_urlinfo_t *info)
  * URN_INDEX kind).
  */
 static size_t
-dmesh_entry_compact(const struct dmesh_entry *dme, gchar *buf, size_t size)
+dmesh_entry_compact(const struct dmesh_entry *dme, char *buf, size_t size)
 {
 	const dmesh_urlinfo_t *info = &dme->url;
-	const gchar *host;
+	const char *host;
 	size_t rw;
 
 	g_assert(size > 0);
@@ -1267,7 +1309,7 @@ dmesh_entry_compact(const struct dmesh_entry *dme, gchar *buf, size_t size)
  * the buffer.
  */
 static size_t
-dmesh_entry_url_stamp(const struct dmesh_entry *dme, gchar *buf, size_t size)
+dmesh_entry_url_stamp(const struct dmesh_entry *dme, char *buf, size_t size)
 {
 	size_t rw;
 	gboolean quoting;
@@ -1313,10 +1355,10 @@ dmesh_entry_url_stamp(const struct dmesh_entry *dme, gchar *buf, size_t size)
  *
  * @return pointer to static string.
  */
-static const gchar *
+static const char *
 dmesh_entry_to_string(const struct dmesh_entry *dme)
 {
-	static gchar urlstr[1024];
+	static char urlstr[1024];
 
 	(void) dmesh_entry_url_stamp(dme, urlstr, sizeof urlstr);
 	return urlstr;
@@ -1328,14 +1370,15 @@ dmesh_entry_to_string(const struct dmesh_entry *dme)
  *
  * @return the amount of locations filled.
  */
-gint
-dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
+int
+dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, int hcnt)
 {
 	struct dmesh *dm;
 	struct dmesh_entry *selected[MAX_ENTRIES];
-	gint nselected;
-	gint i;
-	gint j;
+	int nselected;
+	int i;
+	int j;
+	gboolean complete_file;
 	list_iter_t *iter;
 
 	/*
@@ -1351,6 +1394,7 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 	 */
 
 	i = 0;
+	complete_file = sha1_of_finished_file(sha1);
 	iter = list_iter_before_head(dm->entries);
 
 	while (list_iter_has_next(iter)) {
@@ -1359,13 +1403,19 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 		if (dme->url.idx != URN_INDEX)
 			continue;
 
-#if 0	/* XXX not yet */
-		if (!dme->good)
-			continue;			/* Only propagate good alt locs */
-#else
-		if (dme->bad)			/* Don't progagate with negative feedback */
-			continue;
-#endif
+		/*
+		 * When downloading (i.e. when the file is not complete), we have the
+		 * neceesary feedback to spot good sources.  When sharing a complete
+		 * file, all we can do is skip entries for which we got bad feedback.
+		 */
+
+		if (complete_file) {
+			if (dme->bad)		/* Skip entries with negative feedback */
+				continue;
+		} else {
+			if (!dme->good)
+				continue;		/* Only propagate good alt locs */
+		}
 
 		if (NET_TYPE_IPV4 != host_addr_net(dme->url.addr))
 			continue;
@@ -1373,6 +1423,9 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 		if (g2_cache_lookup(dme->url.addr, dme->url.port))
 			continue;			/* Don't pollute with G2-only entries */
 
+		if (local_addr_cache_lookup(dme->url.addr, dme->url.port))
+			continue;			/* Don't pollute with our recent addresses */
+
 		g_assert(i < MAX_ENTRIES);
 		selected[i++] = dme;
 	}
@@ -1383,7 +1436,7 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 	if (nselected == 0)
 		return 0;
 
-	g_assert(nselected <= (gint) list_length(dm->entries));
+	g_assert(UNSIGNED(nselected) <= list_length(dm->entries));
 
 	/*
 	 * Second pass: choose at most `hcnt' entries at random.
@@ -1391,10 +1444,10 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 
 	for (i = j = 0; i < nselected && j < hcnt; i++) {
 		struct dmesh_entry *dme;
-		gint nleft = nselected - i;
-		gint npick = random_value(nleft - 1);
-		gint k;
-		gint n;
+		int nleft = nselected - i;
+		int npick = random_value(nleft - 1);
+		int k;
+		int n;
 
 		/*
 		 * The `npick' variable is the index of the selected entry, all
@@ -1455,34 +1508,33 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
  *
  * @return amount of generated data.
  */
-gint
+int
 dmesh_alternate_location(const struct sha1 *sha1,
-	gchar *buf, size_t size, const host_addr_t addr,
-	time_t last_sent, const gchar *vendor,
+	char *buf, size_t size, const host_addr_t addr,
+	time_t last_sent, const char *vendor,
 	fileinfo_t *fi, gboolean request)
 {
-	gchar url[1024];
+	char url[1024];
 	struct dmesh *dm;
 	size_t len = 0;
 	GSList *l;
-	gint nselected = 0;
+	int nselected = 0;
 	struct dmesh_entry *selected[MAX_ENTRIES];
-	gint i;
-	size_t maxslen;			/* Account for trailing NUL + "\r\n" */
+	int i;
 	GSList *by_addr;
 	size_t maxlinelen = 0;
-	gpointer fmt;
+	header_fmt_t *fmt;
 	gboolean added;
 	list_iter_t *iter;
+	gboolean complete_file;
 
 	g_assert(sha1);
 	g_assert(buf);
-	g_assert((gint) size >= 0);
+	g_assert(size_is_non_negative(size));
 	g_assert(size <= INT_MAX);
 
-	if (size <= 3)
+	if (size <= 3)		/* Account for trailing NUL + "\r\n" */
 		return 0;
-	maxslen = size - 3;		/* Account for trailing NUL + "\r\n" */
 
 	/*
 	 * Shall we emit continuations?
@@ -1525,7 +1577,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	by_addr = g_hash_table_lookup(ban_mesh_by_sha1, sha1);
 
 	if (by_addr != NULL) {
-		fmt = header_fmt_make("X-Nalt", ", ", size);
+		fmt = header_fmt_make("X-Nalt", ", ", size, size / 3);
 		if (maxlinelen)
 			header_fmt_set_line_length(fmt, maxlinelen);
 		added = FALSE;
@@ -1538,15 +1590,12 @@ dmesh_alternate_location(const struct sha1 *sha1,
 			if (info->idx != URN_INDEX)
 				continue;
 
-			if (delta_time(banned->ctime, last_sent) > 0) {
-				const gchar *value;
+			if (delta_time(banned->created, last_sent) > 0) {
+				const char *value;
 
 				value = host_addr_port_to_string(info->addr, info->port);
-
-				if (!header_fmt_value_fits(fmt, strlen(value), size / 3))
+				if (!header_fmt_append_value(fmt, value))
 					break;
-
-				header_fmt_append_value(fmt, value);
 				added = TRUE;
 			}
 		}
@@ -1571,11 +1620,11 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	 * Start filling the buffer.
 	 */
 
-	fmt = header_fmt_make("X-Alt", ", ", size);
+	/* `len' is non-zero if X-Nalt was generated */
+	fmt = header_fmt_make("X-Alt", ", ", size - len, size - len);
 	if (maxlinelen)
 		header_fmt_set_line_length(fmt, maxlinelen);
 	added = FALSE;
-	maxslen -= len;		/* `len' is non-zero if X-Nalt was generated */
 
 	/*
 	 * PFSP-server: If we have to list ourselves in the mesh, do so
@@ -1615,14 +1664,14 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		url_len = dmesh_entry_compact(&ourselves, url, sizeof url);
 		g_assert((size_t) -1 != url_len && url_len < sizeof url);
 
-		if (!header_fmt_value_fits(fmt, url_len + strlen(tls_hex), maxslen))
+		if (!header_fmt_value_fits(fmt, url_len + strlen(tls_hex)))
 			goto nomore;
 
 		if (tls_enabled()) {
 			header_fmt_append_value(fmt, tls_hex);
 		}
-		header_fmt_append_value(fmt, url);
-		added = TRUE;
+		if (header_fmt_append_value(fmt, url))
+			added = TRUE;
 	}
 
 	/*
@@ -1632,14 +1681,14 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	if (dm == NULL)						/* SHA1 unknown */
 		goto nomore;
 
-	if (delta_time(dm->last_update, last_sent) <= 0)	/* No new insertion */
+	if (delta_time(dm->last_update, last_sent) <= 0)	/* No change occurred */
 		goto nomore;
 
 	/*
 	 * Expire old entries.  If none remain, free entry and return.
 	 */
 
-	dm_expire(dm, MAX_LIFETIME);
+	dm_expire(dm);
 
 	if (list_length(dm->entries) == 0) {
 		dmesh_dispose(sha1);
@@ -1661,17 +1710,24 @@ dmesh_alternate_location(const struct sha1 *sha1,
 
 	i = 0;
 	iter = list_iter_before_head(dm->entries);
+	complete_file = sha1_of_finished_file(sha1);
 
 	while (list_iter_has_next(iter)) {
 		struct dmesh_entry *dme = list_iter_next(iter);
 
-#if 0	/* XXX not yet */
-		if (!dme->good)			/* Only report sources we've checked */
-			continue;
-#else
-		if (dme->bad)			/* Don't progagate with negative feedback */
-			continue;
-#endif
+		/*
+		 * When downloading (i.e. when the file is not complete), we have the
+		 * neceesary feedback to spot good sources.  When sharing a complete
+		 * file, all we can do is skip entries for which we got bad feedback.
+		 */
+
+		if (complete_file) {
+			if (dme->bad)		/* Skip entries with negative feedback */
+				continue;
+		} else {
+			if (!dme->good)
+				continue;		/* Only propagate good alt locs */
+		}
 
 		if (delta_time(dme->inserted, last_sent) <= 0)
 			continue;
@@ -1685,6 +1741,9 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		if (g2_cache_lookup(dme->url.addr, dme->url.port))
 			continue;			/* Don't pollute with G2-only entries */
 
+		if (local_addr_cache_lookup(dme->url.addr, dme->url.port))
+			continue;			/* Don't pollute with our recent addresses */
+
 		g_assert(i < MAX_ENTRIES);
 
 		selected[i++] = dme;
@@ -1696,7 +1755,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	if (nselected == 0)
 		goto nomore;
 
-	g_assert(nselected <= (gint) list_length(dm->entries));
+	g_assert(UNSIGNED(nselected) <= list_length(dm->entries));
 
 	/*
 	 * Second pass.
@@ -1704,10 +1763,10 @@ dmesh_alternate_location(const struct sha1 *sha1,
 
 	for (i = 0; i < nselected; i++) {
 		struct dmesh_entry *dme;
-		gint nleft = nselected - i;
-		gint npick = random_value(nleft - 1);
-		gint j;
-		gint n;
+		int nleft = nselected - i;
+		int npick = random_value(nleft - 1);
+		int j;
+		int n;
 		size_t url_len;
 
 		/*
@@ -1736,11 +1795,8 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		/* Buffer was large enough */
 		g_assert((size_t) -1 != url_len && url_len < sizeof url);
 
-		if (!header_fmt_value_fits(fmt, url_len, maxslen))
-			continue;
-
-		header_fmt_append_value(fmt, url);
-		added = TRUE;
+		if (header_fmt_append_value(fmt, url))
+			added = TRUE;
 	}
 
 nomore:
@@ -1760,37 +1816,31 @@ nomore:
 }
 
 /**
- * Parse the value of the X-Gnutella-Content-URN header in `value', looking
+ * Parse the value of the X-(Gnutella-)Content-URN header in `value', looking
  * for a SHA1.  When found, the SHA1 is extracted and placed into the given
  * `digest' buffer.
  *
  * @return whether we successfully extracted the SHA1.
  */
 gboolean
-dmesh_collect_sha1(const gchar *value, struct sha1 *sha1)
+dmesh_collect_sha1(const char *value, struct sha1 *sha1)
 {
-	const gchar *p;
+	strtok_t *st;
+	const char *tok;
+	gboolean found = FALSE;
 
-	for (p = value; NULL != p && '\0' != *p; /* NOTHING */) {
+	st = strtok_make_strip(value);
 
-		/*
-		 * Skip leading spaces, if any.
-		 */
-
-		p = skip_ascii_spaces(p);
-		if (urn_get_sha1(p, sha1))
-			return TRUE;
-
-		/*
-		 * Advance past the next ',', if any.
-		 */
-
-		p = strchr(p, ',');
-		if (p)
-			p++;
+	while ((tok = strtok_next(st, ","))) {
+		if (urn_get_sha1(tok, sha1)) {
+			found = TRUE;
+			break;
+		}
 	}
 
-	return FALSE;
+	strtok_free(st);
+
+	return found;
 }
 
 /**
@@ -1804,16 +1854,16 @@ dmesh_collect_sha1(const gchar *value, struct sha1 *sha1)
  * where udata is opaque user-supplied data.
  */
 static void
-dmesh_parse_addr_port_list(const struct sha1 *sha1, const gchar *value,
+dmesh_parse_addr_port_list(const struct sha1 *sha1, const char *value,
 	dmesh_add_cb func, gpointer udata)
 {
-	const gchar *tls_hex, *p, *next;
+	const char *tls_hex, *p, *next;
 
 	tls_hex = NULL;
 	next = value;
 
 	while (NULL != (p = next)) {
-		const gchar *start, *endptr;
+		const char *start, *endptr;
 		host_addr_t addr;
 		guint16 port;
 		gboolean ok;
@@ -1837,7 +1887,7 @@ dmesh_parse_addr_port_list(const struct sha1 *sha1, const gchar *value,
 		 */	
 		ok = string_to_host_addr(start, &endptr, &addr);
 		if (ok && ':' == *endptr) {
-			gint error;
+			int error;
 				
 			port = parse_uint16(&endptr[1], &endptr, 10, &error);
 			ok = !error && port > 0; 
@@ -1867,7 +1917,7 @@ dmesh_collect_compact_locations_cback(
  * for a given SHA1 key given in the new compact form.
  */
 void
-dmesh_collect_compact_locations(const struct sha1 *sha1, const gchar *value)
+dmesh_collect_compact_locations(const struct sha1 *sha1, const char *value)
 {
 	dmesh_parse_addr_port_list(sha1, value,
 		dmesh_collect_compact_locations_cback, NULL);
@@ -1891,7 +1941,7 @@ dmesh_collect_negative_locations_cback(
  */
 void
 dmesh_collect_negative_locations(
-	const struct sha1 *sha1, const gchar *value, host_addr_t reporter)
+	const struct sha1 *sha1, const char *value, host_addr_t reporter)
 {
 	dmesh_parse_addr_port_list(sha1, value,
 		dmesh_collect_negative_locations_cback, &reporter);
@@ -1902,15 +1952,15 @@ dmesh_collect_negative_locations(
  * sources for a given SHA1 key.
  */
 void
-dmesh_collect_locations(const struct sha1 *sha1, const gchar *value)
+dmesh_collect_locations(const struct sha1 *sha1, const char *value)
 {
-	const gchar *p = value;
+	const char *p = value;
 	guchar c;
 	time_t now = tm_time();
 	gboolean finished = FALSE;
 
 	do {
-		const gchar *date_start, *url_start;
+		const char *date_start, *url_start;
 		time_t stamp;
 		gboolean ok;
 		dmesh_urlinfo_t info;
@@ -1996,7 +2046,7 @@ dmesh_collect_locations(const struct sha1 *sha1, const gchar *value)
 		 */
 
 		{
-			gchar *url;
+			char *url;
 
 			url = g_strndup(url_start, p - url_start);
 			ok = dmesh_url_parse(url, &info);
@@ -2068,7 +2118,7 @@ dmesh_collect_locations(const struct sha1 *sha1, const gchar *value)
 		 */
 
 		if (p != date_start) {
-			gchar *date;
+			char *date;
 
 			g_assert((guchar) *p == c);
 			date = g_strndup(date_start, p - date_start);
@@ -2118,11 +2168,11 @@ dmesh_collect_locations(const struct sha1 *sha1, const gchar *value)
 
 	skip_add:
 		if (GNET_PROPERTY(dmesh_debug) > 4)
-			g_message("MESH %s: %s \"%s\", stamp=%u age=%d",
+			g_message("MESH %s: %s \"%s\", stamp=%u age=%u",
 				sha1_base32(sha1),
 				ok ? "added" : "rejected",
 				dmesh_urlinfo_to_string(&info), (guint) stamp,
-				(gint) delta_time(now, stamp));
+				(unsigned) delta_time(now, stamp));
 
 		if (c == '\0')				/* Reached end of string */
 			finished = TRUE;
@@ -2141,12 +2191,12 @@ dmesh_collect_locations(const struct sha1 *sha1, const gchar *value)
  *
  * @returns the amount of locations inserted.
  */
-static gint
-dmesh_alt_loc_fill(const struct sha1 *sha1, dmesh_urlinfo_t *buf, gint count)
+static int
+dmesh_alt_loc_fill(const struct sha1 *sha1, dmesh_urlinfo_t *buf, int count)
 {
 	struct dmesh *dm;
 	list_iter_t *iter;
-	gint i;
+	int i;
 
 	g_assert(sha1);
 	g_assert(buf);
@@ -2221,7 +2271,7 @@ dmesh_check_results_set(gnet_results_set_t *rs)
 
 			if (rc->alt_locs != NULL) {
 				gnet_host_vec_t *alt = rc->alt_locs;
-				gint i;
+				int i;
 
 				for (i = gnet_host_vec_count(alt) - 1; i >= 0; i--) {
 					struct gnutella_host host;
@@ -2257,7 +2307,7 @@ dmesh_multiple_downloads(const struct sha1 *sha1,
 	filesize_t size, fileinfo_t *fi)
 {
 	dmesh_urlinfo_t buffer[DMESH_MAX], *p;
-	gint n;
+	int n;
 	time_t now;
 
 	n = dmesh_alt_loc_fill(sha1, buffer, DMESH_MAX);
@@ -2267,7 +2317,7 @@ dmesh_multiple_downloads(const struct sha1 *sha1,
 	now = tm_time();
 
 	for (p = buffer; n > 0; n--, p++) {
-		const gchar *filename;
+		const char *filename;
 
 		if (GNET_PROPERTY(dmesh_debug) > 2)
 			g_message("ALT-LOC queuing from MESH: %s",
@@ -2279,7 +2329,7 @@ dmesh_multiple_downloads(const struct sha1 *sha1,
 			size,
 			p->addr,
 			p->port,
-			blank_guid,
+			&blank_guid,
 			NULL,	/* hostname */
 			sha1,
 			NULL,	/* TTH */
@@ -2325,7 +2375,7 @@ typedef void (*header_func_t)(FILE *out);
  * The storing callback for each item is `store_cb'.
  */
 static void
-dmesh_store_hash(const gchar *what, GHashTable *hash, const gchar *file,
+dmesh_store_hash(const char *what, GHashTable *hash, const char *file,
 	header_func_t header_cb, GHFunc store_cb)
 {
 	FILE *out;
@@ -2379,11 +2429,11 @@ static void
 dmesh_retrieve(void)
 {
 	FILE *f;
-	gchar tmp[4096];
+	char tmp[4096];
 	struct sha1 sha1;
 	gboolean has_sha1 = FALSE;
 	gboolean skip = FALSE, truncated = FALSE;
-	gint line = 0;
+	int line = 0;
 	file_path_t fp[1];
 
 	file_path_set(fp, settings_config_dir(), dmesh_file);
@@ -2460,7 +2510,7 @@ dmesh_ban_store_kv(gpointer key, gpointer value, gpointer udata)
 	g_assert(key == dmb->info);
 
 	fprintf(out, "%lu %s\n",
-		(gulong) dmb->ctime, dmesh_urlinfo_to_string(dmb->info));
+		(gulong) dmb->created, dmesh_urlinfo_to_string(dmb->info));
 }
 
 /**
@@ -2495,11 +2545,11 @@ static void
 dmesh_ban_retrieve(void)
 {
 	FILE *in;
-	gchar tmp[1024];
-	gint line = 0;
+	char tmp[1024];
+	int line = 0;
 	time_t stamp;
-	const gchar *p;
-	gint error;
+	const char *p;
+	int error;
 	dmesh_urlinfo_t info;
 	file_path_t fp;
 
