@@ -1,5 +1,5 @@
 /*
- * $Id: dq.c 14719 2007-08-31 01:55:05Z cbiere $
+ * $Id: dq.c 16169 2009-03-15 18:23:47Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,16 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: dq.c 14719 2007-08-31 01:55:05Z cbiere $")
+RCSID("$Id: dq.c 16169 2009-03-15 18:23:47Z rmanfredi $")
 
 #ifdef I_MATH
 #include <math.h>	/* For pow() */
 #endif	/* I_MATH */
 
 #include "dq.h"
-#include "mq.h"
 #include "gmsg.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "nodes.h"
 #include "gnet_stats.h"
@@ -65,6 +63,7 @@ RCSID("$Id: dq.c 14719 2007-08-31 01:55:05Z cbiere $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/pmsg.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 
@@ -112,8 +111,12 @@ RCSID("$Id: dq.c 14719 2007-08-31 01:55:05Z cbiere $")
 struct next_up {
 	node_id_t node_id;		/**< Selected node ID */
 	query_hashvec_t *qhv;	/**< Query hash vector for the query */
-	gint can_route;			/**< -1 = unknown, otherwise TRUE / FALSE */
-	gint queue_pending;		/**< -1 = unknown, otherwise cached queue size */
+	int can_route;			/**< -1 = unknown, otherwise TRUE / FALSE */
+	int queue_pending;		/**< -1 = unknown, otherwise cached queue size */
+};
+
+struct dquery_id {
+	guint64 value;
 };
 
 typedef enum {
@@ -125,14 +128,14 @@ typedef enum {
  */
 typedef struct dquery {
 	dquery_magic_t magic;
-	node_id_t node_id;		/**< ID of the node that originated the query */
-	guint32 qid;			/**< Unique query ID, to detect ghosts */
 	guint32 flags;			/**< Operational flags */
+	node_id_t node_id;		/**< ID of the node that originated the query */
+	struct dquery_id qid;	/**< Unique query ID, to detect ghosts */
 	gnet_search_t sh;		/**< Search handle, if node ID = NODE_ID_SELF */
 	pmsg_t *mb;				/**< The search messsage "template" */
 	query_hashvec_t *qhv;	/**< Query hash vector for QRP filtering */
 	GHashTable *queried;	/**< Contains node IDs that we queried so far */
-	const gchar *lmuid;		/**< For proxied query: the original leaf MUID */
+	const struct guid *lmuid;/**< For proxied query: the original leaf MUID */
 	guint16 query_flags;	/**< Flags from the marked query speed field */
 	guint8 ttl;				/**< Initial query TTL */
 	guint32 horizon;		/**< Theoretical horizon reached thus far */
@@ -154,21 +157,21 @@ typedef struct dquery {
 	time_t start;			/**< Time at which it started */
 	time_t stop;			/**< Time at which it was terminated */
 	struct next_up *nv;		/**< Previous "next UP vector" */
-	gint nv_count;			/**< Number of items allocated for `nv' */
-	gint nv_found;			/**< Valid entries in `nv' */
+	int nv_count;			/**< Number of items allocated for `nv' */
+	int nv_found;			/**< Valid entries in `nv' */
 	pmsg_t *by_ttl[DQ_MAX_TTL];	/**< Copied mesages, one for each TTL */
 } dquery_t;
 
 enum {
-	DQ_F_ID_CLEANING	= 1 << 0,	/**< Cleaning the `by_node_id' table */
-	DQ_F_LINGER			= 1 << 1,	/**< Lingering to monitor extra hits */
-	DQ_F_LEAF_GUIDED	= 1 << 2,	/**< Leaf-guided query */
-	DQ_F_WAITING		= 1 << 3,	/**< Waiting guidance reply from leaf */
-	DQ_F_GOT_GUIDANCE	= 1 << 4,	/**< Got some leaf guidance */
-	DQ_F_USR_CANCELLED	= 1 << 5,	/**< Explicitely cancelled by user */
-	DQ_F_ROUTING_HITS	= 1 << 6,	/**< We'll be routing all hits */
+	DQ_F_LOCAL			= 1 << 8,	/**< Local query made by this node */
 	DQ_F_EXITING		= 1 << 7,	/**< Final cleanup at exit time */
-	DQ_F_REMOVED		= 1 << 8
+	DQ_F_ROUTING_HITS	= 1 << 6,	/**< We'll be routing all hits */
+	DQ_F_USR_CANCELLED	= 1 << 5,	/**< Explicitely cancelled by user */
+	DQ_F_GOT_GUIDANCE	= 1 << 4,	/**< Got some leaf guidance */
+	DQ_F_WAITING		= 1 << 3,	/**< Waiting guidance reply from leaf */
+	DQ_F_LEAF_GUIDED	= 1 << 2,	/**< Leaf-guided query */
+	DQ_F_LINGER			= 1 << 1,	/**< Lingering to monitor extra hits */
+	DQ_F_ID_CLEANING	= 1 << 0	/**< Cleaning the `by_node_id' table */
 };
 
 /**
@@ -216,9 +219,8 @@ static GHashTable *by_leaf_muid;
  * meta-information about it.
  */
 struct pmsg_info {
+	struct dquery_id qid;/**< Query ID of the dynamic query */
 	node_id_t node_id;	/**< The ID of the node we sent it to */
-	dquery_t *dq;		/**< The dynamic query that sent the query */
-	guint32 qid;		/**< Query ID of the dynamic query */
 	guint16 degree;		/**< The advertised degree of the destination node */
 	guint8 ttl;			/**< The TTL used for that query */
 };
@@ -236,8 +238,6 @@ struct pmsg_info {
 
 static guint32 hosts[MAX_DEGREE][MAX_TTL];	/**< Pre-computed horizon */
 
-static guint32 dyn_query_id = 0;
-
 static void dq_send_next(dquery_t *dq);
 static void dq_terminate(dquery_t *dq);
 
@@ -260,8 +260,8 @@ dquery_check(dquery_t *dq)
 static void
 fill_hosts(void)
 {
-	gint i;
-	gint j;
+	int i;
+	int j;
 
 	for (i = 0; i < MAX_DEGREE; i++) {
 		hosts[i][0] = 1;
@@ -283,10 +283,10 @@ fill_hosts(void)
  * is deperdition due to flow-control, network cycles, etc...
  */
 static guint32
-dq_get_horizon(gint degree, gint ttl)
+dq_get_horizon(int degree, int ttl)
 {
-	gint i;
-	gint j;
+	int i;
+	int j;
 
 	g_assert(degree > 0);
 	g_assert(ttl > 0);
@@ -314,7 +314,7 @@ dq_kept_results(dquery_t *dq)
 	 * update the amount now.
 	 */
 
-	if (node_id_self(dq->node_id))
+	if (dq->flags & DQ_F_LOCAL)
 		return dq->kept_results = search_get_kept_results_by_handle(dq->sh);
 
 	/*
@@ -335,15 +335,15 @@ dq_kept_results(dquery_t *dq)
  * specified node, assuming hosts are equally split among the remaining
  * connections we have yet to query.
  */
-static gint
-dq_select_ttl(dquery_t *dq, gnutella_node_t *node, gint connections)
+static int
+dq_select_ttl(dquery_t *dq, gnutella_node_t *node, int connections)
 {
 	guint32 needed;
 	guint32 results;
-	gdouble results_per_up;
-	gdouble hosts_to_reach;
-	gdouble hosts_to_reach_via_node;
-	gint ttl;
+	double results_per_up;
+	double hosts_to_reach;
+	double hosts_to_reach_via_node;
+	int ttl;
 
 	dquery_check(dq);
 	g_assert(connections > 0);
@@ -354,8 +354,8 @@ dq_select_ttl(dquery_t *dq, gnutella_node_t *node, gint connections)
 	g_assert(needed > 0);		/* Or query would have been stopped */
 
 	results_per_up = dq->results / MAX(dq->horizon, 1);
-	hosts_to_reach = (gdouble) needed / MAX(results_per_up, (gdouble) 0.000001);
-	hosts_to_reach_via_node = hosts_to_reach / (gdouble) connections;
+	hosts_to_reach = (double) needed / MAX(results_per_up, (double) 0.000001);
+	hosts_to_reach_via_node = hosts_to_reach / (double) connections;
 
 	/*
 	 * Now iteratively find the TTL needed to reach the desired number
@@ -393,7 +393,6 @@ dq_pmi_alloc(dquery_t *dq, guint16 degree, guint8 ttl, const node_id_t node_id)
 	dquery_check(dq);
 
 	pmi = walloc(sizeof(*pmi));
-	pmi->dq = dq;
 	pmi->qid = dq->qid;
 	pmi->degree = degree;
 	pmi->ttl = ttl;
@@ -418,17 +417,27 @@ dq_pmi_free(struct pmsg_info *pmi)
  * Check whether query bearing the specified ID is still alive and has
  * not been cancelled yet.
  */
-static gboolean
-dq_alive(dquery_t *dq, guint32 qid)
+static dquery_t *
+dq_alive(struct dquery_id qid)
 {
+	dquery_t *dq;
+
 	/* NOTE: dqueries might have been freed already, as dq_pmsg_free()
 	 *		 might still call this function after dq_close().
 	 */
-	if (dqueries && g_hash_table_lookup(dqueries, dq)) {
+	dq = dqueries ? g_hash_table_lookup(dqueries, &qid) : NULL;
+	if (dq) {
 		dquery_check(dq);
-		return dq->qid == qid;		/* In case it reused the same address */
 	}
-	return FALSE;
+	return dq;
+}
+
+static const char *
+dquery_id_to_string(struct dquery_id id)
+{
+	static char buf[UINT64_DEC_BUFLEN];
+	uint64_to_string_buf(id.value, buf, sizeof buf);
+	return buf;
 }
 
 /**
@@ -438,7 +447,7 @@ static void
 dq_pmsg_free(pmsg_t *mb, gpointer arg)
 {
 	struct pmsg_info *pmi = arg;
-	dquery_t *dq = pmi->dq;
+	dquery_t *dq;
 
 	/* NOTE: No dquery_check() because the memory might have been freed
 	 *		 already! See dq_alive and the comment below.
@@ -449,12 +458,11 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 	/*
 	 * It is possible that whilst the message was in the message queue,
 	 * the dynamic query was cancelled.  Therefore, we need to ensure that
-	 * the recorded query is still alive.  We use both the combination of
-	 * a hash table and a unique ID in case the address of an old dquery_t
-	 * object is reused later.
+	 * the recorded query is still alive. 
 	 */
 
-	if (!dq_alive(dq, pmi->qid))
+	dq = dq_alive(pmi->qid);
+	if (NULL == dq)
 		goto cleanup;
 
 	g_assert(dq->pending > 0);
@@ -475,8 +483,9 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		node_id_unref(key);
 
 		if (GNET_PROPERTY(dq_debug) > 19)
-			g_message("DQ[%d] %snode #%s degree=%d dropped message TTL=%d",
-				dq->qid, node_id_self(dq->node_id) ? "(local) " : "",
+			g_message("DQ[%s] %snode #%s degree=%d dropped message TTL=%d",
+				dquery_id_to_string(dq->qid),
+				node_id_self(dq->node_id) ? "(local) " : "",
 				node_id_to_string(pmi->node_id), pmi->degree, pmi->ttl);
 
 		/*
@@ -500,13 +509,15 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		dq->up_sent++;
 
 		if (GNET_PROPERTY(dq_debug) > 19) {
-			g_message("DQ[%d] %snode #%s degree=%d sent message TTL=%d",
-				dq->qid, node_id_self(dq->node_id) ? "(local) " : "",
+			g_message("DQ[%s] %snode #%s degree=%d sent message TTL=%d",
+				dquery_id_to_string(dq->qid),
+				node_id_self(dq->node_id) ? "(local) " : "",
 				node_id_to_string(pmi->node_id), pmi->degree, pmi->ttl);
-			g_message("DQ[%d] %s(%d secs) queried %d UP%s, "
+			g_message("DQ[%s] %s(%d secs) queried %d UP%s, "
 				"horizon=%d, results=%d",
-				dq->qid, node_id_self(dq->node_id) ? "local " : "",
-				(gint) (tm_time() - dq->start),
+				dquery_id_to_string(dq->qid),
+				node_id_self(dq->node_id) ? "local " : "",
+				(int) (tm_time() - dq->start),
 				dq->up_sent, dq->up_sent == 1 ? "" :"s",
 				dq->horizon, dq->results);
 		}
@@ -521,12 +532,12 @@ cleanup:
  * If no such message exists yet, create it from the "template" message.
  */
 static pmsg_t *
-dq_pmsg_by_ttl(dquery_t *dq, gint ttl)
+dq_pmsg_by_ttl(dquery_t *dq, int ttl)
 {
 	pmsg_t *mb;
 	pmsg_t *t;
 	pdata_t *db;
-	gint len;
+	int len;
 
 	dquery_check(dq);
 	g_assert(ttl > 0 && ttl <= DQ_MAX_TTL);
@@ -576,11 +587,11 @@ dq_pmsg_by_ttl(dquery_t *dq, gint ttl)
  *
  * @return amount of nodes we found.
  */
-static gint
-dq_fill_probe_up(dquery_t *dq, gnutella_node_t **nv, gint ncount)
+static int
+dq_fill_probe_up(dquery_t *dq, gnutella_node_t **nv, int ncount)
 {
 	const GSList *sl;
-	gint i = 0;
+	int i = 0;
 
 	dquery_check(dq);
 
@@ -631,7 +642,7 @@ dq_free_next_up(dquery_t *dq)
 	g_assert((NULL == dq->nv) ^ (dq->nv_count > 0));
 
 	if (dq->nv) {
-		gint i;
+		int i;
 
 		for (i = 0; i < dq->nv_found; i++) {
 			node_id_unref(dq->nv[i].node_id);
@@ -652,11 +663,11 @@ dq_free_next_up(dquery_t *dq)
  *
  * @return amount of nodes we found.
  */
-static gint
-dq_fill_next_up(dquery_t *dq, struct next_up *nv, gint ncount)
+static int
+dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 {
 	const GSList *sl;
-	gint i = 0;
+	int i = 0;
 	GHashTable *old = NULL;
 
 	dquery_check(dq);
@@ -668,7 +679,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, gint ncount)
 	 */
 
 	if (dq->nv != NULL) {
-		gint j;
+		int j;
 
 		old = g_hash_table_new(node_id_hash, node_id_eq_func);
 
@@ -800,17 +811,18 @@ free_node_id(gpointer key, gpointer value, gpointer unused_udata)
 static void
 dq_free(dquery_t *dq)
 {
-	gint i;
+	int i;
 
 	dquery_check(dq);
-	g_assert(g_hash_table_lookup(dqueries, dq));
+	g_assert(g_hash_table_lookup(dqueries, &dq->qid) == dq);
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] %s(%d secs; +%d secs) node #%s ending: "
+		g_message("DQ[%s] %s(%d secs; +%d secs) node #%s ending: "
 			"ttl=%d, queried=%d, horizon=%d, results=%d+%d",
-			dq->qid, node_id_self(dq->node_id) ? "local " : "",
-			(gint) (tm_time() - dq->start),
-			(dq->flags & DQ_F_LINGER) ? (gint) (tm_time() - dq->stop) : 0,
+			dquery_id_to_string(dq->qid),
+			node_id_self(dq->node_id) ? "local " : "",
+			(int) (tm_time() - dq->start),
+			(dq->flags & DQ_F_LINGER) ? (int) (tm_time() - dq->stop) : 0,
 			node_id_to_string(dq->node_id), dq->ttl, dq->up_sent, dq->horizon,
 			dq->results, dq->linger_results);
 
@@ -860,7 +872,7 @@ dq_free(dquery_t *dq)
 	}
 
 	if (!(dq->flags & DQ_F_EXITING))
-		g_hash_table_remove(dqueries, dq);
+		g_hash_table_remove(dqueries, &dq->qid);
 
 	/*
 	 * Remove query from the `by_node_id' table but only if the node ID
@@ -880,7 +892,6 @@ dq_free(dquery_t *dq)
 		gboolean found;
 		GSList *list;
 
-		g_assert(0 == (DQ_F_REMOVED & dq->flags));
 		found = g_hash_table_lookup_extended(by_node_id, dq->node_id,
 					NULL, &value);
 
@@ -896,7 +907,6 @@ dq_free(dquery_t *dq)
 			g_hash_table_remove(by_node_id, dq->node_id);
 			g_assert(!g_hash_table_lookup_extended(by_node_id, dq->node_id,
 						NULL, NULL));
-			dq->flags |= DQ_F_REMOVED; 
 		} else if (list != value) {
 			dquery_t *key = list->data;
 
@@ -958,7 +968,7 @@ dq_expired(cqueue_t *unused_cq, gpointer obj)
 	(void) unused_cq;
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] expired", dq->qid);
+		g_message("DQ[%s] expired", dquery_id_to_string(dq->qid));
 
 	dq->expire_ev = NULL;	/* Indicates callback fired */
 
@@ -988,7 +998,7 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 {
 	dquery_t *dq = obj;
 	gnutella_node_t *n;
-	gint timeout;
+	int timeout;
 	guint32 avg;
 	guint32 last;
 	gboolean was_waiting = FALSE;
@@ -1014,8 +1024,9 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 		dq->stat_timeouts++;
 
 		if (GNET_PROPERTY(dq_debug) > 19)
-			g_message("DQ[%d] (%d secs) timeout #%u waiting for status results",
-				dq->qid, (gint) (tm_time() - dq->start), dq->stat_timeouts);
+			g_message("DQ[%s] (%d secs) timeout #%u waiting for status results",
+				dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
+				dq->stat_timeouts);
 		dq->flags &= ~DQ_F_WAITING;
 
 		if (
@@ -1027,8 +1038,9 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 
 			if (GNET_PROPERTY(dq_debug) > 19)
 				g_message(
-					"DQ[%d] (%d secs) turned off leaf-guidance for node #%s",
-					dq->qid, (gint) (tm_time() - dq->start),
+					"DQ[%s] (%d secs) turned off leaf-guidance for node #%s",
+					dquery_id_to_string(dq->qid),
+					(int) (tm_time() - dq->start),
 					node_id_to_string(dq->node_id));
 		}
 
@@ -1045,8 +1057,9 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 	if (!(dq->flags & (DQ_F_LEAF_GUIDED|DQ_F_ROUTING_HITS))) {
 		if (GNET_PROPERTY(dq_debug))
 			g_message(
-				"DQ[%d] terminating unguided & unrouted (queried %u UP%s)",
-				dq->qid, dq->up_sent, dq->up_sent == 1 ? "" : "s");
+				"DQ[%s] terminating unguided & unrouted (queried %u UP%s)",
+				dquery_id_to_string(dq->qid), dq->up_sent,
+				dq->up_sent == 1 ? "" : "s");
 		dq_terminate(dq);
 		return;
 	}
@@ -1089,16 +1102,16 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 
 	if (n == NULL) {
 		if (GNET_PROPERTY(dq_debug) > 19)
-			g_message("DQ[%d] (%d secs) node #%s appears to be dead",
-				dq->qid, (gint) (tm_time() - dq->start),
+			g_message("DQ[%s] (%d secs) node #%s appears to be dead",
+				dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
 				node_id_to_string(dq->node_id));
 		dq_free(dq);
 		return;
 	}
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] (%d secs) requesting node #%s for status (kept=%u)",
-			dq->qid, (gint) (tm_time() - dq->start),
+		g_message("DQ[%s] (%d secs) requesting node #%s for status (kept=%u)",
+			dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
 			node_id_to_string(dq->node_id), dq->kept_results);
 
 	dq->flags |= DQ_F_WAITING;
@@ -1121,8 +1134,8 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 	timeout = MAX(timeout, DQ_STATUS_TIMEOUT);
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] status reply timeout set to %d s", dq->qid,
-			timeout / 1000);
+		g_message("DQ[%s] status reply timeout set to %d s",
+			dquery_id_to_string(dq->qid), timeout / 1000);
 
 	dq->results_ev = cq_insert(callout_queue, timeout, dq_results_expired, dq);
 }
@@ -1133,7 +1146,7 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 static void
 dq_terminate(dquery_t *dq)
 {
-	gint delay;
+	int delay;
 
 	g_assert(!(dq->flags & DQ_F_LINGER));
 	g_assert(dq->results_ev == NULL);
@@ -1158,9 +1171,9 @@ dq_terminate(dquery_t *dq)
 	dq->stop = tm_time();
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] (%d secs) node #%s lingering: "
+		g_message("DQ[%s] (%d secs) node #%s lingering: "
 			"ttl=%d, queried=%d, horizon=%d, results=%d",
-			dq->qid, (gint) (tm_time() - dq->start),
+			dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
 			node_id_to_string(dq->node_id),
 			dq->ttl, dq->up_sent, dq->horizon, dq->results);
 }
@@ -1168,13 +1181,13 @@ dq_terminate(dquery_t *dq)
 /**
  * qsort() callback for sorting nodes by increasing queue size.
  */
-static gint
+static int
 node_mq_cmp(const void *np1, const void *np2)
 {
 	const gnutella_node_t *n1 = *(const gnutella_node_t **) np1;
 	const gnutella_node_t *n2 = *(const gnutella_node_t **) np2;
-	gint qs1 = NODE_MQUEUE_PENDING(n1);
-	gint qs2 = NODE_MQUEUE_PENDING(n2);
+	int qs1 = NODE_MQUEUE_PENDING(n1);
+	int qs2 = NODE_MQUEUE_PENDING(n2);
 
 	/*
 	 * We don't cache the results of NODE_MQUEUE_PENDING() like we do in
@@ -1191,14 +1204,14 @@ node_mq_cmp(const void *np1, const void *np2)
  * qsort() callback for sorting nodes by increasing queue size, with a
  * preference towards nodes that have a QRP match.
  */
-static gint
+static int
 node_mq_qrp_cmp(const void *np1, const void *np2)
 {
 	struct next_up *nu1 = deconstify_gpointer(np1);
 	struct next_up *nu2 = deconstify_gpointer(np2);
 	const gnutella_node_t *n1, *n2;
-	gint qs1 = nu1->queue_pending;
-	gint qs2 = nu2->queue_pending;
+	int qs1 = nu1->queue_pending;
+	int qs2 = nu2->queue_pending;
 
 	/*
 	 * Cache the results of NODE_MQUEUE_PENDING() since it involves
@@ -1245,7 +1258,7 @@ node_mq_qrp_cmp(const void *np1, const void *np2)
  * adjusted down accordingly.
  */
 static void
-dq_send_query(dquery_t *dq, gnutella_node_t *n, gint ttl)
+dq_send_query(dquery_t *dq, gnutella_node_t *n, int ttl)
 {
 	struct pmsg_info *pmi;
 	pmsg_t *mb;
@@ -1271,13 +1284,23 @@ dq_send_query(dquery_t *dq, gnutella_node_t *n, gint ttl)
 	mb = pmsg_clone_extend(mb, dq_pmsg_free, pmi);
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] (%d secs) queuing ttl=%d to #%s %s <%s> Q=%d bytes",
-			dq->qid, (gint) delta_time(tm_time(), dq->start),
+		g_message("DQ[%s] (%d secs) queuing ttl=%d to #%s %s <%s> Q=%d bytes",
+			dquery_id_to_string(dq->qid),
+			(int) delta_time(tm_time(), dq->start),
 			pmi->ttl, node_id_to_string(NODE_ID(n)),
-			node_addr(n), node_vendor(n), (gint) NODE_MQUEUE_PENDING(n));
+			node_addr(n), node_vendor(n), (int) NODE_MQUEUE_PENDING(n));
 
 	dq->pending++;
-	gmsg_mb_sendto_one(n, mb);
+
+	/*
+	 * If query is not local, the messages we send are as if we had
+	 * relayed the original query message (for tracing / logging purposes).
+	 */
+
+	if (dq->flags & DQ_F_LOCAL)
+		gmsg_mb_sendto_one(n, mb);
+	else
+		gmsg_mb_routeto_one(node_by_id(dq->node_id), n, mb);
 }
 
 /**
@@ -1290,11 +1313,11 @@ static void
 dq_send_next(dquery_t *dq)
 {
 	struct next_up *nv;
-	gint ncount = GNET_PROPERTY(max_connections);
-	gint found;
-	gint ttl;
-	gint timeout;
-	gint i;
+	int ncount = GNET_PROPERTY(max_connections);
+	int found;
+	int ttl;
+	int timeout;
+	int i;
 	gboolean sent = FALSE;
 	guint32 results;
 
@@ -1307,7 +1330,8 @@ dq_send_next(dquery_t *dq)
 
 	if (GNET_PROPERTY(current_peermode) != NODE_P_ULTRA) {
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminating (no longer an ultra node)", dq->qid);
+			g_message("DQ[%s] terminating (no longer an ultra node)",
+				dquery_id_to_string(dq->qid));
 		goto terminate;
 	}
 
@@ -1320,9 +1344,10 @@ dq_send_next(dquery_t *dq)
 
 	if (dq->horizon >= DQ_MAX_HORIZON || results >= dq->max_results) {
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminating "
+			g_message("DQ[%s] terminating "
 				"(UPs=%u, horizon=%u >= %d, %s results=%u >= %u)",
-				dq->qid, dq->up_sent, dq->horizon, DQ_MAX_HORIZON,
+				dquery_id_to_string(dq->qid), dq->up_sent, dq->horizon,
+				DQ_MAX_HORIZON,
 				(dq->flags & DQ_F_GOT_GUIDANCE) ? "guided" : "unguided",
 				results, dq->max_results);
 		goto terminate;
@@ -1338,9 +1363,9 @@ dq_send_next(dquery_t *dq)
 
 	if (dq->results + dq->oob_results > dq->fin_results) {
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminating "
+			g_message("DQ[%s] terminating "
 				"(UPs=%u, seen=%u + OOB=%u >= %u -- %s kept=%u)",
-				dq->qid, dq->up_sent,
+				dquery_id_to_string(dq->qid), dq->up_sent,
 				dq->results, dq->oob_results, dq->fin_results,
 				(dq->flags & DQ_F_GOT_GUIDANCE) ? "guided" : "unguided",
 				results);
@@ -1357,8 +1382,8 @@ dq_send_next(dquery_t *dq)
 			GNET_PROPERTY(max_connections) - GNET_PROPERTY(normal_connections)
 	) {
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminating (queried UPs=%u >= %u)",
-				dq->qid, dq->up_sent,
+			g_message("DQ[%s] terminating (queried UPs=%u >= %u)",
+				dquery_id_to_string(dq->qid), dq->up_sent,
 				GNET_PROPERTY(max_connections) -
 					GNET_PROPERTY(normal_connections));
 		goto terminate;
@@ -1373,8 +1398,8 @@ dq_send_next(dquery_t *dq)
 
 	if (dq->pending >= DQ_MAX_PENDING) {
 		if (GNET_PROPERTY(dq_debug) > 19)
-			g_message("DQ[%d] waiting for %u ms (pending=%u)",
-				dq->qid, dq->result_timeout, dq->pending);
+			g_message("DQ[%s] waiting for %u ms (pending=%u)",
+				dquery_id_to_string(dq->qid), dq->result_timeout, dq->pending);
 		dq->results_ev = cq_insert(callout_queue,
 			dq->result_timeout, dq_results_expired, dq);
 		return;
@@ -1386,8 +1411,8 @@ dq_send_next(dquery_t *dq)
 	g_assert(dq->nv == nv);		/* Saved for next time */
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] still %d UP%s to query (results %sso far: %u)",
-			dq->qid, found, found == 1 ? "" : "s",
+		g_message("DQ[%s] still %d UP%s to query (results %sso far: %u)",
+			dquery_id_to_string(dq->qid), found, found == 1 ? "" : "s",
 			(dq->flags & DQ_F_GOT_GUIDANCE) ? "reported kept " : "", results);
 
 	if (found == 0) {
@@ -1421,8 +1446,9 @@ dq_send_next(dquery_t *dq)
 			!qrp_node_can_route(node, dq->qhv)
 		) {
 			if (GNET_PROPERTY(dq_debug) > 19)
-				g_message("DQ[%d] TTL=1, skipping node #%s: can't route query!",
-					dq->qid, node_id_to_string(NODE_ID(node)));
+				g_message("DQ[%s] TTL=1, skipping node #%s: can't route query!",
+					dquery_id_to_string(dq->qid),
+					node_id_to_string(NODE_ID(node)));
 
 			continue;
 		}
@@ -1464,8 +1490,9 @@ dq_send_next(dquery_t *dq)
 	}
 
 	if (GNET_PROPERTY(dq_debug) > 1)
-		g_message("DQ[%d] (%d secs) timeout set to %d ms (pending=%d)",
-			dq->qid, (gint) (tm_time() - dq->start), timeout, dq->pending);
+		g_message("DQ[%s] (%d secs) timeout set to %d ms (pending=%d)",
+			dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start), timeout,
+			dq->pending);
 
 	dq->results_ev = cq_insert(callout_queue, timeout, dq_results_expired, dq);
 	return;
@@ -1483,10 +1510,10 @@ static void
 dq_send_probe(dquery_t *dq)
 {
 	gnutella_node_t **nv;
-	gint ncount = GNET_PROPERTY(max_connections);
-	gint found;
-	gint ttl = dq->ttl;
-	gint i;
+	int ncount = GNET_PROPERTY(max_connections);
+	int found;
+	int ttl = dq->ttl;
+	int i;
 
 	dquery_check(dq);
 	g_assert(dq->results_ev == NULL);
@@ -1495,8 +1522,8 @@ dq_send_probe(dquery_t *dq)
 	found = dq_fill_probe_up(dq, nv, ncount);
 
 	if (GNET_PROPERTY(dq_debug) > 19)
-		g_message("DQ[%d] found %d UP%s to probe",
-			dq->qid, found, found == 1 ? "" : "s");
+		g_message("DQ[%s] found %d UP%s to probe",
+			dquery_id_to_string(dq->qid), found, found == 1 ? "" : "s");
 
 	/*
 	 * If we don't find any suitable UP holding that content, then
@@ -1552,6 +1579,15 @@ cleanup:
 	wfree(nv, ncount * sizeof nv[0]);
 }
 
+static struct dquery_id
+dquery_id_create(void)
+{
+	static struct dquery_id id;
+	id.value++;
+	g_assert(0 != id.value);
+	return id;
+}
+
 /**
  * Common initialization code for a dynamic query.
  */
@@ -1561,7 +1597,7 @@ dq_common_init(dquery_t *dq)
 	gconstpointer head;
 
 	dquery_check(dq);
-	dq->qid = dyn_query_id++;
+	dq->qid = dquery_id_create();
 	dq->queried = g_hash_table_new(node_id_hash, node_id_eq_func);
 	dq->result_timeout = DQ_QUERY_TIMEOUT;
 	dq->start = tm_time();
@@ -1577,13 +1613,13 @@ dq_common_init(dquery_t *dq)
 	 * Record the query as being "alive".
 	 */
 
-	g_hash_table_insert(dqueries, dq, GINT_TO_POINTER(1));
+	g_hash_table_insert(dqueries, &dq->qid, dq);
 
 	/*
 	 * If query is not for the local node, insert it in `by_node_id'.
 	 */
 
-	if (!node_id_self(dq->node_id)) {
+	if (!(dq->flags & DQ_F_LOCAL)) {
 		gpointer value;
 		gboolean found;
 		GSList *list;
@@ -1612,7 +1648,7 @@ dq_common_init(dquery_t *dq)
 		g_warning("conflicting MUID \"%s\" for dynamic query, ignoring.",
 			guid_hex_str(gnutella_header_get_muid(head)));
 	else {
-		const gchar *muid = atom_guid_get(gnutella_header_get_muid(head));
+		const struct guid *muid = atom_guid_get(gnutella_header_get_muid(head));
 		gm_hash_table_insert_const(by_muid, muid, dq);
 	}
 
@@ -1646,15 +1682,16 @@ dq_common_init(dquery_t *dq)
 		packet = pmsg_start(dq->mb);
 		flags = gnutella_msg_search_get_flags(packet);
 
-		g_message("DQ[%d] created for node #%s: TTL=%d max_results=%d "
+		g_message("DQ[%s] created for node #%s: TTL=%d max_results=%d "
 			"guidance=%s routing=%s "
 			"MUID=%s%s%s q=\"%s\" flags=0x%x (%s%s%s%s%s%s%s)",
-			dq->qid, node_id_to_string(dq->node_id), dq->ttl, dq->max_results,
+			dquery_id_to_string(dq->qid), node_id_to_string(dq->node_id),
+			dq->ttl, dq->max_results,
 			(dq->flags & DQ_F_LEAF_GUIDED) ? "yes" : "no",
 			(dq->flags & DQ_F_ROUTING_HITS) ? "yes" : "no",
 			guid_hex_str(gnutella_header_get_muid(head)),
 			dq->lmuid ? " leaf-MUID=" : "",
-			dq->lmuid ? data_hex_str(dq->lmuid, GUID_RAW_SIZE): "",
+			dq->lmuid ? data_hex_str(dq->lmuid->v, GUID_RAW_SIZE): "",
 			gnutella_msg_search_get_text(packet), flags,
 			(flags & QUERY_F_MARK) ? "MARKED" : "",
 			(flags & QUERY_F_FIREWALLED) ? " FW" : "",
@@ -1676,7 +1713,7 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv)
 	dquery_t *dq;
 	guint16 flags;
 	gboolean flags_valid;
-	const gchar *leaf_muid;
+	const struct guid *leaf_muid;
 
 	/* Query from leaf node */
 	g_assert(NODE_IS_LEAF(n));
@@ -1848,7 +1885,7 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	dq->fin_results = dq->max_results * 100 / DQ_PERCENT_KEPT;
 	dq->ttl = MIN(GNET_PROPERTY(my_ttl), DQ_MAX_TTL);
 	dq->alive = NULL;
-	dq->flags = DQ_F_ROUTING_HITS;		/* We get our own hits! */
+	dq->flags = DQ_F_ROUTING_HITS | DQ_F_LOCAL;		/* We get our own hits! */
 
 	gnet_stats_count_general(GNR_LOCAL_DYN_QUERIES, 1);
 
@@ -1879,8 +1916,8 @@ dq_node_removed(const node_id_t node_id)
 		dquery_check(dq);
 
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminated by node #%s removal (queried %u UP%s)",
-				dq->qid, node_id_to_string(dq->node_id),
+			g_message("DQ[%s] terminated by node #%s removal (queried %u UP%s)",
+				dquery_id_to_string(dq->qid), node_id_to_string(dq->node_id),
 				dq->up_sent, dq->up_sent == 1 ? "" : "s");
 		
 		/* Don't remove query from the table in dq_free() */
@@ -1907,7 +1944,8 @@ dq_node_removed(const node_id_t node_id)
  * should not forward the results anyway.
  */
 static gboolean
-dq_count_results(const gchar *muid, gint count, guint16 status, gboolean oob)
+dq_count_results(const struct guid *muid,
+	int count, guint16 status, gboolean oob)
 {
 	dquery_t *dq;
 
@@ -1946,15 +1984,17 @@ dq_count_results(const gchar *muid, gint count, guint16 status, gboolean oob)
 	) {
 		if (GNET_PROPERTY(dq_debug) > 19) {
 			if (dq->flags & DQ_F_LINGER)
-				g_message("DQ[%d] %s(%d secs; +%d secs) +%d ignored (firewall)",
-					dq->qid, node_id_self(dq->node_id) ? "local " : "",
-					(gint) (tm_time() - dq->start),
-					(gint) (tm_time() - dq->stop),
+				g_message("DQ[%s] %s(%d secs; +%d secs) +%d ignored (firewall)",
+					dquery_id_to_string(dq->qid),
+					node_id_self(dq->node_id) ? "local " : "",
+					(int) (tm_time() - dq->start),
+					(int) (tm_time() - dq->stop),
 					count);
 			else
-				g_message("DQ[%d] %s(%d secs) +%d ignored (firewall)",
-					dq->qid, node_id_self(dq->node_id) ? "local " : "",
-					(gint) (tm_time() - dq->start),
+				g_message("DQ[%s] %s(%d secs) +%d ignored (firewall)",
+					dquery_id_to_string(dq->qid),
+					node_id_self(dq->node_id) ? "local " : "",
+					(int) (tm_time() - dq->start),
 					count);
 		}
 
@@ -1974,18 +2014,20 @@ dq_count_results(const gchar *muid, gint count, guint16 status, gboolean oob)
 		if (node_id_self(dq->node_id))
 			dq->kept_results = search_get_kept_results_by_handle(dq->sh);
 		if (dq->flags & DQ_F_LINGER)
-			g_message("DQ[%d] %s(%d secs; +%d secs) "
+			g_message("DQ[%s] %s(%d secs; +%d secs) "
 				"+%d %slinger_results=%d kept=%d",
-				dq->qid, node_id_self(dq->node_id) ? "local " : "",
-				(gint) (tm_time() - dq->start),
-				(gint) (tm_time() - dq->stop),
+				dquery_id_to_string(dq->qid),
+				node_id_self(dq->node_id) ? "local " : "",
+				(int) (tm_time() - dq->start),
+				(int) (tm_time() - dq->stop),
 				count, oob ? "OOB " : "",
 				dq->linger_results, dq->kept_results);
 		else
-			g_message("DQ[%d] %s(%d secs) "
+			g_message("DQ[%s] %s(%d secs) "
 				"+%d %sresults=%d new=%d kept=%d oob=%d",
-				dq->qid, node_id_self(dq->node_id) ? "local " : "",
-				(gint) (tm_time() - dq->start),
+				dquery_id_to_string(dq->qid),
+				node_id_self(dq->node_id) ? "local " : "",
+				(int) (tm_time() - dq->start),
 				count, oob ? "OOB " : "",
 				dq->results, dq->new_results, dq->kept_results,
 				dq->oob_results);
@@ -2008,7 +2050,7 @@ dq_count_results(const gchar *muid, gint count, guint16 status, gboolean oob)
  * whether we should forward the results.
  */
 gboolean
-dq_got_results(const gchar *muid, guint count, guint32 status)
+dq_got_results(const struct guid *muid, guint count, guint32 status)
 {
 	return dq_count_results(muid, count, status, FALSE);
 }
@@ -2021,7 +2063,7 @@ dq_got_results(const gchar *muid, guint count, guint32 status)
  * results should not be claimed.
  */
 gboolean
-dq_oob_results_ind(const gchar *muid, gint count)
+dq_oob_results_ind(const struct guid *muid, int count)
 {
 	return dq_count_results(muid, count, 0, TRUE);
 }
@@ -2033,7 +2075,7 @@ dq_oob_results_ind(const gchar *muid, gint count)
  * were finally claimed and parsed).
  */
 void
-dq_oob_results_got(const gchar *muid, guint count)
+dq_oob_results_got(const struct guid *muid, guint count)
 {
 	dquery_t *dq;
 
@@ -2071,7 +2113,8 @@ dq_oob_results_got(const gchar *muid, guint count)
  * The special value 0xffff is a request to stop the query immediately.
  */
 void
-dq_got_query_status(const gchar *muid, const node_id_t node_id, guint16 kept)
+dq_got_query_status(const struct guid *muid,
+	const node_id_t node_id, guint16 kept)
 {
 	dquery_t *dq;
 
@@ -2106,20 +2149,21 @@ dq_got_query_status(const gchar *muid, const node_id_t node_id, guint16 kept)
 
 			if (GNET_PROPERTY(dq_debug) > 19)
 				g_message(
-					"DQ[%d] (%d secs) turned on leaf-guidance for node #%s",
-					dq->qid, (gint) (tm_time() - dq->start),
+					"DQ[%s] (%d secs) turned on leaf-guidance for node #%s",
+					dquery_id_to_string(dq->qid),
+					(int) (tm_time() - dq->start),
 					node_id_to_string(dq->node_id));
 		}
 	}
 
 	if (GNET_PROPERTY(dq_debug) > 19) {
 		if (dq->flags & DQ_F_LINGER)
-			g_message("DQ[%d] (%d secs; +%d secs) kept_results=%d",
-				dq->qid, (gint) (tm_time() - dq->start),
-				(gint) (tm_time() - dq->stop), dq->kept_results);
+			g_message("DQ[%s] (%d secs; +%d secs) kept_results=%d",
+				dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
+				(int) (tm_time() - dq->stop), dq->kept_results);
 		else
-			g_message("DQ[%d] (%d secs) %ssolicited, kept_results=%d",
-				dq->qid, (gint) (tm_time() - dq->start),
+			g_message("DQ[%s] (%d secs) %ssolicited, kept_results=%d",
+				dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
 				(dq->flags & DQ_F_WAITING) ? "" : "un", dq->kept_results);
 	}
 
@@ -2133,8 +2177,9 @@ dq_got_query_status(const gchar *muid, const node_id_t node_id, guint16 kept)
 
 	if (kept == 0xffff) {
 		if (GNET_PROPERTY(dq_debug))
-			g_message("DQ[%d] terminating at user's request (queried %u UP%s)",
-				dq->qid, dq->up_sent, dq->up_sent == 1 ? "" : "s");
+			g_message("DQ[%s] terminating at user's request (queried %u UP%s)",
+				dquery_id_to_string(dq->qid), dq->up_sent,
+				dq->up_sent == 1 ? "" : "s");
 
 		dq->flags |= DQ_F_USR_CANCELLED;
 
@@ -2170,14 +2215,15 @@ struct cancel_context {
  * -- hash table iterator callback
  */
 static void
-dq_cancel_local(gpointer key, gpointer unused_value, gpointer udata)
+dq_cancel_local(gpointer key, gpointer value, gpointer udata)
 {
 	struct cancel_context *ctx = udata;
-	dquery_t *dq = key;
+	dquery_t *dq = value;
 
-	(void) unused_value;
 	dquery_check(dq);
-	if (node_id_self(dq->node_id) && dq->sh == ctx->handle) {
+	g_assert(&dq->qid == key);
+
+	if ((dq->flags & DQ_F_LOCAL) && dq->sh == ctx->handle) {
 		ctx->cancelled = g_slist_prepend(ctx->cancelled, dq);
 	}
 }
@@ -2215,7 +2261,7 @@ dq_search_closed(gnet_search_t handle)
  * any more, in which case nothing is returned into `wanted'.
  */
 gboolean
-dq_get_results_wanted(const gchar *muid, guint32 *wanted)
+dq_get_results_wanted(const struct guid *muid, guint32 *wanted)
 {
 	dquery_t *dq;
 
@@ -2258,13 +2304,27 @@ dq_get_results_wanted(const gchar *muid, guint32 *wanted)
 	return TRUE;
 }
 
+static unsigned
+dquery_id_hash(const void *key)
+{
+	const struct dquery_id *id = key;
+	return (unsigned) (id->value >> 32) ^ (unsigned) id->value;
+}
+
+static int
+dquery_id_equal(const void *p, const void *q)
+{
+	const struct dquery_id *a = p, *b = q;
+	return a->value == b->value;
+}
+
 /**
  * Initialize dynamic querying.
  */
 void
 dq_init(void)
 {
-	dqueries = g_hash_table_new(NULL, NULL);
+	dqueries = g_hash_table_new(dquery_id_hash, dquery_id_equal);
 	by_node_id = g_hash_table_new(node_id_hash, node_id_eq_func);
 	by_muid = g_hash_table_new(guid_hash, guid_eq);
 	by_leaf_muid = g_hash_table_new(guid_hash, guid_eq);
@@ -2275,12 +2335,12 @@ dq_init(void)
  * Hashtable iteration callback to free the dquery_t object held as the key.
  */
 static void
-free_query(gpointer key, gpointer unused_value, gpointer unused_udata)
+free_query(gpointer key, gpointer value, gpointer unused_udata)
 {
-	dquery_t *dq = key;
+	dquery_t *dq = value;
 
 	dquery_check(dq);
-	(void) unused_value;
+	g_assert(&dq->qid == key);
 	(void) unused_udata;
 
 	dq->flags |= DQ_F_EXITING;		/* So nothing is removed from the table */
@@ -2296,7 +2356,7 @@ static void
 free_query_list(gpointer key, gpointer value, gpointer unused_udata)
 {
 	GSList *sl, *list = value;
-	gint count = g_slist_length(list);
+	int count = g_slist_length(list);
 
 	(void) unused_udata;
 	g_warning("remained %d un-freed dynamic quer%s for node #%u",
