@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $
+ * $Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $")
+RCSID("$Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #include <zlib.h>
 
@@ -43,6 +43,7 @@ RCSID("$Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $")
 #include "ggep.h"
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
 #include "lib/misc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -94,8 +95,8 @@ RCSID("$Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $")
  * each time a new extension is found.
  */
 typedef struct extdesc {
-	const gchar *ext_phys_payload;	/**< Start of payload buffer */
-	const gchar *ext_payload;		/**< "virtual" payload */
+	const char *ext_phys_payload;	/**< Start of payload buffer */
+	const char *ext_payload;		/**< "virtual" payload */
 	guint16 ext_phys_len;		/**< Extension length (header + payload) */
 	guint16 ext_phys_paylen;	/**< Extension payload length */
 	guint16 ext_paylen;			/**< "virtual" payload length */
@@ -105,7 +106,7 @@ typedef struct extdesc {
 		struct {
 			gboolean extu_cobs;			/**< Payload is COBS-encoded */
 			gboolean extu_deflate;		/**< Payload is deflated */
-			const gchar *extu_id;		/**< Extension ID */
+			const char *extu_id;		/**< Extension ID */
 		} extu_ggep;
 	} ext_u;
 
@@ -122,7 +123,7 @@ typedef struct extdesc {
 #define ext_ggep_deflate	ext_u.extu_ggep.extu_deflate
 #define ext_ggep_id			ext_u.extu_ggep.extu_id
 
-static const gchar * const extype[] = {
+static const char * const extype[] = {
 	"UNKNOWN",					/**< EXT_UNKNOWN */
 	"XML",						/**< EXT_XML */
 	"HUGE",						/**< EXT_HUGE */
@@ -138,7 +139,7 @@ static const gchar * const extype[] = {
  * Reserved word description.
  */
 struct rwtable {
-	const gchar *rw_name;	/**< Representation */
+	const char *rw_name;	/**< Representation */
 	ext_token_t rw_token;	/**< Token value */
 };
 
@@ -163,6 +164,8 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(BH),		/**< Browseable host indication */
 	GGEP_ID(C),			/**< Result Code in HEAD Pongs */
 	GGEP_ID(CT),		/**< Resource creation time */
+	GGEP_ID(DHT),		/**< DHT version and flags, in pongs */
+	GGEP_ID(DHTIPP),	/**< DHT nodes in packed IP:Port format (pongs) */
 	GGEP_ID(DU),		/**< Average servent uptime */
 	GGEP_ID(F),			/**< Flags in HEAD Pongs */
 	GGEP_ID(FW),		/**< Firewalled-to-Firewalled protocol version */
@@ -190,12 +193,22 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(SO),		/**< Secure OOB */
 	GGEP_ID(T),			/**< Same as ALT_TLS but for HEAD Pongs */
 	GGEP_ID(TLS),		/**< TLS support indication */
+	GGEP_ID(TT),		/**< Tigertree root hash (TTH); binary */
 	GGEP_ID(UA),		/**< User-Agent string */
 	GGEP_ID(UDPHC),		/**< Is an UDP hostcache (UHC) , in pongs */
 	GGEP_ID(UP),		/**< Ultrapeer information about free slots */
 	GGEP_ID(V),			/**< Vendor code, in HEAD Pongs */
 	GGEP_ID(VC),		/**< Vendor code, in pongs */
 	GGEP_ID(VMSG),		/**< Array of supported vendor message codes */
+	{ "client-id", EXT_T_GGEP_client_id }, /**< GUID in ALOC v0.0 */
+	GGEP_ID(features),	/**< Unknown value, PROX v0.0 */
+	GGEP_ID(firewalled),/**< Firewalled status in ALOC v0.0 */
+	{ "fwt-version", EXT_T_GGEP_fwt_version }, /**< FW2FW version, PROX v0.0 */
+	GGEP_ID(length),	/**< File length in ALOC v0.1 */
+	GGEP_ID(port),		/**< Servent's Port in ALOC v0.0 */
+	GGEP_ID(proxies),	/**< Push proxies in PROX v0.0 */
+	GGEP_ID(tls),		/**< TLS support in ALOC v0.1 */
+	GGEP_ID(ttroot),	/**< TTH root in ALOC v0.1 */
 	GGEP_ID(u),			/**< HUGE URN in ASCII */
 
 #undef GGEP_ID
@@ -212,7 +225,7 @@ static const struct rwtable ggeptable[] =
 static ext_token_t
 rw_screen(gboolean case_sensitive,
 	const struct rwtable *table, size_t size,
-	const gchar *word, const gchar **retkw)
+	const char *word, const char **retkw)
 {
 	g_assert(retkw);
 
@@ -225,10 +238,10 @@ rw_screen(gboolean case_sensitive,
 	} G_STMT_END
 
 	if (case_sensitive)
-		BINARY_SEARCH(const gchar *, word, size,
+		BINARY_SEARCH(const char *, word, size,
 				strcmp, GET_KEY, FOUND);
 	else
-		BINARY_SEARCH(const gchar *, word, size,
+		BINARY_SEARCH(const char *, word, size,
 				ascii_strcasecmp, GET_KEY, FOUND);
 
 #undef FOUND
@@ -242,7 +255,7 @@ rw_screen(gboolean case_sensitive,
  * Ensure the reserved-word table is lexically sorted.
  */
 static void
-rw_is_sorted(const gchar *name,
+rw_is_sorted(const char *name,
 	const struct rwtable *table, size_t size)
 {
 	size_t i;
@@ -260,7 +273,7 @@ rw_is_sorted(const gchar *name,
 				name, e->rw_name);
 
 		if (ggeptable == table) {
-			const gchar *s;
+			const char *s;
 
 		   	s = ext_ggep_name(e->rw_token);
 			if (0 != strcmp(s, e->rw_name)) {
@@ -276,7 +289,7 @@ rw_is_sorted(const gchar *name,
  * If keyword was found, its static shared string is returned in `retkw'.
  */
 static ext_token_t
-rw_ggep_screen(gchar *word, const gchar **retkw)
+rw_ggep_screen(char *word, const char **retkw)
 {
 	ext_token_t t;
 
@@ -290,7 +303,7 @@ rw_ggep_screen(gchar *word, const gchar **retkw)
  * If keyword was found, its static shared string is returned in `retkw'.
  */
 static ext_token_t
-rw_urn_screen(const gchar *word, const gchar **retkw)
+rw_urn_screen(const char *word, const char **retkw)
 {
 	return rw_screen(FALSE, urntable, G_N_ELEMENTS(urntable), word, retkw);
 }
@@ -306,11 +319,11 @@ static GHashTable *ext_names = NULL;
  *
  * @return an atom string of that printable form.
  */
-static gchar *
-ext_name_atom(const gchar *name)
+static char *
+ext_name_atom(const char *name)
 {
-	gchar *key;
-	gchar *atom;
+	char *key;
+	char *atom;
 
 	/*
 	 * Look whether we already known about this name.
@@ -374,20 +387,20 @@ ext_names_kv_free(gpointer key, gpointer value, gpointer unused_udata)
 /**
  * Parses a GGEP block (can hold several extensions).
  */
-static gint
-ext_ggep_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+static int
+ext_ggep_parse(const char **retp, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = *retp;
-	const gchar *end = &p[len];
-	const gchar *lastp = p;				/* Last parsed point */
-	gint count;
+	const char *p = *retp;
+	const char *end = &p[len];
+	const char *lastp = p;				/* Last parsed point */
+	int count;
 
 	for (count = 0; count < exvcnt && p < end; /* empty */) {
 		guchar flags;
-		gchar id[GGEP_F_IDLEN + 1];
+		char id[GGEP_F_IDLEN + 1];
 		guint id_len, data_length, i;
 		gboolean length_ended = FALSE;
-		const gchar *name;
+		const char *name;
 		extdesc_t *d;
 
 		g_assert(exv->opaque == NULL);
@@ -421,7 +434,7 @@ ext_ggep_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 		 */
 
 		for (i = 0; i < id_len; i++) {
-			gint c = *p++;
+			int c = *p++;
 			if (c == '\0' || !isascii(c) || is_ascii_cntrl(c))
 				goto abort;
 			id[i] = c; 
@@ -580,11 +593,11 @@ abort:
 	return 0;		/* Cannot be a GGEP block: leave parsing pointer intact */
 }
 
-static gint
-ext_urn_bad_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+static int
+ext_urn_bad_parse(const char **retp, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = *retp;
-	const gchar *lastp = p;				/* Last parsed point */
+	const char *p = *retp;
+	const char *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -623,16 +636,16 @@ ext_urn_bad_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 /**
  * Parses a URN block (one URN only).
  */
-static gint
-ext_huge_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+static int
+ext_huge_parse(const char **retp, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = *retp;
-	const gchar *end = &p[len];
-	const gchar *lastp = p;				/* Last parsed point */
+	const char *p = *retp;
+	const char *end = &p[len];
+	const char *lastp = p;				/* Last parsed point */
 	ext_token_t token;
-	const gchar *payload_start = NULL;
-	gint data_length;
-	const gchar *name = NULL;
+	const char *payload_start = NULL;
+	int data_length;
+	const char *name = NULL;
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -669,9 +682,9 @@ ext_huge_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	 */
 
 	{
-		const gchar *name_start, *name_end;
+		const char *name_start, *name_end;
 		size_t name_len;
-		gchar name_buf[16];
+		char name_buf[16];
 
 		name_start = p;
 		name_end = memchr(p, ':', end - name_start);
@@ -734,12 +747,12 @@ found:
 /**
  * Parses a XML block (grabs the whole xml up to the first NUL or separator).
  */
-static gint
-ext_xml_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+static int
+ext_xml_parse(const char **retp, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = *retp;
-	const gchar *end = &p[len];
-	const gchar *lastp = p;				/* Last parsed point */
+	const char *p = *retp;
+	const char *end = &p[len];
+	const char *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -781,12 +794,12 @@ ext_xml_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
  *
  * If `skip' is TRUE, we don't resync on the first resync point.
  */
-static gint
-ext_unknown_parse(const gchar **retp, gint len, extvec_t *exv,
-	gint exvcnt, gboolean skip)
+static int
+ext_unknown_parse(const char **retp, int len, extvec_t *exv,
+	int exvcnt, gboolean skip)
 {
-	const gchar *p = *retp;
-	const gchar *lastp = p;				/* Last parsed point */
+	const char *p = *retp;
+	const char *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -857,12 +870,12 @@ ext_unknown_parse(const gchar **retp, gint len, extvec_t *exv,
  * If more that one separator in a row is found, they are all wrapped as a
  * "none" extension.
  */
-static gint
-ext_none_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+static int
+ext_none_parse(const char **retp, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = *retp;
-	const gchar *end = &p[len];
-	const gchar *lastp = p;				/* Last parsed point */
+	const char *p = *retp;
+	const char *end = &p[len];
+	const char *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -912,9 +925,9 @@ ext_none_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 static void
 ext_merge_adjacent(extvec_t *exv, extvec_t *next)
 {
-	const gchar *end;
-	const gchar *nend;
-	const gchar *nbase;
+	const char *end;
+	const char *nend;
+	const char *nbase;
 	guint16 added;
 	extdesc_t *d = exv->opaque;
 	extdesc_t *nd = next->opaque;
@@ -970,11 +983,11 @@ ext_merge_adjacent(extvec_t *exv, extvec_t *next)
  *
  * @return the number of filled entries.
  */
-gint
-ext_parse(const gchar *buf, gint len, extvec_t *exv, gint exvcnt)
+int
+ext_parse(const char *buf, int len, extvec_t *exv, int exvcnt)
 {
-	const gchar *p = buf, *end = &buf[len];
-	gint cnt = 0;
+	const char *p = buf, *end = &buf[len];
+	int cnt = 0;
 
 	g_assert(buf);
 	g_assert(len > 0);
@@ -983,8 +996,8 @@ ext_parse(const gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 	g_assert(exv->opaque == NULL);
 
 	while (p < end && exvcnt > 0) {
-		const gchar *old_p = p;
-		gint found = 0;
+		const char *old_p = p;
+		int found = 0;
 
 		g_assert(len > 0);
 
@@ -1086,14 +1099,14 @@ out:
  * @returns the allocated inflated buffer, and its inflated length in `retlen'.
  * @returns NULL on error.
  */
-static gchar *
-ext_ggep_inflate(const gchar *buf, gint len, guint16 *retlen, const gchar *name)
+static char *
+ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 {
-	gchar *result;					/* Inflated buffer */
-	gint rsize;						/* Result's buffer size */
+	char *result;					/* Inflated buffer */
+	int rsize;						/* Result's buffer size */
 	z_streamp inz;
-	gint ret;
-	gint inflated;					/* Amount of inflated data so far */
+	int ret;
+	int inflated;					/* Amount of inflated data so far */
 	gboolean failed = FALSE;
 
 	g_assert(buf);
@@ -1214,9 +1227,9 @@ ext_ggep_inflate(const gchar *buf, gint len, guint16 *retlen, const gchar *name)
 static void
 ext_ggep_decode(const extvec_t *e)
 {
-	const gchar *pbase;				/* Current payload base */
+	const char *pbase;				/* Current payload base */
 	size_t plen;					/* Curernt payload length */
-	gchar *uncobs = NULL;			/* COBS-decoded buffer */
+	char *uncobs = NULL;			/* COBS-decoded buffer */
 	size_t uncobs_len = 0;			/* Length of walloc()'ed buffer */
 	size_t result;					/* Decoded length */
 	extdesc_t *d;
@@ -1358,7 +1371,7 @@ ext_paylen(const extvec_t *e)
  * of the header: don't read past the ext_headlen() first bytes of the
  * header.
  */
-const gchar *
+const char *
 ext_base(const extvec_t *e)
 {
 	extdesc_t *d = e->opaque;
@@ -1388,7 +1401,7 @@ guint16
 ext_len(const extvec_t *e)
 {
 	extdesc_t *d = e->opaque;
-	gint headlen;
+	int headlen;
 
 	g_assert(e->opaque != NULL);
 
@@ -1403,7 +1416,7 @@ ext_len(const extvec_t *e)
 /**
  * @returns extension's GGEP ID, or "" if not a GGEP one.
  */
-const gchar *
+const char *
 ext_ggep_id_str(const extvec_t *e)
 {
 	extdesc_t *d = e->opaque;
@@ -1470,8 +1483,8 @@ ext_has_ascii_word(const extvec_t *e)
  * Dump an extension to specified stdio stream.
  */
 static void
-ext_dump_one(FILE *f, const extvec_t *e, const gchar *prefix,
-	const gchar *postfix, gboolean payload)
+ext_dump_one(FILE *f, const extvec_t *e, const char *prefix,
+	const char *postfix, gboolean payload)
 {
 	guint16 paylen;
 
@@ -1529,10 +1542,10 @@ ext_dump_one(FILE *f, const extvec_t *e, const gchar *prefix,
  * non-printable characters, as text otherwise.
  */
 void
-ext_dump(FILE *fd, const extvec_t *exv, gint exvcnt,
-	const gchar *prefix, const gchar *postfix, gboolean payload)
+ext_dump(FILE *fd, const extvec_t *exv, int exvcnt,
+	const char *prefix, const char *postfix, gboolean payload)
 {
-	gint i;
+	int i;
 
 	for (i = 0; i < exvcnt; i++)
 		ext_dump_one(fd, &exv[i], prefix, postfix, payload);
@@ -1543,9 +1556,9 @@ ext_dump(FILE *fd, const extvec_t *exv, gint exvcnt,
  * all set to NULL.
  */
 void
-ext_prepare(extvec_t *exv, gint exvcnt)
+ext_prepare(extvec_t *exv, int exvcnt)
 {
-	gint i;
+	int i;
 
 	for (i = 0; i < exvcnt; i++)
 		exv[i].opaque = NULL;
@@ -1556,9 +1569,9 @@ ext_prepare(extvec_t *exv, gint exvcnt)
  * and of any allocated "virtual" payload.
  */
 void
-ext_reset(extvec_t *exv, gint exvcnt)
+ext_reset(extvec_t *exv, int exvcnt)
 {
-	gint i;
+	int i;
 	
 	for (i = 0; i < exvcnt; i++) {
 		extvec_t *e = &exv[i];
@@ -1572,9 +1585,10 @@ ext_reset(extvec_t *exv, gint exvcnt)
 		if (d->ext_payload != NULL && d->ext_payload != d->ext_phys_payload) {
 			gpointer p = deconstify_gpointer(d->ext_payload);
 			if (d->ext_rpaylen == 0) {
-				g_free(p);
+				G_FREE_NULL(p);
 			} else {
 				wfree(p, d->ext_rpaylen);
+				p = NULL;
 			}
 			d->ext_payload = NULL;
 		}
@@ -1584,7 +1598,7 @@ ext_reset(extvec_t *exv, gint exvcnt)
 	}
 }
 
-const gchar *
+const char *
 ext_ggep_name(ext_token_t id)
 {
 	size_t i;
