@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.h 15154 2008-03-19 00:25:28Z cbiere $
+ * $Id: extensions.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -82,6 +82,8 @@ typedef enum ext_token {
 	EXT_T_GGEP_BH,			/**< Browseable host indication */
 	EXT_T_GGEP_C,			/**< Result Code in HEAD Pongs */
 	EXT_T_GGEP_CT,			/**< Resource creation time */
+	EXT_T_GGEP_DHT,			/**< DHT version and flags, in pongs */
+	EXT_T_GGEP_DHTIPP,		/**< DHT nodes in packed IP:Port format (pongs) */
 	EXT_T_GGEP_DU,			/**< Daily Uptime */
 	EXT_T_GGEP_F,			/**< Flags in HEAD Pongs */
 	EXT_T_GGEP_FW,			/**< Firewalled-to-Firewalled protocol version */
@@ -90,6 +92,7 @@ typedef enum ext_token {
 	EXT_T_GGEP_GTKG_TLS,	/**< GTKG TLS support indication */
 	/* watch out, below is off-order */
 	EXT_T_GGEP_GTKGV1,		/**< GTKG version indication #1 */
+	/* keep remaining sorted */
 	EXT_T_GGEP_GUE,			/**< GUESS support */
 	EXT_T_GGEP_H,			/**< GGEP binary hash value */
 	EXT_T_GGEP_HNAME,		/**< Hostname info, in query hits */
@@ -110,12 +113,22 @@ typedef enum ext_token {
 	EXT_T_GGEP_SO,			/**< Secure OOB */
 	EXT_T_GGEP_T,			/**< Same as ALT_TLS but for HEAD Pongs */
 	EXT_T_GGEP_TLS,			/**< Supports TLS */
+	EXT_T_GGEP_TT,			/**< Tigertree root hash (TTH); binary */
 	EXT_T_GGEP_UA,			/**< User-Agent string */
 	EXT_T_GGEP_UDPHC,		/**< UDP HostCache, in pongs (UHC) */
 	EXT_T_GGEP_UP,			/**< UltraPeer information */
 	EXT_T_GGEP_V,			/**< Vendor Code in HEAD Pongs */
 	EXT_T_GGEP_VC,			/**< Vendor Code */
 	EXT_T_GGEP_VMSG,		/**< Array of vendor message codes supported */
+	EXT_T_GGEP_client_id,	/**< "client-id" in ALOC & PROX v0.0 (i.e. GUID) */
+	EXT_T_GGEP_features,	/**< Unknown value, PROX v0.0 */
+	EXT_T_GGEP_firewalled,	/**< Firewalled status in ALOC v0.0 */
+	EXT_T_GGEP_fwt_version,	/**< Fw-to-fw transfer version, PROX v0.0 */
+	EXT_T_GGEP_length,		/**< File length in ALOC v0.1 */
+	EXT_T_GGEP_port,		/**< Servent's Port in ALOC v0.0 */
+	EXT_T_GGEP_proxies,		/**< Push proxies in PROX v0.0 */
+	EXT_T_GGEP_tls,			/**< Servent TLS support indication in ALOC v0.1 */
+	EXT_T_GGEP_ttroot,		/**< TTH root in ALOC v0.1 */
 	EXT_T_GGEP_u,			/**< HUGE URN in ASCII */
 
 	EXT_T_TOKEN_COUNT
@@ -147,7 +160,7 @@ typedef enum ext_token {
  * and access to the whole length via ext_len().
  */
 typedef struct extvec {
-	const gchar *ext_name;	/**< Extension name (may be NULL) */
+	const char *ext_name;	/**< Extension name (may be NULL) */
 	ext_token_t ext_token;	/**< Extension token */
 	ext_type_t ext_type;	/**< Extension type */
 	gpointer opaque;		/**< Internal information */
@@ -162,24 +175,24 @@ typedef struct extvec {
 void ext_init(void);
 void ext_close(void);
 
-void ext_prepare(extvec_t *exv, gint exvcnt);
-gint ext_parse(const gchar *buf, gint len, extvec_t *exv, gint exvcnt);
-void ext_reset(extvec_t *exv, gint exvcnt);
+void ext_prepare(extvec_t *exv, int exvcnt);
+int ext_parse(const char *buf, int len, extvec_t *exv, int exvcnt);
+void ext_reset(extvec_t *exv, int exvcnt);
 
 gboolean ext_is_printable(const extvec_t *e);
 gboolean ext_is_ascii(const extvec_t *e);
 gboolean ext_has_ascii_word(const extvec_t *e);
 
-void ext_dump(FILE *fd, const extvec_t *extvec, gint extcnt,
-	const gchar *prefix, const gchar *postfix, gboolean payload);
+void ext_dump(FILE *fd, const extvec_t *extvec, int extcnt,
+	const char *prefix, const char *postfix, gboolean payload);
 
 gconstpointer ext_payload(const extvec_t *e);
 guint16 ext_paylen(const extvec_t *e);
-const gchar *ext_base(const extvec_t *e);
+const char *ext_base(const extvec_t *e);
 guint16 ext_headlen(const extvec_t *e);
 guint16 ext_len(const extvec_t *e);
-const gchar *ext_ggep_id_str(const extvec_t *e);
-const gchar *ext_ggep_name(ext_token_t id);
+const char *ext_ggep_id_str(const extvec_t *e);
+const char *ext_ggep_name(ext_token_t id);
 
 #endif	/* _core_extensions_h_ */
 
