@@ -1,5 +1,5 @@
 /*
- * $Id: ggep_type.c 15056 2007-12-09 01:32:27Z cbiere $
+ * $Id: ggep_type.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,18 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: ggep_type.c 15056 2007-12-09 01:32:27Z cbiere $")
+RCSID("$Id: ggep_type.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
 #include "ggep.h"
 #include "ggep_type.h"
 #include "hosts.h"				/* For struct gnutella_host */
 
 #include "lib/endian.h"
+#include "lib/gnet_host.h"
 #include "lib/misc.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
-#include "if/core/hosts.h"
 #include "if/core/search.h"
 
 #include "lib/override.h"		/* Must be the last header included */
@@ -128,9 +128,9 @@ ggept_h_tth_extract(extvec_t *exv, struct tth *tth)
 ggept_status_t
 ggept_gtkgv1_extract(extvec_t *exv, struct ggep_gtkgv1 *info)
 {
-	const gchar *payload;
-	const gchar *p;
-	gint tlen;
+	const char *payload;
+	const char *p;
+	int tlen;
 
 	g_assert(exv->ext_type == EXT_GGEP);
 	g_assert(exv->ext_token == EXT_T_GGEP_GTKGV1);
@@ -155,7 +155,7 @@ ggept_gtkgv1_extract(extvec_t *exv, struct ggep_gtkgv1 *info)
 static ggept_status_t
 ggept_ip_vec_extract(extvec_t *exv, gnet_host_vec_t **hvec)
 {
-	gint len;
+	int len;
 
 	g_assert(exv);
 	g_assert(hvec);
@@ -171,7 +171,7 @@ ggept_ip_vec_extract(extvec_t *exv, gnet_host_vec_t **hvec)
 
 	if (hvec) {
 		gnet_host_vec_t *vec;
-		const gchar *p;
+		const char *p;
 		guint n, i;
 
 		vec = gnet_host_vec_alloc();
@@ -232,11 +232,11 @@ ggept_push_extract(extvec_t *exv, gnet_host_vec_t **hvec)
  * extracted something in the supplied buffer.
  */
 ggept_status_t
-ggept_hname_extract(extvec_t *exv, gchar *buf, gint len)
+ggept_hname_extract(extvec_t *exv, char *buf, int len)
 {
-	gint tlen;
-	gint slen;
-	const gchar *payload;
+	int tlen;
+	int slen;
+	const char *payload;
 
 	g_assert(len >= 0);
 	g_assert(exv->ext_type == EXT_GGEP);
@@ -266,7 +266,7 @@ ggept_hname_extract(extvec_t *exv, gchar *buf, gint len)
 	 * IP address.
 	 */
 	{
-		const gchar *endptr;
+		const char *endptr;
 		host_addr_t addr;
 
 		if (
@@ -291,10 +291,10 @@ ggept_hname_extract(extvec_t *exv, gchar *buf, gint len)
  *
  * @return the length in bytes of the encoded variable-length integer.
  */
-static inline gint
-ggep_vlint_encode(guint64 v, gchar *data)
+static inline int
+ggep_vlint_encode(guint64 v, char *data)
 {
-	gchar *p;
+	char *p;
 
 	for (p = data; v != 0; v >>= 8)	{
 		*p++ = v & 0xff;
@@ -313,7 +313,7 @@ ggep_vlint_encode(guint64 v, gchar *data)
  * @return The decoded value.
  */
 static inline guint64
-ggep_vlint_decode(const gchar *data, size_t len)
+ggep_vlint_decode(const char *data, size_t len)
 {
 	guint64 v;
 	guint i;
@@ -328,16 +328,18 @@ ggep_vlint_decode(const gchar *data, size_t len)
 }
 
 /**
- * Extract filesize length into `filesize' from GGEP "LF" extension.
+ * Extract filesize length into `filesize' from extension encoded in variable-
+ * length little endian with leading zeroes stripped.
+ *
+ * This is the format used by the payload of GGEP "LF" for instance.
  */
 ggept_status_t
-ggept_lf_extract(extvec_t *exv, guint64 *filesize)
+ggept_filesize_extract(extvec_t *exv, guint64 *filesize)
 {
 	guint64 fs;
 	size_t len;
 
 	g_assert(exv->ext_type == EXT_GGEP);
-	g_assert(exv->ext_token == EXT_T_GGEP_LF);
 
 	len = ext_paylen(exv);
 	if (len < 1 || len > 8) {
@@ -384,14 +386,18 @@ ggept_gtkg_ipv6_extract(extvec_t *exv, host_addr_t *addr)
 
 
 /**
- * Encode `filesize' for the GGEP "LF" extension into `data'.
+ * Encode `filesize' in variable-length little endian, with leading zeroes
+ * stripped, into `data'.
  *
- * @param filesize The filesize to encode.
- * @param data A buffer of at least 8 bytes.
- * @return the amount of chars written.
+ * This is used in extensions such as GGEP "LF" which carry the file length.
+ *
+ * @param filesize	The filesize to encode.
+ * @param data		A buffer of at least 8 bytes.
+ *
+ * @return the amount of bytes written.
  */
 guint
-ggept_lf_encode(guint64 filesize, gchar *data)
+ggept_filesize_encode(guint64 filesize, char *data)
 {
 	return ggep_vlint_encode(filesize, data);
 }
@@ -427,7 +433,7 @@ ggept_du_extract(extvec_t *exv, guint32 *uptime)
  * @return the amount of chars written.
  */
 guint
-ggept_du_encode(guint32 uptime, gchar *data)
+ggept_du_encode(guint32 uptime, char *data)
 {
 	return ggep_vlint_encode(uptime, data);
 }
@@ -460,7 +466,7 @@ ggept_ct_extract(extvec_t *exv, time_t *stamp_ptr)
  * @return the amount of chars written.
  */
 guint
-ggept_ct_encode(time_t timestamp, gchar *data)
+ggept_ct_encode(time_t timestamp, char *data)
 {
 	return ggep_vlint_encode(timestamp, data);
 }
