@@ -1,5 +1,5 @@
 /*
- * $Id: gmsg.c 14719 2007-08-31 01:55:05Z cbiere $
+ * $Id: gmsg.c 16200 2009-03-18 19:43:17Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,30 +35,33 @@
 
 #include "common.h"
 
-RCSID("$Id: gmsg.c 14719 2007-08-31 01:55:05Z cbiere $")
+RCSID("$Id: gmsg.c 16200 2009-03-18 19:43:17Z rmanfredi $")
 
 #include <zlib.h>	/* Z_DEFAULT_COMPRESSION */
 
 #include "gmsg.h"
-#include "pmsg.h"
 #include "nodes.h"
 #include "sq.h"
-#include "mq.h"
+#include "mq_tcp.h"
 #include "routing.h"
 #include "vmsg.h"
 #include "search.h"
 #include "gnet_stats.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/dht/kmsg.h"
+#include "if/dht/kademlia.h"
 
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-static const gchar *msg_name[256];
+static const char *msg_name[256];
 static guint8 msg_weight[256];	/**< For gmsg_cmp() */
+static guint8 kmsg_weight[256];	/**< For gmsg_cmp() */
 
 /**
  * Ensure that the gnutella message header has the correct size,
@@ -134,7 +137,7 @@ gmsg_dump(FILE *out, gconstpointer data, guint32 size)
 	g_assert(size >= GTA_HEADER_SIZE);
 
 	dump_hex(out, gmsg_infostr_full(data),
-		(gchar *) data + GTA_HEADER_SIZE, size - GTA_HEADER_SIZE);
+		(char *) data + GTA_HEADER_SIZE, size - GTA_HEADER_SIZE);
 }
 
 /**
@@ -156,19 +159,17 @@ gmsg_split_dump(FILE *out, gconstpointer head, gconstpointer data,
 void
 gmsg_init(void)
 {
-	gint i;
+	int i;
 
 	for (i = 0; i < 256; i++) {
-		const gchar *s;
-		guint w;
-
-		s = "unknown";
-		w = 0;
+		const char *s = "unknown";
+		guint w = 0;
 
 		switch ((enum gta_msg) i) {
+		case GTA_MSG_DHT:            w = 0; s = "DHT"; break;
 		case GTA_MSG_INIT:           w = 1; s = "Ping"; break;
-		case GTA_MSG_INIT_RESPONSE:  w = 3; s = "Pong"; break;
 		case GTA_MSG_SEARCH:         w = 2; s = "Query"; break;
+		case GTA_MSG_INIT_RESPONSE:  w = 3; s = "Pong"; break;
 		case GTA_MSG_SEARCH_RESULTS: w = 4; s = "Q-Hit"; break;
 		case GTA_MSG_PUSH_REQUEST:   w = 5; s = "Push"; break;
 		case GTA_MSG_RUDP:   		 		s = "RUDP"; break;
@@ -181,12 +182,43 @@ gmsg_init(void)
 		msg_name[i] = s;
 		msg_weight[i] = w;
 	}
+
+	/*
+	 * We need to be able to compare Gnutella and Kademlia messages since
+	 * they can both be found in the same UDP queue.
+	 *
+	 * The messages with a weight of 0 can be dropped quite safely in
+	 * flow-control situations.  See also kmsg_can_drop().
+	 *
+	 * NB: This is defined here and not in the DHT sources to avoid a costly
+	 * function call in gmsg_cmp() and also because respective weights of
+	 * Gnutella and Kademlia messages must be defined with knowledge of each
+	 * other.
+	 */
+
+	for (i = 0; i < 256; i++) {
+		guint w = 0;
+
+		switch ((enum kda_msg) i) {
+		case KDA_MSG_PING_REQUEST:        w = 1; break;
+		case KDA_MSG_PING_RESPONSE:       w = 2; break;
+		case KDA_MSG_STORE_REQUEST:       w = 0; break;
+		case KDA_MSG_STORE_RESPONSE:      w = 3; break;
+		case KDA_MSG_FIND_NODE_REQUEST:   w = 0; break;
+		case KDA_MSG_FIND_NODE_RESPONSE:  w = 0; break;
+		case KDA_MSG_FIND_VALUE_REQUEST:  w = 0; break;
+		case KDA_MSG_FIND_VALUE_RESPONSE: w = 7; break;
+		case KDA_MSG_STATS_REQUEST:       w = 0; break;	/* UNUSED */
+		case KDA_MSG_STATS_RESPONSE:      w = 0; break;	/* UNUSED */
+		}
+		kmsg_weight[i] = w;
+	}
 }
 
 /**
  * Convert message function number into name.
  */
-const gchar *
+const char *
 gmsg_name(guint function)
 {
 	if (function > 255)
@@ -244,7 +276,7 @@ gmsg_to_deflated_pmsg(gconstpointer msg, guint32 size)
 	 */
 
 	buf = walloc(blen);
-	data = (const gchar *) msg + GTA_HEADER_SIZE;
+	data = (const char *) msg + GTA_HEADER_SIZE;
 	z = zlib_deflater_make_into(data, plen, buf, blen, Z_DEFAULT_COMPRESSION);
 
 	switch (zlib_deflate(z, plen)) {
@@ -419,10 +451,32 @@ gmsg_mb_sendto_all(const GSList *sl, pmsg_t *mb)
 		struct gnutella_node *dn = sl->data;
 		if (!NODE_IS_ESTABLISHED(dn))
 			continue;
-		mq_putq(dn->outq, pmsg_clone(mb));
+		mq_tcp_putq(dn->outq, pmsg_clone(mb), NULL);
 	}
 }
 
+/**
+ * Route message to one node.
+ *
+ * The supplied mb is NOT cloned, it is up to the caller to ensure that
+ * a private instance is supplied.
+ */
+void
+gmsg_mb_routeto_one(const struct gnutella_node *from,
+	const struct gnutella_node *to, pmsg_t *mb)
+{
+	g_assert(!pmsg_was_sent(mb));
+	gmsg_header_check(cast_to_gconstpointer(pmsg_start(mb)), pmsg_size(mb));
+
+	if (!NODE_IS_WRITABLE(to))
+		return;
+
+	if (GNET_PROPERTY(gmsg_debug) > 5 && gmsg_hops(pmsg_start(mb)) == 0)
+		gmsg_dump(stdout, pmsg_start(mb), pmsg_size(mb));
+
+	mq_tcp_putq(to->outq, mb, from);
+}
+
 /**
  * Send message to one node.
  *
@@ -430,18 +484,9 @@ gmsg_mb_sendto_all(const GSList *sl, pmsg_t *mb)
  * a private instance is supplied.
  */
 void
-gmsg_mb_sendto_one(struct gnutella_node *n, pmsg_t *mb)
+gmsg_mb_sendto_one(const struct gnutella_node *n, pmsg_t *mb)
 {
-	g_assert(!pmsg_was_sent(mb));
-	gmsg_header_check(cast_to_gconstpointer(pmsg_start(mb)), pmsg_size(mb));
-
-	if (!NODE_IS_WRITABLE(n))
-		return;
-
-	if (GNET_PROPERTY(gmsg_debug) > 5 && gmsg_hops(pmsg_start(mb)) == 0)
-		gmsg_dump(stdout, pmsg_start(mb), pmsg_size(mb));
-
-	mq_putq(n->outq, mb);
+	gmsg_mb_routeto_one(NULL, n, mb);
 }
 
 /**
@@ -460,7 +505,7 @@ gmsg_sendto_one(struct gnutella_node *n, gconstpointer msg, guint32 size)
 	if (GNET_PROPERTY(gmsg_debug) > 5 && gmsg_hops(msg) == 0)
 		gmsg_dump(stdout, msg, size);
 
-	mq_putq(n->outq, gmsg_to_pmsg(msg, size));
+	mq_tcp_putq(n->outq, gmsg_to_pmsg(msg, size), NULL);
 }
 
 /**
@@ -481,7 +526,7 @@ gmsg_ctrl_sendto_one(struct gnutella_node *n, gconstpointer msg, guint32 size)
 	if (GNET_PROPERTY(gmsg_debug) > 6 && gmsg_hops(msg) == 0)
 		gmsg_dump(stdout, msg, size);
 
-	mq_putq(n->outq, gmsg_to_ctrl_pmsg(msg, size));
+	mq_tcp_putq(n->outq, gmsg_to_ctrl_pmsg(msg, size), NULL);
 }
 
 /**
@@ -508,21 +553,41 @@ gmsg_search_sendto_one(
 /**
  * Send message consisting of header and data to one node.
  */
-void
-gmsg_split_sendto_one(struct gnutella_node *n,
+static void
+gmsg_split_send_from_to(struct gnutella_node *from, struct gnutella_node *to,
 	gconstpointer head, gconstpointer data, guint32 size)
 {
-	g_return_if_fail(!NODE_IS_UDP(n));
+	g_return_if_fail(!NODE_IS_UDP(to));
 
 	gmsg_header_check(head, size);
 
-	if (!NODE_IS_WRITABLE(n))
+	if (!NODE_IS_WRITABLE(to))
 		return;
 
 	if (GNET_PROPERTY(gmsg_debug) > 6)
 		gmsg_split_dump(stdout, head, data, size);
 
-	mq_putq(n->outq, gmsg_split_to_pmsg(head, data, size));
+	mq_tcp_putq(to->outq, gmsg_split_to_pmsg(head, data, size), from);
+}
+
+/**
+ * Send message consisting of header and data to one node.
+ */
+void
+gmsg_split_sendto_one(struct gnutella_node *n,
+	gconstpointer head, gconstpointer data, guint32 size)
+{
+	gmsg_split_send_from_to(NULL, n, head, data, size);
+}
+
+/**
+ * Route message consisting of header and data to one node.
+ */
+static void
+gmsg_split_routeto_one(struct gnutella_node *from, struct gnutella_node *to,
+	gconstpointer head, gconstpointer data, guint32 size)
+{
+	gmsg_split_send_from_to(from, to, head, data, size);
 }
 
 /**
@@ -542,7 +607,7 @@ gmsg_sendto_all(const GSList *sl, gconstpointer msg, guint32 size)
 		struct gnutella_node *dn = sl->data;
 		if (!NODE_IS_ESTABLISHED(dn))
 			continue;
-		mq_putq(dn->outq, pmsg_clone(mb));
+		mq_tcp_putq(dn->outq, pmsg_clone(mb), NULL);
 	}
 
 	pmsg_free(mb);
@@ -581,13 +646,14 @@ gmsg_search_sendto_all(
 }
 
 /**
- * Send message consisting of header and data to all nodes in the list
- * but one node.
+ * Route message from ``from'' consisting of header and data to all nodes in
+ * the list but one node ``n''.
  *
  * We never broadcast anything to a leaf node.  Those are handled specially.
  */
-void
-gmsg_split_sendto_all_but_one(const GSList *sl, const struct gnutella_node *n,
+static void
+gmsg_split_routeto_all_but_one(const struct gnutella_node *from,
+	const GSList *sl, const struct gnutella_node *n,
 	gconstpointer head, gconstpointer data, guint32 size)
 {
 	pmsg_t *mb = gmsg_split_to_pmsg(head, data, size);
@@ -618,18 +684,20 @@ gmsg_split_sendto_all_but_one(const GSList *sl, const struct gnutella_node *n,
 			continue;
 		if (n->header_flags && !NODE_CAN_SFLAG(dn))
 			continue;
-		mq_putq(dn->outq, pmsg_clone(mb));
+		mq_tcp_putq(dn->outq, pmsg_clone(mb), from);
 	}
 
 	pmsg_free(mb);
 }
 
 /**
- * Send message consisting of header and data to all the nodes in the list.
+ * Route message consisting of header and data to all the nodes in the list.
  */
 void
-gmsg_split_sendto_all(
-	const GSList *sl, gconstpointer head, gconstpointer data, guint32 size)
+gmsg_split_routeto_all(
+	const GSList *sl,
+	const struct gnutella_node *from,
+	gconstpointer head, gconstpointer data, guint32 size)
 {
 	pmsg_t *mb = gmsg_split_to_pmsg(head, data, size);
 
@@ -647,7 +715,7 @@ gmsg_split_sendto_all(
 		 * We have already tested that the node was being writable.
 		 */
 
-		mq_putq(dn->outq, pmsg_clone(mb));
+		mq_tcp_putq(dn->outq, pmsg_clone(mb), from);
 	}
 
 	pmsg_free(mb);
@@ -678,17 +746,12 @@ gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt)
 			return;
 		}
 
-		gmsg_split_sendto_one(rt_node, &n->header,
+		gmsg_split_routeto_one(n, rt_node, &n->header,
 				n->data, n->size + GTA_HEADER_SIZE);
 		return;
 	case ROUTE_ALL_BUT_ONE:
 		g_assert(n == rt_node);
-		gmsg_split_sendto_all_but_one(node_all_nodes(), rt_node,
-			&n->header, n->data, n->size + GTA_HEADER_SIZE);
-		return;
-	case ROUTE_NO_DUPS_BUT_ONE:
-		g_assert(n == rt_node);
-		gmsg_split_sendto_all_but_one(node_all_but_broken_gtkg(), rt_node,
+		gmsg_split_routeto_all_but_one(n, node_all_nodes(), rt_node,
 			&n->header, n->data, n->size + GTA_HEADER_SIZE);
 		return;
 	case ROUTE_MULTI:
@@ -696,7 +759,7 @@ gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt)
 			rt_node = sl->data;
 			if (n->header_flags && !NODE_CAN_SFLAG(rt_node))
 				continue;
-			gmsg_split_sendto_one(rt_node,
+			gmsg_split_routeto_one(n, rt_node,
 				&n->header, n->data, n->size + GTA_HEADER_SIZE);
 		}
 		return;
@@ -771,7 +834,7 @@ gmsg_install_presend(pmsg_t *mb)
  * and there's not enough room in the queue.
  */
 gboolean
-gmsg_can_drop(gconstpointer pdu, gint size)
+gmsg_can_drop(gconstpointer pdu, int size)
 {
 	if ((size_t) size < GTA_HEADER_SIZE)
 		return TRUE;
@@ -781,6 +844,8 @@ gmsg_can_drop(gconstpointer pdu, gint size)
 	case GTA_MSG_SEARCH:
 	case GTA_MSG_INIT_RESPONSE:
 		return TRUE;
+	case GTA_MSG_DHT:
+		return kmsg_can_drop(pdu, size);
 	default:
 		return FALSE;
 	}
@@ -791,14 +856,19 @@ gmsg_can_drop(gconstpointer pdu, gint size)
  *
  * @return algebraic -1/0/+1 depending on relative order.
  */
-gint
-gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
+int
+gmsg_cmp(gconstpointer h1, gconstpointer h2)
 {
-	gconstpointer h1 = pdu1, h2 = pdu2;
-	gint w1, w2;
-	
-	w1 = msg_weight[gnutella_header_get_function(h1)];
-	w2 = msg_weight[gnutella_header_get_function(h2)];
+	int w1, w2;
+	guint8 f1, f2;
+
+	f1 = gnutella_header_get_function(h1);
+	f2 = gnutella_header_get_function(h2);
+
+	w1 = (f1 == GTA_MSG_DHT) ?
+		kmsg_weight[kademlia_header_get_function(h1)] :  msg_weight[f1];
+	w2 = (f2 == GTA_MSG_DHT) ?
+		kmsg_weight[kademlia_header_get_function(h2)] :  msg_weight[f2];
 
 	/*
 	 * The more weight a message type has, the more prioritary it is.
@@ -810,6 +880,21 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
 	/*
 	 * Same weight.
 	 *
+	 * DHT messages are less prioritary than Gnutella.
+	 * Between 2 DHT messages, keep the shortest.
+	 */
+
+	if (f1 == GTA_MSG_DHT) {
+		return f2 == GTA_MSG_DHT ?
+			CMP(gnutella_header_get_size(h2), gnutella_header_get_size(h1)) :
+			-1;
+	} else if (f2 == GTA_MSG_DHT) {
+		return +1;		/* Gnutella message (f1) more prioritary */
+	}
+
+	/*
+	 * Same weight, both Gnutella messages.
+	 *
 	 * Compare hops.
 	 *
 	 * For queries: the more hops a message has travelled, the less prioritary
@@ -821,7 +906,7 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
 	 */
 
 	if (gnutella_header_get_hops(h1) == gnutella_header_get_hops(h2)) {
-		switch (gnutella_header_get_function(h1)) {
+		switch (f1) {
 		case GTA_MSG_PUSH_REQUEST:
 		case GTA_MSG_SEARCH_RESULTS:
 			return CMP(gnutella_header_get_ttl(h2),
@@ -830,7 +915,7 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
 			return 0;
 		}
 	} else {
-		switch (gnutella_header_get_function(h1)) {
+		switch (f1) {
 		case GTA_MSG_INIT:
 		case GTA_MSG_SEARCH:
 		case GTA_MSG_QRP:
@@ -852,21 +937,29 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
  * message that contains the leading header immediately followed by the
  * payload of that message.
  */
-gchar *
+char *
 gmsg_infostr_full(gconstpointer msg)
 {
-	const gchar *data = (const gchar *) msg + GTA_HEADER_SIZE;
+	const char *data = (const char *) msg + GTA_HEADER_SIZE;
 
 	return gmsg_infostr_full_split(msg, data);
 }
 
+/**
+ * Same a gmsg_infostr() but fills the supplied buffer with the formatted
+ * string and returns the amount of bytes written.
+ */
 static size_t
-gmsg_infostr_to_buf(gconstpointer msg, gchar *buf, size_t buf_size)
+gmsg_infostr_to_buf(gconstpointer msg, char *buf, size_t buf_size)
 {
+	guint8 function = gnutella_header_get_function(msg);
 	guint16 size = gmsg_size(msg);
 
+	if (GTA_MSG_DHT == function)
+		return kmsg_infostr_to_buf(msg, buf, size);
+
 	return gm_snprintf(buf, buf_size, "%s (%u byte%s) %s[hops=%d, TTL=%d]",
-		gmsg_name(gnutella_header_get_function(msg)),
+		gmsg_name(function),
 		size, size == 1 ? "" : "s",
 		gnutella_header_get_ttl(msg) & GTA_UDP_DEFLATED ? "deflated " : "",
 		gnutella_header_get_hops(msg),
@@ -881,10 +974,10 @@ gmsg_infostr_to_buf(gconstpointer msg, gchar *buf, size_t buf_size)
  * that can also decompile vendor messages given a pointer on the header
  * and on the data of the message (which may not be consecutive in memory).
  */
-gchar *
+char *
 gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
 {
-	static gchar a[160];
+	static char a[160];
 
 	switch (gnutella_header_get_function(head)) {
 	case GTA_MSG_VENDOR:
@@ -915,10 +1008,10 @@ gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
  *
  *     msg_type (payload length) [hops=x, TTL=x]
  */
-const gchar *
+const char *
 gmsg_infostr(gconstpointer msg)
 {
-	static gchar buf[80];
+	static char buf[80];
 	gmsg_infostr_to_buf(msg, buf, sizeof buf);
 	return buf;
 }
@@ -926,10 +1019,10 @@ gmsg_infostr(gconstpointer msg)
 /**
  * Same as gmsg_infostr(), but different static buffer.
  */
-static gchar *
+static char *
 gmsg_infostr2(gconstpointer msg)
 {
-	static gchar buf[80];
+	static char buf[80];
 	gmsg_infostr_to_buf(msg, buf, sizeof buf);
 	return buf;
 }
@@ -938,7 +1031,7 @@ gmsg_infostr2(gconstpointer msg)
  * Log dropped message, and reason.
  */
 void
-gmsg_log_dropped(gconstpointer msg, const gchar *reason, ...)
+gmsg_log_dropped(gconstpointer msg, const char *reason, ...)
 {
 	fputs("DROP ", stdout);
 	fputs(gmsg_infostr2(msg), stdout);	/* Allows gmsg_infostr() in arglist */
@@ -958,7 +1051,7 @@ gmsg_log_dropped(gconstpointer msg, const gchar *reason, ...)
  * Log bad message, the node's vendor, and reason.
  */
 void
-gmsg_log_bad(const struct gnutella_node *n, const gchar *reason, ...)
+gmsg_log_bad(const struct gnutella_node *n, const char *reason, ...)
 {
 	g_message("BAD <%s> %s ", node_vendor(n), node_addr(n));
 
