@@ -1,5 +1,5 @@
 /*
- * $Id: gmsg.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: gmsg.h 16170 2009-03-15 18:40:01Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,11 +39,11 @@
 #include "common.h"
 
 #include "gnutella.h"
-#include "pmsg.h"
 
 #include "if/core/search.h"
 
 #include "lib/endian.h"
+#include "lib/pmsg.h"
 
 struct gnutella_node;
 struct route_dest;
@@ -98,7 +98,7 @@ typedef enum {
  */
 
 void gmsg_init(void);
-const gchar *gmsg_name(guint function);
+const char *gmsg_name(guint function);
 gmsg_valid_t gmsg_size_valid(gconstpointer msg, guint16 *size);
 
 pmsg_t *gmsg_to_pmsg(gconstpointer msg, guint32 size);
@@ -112,7 +112,9 @@ pmsg_t * gmsg_split_to_pmsg_extend(gconstpointer head, gconstpointer data,
 			guint32 size, pmsg_free_t free_cb, gpointer arg);
 
 void gmsg_mb_sendto_all(const GSList *sl, pmsg_t *mb);
-void gmsg_mb_sendto_one(struct gnutella_node *n, pmsg_t *mb);
+void gmsg_mb_sendto_one(const struct gnutella_node *n, pmsg_t *mb);
+void gmsg_mb_routeto_one(const struct gnutella_node *from,
+	const struct gnutella_node *to, pmsg_t *mb);
 
 void gmsg_sendto_one(struct gnutella_node *n, gconstpointer msg, guint32 size);
 void gmsg_ctrl_sendto_one(struct gnutella_node *n,
@@ -120,27 +122,25 @@ void gmsg_ctrl_sendto_one(struct gnutella_node *n,
 void gmsg_split_sendto_one(struct gnutella_node *n,
 		gconstpointer head, gconstpointer data, guint32 size);
 void gmsg_sendto_all(const GSList *l, gconstpointer msg, guint32 size);
-void gmsg_split_sendto_all(const GSList *l,
+void gmsg_split_routeto_all(const GSList *l,
+		const struct gnutella_node *from,
 		gconstpointer head, gconstpointer data, guint32 size);
-void gmsg_split_sendto_all_but_one(const GSList *sl,
-		const struct gnutella_node *n, gconstpointer head, gconstpointer data,
-		guint32 size);
 void gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt);
 
-gboolean gmsg_can_drop(gconstpointer pdu, gint size);
+gboolean gmsg_can_drop(gconstpointer pdu, int size);
 gboolean gmsg_is_oob_query(gconstpointer msg);
 gboolean gmsg_split_is_oob_query(gconstpointer head, gconstpointer data);
-gint gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2);
-const gchar *gmsg_infostr(gconstpointer msg);
-gchar *gmsg_infostr_full(gconstpointer msg);
-gchar *gmsg_infostr_full_split(gconstpointer head, gconstpointer data);
+int gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2);
+const char *gmsg_infostr(gconstpointer msg);
+char *gmsg_infostr_full(gconstpointer msg);
+char *gmsg_infostr_full_split(gconstpointer head, gconstpointer data);
 
 void gmsg_install_presend(pmsg_t *mb);
 
 void gmsg_log_dropped(gconstpointer msg,
-	const gchar *reason, ...) G_GNUC_PRINTF(2, 3);
+	const char *reason, ...) G_GNUC_PRINTF(2, 3);
 void gmsg_log_bad(const struct gnutella_node *n,
-	const gchar *reason, ...) G_GNUC_PRINTF(2, 3);
+	const char *reason, ...) G_GNUC_PRINTF(2, 3);
 
 void gmsg_search_sendto_one(struct gnutella_node *n, gnet_search_t sh,
 	gconstpointer msg, guint32 size);
