@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 14601 2007-08-20 19:26:41Z cbiere $
+ * $Id: gnet_stats.c 16177 2009-03-16 17:44:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gnet_stats.c 14601 2007-08-20 19:26:41Z cbiere $")
+RCSID("$Id: gnet_stats.c 16177 2009-03-16 17:44:43Z rmanfredi $")
 
 #include "gnet_stats.h"
 #include "gmsg.h"
@@ -55,10 +55,10 @@ static gnet_stats_t gnet_udp_stats;
  *** Public functions
  ***/
 
-const gchar *
+const char *
 gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 {
-	static const gchar * const msg_drop_reasons[] = {
+	static const char * const msg_drop_reasons[] = {
 		N_("Bad size"),						 /**< MSG_DROP_BAD_SIZE */
 		N_("Too small"),					 /**< MSG_DROP_TOO_SMALL */
 		N_("Too large"),					 /**< MSG_DROP_TOO_LARGE */
@@ -91,9 +91,12 @@ gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 		N_("Evil filename"),				 /**< MSG_DROP_EVIL */
 		N_("Payload inflating error"),		 /**< MSG_DROP_INFLATE_ERROR */
 		N_("Unknown header flags present"),/**< MSG_DROP_UNKNOWN_HEADER_FLAGS */
-		N_("Own search results"),			 /**< MSG_DROP_OWN_RESULTS */
+		N_("Own search results"),			 /**< MSG_DROP_OWN_RESULT */
 		N_("Ancient query format"),			 /**< MSG_DROP_ANCIENT_QUERY */
 		N_("Blank Servent ID"),				 /**< MSG_DROP_BLANK_SERVENT_ID */
+		N_("DHT Invalid security token"),	 /**< MSG_DROP_DHT_INVALID_TOKEN */
+		N_("DHT Too many STORE requests"),	 /**< MSG_DROP_DHT_TOO_MANY_STORE */
+		N_("DHT Malformed message"),		 /**< MSG_DROP_DHT_UNPARSEABLE */
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(msg_drop_reasons) == MSG_DROP_REASON_COUNT);
@@ -102,6 +105,143 @@ gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 	return msg_drop_reasons[reason];
 }
 
+const char *
+gnet_stats_general_to_string(gnr_stats_t type)
+{
+	/* Do NOT translate any of these strings */
+
+	static const char * const type_string[] = {
+	"routing_errors",
+	"local_searches",
+	"local_hits",
+	"local_query_hits",
+	"oob_proxied_query_hits",
+	"oob_queries",
+	"oob_queries_stripped",
+	"dups_with_higher_ttl",
+	"query_oob_proxied_dups",
+	"oob_hits_for_proxied_queries",
+	"oob_hits_with_alien_ip",
+	"unclaimed_oob_hits",
+	"partially_claimed_oob_hits",
+	"spurious_oob_hit_claim",
+	"unrequested_oob_hits",
+	"query_compact_count",
+	"query_compact_size",
+	"query_utf8",
+	"query_sha1",
+	"broadcasted_pushes",
+	"push_proxy_udp_relayed",
+	"push_proxy_tcp_relayed",
+	"push_proxy_broadcasted",
+	"push_proxy_route_not_proxied",
+	"push_proxy_failed",
+	"push_relayed_via_local_route",
+	"push_relayed_via_table_route",
+	"local_dyn_queries",
+	"leaf_dyn_queries",
+	"oob_proxied_queries",
+	"dyn_queries_completed_full",
+	"dyn_queries_completed_partial",
+	"dyn_queries_completed_zero",
+	"dyn_queries_linger_extra",
+	"dyn_queries_linger_results",
+	"dyn_queries_linger_completed",
+	"gtkg_total_queries",
+	"gtkg_requeries",
+	"queries_with_ggep_h",
+	"giv_callbacks",
+	"giv_discarded",
+	"queue_callbacks",
+	"queue_discarded",
+	"udp_bogus_source_ip",
+	"udp_alien_message",
+	"udp_unprocessed_message",
+	"udp_tx_compressed",
+	"udp_rx_compressed",
+	"udp_larger_hence_not_compressed",
+	"consolidated_servers",
+	"dup_downloads_in_consolidation",
+	"discovered_server_guid",
+	"changed_server_guid",
+	"guid_collisions",
+	"received_known_fw_node_info",
+	"attempted_resource_switching",
+	"attempted_resource_switching_after_error",
+	"successful_resource_switching",
+	"successful_plain_resource_switching",
+	"successful_resource_switching_after_error",
+	"queued_after_switching",
+	"sunk_data",
+	"ignored_data",
+	"ignoring_after_mismatch",
+	"ignoring_to_preserve_connection",
+	"ignoring_during_aggressive_swarming",
+	"ignoring_refused",
+	"client_resource_switching",
+	"client_plain_resource_switching",
+	"client_followup_after_error",
+	"parq_slot_resource_switching",
+	"parq_retry_after_violation",
+	"parq_retry_after_kick_out",
+	"parq_slot_limit_overrides",
+	"parq_quick_slots_granted",
+	"parq_queue_sending_attempts",
+	"parq_queue_sent",
+	"parq_queue_follow_ups",
+	"seeding_of_orphan",
+	"dht_estimated_size",
+	"dht_kball_furthest",
+	"dht_kball_closest",
+	"dht_routing_buckets",
+	"dht_routing_leaves",
+	"dht_routing_max_depth",
+	"dht_routing_good_nodes",
+	"dht_routing_stale_nodes",
+	"dht_routing_pending_nodes",
+	"dht_routing_evicted_nodes",
+	"dht_routing_evicted_firewalled_nodes",
+	"dht_routing_promoted_pending_nodes",
+	"dht_completed_bucket_refresh",
+	"dht_denied_unsplitable_bucket_refresh",
+	"dht_bucket_alive_check",
+	"dht_alive_pings_to_good_nodes",
+	"dht_alive_pings_to_stale_nodes",
+	"dht_rejected_value_on_quota",
+	"dht_rejected_value_on_creator",
+	"dht_keys_held",
+	"dht_cached_keys_held",
+	"dht_values_held",
+	"dht_fetch_local_hits",
+	"dht_fetch_local_cached_hits",
+	"dht_returned_expanded_values",
+	"dht_returned_secondary_keys",
+	"dht_claimed_secondary_keys",
+	"dht_returned_expanded_cached_values",
+	"dht_returned_cached_secondary_keys",
+	"dht_claimed_cached_secondary_keys",
+	"dht_published",
+	"dht_removed",
+	"dht_stale_replication",
+	"dht_replication",
+	"dht_republish",
+	"dht_secondary_key_fetch",
+	"dht_dup_values",
+	"dht_kuid_collisions",
+	"dht_own_kuid_collisions",
+	"dht_alt_loc_lookups",
+	"dht_push_proxy_lookups",
+	"dht_successful_alt_loc_lookups",
+	"dht_successful_push_proxy_lookups",
+	"dht_seeding_of_orphan",
+	};
+
+	STATIC_ASSERT(G_N_ELEMENTS(type_string) == GNR_TYPE_COUNT);
+	g_return_val_if_fail(UNSIGNED(type) < G_N_ELEMENTS(type_string),
+		NULL);
+	return type_string[type];
+}
+
 void
 gnet_stats_init(void)
 {
@@ -122,6 +262,7 @@ gnet_stats_init(void)
 		case GTA_MSG_QRP:            m = MSG_QRP; break;
 		case GTA_MSG_HSEP_DATA:		 m = MSG_HSEP; break;
 		case GTA_MSG_BYE:      		 m = MSG_BYE; break;
+		case GTA_MSG_DHT:      		 m = MSG_DHT; break;
 		}
 		stats_lut[i] = m;
 	}
@@ -298,7 +439,7 @@ gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
 	guint type;
 	gnet_stats_t *stats;
 
-	g_assert((gint) reason >= 0 && reason < MSG_DROP_REASON_COUNT);
+	g_assert(UNSIGNED(reason) < MSG_DROP_REASON_COUNT);
 
     size = n->size + sizeof(n->header);
 	type = stats_lut[gnutella_header_get_function(&n->header)];
@@ -320,12 +461,28 @@ gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
 			gnet_stats_drop_reason_to_string(reason));
 }
 
+/**
+ * Update the general stats counter by given signed delta.
+ */
 void
-gnet_stats_count_general(gnr_stats_t type, guint32 x)
+gnet_stats_count_general(gnr_stats_t type, int delta)
 {
-	g_assert((gint) type >= 0 && type < GNR_TYPE_COUNT);
+	size_t i = type;
 
-    gnet_stats.general[type] += x;
+	g_assert(i < GNR_TYPE_COUNT);
+    gnet_stats.general[i] += delta;
+}
+
+/**
+ * Set the general stats counter to the given value.
+ */
+void
+gnet_stats_set_general(gnr_stats_t type, guint64 value)
+{
+	size_t i = type;
+
+	g_assert(i < GNR_TYPE_COUNT);
+    gnet_stats.general[i] = value;
 }
 
 void
@@ -334,7 +491,8 @@ gnet_stats_count_dropped_nosize(
 {
 	guint type;
 	gnet_stats_t *stats;
-	g_assert((gint) reason >= 0 && reason < MSG_DROP_REASON_COUNT);
+
+	g_assert(UNSIGNED(reason) < MSG_DROP_REASON_COUNT);
 
 	type = stats_lut[gnutella_header_get_function(&n->header)];
 	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
@@ -354,22 +512,25 @@ gnet_stats_count_flowc(gconstpointer head)
 	guint t;
 	guint i;
 	guint16 size = gmsg_size(head);
+	guint8 function = gnutella_header_get_function(head);
+	guint8 ttl = gnutella_header_get_ttl(head);
+	guint8 hops = gnutella_header_get_hops(head);
 
-#if 0
-	g_message("FLOWC function=%d ttl=%d hops=%d", h->function, h->ttl, h->hops);
-#endif
+	if (GNET_PROPERTY(node_debug) > 3)
+		g_message("FLOWC function=%d ttl=%d hops=%d", function, ttl, hops);
 
-	t = stats_lut[gnutella_header_get_function(head)];
+	t = stats_lut[function];
 
-	i = MIN(gnutella_header_get_hops(head), STATS_FLOWC_COLUMNS - 1);
+	i = MIN(hops, STATS_FLOWC_COLUMNS - 1);
 	gnet_stats.pkg.flowc_hops[i][t]++;
 	gnet_stats.pkg.flowc_hops[i][MSG_TOTAL]++;
 	gnet_stats.byte.flowc_hops[i][t] += size;
 	gnet_stats.byte.flowc_hops[i][MSG_TOTAL] += size;
 
-	i = MIN(gnutella_header_get_ttl(head), STATS_FLOWC_COLUMNS - 1);
+	i = MIN(ttl, STATS_FLOWC_COLUMNS - 1);
 
-	g_assert(i != 0);			/* Cannot send a message with TTL=0 */
+	/* Cannot send a message with TTL=0 (DHT messages are not Gnutella) */
+	g_assert(function == GTA_MSG_DHT || i != 0);
 
 	gnet_stats.pkg.flowc_ttl[i][t]++;
 	gnet_stats.pkg.flowc_ttl[i][MSG_TOTAL]++;
