@@ -1,5 +1,5 @@
 /*
- * $Id: gnutella.h 14719 2007-08-31 01:55:05Z cbiere $
+ * $Id: gnutella.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -30,6 +30,8 @@
 
 #include "if/core/gnutella.h"
 
+struct guid;
+
 /*
  * Constants
  */
@@ -308,11 +310,11 @@ gnutella_msg_search_get_flags(const void *data)
  * Compute start of search string (which is NUL terminated) in query.
  * The "+2" skips the "flags" field in the query.
  */
-static inline const gchar *
+static inline const char *
 gnutella_msg_search_get_text(const void *data)
 {
 	const guint8 *u8 = data;
-	return (const gchar *) &u8[GTA_HEADER_SIZE + 2];
+	return (const char *) &u8[GTA_HEADER_SIZE + 2];
 }
 
 /* The logic layout of the PUSH message specific payload is as follows:
@@ -349,7 +351,7 @@ gnutella_msg_push_request_header(gnutella_msg_push_request_t *msg)
 
 static inline void
 gnutella_msg_push_request_set_guid(gnutella_msg_push_request_t *msg,
-	const gchar *guid)
+	const struct guid *guid)
 {
 	guint8 *u8 = (void *) msg;
 	memcpy(&u8[GTA_HEADER_SIZE], guid, 16);	
@@ -466,10 +468,10 @@ gnutella_msg_qrp_reset_set_table_length(gnutella_msg_qrp_reset_t *msg,
 
 static inline void
 gnutella_msg_qrp_reset_set_infinity(gnutella_msg_qrp_reset_t *msg,
-	guint8 infinity)
+	guint8 inf_value)
 {
 	guint8 *u8 = (void *) msg;
-	u8[GTA_HEADER_SIZE + 5] = infinity;
+	u8[GTA_HEADER_SIZE + 5] = inf_value;
 }
 
 /* The logic layout of the QRP PATCH message specific payload is as follows:
@@ -596,10 +598,7 @@ static inline guint32
 gnutella_vendor_get_code(const void *data)
 {
 	const guint8 *u8 = data;
-	guint32 code;
-	/* Keep the big-endian byte order */
-	memcpy(&code, &u8[0], 4);
-	return code;
+	return peek_be32(&u8[0]);
 }
 
 static inline void
