@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c 15152 2008-03-18 21:10:06Z cbiere $
+ * $Id: hcache.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -49,7 +49,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hcache.c 15152 2008-03-18 21:10:06Z cbiere $")
+RCSID("$Id: hcache.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "bogons.h"
 #include "hcache.h"
@@ -59,6 +59,7 @@ RCSID("$Id: hcache.c 15152 2008-03-18 21:10:06Z cbiere $")
 #include "pcache.h"
 #include "settings.h"
 
+#include "lib/ascii.h"
 #include "lib/file.h"
 #include "lib/getdate.h"
 #include "lib/hashlist.h"
@@ -93,7 +94,7 @@ typedef struct hostcache_entry {
  * A hostcache table.
  */
 typedef struct hostcache {
-	const gchar		*name;		        /**< Name of the cache */
+	const char		*name;		        /**< Name of the cache */
 	hcache_type_t   type;				/**< Cache type */
 
     gboolean        addr_only;          /**< Use IP only, port always 0 */
@@ -104,7 +105,7 @@ typedef struct hostcache {
     guint           misses;             /**< Misses to the cache */
 
     gnet_property_t hosts_in_catcher;   /**< Property to update host count */
-    gint            mass_update;        /**< If a mass update is in progess */
+    int            mass_update;        /**< If a mass update is in progess */
 } hostcache_t;
 
 static hostcache_t *caches[HCACHE_MAX];
@@ -118,7 +119,7 @@ static gboolean hcache_close_running = FALSE;
  * Has to be in the same order as in the hcache_type_t definition
  * in gnet_nodes.h.
  */
-static const gchar * const names[HCACHE_MAX] = {
+static const char * const names[HCACHE_MAX] = {
     "fresh regular",
     "valid regular",
     "fresh ultra",
@@ -129,7 +130,7 @@ static const gchar * const names[HCACHE_MAX] = {
     "none",
 };
 
-static const gchar * const host_type_names[HOST_MAX] = {
+static const char * const host_type_names[HOST_MAX] = {
     "any",
     "ultra",
 };
@@ -231,7 +232,7 @@ hce_free(struct hostcache_entry *hce)
  * Output contents information about a hostcache.
  */
 static void
-hcache_dump_info(const struct hostcache *hc, const gchar *what)
+hcache_dump_info(const struct hostcache *hc, const char *what)
 {
     g_message("[%s|%s] %u hosts (%u hits, %u misses)",
         hc->name, what, hash_list_length(hc->hostlist), hc->hits, hc->misses);
@@ -468,7 +469,7 @@ hcache_unlink_host(hostcache_t *hc, gnet_host_t *host)
 /**
  * Convert host cache type to string.
  */
-const gchar *
+const char *
 hcache_type_to_string(hcache_type_t type)
 {
 	g_assert((guint) type < HCACHE_MAX);
@@ -479,7 +480,7 @@ hcache_type_to_string(hcache_type_t type)
 /**
  * Convert host type to string.
  */
-const gchar *
+const char *
 host_type_to_string(host_type_t type)
 {
 	g_assert((guint) type < HOST_MAX);
@@ -487,7 +488,7 @@ host_type_to_string(host_type_t type)
 	return host_type_names[type];
 }
 
-static gint
+static int
 hcache_slots_max(hcache_type_t type)
 {
 	g_assert(UNSIGNED(type) < HCACHE_MAX);
@@ -518,10 +519,10 @@ hcache_slots_max(hcache_type_t type)
  * Several types share common pools. Adding a host of one type may
  * affect the number of slots left on other types.
  */
-static gint
+static int
 hcache_slots_left(hcache_type_t type)
 {
-	gint limit, current = 0;
+	int limit, current = 0;
 
     g_assert(UNSIGNED(type) < HCACHE_MAX);
 
@@ -564,7 +565,7 @@ hcache_request_slot(hcache_type_t type)
 
 	return limit > 0
 		&& left > 0
-		&& ((left > limit / 2) || (random_raw() % limit < left));
+		&& ((left > limit / 2) || (random_u32() % limit < left));
 }
 
 /**
@@ -582,7 +583,7 @@ hcache_request_slot(hcache_type_t type)
  */
 static gboolean
 hcache_add_internal(hcache_type_t type, time_t added,
-	const host_addr_t addr, guint16 port, const gchar *what)
+	const host_addr_t addr, guint16 port, const char *what)
 {
 	gnet_host_t *host;
 	hostcache_t *hc;
@@ -642,7 +643,7 @@ hcache_add_internal(hcache_type_t type, time_t added,
 		port >= 6346 &&
 		port <= 6350 &&
 		!host_low_on_pongs &&
-		random_raw() % 256 > 31
+		random_u32() % 256 > 31
 	) {
 		return FALSE;
 	}
@@ -771,7 +772,7 @@ hcache_add_internal(hcache_type_t type, time_t added,
 
 gboolean
 hcache_add(hcache_type_t type,
-	const host_addr_t addr, guint16 port, const gchar *what)
+	const host_addr_t addr, guint16 port, const char *what)
 {
 	return hcache_add_internal(type, tm_time(), addr, port, what);
 }
@@ -781,7 +782,7 @@ hcache_add(hcache_type_t type,
  */
 gboolean
 hcache_add_caught(host_type_t type, const host_addr_t addr, guint16 port,
-	const gchar *what)
+	const char *what)
 {
     switch (type) {
     case HOST_ANY:
@@ -801,7 +802,7 @@ hcache_add_caught(host_type_t type, const host_addr_t addr, guint16 port,
  */
 gboolean
 hcache_add_valid(host_type_t type, const host_addr_t addr, guint16 port,
-	const gchar *what)
+	const char *what)
 {
     switch (type) {
     case HOST_ANY:
@@ -1002,7 +1003,7 @@ void
 hcache_prune(hcache_type_t type)
 {
 	hostcache_t *hc;
-    gint extra;
+    int extra;
 
     g_assert((guint) type < HCACHE_MAX);
 
@@ -1060,10 +1061,10 @@ hcache_prune(hcache_type_t type)
  *
  * @return amount of hosts filled
  */
-gint
-hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
+int
+hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, int hcount)
 {
-	gint i;
+	int i;
 	hostcache_t *hc = NULL;
 	GHashTable *seen_host = g_hash_table_new(host_hash, host_eq);
 	hash_list_iter_t *iter;
@@ -1230,7 +1231,7 @@ hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
  * amount of hosts we can store.
  */
 static hostcache_t *
-hcache_alloc(hcache_type_t type, gnet_property_t catcher, const gchar *name)
+hcache_alloc(hcache_type_t type, gnet_property_t catcher, const char *name)
 {
 	struct hostcache *hc;
 
@@ -1266,7 +1267,7 @@ hcache_free(hostcache_t *hc)
 static void
 hcache_load_file(hostcache_t *hc, FILE *f)
 {
-	gchar buffer[1024];
+	char buffer[1024];
 	time_t now;
 
 	g_return_if_fail(hc);
@@ -1274,7 +1275,7 @@ hcache_load_file(hostcache_t *hc, FILE *f)
 
 	now = tm_time();
 	while (fgets(buffer, sizeof buffer, f)) {
-		const gchar *endptr;
+		const char *endptr;
 		host_addr_t addr;
 		guint16 port;
 		time_t added;
@@ -1306,7 +1307,7 @@ hcache_load_file(hostcache_t *hc, FILE *f)
  * Loads caught hosts from text file.
  */
 static void
-hcache_retrieve(hostcache_t *hc, const gchar *filename)
+hcache_retrieve(hostcache_t *hc, const char *filename)
 {
 	file_path_t fp[1];
 	FILE *f;
@@ -1347,7 +1348,7 @@ hcache_write(FILE *f, hostcache_t *hc)
  * If `extra' is not HCACHE_NONE, it is appended after the dump of `type'.
  */
 static void
-hcache_store(hcache_type_t type, const gchar *filename, hcache_type_t extra)
+hcache_store(hcache_type_t type, const char *filename, hcache_type_t extra)
 {
 	FILE *f;
 	file_path_t fp;
@@ -1481,7 +1482,7 @@ hcache_store_if_dirty(host_type_t type)
 {
 	gnet_property_t prop;
 	hcache_type_t first, second;
-	const gchar *file;
+	const char *file;
 
 	switch (type) {
     case HOST_ANY:
