@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.h 13263 2007-04-07 00:23:15Z cbiere $
+ * $Id: hcache.h 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -40,7 +40,7 @@
 #include "common.h"
 
 #include "if/core/hcache.h"
-#include "if/core/hosts.h"
+#include "lib/gnet_host.h"
 
 /*
  * Global Functions
@@ -53,20 +53,20 @@ void hcache_retrieve_all(void);
 
 void hcache_timer(time_t now);
 
-const gchar *host_type_to_string(host_type_t type);
-const gchar *hcache_type_to_string(hcache_type_t type);
+const char *host_type_to_string(host_type_t type);
+const char *hcache_type_to_string(hcache_type_t type);
 
 gboolean hcache_add(
     hcache_type_t type, const host_addr_t addr, guint16 port,
-    const gchar *what);
+    const char *what);
 
 gboolean hcache_add_caught(
     host_type_t type, const host_addr_t addr, guint16 port,
-    const gchar *what);
+    const char *what);
 
 gboolean hcache_add_valid(
     host_type_t type, const host_addr_t addr, guint16 port,
-    const gchar *what);
+    const char *what);
 
 gboolean hcache_node_is_bad(const host_addr_t addr);
 
@@ -75,8 +75,8 @@ void hcache_prune(hcache_type_t type);
 guint hcache_size(host_type_t type);
 gboolean hcache_is_low(host_type_t type);
 
-gint hcache_fill_caught_array(
-	host_type_t type, gnet_host_t *hosts, gint hcount);
+int hcache_fill_caught_array(
+	host_type_t type, gnet_host_t *hosts, int hcount);
 
 gboolean hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port);
 gboolean hcache_find_nearby(host_type_t type,
