@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.c 14721 2007-08-31 02:13:17Z cbiere $
+ * $Id: hosts.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hosts.c 14721 2007-08-31 02:13:17Z cbiere $")
+RCSID("$Id: hosts.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
 #include "sockets.h"
 #include "hosts.h"
@@ -54,271 +54,40 @@ RCSID("$Id: hosts.c 14721 2007-08-31 02:13:17Z cbiere $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"	/* Must be the last header included */
 
-gboolean host_low_on_pongs = FALSE;			/**< True when less than 12% full */
+#define HOST_PINGING_PERIOD		30		/**< Try pinging every 30 seconds */
+
+gboolean host_low_on_pongs = FALSE;		/**< True when less than 12% full */
 
 static gboolean in_shutdown = FALSE;
 
-/***
- *** Host hashing.
- ***/
-
-/**
- * Hash function for use in g_hash_table_new.
+/*
+ * Avoid nodes being stuck helplessly due to completely stale caches.
+ * @return TRUE if an UHC may be contact, FALSE if it's not permissable.
  */
-guint
-host_hash(gconstpointer key)
+static gboolean
+host_cache_allow_bypass(void)
 {
-	const gnet_host_t *host = key;
-	host_addr_t addr;
-	guint16 port;
+	static time_t last_try;
 
-	addr = gnet_host_get_addr(host);
-	port = gnet_host_get_port(host);
-	return host_addr_hash(addr) ^ ((port << 16) | port);
-}
+	if (node_count() > 0)
+		return FALSE;
 
-/**
- * Compare function which returns TRUE if the hosts are equal.
- *
- * @note For use in g_hash_table_new.
- */
-gint
-host_eq(gconstpointer v1, gconstpointer v2)
-{
-	const gnet_host_t *h1 = v1, *h2 = v2;
+	/* Wait at least 2 minutes after starting up */
+	if (delta_time(tm_time(), GNET_PROPERTY(start_stamp)) < 2 * 60)
+		return FALSE;
 
-	return gnet_host_get_port(h1) == gnet_host_get_port(h2) &&
-		host_addr_equal(gnet_host_get_addr(h1), gnet_host_get_addr(h2));
-}
+	/* Allow again after 12 hours, useful after unexpected network outtage or
+	 * downtime */
+	if (last_try && delta_time(tm_time(), last_try) < 12 * 3600)
+		return FALSE;
 
-/**
- * Compare function which returns 0 if the hosts are equal, otherwise 1.
- *
- * @note For use in g_list_find_custom.
- */
-gint
-host_cmp(gconstpointer v1, gconstpointer v2)
-{
-	return host_eq(v1, v2) ? 0 : 1;
-}
-
-void
-gnet_host_vec_free(gnet_host_vec_t **vec_ptr)
-{
-	g_assert(vec_ptr != NULL);
-
-	if (*vec_ptr) {
-		gnet_host_vec_t *vec;
-	
-		vec = *vec_ptr;
-		WFREE_NULL(vec->hvec_v4, vec->n_ipv4 * sizeof vec->hvec_v4[0]);
-		WFREE_NULL(vec->hvec_v6, vec->n_ipv6 * sizeof vec->hvec_v6[0]);
-		wfree(vec, sizeof *vec);
-		*vec_ptr = NULL;
-	}
-}
-
-gnet_host_vec_t *
-gnet_host_vec_alloc(void)
-{
-	static const gnet_host_vec_t zero_vec;
-	return wcopy(&zero_vec, sizeof zero_vec);
-}
-
-gnet_host_vec_t *
-gnet_host_vec_copy(const gnet_host_vec_t *vec)
-{
-	gnet_host_vec_t *vec_copy;
-
-	g_return_val_if_fail(vec, NULL);
-	g_return_val_if_fail(vec->n_ipv4 + vec->n_ipv6 > 0, NULL);
-
-	vec_copy = wcopy(vec, sizeof *vec);
-	if (vec->n_ipv4 > 0) {
-		vec_copy->hvec_v4 = wcopy(vec->hvec_v4,
-								vec->n_ipv4 * sizeof *vec->hvec_v4);
-	}
-	if (vec->n_ipv6 > 0) {
-		vec_copy->hvec_v6 = wcopy(vec->hvec_v6,
-								vec->n_ipv6 * sizeof *vec->hvec_v6);
-	}
-	return vec_copy;
-}
-
-void
-gnet_host_vec_add(gnet_host_vec_t *vec, host_addr_t addr, guint16 port)
-{
-	g_return_if_fail(vec);
-
-	switch (host_addr_net(addr)) {
-	case NET_TYPE_IPV4:
-		if (vec->n_ipv4 < 255) {
-			gchar *dest;
-			size_t size, old_size;
-			
-			old_size = vec->n_ipv4 * sizeof *vec->hvec_v4;
-			vec->n_ipv4++;	
-			size = vec->n_ipv4 * sizeof *vec->hvec_v4;
-			vec->hvec_v4 = vec->hvec_v4
-								? wrealloc(vec->hvec_v4, old_size, size)
-								: walloc(size);
-			dest = cast_to_gpointer(&vec->hvec_v4[vec->n_ipv4 - 1]);
-			poke_be32(&dest[0], host_addr_ipv4(addr));
-			poke_le16(&dest[4], port);
-		}
-		break;
-	case NET_TYPE_IPV6:
-		if (vec->n_ipv6 < 255) {
-			gchar *dest;
-			size_t size, old_size;
-			
-			old_size = vec->n_ipv6 * sizeof *vec->hvec_v6;
-			vec->n_ipv6++;	
-			size = vec->n_ipv6 * sizeof *vec->hvec_v6;
-			vec->hvec_v6 = vec->hvec_v6
-								? wrealloc(vec->hvec_v6, old_size, size)
-								: walloc(size);
-			dest = cast_to_gpointer(&vec->hvec_v6[vec->n_ipv6 - 1]);
-			memcpy(dest, host_addr_ipv6(&addr), 16);
-			poke_le16(&dest[16], port);
-		}
-		break;
-	case NET_TYPE_LOCAL:
-	case NET_TYPE_NONE:
-		break;
-	}
-}
-
-gnet_host_vec_t *
-gnet_host_vec_create(gnet_host_t *hvec, gint hcnt)
-{
-	gnet_host_vec_t *vec;
-	guint n_ipv6 = 0, n_ipv4 = 0;
-	gint i;
-
-	g_assert(hcnt >= 0);
-
-	for (i = 0; i < hcnt; i++) {
-		switch (gnet_host_get_net(&hvec[i])) {
-		case NET_TYPE_IPV4: n_ipv4++; break;
-		case NET_TYPE_IPV6: n_ipv6++; break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			break;
-		}
-	}
-
-	vec = gnet_host_vec_alloc();
-	vec->n_ipv4 = MIN(n_ipv4, 255);
-	vec->n_ipv6 = MIN(n_ipv6, 255);
-
-	if (vec->n_ipv4 > 0) {
-		vec->hvec_v4 = walloc(vec->n_ipv4 * sizeof *vec->hvec_v4);
-	}
-	if (vec->n_ipv6 > 0) {
-		vec->hvec_v6 = walloc(vec->n_ipv6 * sizeof *vec->hvec_v6);
-	}
-
-	n_ipv4 = 0;
-	n_ipv6 = 0;
-
-	for (i = 0; i < hcnt; i++) {
-		host_addr_t addr = gnet_host_get_addr(&hvec[i]);
-		guint16 port = gnet_host_get_port(&hvec[i]);
-		
-		switch (gnet_host_get_net(&hvec[i])) {
-		case NET_TYPE_IPV4:
-			if (n_ipv4 < vec->n_ipv4) {
-				gchar *dest = cast_to_gpointer(&vec->hvec_v4[n_ipv4++]);
-				poke_be32(&dest[0], host_addr_ipv4(addr));
-				poke_le16(&dest[4], port);
-			}
-		case NET_TYPE_IPV6:
-			if (n_ipv6 < vec->n_ipv6) {
-				gchar *dest = cast_to_gpointer(&vec->hvec_v6[n_ipv6++]);
-				memcpy(dest, host_addr_ipv6(&addr), 16);
-				poke_le16(&dest[16], port);
-			}
-			break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			break;
-		}
-	}
-	return vec;
-}
-
-gnet_host_vec_t *
-gnet_host_vec_from_list(const GSList *list)
-{
-	const GSList *iter;
-	gnet_host_vec_t *vec;
-	guint n_ipv6 = 0, n_ipv4 = 0, hcnt;
-
-	hcnt = 0;
-	for (iter = list; NULL != iter; iter = g_slist_next(iter)) {
-		const gnet_host_t *host = iter->data;
-
-		switch (gnet_host_get_net(host)) {
-		case NET_TYPE_IPV4:
-			n_ipv4++;
-			hcnt++;
-			break;
-		case NET_TYPE_IPV6:
-			n_ipv6++;
-			hcnt++;
-			break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			break;
-		}
-	}
-	if (0 == hcnt)
-		return NULL;
-
-	vec = gnet_host_vec_alloc();
-	vec->n_ipv4 = MIN(n_ipv4, 255);
-	vec->n_ipv6 = MIN(n_ipv6, 255);
-
-	if (vec->n_ipv4 > 0) {
-		vec->hvec_v4 = walloc(vec->n_ipv4 * sizeof *vec->hvec_v4);
-	}
-	if (vec->n_ipv6 > 0) {
-		vec->hvec_v6 = walloc(vec->n_ipv6 * sizeof *vec->hvec_v6);
-	}
-
-	n_ipv4 = 0;
-	n_ipv6 = 0;
-
-	for (iter = list; NULL != iter; iter = g_slist_next(iter)) {
-		const gnet_host_t *host = iter->data;
-		host_addr_t addr = gnet_host_get_addr(host);
-		guint16 port = gnet_host_get_port(host);
-		
-		switch (gnet_host_get_net(host)) {
-		case NET_TYPE_IPV4:
-			if (n_ipv4 < vec->n_ipv4) {
-				gchar *dest = cast_to_gpointer(&vec->hvec_v4[n_ipv4++]);
-				poke_be32(&dest[0], host_addr_ipv4(addr));
-				poke_le16(&dest[4], port);
-			}
-		case NET_TYPE_IPV6:
-			if (n_ipv6 < vec->n_ipv6) {
-				gchar *dest = cast_to_gpointer(&vec->hvec_v6[n_ipv6++]);
-				memcpy(dest, host_addr_ipv6(&addr), 16);
-				poke_le16(&dest[16], port);
-			}
-			break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			break;
-		}
-	}
-	return vec;
+	last_try = tm_time();
+	return TRUE;
 }
 
 /***
@@ -331,9 +100,8 @@ gnet_host_vec_from_list(const GSList *list)
 void
 host_timer(void)
 {
-	static gint called = 0;
     guint count;
-	gint missing;
+	int missing;
 	host_addr_t addr;
 	guint16 port;
 	host_type_t htype;
@@ -353,11 +121,12 @@ host_timer(void)
 	 * Also, we don't connect each time we are called.
 	 */
 
-	if (!GNET_PROPERTY(is_inet_connected) && missing) {
-		if (0 == (called++ & 0xf))		/* Once every 16 attempts */
-			missing = 1;
-		else
-			missing = 0;			/* Don't connect this run */
+	if (!GNET_PROPERTY(is_inet_connected)) {
+		static time_t last_try;
+
+		if (last_try && delta_time(tm_time(), last_try) < 20)
+			return;
+		last_try = tm_time();
 	}
 
 	/*
@@ -403,12 +172,27 @@ host_timer(void)
 
     if (!GNET_PROPERTY(stop_host_get)) {
         if (missing > 0) {
-            guint fan, max_pool, to_add;
+			static time_t last_try;
+            unsigned fan, max_pool, to_add;
 
             max_pool = MAX(GNET_PROPERTY(quick_connect_pool_size), max_nodes);
             fan = (missing * GNET_PROPERTY(quick_connect_pool_size))/ max_nodes;
             to_add = GNET_PROPERTY(is_inet_connected) ? fan : (guint) missing;
 
+			/*
+			 * Every so many calls, attempt to ping all our neighbours to
+			 * get fresh pongs, in case our host cache is not containing
+			 * sufficiently fresh hosts and we keep getting connection failures.
+			 */
+
+			if (
+				0 == last_try ||
+				delta_time(tm_time(), last_try) >= HOST_PINGING_PERIOD
+			) {
+				ping_all_neighbours();
+				last_try = tm_time();
+			}
+
             /*
              * Make sure that we never use more connections then the
              * quick pool or the maximum number of hosts allow.
@@ -417,8 +201,8 @@ host_timer(void)
                 to_add = max_pool - count;
 
             if (GNET_PROPERTY(dbg) > 10) {
-                g_message("host_timer - connecting - add: %d fan:%d  miss:%d "
-                     "max_hosts:%d   count:%d   extra:%d",
+                g_message("host_timer - connecting - "
+					"add: %d fan:%d miss:%d max_hosts:%d count:%d extra:%d",
 					 to_add, fan, missing, max_nodes, count,
 					 GNET_PROPERTY(quick_connect_pool_size));
             }
@@ -431,7 +215,7 @@ host_timer(void)
 				}
 			}
 
-			if (missing > 0) {
+			if (missing > 0 || host_cache_allow_bypass()) {
 				if (!uhc_is_waiting()) {
 					uhc_get_hosts();	/* Get from UDP pong caches */
 				} else if (GNET_PROPERTY(bootstrap_debug) > 2)
@@ -462,18 +246,18 @@ host_init(void)
 }
 
 /**
- * @return the address:port of a host
+ * Check whether host's address is usable: routable and not bogus.
  */
-const gchar *
-gnet_host_to_string(const struct gnutella_host *h)
+gboolean
+host_address_is_usable(const host_addr_t addr)
 {
-	static gchar buf[HOST_ADDR_PORT_BUFLEN];
-	host_addr_t addr;
-	guint16 port;
+	if (!host_addr_is_routable(addr))
+		return FALSE;
 
-	packed_host_unpack(h->data, &addr, &port);
-	host_addr_port_to_string_buf(addr, port, buf, sizeof buf);
-	return buf;
+	if (bogons_check(addr))
+		return FALSE;
+
+	return TRUE;
 }
 
 /**
@@ -488,13 +272,7 @@ host_is_valid(const host_addr_t addr, guint16 port)
 	if (!port_is_valid(port))
 		return FALSE;
 
-	if (!host_addr_is_routable(addr))
-		return FALSE;
-
-	if (bogons_check(addr))
-		return FALSE;
-
-	return TRUE;
+	return host_address_is_usable(addr);
 }
 
 /**
@@ -572,10 +350,10 @@ free_networks(void)
  * the local_networks array. IP's are in network order.
  */
 void
-parse_netmasks(const gchar *str)
+parse_netmasks(const char *str)
 {
-	gchar **masks = g_strsplit(str, ";", 0);
-	gchar *p;
+	char **masks = g_strsplit(str, ";", 0);
+	char *p;
 	guint32 mask_div;
 	int i;
 
@@ -607,7 +385,7 @@ parse_netmasks(const gchar *str)
 					g_warning("parse_netmasks(): Invalid netmask: \"%s\"", p);
 			}
 			else {
-				gint error;
+				int error;
 
 				mask_div = parse_uint32(p, NULL, 10, &error);
 				mask_div = MIN(32, mask_div);
