@@ -1,5 +1,5 @@
 /*
- * $Id: hsep.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: hsep.c 16289 2009-03-29 08:41:43Z rmanfredi $
  *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
@@ -95,7 +95,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hsep.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: hsep.c 16289 2009-03-29 08:41:43Z rmanfredi $")
 
 #include "gmsg.h"
 #include "routing.h"
@@ -421,7 +421,7 @@ hsep_reset(void)
  */
 
 void
-hsep_connection_init(struct gnutella_node *n)
+hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor)
 {
 	static const hsep_ctx_t zero_hsep;
 	time_t now = tm_time();
@@ -436,6 +436,8 @@ hsep_connection_init(struct gnutella_node *n)
 	n->hsep = walloc(sizeof *n->hsep);
 	*n->hsep = zero_hsep; /* Initializes everything to 0 */
 	n->hsep->last_sent = now;
+	n->hsep->major = major;
+	n->hsep->minor = minor;
 
 	/* this is what we know before receiving the first message */
 
@@ -658,7 +660,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	for (k = 0, i = 1; k < max; k++, i++) {
 
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			gchar buf[G_N_ELEMENTS(messaget[0])][32];
+			char buf[G_N_ELEMENTS(messaget[0])][32];
 
 			for (j = 0; j < G_N_ELEMENTS(buf); j++)
 				uint64_to_string_buf(messaget[k][j], buf[j], sizeof buf[0]);
@@ -810,7 +812,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 
 	for (i = 0; i < opttriples; i++) {
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			gchar buf[G_N_ELEMENTS(hsep_own)][32];
+			char buf[G_N_ELEMENTS(hsep_own)][32];
 
 			for (j = 0; j < G_N_ELEMENTS(buf); j++) {
 				guint64 v;
@@ -937,6 +939,7 @@ hsep_close(void)
 	event_destroy(hsep_global_table_changed_event);
 }
 
+#if 0 /* UNUSED */
 /**
  * Checks whether the global HSEP table has changed since the
  * specified point in time. Returns TRUE if this is the case,
@@ -948,6 +951,7 @@ hsep_has_global_table_changed(time_t since)
 {
 	return delta_time(hsep_last_global_table_change, since) > 0;
 }
+#endif /* UNUSED */
 
 /**
  * Gets a HSEP-compatible triple for all non-HSEP nodes.
@@ -1006,10 +1010,10 @@ hsep_get_non_hsep_triple(hsep_triple *tripledest)
  * @attention
  * NB: The static buffers for each column are disjunct.
  */
-const gchar *
-hsep_get_static_str(gint row, gint column)
+const char *
+hsep_get_static_str(int row, int column)
 {
-	const gchar *ret = NULL;
+	const char *ret = NULL;
 	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
 	hsep_triple other[1];
 	guint64 v;
@@ -1020,7 +1024,7 @@ hsep_get_static_str(gint row, gint column)
     switch (column) {
     case HSEP_IDX_NODES:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_NODES] + other[0][HSEP_IDX_NODES];
 			uint64_to_string_buf(v, buf, sizeof buf);
@@ -1030,7 +1034,7 @@ hsep_get_static_str(gint row, gint column)
 
     case HSEP_IDX_FILES:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_FILES] + other[0][HSEP_IDX_FILES];
 			uint64_to_string_buf(v, buf, sizeof buf);
@@ -1040,7 +1044,7 @@ hsep_get_static_str(gint row, gint column)
 
 	case HSEP_IDX_KIB:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			/* Make a copy because concurrent usage of short_kb_size()
 	 	 	 * could be hard to discover. */
@@ -1060,7 +1064,7 @@ hsep_get_static_str(gint row, gint column)
 /**
  * @returns the size of the global hsep table
  */
-gint
+int
 hsep_get_table_size(void)
 {
 	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
