@@ -1,5 +1,5 @@
 /*
- * $Id: hsep.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: hsep.h 16289 2009-03-29 08:41:43Z rmanfredi $
  *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
@@ -68,7 +68,7 @@ struct gnutella_node;
 void hsep_init(void);
 void hsep_reset(void);
 void hsep_close(void);
-void hsep_connection_init(struct gnutella_node *n);
+void hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor);
 void hsep_connection_close(struct gnutella_node *n);
 void hsep_send_msg(struct gnutella_node *, time_t now);
 void hsep_process_msg(struct gnutella_node *, time_t now);
@@ -78,7 +78,7 @@ unsigned int hsep_get_global_table(hsep_triple *buffer,
 	unsigned int maxtriples);
 unsigned int hsep_get_connection_table(const struct gnutella_node *n,
 	hsep_triple *buffer, unsigned int maxtriples);
-const gchar *hsep_get_static_str(gint row, gint column);
+const char *hsep_get_static_str(int row, int column);
 
 #endif /* _core_hsep_h_ */
 
