@@ -1,5 +1,5 @@
 /*
- * $Id: http.c 14707 2007-08-27 05:53:17Z rmanfredi $
+ * $Id: http.c 16275 2009-03-28 17:40:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: http.c 14707 2007-08-27 05:53:17Z rmanfredi $")
+RCSID("$Id: http.c 16275 2009-03-28 17:40:31Z rmanfredi $")
 
 #include "nodes.h"
 #include "http.h"
@@ -50,6 +50,7 @@ RCSID("$Id: http.c 14707 2007-08-27 05:53:17Z rmanfredi $")
 #include "clock.h"
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/header.h"
@@ -85,25 +86,26 @@ static GSList *sl_outgoing = NULL;	/**< To spot reply timeouts */
  */
 gboolean
 http_send_status(
-	struct gnutella_socket *s, gint code, gboolean keep_alive,
-	http_extra_desc_t *hev, gint hevcnt,
-	const gchar *reason, ...)
+	http_layer_t layer,
+	struct gnutella_socket *s, int code, gboolean keep_alive,
+	http_extra_desc_t *hev, int hevcnt,
+	const char *reason, ...)
 {
-	gchar header[2560];			/* 2.5 K max */
-	gchar status_msg[512];
+	char header[2560];			/* 2.5 K max */
+	char status_msg[512];
 	size_t rw, minimal_rw;
 	size_t header_size = sizeof(header);
 	ssize_t sent;
-	gint i;
+	int i;
 	va_list args;
-	const gchar *conn_close = keep_alive ? "" : "Connection: close\r\n";
-	const gchar *no_content = "Content-Length: 0\r\n";
-	const gchar *version;
-	const gchar *date;
-	const gchar *token;
-	const gchar *body = NULL;
+	const char *conn_close = keep_alive ? "" : "Connection: close\r\n";
+	const char *no_content = "Content-Length: 0\r\n";
+	const char *version;
+	const char *date;
+	const char *token;
+	const char *body = NULL;
 	gboolean saturated = bsched_saturated(BSCHED_BWS_OUT);
-	gint cb_flags = 0;
+	int cb_flags = 0;
 
 	va_start(args, reason);
 	gm_vsnprintf(status_msg, sizeof(status_msg)-1, reason, args);
@@ -169,6 +171,10 @@ http_send_status(
 	if (code < 300 || !keep_alive || body)
 		no_content = "";
 
+	/*
+	 * Construct a minimal header: what we shall always send.
+	 */
+
 	g_assert(header_size <= sizeof header);
 
 	date = timestamp_rfc1123_to_string(clock_loc2gmt(tm_time()));
@@ -235,7 +241,7 @@ http_send_status(
 	if (body) {
 		rw += gm_snprintf(&header[rw], header_size - rw, "%s", body);
 	}
-	if (rw >= header_size && hev) {
+	if (rw >= header_size && (hev || body)) {
 		g_warning("HTTP status %d (%s) too big, ignoring extra information",
 			code, status_msg);
 
@@ -258,10 +264,25 @@ http_send_status(
 			host_addr_to_string(s->addr));
 		return FALSE;
 	} else {
-		if (GNET_PROPERTY(http_debug) > 2)
-			g_message("----Sent HTTP Status to %s (%lu bytes):\n%.*s\n----",
-				host_addr_to_string(s->addr), (gulong) rw,
-				(int) MIN(rw, INT_MAX), header);
+		guint32 trace = 0;
+
+		switch (layer) {
+		case HTTP_PUSH_PROXY:
+			trace = GNET_PROPERTY(push_proxy_trace) & SOCK_TRACE_OUT;
+			break;
+		case HTTP_UPLOAD:
+			trace = GNET_PROPERTY(upload_trace) & SOCK_TRACE_OUT;
+			break;
+		case HTTP_OTHER:
+			trace = GNET_PROPERTY(http_trace) & SOCK_TRACE_OUT;
+			break;
+		}
+
+		if (trace) {
+			g_message("----Sent HTTP status to %s (%lu bytes):",
+				host_addr_to_string(s->addr), (gulong) rw);
+			dump_string(stderr, header, rw, "----");
+		}
 	}
 
 	return TRUE;
@@ -273,7 +294,7 @@ http_send_status(
  * Add an X-Hostname line bearing the fully qualified hostname.
  */
 size_t
-http_hostname_add(gchar *buf, size_t size, gpointer unused_arg, guint32 flags)
+http_hostname_add(char *buf, size_t size, gpointer unused_arg, guint32 flags)
 {
 	size_t len;
 
@@ -296,7 +317,7 @@ http_hostname_add(gchar *buf, size_t size, gpointer unused_arg, guint32 flags)
  * Add a Retry-After header.
  */
 size_t
-http_retry_after_add(gchar *buf, size_t size,
+http_retry_after_add(char *buf, size_t size,
 	gpointer arg, guint32 unused_flags)
 {
 	size_t len;
@@ -319,12 +340,12 @@ http_retry_after_add(gchar *buf, size_t size,
  *
  * @return status code, -1 on error.
  */
-static gint
-code_message_parse(const gchar *line, const gchar **msg)
+static int
+code_message_parse(const char *line, const char **msg)
 {
-	const gchar *endptr;
+	const char *endptr;
 	guint32 v;
-	gint error;
+	int error;
 
 	/*
 	 * We expect exactly 3 status digits.
@@ -370,12 +391,12 @@ code_message_parse(const gchar *line, const gchar **msg)
  * NB: this routine is also used to parse GNUTELLA status codes, since
  * they follow the same pattern as HTTP status codes.
  */
-gint
-http_status_parse(const gchar *line,
-	const gchar *proto, const gchar **msg, guint *major, guint *minor)
+int
+http_status_parse(const char *line,
+	const char *proto, const char **msg, guint *major, guint *minor)
 {
 	guchar c;
-	const gchar *p;
+	const char *p;
 
 	/*
 	 * Skip leading spaces.
@@ -480,9 +501,9 @@ http_status_parse(const gchar *line,
  */
 gboolean
 http_extract_version(
-	const gchar *request, size_t len, guint *major, guint *minor)
+	const char *request, size_t len, guint *major, guint *minor)
 {
-	const gchar *p;
+	const char *p;
 	size_t limit, i;
 
 	/*
@@ -545,7 +566,7 @@ http_extract_version(
  *** HTTP URL parsing.
  ***/
 
-static const gchar * const parse_errstr[] = {
+static const char * const parse_errstr[] = {
 	"OK",								/**< HTTP_URL_OK */
 	"Not an http URI",					/**< HTTP_URL_NOT_HTTP */
 	"More than one <user>:<password>",	/**< HTTP_URL_MULTIPLE_CREDENTIALS */
@@ -560,10 +581,10 @@ static const gchar * const parse_errstr[] = {
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *
+const char *
 http_url_strerror(http_url_error_t errnum)
 {
-	if ((gint) errnum < 0 || errnum >= G_N_ELEMENTS(parse_errstr))
+	if ((int) errnum < 0 || errnum >= G_N_ELEMENTS(parse_errstr))
 		return "Invalid error code";
 
 	return parse_errstr[errnum];
@@ -579,15 +600,15 @@ http_url_strerror(http_url_error_t errnum)
  *
  */
 gboolean
-http_url_parse(const gchar *url, guint16 *port, const gchar **host,
-	const gchar **path)
+http_url_parse(const char *url, guint16 *port, const char **host,
+	const char **path)
 {
-	static gchar hostname[MAX_HOSTLEN + 1];
+	static char hostname[MAX_HOSTLEN + 1];
 	struct {
-		const gchar *host, *path;
+		const char *host, *path;
 		guint16 port;
 	} tmp;
-	const gchar *endptr, *p;
+	const char *endptr, *p;
 	host_addr_t addr;
 
 	g_assert(url);
@@ -638,7 +659,7 @@ http_url_parse(const gchar *url, guint16 *port, const gchar **host,
 	if (':' != *p) {
 		*port = HTTP_PORT;
 	} else {
-		gint error;
+		int error;
 		guint32 u;
 
 		g_assert(':'== *p);
@@ -681,13 +702,13 @@ http_url_parse(const gchar *url, guint16 *port, const gchar **host,
  * and whose `written' bytes have already been sent out.
  */
 http_buffer_t *
-http_buffer_alloc(gchar *buf, size_t len, size_t written)
+http_buffer_alloc(char *buf, size_t len, size_t written)
 {
 	http_buffer_t *b;
 
 	g_assert(buf);
 	g_assert(len > 0 && len <= INT_MAX);
-	g_assert((gint) written >= 0 && written < len);
+	g_assert((int) written >= 0 && written < len);
 
 	b = walloc(sizeof(*b));
 	b->hb_arena = walloc(len);		/* Should be small enough for walloc */
@@ -722,12 +743,12 @@ http_buffer_free(http_buffer_t *b)
  *
  * @return -1 on error, zero on success.
  */
-gint
-http_content_range_parse(const gchar *buf,
+int
+http_content_range_parse(const char *buf,
 		filesize_t *start, filesize_t *end, filesize_t *total)
 {
-	const gchar *s = buf, *endptr;
-	gint error;
+	const char *s = buf, *endptr;
+	int error;
 
 	/*
 	 * HTTP/1.1 -- RFC 2616 -- 3.12 Range Units
@@ -793,7 +814,7 @@ http_content_range_parse(const gchar *buf,
  */
 static GSList *
 http_range_add(GSList *list, filesize_t start, filesize_t end,
-	const gchar *field, const gchar *vendor, gboolean *ignored)
+	const char *field, const char *vendor, gboolean *ignored)
 {
 	GSList *l;
 	GSList *prev;
@@ -824,8 +845,8 @@ http_range_add(GSList *list, filesize_t start, filesize_t end,
 				http_range_t *pr = (http_range_t *) prev->data;
 
 				if (pr->end >= start) {
-					gchar start_buf[UINT64_DEC_BUFLEN];
-					gchar end_buf[UINT64_DEC_BUFLEN];
+					char start_buf[UINT64_DEC_BUFLEN];
+					char end_buf[UINT64_DEC_BUFLEN];
 
 					uint64_to_string_buf(start, start_buf, sizeof start_buf);
 					uint64_to_string_buf(end, end_buf, sizeof end_buf);
@@ -845,8 +866,8 @@ http_range_add(GSList *list, filesize_t start, filesize_t end,
 			if (next != NULL) {
 				http_range_t *nr = (http_range_t *) next->data;
 				if (nr->start <= end) {
-					gchar start_buf[UINT64_DEC_BUFLEN];
-				   	gchar end_buf[UINT64_DEC_BUFLEN];
+					char start_buf[UINT64_DEC_BUFLEN];
+				   	char end_buf[UINT64_DEC_BUFLEN];
 
 					uint64_to_string_buf(start, start_buf, sizeof start_buf);
 					uint64_to_string_buf(end, end_buf, sizeof end_buf);
@@ -866,8 +887,8 @@ http_range_add(GSList *list, filesize_t start, filesize_t end,
 		}
 
 		if (r->end >= start) {
-			gchar start_buf[UINT64_DEC_BUFLEN];
-			gchar end_buf[UINT64_DEC_BUFLEN];
+			char start_buf[UINT64_DEC_BUFLEN];
+			char end_buf[UINT64_DEC_BUFLEN];
 
 			uint64_to_string_buf(start, start_buf, sizeof start_buf);
 			uint64_to_string_buf(end, end_buf, sizeof end_buf);
@@ -914,12 +935,12 @@ ignored:
  */
 GSList *
 http_range_parse(
-	const gchar *field, const gchar *value, filesize_t size,
-	const gchar *vendor)
+	const char *field, const char *value, filesize_t size,
+	const char *vendor)
 {
-	static const gchar unit[] = "bytes";
+	static const char unit[] = "bytes";
 	GSList *ranges = NULL;
-	const gchar *str = value;
+	const char *str = value;
 	guchar c;
 	filesize_t start;
 	filesize_t end;
@@ -929,7 +950,7 @@ http_range_parse(
 	gboolean skipping;
 	gboolean minus_seen;
 	gboolean ignored;
-	gint count = 0;
+	int count = 0;
 
 	g_assert(size > 0);
 	vendor = vendor ? vendor : "unknown";
@@ -989,7 +1010,7 @@ http_range_parse(
 			if (!minus_seen) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d (no range?): "
-					"%s", field, vendor, (gint) (str - value) - 1, value);
+					"%s", field, vendor, (int) (str - value) - 1, value);
 				goto reset;
 			}
 
@@ -997,7 +1018,7 @@ http_range_parse(
 				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(incomplete negative range): %s",
-					field, vendor, (gint) (str - value) - 1, value);
+					field, vendor, (int) (str - value) - 1, value);
 				goto reset;
 			}
 
@@ -1005,7 +1026,7 @@ http_range_parse(
 				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(swapped range?): %s", field, vendor,
-					(gint) (str - value) - 1, value);
+					(int) (str - value) - 1, value);
 				goto reset;
 			}
 
@@ -1017,7 +1038,7 @@ http_range_parse(
 				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(ignored range #%d): %s",
-					field, vendor, (gint) (str - value) - 1, count,
+					field, vendor, (int) (str - value) - 1, count,
 					value);
 			}
 
@@ -1031,7 +1052,7 @@ http_range_parse(
 			if (minus_seen) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d (spurious '-'): %s",
-					field, vendor, (gint) (str - value) - 1, value);
+					field, vendor, (int) (str - value) - 1, value);
 				goto resync;
 			}
 			minus_seen = TRUE;
@@ -1040,7 +1061,7 @@ http_range_parse(
 					if (GNET_PROPERTY(http_debug))
 						g_warning("weird %s header from <%s>, offset %d "
 							"(negative range in reply): %s",
-							field, vendor, (gint) (str - value) - 1, value);
+							field, vendor, (int) (str - value) - 1, value);
 					goto resync;
 				}
 				start = HTTP_OFFSET_MAX;	/* Indicates negative range */
@@ -1050,8 +1071,8 @@ http_range_parse(
 		}
 
 		if (is_ascii_digit(c)) {
-			gint error;
-			const gchar *dend;
+			int error;
+			const char *dend;
 			guint64 val = parse_uint64(str - 1, &dend, 10, &error);
 
 			/* Started with digit! */
@@ -1063,7 +1084,7 @@ http_range_parse(
 				if (GNET_PROPERTY(http_debug))
 					g_warning("weird %s header from <%s>, offset %d "
 						"(spurious boundary %s): %s",
-						field, vendor, (gint) (str - value) - 1,
+						field, vendor, (int) (str - value) - 1,
 						uint64_to_string(val), value);
 				goto resync;
 			}
@@ -1081,7 +1102,7 @@ http_range_parse(
 					if (GNET_PROPERTY(http_debug))
 						g_warning("weird %s header from <%s>, offset %d "
 							"(no '-' before boundary %s): %s",
-							field, vendor, (gint) (str - value) - 1,
+							field, vendor, (int) (str - value) - 1,
 							uint64_to_string(val), value);
 					goto resync;
 				}
@@ -1102,7 +1123,7 @@ http_range_parse(
 		if (GNET_PROPERTY(http_debug))
 			g_warning("weird %s header from <%s>, offset %d "
 			"(unexpected char '%c'): %s",
-			field, vendor, (gint) (str - value) - 1, c, value);
+			field, vendor, (int) (str - value) - 1, c, value);
 
 		/* FALL THROUGH */
 
@@ -1125,7 +1146,7 @@ http_range_parse(
 			if (GNET_PROPERTY(http_debug))
 				g_warning("weird %s header from <%s>, offset %d "
 				"(incomplete trailing negative range): %s",
-				field, vendor, (gint) (str - value) - 1, value);
+				field, vendor, (int) (str - value) - 1, value);
 			goto final;
 		}
 
@@ -1133,7 +1154,7 @@ http_range_parse(
 			if (GNET_PROPERTY(http_debug))
 				g_warning("weird %s header from <%s>, offset %d "
 				"(swapped trailing range?): %s", field, vendor,
-				(gint) (str - value) - 1, value);
+				(int) (str - value) - 1, value);
 			goto final;
 		}
 
@@ -1144,7 +1165,7 @@ http_range_parse(
 			if (GNET_PROPERTY(http_debug))
 				g_warning("weird %s header from <%s>, offset %d "
 				"(ignored final range #%d): %s",
-				field, vendor, (gint) (str - value) - 1, count,
+				field, vendor, (int) (str - value) - 1, count,
 				value);
 	}
 
@@ -1205,16 +1226,16 @@ http_range_size(const GSList *list)
 /**
  * @returns a pointer to static data, containing the available ranges.
  */
-const gchar *
+const char *
 http_range_to_string(const GSList *list)
 {
-	static gchar str[4096];
+	static char str[4096];
 	const GSList *sl = list;
 	size_t rw;
 
 	for (rw = 0; sl && (size_t) rw < sizeof(str); sl = g_slist_next(sl)) {
 		const http_range_t *r = (const http_range_t *) sl->data;
-		gchar start_buf[UINT64_DEC_BUFLEN], end_buf[UINT64_DEC_BUFLEN];
+		char start_buf[UINT64_DEC_BUFLEN], end_buf[UINT64_DEC_BUFLEN];
 
 		uint64_to_string_buf(r->start, start_buf, sizeof start_buf);
 		uint64_to_string_buf(r->end, end_buf, sizeof end_buf);
@@ -1415,7 +1436,7 @@ http_range_merge(GSList *old_list, GSList *new_list)
  *** Asynchronous HTTP error code management.
  ***/
 
-static const gchar * const error_str[] = {
+static const char * const error_str[] = {
 	"OK",									/**< HTTP_ASYNC_OK */
 	"Invalid HTTP URL",						/**< HTTP_ASYNC_BAD_URL */
 	"Connection failed",					/**< HTTP_ASYNC_CONN_FAILED */
@@ -1439,7 +1460,7 @@ guint http_async_errno;		/**< Used to return error codes during setup */
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *
+const char *
 http_async_strerror(guint errnum)
 {
 	if (errnum >= G_N_ELEMENTS(error_str))
@@ -1460,25 +1481,25 @@ enum http_reqtype {
 	NUM_HTTP_REQTYPES
 };
 
-static const gchar * const http_verb[NUM_HTTP_REQTYPES] = {
+static const char * const http_verb[NUM_HTTP_REQTYPES] = {
 	"HEAD",
 	"GET",
 	"POST",
 };
 
-#define HTTP_ASYNC_MAGIC 0xa91cf3eeU
+enum http_async_magic { HTTP_ASYNC_MAGIC = 0xa91cf3eeU };
 
 /**
  * An asynchronous HTTP request.
  */
 struct http_async {
-	guint magic;					/**< Magic number */
+	enum http_async_magic magic;	/**< Magic number */
 	enum http_reqtype type;			/**< Type of request */
 	http_state_t state;				/**< Current request state */
 	guint32 flags;					/**< Operational flags */
-	const gchar *url;				/**< Initial URL request (atom) */
-	const gchar *path;				/**< Path to request (atom) */
-	const gchar *host;				/**< Hostname, if not a numeric IP (atom) */
+	const char *url;				/**< Initial URL request (atom) */
+	const char *path;				/**< Path to request (atom) */
+	const char *host;				/**< Hostname, if not a numeric IP (atom) */
 	struct gnutella_socket *socket;	/**< Attached socket */
 	http_header_cb_t header_ind;	/**< Callback for headers */
 	http_data_cb_t data_ind;		/**< Callback for data */
@@ -1499,6 +1520,8 @@ struct http_async {
 	 */
 
 	http_op_request_t op_request;	/**< Creates HTTP request */
+	http_op_reqsent_t op_reqsent;	/**< Call back when HTTP request sent */
+	http_op_gotreply_t op_gotreply;	/**< Call back when HTTP reply received */
 };
 
 /*
@@ -1519,6 +1542,13 @@ struct http_async {
 
 static GSList *sl_ha_freed = NULL;		/* Pending physical removal */
 
+static inline void
+http_async_check(const struct http_async *ha)
+{
+	g_assert(ha != NULL);
+	g_assert(HTTP_ASYNC_MAGIC == ha->magic);
+}
+
 /**
  * Get URL and request information, given opaque handle.
  * This can be used by client code to log request parameters.
@@ -1527,14 +1557,14 @@ static GSList *sl_ha_freed = NULL;		/* Pending physical removal */
  * if it is a non-NULL pointer, `path' with the request path, `addr' and `port'
  * with the server address/port.
  */
-const gchar *
+const char *
 http_async_info(
-	struct http_async *handle, const gchar **req, const gchar **path,
+	struct http_async *handle, const char **req, const char **path,
 	host_addr_t *addr, guint16 *port)
 {
 	struct http_async *ha = handle;
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 
 	if (req)  *req  = http_verb[ha->type];
 	if (path) *path = ha->path;
@@ -1554,7 +1584,7 @@ http_async_set_opaque(struct http_async *handle, gpointer data,
 {
 	struct http_async *ha = handle;
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(data != NULL);
 
 	ha->user_opaque = data;
@@ -1581,8 +1611,7 @@ http_async_free_recursive(struct http_async *ha)
 {
 	GSList *l;
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-
+	http_async_check(ha);
 	g_assert(sl_outgoing);
 
 	atom_str_free_null(&ha->url);
@@ -1634,6 +1663,7 @@ http_async_free(struct http_async *ha)
 {
 	struct http_async *hax;
 
+	http_async_check(ha);
 	g_assert(sl_outgoing);
 
 	/*
@@ -1676,8 +1706,7 @@ http_async_free_pending(void)
 void
 http_async_close(struct http_async *ha)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-
+	http_async_check(ha);
 	http_async_free(ha);
 }
 
@@ -1687,7 +1716,7 @@ http_async_close(struct http_async *ha)
 static void
 http_async_remove(struct http_async *ha, http_errtype_t type, gpointer code)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 
 	(*ha->error_ind)(ha, type, code);
 	http_async_free(ha);
@@ -1699,6 +1728,7 @@ http_async_remove(struct http_async *ha, http_errtype_t type, gpointer code)
 void
 http_async_cancel(struct http_async *handle)
 {
+	http_async_check(handle);
 	http_async_remove(handle, HTTP_ASYNC_ERROR,
 		GINT_TO_POINTER(HTTP_ASYNC_CANCELLED));
 }
@@ -1707,8 +1737,9 @@ http_async_cancel(struct http_async *handle)
  * Cancel request (internal error).
  */
 void
-http_async_error(struct http_async *handle, gint code)
+http_async_error(struct http_async *handle, int code)
 {
+	http_async_check(handle);
 	http_async_remove(handle, HTTP_ASYNC_ERROR, GINT_TO_POINTER(code));
 }
 
@@ -1716,8 +1747,9 @@ http_async_error(struct http_async *handle, gint code)
  * Cancel request (system call error).
  */
 static void
-http_async_syserr(struct http_async *handle, gint code)
+http_async_syserr(struct http_async *handle, int code)
 {
+	http_async_check(handle);
 	http_async_remove(handle, HTTP_ASYNC_SYSERR, GINT_TO_POINTER(code));
 }
 
@@ -1725,8 +1757,9 @@ http_async_syserr(struct http_async *handle, gint code)
  * Cancel request (header parsing error).
  */
 static void
-http_async_headerr(struct http_async *handle, gint code)
+http_async_headerr(struct http_async *handle, int code)
 {
+	http_async_check(handle);
 	http_async_remove(handle, HTTP_ASYNC_HEADER, GINT_TO_POINTER(code));
 }
 
@@ -1735,10 +1768,12 @@ http_async_headerr(struct http_async *handle, gint code)
  */
 static void
 http_async_http_error(struct http_async *handle, struct header *header,
-	gint code, const gchar *message)
+	int code, const char *message)
 {
 	http_error_t he;
 
+	http_async_check(handle);
+
 	he.header = header;
 	he.code = code;
 	he.message = message;
@@ -1746,6 +1781,38 @@ http_async_http_error(struct http_async *handle, struct header *header,
 	http_async_remove(handle, HTTP_ASYNC_HTTP, &he);
 }
 
+/**
+ * Build HTTP "host:port" string for the remote host, suitable for inclusion
+ * in the Host header of the HTTP request.
+ *
+ * @return pointer to static data
+ */
+const char *
+http_async_remote_host_port(const struct http_async *ha)
+{
+	static char buf[MAX_HOSTLEN + UINT32_DEC_BUFLEN + 1];
+	struct gnutella_socket *s;
+
+	STATIC_ASSERT(HOST_ADDR_BUFLEN <= MAX_HOSTLEN);
+	http_async_check(ha);
+
+	s = ha->socket;
+
+	if (ha->host) {
+		if (s->port != HTTP_PORT)
+			gm_snprintf(buf, sizeof buf, "%s:%u", ha->host, (guint) s->port);
+		else
+			g_strlcpy(buf, ha->host, sizeof buf);
+	} else {
+		if (s->port != HTTP_PORT)
+			host_addr_port_to_string_buf(s->addr, s->port, buf, sizeof buf);
+		else
+			host_addr_to_string_buf(s->addr, buf, sizeof buf);
+	}
+
+	return buf;
+}
+
 /**
  * Default callback invoked to build the HTTP request.
  *
@@ -1759,32 +1826,73 @@ http_async_http_error(struct http_async *handle, struct header *header,
  * header.
  */
 static size_t
-http_async_build_request(struct http_async *unused_handle,
-	gchar *buf, size_t len,
-	const gchar *verb, const gchar *path, const gchar *host, guint16 port)
+http_async_build_request(const struct http_async *ha,
+	char *buf, size_t len, const char *verb, const char *path)
 {
 	size_t rw;
-	gchar port_str[32];
 
-	(void) unused_handle;
+	http_async_check(ha);
 	g_assert(len <= INT_MAX);
 
-	if (port != HTTP_PORT) {
-		gm_snprintf(port_str, sizeof port_str, ":%u", (guint) port);
-	} else {
-		port_str[0] = '\0';
-	}
 	rw = gm_snprintf(buf, len,
 		"%s %s HTTP/1.1\r\n"
-		"Host: %s%s\r\n"
+		"Host: %s\r\n"
 		"User-Agent: %s\r\n"
 		"Connection: close\r\n"
 		"\r\n",
-		verb, path, host, port_str, version_string);
+		verb, path,
+		http_async_remote_host_port(ha),
+		version_string);
 
 	return rw;
 }
 
+/**
+ * Default callback invoked when the HTTP request has been sent.
+ *
+ * @param unused_ha		the (unused) HTTP async request descriptor
+ * @param s				the socket on which we wrote the request
+ * @param req			the actual request string
+ * @param len			the length of the request string
+ * @param deferred		if TRUE, full request sending was deferred earlier
+ */
+static void
+http_async_sent_request(const struct http_async *unused_ha,
+	const struct gnutella_socket *s, const char *req, size_t len,
+	gboolean deferred)
+{
+	(void) unused_ha;
+
+	if (GNET_PROPERTY(http_trace) & SOCK_TRACE_OUT) {
+		g_message("----Sent HTTP request%s to %s (%u bytes):",
+			deferred ? " completely" : "",
+			host_addr_port_to_string(s->addr, s->port), (unsigned) len);
+		dump_string(stderr, req, len, "----");
+	}
+}
+
+/**
+ * Default callback invoked when we got the whole HTTP reply.
+ *
+ * @param unused_ha		the (unused) HTTP async request descriptor
+ * @param s				the socket on which we got the reply
+ * @param status		the first HTTP status line
+ * @param header		the parsed header structure
+ */
+static void
+http_async_got_reply(const struct http_async *unused_ha,
+	const struct gnutella_socket *s, const char *status, const header_t *header)
+{
+	(void) unused_ha;
+
+	if (GNET_PROPERTY(http_trace) & SOCK_TRACE_IN) {
+		g_message("----Got HTTP reply from %s:",
+			host_addr_to_string(s->addr));
+		fprintf(stderr, "%s", status);
+		header_dump(stderr, header, "----");
+	}
+}
+
 /**
  * Internal creation routine for HTTP asynchronous requests.
  *
@@ -1807,7 +1915,7 @@ http_async_build_request(struct http_async *unused_handle,
  */
 static struct http_async *
 http_async_create(
-	const gchar *url,				/* Either full URL or path */
+	const char *url,				/* Either full URL or path */
 	const host_addr_t addr,			/* Optional: 0 means grab from url */
 	guint16 port,					/* Optional, must be given when IP given */
 	enum http_reqtype type,
@@ -1818,7 +1926,7 @@ http_async_create(
 {
 	struct gnutella_socket *s;
 	struct http_async *ha;
-	const gchar *path, *host = NULL;
+	const char *path, *host = NULL;
 
 	g_assert(url);
 	g_assert(error_ind);
@@ -1832,6 +1940,7 @@ http_async_create(
 	 */
 
 	if (!is_host_addr(addr)) {
+		host_addr_t ip;
 		guint16 uport;
 
 		if (!http_url_parse(url, &uport, &host, &path)) {
@@ -1841,11 +1950,21 @@ http_async_create(
 
 		g_assert(host != NULL);
 
-		s = socket_connect_by_name(host, uport, SOCK_TYPE_HTTP, 0);
-	} else {
-		host = host_addr_port_to_string(addr, port);
-		path = url;
+		/*
+		 * Host can be an IP address or a hostname.  If it is a hostname,
+		 * we want to keep it as such in ha->host, otherwise we keep only
+		 * the IP address as part of the socket structure.
+		 */
 
+		if (string_to_host_addr(host, NULL, &ip)) {
+			host = NULL;
+			s = socket_connect(ip, uport, SOCK_TYPE_HTTP, 0);
+		} else {
+			s = socket_connect_by_name(host, uport, SOCK_TYPE_HTTP, 0);
+		}
+	} else {
+		host = NULL;
+		path = url;
 		s = socket_connect(addr, port, SOCK_TYPE_HTTP, 0);
 	}
 
@@ -1868,7 +1987,7 @@ http_async_create(
 	ha->flags = 0;
 	ha->url = atom_str_get(url);
 	ha->path = atom_str_get(path);
-	ha->host = atom_str_get(host);
+	ha->host = host ? atom_str_get(host) : NULL;
 	ha->socket = s;
 	ha->header_ind = header_ind;
 	ha->data_ind = data_ind;
@@ -1884,6 +2003,8 @@ http_async_create(
 	ha->delayed = NULL;
 	ha->allow_redirects = FALSE;
 	ha->op_request = http_async_build_request;
+	ha->op_reqsent = http_async_sent_request;
+	ha->op_gotreply = http_async_got_reply;
 
 	sl_outgoing = g_slist_prepend(sl_outgoing, ha);
 
@@ -1903,6 +2024,8 @@ http_async_create(
 static void
 http_async_newstate(struct http_async *ha, http_state_t state)
 {
+	http_async_check(ha);
+
 	ha->state = state;
 	ha->last_update = tm_time();
 
@@ -1926,7 +2049,7 @@ http_async_newstate(struct http_async *ha, http_state_t state)
  */
 struct http_async *
 http_async_get(
-	const gchar *url,
+	const char *url,
 	http_header_cb_t header_ind,
 	http_data_cb_t data_ind,
 	http_error_cb_t error_ind)
@@ -1941,7 +2064,7 @@ http_async_get(
  */
 struct http_async *
 http_async_get_addr(
-	const gchar *path,
+	const char *path,
 	const host_addr_t addr,
 	guint16 port,
 	http_header_cb_t header_ind,
@@ -1959,19 +2082,41 @@ http_async_get_addr(
 void
 http_async_set_op_request(struct http_async *ha, http_op_request_t op)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(op != NULL);
 
 	ha->op_request = op;
 }
 
+/**
+ * Set callback to invoke when HTTP request is sent.
+ */
+void http_async_set_op_reqsent(struct http_async *ha, http_op_reqsent_t op)
+{
+	http_async_check(ha);
+	g_assert(op != NULL);
+
+	ha->op_reqsent = op;
+}
+
+/**
+ * Set callback to invoke when HTTP reply has been fully received.
+ */
+void http_async_set_op_gotreply(struct http_async *ha, http_op_gotreply_t op)
+{
+	http_async_check(ha);
+	g_assert(op != NULL);
+
+	ha->op_gotreply = op;
+}
+
 /**
  * Defines callback to invoke when the request changes states.
  */
 void
 http_async_on_state_change(struct http_async *ha, http_state_change_t fn)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(fn != NULL);
 
 	ha->state_chg = fn;
@@ -1983,7 +2128,7 @@ http_async_on_state_change(struct http_async *ha, http_state_change_t fn)
 void
 http_async_allow_redirects(struct http_async *ha, gboolean allow)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 
 	ha->allow_redirects = allow;
 }
@@ -1994,9 +2139,9 @@ http_async_allow_redirects(struct http_async *ha, gboolean allow)
  */
 static gboolean
 http_subreq_header_ind(struct http_async *ha, struct header *header,
-	gint code, const gchar *message)
+	int code, const char *message)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->header_ind);
 
@@ -2008,9 +2153,9 @@ http_subreq_header_ind(struct http_async *ha, struct header *header,
  * Reroute to parent request.
  */
 static void
-http_subreq_data_ind(struct http_async *ha, gchar *data, gint len)
+http_subreq_data_ind(struct http_async *ha, char *data, int len)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->data_ind);
 
@@ -2024,7 +2169,7 @@ http_subreq_data_ind(struct http_async *ha, gchar *data, gint len)
 static void
 http_subreq_error_ind(struct http_async *ha, http_errtype_t error, gpointer val)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->error_ind);
 
@@ -2041,10 +2186,12 @@ http_subreq_error_ind(struct http_async *ha, http_errtype_t error, gpointer val)
  */
 static gboolean
 http_async_subrequest(
-	struct http_async *parent, gchar *url, enum http_reqtype type)
+	struct http_async *parent, char *url, enum http_reqtype type)
 {
 	struct http_async *child;
 
+	http_async_check(parent);
+
 	/*
 	 * We're installing our own callbacks to transparently reroute them
 	 * to the user-supplied callbacks for the parent request, hence making
@@ -2062,6 +2209,8 @@ http_async_subrequest(
 	 */
 
 	child->op_request = parent->op_request;
+	child->op_reqsent = parent->op_reqsent;
+	child->op_gotreply = parent->op_gotreply;
 
 	/*
 	 * Indicate that the child request now has control, the parent request
@@ -2079,8 +2228,10 @@ http_async_subrequest(
  * Redirect current HTTP request to some other URL.
  */
 static void
-http_redirect(struct http_async *ha, gchar *url)
+http_redirect(struct http_async *ha, char *url)
 {
+	http_async_check(ha);
+
 	/*
 	 * If this request already has a parent, then we're already
 	 * a redirection.  We're currently not allowing it.  When we do, it
@@ -2136,6 +2287,7 @@ http_got_data(struct http_async *ha, gboolean eof)
 {
 	struct gnutella_socket *s = ha->socket;
 
+	http_async_check(ha);
 	g_assert(s);
 	g_assert(eof || s->pos > 0);		/* If not EOF, there must be data */
 
@@ -2160,14 +2312,14 @@ http_got_data(struct http_async *ha, gboolean eof)
  * Read them and pass them to http_got_data().
  */
 static void
-http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
+http_data_read(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	struct http_async *ha = data;
 	struct gnutella_socket *s = ha->socket;
 	ssize_t r;
 
+	http_async_check(ha);
 	(void) unused_source;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
 		socket_eof(s);
@@ -2175,7 +2327,7 @@ http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	g_assert((gint) s->pos >= 0 && s->pos <= s->buf_size);
+	g_assert((int) s->pos >= 0 && s->pos <= s->buf_size);
 
 	if (s->pos == s->buf_size) {
 		http_async_error(ha, HTTP_ASYNC_IO_ERROR);
@@ -2206,20 +2358,20 @@ http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
 static void
 http_got_header(struct http_async *ha, header_t *header)
 {
-	struct gnutella_socket *s = ha->socket;
-	const gchar *status = getline_str(s->getline);
-	gint ack_code;
-	const gchar *ack_message = "";
-	gchar *buf;
+	struct gnutella_socket *s;
+	const char *status;
+	int ack_code;
+	const char *ack_message = "";
+	char *buf;
 	guint http_major = 0, http_minor = 0;
 
-	if (GNET_PROPERTY(http_debug) > 2) {
-		g_message("----Got HTTP reply from %s:",
-			host_addr_to_string(s->addr));
-		g_message("%s", status);
-		header_dump(header, stderr);
-		g_message("----");
-	}
+	http_async_check(ha);
+
+	s = ha->socket;
+	status = getline_str(s->getline);
+
+	/* Log HTTP headers */
+	(*ha->op_gotreply)(ha, s, status, header);
 
 	/*
 	 * Check status.
@@ -2334,7 +2486,7 @@ http_got_header(struct http_async *ha, header_t *header)
 http_state_t
 http_async_state(struct http_async *ha)
 {
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 
 	/*
 	 * Special-case redirected request: they have at least one child.
@@ -2376,8 +2528,7 @@ call_http_got_header(gpointer obj, header_t *header)
 {
 	struct http_async *ha = obj;
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-
+	http_async_check(ha);
 	http_got_header(ha, header);
 }
 
@@ -2391,8 +2542,7 @@ http_header_start(gpointer obj)
 {
 	struct http_async *ha = obj;
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-
+	http_async_check(ha);
 	http_async_newstate(ha, HTTP_AS_HEADERS);
 }
 
@@ -2402,6 +2552,8 @@ http_header_start(gpointer obj)
 static void
 http_async_request_sent(struct http_async *ha)
 {
+	http_async_check(ha);
+
 	http_async_newstate(ha, HTTP_AS_REQ_SENT);
 
 	/*
@@ -2419,21 +2571,25 @@ http_async_request_sent(struct http_async *ha)
  * sending the HTTP request.
  */
 static void
-http_async_write_request(gpointer data, gint unused_source,
+http_async_write_request(gpointer data, int unused_source,
 	inputevt_cond_t cond)
 {
 	struct http_async *ha = data;
-	struct gnutella_socket *s = ha->socket;
-	http_buffer_t *r = ha->delayed;
+	struct gnutella_socket *s;
+	http_buffer_t *r;
 	ssize_t sent;
 	size_t rw;
-	gchar *base;
+	char *base;
 
 	(void) unused_source;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-	g_assert(r != NULL);
+
+	http_async_check(ha);
+	g_assert(ha->delayed != NULL);
 	g_assert(ha->state == HTTP_AS_REQ_SENDING);
 
+	s = ha->socket;
+	r = ha->delayed;
+
 	if (cond & INPUT_EVENT_EXCEPTION) {
 		socket_eof(s);
 		http_async_error(ha, HTTP_ASYNC_IO_ERROR);
@@ -2452,11 +2608,10 @@ http_async_write_request(gpointer data, gint unused_source,
 	} else if ((size_t) sent < rw) {
 		http_buffer_add_read(r, sent);
 		return;
-	} else if (GNET_PROPERTY(http_debug) > 2) {
-		g_message("----"
-			"Sent HTTP request completely to %s (%d bytes):\n%.*s\n----\n",
-			host_addr_port_to_string(s->addr, s->port), http_buffer_length(r),
-			http_buffer_length(r), http_buffer_base(r));
+	} else {
+		/* Log HTTP request */
+		(*ha->op_reqsent)(ha, s,
+			http_buffer_base(r), http_buffer_length(r), TRUE);
 	}
 
 	/*
@@ -2486,9 +2641,9 @@ http_async_connected(struct http_async *ha)
 	struct gnutella_socket *s;
 	size_t rw;
 	ssize_t sent;
-	gchar req[2048];
+	char req[2048];
 
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 
 	s = ha->socket;
 	socket_check(s);
@@ -2499,8 +2654,7 @@ http_async_connected(struct http_async *ha)
 	 */
 
 	rw = (*ha->op_request)(ha, req, sizeof(req),
-		(gchar *) http_verb[ha->type], ha->path,
-		ha->host ? ha->host : host_addr_to_string(s->addr), s->port);
+		(char *) http_verb[ha->type], ha->path);
 
 	if (rw >= sizeof(req)) {
 		http_async_error(ha, HTTP_ASYNC_REQ2BIG);
@@ -2521,7 +2675,7 @@ http_async_connected(struct http_async *ha)
 		return;
 	} else if ((size_t) sent < rw) {
 		g_warning("partial HTTP request write to %s: only %d of %d bytes sent",
-			host_addr_port_to_string(s->addr, s->port), (gint) sent, (gint) rw);
+			host_addr_port_to_string(s->addr, s->port), (int) sent, (int) rw);
 
 		g_assert(ha->delayed == NULL);
 
@@ -2536,9 +2690,9 @@ http_async_connected(struct http_async *ha)
 		socket_evt_set(s, INPUT_EVENT_WX, http_async_write_request, ha);
 
 		return;
-	} else if (GNET_PROPERTY(http_debug) > 2) {
-		g_message("----Sent HTTP request to %s (%d bytes):\n%.*s\n----",
-			host_addr_port_to_string(s->addr, s->port), (int) rw, (int) rw, req);
+	} else {
+		/* Log HTTP request */
+		(*ha->op_reqsent)(ha, s, req, rw, FALSE);
 	}
 
 	http_async_request_sent(ha);
@@ -2553,13 +2707,15 @@ void
 http_async_log_error_dbg(struct http_async *handle,
 	http_errtype_t type, gpointer v, guint32 dbg_level)
 {
-	const gchar *url;
-	const gchar *req;
-	gint error = GPOINTER_TO_INT(v);
+	const char *url;
+	const char *req;
+	int error = GPOINTER_TO_INT(v);
 	http_error_t *herror = v;
 	host_addr_t addr;
 	guint16 port;
 
+	http_async_check(handle);
+
 	url = http_async_info(handle, &req, NULL, &addr, &port);
 
 	switch (type) {
@@ -2623,26 +2779,28 @@ http_async_log_error(struct http_async *handle, http_errtype_t type, gpointer v)
  ***/
 
 static void
-err_line_too_long(gpointer obj)
+err_line_too_long(gpointer obj, header_t *unused_head)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	(void) unused_head;
+	http_async_check(ha);
 	http_async_error(ha, HTTP_ASYNC_HEAD2BIG);
 }
 
 static void
-err_header_error(gpointer obj, gint error)
+err_header_error(gpointer obj, int error)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	http_async_headerr(ha, error);
 }
 
 static void
-err_input_exception(gpointer obj)
+err_input_exception(gpointer obj, header_t *unused_head)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	(void) unused_head;
+	http_async_check(ha);
 	http_async_error(ha, HTTP_ASYNC_IO_ERROR);
 }
 
@@ -2650,23 +2808,24 @@ static void
 err_input_buffer_full(gpointer obj)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	http_async_error(ha, HTTP_ASYNC_IO_ERROR);
 }
 
 static void
-err_header_read_error(gpointer obj, gint error)
+err_header_read_error(gpointer obj, int error)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_check(ha);
 	http_async_syserr(ha, error);
 }
 
 static void
-err_header_read_eof(gpointer obj)
+err_header_read_eof(gpointer obj, header_t *unused_head)
 {
 	struct http_async *ha = obj;
-	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	(void) unused_head;
+	http_async_check(ha);
 	http_async_error(ha, HTTP_ASYNC_EOF);
 }
 
@@ -2692,8 +2851,8 @@ http_timer(time_t now)
 retry:
 	for (l = sl_outgoing; l; l = l->next) {
 		struct http_async *ha = l->data;
-		gint elapsed = delta_time(now, ha->last_update);
-		gint timeout = ha->bio
+		int elapsed = delta_time(now, ha->last_update);
+		int timeout = ha->bio
 			? GNET_PROPERTY(download_connected_timeout)
 			: GNET_PROPERTY(download_connecting_timeout);
 
