@@ -1,5 +1,5 @@
 /*
- * $Id: huge.c 15050 2007-12-07 18:49:19Z cbiere $
+ * $Id: huge.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: huge.c 15050 2007-12-07 18:49:19Z cbiere $")
+RCSID("$Id: huge.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "huge.h"
 #include "share.h"
@@ -90,7 +90,7 @@ RCSID("$Id: huge.c 15050 2007-12-07 18:49:19Z cbiere $")
  */
 
 struct sha1_cache_entry {
-    const gchar *file_name;		/**< Full path name (atom)          */
+    const char *file_name;		/**< Full path name (atom)          */
 	const struct sha1 *sha1;	/**< SHA-1 (binary; atom)			*/
 	const struct tth *tth;		/**< TTH (binary; atom)				*/
     filesize_t  size;			/**< File size                      */
@@ -172,7 +172,7 @@ cache_entry_print(FILE *f, const char *filename,
 	const struct sha1 *sha1, const struct tth *tth,
 	filesize_t size, time_t mtime)
 {
-	gchar size_buf[UINT64_DEC_BUFLEN], mtime_buf[UINT64_DEC_BUFLEN];
+	char size_buf[UINT64_DEC_BUFLEN], mtime_buf[UINT64_DEC_BUFLEN];
 
 	g_return_if_fail(f);
 	g_return_if_fail(filename);
@@ -292,7 +292,7 @@ parse_and_append_cache_entry(char *line)
 	const char *file_name;
 	char *file_name_end;
 	const char *p, *end; /* pointers to scan the line */
-	gint c, error;
+	int c, error;
 	filesize_t size;
 	time_t mtime;
 	struct sha1 sha1;
@@ -489,6 +489,13 @@ huge_need_sha1(struct shared_file *sf)
 
 	shared_file_check(sf);
 
+	/*
+	 * After a rescan, there might be files in the queue which are
+	 * no longer shared.
+	 */
+	if (!(SHARE_F_INDEXED & shared_file_flags(sf)))
+		return FALSE;
+
 	/*
 	 * XXX HACK ALERT
 	 *
@@ -551,7 +558,7 @@ huge_verify_callback(const struct verify *ctx, enum verify_status status,
 		gnet_prop_set_boolean_val(PROP_SHA1_REBUILDING, TRUE);
 		return huge_need_sha1(sf);
 	case VERIFY_PROGRESS:
-		return TRUE;
+		return 0 != (SHARE_F_INDEXED & shared_file_flags(sf));
 	case VERIFY_DONE:
 		huge_update_hashes(sf, verify_sha1_digest(ctx), NULL);
 		/* FALL THROUGH */
@@ -712,7 +719,7 @@ huge_close(void)
  * improbable hashes.
  */
 gboolean
-huge_improbable_sha1(const gchar *buf, size_t len)
+huge_improbable_sha1(const char *buf, size_t len)
 {
 	size_t ilen = 0;			/* Length of the improbable sequence */
 	size_t i, longest = 0;
@@ -748,7 +755,7 @@ huge_improbable_sha1(const gchar *buf, size_t len)
  * @return TRUE if the SHA1 was valid and properly decoded, FALSE on error.
  */
 gboolean
-huge_sha1_extract32(const gchar *buf, size_t len, struct sha1 *sha1,
+huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	gconstpointer header)
 {
 	if (len != SHA1_BASE32_SIZE || huge_improbable_sha1(buf, len))
@@ -765,7 +772,7 @@ huge_sha1_extract32(const gchar *buf, size_t len, struct sha1 *sha1,
 		if (GNET_PROPERTY(dbg)) {
 			if (is_printable(buf, len)) {
 				g_warning("%s has bad SHA1 (len=%d): %.*s, hex: %s",
-					gmsg_infostr(header), (gint) len, (gint) len, buf,
+					gmsg_infostr(header), (int) len, (int) len, buf,
 					data_hex_str(sha1->data, sizeof sha1->data));
 			} else
 				goto bad;		/* SHA1 should be printable originally */
@@ -795,7 +802,7 @@ bad:
 }
 
 gboolean
-huge_tth_extract32(const gchar *buf, size_t len, struct tth *tth,
+huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
 	gconstpointer header)
 {
 	if (len != TTH_BASE32_SIZE)
@@ -829,9 +836,9 @@ bad:
  * about other sources for this file.
  */
 void
-huge_collect_locations(const struct sha1 *sha1, header_t *header)
+huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 {
-	gchar *alt;
+	char *alt;
    
 
 	g_return_if_fail(sha1);
