@@ -1,5 +1,5 @@
 /*
- * $Id: huge.h 15050 2007-12-07 18:49:19Z cbiere $
+ * $Id: huge.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -62,12 +62,13 @@ gboolean sha1_is_cached(const struct shared_file *sf);
 gboolean huge_update_hashes(struct shared_file *sf,
 	const struct sha1 *sha1, const struct tth *tth);
 
-gboolean huge_improbable_sha1(const gchar *buf, size_t len);
-gboolean huge_sha1_extract32(const gchar *buf, size_t len, struct sha1 *sha1,
+gboolean huge_improbable_sha1(const char *buf, size_t len);
+gboolean huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	gconstpointer header);
-gboolean huge_tth_extract32(const gchar *buf, size_t len, struct tth *tth,
+gboolean huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
 	gconstpointer header);
-void huge_collect_locations(const struct sha1 *sha1, struct header *header);
+void huge_collect_locations(
+	const struct sha1 *sha1, const struct header *header);
 
 #endif	/* _core_huge_h_ */
 
