@@ -1,5 +1,5 @@
 /*
- * $Id: ignore.h 15003 2007-11-05 21:44:15Z rmanfredi $
+ * $Id: ignore.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -55,12 +55,12 @@ struct sha1;
 
 void ignore_timer(time_t now);
 enum ignore_val ignore_is_requested(
-	const gchar *file, filesize_t size, const struct sha1 *sha1);
-const gchar *ignore_reason_to_string(enum ignore_val);
+	const char *file, filesize_t size, const struct sha1 *sha1);
+const char *ignore_reason_to_string(enum ignore_val);
 
-void ignore_add_filesize(const gchar *file, filesize_t size);
-void ignore_add_sha1(const gchar *file, const struct sha1 *sha1);
-const gchar *ignore_sha1_filename(const struct sha1 *sha1);
+void ignore_add_filesize(const char *file, filesize_t size);
+void ignore_add_sha1(const char *file, const struct sha1 *sha1);
+const char *ignore_sha1_filename(const struct sha1 *sha1);
 void ignore_timer(time_t unused_now);
 
 #endif	/* _core_ignore_h_ */
