@@ -1,5 +1,5 @@
 /*
- * $Id: ioheader.c 14793 2007-09-07 18:34:02Z rmanfredi $
+ * $Id: ioheader.c 16275 2009-03-28 17:40:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ioheader.c 14793 2007-09-07 18:34:02Z rmanfredi $")
+RCSID("$Id: ioheader.c 16275 2009-03-28 17:40:31Z rmanfredi $")
 
 #include "ioheader.h"
 #include "sockets.h"
@@ -69,7 +69,7 @@ struct io_header {
 	io_start_cb_t header_read_start;/**< Called when reading first byte */
 	GString *text;					/**< Full header text */
 	guint read_bytes;				/**< Amount of bytes read */
-	gint flags;
+	int flags;
 };
 
 /**
@@ -82,8 +82,8 @@ io_check(const gpointer opaque)
 
 	g_assert(ih);
 	g_assert(ih->io_opaque);
-	g_assert((gchar *) ih->io_opaque > (gchar *) ih->resource);
-	g_assert(((gchar *) ih->io_opaque - (gchar *) ih->resource) < 1024);
+	g_assert((char *) ih->io_opaque > (char *) ih->resource);
+	g_assert(((char *) ih->io_opaque - (char *) ih->resource) < 1024);
 	g_assert(*ih->io_opaque == opaque);
 }
 
@@ -140,7 +140,7 @@ io_getline(const gpointer opaque)
  * It is up to the caller to strdup the data if needed.
  * The returned data will be freed when io_free() is called.
  */
-gchar *
+char *
 io_gettext(const gpointer opaque)
 {
 	struct io_header *ih = opaque;
@@ -175,7 +175,7 @@ io_header_parse(struct io_header *ih)
 	struct gnutella_socket *s = ih->socket;
 	header_t *header = ih->header;
 	size_t parsed;
-	gint error;
+	int error;
 
 	/*
 	 * Read header a line at a time.  We have exacly s->pos chars to handle.
@@ -189,9 +189,8 @@ nextline:
 			host_addr_to_string(s->addr));
 		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
 		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(header, stderr);
-		fprintf(stderr, "------\n");
-		(*ih->error->line_too_long)(ih->resource);
+		header_dump(stderr, header, "-----");
+		(*ih->error->line_too_long)(ih->resource, header);
 		return;
 		/* NOTREACHED */
 	case READ_DONE:
@@ -272,8 +271,7 @@ nextline:
 		g_warning("io_header_parse: %s, disconnecting from %s",
 			header_strerror(error),	host_addr_to_string(s->addr));
 		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(header, stderr);
-		fprintf(stderr, "------\n");
+		header_dump(stderr, header, "-----");
 		dump_hex(stderr, "Header Line", getline_str(ih->getline),
 			MIN(getline_length(ih->getline), 128));
 		(*ih->error->header_error)(ih->resource, error);
@@ -286,8 +284,7 @@ nextline:
 			dump_hex(stderr, "Header Line",
 				getline_str(ih->getline), getline_length(ih->getline));
 			fprintf(stderr, "------ Header Dump (so far):\n");
-			header_dump(header, stderr);
-			fprintf(stderr, "------\n");
+			header_dump(stderr, header, "-----");
 		}
 		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
@@ -303,7 +300,7 @@ nextline:
                 host_addr_to_string(s->addr));
             dump_hex(stderr, "Extra Data", s->buf, MIN(s->pos, 512));
         }
-		(*ih->error->header_extra_data)(ih->resource);
+		(*ih->error->header_extra_data)(ih->resource, ih->header);
 		return;
 	}
 
@@ -349,7 +346,7 @@ nextline:
  * Read data is then handed out to io_header_parse() for analysis.
  */
 static void
-io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
+io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	struct io_header *ih = data;
 	struct gnutella_socket *s = ih->socket;
@@ -362,7 +359,7 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
 		socket_eof(s);
-		(*ih->error->input_exception)(ih->resource);
+		(*ih->error->input_exception)(ih->resource, ih->header);
 		return;
 	}
 
@@ -400,7 +397,7 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	r = bws_read(ih->bws, &s->wio, &s->buf[s->pos], count);
 	if (r == 0) {
 		socket_eof(s);
-		(*ih->error->header_read_eof)(ih->resource);
+		(*ih->error->header_read_eof)(ih->resource, ih->header);
 		return;
 	} else if ((ssize_t) -1 == r) {
 	  	if (!is_temporary_error(errno)) {
@@ -437,7 +434,7 @@ io_get_header(
 	gpointer *io_opaque,		/**< Field address in resource's structure */
 	bsched_bws_t bws,			/**< B/w scheduler from which we read */
 	struct gnutella_socket *s,	/**< Socket from which we're reading */
-	gint flags,					/**< I/O parsing flags */
+	int flags,					/**< I/O parsing flags */
 	io_done_cb_t done,			/**< Mandatory: final callback when all done */
 	io_start_cb_t start,		/**< Optional: called when reading 1st byte */
 	const struct io_error *error) /**< Mandatory: error callbacks for resource */
@@ -446,8 +443,8 @@ io_get_header(
 
 	g_assert(resource);
 	g_assert(io_opaque);
-	g_assert((gchar *) io_opaque > (gchar *) resource);
-	g_assert(((gchar *) io_opaque - (gchar *) resource) < 1024);
+	g_assert((char *) io_opaque > (char *) resource);
+	g_assert(((char *) io_opaque - (char *) resource) < 1024);
 	g_assert(bws < NUM_BSCHED_BWS);
 	g_assert(s);
 	g_assert(done);
@@ -508,7 +505,7 @@ io_get_header(
 void
 io_continue_header(
 	gpointer opaque,			/**< Existing header parsing context */
-	gint flags,					/**< New I/O parsing flags */
+	int flags,					/**< New I/O parsing flags */
 	io_done_cb_t done,			/**< Mandatory: final callback when all done */
 	io_start_cb_t start)		/**< Optional: called when reading 1st byte */
 {
