@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c 13789 2007-05-29 22:21:28Z cbiere $
+ * $Id: mq.c 16264 2009-03-27 21:24:11Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,94 +35,114 @@
 
 #include "common.h"
 
-RCSID("$Id: mq.c 13789 2007-05-29 22:21:28Z cbiere $")
+RCSID("$Id: mq.c 16264 2009-03-27 21:24:11Z rmanfredi $")
+
+#define MQ_INTERNAL
+#include "mq.h"
 
 #include "nodes.h"
-#include "mq.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "gnet_stats.h"
 
 #include "lib/cq.h"
 #include "lib/glib-missing.h"	/* For gm_snprintf() */
+#include "lib/misc.h"			/* For size_saturate_add() */
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
+#define MQ_DEBUG_LVL(q)	(*q->debug)
+
 static void qlink_free(mqueue_t *q);
 static void mq_update_flowc(mqueue_t *q);
 static gboolean make_room_header(
-	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset);
+	mqueue_t *q, char *header, guint prio, int needed, int *offset);
 static void mq_swift_timer(cqueue_t *cq, gpointer obj);
 
+guint32 mq_debug(const mqueue_t *q)
+{
+	return *q->debug;
+}
+
 gboolean
-mq_is_flow_controlled(const struct mqueue *q)
+mq_is_flow_controlled(const mqueue_t *q)
 {
 	return 0 != (q->flags & MQ_FLOWC);
 }
 
 gboolean
-mq_is_swift_controlled(const struct mqueue *q)
+mq_is_swift_controlled(const mqueue_t *q)
 {
 	return 0 != (q->flags & MQ_SWIFT);
 }
 
-gint
-mq_maxsize(const struct mqueue *q)
+int
+mq_maxsize(const mqueue_t *q)
 {
 	return q->maxsize;
 }
 
-gint mq_size(const struct mqueue *q)
+int
+mq_size(const mqueue_t *q)
 {
 	return q->size;
 }
 
-gint
-mq_lowat(const struct mqueue *q)
+int
+mq_lowat(const mqueue_t *q)
 {
 	return q->lowat;
 }
 
-gint
-mq_hiwat(const struct mqueue *q)
+int
+mq_hiwat(const mqueue_t *q)
 {
 	return q->hiwat;
 }
 
-gint
-mq_count(const struct mqueue *q)
+int
+mq_count(const mqueue_t *q)
 {
 	return q->count;
 }
 
-gint
-mq_pending(const struct mqueue *q)
+int
+mq_pending(const mqueue_t *q)
 {
 	return q->size + tx_pending(q->tx_drv);
 }
 
 struct bio_source *
-mq_bio(const struct mqueue *q)
+mq_bio(const mqueue_t *q)
 {
 	return tx_bio_source(q->tx_drv);
 }
 
 struct gnutella_node *
-mq_node(const struct mqueue *q)
+mq_node(const mqueue_t *q)
 {
 	return q->node;
 }
 
+/**
+ * Would `additional' bytes of traffic cause the queue to enter flow-control?
+ */
+gboolean
+mq_would_flow_control(const mqueue_t *q, size_t additional)
+{
+	return size_saturate_add(q->size, additional) >= UNSIGNED(q->hiwat);
+}
+
 /**
  * Compute readable queue information.
  */
-const gchar *
+const char *
 mq_info(const mqueue_t *q)
 {
-	static gchar buf[160];
+	static char buf[160];
 
 	if (q->magic != MQ_MAGIC) {
 		gm_snprintf(buf, sizeof(buf),
@@ -209,11 +229,11 @@ mq_remove_linkable(mqueue_t *q, GList *l)
  * Check queue's sanity.
  */
 void
-mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line)
+mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 {
-	gint qcount;
-	gint qlink_alive = 0;
-	gint n;
+	int qcount;
+	int qlink_alive = 0;
+	int n;
 
 	g_assert(q);
 
@@ -285,8 +305,7 @@ void
 mq_free(mqueue_t *q)
 {
 	GList *l;
-	gint n;
-	pmsg_t *mb;
+	int n;
 
 	tx_free(q->tx_drv);		/* Get rid of lower layers */
 
@@ -304,10 +323,7 @@ mq_free(mqueue_t *q)
 
 	cq_cancel(callout_queue, &q->swift_ev);
 	g_list_free(q->qhead);
-
-	while ((mb = slist_shift(q->qwait)))
-		pmsg_free(mb);
-	slist_free(&q->qwait);
+	pmsg_slist_free(&q->qwait);
 
 	q->qhead = NULL;
 	q->magic = 0;
@@ -322,7 +338,7 @@ mq_free(mqueue_t *q)
  * queue is updated, but not the flow-control information.
  */
 static GList *
-mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
+mq_rmlink_prev(mqueue_t *q, GList *l, int size)
 {
 	GList *prev = g_list_previous(l);
 
@@ -349,14 +365,14 @@ mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
 static void
 mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 {
-	gint elapsed = q->swift_elapsed;	/* Elapsed since we were scheduled */
-	gint target_to_lowmark;
-	gint flushed_till_next_timer;
-	gint added_till_next_timer;
-	gfloat period_ratio;
-	gint added;
-	gint needed;
-	gint extra;
+	int elapsed = q->swift_elapsed;	/* Elapsed since we were scheduled */
+	int target_to_lowmark;
+	int flushed_till_next_timer;
+	int added_till_next_timer;
+	float period_ratio;
+	int added;
+	int needed;
+	int extra;
 
 	g_assert(q->flags & MQ_FLOWC);
 	g_assert(q->size > q->lowat);	/* Or we would have left FC */
@@ -377,12 +393,12 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	 * the queue.
 	 */
 
-	period_ratio = (gfloat) q->swift_elapsed / (gfloat) elapsed;
+	period_ratio = (float) q->swift_elapsed / (float) elapsed;
 	target_to_lowmark = q->size - q->lowat;
 	added = q->size - q->last_size + q->flowc_written;
 
-	flushed_till_next_timer = (gint) (q->flowc_written * period_ratio);
-	added_till_next_timer = added <= 0 ? 0 : (gint) (added * period_ratio);
+	flushed_till_next_timer = (int) (q->flowc_written * period_ratio);
+	added_till_next_timer = added <= 0 ? 0 : (int) (added * period_ratio);
 
 	/*
 	 * Now compute the amount of bytes we need to forcefully drop to be
@@ -425,7 +441,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		gnutella_header_set_ttl(&q->header, GNET_PROPERTY(max_ttl));
 
 		if (needed > 0)
-			make_room_header(q, (gchar*) &q->header, PMSG_P_DATA, needed, NULL);
+			make_room_header(q, (char*) &q->header, PMSG_P_DATA, needed, NULL);
 
 		/*
 		 * Whether or not we were able to make enough room at this point
@@ -436,7 +452,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 */
 
 	} else {
-		gint ttl;
+		int ttl;
 
 		/*
 		 * We're going to drop query hits...
@@ -458,7 +474,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 */
 
 		for (ttl = GNET_PROPERTY(hard_ttl_limit); ttl >= 0; ttl--) {
-			gint old_size;
+			int old_size;
 
 			if (needed <= 0)
 				break;
@@ -469,7 +485,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 			gnutella_header_set_ttl(&q->header, ttl);
 
 			if (
-				make_room_header(q, (gchar*) &q->header, PMSG_P_DATA,
+				make_room_header(q, (char*) &q->header, PMSG_P_DATA,
 					needed, NULL)
 			)
 				break;
@@ -554,7 +570,7 @@ mq_enter_flowc(mqueue_t *q)
 
 	node_tx_enter_flowc(q->node);	/* Signal flow control */
 
-	if (GNET_PROPERTY(dbg) > 4)
+	if (MQ_DEBUG_LVL(q) > 4)
 		printf("entering FLOWC for node %s (%d bytes queued)\n",
 			node_addr(q->node), q->size);
 }
@@ -567,7 +583,7 @@ mq_leave_flowc(mqueue_t *q)
 {
 	g_assert(q->flags & MQ_FLOWC);
 
-	if (GNET_PROPERTY(dbg) > 4)
+	if (MQ_DEBUG_LVL(q) > 4)
 		printf("leaving %s for node %s (%d bytes queued)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			node_addr(q->node), q->size);
@@ -619,6 +635,15 @@ mq_clear(mqueue_t *q)
 	if (q->count == 0)
 		return;					/* Queue is empty */
 
+	/*
+	 * If there are extended message blocks in the queue, freeing them
+	 * could cause the callback to attempt to queue something again.  Hence
+	 * we must mark we're clearing the queue to avoid deadly recursions that
+	 * would corrupt the qlink array.
+	 */
+
+	q->flags |= MQ_CLEAR;
+
 	while (q->qhead) {
 		GList *l = q->qhead;
 		pmsg_t *mb = l->data;
@@ -639,6 +664,8 @@ mq_clear(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
+	q->flags &= ~MQ_CLEAR;
+
 	mq_update_flowc(q);
 
 	/*
@@ -674,12 +701,23 @@ mq_shutdown(mqueue_t *q)
 	tx_shutdown(q->tx_drv);		/* No further output will be made */
 }
 
+/**
+ * Request explicit flush of all buffered data in the TX stack.
+ */
+void
+mq_flush(mqueue_t *q)
+{
+	g_assert(q);
+
+	tx_flush(q->tx_drv);
+}
+
 /**
  * Compare two pointers to links based on their relative priorities, then
  * based on their held Gnutella messages.
  * -- qsort() callback
  */
-static gint
+static int
 qlink_cmp(const void *a, const void *b)
 {
 	const GList * const *l1 = a, * const *l2 = b;
@@ -698,7 +736,7 @@ static void
 qlink_create(mqueue_t *q)
 {
 	GList *l;
-	gint n;
+	int n;
 
 	g_assert(q->qlink == NULL);
 
@@ -748,7 +786,7 @@ qlink_free(mqueue_t *q)
  * before the position indicated by `hint'.
  */
 static void
-qlink_insert_before(mqueue_t *q, gint hint, GList *l)
+qlink_insert_before(mqueue_t *q, int hint, GList *l)
 {
 	g_assert(hint >= 0 && hint < q->qlink_count);
 	g_assert(qlink_cmp(&q->qlink[hint], &l) >= 0);	/* `hint' >= `l' */
@@ -773,7 +811,7 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 	q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 
 	{
-		gint i;
+		int i;
 
 		/* Shift right */
 		for (i = q->qlink_count - 1; i > hint; i--) {
@@ -789,8 +827,8 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 static void
 qlink_insert(mqueue_t *q, GList *l)
 {
-	gint low = 0;
-	gint high = q->qlink_count - 1;
+	int low = 0;
+	int high = q->qlink_count - 1;
 	GList **qlink = q->qlink;
 
 	g_assert(l->data != NULL);
@@ -836,17 +874,17 @@ qlink_insert(mqueue_t *q, GList *l)
 	 */
 
 	while (low <= high) {
-		gint mid = low + (high - low) / 2;
-		gint c;
+		int mid = low + (high - low) / 2;
+		int c;
 
 		/*
 		 * If we end up in a NULL spot, inspect the [low, high] range.
 		 */
 
 		if (qlink[mid] == NULL) {
-			gint n;
-			gint lowest_non_null = -1;
-			gint highest_non_null = -1;
+			int n;
+			int lowest_non_null = -1;
+			int highest_non_null = -1;
 
 			/*
 			 * Go back towards `low' to find a non-NULL entry.
@@ -952,7 +990,7 @@ static void
 qlink_remove(mqueue_t *q, GList *l)
 {
 	GList **qlink = q->qlink;
-	gint n = q->qlink_count;
+	int n = q->qlink_count;
 
 	g_assert(qlink);
 	g_assert(n > 0);
@@ -967,7 +1005,7 @@ qlink_remove(mqueue_t *q, GList *l)
 
 	if (n > q->count * 3) {
 		GList **dest = qlink;
-		gint copied = 0;
+		int copied = 0;
 		gboolean found = FALSE;
 
 		while (n-- > 0) {
@@ -1021,9 +1059,9 @@ qlink_remove(mqueue_t *q, GList *l)
  * @returns TRUE if we were able to make enough room.
  */
 static gboolean
-make_room(mqueue_t *q, pmsg_t *mb, gint needed, gint *offset)
+make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
 {
-	gchar *header = pmsg_start(mb);
+	char *header = pmsg_start(mb);
 	guint prio = pmsg_prio(mb);
 
 	return make_room_header(q, header, prio, needed, offset);
@@ -1035,16 +1073,16 @@ make_room(mqueue_t *q, pmsg_t *mb, gint needed, gint *offset)
  */
 static gboolean
 make_room_header(
-	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset)
+	mqueue_t *q, char *header, guint prio, int needed, int *offset)
 {
-	gint n;
-	gint dropped = 0;				/* Amount of messages dropped */
+	int n;
+	int dropped = 0;				/* Amount of messages dropped */
 	gboolean qlink_corrupted = FALSE;	/* BUG catcher */
 
 	g_assert(needed > 0);
 	mq_check(q, 0);
 
-	if (GNET_PROPERTY(dbg) > 5)
+	if (MQ_DEBUG_LVL(q) > 5)
 		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			needed, (gulong) q, node_addr(q->node));
@@ -1081,8 +1119,8 @@ restart:
 	for (n = 0; needed >= 0 && n < q->qlink_count; n++) {
 		GList *item = q->qlink[n];
 		pmsg_t *cmb;
-		gchar *cmb_start;
-		gint cmb_size;
+		char *cmb_start;
+		int cmb_size;
 
 		/*
 		 * If slot was NULLified, skip it.
@@ -1159,7 +1197,7 @@ restart:
 		 * Drop message.
 		 */
 
-		if (GNET_PROPERTY(dbg) > 4)
+		if (MQ_DEBUG_LVL(q) > 4)
 			gmsg_log_dropped(pmsg_start(cmb),
 				"to %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
@@ -1182,7 +1220,7 @@ restart:
 	if (dropped)
 		node_add_txdrop(q->node, dropped);	/* Dropped during TX */
 
-	if (GNET_PROPERTY(dbg) > 5)
+	if (MQ_DEBUG_LVL(q) > 5)
 		printf("%s end purge: %d bytes (count=%d) for node %s, need=%d\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			q->size, q->count, node_addr(q->node), needed);
@@ -1218,10 +1256,10 @@ restart:
  * Put message in this queue.
  */
 static void
-mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
+mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 {
-	gint needed;
-	gint qlink_offset = -1;
+	int needed;
+	int qlink_offset = -1;
 	GList *new = NULL;
 	gboolean make_room_called = FALSE;
 	gboolean has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
@@ -1242,7 +1280,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		!make_room(q, mb, msize, &qlink_offset)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
-		if (GNET_PROPERTY(dbg) > 4)
+		if (MQ_DEBUG_LVL(q) > 4)
 			gmsg_log_dropped(pmsg_start(mb),
 				"to FLOWC node %s, %d bytes queued",
 				node_addr(q->node), q->size);
@@ -1280,14 +1318,14 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		gnet_stats_count_flowc(pmsg_start(mb));
 
 		if (has_normal_prio) {
-			if (GNET_PROPERTY(dbg) > 4)
+			if (MQ_DEBUG_LVL(q) > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [FULL]",
 					node_addr(q->node), q->size);
 
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
-			if (GNET_PROPERTY(dbg) > 4)
+			if (MQ_DEBUG_LVL(q) > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [KILLING]",
 					node_addr(q->node), q->size);
