@@ -1,5 +1,5 @@
 /*
- * $Id: mq.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: mq.h 16264 2009-03-27 21:24:11Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,12 +36,15 @@
 #ifndef _core_mq_h_
 #define _core_mq_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "gnutella.h"
-#include "pmsg.h"
 #include "tx.h"
 
+#include "lib/cq.h"
+#include "lib/pmsg.h"
+#include "lib/slist.h"
+
 typedef struct mqueue mqueue_t;
 
 /**
@@ -53,12 +56,14 @@ struct mq_ops {
 };
 
 struct mq_cops {
-	void (*puthere)(mqueue_t *q, pmsg_t *mb, gint msize);
+	void (*puthere)(mqueue_t *q, pmsg_t *mb, int msize);
 	void (*qlink_remove)(mqueue_t *q, GList *l);
-	GList *(*rmlink_prev)(mqueue_t *q, GList *l, gint size);
+	GList *(*rmlink_prev)(mqueue_t *q, GList *l, int size);
 	void (*update_flowc)(mqueue_t *q);
 };
 
+#ifdef MQ_INTERNAL
+
 enum mq_magic {
 	MQ_MAGIC = 0x33990ee
 };
@@ -88,42 +93,50 @@ struct mqueue {
 	const struct mq_ops *ops;		/**< Polymorphic operations */
 	const struct mq_cops *cops;		/**< Common operations */
 	txdrv_t *tx_drv;				/**< Network TX stack driver */
-	GList *qhead;			/**< The queue head, new messages are prepended */
-	GList *qtail;			/**< The queue tail, oldest message to send first */
-	GList **qlink;			/**< Sorted array of (GList *) entries, or NULL */
-	gpointer swift_ev;		/**< Callout queue event in "swift" mode */
-	gint swift_elapsed;		/**< Scheduled elapsed time, in ms */
-	gint qlink_count;		/**< Amount of entries in `qlink' */
-	gint maxsize;			/**< Maximum size of this queue (total queued) */
-	gint count;				/**< Amount of messages queued */
-	gint hiwat;				/**< High watermark */
-	gint lowat;				/**< Low watermark */
-	gint size;				/**< Current amount of bytes queued */
-	gint flags;				/**< Status flags */
-	gint last_written;		/**< Amount last written by service routine */
-	gint flowc_written;		/**< Amount written during flow control */
-	gint last_size;			/**< Queue size at last "swift" event callback */
+	GList *qhead, *qtail, **qlink;
+	slist_t *qwait;			/**< Waiting queue during putq recursions */
+	cevent_t *swift_ev;		/**< Callout queue event in "swift" mode */
+	const guint32 *debug;	/**< Debug config variable for this queue */
+	int swift_elapsed;		/**< Scheduled elapsed time, in ms */
+	int qlink_count;		/**< Amount of entries in `qlink' */
+	int maxsize;			/**< Maximum size of this queue (total queued) */
+	int count;				/**< Amount of messages queued */
+	int hiwat;				/**< High watermark */
+	int lowat;				/**< Low watermark */
+	int size;				/**< Current amount of bytes queued */
+	int flags;				/**< Status flags */
+	int last_written;		/**< Amount last written by service routine */
+	int flowc_written;		/**< Amount written during flow control */
+	int last_size;			/**< Queue size at last "swift" event callback */
+    int putq_entered;		/**< For recursion checks in mq_putq() */
 };
 
 /*
  * Queue flags.
  */
 
-#define MQ_FLOWC		0x00000001	/**< In flow control */
-#define MQ_DISCARD		0x00000002	/**< No writing, discard message */
-#define MQ_SWIFT		0x00000004	/**< Swift mode, dropping more traffic */
-#define MQ_WARNZONE		0x00000008	/**< Between hiwat and lowat */
+enum {
+	MQ_CLEAR	= (1 << 4),	/**< Running mq_clear() */
+	MQ_WARNZONE	= (1 << 3),	/**< Between hiwat and lowat */
+	MQ_SWIFT	= (1 << 2),	/**< Swift mode, dropping more traffic */
+	MQ_DISCARD	= (1 << 1),	/**< No writing, discard message */
+	MQ_FLOWC	= (1 << 0)	/**< In flow control */
+};
 
-#define mq_is_flow_controlled(q)	((q)->flags & MQ_FLOWC)
-#define mq_is_swift_controlled(q)	((q)->flags & MQ_SWIFT)
-#define mq_maxsize(q)				((q)->maxsize)
-#define mq_size(q)					((q)->size)
-#define mq_lowat(q)					((q)->lowat)
-#define mq_hiwat(q)					((q)->hiwat)
-#define mq_count(q)					((q)->count)
-#define mq_pending(q)				((q)->size + tx_pending((q)->tx_drv))
-#define mq_bio(q)					(tx_bio_source((q)->tx_drv))
-#define mq_node(q)					((q)->node)
+#endif /* MQ_INTERNAL */
+
+gboolean mq_is_flow_controlled(const mqueue_t *q);
+gboolean mq_is_swift_controlled(const mqueue_t *q);
+gboolean mq_would_flow_control(const mqueue_t *q, size_t additional);
+guint32 mq_debug(const mqueue_t *q);
+int mq_maxsize(const mqueue_t *q);
+int mq_size(const mqueue_t *q);
+int mq_lowat(const mqueue_t *q);
+int mq_hiwat(const mqueue_t *q);
+int mq_count(const mqueue_t *q);
+int mq_pending(const mqueue_t *q);
+struct bio_source *mq_bio(const mqueue_t *q);
+struct gnutella_node *mq_node(const mqueue_t *q);
 
 /*
  * Public interface
@@ -132,11 +145,13 @@ struct mqueue {
 void mq_putq(mqueue_t *q, pmsg_t *mb);
 void mq_free(mqueue_t *q);
 void mq_clear(mqueue_t *q);
+void mq_flush(mqueue_t *q);
 void mq_discard(mqueue_t *q);
 void mq_shutdown(mqueue_t *q);
 void mq_fill_ops(struct mq_ops *ops);
 
 const struct mq_cops *mq_get_cops(void);
+const char *mq_info(const mqueue_t *q);
 
 /*
  * Message queue assertions.
@@ -144,7 +159,7 @@ const struct mq_cops *mq_get_cops(void);
 
 #define MQ_DEBUG
 #ifdef MQ_DEBUG
-void mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line);
+void mq_check_track(mqueue_t *q, int offset, const char *where, int line);
 
 #define mq_check(x,y)	mq_check_track((x), (y), _WHERE_, __LINE__)
 #else
