@@ -1,5 +1,5 @@
 /*
- * $Id: mq_tcp.c 13129 2007-03-17 19:12:43Z cbiere $
+ * $Id: mq_tcp.c 16204 2009-03-18 20:03:57Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,16 +35,19 @@
 
 #include "common.h"
 
-RCSID("$Id: mq_tcp.c 13129 2007-03-17 19:12:43Z cbiere $")
+RCSID("$Id: mq_tcp.c 16204 2009-03-18 20:03:57Z rmanfredi $")
+
+#define MQ_INTERNAL
+#include "mq.h"
 
 #include "nodes.h"
-#include "mq.h"
 #include "mq_tcp.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "tx.h"
 #include "gnet_stats.h"
+#include "dump.h"
 
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -57,18 +60,14 @@ RCSID("$Id: mq_tcp.c 13129 2007-03-17 19:12:43Z cbiere $")
 #define MQ_MINSEND		256		/**< Minimum size we try to send */
 
 static void mq_tcp_service(gpointer data);
-static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb);
-static const struct mq_ops mq_tcp_ops = {
-	mq_tcp_putq,			/* putq */
-};
-
+static const struct mq_ops mq_tcp_ops;
 
 /**
  * Create new message queue capable of holding `maxsize' bytes, and
  * owned by the supplied node.
  */
 mqueue_t *
-mq_tcp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
+mq_tcp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
 {
 	mqueue_t *q;
 
@@ -83,6 +82,7 @@ mq_tcp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 	q->qwait = slist_new();
 	q->ops = &mq_tcp_ops;
 	q->cops = mq_get_cops();
+	q->debug = GNET_PROPERTY_PTR(mq_tcp_debug);
 
 	tx_srv_register(nd, mq_tcp_service, q);
 
@@ -97,13 +97,13 @@ mq_tcp_service(gpointer data)
 {
 	mqueue_t *q = (mqueue_t *) data;
 	static struct iovec iov[MQ_MAXIOV];
-	gint iovsize;
-	gint iovcnt;
-	gint sent;
+	int iovsize;
+	int iovcnt;
+	int sent;
 	ssize_t r;
 	GList *l;
-	gint dropped;
-	gint maxsize;
+	int dropped;
+	int maxsize;
 	gboolean saturated;
 	gboolean has_prioritary = FALSE;
 
@@ -130,7 +130,7 @@ again:
 	for (l = q->qtail; l && iovsize > 0; /* empty */) {
 		struct iovec *ie;
 		pmsg_t *mb = (pmsg_t *) l->data;
-		gchar *mbs = pmsg_start(mb);
+		char *mbs = pmsg_start(mb);
 
 		/*
 		 * Don't build too much.
@@ -218,7 +218,7 @@ again:
 		pmsg_t *mb = (pmsg_t *) l->data;
 
 		if ((guint) r >= ie->iov_len) {			/* Completely written */
-			gchar *mb_start = pmsg_start(mb);
+			char *mb_start = pmsg_start(mb);
 			guint8 function = gmsg_function(mb_start);
 			sent++;
 			pmsg_mark_sent(mb);
@@ -296,32 +296,45 @@ update_servicing:
 
 /**
  * Enqueue message, which becomes owned by the queue.
+ *
+ * @param q			the queue to which we enqueue message
+ * @param mb		the message block to enqueue
+ * @param from		for TX traffic dump: the origin of message, NULL if local
  */
-static void
-mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
+void
+mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from)
 {
-	gint size;				/* Message size */
-	gchar *mbs;				/* Start of message */
+	int size;				/* Message size */
+	char *mbs;				/* Start of message */
 	guint8 function;		/* Gnutella message function */
 	guint8 hops;			/* Gnutella message hop count */
 	gboolean prioritary;	/* Is message prioritary? */
 	gboolean error = FALSE;
 
+	dump_tx_tcp_packet(from, q->node, mb);
+
 again:
 	g_assert(q);
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
+	g_assert(q->ops == &mq_tcp_ops);	/* Is a TCP queue */
+
+	/*
+	 * Trap messages enqueued whilst in the middle of an mq_clear() operation
+	 * by marking them as sent and dropping them.  Idem if queue was
+	 * put in "discard" mode.
+	 */
+
+	if (q->flags & (MQ_CLEAR | MQ_DISCARD)) {
+		pmsg_mark_sent(mb);	/* Let them think it was sent */
+		goto cleanup;		/* Drop message */
+	}
+
 	mq_check(q, 0);
 
 	size = pmsg_size(mb);
-
 	if (size == 0) {
-		g_warning("mq_putq: called with empty message");
-		goto cleanup;
-	}
-
-	if (q->flags & MQ_DISCARD) {
-		g_warning("mq_putq: called whilst queue shutdown");
+		g_warning("mq_tcp_putq: called with empty message");
 		goto cleanup;
 	}
 
@@ -464,4 +477,19 @@ cleanup:
 	return;
 }
 
+/**
+ * Disable plain mq_putq() operation on a TCP queue.
+ */
+static void
+mq_no_putq(mqueue_t *unused_q, pmsg_t *unused_mb)
+{
+	(void) unused_q;
+	(void) unused_mb;
+	g_error("plain mq_putq() forbidden on TCP queue -- use mq_tcp_putq()");
+}
+
+static const struct mq_ops mq_tcp_ops = {
+	mq_no_putq,			/* putq */
+};
+
 /* vi: set ts=4 sw=4 cindent: */
