@@ -1,5 +1,5 @@
 /*
- * $Id: mq_udp.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: mq_udp.c 16204 2009-03-18 20:03:57Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,18 +35,21 @@
 
 #include "common.h"
 
-RCSID("$Id: mq_udp.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: mq_udp.c 16204 2009-03-18 20:03:57Z rmanfredi $")
+
+#define MQ_INTERNAL
+#include "mq.h"
 
 #include "nodes.h"
-#include "mq.h"
 #include "mq_tcp.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "tx.h"
 #include "gnet_stats.h"
 #include "hosts.h"
-#include "core/mq_udp.h"
+#include "mq_udp.h"
+#include "dump.h"
 
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -150,7 +153,7 @@ mq_udp_attach_metadata(pmsg_t *mb, const gnet_host_t *to)
  * owned by the supplied node.
  */
 mqueue_t *
-mq_udp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
+mq_udp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
 {
 	mqueue_t *q;
 
@@ -165,6 +168,7 @@ mq_udp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 	q->qwait = slist_new();
 	q->ops = &mq_udp_ops;
 	q->cops = mq_get_cops();
+	q->debug = GNET_PROPERTY_PTR(mq_udp_debug);
 
 	tx_srv_register(nd, mq_udp_service, q);
 
@@ -178,10 +182,10 @@ static void
 mq_udp_service(gpointer data)
 {
 	mqueue_t *q = data;
-	gint r;
+	int r;
 	GList *l;
-	gint sent;
-	gint dropped;
+	int sent;
+	int dropped;
 
 	mq_check(q, 0);
 	g_assert(q->count);		/* Queue is serviced, we must have something */
@@ -195,8 +199,8 @@ mq_udp_service(gpointer data)
 
 	for (l = q->qtail; l; /* empty */) {
 		pmsg_t *mb = l->data;
-		gchar *mb_start = pmsg_start(mb);
-		gint mb_size = pmsg_size(mb);
+		char *mb_start = pmsg_start(mb);
+		int mb_size = pmsg_size(mb);
 		struct mq_udp_info *mi = pmsg_get_metadata(mb);
 		guint8 function;
 
@@ -284,28 +288,38 @@ void
 mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 {
 	size_t size;
-	gchar *mbs;
+	char *mbs;
 	guint8 function;
 	guint8 hops;
 	pmsg_t *mbe = NULL;		/* Extended message with destination info */
 	gboolean error = FALSE;
 
+	dump_tx_udp_packet(to, mb);
+
 again:
 	g_assert(q);
 	g_assert(mb);
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
 	g_assert(q->ops == &mq_udp_ops);	/* Is an UDP queue */
+
+	/*
+	 * Trap messages enqueued whilst in the middle of an mq_clear() operation
+	 * by marking them as sent and dropping them.  Idem if queue was
+	 * put in "discard" mode.
+	 */
+
+	if (q->flags & (MQ_CLEAR | MQ_DISCARD)) {
+		pmsg_mark_sent(mb);	/* Let them think it was sent */
+		goto cleanup;		/* Drop message */
+	}
+
 	mq_check(q, 0);
 
 	size = pmsg_size(mb);
-	if (size == 0) {
-		g_warning("mq_putq: called with empty message");
-		goto cleanup;
-	}
 
-	if (q->flags & MQ_DISCARD) {
-		g_warning("mq_putq: called whilst queue shutdown");
+	if (size == 0) {
+		g_warning("mq_udp_putq: called with empty message");
 		goto cleanup;
 	}
 
@@ -375,6 +389,10 @@ again:
 			default:
 				break;
 			}
+
+			if (GNET_PROPERTY(mq_udp_debug) > 5)
+				g_message("MQ UDP sent %s", gmsg_infostr_full(pmsg_start(mb)));
+
 			goto cleanup;
 		}
 
@@ -386,13 +404,16 @@ again:
 
 		if (written > 0) {
 			g_warning("partial UDP write (%d bytes) to %s for %d-byte datagram",
-				(gint) written, gnet_host_to_string(to), (gint) size);
+				(int) written, gnet_host_to_string(to), (int) size);
 			goto cleanup;
 		}
 
 		/* FALL THROUGH */
 	}
 
+	if (GNET_PROPERTY(mq_udp_debug) > 5)
+		g_message("MQ UDP queued %s", gmsg_infostr_full(pmsg_start(mb)));
+
 	/*
 	 * Attach the destination information as metadata to the message, unless
 	 * it is already known (possible only during unfolding of the queued data
