@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 15053 2007-12-08 01:15:45Z cbiere $
+ * $Id: nodes.h 16179 2009-03-16 18:16:53Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -44,6 +44,8 @@
 
 #include "lib/header.h"
 
+struct guid;
+
 typedef enum {
 	NODE_MAGIC = 0x67f8e02f
 } node_magic_t;
@@ -105,7 +107,7 @@ typedef struct gnutella_node {
 	node_peer_t peermode;		/**< Operating mode (leaf, ultra, normal) */
 	node_peer_t start_peermode;	/**< Operating mode when handshaking begun */
 
-	gchar error_str[256];		/**< To sprintf() error strings with vars */
+	char error_str[256];		/**< To sprintf() error strings with vars */
 	struct gnutella_socket *socket;		/**< Socket of the node */
 	guint16 country;			/**< Country of origin -- encoded ISO3166 */
 	guint8 proto_major;			/**< Handshaking protocol major number */
@@ -115,18 +117,18 @@ typedef struct gnutella_node {
 	guint8 qrp_minor;			/**< Query routing protocol minor number */
 	guint8 uqrp_major;			/**< UP Query routing protocol major number */
 	guint8 uqrp_minor;			/**< UP Query routing protocol minor number */
-	const gchar *vendor;		/**< Vendor information (always UTF-8) */
+	const char *vendor;		/**< Vendor information (always UTF-8) */
 	vendor_code_t vcode;		/**< Vendor code (vcode.u32 == 0 if unknown) */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 
 	gnutella_header_t header;		/**< Header of the current message */
 	extvec_t extvec[MAX_EXTVEC];	/**< GGEP extensions in "fat" messages */
-	gint extcount;					/**< Amount of extensions held */
+	int extcount;					/**< Amount of extensions held */
 
 	guint16 size; /**< How many bytes we need to read for the current message */
 	guint16 header_flags;		/**< Header flags (new message architecture) */
 
-	gchar *data;				/**< data of the current message */
+	char *data;				/**< data of the current message */
 	guint32 pos;				/**< write position in data */
 
 	gnet_node_state_t status;	/**< See possible values below */
@@ -168,7 +170,7 @@ typedef struct gnutella_node {
 	time_t leaf_flowc_start;	/**< Time when leaf flow-controlled queries */
 	time_delta_t shutdown_delay; /**< How long we can stay in shutdown mode */
 
-	const gchar *remove_msg;	/**< Reason of removing */
+	const char *remove_msg;	/**< Reason of removing */
 
 	host_addr_t addr;			/**< ip of the node */
 	guint16 port;				/**< port of the node */
@@ -212,7 +214,7 @@ typedef struct gnutella_node {
 	guint ping_throttle;		/**< Period for accepting new pings (secs) */
 	time_t ping_accept;			/**< Time after which we accept new pings */
 	time_t next_ping;			/**< When to send a ping, for "OLD" clients */
-	gchar ping_guid[GUID_RAW_SIZE];	/**< The GUID of the last accepted ping */
+	struct guid ping_guid;		/**< The GUID of the last accepted ping */
 	guchar pong_needed[MAX_CACHE_HOPS+1];	/**< Pongs needed, by hop value */
 	guchar pong_missing;	/**< Sum(pong_needed[i]), i = 0..MAX_CACHE_HOPS */
 
@@ -222,7 +224,7 @@ typedef struct gnutella_node {
 	guint32 gnet_kbytes_count;	/**< Used to answer "Crawling" pings */
 	host_addr_t gnet_pong_addr;	/**< When != 0, last IP we got in pong */
 	host_addr_t gnet_qhit_addr;	/**< When != 0, last IP we got in query hit */
-	const gchar *guid;			/**< GUID of node (atom) seen on the network */
+	const struct guid *guid;	/**< GUID of node (atom) seen on the network */
 
 	guint32 svn_release_revision;	/**< Latest SVN release known by the node */
 
@@ -237,13 +239,13 @@ typedef struct gnutella_node {
 	 * Traffic statistics -- RAM, 13/05/2002.
 	 */
 
-	gint32 tx_given;			/**< Bytes fed to the TX stack (from top) */
-	gint32 tx_deflated;			/**< Bytes deflated by the TX stack */
-	gint32 tx_written;			/**< Bytes written by the TX stack */
+	guint64 tx_given;			/**< Bytes fed to the TX stack (from top) */
+	guint64 tx_deflated;		/**< Bytes deflated by the TX stack */
+	guint64 tx_written;			/**< Bytes written by the TX stack */
 
-	gint32 rx_given;			/**< Bytes fed to the RX stack (from bottom) */
-	gint32 rx_inflated;			/**< Bytes inflated by the RX stack */
-	gint32 rx_read;				/**< Bytes read from the RX stack */
+	guint64 rx_given;			/**< Bytes fed to the RX stack (from bottom) */
+	guint64 rx_inflated;		/**< Bytes inflated by the RX stack */
+	guint64 rx_read;			/**< Bytes read from the RX stack */
 
 	/*
 	 * Various Gnutella statistics -- RAM, 10/12/2003.
@@ -321,8 +323,8 @@ enum {
 	NODE_A_DYN_QUERY	= 1 << 12,	/**< Node can perform dynamic queries */
 	NODE_A_CAN_SFLAG	= 1 << 13,	/**< Node supports flags in headers */
  
-	NODE_A_NO_KEPT_ZERO	= 1 << 14,	/**< For GTKG < 2006-08-15: no kept=0! */
-	NODE_A_NO_DUPS		= 1 << 15,	/**< For broken old GTKG: no dups! */
+	NODE_A_UNUSED_1		= 1 << 14,	/**< UNUSED */
+	NODE_A_UNUSED_2		= 1 << 15,	/**< UNUSED */
 	NODE_A_CAN_HSEP		= 1 << 16,	/**< Node supports HSEP */
 	NODE_A_CAN_QRP		= 1 << 17,	/**< Node supports query routing */
 	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
@@ -496,7 +498,7 @@ enum {
 #define GNUTELLA_HELLO "GNUTELLA CONNECT/"
 #define GNUTELLA_HELLO_LENGTH	(sizeof(GNUTELLA_HELLO) - 1)
 
-extern const gchar *start_rfc822_date;
+extern const char *start_rfc822_date;
 
 extern GHookList node_added_hook_list;
 extern struct gnutella_node *node_added;
@@ -521,17 +523,17 @@ gboolean node_host_is_connected(const host_addr_t addr, guint16 port);
 void node_add_socket(struct gnutella_socket *s, const host_addr_t addr,
 		guint16 port, guint32 flags);
 void node_remove(struct gnutella_node *,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
 guint node_remove_by_addr(const host_addr_t addr, guint16 port);
-void node_bye(gnutella_node_t *, gint code,
-	const gchar * reason, ...) G_GNUC_PRINTF(3, 4);
+void node_bye(gnutella_node_t *, int code,
+	const char * reason, ...) G_GNUC_PRINTF(3, 4);
 void node_real_remove(gnutella_node_t *);
 void node_eof(struct gnutella_node *n,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
 void node_shutdown(struct gnutella_node *n,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
-void node_bye_if_writable(struct gnutella_node *n, gint code,
-	const gchar * reason, ...) G_GNUC_PRINTF(3, 4);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
+void node_bye_if_writable(struct gnutella_node *n, int code,
+	const char * reason, ...) G_GNUC_PRINTF(3, 4);
 void node_init_outgoing(struct gnutella_node *);
 void node_sent_ttl0(struct gnutella_node *n);
 void node_disableq(struct gnutella_node *n);
@@ -555,20 +557,20 @@ void node_qrt_install(struct gnutella_node *n, struct routing_table *);
 void node_qrt_patched(struct gnutella_node *n, struct routing_table *);
 
 void send_node_error(struct gnutella_socket *s, int code,
-	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
+	const char *msg, ...) G_GNUC_PRINTF(3, 4);
 
-void node_add_sent(gnutella_node_t *n, gint x);
-void node_add_txdrop(gnutella_node_t *n, gint x);
-void node_add_rxdrop(gnutella_node_t *n, gint x);
+void node_add_sent(gnutella_node_t *n, int x);
+void node_add_txdrop(gnutella_node_t *n, int x);
+void node_add_rxdrop(gnutella_node_t *n, int x);
 
-void node_set_vendor(gnutella_node_t *n, const gchar *vendor);
+void node_set_vendor(gnutella_node_t *n, const char *vendor);
 
 void node_set_hops_flow(gnutella_node_t *n, guint8 hops);
 void node_set_online_mode(gboolean on);
 void node_current_peermode_changed(node_peer_t mode);
-const gchar *node_addr(const gnutella_node_t *n);
-const gchar *node_addr2(const gnutella_node_t *n);
-const gchar *node_gnet_addr(const gnutella_node_t *n);
+const char *node_addr(const gnutella_node_t *n);
+const char *node_addr2(const gnutella_node_t *n);
+const char *node_gnet_addr(const gnutella_node_t *n);
 
 void node_connect_back(const gnutella_node_t *n, guint16 port);
 void node_connected_back(struct gnutella_socket *s);
@@ -576,15 +578,13 @@ void node_connected_back(struct gnutella_socket *s);
 void node_mark_bad_vendor(struct gnutella_node *n);
 
 void node_proxying_remove(gnutella_node_t *n);
-gboolean node_proxying_add(gnutella_node_t *n, const gchar *guid);
+gboolean node_proxying_add(gnutella_node_t *n, const struct guid *guid);
 void node_proxy_add(gnutella_node_t *n, const host_addr_t addr, guint16 port);
 void node_proxy_cancel_all(void);
-size_t node_http_proxies_add(gchar *buf, size_t size,
+size_t node_http_proxies_add(char *buf, size_t size,
 			gpointer arg, guint32 flags);
 const GSList *node_push_proxies(void);
 const GSList *node_all_nodes(void);
-const GSList *node_all_but_broken_gtkg(void);
-
 
 gnutella_node_t *node_by_id(const node_id_t node_id);
 gnutella_node_t *node_active_by_id(const node_id_t node_id);
@@ -592,15 +592,17 @@ void node_set_leaf_guidance(const node_id_t node_id, gboolean supported);
 
 void node_became_firewalled(void);
 void node_became_udp_firewalled(void);
-void node_set_socket_rx_size(gint rx_size);
+void node_set_socket_rx_size(int rx_size);
 
 mqueue_t *node_udp_get_outq(enum net_type net);
+gboolean node_udp_is_flow_controlled(void);
+gboolean node_udp_would_flow_control(size_t additional);
 void node_udp_disable(void);
 void node_udp_process(struct gnutella_socket *s);
 gnutella_node_t *node_udp_get_addr_port(const host_addr_t addr, guint16 port);
 
 void node_can_tsync(gnutella_node_t *n);
-void node_crawl(gnutella_node_t *n, gint ucnt, gint lcnt, guint8 features);
+void node_crawl(gnutella_node_t *n, int ucnt, int lcnt, guint8 features);
 
 void node_update_udp_socket(void);
 void node_check_remote_ip_header(const host_addr_t peer, header_t *head);
@@ -609,8 +611,8 @@ guint feed_host_cache_from_headers(header_t *headers,
 	host_type_t sender, gboolean gnet, const host_addr_t peer);
 
 gnutella_node_t *node_browse_prepare(
-	gnet_host_t *host, const gchar *vendor, gnutella_header_t *header,
-	gchar *data, guint32 size);
+	gnet_host_t *host, const char *vendor, gnutella_header_t *header,
+	char *data, guint32 size);
 void node_browse_cleanup(gnutella_node_t *n);
 void node_kill_hostiles(void);
 void node_supports_tls(struct gnutella_node *);
@@ -625,14 +627,14 @@ node_check(const struct gnutella_node * const n)
 	g_assert(NODE_MAGIC == n->magic);
 }
 
-static inline const gchar *
+static inline const struct guid *
 node_guid(const struct gnutella_node * const n)
 {
 	return n->guid;
 }
 
-gboolean node_set_guid(struct gnutella_node *n, const gchar *guid);
-struct gnutella_node *node_by_guid(const gchar *guid);
+gboolean node_set_guid(struct gnutella_node *n, const struct guid *guid);
+struct gnutella_node *node_by_guid(const struct guid *guid);
 
 #endif /* _core_nodes_h_ */
 
