@@ -1,5 +1,5 @@
 /*
- * $Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: oob.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,14 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $")
+RCSID("$Id: oob.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "oob.h"
 #include "hosts.h"
 #include "nodes.h"
 #include "share.h"
 #include "guid.h"
-#include "pmsg.h"
 #include "mq.h"
 #include "mq_udp.h"
 #include "vmsg.h"
@@ -55,6 +54,7 @@ RCSID("$Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $")
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/fifo.h"
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -66,29 +66,35 @@ RCSID("$Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $")
 #define OOB_MAX_QUEUED		50				/**< Max # of messages per host */
 #define OOB_MAX_RETRY		3				/**< Retry # if LIME/12v2 dropped */
 
-#define OOB_MAX_QHIT_SIZE	645				/**< Flush hits larger than this */
-#define OOB_MAX_DQHIT_SIZE	1075			/**< Flush limit for deflated hits */
+#define OOB_MAX_QHIT_SIZE	645			/**< Flush hits larger than this */
+#define OOB_MAX_DQHIT_SIZE	1075		/**< Flush limit for deflated hits */
+
+typedef enum {
+	OOB_RESULTS_MAGIC = 0x7ae5e685
+} oob_results_magic_t;
 
 /**
  * A set of hits awaiting delivery.
  */
 struct oob_results {
-	gpointer ev_expire;		/**< Global expiration event */
-	gpointer ev_timeout;	/**< Reply waiting timeout */
-	gchar *muid;			/**< (atom) MUID of the query that generated hits */
+	oob_results_magic_t	magic;
+	int refcount;
+	cevent_t *ev_expire;	/**< Global expiration event */
+	cevent_t *ev_timeout;	/**< Reply waiting timeout */
+	const struct guid *muid;/**< (atom) MUID of the query that generated hits */
 	GSList *files;			/**< List of shared_file_t */
 	gnet_host_t dest;		/**< The host to which we must deliver */
-	gint count;				/**< Amount of hits to deliver */
-	gint notify_requeued;	/**< Amount of LIME/12v2 requeued after dropping */
-	gboolean use_ggep_h;	/**< Whether GGEP "H" can be used for SHA1 coding */
-	gint refcount;
+	int count;				/**< Amount of hits to deliver */
+	int notify_requeued;	/**< Amount of LIME/12v2 requeued after dropping */
+	gboolean secure;		/**< TRUE -> secure OOB, FALSE -> normal OOB */
+	gboolean ggep_h;		/**< TRUE -> use GGEP H, FALSE -> plain text */
 };
 
 /**
  * Indexes all OOB queries by MUID.
  * This hash table records MUID => "struct oob_results"
  */
-static GHashTable *results_by_muid = NULL;
+static GHashTable *results_by_muid;
 
 /**
  * Each servent, as identified by its IP:port, is given a FIFO for queuing
@@ -103,10 +109,10 @@ static GHashTable *servent_by_host = NULL;
  * A servent entry, used as values in the `servent_by_host' table.
  */
 struct gservent {
-	gpointer ev_service;	/**< Callout event for servicing FIFO */
-	gnet_host_t *host;		/**< The servent host (also used as key for table) */
-	fifo_t *fifo;			/**< The servent's FIFO, holding pmsg_t items */
-	gboolean can_deflate;	/**< Whether servent supports UDP compression */
+	cevent_t *ev_service; /**< Callout event for servicing FIFO */
+	gnet_host_t *host;	  /**< The servent host (also used as key for table) */
+	fifo_t *fifo;		  /**< The servent's FIFO, holding pmsg_t items */
+	gboolean can_deflate; /**< Whether servent supports UDP compression */
 };
 
 /*
@@ -128,37 +134,49 @@ static void results_destroy(cqueue_t *cq, gpointer obj);
 static void servent_free(struct gservent *s);
 static void oob_send_reply_ind(struct oob_results *r);
 
-static gint num_oob_records;	/**< Leak and duplicate free detector */
+static int num_oob_records;	/**< Leak and duplicate free detector */
+static gboolean oob_shutdown_running;
+
+static void
+oob_results_check(const struct oob_results *r)
+{
+	g_assert(r);
+	g_assert(OOB_RESULTS_MAGIC == r->magic);
+	g_assert(r->refcount >= 0);
+	g_assert(r->muid);
+}
 
 /**
  * Create new "struct oob_results" to handle the initial negotiation of
  * results delivery via the sent LIME/12v2 and the expected LIME/11v2 reply.
  */
 static struct oob_results *
-results_make(const gchar *muid, GSList *files,
-	gint count, gnet_host_t *to, gboolean ggep_h)
+results_make(const struct guid *muid, GSList *files, int count,
+	gnet_host_t *to, gboolean secure, gboolean ggep_h)
 {
 	static const struct oob_results zero_results;
 	struct oob_results *r;
 
+	g_return_val_if_fail(!g_hash_table_lookup(results_by_muid, muid), NULL);
+
 	r = walloc(sizeof *r);
 	*r = zero_results;
-
+	r->magic = OOB_RESULTS_MAGIC;
 	r->muid = atom_guid_get(muid);
 	r->files = files;
 	r->count = count;
 	r->dest = *to;			/* Struct copy */
-	r->use_ggep_h = ggep_h;
-	r->refcount = 1;
+	r->secure = secure;
+	r->ggep_h = ggep_h;
 
 	r->ev_expire = cq_insert(callout_queue, OOB_EXPIRE_MS, results_destroy, r);
+	r->refcount++;
 
-	g_assert(!g_hash_table_lookup(results_by_muid, r->muid));
-	g_hash_table_insert(results_by_muid, r->muid, r);
+	gm_hash_table_insert_const(results_by_muid, r->muid, r);
 
 	g_assert(num_oob_records >= 0);
 	num_oob_records++;
-	if (query_debug > 1)
+	if (GNET_PROPERTY(query_debug) > 1)
 		g_message("results_make: num_oob_records=%d", num_oob_records);
 
 	return r;
@@ -172,45 +190,42 @@ results_free_remove(struct oob_results *r)
 {
 	GSList *sl;
 
-	g_assert(r->refcount > 0);
+	oob_results_check(r);
+	
+	if (r->ev_expire) {
+		cq_cancel(callout_queue, &r->ev_expire);
+		g_assert(r->refcount > 0);
+		r->refcount--;
+	}
+	if (r->ev_timeout) {
+		cq_cancel(callout_queue, &r->ev_timeout);
+		g_assert(r->refcount > 0);
+		r->refcount--;
+	}
 
-	r->refcount--;
-
-	if (NULL == r->ev_expire || 0 == r->refcount) {
-		if (r->muid) {
+	if (0 == r->refcount) {
+		/* We must not modify the hash table whilst iterating over it */
+		if (!oob_shutdown_running) {
 			g_assert(r == g_hash_table_lookup(results_by_muid, r->muid));
 			g_hash_table_remove(results_by_muid, r->muid);
-
-			atom_guid_free(r->muid);
-			r->muid = NULL;
 		}
+		atom_guid_free_null(&r->muid);
 
 		for (sl = r->files; sl; sl = g_slist_next(sl)) {
 			shared_file_t *sf = sl->data;
-			shared_file_unref(sf);
+			shared_file_unref(&sf);
 		}
 		g_slist_free(r->files);
 		r->files = NULL;
+
+		g_assert(num_oob_records > 0);
+		num_oob_records--;
+		if (GNET_PROPERTY(query_debug) > 2)
+			g_message("results_free: num_oob_records=%d", num_oob_records);
+
+		r->magic = 0;
+		wfree(r, sizeof *r);
 	}
-
-	if (r->refcount > 0)
-		return;
-
-	g_assert(num_oob_records > 0);
-	num_oob_records--;
-	if (query_debug > 2)
-		g_message("results_free: num_oob_records=%d", num_oob_records);
-
-	if (r->ev_expire) {
-		cq_cancel(callout_queue, r->ev_expire);
-		r->ev_expire = NULL;
-	}
-	if (r->ev_timeout) {
-		cq_cancel(callout_queue, r->ev_timeout);
-		r->ev_timeout = NULL;
-	}
-
-	wfree(r, sizeof *r);
 }
 
 /**
@@ -222,16 +237,18 @@ results_destroy(cqueue_t *unused_cq, gpointer obj)
 	struct oob_results *r = obj;
 
 	(void) unused_cq;
+	oob_results_check(r);
 
-	if (query_debug)
+	if (GNET_PROPERTY(query_debug))
 		g_message("OOB query %s from %s expired with unclaimed %d hit%s",
-			guid_hex_str(r->muid),
-			host_addr_port_to_string(r->dest.addr, r->dest.port),
+			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s");
 
 	gnet_stats_count_general(GNR_UNCLAIMED_OOB_HITS, 1);
 
 	r->ev_expire = NULL;		/* The timer which just triggered */
+	r->refcount--;
+
 	results_free_remove(r);
 }
 
@@ -244,16 +261,18 @@ results_timeout(cqueue_t *unused_cq, gpointer obj)
 	struct oob_results *r = obj;
 
 	(void) unused_cq;
+	oob_results_check(r);
 
-	if (query_debug)
+	if (GNET_PROPERTY(query_debug))
 		g_message("OOB query %s, no ACK from %s to claim %d hit%s",
-			guid_hex_str(r->muid),
-			host_addr_port_to_string(r->dest.addr, r->dest.port),
+			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s");
 
 	gnet_stats_count_general(GNR_UNCLAIMED_OOB_HITS, 1);
 
 	r->ev_timeout = NULL;		/* The timer which just triggered */
+	r->refcount--;
+
 	results_free_remove(r);
 }
 
@@ -273,7 +292,7 @@ servent_free_remove(struct gservent *s)
  * Per a suggestion of Daniel Stutzbach, we wait BASE + RAND*random secs,
  * where "random" is a real random number between 0 and 1.
  */
-static gint
+static int
 deliver_delay(void)
 {
 	return OOB_DELIVER_BASE_MS + random_value(OOB_DELIVER_RAND_MS);
@@ -296,15 +315,14 @@ servent_service(cqueue_t *cq, gpointer obj)
 	if (mb == NULL)
 		goto remove;
 
-	q = node_udp_get_outq(host_addr_net(s->host->addr));
+	q = node_udp_get_outq(host_addr_net(gnet_host_get_addr(s->host)));
 	if (q == NULL)
 		goto udp_disabled;
 
-	if (udp_debug > 19)
+	if (GNET_PROPERTY(udp_debug) > 19)
 		g_message("UDP queuing OOB %s to %s for %s",
-			gmsg_infostr_full(pmsg_start(mb)),
-			host_addr_port_to_string(s->host->addr, s->host->port),
-			guid_hex_str(pmsg_start(mb)));
+			gmsg_infostr_full(pmsg_start(mb)), gnet_host_to_string(s->host),
+			guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))));
 
 	/*
 	 * Count enqueued deflated payloads, only when server was marked as
@@ -360,7 +378,7 @@ servent_make(gnet_host_t *host, gboolean can_deflate)
 static void
 free_pmsg(gpointer item, gpointer unused_udata)
 {
-	pmsg_t *mb = (pmsg_t *) item;
+	pmsg_t *mb = item;
 
 	(void) unused_udata;
 	pmsg_free(mb);
@@ -372,8 +390,7 @@ free_pmsg(gpointer item, gpointer unused_udata)
 static void
 servent_free(struct gservent *s)
 {
-	if (s->ev_service)
-		cq_cancel(callout_queue, s->ev_service);
+	cq_cancel(callout_queue, &s->ev_service);
 	wfree(s->host, sizeof *s->host);
 	fifo_free_all(s->fifo, free_pmsg, NULL);
 	wfree(s, sizeof *s);
@@ -386,7 +403,7 @@ servent_free(struct gservent *s)
 static void
 oob_record_hit(gpointer data, size_t len, gpointer udata)
 {
-	struct gservent *s = (struct gservent *) udata;
+	struct gservent *s = udata;
 
 	g_assert(len <= INT_MAX);
 	fifo_put(s->fifo, s->can_deflate ?
@@ -401,28 +418,32 @@ oob_record_hit(gpointer data, size_t len, gpointer udata)
  * @param n			where we got the message from
  * @param muid		the query identifier
  * @param wanted	the amount of results they want delivered
+ * @param token		the token for secure OOB
  */
 void
-oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted)
+oob_deliver_hits(struct gnutella_node *n, const struct guid *muid,
+	guint8 wanted, const struct array *token)
 {
 	struct oob_results *r;
 	struct gservent *s;
-	gint deliver_count;
+	int deliver_count;
 	gboolean servent_created = FALSE;
 
 	g_assert(NODE_IS_UDP(n));
+	g_assert(token);
 
 	r = g_hash_table_lookup(results_by_muid, muid);
 
 	if (r == NULL) {
 		gnet_stats_count_general(GNR_SPURIOUS_OOB_HIT_CLAIM, 1);
-		if (query_debug)
+		if (GNET_PROPERTY(query_debug))
 			g_warning("OOB got spurious LIME/11 from %s for %s, "
 				"asking for %d hit%s",
 				node_addr(n), guid_hex_str(muid),
 				wanted, wanted == 1 ? "" : "s");
 		return;
 	}
+	oob_results_check(r);
 
 	/*
 	 * Here's what could happen with proxied OOB queries:
@@ -448,17 +469,22 @@ oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted)
 	 *		--RAM, 2004-09-10
 	 */
 
-	if (!host_addr_equal(n->addr, r->dest.addr)) {
-		g_warning("OOB query %s must have been proxied: it had IP %s:%u, "
+	if (!host_addr_equal(n->addr, gnet_host_get_addr(&r->dest))) {
+		/**
+		 * The sender's IP address can of course change any time as
+		 * dynamic IP addresses are very common. The sender might also
+		 * have multiple network interfaces.
+		 */
+		
+		g_warning("OOB query %s might have been proxied: it had IP %s, "
 			"but the LIME/11v2 ACK comes from %s", guid_hex_str(muid),
-			host_addr_to_string(r->dest.addr), r->dest.port, node_addr(n));
+			gnet_host_to_string(&r->dest), node_addr(n));
 
 		/*
 		 * We'll send the hits to the host from where the ACK comes.
 		 */
 
-		r->dest.addr = n->addr;
-		r->dest.port = n->port;
+		gnet_host_set(&r->dest, n->addr, n->port);
 	}
 
 	/*
@@ -488,7 +514,7 @@ oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted)
 
 	deliver_count = (wanted == 255) ? r->count : MIN(wanted, r->count);
 
-	if (query_debug || udp_debug)
+	if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
 		g_message("OOB query %s: host %s wants %d hit%s, delivering %d",
 			guid_hex_str(r->muid), node_addr(n), wanted, wanted == 1 ? "" : "s",
 			deliver_count);
@@ -497,9 +523,7 @@ oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted)
 		qhit_build_results(
 			r->files, deliver_count,
 			s->can_deflate ? OOB_MAX_DQHIT_SIZE : OOB_MAX_QHIT_SIZE,
-			oob_record_hit, s,
-			r->muid,
-			r->use_ggep_h);
+			oob_record_hit, s, r->muid, r->ggep_h, token);
 
 	if (wanted < r->count)
 		gnet_stats_count_general(GNR_PARTIALLY_CLAIMED_OOB_HITS, 1);
@@ -531,6 +555,8 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 	struct oob_results *r = arg;
 
 	g_assert(pmsg_is_extended(mb));
+	oob_results_check(r);
+	r->refcount--;
 
 	/*
 	 * If we sent the message, great!  Arm a timer to ensure we get a
@@ -547,12 +573,10 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 			results_free_remove(r);
 		} else {
 
-			if (query_debug || udp_debug)
+			if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
 				g_message("OOB query %s, notified %s about %d hit%s",
-						guid_hex_str(r->muid),
-						host_addr_port_to_string(r->dest.addr, r->dest.port),
-						r->count,
-						r->count == 1 ? "" : "s");
+					guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
+					r->count, r->count == 1 ? "" : "s");
 
 			/*
 			 * If we don't get any ACK back, we'll discard the results.
@@ -560,23 +584,22 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 
 			r->ev_timeout = cq_insert(callout_queue, OOB_TIMEOUT_MS,
 					results_timeout, r);
+			r->refcount++;
 		}
+	} else {
+		/*
+		 * If we were not able to send the message,
+		 */
 
-		return;
+		if (GNET_PROPERTY(query_debug))
+			g_message("OOB query %s, previous LIME12/v2 #%d was dropped",
+					guid_hex_str(r->muid), r->notify_requeued);
+
+		if (++r->notify_requeued < OOB_MAX_RETRY)
+			oob_send_reply_ind(r);
+		else
+			results_free_remove(r);
 	}
-
-	/*
-	 * If we were not able to send the message,
-	 */
-
-	if (query_debug)
-		g_message("OOB query %s, previous LIME12/v2 #%d was dropped",
-			guid_hex_str(r->muid), r->notify_requeued);
-
-	if (++r->notify_requeued < OOB_MAX_RETRY)
-		oob_send_reply_ind(r);
-	else
-		results_free_remove(r);
 }
 
 /**
@@ -589,19 +612,20 @@ oob_send_reply_ind(struct oob_results *r)
 	pmsg_t *mb;
 	pmsg_t *emb;
 
-	q = node_udp_get_outq(host_addr_net(r->dest.addr));
+	oob_results_check(r);
+
+	q = node_udp_get_outq(host_addr_net(gnet_host_get_addr(&r->dest)));
 	if (q == NULL)
 		return;
 
-	mb = vmsg_build_oob_reply_ind(r->muid, MIN(r->count, 255));
+	mb = vmsg_build_oob_reply_ind(r->muid, MIN(r->count, 255), r->secure);
 	emb = pmsg_clone_extend(mb, oob_pmsg_free, r);
 	r->refcount++;
 	pmsg_free(mb);
 
-	if (query_debug || udp_debug)
+	if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
 		g_message("OOB query %s, notifying %s about %d hit%s, try #%d",
-			guid_hex_str(r->muid), host_addr_port_to_string(r->dest.addr,
-				r->dest.port),
+			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s", r->notify_requeued);
 
 	mq_udp_putq(q, emb, &r->dest);
@@ -614,11 +638,12 @@ oob_send_reply_ind(struct oob_results *r)
  * @param n				the node from which we got the query
  * @param files			the list of shared_file_t entries that make up results
  * @param count			the amount of results
- * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
+ * @param secure		whether secure OOB was requested
+ * @param ggep_h		whether GGEP H is understood
  */
 void
-oob_got_results(
-	struct gnutella_node *n, GSList *files, gint count, gboolean use_ggep_h)
+oob_got_results(struct gnutella_node *n, GSList *files,
+	int count, gboolean secure, gboolean ggep_h)
 {
 	struct oob_results *r;
 	gnet_host_t to;
@@ -629,14 +654,12 @@ oob_got_results(
 	g_assert(files != NULL);
 
 	guid_oob_get_addr_port(gnutella_header_get_muid(&n->header), &addr, &port);
-
-	to.addr = addr;
-	to.port = port;
-
-	r = results_make(gnutella_header_get_muid(&n->header),
-			files, count, &to, use_ggep_h);
-
-	oob_send_reply_ind(r);
+	gnet_host_set(&to, addr, port);
+	r = results_make(gnutella_header_get_muid(&n->header), files, count, &to,
+			secure, ggep_h);
+	if (r) {
+		oob_send_reply_ind(r);
+	}
 }
 
 /**
@@ -655,20 +678,19 @@ oob_init(void)
 static void
 free_oob_kv(gpointer key, gpointer value, gpointer unused_udata)
 {
-	gchar *muid = key;
 	struct oob_results *r = value;
 
 	(void) unused_udata;
-	g_assert(r->refcount > 0);
-	g_assert(NULL != r->muid);
-	g_assert(muid == r->muid);		/* Key is same as results's MUID */
+	oob_results_check(r);
+	g_assert(key == r->muid);		/* Key is same as results's MUID */
 
-	/* Must be freed here to prevent that results_free_remove() tries
-	 * to remove it from the hash table whilst iterating over it. */
-	atom_guid_free(r->muid);
-	r->muid = NULL;
-
-	r->refcount = 1; /* Enforce release */
+	r->refcount = 0; /* Enforce release */
+	if (r->ev_timeout) {
+		r->refcount++;
+	}
+	if (r->ev_expire) {
+		r->refcount++;
+	}
 	results_free_remove(r);
 }
 
@@ -678,8 +700,8 @@ free_oob_kv(gpointer key, gpointer value, gpointer unused_udata)
 static void
 free_servent_kv(gpointer key, gpointer value, gpointer unused_udata)
 {
-	gnet_host_t *host = (gnet_host_t *) key;
-	struct gservent *s = (struct gservent *) value;
+	gnet_host_t *host = key;
+	struct gservent *s = value;
 
 	(void) unused_udata;
 	g_assert(host == s->host);		/* Key is same as servent's host */
@@ -693,6 +715,8 @@ free_servent_kv(gpointer key, gpointer value, gpointer unused_udata)
 void
 oob_shutdown(void)
 {
+	oob_shutdown_running = TRUE;
+
 	g_hash_table_foreach(results_by_muid, free_oob_kv, NULL);
 	g_hash_table_destroy(results_by_muid);
 	results_by_muid = NULL;
