@@ -1,5 +1,5 @@
 /*
- * $Id: parq.h 14932 2007-09-19 01:49:18Z cbiere $
+ * $Id: parq.h 16219 2009-03-19 23:33:24Z rmanfredi $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -51,17 +51,18 @@
  * Public interface.
  */
 
+struct guid;
 struct parq_dl_queued;
 struct parq_ul_queued;
 
 void parq_init(void);
 void parq_close(void);
 
-const gchar *get_parq_dl_id(const struct download *);
+const char *get_parq_dl_id(const struct download *);
 void parq_dl_reparent_id(struct download *d, struct download *cd);
 
 struct parq_dl_queued *parq_dl_create(const struct download *);
-void parq_dl_add_id(struct download *, const gchar *new_id);
+void parq_dl_add_id(struct download *, const char *new_id);
 void parq_dl_remove(struct download *);
 void parq_dl_free(struct download *);
 
@@ -69,26 +70,26 @@ void parq_download_retry_active_queued(struct download *);
 gboolean parq_download_supports_parq(header_t *);
 gboolean parq_download_parse_queue_status(struct download *, header_t *);
 gboolean parq_download_is_active_queued(const struct download *);
-void parq_download_add_header(gchar *buf, size_t len, size_t *rw,
+void parq_download_add_header(char *buf, size_t len, size_t *rw,
 	struct download *);
 gboolean parq_download_is_passive_queued(const struct download *);
 void parq_download_queue_ack(struct gnutella_socket *);
 
 void parq_upload_timer(time_t now);
 
-size_t parq_upload_add_headers(gchar *buf, size_t size,
+size_t parq_upload_add_headers(char *buf, size_t size,
 	gpointer arg, guint32 flags);
-size_t parq_upload_add_header_id(gchar *buf, size_t size,
+size_t parq_upload_add_header_id(char *buf, size_t size,
 	gpointer arg, guint32 flags);
 
-struct parq_ul_queued *parq_upload_get(struct upload *, header_t *,
-				gboolean replacing);
+struct parq_ul_queued *parq_upload_get(struct upload *, const header_t *);
 gboolean parq_upload_request(struct upload *);
 gboolean parq_upload_request_force(struct upload *, struct parq_ul_queued *);
 guint parq_upload_lookup_position(const struct upload *);
-const gchar * parq_upload_lookup_id(const struct upload *);
+const struct guid *parq_upload_lookup_id(const struct upload *);
 gboolean parq_upload_queue_full(struct upload *);
 guint parq_upload_lookup_size(const struct upload *);
+void parq_upload_update_downloaded(const struct upload *u);
 
 time_t parq_upload_lifetime(const struct upload *);
 time_t parq_upload_retry(const struct upload *);
@@ -105,7 +106,6 @@ void parq_upload_upload_got_cloned(struct upload *u, struct upload *cu);
 void parq_upload_force_remove(struct upload *);
 void parq_upload_add(struct upload *);
 void parq_upload_busy(struct upload *, struct parq_ul_queued *);
-void parq_upload_save_queue(void);
 void parq_upload_send_queue_conf(struct upload *);
 
 gboolean parq_ul_id_sent(const struct upload *);
