@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.c 15129 2008-03-11 10:40:37Z rmanfredi $
+ * $Id: pcache.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pcache.c 15129 2008-03-11 10:40:37Z rmanfredi $")
+RCSID("$Id: pcache.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
 #include "alive.h"
 #include "extensions.h"
@@ -53,23 +53,26 @@ RCSID("$Id: pcache.c 15129 2008-03-11 10:40:37Z rmanfredi $")
 #include "settings.h"
 #include "share.h" /* For shared_files_scanned() and shared_kbytes_scanned(). */
 #include "sockets.h"
-#include "tls_cache.h"
+#include "ipp_cache.h"
 #include "udp.h"
 #include "uhc.h"
 #include "version.h"
 
-#include "if/core/hosts.h"
 #include "if/gnet_property_priv.h"
+#include "if/dht/kademlia.h"
+#include "if/dht/dht.h"
 
 #include "lib/aging.h"
 #include "lib/atoms.h"
 #include "lib/endian.h"
+#include "lib/gnet_host.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
 #define PCACHE_MAX_FILES	10000000	/**< Arbitrarily large file count */
 #define PCACHE_UHC_MAX_IP	30			/**< Max amount of IP:port returned */
+#define PCACHE_DHT_MAX_IP	10			/**< Max amount of IP:port returned */
 
 /**
  * Basic pong information.
@@ -87,7 +90,10 @@ enum ping_flag {
 	PING_F_UHC_LEAF		= (1 << 1),	/**< UHC ping, wants leaf slots */
 	PING_F_UHC_ULTRA	= (1 << 2),	/**< UHC ping, wants ultra slots */
 	PING_F_UHC_ANY		= (PING_F_UHC_LEAF | PING_F_UHC_ULTRA),
-	PING_F_IP			= (1 << 3)	/**< GGEP IP */
+	PING_F_IP			= (1 << 3),	/**< GGEP IP */
+	PING_F_DHTIPP		= (1 << 4),	/**< GGEP DHTIPP, wants DHT hosts */
+
+	PING_LAST_ENUM_FLAG
 };
 
 static pong_meta_t local_meta;
@@ -148,11 +154,11 @@ send_ping(struct gnutella_node *n, guint8 ttl)
  * @return pointer to static data, and the size of the message in `size'.
  */
 gnutella_msg_init_t *
-build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
+build_ping_msg(const struct guid *muid, guint8 ttl, gboolean uhc, guint32 *size)
 {
 	static union {
 		gnutella_msg_init_t s;
-		gchar buf[256];
+		char buf[256];
 		guint64 align8;
 	} msg_init;
 	gnutella_msg_init_t *m = &msg_init.s;
@@ -183,7 +189,7 @@ build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
 		guchar *ggep;
 		ggep_stream_t gs;
 		gboolean ok;
-		gchar spp;
+		char spp;
 
 		ggep = cast_to_gpointer(&m[1]);
 		ggep_stream_init(&gs, ggep, sizeof msg_init.buf - sizeof *m);
@@ -210,6 +216,13 @@ build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
 			g_assert(ok);
 		}
 
+		/*
+		 * If DHT is not seeded yet, ask for more hosts through "DHTIPP".
+		 */
+
+		if (dht_enabled() && !dht_seeded())
+			ggep_stream_pack(&gs, GGEP_NAME(DHTIPP), NULL, 0, 0);
+
 		sz += ggep_stream_close(&gs);
 	}
 
@@ -228,13 +241,13 @@ build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
  */
 static gnutella_msg_init_response_t *
 build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
-	guint8 hops, guint8 ttl, const gchar *muid,
+	guint8 hops, guint8 ttl, const struct guid *muid,
 	struct pong_info *info, pong_meta_t *meta, enum ping_flag flags,
 	guint32 *size)
 {
 	static union {
 		gnutella_msg_init_response_t s;
-		gchar buf[1024];
+		char buf[1024];
 		guint64 align8;
 	} msg_pong;
 	gnutella_msg_init_response_t *pong = &msg_pong.s;
@@ -274,7 +287,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 	ggep_stream_init(&gs, ggep, sizeof msg_pong.buf - sizeof *pong);
 
 	/*
-	 * First, start with metadata about our host.
+	 * First, start with metadata about the host.
 	 */
 
 	if (meta != NULL) {
@@ -315,7 +328,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		}
 
 		if (meta->flags & PONG_META_HAS_DU) {	/* Daily average uptime */
-			gchar uptime[sizeof(guint64)];
+			char uptime[sizeof(guint64)];
 			guint32 value = MIN(meta->daily_uptime, 86400);
 			guint len;
 
@@ -331,6 +344,16 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		if (meta->flags & PONG_META_HAS_TLS) {
 			ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0);
 		}
+
+		if (meta->flags & PONG_META_HAS_DHT) {
+			gboolean ok;
+
+			ok = ggep_stream_begin(&gs, GGEP_NAME(DHT), 0) &&
+			ggep_stream_write(&gs, &meta->dht_major, 1) &&
+			ggep_stream_write(&gs, &meta->dht_minor, 1) &&
+			ggep_stream_write(&gs, &meta->dht_mode, 1) &&
+			ggep_stream_end(&gs);
+		}
 	}
 
 	/*
@@ -346,7 +369,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		 */
 
 		gnet_host_t host[PCACHE_UHC_MAX_IP];
-		gint hcount;
+		int hcount;
 
 		hcount = hcache_fill_caught_array(HOST_ULTRA, host, PCACHE_UHC_MAX_IP);
 
@@ -354,7 +377,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 			guchar tls_bytes[(G_N_ELEMENTS(host) + 7) / 8];
 			guint tls_index, tls_length;
 			gboolean ok;
-			gint i;
+			int i;
 
 			/*
 			 * The binary data that makes up IPP does not deflate well.
@@ -370,7 +393,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 			ok = ggep_stream_begin(&gs, GGEP_NAME(IPP), 0);
 
 			for (i = 0; ok && i < hcount; i++) {
-				gchar addr_buf[6];
+				char addr_buf[6];
 				host_addr_t addr;
 				guint16 port;
 
@@ -400,8 +423,49 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		}
 	}
 
+	/*
+	 * If they gave "DHTIPP" in their ping, send them a packed list of
+	 * valid DHT contacts, in addr:port packed format, similar to "IPP".
+	 *
+	 * NB: The port here is little-endian, since this is a Gnutella message
+	 * and not a Kademlia one.
+	 */
+
+	if (0 != (flags & PING_F_DHTIPP)) {
+		gnet_host_t host[PCACHE_DHT_MAX_IP];
+		int hcount;
+
+		hcount = dht_fill_random(host, G_N_ELEMENTS(host));
+
+		if (hcount > 0) {
+			gboolean ok;
+			int i;
+
+			ok = ggep_stream_begin(&gs, GGEP_NAME(DHTIPP), 0);
+
+			for (i = 0; ok && i < hcount; i++) {
+				char addr_buf[6];
+				host_addr_t addr;
+				guint16 port;
+
+				/* @todo TODO: IPv6 */
+				if (NET_TYPE_IPV4 != gnet_host_get_net(&host[i]))
+					continue;
+				
+				addr = gnet_host_get_addr(&host[i]);
+				port = gnet_host_get_port(&host[i]);
+				poke_be32(&addr_buf[0], host_addr_ipv4(addr));
+				poke_le16(&addr_buf[4], port);
+
+				ok = ggep_stream_write(&gs, addr_buf, sizeof addr_buf);
+			}
+
+			ok = ok && ggep_stream_end(&gs);
+		}
+	}
+
 	if ((flags & PING_F_IP) && NET_TYPE_IPV4 == host_addr_net(sender_addr)) {
-		gchar ip_port[6];
+		char ip_port[6];
 
 		/* Ip Port (not UHC IPP!)*/
 		if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(ggep_debug))
@@ -432,7 +496,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 static void
 send_pong(
 	struct gnutella_node *n, gboolean control, enum ping_flag flags,
-	guint8 hops, guint8 ttl, const gchar *muid,
+	guint8 hops, guint8 ttl, const struct guid *muid,
 	struct pong_info *info, pong_meta_t *meta)
 {
 	gnutella_msg_init_response_t *r;
@@ -463,14 +527,18 @@ send_pong(
 }
 
 /**
- * Determine whether this is an UHC ping (mentionning "SCP" support).
+ * Scan extensions in the incoming ping to determine whether this is
+ * an UHC ping (mentionning "SCP" support), or whether the ping is requesting
+ * DHT hosts in packed IP format (mentionning "DHTIPP").
  *
- * @return UHC_NONE if not an UHC ping, the UHC type otherwise.
+ * @return ping flags summarizing the remote request.  For instance PING_F_UHC
+ * will indicate an UHC ping, and PING_F_DHTIPP will indicate a ping for
+ * DHT hosts.
  */
 static enum ping_flag
 ping_type(const gnutella_node_t *n)
 {
-	gint i;
+	int i;
 	enum ping_flag flags = PING_F_NONE;
 
 	for (i = 0; i < n->extcount; i++) {
@@ -510,6 +578,11 @@ ping_type(const gnutella_node_t *n)
 			}
 			break;
 
+		case EXT_T_GGEP_DHTIPP:
+			if (0 == ext_paylen(e))
+				flags |= PING_F_DHTIPP;
+			break;
+
 		default: ;
 		}
 
@@ -612,6 +685,19 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 		local_meta.flags |= PONG_META_HAS_IPV6;
 	}
 
+	/*
+	 * If the DHT is up and running and we're not firewalled, send an
+	 * indication.  These tagged pongs are used by hosts to bootstrap
+	 * their DHT routing table and join.
+	 */
+
+	if (!GNET_PROPERTY(is_udp_firewalled) && dht_bootstrapped()) {
+		local_meta.dht_major = KDA_VERSION_MAJOR;
+		local_meta.dht_minor = KDA_VERSION_MINOR;
+		local_meta.dht_mode = DHT_MODE_ACTIVE;	/* XXX Active mode */
+		local_meta.flags |= PONG_META_HAS_DHT;
+	}
+
 	local_meta.flags |= tls_enabled() ? PONG_META_HAS_TLS : 0;
 
 	send_pong(n, control, flags, 0,
@@ -700,7 +786,7 @@ static time_t pcache_expire_time = 0;
 static struct aging *udp_pings;
 
 struct cached_pong {		/**< A cached pong */
-	gint refcount;			/**< How many lists reference us? */
+	int refcount;			/**< How many lists reference us? */
 	node_id_t node_id;		/**< The node ID from which we got that pong */
 	node_id_t last_sent_id; /**< Node ID we last sent this pong to */
 	struct pong_info info;	/**< Values from the pong message */
@@ -708,7 +794,7 @@ struct cached_pong {		/**< A cached pong */
 };
 
 struct cache_line {			/**< A cache line for a given hop value */
-	gint hops;				/**< Hop count of this cache line */
+	int hops;				/**< Hop count of this cache line */
 	GSList *pongs;			/**< List of cached_pong */
 	GSList *cursor;			/**< Cursor within list: last item traversed */
 };
@@ -717,7 +803,7 @@ struct recent {
 	GHashTable *ht_recent_pongs;	/**< Recent pongs we know about */
 	GList *recent_pongs;			/**< Recent pongs we got */
 	GList *last_returned_pong;		/**< Last returned from list */
-	gint recent_pong_count;			/**< # of pongs in recent list */
+	int recent_pong_count;			/**< # of pongs in recent list */
 };
 
 #define PONG_CACHE_SIZE		(MAX_CACHE_HOPS+1)
@@ -754,7 +840,7 @@ cached_pong_hash(gconstpointer key)
 	return host_addr_hash(cp->info.addr) ^
 		((cp->info.port << 16) | cp->info.port);
 }
-static gint
+static int
 cached_pong_eq(gconstpointer v1, gconstpointer v2)
 {
 	const struct cached_pong *h1 = v1, *h2 = v2;
@@ -769,8 +855,8 @@ cached_pong_eq(gconstpointer v1, gconstpointer v2)
 void
 pcache_init(void)
 {
-	gint h;
-	gchar *lang = NULL;
+	int h;
+	char *lang = NULL;
 
 	memset(pong_cache, 0, sizeof pong_cache);
 	memset(recent_pongs, 0, sizeof recent_pongs);
@@ -824,10 +910,10 @@ G_STMT_START {											\
 #undef GET_LANG
 
 	if (lang != NULL) {
-		gint len = strlen(lang);
+		int len = strlen(lang);
 
 		if (len > 0) {
-			gint i;
+			int i;
 
 			local_meta.flags |= PONG_META_HAS_LOC;
 
@@ -983,7 +1069,7 @@ add_recent_pong(host_type_t type, struct cached_pong *cp)
 {
 	struct recent *rec;
 
-	g_assert((gint) type >= 0 && type < HOST_MAX);
+	g_assert((int) type >= 0 && type < HOST_MAX);
 
 	rec = &recent_pongs[type];
 
@@ -1043,7 +1129,7 @@ pcache_clear_recent(host_type_t type)
 	GList *l;
 	struct recent *rec;
 
-	g_assert((gint) type >= 0 && type < HOST_MAX);
+	g_assert((int) type >= 0 && type < HOST_MAX);
 
 	rec = &recent_pongs[type];
 
@@ -1089,8 +1175,8 @@ pcache_outgoing_connection(struct gnutella_node *n)
 static void
 pcache_expire(void)
 {
-	gint i;
-	gint entries = 0;
+	int i;
+	int entries = 0;
 
 	for (i = 0; i < PONG_CACHE_SIZE; i++) {
 		struct cache_line *cl = &pong_cache[i];
@@ -1137,15 +1223,19 @@ pcache_close(void)
  * older client if and only if at least OLD_PING_PERIOD seconds have
  * elapsed since our last ping, as determined by `next_ping'.
  */
-static void
-ping_all_neighbours(time_t now)
+void
+ping_all_neighbours(void)
 {
 	const GSList *sl;
 	GSList *may_ping = NULL;
 	GSList *to_ping = NULL;
-	gint ping_cnt = 0;
-	gint selected = 0;
-	gint left;
+	int ping_cnt = 0;
+	int selected = 0;
+	int left;
+	time_t now = tm_time();
+
+	if (GNET_PROPERTY(pcache_debug))
+		g_message("PCACHE attempting to ping all neighbours");
 
 	/*
 	 * Because nowadays the network has a higher outdegree for ultrapeers,
@@ -1198,6 +1288,11 @@ ping_all_neighbours(time_t now)
 		if (!(n->attrs & NODE_A_PONG_CACHING))
 			n->next_ping = time_advance(now, OLD_PING_PERIOD);
 
+		if (GNET_PROPERTY(pcache_debug))
+			g_message("PCACHE pinging \"%s\" %s",
+				(n->attrs & NODE_A_PONG_CACHING) ? "new" : "old",
+				host_addr_port_to_string(n->addr, n->port));
+
 		send_ping(n, GNET_PROPERTY(my_ttl));
 	}
 
@@ -1216,7 +1311,7 @@ pcache_possibly_expired(time_t now)
 		pcache_expire();
 		pcache_expire_time = time_advance(now,
 							cache_lifespan(GNET_PROPERTY(current_peermode)));
-		ping_all_neighbours(now);
+		ping_all_neighbours();
 	}
 }
 
@@ -1246,12 +1341,12 @@ pcache_set_peermode(node_peer_t mode)
 static void
 setup_pong_demultiplexing(struct gnutella_node *n, guint8 ttl)
 {
-	gint remains;
-	gint h;
+	int remains;
+	int h;
 
 	g_assert(gnutella_header_get_function(&n->header) == GTA_MSG_INIT);
 
-	memcpy(n->ping_guid, gnutella_header_get_muid(&n->header), 16);
+	memcpy(&n->ping_guid, gnutella_header_get_muid(&n->header), 16);
 	memset(n->pong_needed, 0, sizeof(n->pong_needed));
 	n->pong_missing = 0;
 
@@ -1302,7 +1397,7 @@ iterate_on_cached_line(
 	struct gnutella_node *n, struct cache_line *cl, guint8 ttl,
 	GSList *start, GSList *end, gboolean strict)
 {
-	gint hops = cl->hops;
+	int hops = cl->hops;
 	GSList *sl;
 
 	sl = start;
@@ -1344,7 +1439,7 @@ iterate_on_cached_line(
 		g_assert(hops < 255);		/* Because of MAX_CACHE_HOPS */
 
 		send_pong(n, FALSE, PING_F_NONE,
-			hops + 1, ttl, n->ping_guid, &cp->info, cp->meta);
+			hops + 1, ttl, &n->ping_guid, &cp->info, cp->meta);
 
 		n->pong_missing--;
 
@@ -1384,7 +1479,7 @@ send_cached_pongs(
 	struct gnutella_node *n,
 	struct cache_line *cl, guint8 ttl, gboolean strict)
 {
-	gint hops = cl->hops;
+	int hops = cl->hops;
 	GSList *old = cl->cursor;
 
 	if (strict && !n->pong_needed[hops])
@@ -1532,7 +1627,7 @@ pong_all_neighbours_but_one(
 		g_assert(hops < 255);
 
 		send_pong(cn, FALSE, PING_F_NONE,
-			hops + 1, ttl, cn->ping_guid, &cp->info, cp->meta);
+			hops + 1, ttl, &cn->ping_guid, &cp->info, cp->meta);
 
 		if (GNET_PROPERTY(pcache_debug) > 7)
 			printf("pong_all: sent cached pong %s (hops=%d, TTL=%d) to %s "
@@ -1550,14 +1645,14 @@ static void
 pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 {
 	const GSList *sl;
-	gint leaves;
+	int leaves;
 	struct gnutella_node *leaf = NULL;
 
 	g_assert(GNET_PROPERTY(current_peermode) == NODE_P_ULTRA);
 
 	for (sl = node_all_nodes(), leaves = 0; sl; sl = g_slist_next(sl)) {
 		struct gnutella_node *cn = sl->data;
-		gint threshold;
+		int threshold;
 
 		if (cn->pong_missing)	/* A job for pong_all_neighbours_but_one() */
 			continue;
@@ -1577,9 +1672,9 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 		 */
 
 		leaves++;
-		threshold = (gint) (1000.0 / leaves);
+		threshold = (int) (1000.0 / leaves);
 
-		if ((gint) random_value(999) < threshold)
+		if ((int) random_value(999) < threshold)
 			leaf = cn;
 	}
 
@@ -1591,7 +1686,7 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 	 */
 
 	if (leaf != NULL) {
-		send_pong(leaf, FALSE, PING_F_NONE, hops + 1, ttl, leaf->ping_guid,
+		send_pong(leaf, FALSE, PING_F_NONE, hops + 1, ttl, &leaf->ping_guid,
 			&cp->info, cp->meta);
 
 		if (GNET_PROPERTY(pcache_debug) > 7)
@@ -1610,7 +1705,7 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 static pong_meta_t *
 pong_extract_metadata(struct gnutella_node *n)
 {
-	gint i;
+	int i;
 	pong_meta_t *meta = NULL;
 
 #define ALLOCATE(f) do {					\
@@ -1727,6 +1822,22 @@ pong_extract_metadata(struct gnutella_node *n)
 		case EXT_T_GGEP_TLS:
 			ALLOCATE(TLS);
 			break;
+		case EXT_T_GGEP_DHT:
+			/*
+			 * Host is part of the DHT.
+			 * Indicates version information and operating flags.
+			 */
+
+			paylen = ext_paylen(e);
+
+			if (paylen >= 3) {
+				payload = ext_payload(e);
+				ALLOCATE(DHT);
+				meta->dht_major = payload[0];
+				meta->dht_minor = payload[1];
+				meta->dht_mode = payload[2];
+			}
+			break;
 		default:
 			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
@@ -1759,7 +1870,7 @@ record_fresh_pong(
 	struct cached_pong *cp;
 	guint8 hop;
 
-	g_assert((gint) type >= 0 && type < HOST_MAX);
+	g_assert((int) type >= 0 && type < HOST_MAX);
 
 	cp = walloc(sizeof *cp);
 
@@ -1964,8 +2075,9 @@ pcache_udp_pong_received(struct gnutella_node *n)
 	host_addr_t ipv4_addr;
 	host_addr_t ipv6_addr;
 	gboolean supports_tls;
+	gboolean supports_dht;
 	guint16 port;
-	gint i;
+	int i;
 
 	g_assert(NODE_IS_UDP(n));
 
@@ -1976,19 +2088,22 @@ pcache_udp_pong_received(struct gnutella_node *n)
 	ipv4_addr = host_addr_peek_ipv4(&n->data[2]);
 	ipv6_addr = zero_host_addr;
 	supports_tls = FALSE;
+	supports_dht = FALSE;
 	
 	/*
 	 * We pretty much ignore pongs we get from UDP, unless they bear
-	 * the GGEP "IPP" extension, containing a packed set of IP:port.
+	 * the GGEP "IPP" or "DHTIPP" extensions, containing a packed set
+	 * of IP:port.
 	 */
 
 	for (i = 0; i < n->extcount; i++) {
 		extvec_t *e = &n->extvec[i];
 		guint16 paylen;
-		const gchar *payload;
+		const char *payload;
 
 		switch (e->ext_token) {
 		case EXT_T_GGEP_IPP:
+		case EXT_T_GGEP_DHTIPP:
 			paylen = ext_paylen(e);
 			payload = ext_payload(e);
 
@@ -1997,7 +2112,16 @@ pcache_udp_pong_received(struct gnutella_node *n)
 					gmsg_infostr(&n->header), ext_ggep_id_str(e),
 					paylen, paylen == 1 ? "" : "s");
 			} else {
-				uhc_ipp_extract(n, payload, paylen); 
+				switch (e->ext_token) {
+				case EXT_T_GGEP_IPP:
+					uhc_ipp_extract(n, payload, paylen); 
+					break;
+				case EXT_T_GGEP_DHTIPP:
+					dht_ipp_extract(n, payload, paylen); 
+					break;
+				default:
+					g_assert_not_reached();
+				}
 			}
 			break;
 		case EXT_T_GGEP_GTKG_IPV6:
@@ -2007,6 +2131,18 @@ pcache_udp_pong_received(struct gnutella_node *n)
 		case EXT_T_GGEP_GTKG_TLS:
 			supports_tls = TRUE;
 			break;
+		case EXT_T_GGEP_DHT:
+			if (ext_paylen(e) >= 3) {
+				guint8 mode;
+
+				payload = ext_payload(e);
+				mode = payload[2];
+
+				/* We want to ping active DHT nodes only */
+				if (mode == DHT_MODE_ACTIVE)
+					supports_dht = TRUE;
+			}
+			break;
 		default:
 			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
@@ -2041,6 +2177,9 @@ pcache_udp_pong_received(struct gnutella_node *n)
 			if (supports_tls) {
 				tls_cache_insert(addr, port);
 			}
+			if (supports_dht) {
+				dht_bootstrap_if_needed(addr, port);
+			}
 		}
 	}
 }
@@ -2238,7 +2377,7 @@ pcache_pong_received(struct gnutella_node *n)
 	 */
 
 	if (!(n->attrs & NODE_A_PONG_CACHING)) {
-		gint ratio = (gint) random_value(100);
+		int ratio = (int) random_value(100);
 		if (ratio >= OLD_CACHE_RATIO) {
 			if (GNET_PROPERTY(pcache_debug) > 7)
 				printf("NOT CACHED pong %s (hops=%d, TTL=%d) from OLD %s\n",
@@ -2294,6 +2433,17 @@ pcache_pong_received(struct gnutella_node *n)
 		pong_random_leaf(cp,
 			CACHE_HOP_IDX(gnutella_header_get_hops(&n->header)),
 			MAX(1, gnutella_header_get_ttl(&n->header)));
+
+	/*
+	 * If host indicates DHT support for non-firewalled node, use that
+	 * address for bootstrapping, if necessary.
+	 */
+
+	if (
+		cp->meta != NULL && (cp->meta->flags & PONG_META_HAS_DHT) &&
+		cp->meta->dht_mode == DHT_MODE_ACTIVE
+	)
+		dht_bootstrap_if_needed(addr, port);
 }
 
 /**
