@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $
+ * $Id: pproxy.c 16275 2009-03-28 17:40:31Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $")
+RCSID("$Id: pproxy.c 16275 2009-03-28 17:40:31Z rmanfredi $")
 
 #include "pproxy.h"
 #include "http.h"
@@ -62,7 +62,7 @@ RCSID("$Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $")
 #include "downloads.h"
 #include "features.h"
 
-#include "lib/misc.h"
+#include "lib/ascii.h"
 #include "lib/atoms.h"
 #include "lib/getline.h"
 #include "lib/header.h"
@@ -82,9 +82,9 @@ RCSID("$Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $")
 static GSList *pproxies = NULL;	/**< Currently active push-proxy requests */
 
 static void send_pproxy_error(struct pproxy *pp, int code,
-	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
+	const char *msg, ...) G_GNUC_PRINTF(3, 4);
 static void pproxy_error_remove(struct pproxy *pp, int code,
-	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
+	const char *msg, ...) G_GNUC_PRINTF(3, 4);
 
 /**
  * Get rid of all the resources attached to the push-proxy struct.
@@ -111,14 +111,14 @@ pproxy_free_resources(struct pproxy *pp)
 static void
 send_pproxy_error_v(
 	struct pproxy *pp,
-	const gchar *ext,
+	const char *ext,
 	int code,
-	const gchar *msg, va_list ap)
+	const char *msg, va_list ap)
 {
-	gchar reason[1024];
-	gchar extra[1024];
+	char reason[1024];
+	char extra[1024];
 	http_extra_desc_t hev[1];
-	gint hevcnt = 0;
+	int hevcnt = 0;
 
 	if (msg) {
 		gm_vsnprintf(reason, sizeof reason, msg, ap);
@@ -150,7 +150,7 @@ send_pproxy_error_v(
 				"ignoring too large extra header (%d bytes)", (int) slen);
 	}
 
-	http_send_status(pp->socket, code, FALSE,
+	http_send_status(HTTP_PUSH_PROXY, pp->socket, code, FALSE,
 			hevcnt ? hev : NULL, hevcnt, "%s", reason);
 
 	pp->error_sent = code;
@@ -161,7 +161,7 @@ send_pproxy_error_v(
  * This can only be done once per connection.
  */
 static void
-send_pproxy_error(struct pproxy *pp, int code, const gchar *msg, ...)
+send_pproxy_error(struct pproxy *pp, int code, const char *msg, ...)
 {
 	va_list args;
 
@@ -174,10 +174,10 @@ send_pproxy_error(struct pproxy *pp, int code, const gchar *msg, ...)
  * The vectorized (message-wise) version of pproxy_remove().
  */
 static void
-pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
+pproxy_remove_v(struct pproxy *pp, const char *reason, va_list ap)
 {
-	const gchar *logreason;
-	gchar errbuf[1024];
+	const char *logreason;
+	char errbuf[1024];
 
 	g_assert(pp != NULL);
 
@@ -220,7 +220,7 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
  * them a 400 error with the reason.
  */
 void
-pproxy_remove(struct pproxy *pp, const gchar *reason, ...)
+pproxy_remove(struct pproxy *pp, const char *reason, ...)
 {
 	va_list args;
 
@@ -235,7 +235,7 @@ pproxy_remove(struct pproxy *pp, const gchar *reason, ...)
  * Utility routine.  Cancel the request, sending back the HTTP error message.
  */
 static void
-pproxy_error_remove(struct pproxy *pp, int code, const gchar *msg, ...)
+pproxy_error_remove(struct pproxy *pp, int code, const char *msg, ...)
 {
 	va_list args, errargs;
 
@@ -312,22 +312,22 @@ pproxy_create(struct gnutella_socket *s)
  * we also return an error to the calling party.
  */
 static gboolean
-get_params(struct pproxy *pp, const gchar *request,
-	const gchar **guid_atom, guint32 *file_idx, gboolean *supports_tls)
+get_params(struct pproxy *pp, const char *request,
+	const struct guid **guid_atom, guint32 *file_idx, gboolean *supports_tls)
 {
 	static const struct {
-		const gchar *req;
-		const gchar *attr;
+		const char *req;
+		const char *attr;
 	} req_types[] = {
 		{ "/gnutella/pushproxy?",	"ServerId" },
 		{ "/gnutella/push-proxy?",	"ServerId" },
 		{ "/gnet/push-proxy?",		"guid" },
 	};
-	gchar *uri;
-	const gchar *attr;
-	gchar *p;
-	const gchar *value;
-	gint datalen;
+	char *uri;
+	const char *attr;
+	char *p;
+	const char *value;
+	int datalen;
 	url_params_t *up;
 	guint i;
 
@@ -374,7 +374,7 @@ get_params(struct pproxy *pp, const gchar *request,
 
 	attr = NULL;
 	for (i = 0; i < G_N_ELEMENTS(req_types); i++) {
-		gchar *q;
+		char *q;
 
 		if (NULL != (q = is_strprefix(uri, req_types[i].req))) {
 			attr = req_types[i].attr;
@@ -413,7 +413,7 @@ get_params(struct pproxy *pp, const gchar *request,
 	datalen = strlen(value);
 
 	if (0 == strcmp(attr, "ServerId")) {
-		gchar *guid;
+		const struct guid *guid;
 
 		/*
 		 * GUID is base32-encoded: valid lengths are 26 (no padding) or 32.
@@ -438,7 +438,7 @@ get_params(struct pproxy *pp, const gchar *request,
 
 		*guid_atom = atom_guid_get(guid);
 	} else if (0 == strcmp(attr, "guid")) {
-		gchar guid[GUID_RAW_SIZE];
+		struct guid guid;
 
 		/*
 		 * GUID in hexadecimal: valid length is 32.
@@ -454,13 +454,13 @@ get_params(struct pproxy *pp, const gchar *request,
 		if (GNET_PROPERTY(push_proxy_debug) > 0)
 			g_message("PUSH-PROXY: decoding %s=%s as hexadecimal", attr, value);
 
-		if (!hex_to_guid(value, guid)) {
+		if (!hex_to_guid(value, &guid)) {
 			pproxy_error_remove(pp, 400, "Malformed push-proxy request: "
 				"parameter \"%s\" is not valid hexadecimal", attr);
 			goto error;
 		}
 
-		*guid_atom = atom_guid_get(guid);
+		*guid_atom = atom_guid_get(&guid);
 	} else {
 		g_error("unhandled parameter \"%s\"", attr);
 	}
@@ -471,7 +471,7 @@ get_params(struct pproxy *pp, const gchar *request,
 
 	value = url_params_get(up, "file");
 	if (value) {
-		gint error;
+		int error;
 
 		/* Ignore errors; parse_uint32() returns 0 on error. */
 		*file_idx = parse_uint32(value, NULL, 10, &error);
@@ -510,15 +510,15 @@ error:
  *			packet on success, an empty array on failure.
  */
 struct array
-build_push(guint8 ttl, guint8 hops, const gchar *guid,
+build_push(guint8 ttl, guint8 hops, const struct guid *guid,
 	host_addr_t addr_v4, host_addr_t addr_v6, guint16 port,
 	guint32 file_idx, gboolean supports_tls)
 {
 	static union {
 		gnutella_msg_push_request_t m;
-		gchar data[1024];
+		char data[1024];
 	} packet;
-	gchar *p = packet.data;
+	char *p = packet.data;
 	size_t len = 0, size = sizeof packet;
 	ggep_stream_t gs;
 
@@ -602,16 +602,16 @@ build_push(guint8 ttl, guint8 hops, const gchar *guid,
  *
  * @return atom, or NULL.
  */
-static const gchar *
-validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
+static const char *
+validate_vendor(char *vendor, char *token, const host_addr_t addr)
 {
-	const gchar *result;
+	const char *result;
 
 	if (vendor) {
 		gboolean faked = !version_check(vendor, token, addr);
 
 		if (faked) {
-			gchar name[1024];
+			char name[1024];
 
 			name[0] = '!';
 			g_strlcpy(&name[1], vendor, sizeof name - 1);
@@ -628,7 +628,7 @@ validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 static void
 pproxy_fetch_addresses(struct pproxy *pp, const char *buf)
 {
-	const gchar *endptr;
+	const char *endptr;
 	host_addr_t addr;
 	guint16 port;
 
@@ -685,19 +685,18 @@ static void
 pproxy_request(struct pproxy *pp, header_t *header)
 {
 	struct gnutella_socket *s = pp->socket;
-	const gchar *request = getline_str(s->getline);
+	const char *request = getline_str(s->getline);
 	struct gnutella_node *n;
-	const gchar *buf;
-	gchar *token;
-	gchar *user_agent;
+	const char *buf;
+	char *token;
+	char *user_agent;
 	GSList *nodes;
 	gboolean supports_tls = FALSE;
 
-	if (GNET_PROPERTY(push_proxy_debug) > 0) {
+	if (GNET_PROPERTY(push_proxy_trace) & SOCK_TRACE_IN) {
 		g_message("----Push-proxy request from %s:\n%s",
 			host_addr_to_string(s->addr), request);
-		header_dump(header, stderr);
-		g_message("----");
+		header_dump(stderr, header, "----");
 	}
 
 	/*
@@ -766,6 +765,19 @@ pproxy_request(struct pproxy *pp, header_t *header)
 
 	n = route_proxy_find(pp->guid);
 
+	/*
+	 * Even if they did no ask for us to be their proxy, locate nodes to
+	 * which we are connected to see if haven't learned of their GUID
+	 * by looking at the query hits they sent out: see update_neighbour_info().
+	 *		--RAM, 2009-03-14
+	 */
+
+	if (NULL == n) {
+		n = node_by_guid(pp->guid);
+		if (n != NULL)
+			gnet_stats_count_general(GNR_PUSH_PROXY_ROUTE_NOT_PROXIED, 1);
+	}
+
 	if (n != NULL) {
 		struct array packet;
 
@@ -785,9 +797,9 @@ pproxy_request(struct pproxy *pp, header_t *header)
 				(gulong) pp->file_idx);
 		} else {
 			gmsg_sendto_one(n, packet.data, packet.size);
-			gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
+			gnet_stats_count_general(GNR_PUSH_PROXY_TCP_RELAYED, 1);
 
-			http_send_status(pp->socket, 202, FALSE, NULL, 0,
+			http_send_status(HTTP_PUSH_PROXY, pp->socket, 202, FALSE, NULL, 0,
 					"Push-proxy: message sent to node");
 
 			pp->error_sent = 202;
@@ -820,14 +832,14 @@ pproxy_request(struct pproxy *pp, header_t *header)
 				host_addr_port_to_string2(pp->addr_v6, pp->port),
 				(gulong) pp->file_idx);
 		} else {
-			gint cnt;
+			int cnt;
 
 			gmsg_sendto_all(nodes, packet.data, packet.size);
 			gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
 
 			cnt = g_slist_length(nodes);
 
-			http_send_status(pp->socket, 203, FALSE, NULL, 0,
+			http_send_status(HTTP_PUSH_PROXY, pp->socket, 203, FALSE, NULL, 0,
 					"Push-proxy: message sent through Gnutella (via %d node%s)",
 					cnt, cnt == 1 ? "" : "s");
 
@@ -850,7 +862,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 		upload_send_giv(pp->addr_v4, pp->port, 0, 1, 0,
 			"<from push-proxy>", FALSE, pp->flags);
 
-		http_send_status(pp->socket, 202, FALSE, NULL, 0,
+		http_send_status(HTTP_PUSH_PROXY, pp->socket, 202, FALSE, NULL, 0,
 			"Push-proxy: you found the target GUID %s",
 			guid_hex_str(pp->guid));
 
@@ -881,26 +893,28 @@ PPROXY(gpointer obj)
 }
 
 static void
-err_line_too_long(gpointer obj)
+err_line_too_long(gpointer obj, header_t *unused_head)
 {
+	(void) unused_head;
 	pproxy_error_remove(PPROXY(obj), 413, "Header too large");
 }
 
 static void
-err_header_error_tell(gpointer obj, gint error)
+err_header_error_tell(gpointer obj, int error)
 {
 	send_pproxy_error(PPROXY(obj), 413, "%s", header_strerror(error));
 }
 
 static void
-err_header_error(gpointer obj, gint error)
+err_header_error(gpointer obj, int error)
 {
 	pproxy_remove(PPROXY(obj), "Failed (%s)", header_strerror(error));
 }
 
 static void
-err_input_exception(gpointer obj)
+err_input_exception(gpointer obj, header_t *unused_head)
 {
+	(void) unused_head;
 	pproxy_remove(PPROXY(obj), "Failed (Input Exception)");
 }
 
@@ -911,20 +925,22 @@ err_input_buffer_full(gpointer obj)
 }
 
 static void
-err_header_read_error(gpointer obj, gint error)
+err_header_read_error(gpointer obj, int error)
 {
 	pproxy_remove(PPROXY(obj), "Failed (Input error: %s)", g_strerror(error));
 }
 
 static void
-err_header_read_eof(gpointer obj)
+err_header_read_eof(gpointer obj, header_t *unused_head)
 {
+	(void) unused_head;
 	pproxy_remove(PPROXY(obj), "Failed (EOF)");
 }
 
 static void
-err_header_extra_data(gpointer obj)
+err_header_extra_data(gpointer obj, header_t *unused_head)
 {
+	(void) unused_head;
 	pproxy_error_remove(PPROXY(obj), 400, "Extra data after HTTP header");
 }
 
@@ -1041,12 +1057,12 @@ cproxy_http_error_ind(struct http_async *handle,
  */
 static gboolean
 cproxy_http_header_ind(struct http_async *handle, header_t *header,
-	gint code, const gchar *message)
+	int code, const char *message)
 {
 	struct cproxy *cp = http_async_get_opaque(handle);
-	gchar *token;
-	gchar *server;
-	gchar *to_free;
+	char *token;
+	char *server;
+	char *to_free;
 
 	g_assert(cp != NULL);
 	g_assert(cp->d != NULL);
@@ -1131,18 +1147,14 @@ cproxy_http_header_ind(struct http_async *handle, header_t *header,
  * @return length of generated request.
  */
 static size_t
-cproxy_build_request(struct http_async *unused_handle, gchar *buf, size_t len,
-	const gchar *verb, const gchar *path, const gchar *unused_host,
-	guint16 unused_port)
+cproxy_build_request(const struct http_async *ha,
+	char *buf, size_t len, const char *verb, const char *path)
 {
-	gchar addr_v4_buf[128];
-	gchar addr_v6_buf[128];
+	char addr_v4_buf[128];
+	char addr_v6_buf[128];
 	host_addr_t addr;
 	gboolean has_ipv4 = FALSE;
 
-	(void) unused_handle;
-	(void) unused_host;
-	(void) unused_port;
 	g_assert(len <= INT_MAX);
 
 	addr = listen_addr();
@@ -1174,17 +1186,64 @@ cproxy_build_request(struct http_async *unused_handle, gchar *buf, size_t len,
 		"%s %s HTTP/1.1\r\n"
 		"User-Agent: %s\r\n"
 		"Connection: close\r\n"
-		"Host:\r\n"
+		"Host: %s\r\n"
 		"X-Token: %s\r\n"
 		"%s"
 		"%s"
 		"\r\n",
 		verb, path, version_string,
+		http_async_remote_host_port(ha),
 		tok_version(),
 		addr_v4_buf,
 		addr_v6_buf);
 }
 
+/**
+ * Redefine callback invoked when the HTTP request has been sent.
+ *
+ * @param unused_ha		the (unused) HTTP async request descriptor
+ * @param s				the socket on which we wrote the request
+ * @param req			the actual request string
+ * @param len			the length of the request string
+ * @param deferred		if TRUE, full request sending was deferred earlier
+ */
+static void
+cproxy_sent_request(const struct http_async *unused_ha,
+	const struct gnutella_socket *s, const char *req, size_t len,
+	gboolean deferred)
+{
+	(void) unused_ha;
+
+	if (GNET_PROPERTY(push_proxy_trace) & SOCK_TRACE_OUT) {
+		g_message("----Sent push-proxy request%s to %s (%u bytes):",
+			deferred ? " completely" : "",
+			host_addr_port_to_string(s->addr, s->port), (unsigned) len);
+		dump_string(stderr, req, len, "----");
+	}
+}
+
+/**
+ * Redefine callback invoked when we got the whole HTTP reply.
+ *
+ * @param unused_ha		the (unused) HTTP async request descriptor
+ * @param s				the socket on which we got the reply
+ * @param status		the first HTTP status line
+ * @param header		the parsed header structure
+ */
+static void
+cproxy_got_reply(const struct http_async *unused_ha,
+	const struct gnutella_socket *s, const char *status, const header_t *header)
+{
+	(void) unused_ha;
+
+	if (GNET_PROPERTY(push_proxy_trace) & SOCK_TRACE_IN) {
+		g_message("----Got push-proxy reply from %s:",
+			host_addr_to_string(s->addr));
+		fprintf(stderr, "%s", status);
+		header_dump(stderr, header, "----");
+	}
+}
+
 /**
  * Invoked when the state of the HTTP async request changes.
  */
@@ -1208,11 +1267,11 @@ cproxy_http_newstate(struct http_async *handle, http_state_t newstate)
  */
 struct cproxy *
 cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
-	const gchar *guid, guint32 file_idx)
+	const struct guid *guid, guint32 file_idx)
 {
 	struct http_async *handle;
 	struct cproxy *cp;
-	gchar path[128];
+	char path[128];
 
 	concat_strings(path, sizeof path,
 		"/gnutella/push-proxy?ServerId=", guid_base32_str(guid),
@@ -1250,6 +1309,8 @@ cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
 
 	http_async_set_opaque(handle, cp, NULL);
 	http_async_set_op_request(handle, cproxy_build_request);
+	http_async_set_op_reqsent(handle, cproxy_sent_request);
+	http_async_set_op_gotreply(handle, cproxy_got_reply);
 	http_async_on_state_change(handle, cproxy_http_newstate);
 
 	return cp;
