@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $
+ * $Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $")
+RCSID("$Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $")
 
 #include "bsched.h"
 #include "dmesh.h"		/* For dmesh_fill_alternate() */
@@ -47,7 +47,7 @@ RCSID("$Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $")
 #include "qhit.h"
 #include "settings.h"	/* For listen_ip() */
 #include "share.h"
-#include "tls_cache.h"
+#include "ipp_cache.h"
 #include "uploads.h"	/* For count_uploads */
 
 #include "if/gnet_property_priv.h"
@@ -60,11 +60,15 @@ RCSID("$Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $")
 #include "lib/tm.h"
 #include "lib/override.h"			/* Must be the last header included */
 
-#define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
-#define QHIT_MAX_RESULTS	255		/**< Maximum amount of hits in a query hit */
+/*
+ * NOTE: LimeWire doesn't like more than 10 results per QHIT packet or
+ *       more than 10 alt-locs per QHIT packet.
+ */
+#define QHIT_MAX_RESULTS	10		/**< Maximum amount of hits in a query hit */
 #define QHIT_MAX_ALT		10		/**< Send out 10 alt-locs per entry, max */
 #define QHIT_MAX_PROXIES	5		/**< Send out 5 push-proxies at most */
 #define QHIT_MAX_GGEP		512		/**< Allocated room for trailing GGEP */
+#define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
 
 /*
  * Minimal trailer length is our code NAME, the open flags, and the GUID.
@@ -81,11 +85,11 @@ RCSID("$Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $")
  */
 
 struct found_struct {
-	gchar data[64 * 1024];		/**< data */
+	char data[64 * 1024];		/**< data */
 	size_t pos;					/**< current write position */
 	size_t files;				/**< amount of file entries */
 	size_t max_size;			/**< max query hit size */
-	const gchar *muid;			/**< the MUID to put in all query hits */
+	const struct guid *muid;	/**< the MUID to put in all query hits */
 	const struct array *token;	/**< Optional secure OOB token */
 	qhit_process_t process;		/**< processor once query hit is built */
 	gpointer udata;				/**< processor argument */
@@ -124,7 +128,7 @@ found_ggep_h(void)
 	return found_get()->use_ggep_h;
 }
 
-static gchar *
+static char *
 found_open(void)
 {
 	struct found_struct *f = found_get();
@@ -234,7 +238,8 @@ found_set_header(void)
 			connect_speed = 0;
 		}
 	}
-	connect_speed /= MAX(1, GNET_PROPERTY(max_uploads));	/* Upload speed expected per slot */
+	/* Upload speed expected per slot */
+	connect_speed /= MAX(1, GNET_PROPERTY(max_uploads));
 
 	gnutella_msg_search_results_set_host_port(msg, socket_listen_port());
 	gnutella_msg_search_results_set_host_ip(msg, host_addr_ipv4(listen_addr()));
@@ -269,7 +274,7 @@ found_token(void)
 }
 
 static void
-found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
+found_init(size_t max_size, const struct guid *xuid, gboolean ggep_h,
 	qhit_process_t proc, gpointer udata, const struct array *token)
 {
 	struct found_struct *f = found_get();
@@ -338,10 +343,10 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 static void
 flush_match(void)
 {
-	gchar trailer[7];
-	gint ggep_len = 0;			/* Size of the GGEP trailer */
+	char trailer[7];
+	int ggep_len = 0;			/* Size of the GGEP trailer */
 	ggep_stream_t gs;
-	gchar *trailer_start;
+	char *trailer_start;
 
 	/*
 	 * Build gtk-gnutella trailer.
@@ -388,7 +393,7 @@ flush_match(void)
 	{
 		guint8 major = GTA_VERSION;
 		guint8 minor = GTA_SUBVERSION;
-		const gchar *revp = GTA_REVCHAR;
+		const char *revp = GTA_REVCHAR;
 		guint8 revchar = (guint8) revp[0];
 		guint8 patch;
 		guint32 release;
@@ -450,7 +455,7 @@ flush_match(void)
 
 			for (iter = nodes; ok && NULL != iter; iter = g_slist_next(iter)) {
 				const struct gnutella_node *n = iter->data;
-				gchar proxy[6];
+				char proxy[6];
 
 				if (NET_TYPE_IPV4 != host_addr_net(n->proxy_addr))
 					continue;
@@ -489,6 +494,7 @@ flush_match(void)
 
 	if (
 		!GNET_PROPERTY(is_firewalled) &&
+		GNET_PROPERTY(give_server_hostname) &&
 		'\0' != GNET_PROPERTY(server_hostname)[0]
 	) {
 		gboolean ok;
@@ -567,9 +573,9 @@ add_file(const struct shared_file *sf)
 {
 	gboolean sha1_available;
 	gnet_host_t hvec[QHIT_MAX_ALT];
-	gint hcnt = 0;
+	int hcnt = 0;
 	guint32 fs32, fs32_le, idx_le;
-	gint ggep_len;
+	int ggep_len;
 	gboolean ok;
 	ggep_stream_t gs;
 	size_t left, needed;
@@ -648,19 +654,12 @@ add_file(const struct shared_file *sf)
 
 	if (sha1_available && !found_ggep_h()) {
 		const struct sha1 * const sha1 = shared_file_sha1(sf);
-		const struct tth * const tth = shared_file_tth(sf);
 
 		/* Good old way: ASCII URN */
 		if (!found_write(sha1_to_urn_string(sha1), SHA1_URN_LENGTH))
 			return FALSE;
 		if (!found_write("\x1c", 1))
 			return FALSE;
-		if (tth) {
-			if (!found_write(tth_to_urn_string(tth), TTH_URN_LENGTH))
-				return FALSE;
-			if (!found_write("\x1c", 1))
-				return FALSE;
-		}
 	}
 
 	/*
@@ -672,8 +671,8 @@ add_file(const struct shared_file *sf)
 	ggep_stream_init(&gs, start, left);
 
 	/*
-	 * Emit the SHA1 as GGEP "H" if they said they understand it.
-	 * Modern way: GGEP "H" for binary URN
+	 * Emit the SHA1 as GGEP "H" if they said they understand it. The modern
+	 * way is GGEP "H" for binary URN but only gtk-gnutella implements it.
 	 */
 
 	if (sha1_available && found_ggep_h()) {
@@ -692,18 +691,34 @@ add_file(const struct shared_file *sf)
 			g_warning("could not write GGEP \"H\" extension in query hit");
 	}
 
+	/*
+	 * First LimeWire emitted TTHs as plain text urn:ttroot:<base32 TTH>.
+	 * Now they are still unaware of GGEP "H" but emit GGEP "TT" with the
+	 * hash in binary form.
+	 */
+	if (sha1_available && !found_ggep_h()) {
+		const struct tth * const tth = shared_file_tth(sf);
+
+		if (tth) {
+			ok = ggep_stream_pack(&gs,
+						GGEP_NAME(TT), tth->data, TTH_RAW_SIZE, GGEP_W_COBS);
+			if (!ok)
+				g_warning("could not write GGEP \"TT\" extension in query hit");
+		}
+	}
+
 	/*
 	 * If the 32-bit size is the magic ~0 escape value, we need to emit
 	 * the real size in the "LF" extension.
 	 */
 
 	if (fs32 == ~0U) {
-		gchar buf[sizeof(guint64)];
-		gint len;
+		char buf[sizeof(guint64)];
+		int len;
 
-		len = ggept_lf_encode(shared_file_size(sf), buf);
+		len = ggept_filesize_encode(shared_file_size(sf), buf);
 
-		g_assert(len > 0 && len <= (gint) sizeof buf);
+		g_assert(len > 0 && len <= (int) sizeof buf);
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(LF), buf, len, GGEP_W_COBS);
 		if (!ok)
@@ -718,7 +733,7 @@ add_file(const struct shared_file *sf)
 	if (hcnt > 0) {
 		guchar tls_bytes[(G_N_ELEMENTS(hvec) + 7) / 8];
 		guint tls_index, tls_length;
-		gint i;
+		int i;
 
 		g_assert(hcnt <= QHIT_MAX_ALT);
 		memset(tls_bytes, 0, sizeof tls_bytes);
@@ -730,7 +745,7 @@ add_file(const struct shared_file *sf)
 		for (i = 0; ok && i < hcnt; i++) {
 			host_addr_t addr;
 			guint16 port;
-			gchar alt[6];
+			char alt[6];
 
 			g_assert(start == gs.outbuf);
 			if (NET_TYPE_IPV4 != gnet_host_get_net(&hvec[i]))
@@ -763,7 +778,7 @@ add_file(const struct shared_file *sf)
 	}
 
 	{
-		const gchar *rp = shared_file_relative_path(sf);
+		const char *rp = shared_file_relative_path(sf);
 		
 		if (rp) {
 			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, strlen(rp), 0);
@@ -778,8 +793,8 @@ add_file(const struct shared_file *sf)
 		/* FIXME: Create time != modification time */
 		mtime = shared_file_modification_time(sf);
 		if ((time_t) -1 != mtime) {
-			gchar buf[sizeof(guint64)];
-			gint len;
+			char buf[sizeof(guint64)];
+			int len;
 
 			/*
 			 * Suppress negative values (if time_t is signed) as this would
@@ -788,7 +803,7 @@ add_file(const struct shared_file *sf)
 			mtime = MAX(0, mtime);
 
 			len = ggept_ct_encode(mtime, buf);
-			g_assert(len >= 0 && len <= (gint) sizeof buf);
+			g_assert(len >= 0 && len <= (int) sizeof buf);
 
 			ok = ggep_stream_pack(&gs, GGEP_NAME(CT), buf, len, GGEP_W_COBS);
 			if (!ok)
@@ -844,7 +859,7 @@ add_file(const struct shared_file *sf)
  * have to be sent out-of-bound
  */
 static void
-found_reset(size_t max_size, const gchar *muid, gboolean ggep_h,
+found_reset(size_t max_size, const struct guid *muid, gboolean ggep_h,
 	qhit_process_t process, gpointer udata, const struct array *token)
 {
 	g_assert(process != NULL);
@@ -863,11 +878,11 @@ found_reset(size_t max_size, const gchar *muid, gboolean ggep_h,
  * @param muid			the query's MUID
  */
 void
-qhit_send_results(struct gnutella_node *n, GSList *files, gint count,
-	const gchar *muid, gboolean ggep_h)
+qhit_send_results(struct gnutella_node *n, GSList *files, int count,
+	const struct guid *muid, gboolean ggep_h)
 {
 	GSList *sl;
-	gint sent = 0;
+	int sent = 0;
 
 	/*
 	 * We can't use n->header.muid as the query's MUID but must rely on the
@@ -919,12 +934,12 @@ qhit_send_results(struct gnutella_node *n, GSList *files, gint count,
  * @param muid			the MUID to use on each generated hit
  */
 void
-qhit_build_results(const GSList *files, gint count, size_t max_msgsize,
-	qhit_process_t cb, gpointer udata, const gchar *muid, gboolean ggep_h,
+qhit_build_results(const GSList *files, int count, size_t max_msgsize,
+	qhit_process_t cb, gpointer udata, const struct guid *muid, gboolean ggep_h,
 	const struct array *token)
 {
 	const GSList *sl;
-	gint sent;
+	int sent;
 
 	g_assert(cb != NULL);
 	g_assert(token);
