@@ -1,5 +1,5 @@
 /*
- * $Id: qrp.h 14022 2007-07-03 01:07:38Z cbiere $
+ * $Id: qrp.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,8 @@
 
 #include "common.h"
 
-#define QRP_MIN_WORD_LENGTH		3	/**< Minimal word length */
+#define QRP_MIN_WORD_LENGTH		3	/**< Minimum word length */
+#define QRP_MAX_CUT_CHARS		5	/**< Maximum truncations  */
 #define QRP_HVEC_MAX	128 	/**< Max: 128 unique words / URNs! */
 
 /*
@@ -52,10 +53,10 @@ enum query_hsrc {
 };
 
 typedef struct qrt_info {
-	gint slots;				/**< Amount of slots */
-	gint generation;		/**< Generation number */
-	gint fill_ratio;		/**< Percentage of slots used */
-	gint pass_throw;		/**< Passing throw, on a d100 */
+	int slots;				/**< Amount of slots */
+	int generation;		/**< Generation number */
+	int fill_ratio;		/**< Percentage of slots used */
+	int pass_throw;		/**< Passing throw, on a d100 */
 } qrt_info_t;
 
 
@@ -100,13 +101,13 @@ struct query_hashvec *qhvec_alloc(guint size);
 void qhvec_free(struct query_hashvec *qhvec);
 void qhvec_reset(struct query_hashvec *qhvec);
 query_hashvec_t * qhvec_clone(const query_hashvec_t *qsrc);
-void qhvec_add(struct query_hashvec *qhvec, const gchar *word,
+void qhvec_add(struct query_hashvec *qhvec, const char *word,
 	enum query_hsrc src);
 gboolean qhvec_has_urn(const struct query_hashvec *qhv);
 guint qhvec_count(const struct query_hashvec *qhv);
 
 GSList *qrt_build_query_target(
-	query_hashvec_t *qhvec, gint hops, gint ttl, struct gnutella_node *source);
+	query_hashvec_t *qhvec, int hops, int ttl, struct gnutella_node *source);
 void qrt_route_query(struct gnutella_node *n, query_hashvec_t *qhvec);
 gboolean qrp_node_can_route(const struct gnutella_node *n,
 			const query_hashvec_t *qhv);
