@@ -1,5 +1,5 @@
 /*
- * $Id: routing.c 15033 2007-11-30 19:41:48Z cbiere $
+ * $Id: routing.c 16179 2009-03-16 18:16:53Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: routing.c 15033 2007-11-30 19:41:48Z cbiere $")
+RCSID("$Id: routing.c 16179 2009-03-16 18:16:53Z rmanfredi $")
 
 #include "search.h"			/* For search_passive. */
 #include "routing.h"
@@ -71,7 +71,7 @@ static struct gnutella_node *fake_node;		/**< Our fake node */
  * their liftime.
  */
 struct message {
-	gchar muid[GUID_RAW_SIZE];	/**< Message UID */
+	struct guid muid;			/**< Message UID */
 	struct message **slot;		/**< Place where we're referenced from */
 	GSList *routes;	            /**< route_data from where the message came */
 	GSList *ttls;				/**< For broadcasted messages: TTL by route */
@@ -97,7 +97,7 @@ struct route_data {
 
 static struct route_data fake_route;		/**< Our fake route_data */
 
-static const gchar *debug_msg[256];
+static const char *debug_msg[256];
 
 /*
  * We're using the message table to store Query hit routes for Push requests,
@@ -134,15 +134,15 @@ static const gchar *debug_msg[256];
 
 static struct {
 	struct message **chunks[MAX_CHUNKS];
-	gint next_idx;				 /**< Next slot to use in "message_array[]" */
-	gint capacity;				 /**< Capacity in terms of messages */
-	gint count;					 /**< Amount really stored */
+	int next_idx;				 /**< Next slot to use in "message_array[]" */
+	int capacity;				 /**< Capacity in terms of messages */
+	int count;					 /**< Amount really stored */
 	GHashTable *messages_hashed; /**< All messages (key = struct message) */
 	time_t last_rotation;		 /**< Last time we restarted from idx=0 */
 } routing;
 
 static gboolean find_message(
-	const gchar *muid, guint8 function, struct message **m);
+	const struct guid *muid, guint8 function, struct message **m);
 
 /**
  * "banned" GUIDs for push routing.
@@ -150,7 +150,7 @@ static gboolean find_message(
  * The following GUIDs are so common that it does not make sense to
  * route pushes to them (i.e. they are are NOT unique on the network!).
  */
-static const gchar * const banned_push[] = {
+static const char * const banned_push[] = {
 	"20d262ff0e6fd6119734004005a207b1",		/**< Morpheus, 29/06/2002 */
 	"9c51e42153d4c94a858f8e8a8391173d",		/**< morph471 4.7.1.326 */
 	"27630b632f070ca9ffc48eb06a72c700",		/**< Morpheus?, 2005-08-30 */
@@ -172,21 +172,21 @@ static GHashTable *ht_proxyfied = NULL;
 struct route_log {
 	host_addr_t addr;			/**< Sender's IP */
 	guint16 port;				/**< Sender's port */
-	gchar muid[GUID_RAW_SIZE];	/**< Message ID */
+	struct guid muid;			/**< Message ID */
 	guint8 function;			/**< Message function */
 	guint8 hops;				/**< Message hops */
 	guint8 ttl;					/**< Message ttl */
 	gboolean handle;			/**< Whether message will be handled */
 	gboolean local;				/**< Whether message originated locally */
 	gboolean new;				/**< Whether message is a new message */
-	gchar extra[80];			/**< Extra text for logging */
+	char extra[80];			/**< Extra text for logging */
 	struct route_dest dest;		/**< Message destination */
 };
 
 static void free_route_list(struct message *m);
 
 static inline gboolean
-is_banned_push(const gchar *guid)
+is_banned_push(const struct guid *guid)
 {
 	return NULL != g_hash_table_lookup(ht_banned_push, guid);
 }
@@ -197,7 +197,7 @@ is_banned_push(const gchar *guid)
 static void
 routing_log_init(struct route_log *route_log,
 	struct gnutella_node *n,
-	const gchar *muid, guint8 function, guint8 hops, guint8 ttl)
+	const struct guid *muid, guint8 function, guint8 hops, guint8 ttl)
 {
 	if (GNET_PROPERTY(routing_debug) <= 7)
 		return;
@@ -215,7 +215,7 @@ routing_log_init(struct route_log *route_log,
 	route_log->function = function;
 	route_log->hops = hops;
 	route_log->ttl = ttl;
-	memcpy(route_log->muid, muid, GUID_RAW_SIZE);
+	route_log->muid = *muid;
 
 	route_log->extra[0] = '\0';
 	route_log->handle = FALSE;
@@ -250,18 +250,18 @@ routing_log_set_new(struct route_log *route_log)
 }
 
 static void routing_log_extra(struct route_log *route_log,
-	const gchar *fmt, ...) G_GNUC_PRINTF(2, 3);
+	const char *fmt, ...) G_GNUC_PRINTF(2, 3);
 
 /**
  * Record extra logging information, appending to existing information.
  */
 static void
-routing_log_extra(struct route_log *route_log, const gchar *fmt, ...)
+routing_log_extra(struct route_log *route_log, const char *fmt, ...)
 {
 	va_list args;
-	gchar *buf;
-	gint buflen;
-	gint len;
+	char *buf;
+	int buflen;
+	int len;
 
 	if (GNET_PROPERTY(routing_debug) <= 7)
 		return;
@@ -280,7 +280,7 @@ routing_log_extra(struct route_log *route_log, const gchar *fmt, ...)
 		buf += len;
 
 		if (buflen > 2) {
-			gint seplen = gm_snprintf(buf, buflen, "; ");
+			int seplen = gm_snprintf(buf, buflen, "; ");
 
 			buflen -= seplen;
 			buf += seplen;
@@ -298,10 +298,10 @@ routing_log_extra(struct route_log *route_log, const gchar *fmt, ...)
 /**
  * @return string representation of message route, as pointer to static data.
  */
-static gchar *
+static char *
 route_string(struct route_dest *dest, const host_addr_t origin_addr)
 {
-	static gchar msg[80];
+	static char msg[80];
 
 	switch (dest->type) {
 	case ROUTE_NONE:
@@ -314,13 +314,9 @@ route_string(struct route_dest *dest, const host_addr_t origin_addr)
 		gm_snprintf(msg, sizeof msg, "all but %s",
 			host_addr_to_string(origin_addr));
 		break;
-	case ROUTE_NO_DUPS_BUT_ONE:
-		gm_snprintf(msg, sizeof msg, "all but %s and broken GTKG",
-			host_addr_to_string(origin_addr));
-		break;
 	case ROUTE_MULTI:
 		{
-			gint count = g_slist_length(dest->ur.u_nodes);
+			int count = g_slist_length(dest->ur.u_nodes);
 			gm_snprintf(msg, sizeof msg, "selected %u node%s",
 				count, count == 1 ? "" : "s");
 		}
@@ -346,7 +342,7 @@ routing_log_flush(struct route_log *route_log)
 		"ROUTE %-21s %s %s %3d/%3d: [%c] %s%s%s-> %s",
 		route_log->local ? "OURSELVES"
 			: host_addr_port_to_string(route_log->addr, route_log->port),
-		debug_msg[route_log->function], guid_hex_str(route_log->muid),
+		debug_msg[route_log->function], guid_hex_str(&route_log->muid),
 		route_log->hops, route_log->ttl, route_log->handle ? 'H' : ' ',
 		route_log->new ? "[NEW] " : "",
 		route_log->extra, route_log->extra[0] == '\0' ? "" : " ",
@@ -359,15 +355,15 @@ routing_log_flush(struct route_log *route_log)
  *
  * @return copy of mangled MUID as pointer to static data.
  */
-static const gchar *
-route_mangled_oob_muid(const gchar *muid)
+static const struct guid *
+route_mangled_oob_muid(const struct guid *muid)
 {
-	static gchar mangled[GUID_RAW_SIZE];
+	static struct guid mangled;
 
-	memcpy(&mangled[4], &muid[4], 9);	/* Clear IP address (bytes 0-3) */
-	mangled[15] = muid[15];				/* Clear port (bytes 13-14) */
+	memcpy(&mangled.v[4], &muid->v[4], 9);	/* Clear IP address (bytes 0-3) */
+	mangled.v[15] = muid->v[15];			/* Clear port (bytes 13-14) */
 
-	return mangled;
+	return &mangled;
 }
 
 /**
@@ -467,13 +463,13 @@ get_next_slot(void)
 	idx = routing.next_idx;
 	chunk_idx = CHUNK_INDEX(idx);
 
-	g_assert((gint) chunk_idx >= 0 && chunk_idx < MAX_CHUNKS);
+	g_assert((int) chunk_idx >= 0 && chunk_idx < MAX_CHUNKS);
 
 	chunk = routing.chunks[chunk_idx];
 
 	if (chunk == NULL) {
 
-		g_assert((gint) idx >= routing.capacity);
+		g_assert((int) idx >= routing.capacity);
 
 		/*
 		 * Chunk does not exist yet, determine whether we should create
@@ -483,7 +479,7 @@ get_next_slot(void)
 		if (idx > 0 && elapsed > TABLE_MIN_CYCLE) {
 			if (GNET_PROPERTY(routing_debug))
 				printf("RT cycling over table, elapsed=%d, holds %d / %d\n",
-					(gint) elapsed, routing.count, routing.capacity);
+					(int) elapsed, routing.count, routing.capacity);
 
 			chunk_idx = 0;
 			idx = routing.next_idx = 0;
@@ -518,7 +514,7 @@ get_next_slot(void)
 		if (idx == 0) {
 			if (GNET_PROPERTY(routing_debug))
 				printf("RT cycling over FORCED, elapsed=%d, holds %d / %d\n",
-					(gint) elapsed, routing.count, routing.capacity);
+					(int) elapsed, routing.count, routing.capacity);
 
 			routing.last_rotation = now;
 		}
@@ -526,7 +522,7 @@ get_next_slot(void)
 
 	g_assert(slot != NULL);
 	g_assert(idx == (guint) routing.next_idx);
-	g_assert((gint) idx >= 0 && idx < (guint) routing.capacity);
+	g_assert((int) idx >= 0 && idx < (guint) routing.capacity);
 
 	/*
 	 * It's OK to go beyond the last allocated chunk (a new chunk will
@@ -647,7 +643,7 @@ static gboolean
 node_ttl_higher(struct gnutella_node *n, struct message *m, guint8 ttl)
 {
 	GSList *l;
-	gint i;
+	int i;
 	struct route_data *route;
 
 	g_assert(n != fake_node);
@@ -681,13 +677,12 @@ node_ttl_higher(struct gnutella_node *n, struct message *m, guint8 ttl)
 /**
  * compares two message structures
  */
-static gint
+static int
 message_compare_func(gconstpointer p, gconstpointer q)
 {
 	const struct message *a = p, *b = q;
 
-	return a->function == b->function &&
-		0 == memcmp(a->muid, b->muid, GUID_RAW_SIZE);
+	return a->function == b->function && guid_eq(&a->muid, &b->muid);
 }
 
 /**
@@ -701,18 +696,27 @@ message_hash_func(gconstpointer key)
 
 	hash = msg->function;
 	for (i = 0; i < 4; i++) {
-		hash ^= peek_le32(&msg->muid[i * 4]);
+		hash ^= peek_le32(&msg->muid.v[i * 4]);
 	}
 	return hash;
 }
 
+/**
+ * Reset this node's GUID.
+ */
+void
+gnet_reset_guid(void)
+{
+	gnet_prop_set_storage(PROP_SERVENT_GUID, &blank_guid, sizeof blank_guid);
+}
+
 /**
  * Init function.
  */
 void
 routing_init(void)
 {
-    gchar guid_buf[GUID_RAW_SIZE];
+    struct guid guid_buf;
 	guint32 i;
 
 	/*
@@ -730,33 +734,41 @@ routing_init(void)
 	ht_banned_push = g_hash_table_new(guid_hash, guid_eq);
 
 	for (i = 0; i < G_N_ELEMENTS(banned_push); i++) {
-		gchar g[GUID_RAW_SIZE];
-		const gchar *hex = banned_push[i];
+		struct guid guid;
+		const char *hex = banned_push[i];
 
-		g_assert(strlen(hex) == 2 * sizeof(g));
+		g_assert(strlen(hex) == 2 * sizeof guid);
 
-		(void) hex_to_guid(hex, g);
+		(void) hex_to_guid(hex, &guid);
 		gm_hash_table_insert_const(ht_banned_push,
-			atom_guid_get(g), GUINT_TO_POINTER(1));
+			atom_guid_get(&guid), GUINT_TO_POINTER(1));
 	}
 
-	/* The Servent GUID is not persistent. A new is generated each session. */
-	do {
-		guid_random_muid(guid_buf);
-		/*
-		 * If by extraordinary, we have generated a banned GUID, retry.
-		 */
-	} while (is_banned_push(guid_buf));
+	/*
+	 * If they did not configure a sticky GUID, or if the GUID ia blank,
+	 * configure a new one.
+	 */
 
-    gnet_prop_set_storage(PROP_SERVENT_GUID, guid_buf, sizeof(guid_buf));
-	g_assert(guid_is_gtkg(guid_buf, NULL, NULL, NULL));
+	gnet_prop_get_storage(PROP_SERVENT_GUID, &guid_buf, sizeof guid_buf);
+
+	if (guid_is_blank(&guid_buf) || !GNET_PROPERTY(sticky_guid)) {
+		do {
+			guid_random_muid(&guid_buf);
+			/*
+			 * If by extraordinary, we have generated a banned GUID, retry.
+			 */
+		} while (is_banned_push(&guid_buf));
+
+		gnet_prop_set_storage(PROP_SERVENT_GUID, &guid_buf, sizeof guid_buf);
+		g_assert(guid_is_gtkg(&guid_buf, NULL, NULL, NULL));
+	}
 
 	/*
 	 * Initialize message type array for routing logs.
 	 */
 
 	for (i = 0; i < 256; i++) {
-		const gchar *s;
+		const char *s;
 
 		s = "UNKN ";
 		switch ((enum gta_msg) i) {
@@ -771,6 +783,7 @@ routing_init(void)
 		case GTA_MSG_QRP:            s = "QRP  "; break;
 		case GTA_MSG_HSEP_DATA:      s = "HSEP "; break;
 		case GTA_MSG_BYE:      		 s = "Bye  "; break;
+		case GTA_MSG_DHT:      		 s = "DHT  "; break;
 		}
 		debug_msg[i] = s;
 	}
@@ -910,7 +923,8 @@ routing_node_remove(struct gnutella_node *node)
  * node emitting it.
  */
 void
-message_add(const gchar *muid, guint8 function, struct gnutella_node *node)
+message_add(const struct guid *muid, guint8 function,
+	struct gnutella_node *node)
 {
 	struct route_data *route;
 	struct message *entry;
@@ -964,7 +978,7 @@ message_add(const gchar *muid, guint8 function, struct gnutella_node *node)
 		g_assert(entry->routes == NULL);
 
 		/* fill in that storage space */
-		memcpy(entry->muid, muid, GUID_RAW_SIZE);
+		entry->muid = *muid;
 		entry->function = function;
 	}
 
@@ -1053,21 +1067,18 @@ purge_dangling_references(struct message *m)
 /**
  * Look for a particular message in the routing tables.
  *
- * If we find the message, returns true, otherwise false.
- *
  * If none of the nodes that sent us the message are still present, then
  * m->routes will be NULL.
+ *
+ * @return TRUE if the message is found.
  */
 static gboolean
-find_message(const gchar *muid, guint8 function, struct message **m)
+find_message(const struct guid *muid, guint8 function, struct message **m)
 {
-	/* @returns TRUE if the message is found */
-	/* Set *node to node if there is a connected node associated
-	   with the message found */
 	struct message dummyMessage;
-	struct message * found_message;
+	struct message *found_message;
 
-	memcpy(dummyMessage.muid, muid, GUID_RAW_SIZE);
+	dummyMessage.muid = *muid;
 	dummyMessage.function = function;
 
 	found_message = g_hash_table_lookup(routing.messages_hashed, &dummyMessage);
@@ -1138,8 +1149,7 @@ static gboolean
 forward_message(
 	struct route_log *route_log,
 	struct gnutella_node **node,
-	struct gnutella_node *target, struct route_dest *dest, GSList *routes,
-	gboolean duplicate)
+	struct gnutella_node *target, struct route_dest *dest, GSList *routes)
 {
 	struct gnutella_node *sender = *node;
 
@@ -1203,41 +1213,28 @@ forward_message(
 		if (routes != NULL) {
 			GSList *l;
 			GSList *nodes = NULL;
-			gint count;
+			int count = 0;
 
 			g_assert(gnutella_header_get_function(&sender->header)
 					== GTA_MSG_PUSH_REQUEST);
 
-			for (l = routes, count = 0; l; l = g_slist_next(l), count++) {
+			for (l = routes; l; l = g_slist_next(l)) {
 				struct route_data *rd = l->data;
 				if (rd->node == sender)
 					continue;
 
-				/*
-				 * There is a bug in GTKG 0.95.x: because it did not track
-				 * the TTL of messages received on a connection basis, it
-				 * wrongfully recorded the message as being duplicates when
-				 * a message with a higher TTL was received from a connection,
-				 * if the TTL was lower than the maximum ever seen for that
-				 * message (accross all the connections).
-				 *
-				 * To avoid the remote end from disconnecting, we now flag
-				 * those old broken nodes and don't forward them ANY duplicate.
-				 *		--RAM, 2005-08-02
-				 */
-
-				if (rd->node->attrs & NODE_A_NO_DUPS)
-					continue;
-
 				nodes = g_slist_prepend(nodes, rd->node);
+				count++;
 			}
 
 			/*
 			 * The `nodes' list will be freed by node_parse().
 			 */
 
-			dest->type = ROUTE_MULTI;
-			dest->ur.u_nodes = nodes;
+			if (count > 0) {
+				dest->type = ROUTE_MULTI;
+				dest->ur.u_nodes = nodes;
+			}
 
 			if (count > 1)
 				gnet_stats_count_general(GNR_BROADCASTED_PUSHES, 1);
@@ -1257,7 +1254,7 @@ forward_message(
 					gnutella_header_get_ttl(&sender->header)
 						> GNET_PROPERTY(max_ttl)
 			) {
-				gint ttl_max;
+				int ttl_max;
 			   
 				/* Trim down */
 				ttl_max = GNET_PROPERTY(max_ttl);
@@ -1276,7 +1273,7 @@ forward_message(
 						gnutella_header_get_ttl(&sender->header));
 			}
 
-			dest->type = duplicate ? ROUTE_NO_DUPS_BUT_ONE : ROUTE_ALL_BUT_ONE;
+			dest->type = ROUTE_ALL_BUT_ONE;
 			dest->ur.u_node = sender;
 		}
 	}
@@ -1298,8 +1295,8 @@ forward_message(
  * the message was a duplicate and it should not be handled locally.
  */
 static gboolean
-check_duplicate(struct route_log *route_log,
-	struct gnutella_node **node, const gchar *mangled, struct message **mp)
+check_duplicate(struct route_log *route_log, struct gnutella_node **node,
+	const struct guid *mangled, struct message **mp)
 {
 	struct gnutella_node *sender = *node;
 
@@ -1355,10 +1352,12 @@ check_duplicate(struct route_log *route_log,
 			higher_ttl = node_ttl_higher(sender, m,
 							gnutella_header_get_ttl(&sender->header));
 
-			if (higher_ttl)
-				routing_log_extra(route_log, "dup message (same node, higher TTL)");
-			else
+			if (higher_ttl) {
+				routing_log_extra(route_log,
+					"dup message (same node, higher TTL)");
+			} else {
 				routing_log_extra(route_log, "dup message (same node)");
+			}
 
 			/*
 			 * That is a really good reason to kick the offender
@@ -1474,20 +1473,23 @@ check_duplicate(struct route_log *route_log,
  */
 static gboolean
 route_push(struct route_log *route_log,
-	struct gnutella_node **node, struct route_dest *dest, gboolean duplicate)
+	struct gnutella_node **node, struct route_dest *dest)
 {
 	struct gnutella_node *sender = *node;
 	struct message *m;
-	const gchar *guid;
+	const struct guid *guid;
+	struct gnutella_node *neighbour;
 
 	/*
 	 * A Push request is not broadcasted as other requests, it is routed
 	 * back along the nodes that have seen Query Hits from the target
 	 * servent of the Push.
+	 *
+	 * The GUID of the target are the leading bytes of the Push message.
 	 */
 
 	g_assert(sender->size > GUID_RAW_SIZE);	/* Must be a valid push */
-	guid = sender->data;
+	guid = cast_to_guid_ptr_const(sender->data);	/* Targetted GUID */
 
 	/*
 	 * Is it for us?
@@ -1525,11 +1527,28 @@ route_push(struct route_log *route_log,
 	}
 
 	/*
-	 * The GUID of the target are the leading bytes of the Push
-	 * message, hence we pass `sender->data' to find_message().
+	 * If we find a local route (one of our neighbours), use that.
+	 * Otherwise look for a route in the routing table.
 	 */
 
-	if (!find_message(guid, QUERY_HIT_ROUTE_SAVE, &m) || m->routes == NULL) {
+	if (NULL != (neighbour = node_by_guid(guid))) {
+		gboolean inserted;
+		message_add(guid, QUERY_HIT_ROUTE_SAVE, neighbour);
+		inserted = find_message(guid, QUERY_HIT_ROUTE_SAVE, &m) && m->routes;
+		g_assert(inserted);
+		gnet_stats_count_general(GNR_PUSH_RELAYED_VIA_LOCAL_ROUTE, 1);
+
+		/*
+		 * If we got a PUSH directly through UDP, and we can relay it to
+		 * a local neighbour, we were most likely the push-proxy for that node.
+		 */
+
+		if (NODE_IS_UDP(sender))
+			gnet_stats_count_general(GNR_PUSH_PROXY_UDP_RELAYED, 1);
+
+	} else if (find_message(guid, QUERY_HIT_ROUTE_SAVE, &m) && m->routes) {
+		gnet_stats_count_general(GNR_PUSH_RELAYED_VIA_TABLE_ROUTE, 1);
+	} else {
 		if (m && m->routes == NULL) {
 			routing_log_extra(route_log, "route to target GUID %s gone",
 				guid_hex_str(guid));
@@ -1549,7 +1568,7 @@ route_push(struct route_log *route_log,
 	 */
 
 	revitalize_entry(m, FALSE);
-	forward_message(route_log, node, NULL, dest, m->routes, duplicate);
+	forward_message(route_log, node, NULL, dest, m->routes);
 
 	return FALSE;		/* We are not the target, don't handle it */
 }
@@ -1561,7 +1580,7 @@ route_push(struct route_log *route_log,
  */
 static gboolean
 route_query(struct route_log *route_log,
-	struct gnutella_node **node, struct route_dest *dest, gboolean duplicate)
+	struct gnutella_node **node, struct route_dest *dest)
 {
 	struct gnutella_node *sender = *node;
 	gboolean is_oob_query;
@@ -1631,7 +1650,7 @@ route_query(struct route_log *route_log,
 		(guint) gnutella_header_get_ttl(&sender->header) +
 			gnutella_header_get_hops(&sender->header) >= GNET_PROPERTY(my_ttl)
 	) {
-		gint ttl_max;
+		int ttl_max;
 	
 		ttl_max = GNET_PROPERTY(my_ttl)
 					- gnutella_header_get_hops(&sender->header) - 1;
@@ -1644,7 +1663,7 @@ route_query(struct route_log *route_log,
 	}
 
 	/* Broadcast */
-	return forward_message(route_log, node, NULL, dest, NULL, duplicate);
+	return forward_message(route_log, node, NULL, dest, NULL);
 }
 
 /**
@@ -1661,10 +1680,11 @@ route_query_hit(struct route_log *route_log,
 	gboolean node_is_target = FALSE;
 	struct gnutella_node *found;
 	gboolean is_oob_proxied;
-	const gchar *origin_guid;
+	const struct guid *origin_guid;
 
 	g_assert(sender->size >= GUID_RAW_SIZE);
-   	origin_guid = &sender->data[sender->size - GUID_RAW_SIZE];
+   	origin_guid =
+		cast_to_guid_ptr_const(&sender->data[sender->size - GUID_RAW_SIZE]);
 
 	/*
 	 * We have to record we have seen a hit reply from the GUID held at
@@ -1913,7 +1933,7 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	gboolean duplicate = FALSE;
 	gboolean route_it = FALSE;
 	struct route_log route_log;
-	const gchar *mangled = NULL;
+	const struct guid *mangled = NULL;
 
 	/* Ensure we never get something bearing our special GUID route marker */
 	g_assert(
@@ -1992,10 +2012,10 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 
 	switch (gnutella_header_get_function(&sender->header)) {
 	case GTA_MSG_PUSH_REQUEST:
-		handle_it = route_push(&route_log, node, dest, duplicate);
+		handle_it = route_push(&route_log, node, dest);
 		break;
 	case GTA_MSG_SEARCH:
-		handle_it = route_query(&route_log, node, dest, duplicate);
+		handle_it = route_query(&route_log, node, dest);
 		break;
 	case GTA_MSG_SEARCH_RESULTS:
 		handle_it = route_query_hit(&route_log, node, dest);
@@ -2034,7 +2054,7 @@ done:
  * @returns boolean indicating whether we have such a route.
  */
 gboolean
-route_exists_for_reply(const gchar *muid, guint8 function)
+route_exists_for_reply(const struct guid *muid, guint8 function)
 {
 	struct message *m;
 
@@ -2052,7 +2072,7 @@ route_exists_for_reply(const gchar *muid, guint8 function)
  * send the packet otherwise.  It is up to the caller to free that list.
  */
 GSList *
-route_towards_guid(const gchar *guid)
+route_towards_guid(const struct guid *guid)
 {
 	struct gnutella_node *node;
 	struct message *m;
@@ -2082,7 +2102,7 @@ route_towards_guid(const gchar *guid)
  * Remove push-proxy entry indexed by GUID.
  */
 void
-route_proxy_remove(const gchar *guid)
+route_proxy_remove(const struct guid *guid)
 {
  	/*
 	 * The GUID atom is still referred to by the node,
@@ -2101,7 +2121,7 @@ route_proxy_remove(const gchar *guid)
  * NB: assumes `guid' is already an atom linked somehow to `n'.
  */
 gboolean
-route_proxy_add(const gchar *guid, struct gnutella_node *n)
+route_proxy_add(const struct guid *guid, struct gnutella_node *n)
 {
 	if (NULL != g_hash_table_lookup(ht_proxyfied, guid))
 		return FALSE;
@@ -2118,7 +2138,7 @@ route_proxy_add(const gchar *guid, struct gnutella_node *n)
  * that node directly.
  */
 struct gnutella_node *
-route_proxy_find(const gchar *guid)
+route_proxy_find(const struct guid *guid)
 {
 	return g_hash_table_lookup(ht_proxyfied, guid);
 }
@@ -2150,7 +2170,7 @@ routing_close(void)
 	for (cnt = 0; cnt < MAX_CHUNKS; cnt++) {
 		struct message **chunk = routing.chunks[cnt];
 		if (chunk != NULL) {
-			gint i;
+			int i;
 			for (i = 0; i < CHUNK_MESSAGES; i++) {
 				struct message *m = chunk[i];
 				if (m != NULL) {
