@@ -1,5 +1,5 @@
 /*
- * $Id: rx_link.c 13371 2007-04-19 23:30:54Z cbiere $
+ * $Id: rx_link.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,15 +38,15 @@
 
 #include "common.h"
 
-RCSID("$Id: rx_link.c 13371 2007-04-19 23:30:54Z cbiere $")
+RCSID("$Id: rx_link.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "sockets.h"
-#include "pmsg.h"
 #include "rx.h"
 #include "rx_link.h"
 #include "rxbuf.h"
 #include "bsched.h"
 
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -64,7 +64,7 @@ struct attr {
  * Invoked when the input file descriptor has more data available.
  */
 static void
-is_readable(gpointer data, gint unused_source, inputevt_cond_t cond)
+is_readable(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	rxdrv_t *rx = data;
 	struct attr *attr = rx->opaque;
