@@ -1,5 +1,5 @@
 /*
- * $Id: rxbuf.c,v 1.5 2005/09/25 21:41:48 rmanfredi Exp $
+ * $Id: rxbuf.c 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,14 +35,15 @@
 
 #include "common.h"
 
-RCSID("$Id: rxbuf.c,v 1.5 2005/09/25 21:41:48 rmanfredi Exp $");
+RCSID("$Id: rxbuf.c 16104 2009-03-10 14:58:09Z rmanfredi $")
 
 #include "rxbuf.h"
-#include "pmsg.h"
 
 #include "if/gnet_property_priv.h"
 
+#include "lib/misc.h"
 #include "lib/palloc.h"
+#include "lib/pmsg.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -51,19 +52,29 @@ RCSID("$Id: rxbuf.c,v 1.5 2005/09/25 21:41:48 rmanfredi Exp $");
  * is held into a list.  Each pdata_t is equipped with a suitable free routine.
  */
 
-#define BUF_COUNT	5		/**< Max amount of buffers we want in pool */
-#define BUF_SIZE	4096	/**< Size of each buffer */
+#define BUF_COUNT	64		/**< Max amount of buffers we want in pool */
 
 static pool_t *rxpool;
 
 /**
  * Put RX buffer back to its pool.
- *
- * Can be called directly, or via pdata_unref() because this routine is
- * installed as the "free routine" of the buffer.
  */
 void
-rxbuf_free(gpointer p, gpointer unused_data)
+rxbuf_free(gpointer p)
+{
+	pdata_t *db = p;
+
+	g_assert(db->d_refcnt == 0);
+
+	db->d_refcnt = 1;		/* Force freeing of buffer */
+	pdata_unref(p);
+}
+
+/**
+ * Free routine for the page-aligned buffer, called by pdata_unref().
+ */
+static void
+rxbuf_data_free(gpointer p, gpointer unused_data)
 {
 	(void) unused_data;
 
@@ -78,9 +89,33 @@ rxbuf_free(gpointer p, gpointer unused_data)
 pdata_t *
 rxbuf_new(void)
 {
-	gchar *phys = palloc(rxpool);
+	char *phys = palloc(rxpool);
 
-	return pdata_allocb(phys, BUF_SIZE, rxbuf_free, NULL);
+	/*
+	 * We want to use page-aligned memory to benefit from possible zero-copy
+	 * on read() operations.  Hence we must use pdata_allocb_ext() to avoid
+	 * embedding the pdata_t header at the beginning of the buffer.
+	 */
+
+	return pdata_allocb_ext(phys, compat_pagesize(), rxbuf_data_free, NULL);
+}
+
+/**
+ * Wrapper over g_malloc().
+ */
+static gpointer
+rxbuf_page_alloc(size_t size)
+{
+	return g_malloc(size);
+}
+
+/**
+ * Wrapper over free().
+ */
+static void
+rxbuf_page_free(gpointer p)
+{
+	g_free(p);
 }
 
 /**
@@ -89,7 +124,8 @@ rxbuf_new(void)
 void
 rxbuf_init(void)
 {
-	rxpool = pool_create(BUF_SIZE, BUF_COUNT);
+	rxpool = pool_create(
+		compat_pagesize(), BUF_COUNT, rxbuf_page_alloc, rxbuf_page_free);
 }
 
 /**
