@@ -1,5 +1,5 @@
 /*
- * $Id: settings.c 15066 2007-12-24 18:04:41Z cbiere $
+ * $Id: settings.c 16133 2009-03-13 13:00:20Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -52,16 +52,20 @@
 #include "share.h"
 #include "sockets.h"
 #include "upload_stats.h"
+#include "routing.h"			/* For gnet_reset_guid() */
+#include "ipp_cache.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 #include "if/core/main.h"		/* For debugging() */
 #include "if/core/net_stats.h"
+#include "if/dht/dht.h"
 
 #include "if/bridge/c2ui.h"
 
 #include "lib/bit_array.h"
 #include "lib/cq.h"
+#include "lib/debug.h"
 #include "lib/file.h"
 #include "lib/getphysmemsize.h"
 #include "lib/glib-missing.h"
@@ -70,18 +74,18 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: settings.c 15066 2007-12-24 18:04:41Z cbiere $")
+RCSID("$Id: settings.c 16133 2009-03-13 13:00:20Z rmanfredi $")
 
-static const gchar config_file[] = "config_gnet";
-static const gchar ul_stats_file[] = "upload_stats";
+static const char config_file[] = "config_gnet";
+static const char ul_stats_file[] = "upload_stats";
 
 static const mode_t IPC_DIR_MODE = S_IRUSR | S_IWUSR | S_IXUSR; /* 0700 */
 static const mode_t PID_FILE_MODE = S_IRUSR | S_IWUSR; /* 0600 */
 static const mode_t CONFIG_DIR_MODE =
 	S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP; /* 0750 */
 
-static gchar *home_dir = NULL;
-static gchar *config_dir = NULL;
+static char *home_dir = NULL;
+static char *config_dir = NULL;
 
 static prop_set_t *properties = NULL;
 
@@ -99,7 +103,7 @@ static prop_set_t *properties = NULL;
  * progressbar_bps_out_avg     0.90u 15/05/2002 progressbar_bws_out_avg
  */
 
-static const gchar pidfile[] = "gtk-gnutella.pid";
+static const char pidfile[] = "gtk-gnutella.pid";
 
 static gboolean settings_init_running;
 
@@ -109,6 +113,27 @@ static void update_uptimes(void);
 
 /* ----------------------------------------- */
 
+/**
+ * Insert local IP:port in the local address cache if combination is valid.
+ */
+static void
+remember_local_addr_port(void)
+{
+	host_addr_t addr;
+	guint16 port;
+
+	addr = listen_addr();
+	port = GNET_PROPERTY(listen_port);
+
+	if (host_is_valid(addr, port))
+		local_addr_cache_insert(addr, port);
+
+	addr = listen_addr6();
+
+	if (host_is_valid(addr, port))
+		local_addr_cache_insert(addr, port);
+}
+
 /**
  * @return the currently used local listening address.
  */
@@ -176,11 +201,11 @@ is_my_address_and_port(const host_addr_t addr, guint16 port)
  *			locked. Other errno values imply that the PID file could not
  *			be created.
  */
-static gint
-ensure_unicity(const gchar *file, gboolean check_only)
+static int
+ensure_unicity(const char *file, gboolean check_only)
 {
 	gboolean locked = FALSE;
-	gint fd;
+	int fd;
 
 	g_assert(file);
 
@@ -206,7 +231,7 @@ ensure_unicity(const gchar *file, gboolean check_only)
 
 		locking_failed = -1 == fcntl(fd, F_SETLK, &fl);
 		if (locking_failed) {
-			gint saved_errno = errno;
+			int saved_errno = errno;
 
 			if (!check_only) {
 				g_warning("fcntl(%d, F_SETLK, ...) failed for \"%s\": %s",
@@ -239,7 +264,7 @@ ensure_unicity(const gchar *file, gboolean check_only)
 	 * fall back to weaker PID locking */
 	if (!locked) {
 		ssize_t r;
-		gchar buf[33];
+		char buf[33];
 
 		r = read(fd, buf, sizeof buf - 1);
 		if ((ssize_t) -1 == r) {
@@ -254,7 +279,7 @@ ensure_unicity(const gchar *file, gboolean check_only)
 		/* Check the PID in the file */
 		{
 			guint64 u;
-			gint error;
+			int error;
 
 			g_assert(r >= 0 && (size_t) r < sizeof buf);
 			buf[r] = '\0';
@@ -299,10 +324,10 @@ failed:
  * Write our pid to the pidfile.
  */
 static void
-save_pid(gint fd)
+save_pid(int fd)
 {
 	size_t len;
-	gchar buf[32];
+	char buf[32];
 
 	g_assert(-1 != fd);
 
@@ -358,16 +383,16 @@ settings_early_init(void)
  * @returns On success zero is returned, otherwise a non-zero is returned
  *			and errno is set.
  */
-gint
+int
 settings_ensure_unicity(gboolean check_only)
 {
-	gint fd;
+	int fd;
 
 	g_assert(config_dir);
 
 	{
-		gchar *path;
-		gint saved_errno;
+		char *path;
+		int saved_errno;
 		
 		path = make_pathname(config_dir, pidfile);
 		fd = ensure_unicity(path, check_only);
@@ -393,11 +418,9 @@ settings_init_session_id(void)
 	SHA1Context ctx;
 	struct sha1 digest;
 	guint32 noise[64];
-	size_t size, i;
+	size_t size;
 
-	for (i = 0; i < G_N_ELEMENTS(noise); i++) {
-		noise[i] = random_raw();
-	}
+	random_bytes(noise, sizeof noise);
 	SHA1Reset(&ctx);
 	SHA1Input(&ctx, &noise, sizeof noise);
 	SHA1Result(&ctx, &digest);
@@ -453,8 +476,20 @@ settings_init(void)
 		exit(EXIT_FAILURE);
 	}
 
-	/* Parse the configuration */
-	prop_load_from_file(properties, config_dir, config_file);
+	/*
+	 * Parse the configuration.
+	 *
+	 * Reset the KUID and the GUID if the file was copied from another
+	 * instance to prevent duplicate IDs on the network which are harmful
+	 * for everyone.
+	 */
+
+	if (!prop_load_from_file(properties, config_dir, config_file)) {
+		if (debugging(0))
+			g_warning("config file \"%s\" was copied over", config_file);
+		dht_reset_kuid();
+		gnet_reset_guid();
+	}
 
 	if (debugging(0)) {
 		g_message("detected amount of physical RAM: %s",
@@ -466,7 +501,7 @@ settings_init(void)
 	}
 
 	{
-		gchar *path;
+		char *path;
 
 		path = make_pathname(config_dir, ul_stats_file);
 		upload_stats_load_history(path);	/* Loads the upload statistics */
@@ -502,30 +537,30 @@ no_config_dir:
 /**
  * Get the config directory
  */
-const gchar *
+const char *
 settings_config_dir(void)
 {
 	g_assert(NULL != config_dir);
-	return (const gchar *) config_dir;
+	return (const char *) config_dir;
 }
 
 /**
  * Gets the home dir.
  */
-const gchar *
+const char *
 settings_home_dir(void)
 {
 	g_assert(NULL != home_dir);
-	return (const gchar *) home_dir;
+	return (const char *) home_dir;
 }
 
 /**
  * Gets the IPC directory.
  */
-static const gchar *
+static const char *
 settings_ipc_dir(void)
 {
-	static const gchar *path;
+	static const char *path;
 
 	if (!path) {
 		path = make_pathname(settings_config_dir(), "ipc");
@@ -536,10 +571,10 @@ settings_ipc_dir(void)
 /**
  * Gets the path of the local socket.
  */
-const gchar *
+const char *
 settings_local_socket_path(void)
 {
-	static const gchar *path;
+	static const char *path;
 
 	if (!path) {
 		path = make_pathname(settings_ipc_dir(), "socket");
@@ -574,7 +609,7 @@ settings_dns_net(void)
 static void
 settings_remove_pidfile(void)
 {
-	gchar *path;
+	char *path;
 
 	g_assert(config_dir);
 
@@ -721,8 +756,8 @@ addr_ipv6_changed(const host_addr_t new_addr, const host_addr_t peer)
 void
 settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer)
 {
-	g_assert(is_host_addr(new_addr)); /* The new IP must be valid */
-	g_assert(is_host_addr(peer)); /* The peer's IP must be valid */
+	g_assert(host_addr_initialized(new_addr));
+	g_assert(host_addr_initialized(peer));
 
 	if (!host_addr_is_routable(new_addr) || !host_addr_is_routable(peer))
 		return;
@@ -781,6 +816,7 @@ void
 settings_shutdown(void)
 {
 	update_uptimes();
+	remember_local_addr_port();
     settings_callbacks_shutdown();
 
     prop_save_to_file(properties, config_dir, config_file);
@@ -896,7 +932,8 @@ get_average_ip_lifetime(time_t now, enum net_type net)
 
 /**
  * Called whenever the IP address we advertise changed.
- * Update the average uptime for a given IP address.
+ * Update the average uptime for a given IP address and remember the new
+ * address in the "local_addr" cache.
  */
 static void
 update_address_lifetime(void)
@@ -947,6 +984,14 @@ update_address_lifetime(void)
 		}
 		gnet_prop_set_timestamp_val(PROP_CURRENT_IP6_STAMP, now);
 	}
+
+	/*
+	 * We remember every local IP:port combination we had for some time in
+	 * order to later spot alternate locations that point back to one of
+	 * our recent IP:port.
+	 */
+
+	remember_local_addr_port();
 }
 
 /**
@@ -1122,6 +1167,21 @@ enable_udp_changed(property_t prop)
 	return FALSE;
 }
 
+static gboolean
+enable_dht_changed(property_t prop)
+{
+	gboolean enabled;
+	
+    gnet_prop_get_boolean_val(prop, &enabled);
+	if (enabled) {
+		dht_initialize(TRUE);
+	} else {
+		dht_close();
+	}
+
+	return FALSE;
+}
+
 static gboolean
 enable_local_socket_changed(property_t prop)
 {
@@ -1130,11 +1190,11 @@ enable_local_socket_changed(property_t prop)
     gnet_prop_get_boolean_val(prop, &enabled);
 	if (enabled) {
 		if (!s_local_listen) {
-			const gchar *ipc_dir;
+			const char *ipc_dir;
 
 			ipc_dir = settings_ipc_dir();
 			if (0 == compat_mkdir(ipc_dir, IPC_DIR_MODE) || EEXIST == errno) {
-				const gchar *socket_path;
+				const char *socket_path;
 
 				socket_path = settings_local_socket_path();
 				s_local_listen = socket_local_listen(socket_path);
@@ -1301,6 +1361,7 @@ listen_port_changed(property_t prop)
         return TRUE;
     } else {
 		old_port = GNET_PROPERTY(listen_port);
+		remember_local_addr_port();
 	}
 
     return FALSE;
@@ -1382,7 +1443,7 @@ node_sendqueue_size_changed(property_t unused_prop)
 static gboolean
 scan_extensions_changed(property_t prop)
 {
-    gchar *s = gnet_prop_get_string(prop, NULL, 0);
+    char *s = gnet_prop_get_string(prop, NULL, 0);
 
     parse_extensions(s);
     G_FREE_NULL(s);
@@ -1393,7 +1454,7 @@ scan_extensions_changed(property_t prop)
 static gboolean
 file_path_changed(property_t prop)
 {
-    gchar *s;
+    char *s;
 
 	s = gnet_prop_get_string(prop, NULL, 0);
 	g_assert(s != NULL);
@@ -1412,7 +1473,7 @@ file_path_changed(property_t prop)
 static gboolean
 shared_dirs_paths_changed(property_t prop)
 {
-    gchar *s = gnet_prop_get_string(prop, NULL, 0);
+    char *s = gnet_prop_get_string(prop, NULL, 0);
 	gboolean ok;
 
 	ok = shared_dirs_parse(s);
@@ -1429,7 +1490,7 @@ shared_dirs_paths_changed(property_t prop)
 static gboolean
 local_netmasks_string_changed(property_t prop)
 {
-    gchar *s = gnet_prop_get_string(prop, NULL, 0);
+    char *s = gnet_prop_get_string(prop, NULL, 0);
 
     parse_netmasks(s);
     G_FREE_NULL(s);
@@ -1561,13 +1622,13 @@ node_online_mode_changed(property_t prop)
 }
 
 static gboolean
-lib_debug_changed(property_t unused_prop)
+lib_debug_changed(property_t prop)
 {
-	(void) unused_prop;
-/* XXX -- common_dbg is no longer a property! --RAM */
-#if 0
-	gnet_prop_get_guint32_val(prop, &common_dbg);
-#endif
+	guint32 val;
+
+	gnet_prop_get_guint32_val(prop, &val);
+	set_library_debug(val);
+
     return FALSE;
 }
 
@@ -2047,6 +2108,11 @@ static prop_map_t property_map[] = {
 		enable_udp_changed,
 		FALSE,				/* UDP socket inited via listen_port_changed() */
 	},
+	{
+		PROP_ENABLE_DHT,
+		enable_dht_changed,
+		FALSE,
+	},
 	{
 		PROP_ENABLE_LOCAL_SOCKET,
 		enable_local_socket_changed,
