@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $
+ * $Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $")
+RCSID("$Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $")
 
 #ifdef I_NETDB
 #include <netdb.h>
@@ -61,7 +61,8 @@ RCSID("$Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $")
 #include "pproxy.h"
 #include "udp.h"
 #include "settings.h"
-#include "tls_cache.h"
+#include "ipp_cache.h"
+#include "gnet_stats.h"
 
 #include "shell/shell.h"
 
@@ -70,6 +71,7 @@ RCSID("$Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $")
 
 #include "lib/socket.h"
 #include "lib/adns.h"
+#include "lib/ascii.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/endian.h"
@@ -92,6 +94,7 @@ RCSID("$Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $")
 
 #define RQST_LINE_LENGTH	256	/**< Reasonable estimate for request line */
 #define SOCK_UDP_RECV_BUF	131072	/**< 128K - Large to avoid loosing dgrams */
+#define MAX_UDP_RECV_LOOP	128		/**< Max messages read from UDP queue */
 
 enum {
 	SOCK_ADNS_PENDING	= 1 << 0,	/**< Don't free() the socket too early */
@@ -106,7 +109,7 @@ struct gnutella_socket *s_udp_listen = NULL;
 struct gnutella_socket *s_udp_listen6 = NULL;
 struct gnutella_socket *s_local_listen = NULL;
 
-static void socket_accept(gpointer data, gint, inputevt_cond_t cond);
+static void socket_accept(gpointer data, int, inputevt_cond_t cond);
 
 static struct gnutella_socket *
 socket_alloc(void)
@@ -180,10 +183,10 @@ socket_ipv6_trt_map(const host_addr_t addr)
  * Return the file descriptor to use for I/O monitoring callbacks on
  * the socket.
  */
-gint
+int
 socket_evt_fd(struct gnutella_socket *s)
 {
-	gint fd = -1;
+	int fd = -1;
 
 	socket_check(s);
 	switch (s->direction) {
@@ -222,7 +225,7 @@ void
 socket_evt_set(struct gnutella_socket *s,
 	inputevt_cond_t cond, inputevt_handler_t handler, gpointer data)
 {
-	gint fd;
+	int fd;
 
 	socket_check(s);
 	g_assert(handler);
@@ -260,7 +263,7 @@ socket_evt_clear(struct gnutella_socket *s)
 
 	if (s->gdk_tag) {
 		if (GNET_PROPERTY(tls_debug) > 4) {
-			gint fd = socket_evt_fd(s);
+			int fd = socket_evt_fd(s);
 			g_message("socket_evt_clear: fd=%d, cond=%s, handler=%p",
 				fd, inputevt_cond_to_string(s->tls.cb_cond), s->tls.cb_handler);
 		}
@@ -295,8 +298,8 @@ socket_register_fd_reclaimer(reclaim_fd_t callback)
 static GSList *sl_incoming = NULL;	/**< To spot inactive sockets */
 
 static void guess_local_addr(struct gnutella_socket *s);
-static void socket_destroy(struct gnutella_socket *s, const gchar *reason);
-static void socket_connected(gpointer data, gint source, inputevt_cond_t cond);
+static void socket_destroy(struct gnutella_socket *s, const char *reason);
+static void socket_connected(gpointer data, int source, inputevt_cond_t cond);
 static void socket_wio_link(struct gnutella_socket *s);
 
 /*
@@ -308,9 +311,9 @@ static void socket_wio_link(struct gnutella_socket *s);
  * go badly.
  */
 static gboolean sol_got = FALSE;
-static gint sol_tcp_cached = -1;
-static gint sol_ip_cached = -1;
-static gint sol_ipv6_cached = -1;
+static int sol_tcp_cached = -1;
+static int sol_ip_cached = -1;
+static int sol_ipv6_cached = -1;
 
 /**
  * Compute and cache values for SOL_TCP and SOL_IP.
@@ -346,7 +349,7 @@ get_sol(void)
 /**
  * @returns SOL_TCP.
  */
-static gint
+static int
 sol_tcp(void)
 {
 	g_assert(sol_got);
@@ -356,7 +359,7 @@ sol_tcp(void)
 /**
  * @returns SOL_IP.
  */
-static gint
+static int
 sol_ip(void)
 {
 	g_assert(sol_got);
@@ -366,7 +369,7 @@ sol_ip(void)
 /**
  * @returns SOL_IPV6.
  */
-static gint
+static int
 sol_ipv6(void)
 {
 	g_assert(sol_got);
@@ -379,8 +382,8 @@ sol_ipv6(void)
  * Set the TOS on the socket.  Routers can use this information to
  * better route the IP datagrams.
  */
-static gint
-socket_tos(const struct gnutella_socket *s, gint tos)
+static int
+socket_tos(const struct gnutella_socket *s, int tos)
 {
 	socket_check(s);
 	g_return_val_if_fail(NET_TYPE_NONE != s->net, 0);
@@ -391,7 +394,7 @@ socket_tos(const struct gnutella_socket *s, gint tos)
 		-1 == setsockopt(s->file_desc, sol_ip(), IP_TOS, &tos, sizeof tos)
 	) {
 		if (ECONNRESET != errno) {
-			const gchar *tosname;
+			const char *tosname;
 
 			switch (tos) {
 			case 0: tosname = "default"; break;
@@ -433,8 +436,8 @@ socket_tos_default(const struct gnutella_socket *s)
 	}
 }
 #else
-static gint
-socket_tos(const struct gnutella_socket *unused_s, gint unused_tos)
+static int
+socket_tos(const struct gnutella_socket *unused_s, int unused_tos)
 {
 	(void) unused_s;
 	(void) unused_tos;
@@ -579,7 +582,7 @@ proxy_connect(int fd)
 	return connect(fd, socket_addr_get_const_sockaddr(&server), len);
 }
 
-static gint
+static int
 send_socks4(struct gnutella_socket *s)
 {
 	size_t length;
@@ -616,7 +619,7 @@ send_socks4(struct gnutella_socket *s)
 	/* Determine the current username */
 	{
 		const struct passwd *user;
-		const gchar *name;
+		const char *name;
 		size_t name_size;
 
 		user = getpwuid(getuid());
@@ -650,7 +653,7 @@ send_socks4(struct gnutella_socket *s)
 	return 0;
 }
 
-static gint
+static int
 recv_socks4(struct gnutella_socket *s)
 {
 	struct {
@@ -707,13 +710,13 @@ recv_socks4(struct gnutella_socket *s)
 	return 0;
 }
 
-static gint
+static int
 connect_http(struct gnutella_socket *s)
 {
 	ssize_t ret;
 	size_t parsed;
-	gint status;
-	const gchar *str;
+	int status;
+	const char *str;
 
 	socket_check(s);
 
@@ -721,13 +724,13 @@ connect_http(struct gnutella_socket *s)
 	case 0:
 		{
 			static const struct {
-				const gchar *s;
+				const char *s;
 			} parts[] = {
 				{ "CONNECT " }, { NULL }, { " HTTP/1.0\r\nHost: " }, { NULL },
 				{ "\r\n\r\n" },
 			};
 			struct iovec iov[G_N_ELEMENTS(parts)];
-			const gchar *host_port = host_addr_port_to_string(s->addr, s->port);
+			const char *host_port = host_addr_port_to_string(s->addr, s->port);
 			size_t size = 0;
 			guint i;
 
@@ -849,14 +852,14 @@ connect_http(struct gnutella_socket *s)
 6: Done
 */
 
-static gint
+static int
 connect_socksv5(struct gnutella_socket *s)
 {
-	static const gchar verstring[] = "\x05\x02\x02";
+	static const char verstring[] = "\x05\x02\x02";
 	ssize_t ret = 0;
 	size_t size;
-	const gchar *name;
-	gint sockid;
+	const char *name;
+	int sockid;
 	host_addr_t addr;
 
 	socket_check(s);
@@ -1241,7 +1244,7 @@ socket_shutdown(void)
  * with the supplied reason.
  */
 static void
-socket_destroy(struct gnutella_socket *s, const gchar *reason)
+socket_destroy(struct gnutella_socket *s, const char *reason)
 {
 	socket_check(s);
 
@@ -1413,13 +1416,13 @@ destroy:
  * will be called as often as necessary to fetch a full line.
  */
 static void
-socket_read(gpointer data, gint source, inputevt_cond_t cond)
+socket_read(gpointer data, int source, inputevt_cond_t cond)
 {
 	struct gnutella_socket *s = data;
 	size_t count;
 	ssize_t r;
 	size_t parsed;
-	const gchar *first, *endptr;
+	const char *first, *endptr;
 
 	(void) source;
 
@@ -1525,8 +1528,10 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		if (
 			is_strprefix(s->buf, "GET ") ||
 			is_strprefix(s->buf, "HEAD ")
-		)
-			http_send_status(s, 414, FALSE, NULL, 0, "Requested URL Too Large");
+		) {
+			http_send_status(HTTP_UPLOAD, s, 414, FALSE, NULL, 0,
+				"Requested URL Too Large");
+		}
 		socket_destroy(s, "Requested URL too large");
 		return;
 	case READ_DONE:
@@ -1583,7 +1588,7 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		goto cleanup;
 	case BAN_MSG:				/* Send specific 403 error message */
 		{
-			const gchar *msg = ban_message(s->addr);
+			const char *msg = ban_message(s->addr);
 
             if (GNET_PROPERTY(socket_debug)) {
                 g_message("rejecting connection from banned %s (%s still): %s",
@@ -1598,7 +1603,8 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 
 				http_extra_callback_set(&hev, http_retry_after_add,
 					GUINT_TO_POINTER(ban_delay(s->addr)));
-				http_send_status(s, 503, FALSE, &hev, 1, "%s", msg);
+				http_send_status(HTTP_UPLOAD, s, 503, FALSE, &hev, 1,
+					"%s", msg);
 			}
 		}
 		goto cleanup;
@@ -1607,12 +1613,12 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 			send_node_error(s, 550, "Banned for %s",
 				short_time_ascii(ban_delay(s->addr)));
 		else {
-			gint delay = ban_delay(s->addr);
+			int delay = ban_delay(s->addr);
 			http_extra_desc_t hev;
 
 			http_extra_callback_set(&hev, http_retry_after_add,
 				GUINT_TO_POINTER(delay));
-			http_send_status(s, 550, FALSE, &hev, 1,
+			http_send_status(HTTP_UPLOAD, s, 550, FALSE, &hev, 1,
 				"Banned for %s", short_time_ascii(delay));
 		}
 		goto cleanup;
@@ -1647,17 +1653,22 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 	 */
 
 	if (hostiles_check(s->addr)) {
-		static const gchar msg[] = "Hostile IP address banned";
+		static const char msg[] = "Hostile IP address banned";
 
 		socket_disable_token(s);
 
-		if (GNET_PROPERTY(socket_debug))
+		if (GNET_PROPERTY(socket_debug)) {
+			const char *string = first;
+			if (!is_printable_iso8859_string(first))
+				string = "<non-printable request>";
 			g_warning("denying connection from hostile %s: \"%s\"",
-				host_addr_to_string(s->addr), first);
+				host_addr_to_string(s->addr), string);
+		}
+
 		if (is_strprefix(first, GNUTELLA_HELLO))
 			send_node_error(s, 550, msg);
 		else
-			http_send_status(s, 550, FALSE, NULL, 0, msg);
+			http_send_status(HTTP_UPLOAD, s, 550, FALSE, NULL, 0, msg);
 		goto cleanup;
 	}
 
@@ -1672,7 +1683,7 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		NULL != (endptr = is_strprefix(first, "GET ")) ||
 		NULL != (endptr = is_strprefix(first, "HEAD "))
 	) {
-		const gchar *uri;
+		const char *uri;
 
 		/*
 		 * We have to decide whether this is an upload request or a
@@ -1706,8 +1717,10 @@ unknown:
 		if (len > 0)
 			dump_hex(stderr, "First Line", first, MIN(len, 160));
 	}
-	if (strstr(first, "HTTP"))
-		http_send_status(s, 501, FALSE, NULL, 0, "Method Not Implemented");
+	if (strstr(first, "HTTP")) {
+		http_send_status(HTTP_UPLOAD, s, 501, FALSE, NULL, 0,
+			"Method Not Implemented");
+	}
 	/* FALL THROUGH */
 
 cleanup:
@@ -1724,7 +1737,7 @@ cleanup:
  * So far there are CONTROL, UPLOAD, DOWNLOAD and HTTP handlers.
  */
 static void
-socket_connected(gpointer data, gint source, inputevt_cond_t cond)
+socket_connected(gpointer data, int source, inputevt_cond_t cond)
 {
 	/* We are connected to somebody */
 
@@ -1809,7 +1822,7 @@ socket_connected(gpointer data, gint source, inputevt_cond_t cond)
 
 	if (0 != (cond & INPUT_EVENT_W)) {
 		/* We are just connected to our partner */
-		gint res, option;
+		int res, option;
 		socklen_t size = sizeof option;
 
 		socket_evt_clear(s);
@@ -2041,13 +2054,13 @@ socket_local_port(struct gnutella_socket *s)
  * Someone is connecting to us.
  */
 static void
-socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
+socket_accept(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	socket_addr_t addr;
 	socklen_t addr_len;
 	struct gnutella_socket *s = data;
 	struct gnutella_socket *t = NULL;
-	gint fd;
+	int fd;
 
 	(void) unused_source;
 	socket_check(s);
@@ -2324,17 +2337,15 @@ socket_udp_accept(struct gnutella_socket *s)
 		 */
 #if defined(CMSG_LEN) && defined(CMSG_SPACE)
 		{
-			static const size_t cmsg_size = 512;
-			static gchar *cmsg_buf;
-			static size_t cmsg_len;
+			union {
+				struct cmsghdr hdr;
+				size_t align;
+				char bytes[CMSG_SPACE(512)];
+			} cmsg_buf;
 
-			if (!cmsg_buf) {
-				cmsg_len = CMSG_LEN(cmsg_size);
-				cmsg_buf = walloc0(CMSG_SPACE(cmsg_size));
-			}
-
-			msg.msg_control = cmsg_buf;
-			msg.msg_controllen = cmsg_len;
+			memset(&cmsg_buf.hdr, 0, sizeof cmsg_buf.hdr);
+			msg.msg_control = cmsg_buf.bytes;
+			msg.msg_controllen = sizeof cmsg_buf.bytes;
 		}
 #endif /* CMSG_LEN && CMSG_SPACE */
 
@@ -2367,6 +2378,12 @@ socket_udp_accept(struct gnutella_socket *s)
 	s->addr = socket_addr_get_addr(from_addr);
 	s->port = socket_addr_get_port(from_addr);
 
+	if (!is_host_addr(s->addr)) {
+		gnet_stats_count_general(GNR_UDP_BOGUS_SOURCE_IP, 1);
+		errno = EINVAL;
+		return (ssize_t) -1;
+	}
+
 	if (has_dst_addr) {
 		static host_addr_t last_addr;
 
@@ -2398,15 +2415,17 @@ socket_udp_accept(struct gnutella_socket *s)
  * Someone is sending us a datagram.
  */
 static void
-socket_udp_event(gpointer data, gint unused_source, inputevt_cond_t cond)
+socket_udp_event(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	struct gnutella_socket *s = data;
-	size_t i, avail;
+	size_t avail, rd;
+	unsigned i;
+	tm_t start;
 
 	(void) unused_source;
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
-		gint error;
+		int error;
 		socklen_t error_len = sizeof error;
 
 		getsockopt(s->file_desc, SOL_SOCKET, SO_ERROR, &error, &error_len);
@@ -2420,17 +2439,27 @@ socket_udp_event(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 * as there are often several packets queued.
 	 */
 
+	if (GNET_PROPERTY(socket_debug))
+		tm_now_exact(&start);
+
 	avail = inputevt_data_available();
-	for (i = 0; i < 16; i++) {
+	avail = (avail != 0) ? avail : 64 * 1024;
+
+	i = 0;
+	rd = 0;
+	do {
 		ssize_t r;
 
 		r = socket_udp_accept(s);
 		if ((ssize_t) -1 == r) {
-			if (!is_temporary_error(errno))
+			if (!is_temporary_error(errno)) {
 				g_warning("ignoring datagram reception error: %s",
 					g_strerror(errno));
+			}
 			break;
 		}
+		i++;
+		rd += r;
 		if ((size_t) r >= avail)
 			break;
 		avail -= r;
@@ -2439,6 +2468,14 @@ socket_udp_event(gpointer data, gint unused_source, inputevt_cond_t cond)
 		 * it refers to header or control msg data. */
 		if (avail <= 32)
 			break;
+	} while (i < MAX_UDP_RECV_LOOP);
+
+	if (i > 16 && GNET_PROPERTY(socket_debug)) {
+		tm_t end;
+		tm_now_exact(&end);
+		g_message(
+			"socket_udp_event() iterated %u times, read %lu bytes in %lf secs",
+			i, (unsigned long) rd, tm_elapsed_f(&end, &start));
 	}
 }
 
@@ -2484,7 +2521,7 @@ socket_set_accept_filters(struct gnutella_socket *s)
 
 #if defined(TCP_DEFER_ACCEPT)
 	{
-		gint timeout;
+		int timeout;
 
 		timeout = MIN(GNET_PROPERTY(tcp_defer_accept_timeout), (guint) INT_MAX);
 		if (
@@ -2500,7 +2537,7 @@ socket_set_accept_filters(struct gnutella_socket *s)
 	{
 		static const struct accept_filter_arg zero_arg;
 		struct accept_filter_arg arg;
-		static const gchar name[] = "dataready";
+		static const char name[] = "dataready";
 
 		arg = zero_arg;
 		STATIC_ASSERT(sizeof arg.af_name >= CONST_STRLEN(name));
@@ -2572,12 +2609,12 @@ socket_set_quickack(struct gnutella_socket *s, int on)
  *
  * @returns non-zero in case of failure, zero on success.
  */
-static gint
+static int
 socket_connect_prepare(struct gnutella_socket *s,
 	host_addr_t addr, guint16 port, enum socket_type type, guint32 flags)
 {
 	static const int enable = 1;
-	gint fd, family;
+	int fd, family;
 
 	socket_check(s);
 
@@ -2596,6 +2633,10 @@ socket_connect_prepare(struct gnutella_socket *s,
 	}
 
 	addr = socket_ipv6_trt_map(addr);
+	if (NET_TYPE_NONE == host_addr_net(addr)) {
+		errno = EINVAL;
+		return -1;
+	}
 	family = host_addr_family(addr);
 	if (-1 == family) {
 		errno = EINVAL;
@@ -2617,7 +2658,7 @@ socket_connect_prepare(struct gnutella_socket *s,
 		}
 
 		if (fd < 0) {
-			gint saved_errno = errno;
+			int saved_errno = errno;
 			g_warning("unable to create a socket (%s)",
 				g_strerror(saved_errno));
 			errno = saved_errno;
@@ -2674,12 +2715,12 @@ socket_connect_prepare(struct gnutella_socket *s,
  *
  * @returns non-zero in case of failure, zero on success.
  */
-static gint
+static int
 socket_connect_finalize(struct gnutella_socket *s, const host_addr_t ha)
 {
 	socket_addr_t addr;
 	socklen_t addr_len;
-	gint res;
+	int res;
 
 	socket_check(s);
 
@@ -2832,7 +2873,7 @@ socket_connect_by_name_helper(const host_addr_t *addrs, size_t n,
 		goto finish;
 	}
 
-	addr = addrs[random_raw() % n];
+	addr = addrs[random_u32() % n];
 	can_tls = 0 != (SOCK_F_TLS & s->flags) || tls_cache_lookup(addr, s->port);
 
 	if (
@@ -2874,7 +2915,7 @@ finish:
  * resolved through async DNS calls.
  */
 struct gnutella_socket *
-socket_connect_by_name(const gchar *host, guint16 port,
+socket_connect_by_name(const char *host, guint16 port,
 	enum socket_type type, guint32 flags)
 {
 	struct gnutella_socket *s;
@@ -2921,12 +2962,12 @@ socket_connect_by_name(const gchar *host, guint16 port,
  *
  * @return The new file descriptor of socket or -1 on failure.
  */
-static gint
+static int
 socket_create_and_bind(const host_addr_t bind_addr,
 	const guint16 port, const int type)
 {
 	gboolean socket_failed;
-	gint fd, saved_errno, family;
+	int fd, saved_errno, family;
 
 	g_assert(SOCK_DGRAM == type || SOCK_STREAM == type);
 
@@ -2979,7 +3020,7 @@ socket_create_and_bind(const host_addr_t bind_addr,
 
 #if defined(HAS_SOCKER_GET)
 	if (fd < 0 && (EACCES == saved_errno || EPERM == saved_errno)) {
-		gchar addr_str[128];
+		char addr_str[128];
 
 		host_addr_to_string_buf(bind_addr, addr_str, sizeof addr_str);
 		fd = socker_get(family, type, 0, addr_str, port);
@@ -2990,14 +3031,14 @@ socket_create_and_bind(const host_addr_t bind_addr,
 #endif /* HAS_SOCKER_GET */
 
 	if (fd < 0) {
-		const gchar *type_str = SOCK_DGRAM == type ? "datagram" : "stream";
-		const gchar *net_str = net_type_to_string(host_addr_net(bind_addr));
+		const char *type_str = SOCK_DGRAM == type ? "datagram" : "stream";
+		const char *net_str = net_type_to_string(host_addr_net(bind_addr));
 
 		if (socket_failed) {
 			g_warning("Unable to create the %s (%s) socket (%s)",
 				type_str, net_str, g_strerror(errno));
 		} else {
-			gchar bind_addr_str[HOST_ADDR_PORT_BUFLEN];
+			char bind_addr_str[HOST_ADDR_PORT_BUFLEN];
 
 			host_addr_port_to_string_buf(bind_addr, port,
 				bind_addr_str, sizeof bind_addr_str);
@@ -3055,7 +3096,7 @@ socket_is_local(const struct gnutella_socket *s)
  * resource of `type'.
  */
 struct gnutella_socket *
-socket_local_listen(const gchar *pathname)
+socket_local_listen(const char *pathname)
 {
 	struct sockaddr_un addr;
 	struct gnutella_socket *s;
@@ -3428,7 +3469,7 @@ socket_set_intern(int fd, int option, unsigned size,
  * If `shrink' is false, refuse to shrink the buffer if its size is larger.
  */
 void
-socket_send_buf(struct gnutella_socket *s, gint size, gboolean shrink)
+socket_send_buf(struct gnutella_socket *s, int size, gboolean shrink)
 {
 	socket_check(s);
 	g_return_if_fail(!(s->flags & SOCK_F_SHUTDOWN));
@@ -3440,7 +3481,7 @@ socket_send_buf(struct gnutella_socket *s, gint size, gboolean shrink)
  * If `shrink' is false, refuse to shrink the buffer if its size is larger.
  */
 void
-socket_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink)
+socket_recv_buf(struct gnutella_socket *s, int size, gboolean shrink)
 {
 	socket_check(s);
 	g_return_if_fail(!(s->flags & SOCK_F_SHUTDOWN));
@@ -3576,7 +3617,7 @@ socket_plain_sendto(
 			socket_addr_get_const_sockaddr(&addr), len);
 
 	if ((ssize_t) -1 == ret && GNET_PROPERTY(udp_debug)) {
-		gint e = errno;
+		int e = errno;
 
 		g_warning("sendto() failed: %s", g_strerror(e));
 		errno = e;
@@ -3690,13 +3731,13 @@ socket_wio_link(struct gnutella_socket *s)
  * MAX_IOV_COUNT entries at a time.
  */
 ssize_t
-safe_readv(wrap_io_t *wio, struct iovec *iov, gint iovcnt)
+safe_readv(wrap_io_t *wio, struct iovec *iov, int iovcnt)
 {
 	size_t got = 0;
 	struct iovec *end = iov + iovcnt;
 	struct iovec *siov;
-	gint siovcnt = MAX_IOV_COUNT;
-	gint iovgot = 0;
+	int siovcnt = MAX_IOV_COUNT;
+	int iovgot = 0;
 
 	for (siov = iov; siov < end; siov += siovcnt) {
 		ssize_t r;
@@ -3740,13 +3781,13 @@ safe_readv(wrap_io_t *wio, struct iovec *iov, gint iovcnt)
  * MAX_IOV_COUNT entries at a time.
  */
 ssize_t
-safe_readv_fd(gint fd, struct iovec *iov, gint iovcnt)
+safe_readv_fd(int fd, struct iovec *iov, int iovcnt)
 {
 	size_t got = 0;
 	struct iovec *end = iov + iovcnt;
 	struct iovec *siov;
-	gint siovcnt = MAX_IOV_COUNT;
-	gint iovgot = 0;
+	int siovcnt = MAX_IOV_COUNT;
+	int iovgot = 0;
 
 	for (siov = iov; siov < end; siov += siovcnt) {
 		ssize_t r;
@@ -3790,11 +3831,11 @@ safe_readv_fd(gint fd, struct iovec *iov, gint iovcnt)
  * MAX_IOV_COUNT entries at a time.
  */
 ssize_t
-safe_writev(wrap_io_t *wio, const struct iovec *iov, gint iovcnt)
+safe_writev(wrap_io_t *wio, const struct iovec *iov, int iovcnt)
 {
 	const struct iovec *siov, *end = &iov[iovcnt];
-	gint siovcnt = MAX_IOV_COUNT;
-	gint iovsent = 0;
+	int siovcnt = MAX_IOV_COUNT;
+	int iovsent = 0;
 	size_t sent = 0;
 
 	for (siov = iov; siov < end; siov += siovcnt) {
@@ -3838,11 +3879,11 @@ safe_writev(wrap_io_t *wio, const struct iovec *iov, gint iovcnt)
  * MAX_IOV_COUNT entries at a time.
  */
 ssize_t
-safe_writev_fd(gint fd, const struct iovec *iov, gint iovcnt)
+safe_writev_fd(int fd, const struct iovec *iov, int iovcnt)
 {
 	const struct iovec *siov, *end = &iov[iovcnt];
-	gint siovcnt = MAX_IOV_COUNT;
-	gint iovsent = 0;
+	int siovcnt = MAX_IOV_COUNT;
+	int iovsent = 0;
 	size_t sent = 0;
 
 	for (siov = iov; siov < end; siov += siovcnt) {
