@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.h 12304 2006-11-11 08:16:33Z cbiere $
+ * $Id: sockets.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -101,18 +101,16 @@ typedef enum {
 
 struct gnutella_socket {
 	socket_magic_t magic;	/**< magic for consistency checks */
-	gint file_desc;			/**< file descriptor */
+	int file_desc;			/**< file descriptor */
 	guint32 flags;			/**< operating flags */
 	guint gdk_tag;			/**< gdk tag */
 
 	enum socket_direction direction;
 	enum socket_type type;
 	enum net_type net;
-	gboolean omit_token;	/**< TRUE if the connection needs no token */
-	gboolean corked;
-	gboolean was_shutdown;	/**< Set if shutdown() was used */
-	gint adns;				/**< status of ADNS resolution */
-	const gchar *adns_msg;	/**< ADNS error message */
+
+	int adns;				/**< status of ADNS resolution */
+	const char *adns_msg;	/**< ADNS error message */
 
 	host_addr_t addr;		/**< IP   of our partner */
 	guint16 port;			/**< Port of our partner */
@@ -123,9 +121,7 @@ struct gnutella_socket {
 
 	struct wrap_io wio;		/**< Wrapped IO object */
 
-#ifdef HAS_GNUTLS
 	struct socket_tls_ctx tls;
-#endif /* HAS_GNUTLS */
 
 	union {
 		struct gnutella_node *node;
@@ -140,7 +136,7 @@ struct gnutella_socket {
 
 	size_t pos;			/**< write position in the buffer */
 	size_t buf_size;	/**< write position in the buffer */
-	gchar *buf;			/**< buffer to put in the data read */
+	char *buf;			/**< buffer to put in the data read */
 };
 
 static inline void
@@ -167,34 +163,20 @@ extern struct gnutella_socket *s_local_listen;
 
 static inline gboolean
 socket_with_tls(const struct gnutella_socket *s)
-#ifdef HAS_GNUTLS
 {
 	return s->tls.enabled && s->tls.stage >= SOCK_TLS_INITIALIZED;
 }
-#else
-{
-	(void) s;
-	return FALSE;
-}
-#endif	/* HAS_GNUTLS */
 
 static inline gboolean
 socket_uses_tls(const struct gnutella_socket *s)
-#ifdef HAS_GNUTLS
 {
-	return s->tls.enabled && s->tls.stage >= SOCK_TLS_ESTABLISHED;
+	return s->tls.enabled && s->tls.stage == SOCK_TLS_ESTABLISHED;
 }
-#else
-{
-	(void) s;
-	return FALSE;
-}
-#endif	/* HAS_GNUTLS */
 
 static inline gboolean
-sock_is_corked(const struct gnutella_socket *s)
+socket_is_corked(const struct gnutella_socket *s)
 {
-	return s->corked;
+	return 0 != (SOCK_F_CORKED & s->flags);
 }
 
 /**
@@ -228,38 +210,39 @@ void socket_free_null(struct gnutella_socket **s_ptr);
 struct gnutella_socket *socket_connect(const host_addr_t, guint16,
 		enum socket_type, guint32 flags);
 struct gnutella_socket *socket_connect_by_name(
-	const gchar *host, guint16, enum socket_type, guint32 flags);
+	const char *host, guint16, enum socket_type, guint32 flags);
 struct gnutella_socket *socket_tcp_listen(const host_addr_t, guint16);
 struct gnutella_socket *socket_udp_listen(const host_addr_t, guint16);
-struct gnutella_socket *socket_local_listen(const gchar *pathname);
+struct gnutella_socket *socket_local_listen(const char *pathname);
 
 void socket_evt_set(struct gnutella_socket *s,
 	inputevt_cond_t cond, inputevt_handler_t handler, gpointer data);
 void socket_evt_clear(struct gnutella_socket *s);
 
-void sock_cork(struct gnutella_socket *s, gboolean on);
-void sock_send_buf(struct gnutella_socket *s, gint size, gboolean shrink);
-void sock_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink);
-void sock_nodelay(struct gnutella_socket *s, gboolean on);
-void sock_tx_shutdown(struct gnutella_socket *s);
+void socket_cork(struct gnutella_socket *s, gboolean on);
+void socket_send_buf(struct gnutella_socket *s, int size, gboolean shrink);
+void socket_recv_buf(struct gnutella_socket *s, int size, gboolean shrink);
+void socket_nodelay(struct gnutella_socket *s, gboolean on);
+void socket_tx_shutdown(struct gnutella_socket *s);
 void socket_tos_default(const struct gnutella_socket *s);
 void socket_tos_throughput(const struct gnutella_socket *s);
 void socket_tos_lowdelay(const struct gnutella_socket *s);
 void socket_tos_normal(const struct gnutella_socket *s);
+void socket_set_quickack(struct gnutella_socket *s, int val);
 gboolean socket_bad_hostname(struct gnutella_socket *s);
 void socket_disable_token(struct gnutella_socket *s);
 gboolean socket_omit_token(struct gnutella_socket *s);
 void socket_set_bind_address(const host_addr_t addr);
-gint socket_evt_fd(struct gnutella_socket *s);
+int socket_evt_fd(struct gnutella_socket *s);
 gboolean socket_is_local(const struct gnutella_socket *s);
 
 void socket_timer(time_t now);
 void socket_shutdown(void);
 
-ssize_t safe_readv(wrap_io_t *wio, struct iovec *iov, gint iovcnt);
-ssize_t safe_readv_fd(gint fd, struct iovec *iov, gint iovcnt);
-ssize_t safe_writev(wrap_io_t *wio, const struct iovec *iov, gint iovcnt);
-ssize_t safe_writev_fd(gint fd, const struct iovec *iov, gint iovcnt);
+ssize_t safe_readv(wrap_io_t *wio, struct iovec *iov, int iovcnt);
+ssize_t safe_readv_fd(int fd, struct iovec *iov, int iovcnt);
+ssize_t safe_writev(wrap_io_t *wio, const struct iovec *iov, int iovcnt);
+ssize_t safe_writev_fd(int fd, const struct iovec *iov, int iovcnt);
 
 #endif /* _core_sockets_h_ */
 
