@@ -1,5 +1,5 @@
 /*
- * $Id: spam.h 13599 2007-05-11 18:15:46Z cbiere $
+ * $Id: spam.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2003, Markus Goetz & Raphael Manfredi
  *
@@ -42,7 +42,7 @@
 #include "common.h"
 #include "spam_sha1.h"
 
-gboolean spam_check_filename_and_size(const gchar *filename, filesize_t size);
+gboolean spam_check_filename_and_size(const char *filename, filesize_t size);
 void spam_init(void);
 void spam_close(void);
 
