@@ -1,5 +1,5 @@
 /*
- * $Id: thex_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: thex_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  * Copyright (c) 2007, Christian Biere
@@ -37,8 +37,9 @@
 
 #include "common.h"
 
-RCSID("$Id: thex_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: thex_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
+#include "bsched.h"
 #include "dime.h"
 #include "share.h"
 #include "special_upload.h"
@@ -49,12 +50,12 @@ RCSID("$Id: thex_upload.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
 #include "tx_chunk.h"
 #include "tx_link.h"
 
-#include "if/core/hosts.h"
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
 #include "lib/atoms.h"
 #include "lib/array.h"
+#include "lib/gnet_host.h"
 #include "lib/tigertree.h"
 #include "lib/misc.h"
 #include "lib/walloc.h"
@@ -403,7 +404,7 @@ thex_upload_open(
 	special_upload_writable_t writable,
 	const struct tx_link_cb *link_cb,
 	struct wrap_io *wio,
-	gint flags)
+	int flags)
 {
 	struct thex_upload *ctx;
 
@@ -429,7 +430,7 @@ thex_upload_open(
 
 		args.cb = link_cb;
 		args.wio = wio;
-		args.bws = BSCHED_BWS_OUT;
+		args.bws = bsched_out_select_by_addr(gnet_host_get_addr(host));
 
 		ctx->tx = tx_make(owner, host, tx_link_get_ops(), &args);
 	}
