@@ -1,5 +1,5 @@
 /*
- * $Id: tls_common.c 13899 2007-06-20 21:49:39Z cbiere $
+ * $Id: tls_common.c 15053 2007-12-08 01:15:45Z cbiere $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -32,10 +32,11 @@
 
 #include "common.h"
 
-RCSID("$Id: tls_common.c 13899 2007-06-20 21:49:39Z cbiere $")
+RCSID("$Id: tls_common.c 15053 2007-12-08 01:15:45Z cbiere $")
 
 #ifdef HAS_GNUTLS
 #include <gnutls/gnutls.h>
+#include <gnutls/x509.h>
 #endif /* HAS_GNUTLS */
 
 #include "tls_common.h"
@@ -45,6 +46,7 @@ RCSID("$Id: tls_common.c 13899 2007-06-20 21:49:39Z cbiere $")
 #include "if/gnet_property_priv.h"
 #include "if/core/settings.h"
 
+#include "lib/array.h"
 #include "lib/header.h"
 #include "lib/misc.h"
 #include "lib/walloc.h"
@@ -70,6 +72,14 @@ tls_socket_get_session(struct gnutella_socket *s)
 	return s->tls.ctx->session;
 }
 
+static inline size_t
+tls_adjust_send_size(struct gnutella_socket *s, size_t size)
+{
+	size_t max_size = gnutls_record_get_max_size(tls_socket_get_session(s));
+	g_assert(max_size > 0);
+	return MIN(size, max_size);
+}
+
 #ifdef XXX_CUSTOM_PUSH_PULL
 static inline void
 tls_transport_debug(const char *op, int fd, size_t size, ssize_t ret)
@@ -126,7 +136,9 @@ tls_socket_evt_change(struct gnutella_socket *s, inputevt_cond_t cond)
 	g_assert(s);
 	g_assert(socket_with_tls(s));	/* No USES yet, may not have handshaked */
 	g_assert(INPUT_EVENT_EXCEPTION != cond);
-	g_assert(0 != s->gdk_tag);
+
+	if (0 == s->gdk_tag)
+		return;
 
 	if (cond != s->tls.cb_cond) {
 		int saved_errno = errno;
@@ -256,7 +268,8 @@ tls_handshake(struct gnutella_socket *s)
 		if (GNET_PROPERTY(tls_debug)) {
 			g_message("TLS handshake succeeded");
 		}
-		tls_socket_evt_change(s, INPUT_EVENT_W);
+		tls_socket_evt_change(s, SOCK_CONN_INCOMING == s->direction
+									? INPUT_EVENT_R : INPUT_EVENT_W);
 		if (GNET_PROPERTY(tls_debug)) {
 			tls_print_session_info(s->addr, s->port, session);
 		}
@@ -282,7 +295,7 @@ tls_handshake(struct gnutella_socket *s)
 		}
 		break;
 	default:
-		if (do_warn) {
+		if (do_warn && GNET_PROPERTY(tls_debug)) {
 			g_warning("gnutls_handshake() failed: host=%s (%s) error=\"%s\"",
 				host_addr_port_to_string(s->addr, s->port),
 				SOCK_CONN_INCOMING == s->direction ? "incoming" : "outgoing",
@@ -296,10 +309,10 @@ tls_handshake(struct gnutella_socket *s)
  * Initiates a new TLS session.
  *
  * @param is_incoming Whether this is an incoming connection.
- * @return The session pointer on success; NULL on failure.
+ * @return 0 on success, -1 on error.
  */
-tls_context_t
-tls_init(const struct gnutella_socket *s)
+int
+tls_init(struct gnutella_socket *s)
 {
 	static const int cipher_list[] = {
 		GNUTLS_CIPHER_AES_256_CBC,
@@ -340,19 +353,20 @@ tls_init(const struct gnutella_socket *s)
 
 	ctx = walloc0(sizeof *ctx);
 	ctx->s = s;
+	s->tls.ctx = ctx;
 
 	if (SOCK_CONN_INCOMING == s->direction) {
 
+		if (gnutls_init(&ctx->session, GNUTLS_SERVER)) {
+			g_warning("gnutls_init() failed");
+			ctx->session = NULL;
+			goto failure;
+		}
 		if (gnutls_anon_allocate_server_credentials(&ctx->server_cred)) {
 			g_warning("gnutls_anon_allocate_server_credentials() failed");
 			goto failure;
 		}
 		gnutls_anon_set_server_dh_params(ctx->server_cred, get_dh_params());
-
-		if (gnutls_init(&ctx->session, GNUTLS_SERVER)) {
-			g_warning("gnutls_init() failed");
-			goto failure;
-		}
 		gnutls_dh_set_prime_bits(ctx->session, TLS_DH_BITS);
 
 		if (gnutls_credentials_set(ctx->session,
@@ -360,7 +374,6 @@ tls_init(const struct gnutella_socket *s)
 			g_warning("gnutls_credentials_set() failed");
 			goto failure;
 		}
-
 		if (server_cert_cred) {
 			if (gnutls_credentials_set(ctx->session,
 					GNUTLS_CRD_CERTIFICATE, server_cert_cred)) {
@@ -369,12 +382,13 @@ tls_init(const struct gnutella_socket *s)
 			}
 		}
 	} else {
-		if (gnutls_anon_allocate_client_credentials(&ctx->client_cred)) {
-			g_warning("gnutls_anon_allocate_client_credentials() failed");
-			goto failure;
-		}
 		if (gnutls_init(&ctx->session, GNUTLS_CLIENT)) {
 			g_warning("gnutls_init() failed");
+			ctx->session = NULL;
+			goto failure;
+		}
+		if (gnutls_anon_allocate_client_credentials(&ctx->client_cred)) {
+			g_warning("gnutls_anon_allocate_client_credentials() failed");
 			goto failure;
 		}
 		if (gnutls_credentials_set(ctx->session,
@@ -410,58 +424,21 @@ tls_init(const struct gnutella_socket *s)
 	gnutls_transport_set_push_function(ctx->session, tls_push);
 	gnutls_transport_set_pull_function(ctx->session, tls_pull);
 #endif /* XXX_CUSTOM_PUSH_PULL */
-	return ctx;
+
+	return 0;
 
 failure:
-	tls_free(&ctx);
-	return NULL;
+	tls_free(s);
+	return -1;
 }
 
 void
-tls_bye(tls_context_t ctx, gboolean is_incoming)
-{
-	int ret;
-	
-	g_return_if_fail(ctx);
-	g_return_if_fail(ctx->session);
-
-	ret = gnutls_bye(ctx->session,
-			is_incoming ? GNUTLS_SHUT_WR : GNUTLS_SHUT_RDWR);
-	if (ret < 0) {
-		switch (ret) {
-		case GNUTLS_E_INTERRUPTED:
-		case GNUTLS_E_AGAIN:
-			break;
-		case GNUTLS_E_PULL_ERROR:
-		case GNUTLS_E_PUSH_ERROR:
-			if (GNET_PROPERTY(tls_debug)) {
-				switch (errno) {
-				case EPIPE:
-				case ECONNRESET:
-					if (GNET_PROPERTY(tls_debug) < 2)
-						break;
-				default:
-					g_message("gnutls_bye() failed: host=%s errno=\"%s\"",
-						host_addr_port_to_string(ctx->s->addr, ctx->s->port),
-						g_strerror(errno));
-				}
-			}
-			break;
-		default:
-			g_warning("gnutls_bye() failed: host=%s error=\"%s\"",
-				host_addr_port_to_string(ctx->s->addr, ctx->s->port),
-				gnutls_strerror(ret));
-		}
-	}
-}
-
-void
-tls_free(tls_context_t *ctx_ptr)
+tls_free(struct gnutella_socket *s)
 {
 	tls_context_t ctx;
 
-	g_assert(ctx_ptr);
-	ctx = *ctx_ptr;
+	socket_check(s);
+	ctx = s->tls.ctx;
 	if (ctx) {
 		if (ctx->session) {
 			gnutls_deinit(ctx->session);
@@ -475,7 +452,7 @@ tls_free(tls_context_t *ctx_ptr)
 			ctx->client_cred = NULL;
 		}
 		wfree(ctx, sizeof *ctx);
-		*ctx_ptr = NULL;
+		s->tls.ctx = NULL;
 	}
 }
 
@@ -523,40 +500,23 @@ tls_global_init(void)
 }
 
 static ssize_t
-tls_write(struct wrap_io *wio, gconstpointer buf, size_t size)
+tls_write_intern(struct wrap_io *wio, gconstpointer buf, size_t size)
 {
-	inputevt_cond_t cond = 0;
 	struct gnutella_socket *s = wio->ctx;
-	const char *p;
-	size_t len;
 	ssize_t ret;
 
-	g_assert(size <= INT_MAX);
-	g_assert(s != NULL);
-	g_assert(buf != NULL);
+	g_assert((0 == s->tls.snarf) ^ (NULL == buf));
+	g_assert((0 == s->tls.snarf) ^ (0 == size));
 
-	g_assert(socket_uses_tls(s));
-
-	if (0 != s->tls.snarf) {
-		p = NULL;
-		len = 0;
-	} else {
-		p = buf;
-		len = size;
-		g_assert(NULL != p && len > 0);
-	}
-
-	ret = gnutls_record_send(tls_socket_get_session(s), p, len);
+	size = tls_adjust_send_size(s, size);
+	ret = gnutls_record_send(tls_socket_get_session(s), buf, size);
 	if (ret < 0) {
 		switch (ret) {
 		case GNUTLS_E_INTERRUPTED:
 		case GNUTLS_E_AGAIN:
-			cond = gnutls_record_get_direction(tls_socket_get_session(s))
-					? INPUT_EVENT_WX : INPUT_EVENT_RX;
-
 			if (0 == s->tls.snarf) {
-				s->tls.snarf = len;
-				ret = len;
+				s->tls.snarf = size;
+				ret = size;
 			} else {
 				errno = VAL_EAGAIN;
 				ret = -1;
@@ -572,28 +532,81 @@ tls_write(struct wrap_io *wio, gconstpointer buf, size_t size)
 			}
 			errno = EIO;
 			ret = -1;
-			break;
+			goto finish;
+
 		default:
 			if (GNET_PROPERTY(tls_debug)) {
 				g_warning("tls_write(): gnutls_record_send() failed: "
-					"host=%s error=\"%s\"",
+					"host=%s snarf=%lu error=\"%s\"",
 					host_addr_port_to_string(s->addr, s->port),
+					(unsigned long) s->tls.snarf,
 					gnutls_strerror(ret));
 			}
 			errno = EIO;
 			ret = -1;
+			goto finish;
 		}
 	} else {
-		if (0 != s->tls.snarf) {
+
+		if (s->tls.snarf) {
+			g_assert(s->tls.snarf >= (size_t) ret);
 			s->tls.snarf -= ret;
 			errno = VAL_EAGAIN;
 			ret = -1;
+			goto finish;
 		}
 	}
 
-	if (s->gdk_tag && cond)
-		tls_socket_evt_change(s, cond);
+	if (s->tls.snarf) {
+		tls_socket_evt_change(s, INPUT_EVENT_WX);
+	}
 
+finish:
+	g_assert(ret == (ssize_t) -1 || (size_t) ret <= size);
+	return ret;
+}
+
+static int
+tls_flush(struct wrap_io *wio)
+{
+	struct gnutella_socket *s = wio->ctx;
+
+	socket_check(s);
+
+	if (s->tls.snarf) {
+		if (GNET_PROPERTY(tls_debug)) {
+			g_message("tls_flush: snarf=%lu host=%s",
+					(gulong) s->tls.snarf,
+					host_addr_port_to_string(s->addr, s->port));
+		}
+		(void ) tls_write_intern(wio, NULL, 0);
+		if (s->tls.snarf)
+			return -1;
+	}
+	return 0;
+}
+
+
+static ssize_t
+tls_write(struct wrap_io *wio, gconstpointer buf, size_t size)
+{
+	struct gnutella_socket *s = wio->ctx;
+	ssize_t ret;
+
+	g_assert(size > 0);
+	g_assert(size <= INT_MAX);
+	g_assert(s != NULL);
+	g_assert(buf != NULL);
+
+	g_assert(socket_uses_tls(s));
+
+	ret = tls_flush(wio);
+	if (0 == ret) {
+		ret = tls_write_intern(wio, buf, size);
+		if (s->gdk_tag) {
+			tls_socket_evt_change(s, INPUT_EVENT_WX);
+		}
+	}
 	g_assert(ret == (ssize_t) -1 || (size_t) ret <= size);
 	return ret;
 }
@@ -601,7 +614,6 @@ tls_write(struct wrap_io *wio, gconstpointer buf, size_t size)
 static ssize_t
 tls_read(struct wrap_io *wio, gpointer buf, size_t size)
 {
-	inputevt_cond_t cond = 0;
 	struct gnutella_socket *s = wio->ctx;
 	ssize_t ret;
 
@@ -611,11 +623,11 @@ tls_read(struct wrap_io *wio, gpointer buf, size_t size)
 
 	g_assert(socket_uses_tls(s));
 
-	if (s->wio.flush(&s->wio) < 0) {
-		if (!is_temporary_error(errno)) {
+	if (tls_flush(wio) && !is_temporary_error(errno)) {
+		if (GNET_PROPERTY(tls_debug)) {
 			g_warning("tls_read: flush error: %s", g_strerror(errno));
-			return -1;
 		}
+		return -1;
 	}
 
 	ret = gnutls_record_recv(tls_socket_get_session(s), buf, size);
@@ -623,8 +635,6 @@ tls_read(struct wrap_io *wio, gpointer buf, size_t size)
 		switch (ret) {
 		case GNUTLS_E_INTERRUPTED:
 		case GNUTLS_E_AGAIN:
-			cond = gnutls_record_get_direction(tls_socket_get_session(s))
-					? INPUT_EVENT_WX : INPUT_EVENT_RX;
 			errno = VAL_EAGAIN;
 			break;
 		case GNUTLS_E_PULL_ERROR:
@@ -649,225 +659,62 @@ tls_read(struct wrap_io *wio, gpointer buf, size_t size)
 		ret = -1;
 	}
 
-	if (s->gdk_tag && cond)
-		tls_socket_evt_change(s, cond);
-
+	if (s->gdk_tag && 0 == s->tls.snarf) {
+		tls_socket_evt_change(s, INPUT_EVENT_RX);
+	}
 	g_assert(ret == (ssize_t) -1 || (size_t) ret <= size);
 	return ret;
 }
 
-static int
-tls_flush(struct wrap_io *wio)
-{
-	struct gnutella_socket *s = wio->ctx;
-	ssize_t ret;
-
-	socket_check(s);
-
-	if (0 == s->tls.snarf)
-		return 0;
-
-	if (GNET_PROPERTY(tls_debug)) {
-		g_message("tls_flush: snarf=%lu", (gulong) s->tls.snarf);
-	}
-	ret = tls_write(wio, "", 0);
-	g_assert((ssize_t)-1 == ret);
-	return (s->tls.snarf > 0 || VAL_EAGAIN != errno) ? -1 : 0;
-}
-
 static ssize_t
 tls_writev(struct wrap_io *wio, const struct iovec *iov, int iovcnt)
 {
-	inputevt_cond_t cond = 0;
 	struct gnutella_socket *s = wio->ctx;
-	ssize_t ret, written;
+	ssize_t ret, done;
 	int i;
 
 	g_assert(socket_uses_tls(s));
 	g_assert(iovcnt > 0);
 
-	if (0 != s->tls.snarf) {
-		ret = gnutls_record_send(tls_socket_get_session(s), NULL, 0);
-		if (ret > 0) {
-			g_assert((ssize_t) s->tls.snarf >= ret);
-			s->tls.snarf -= ret;
-			if (0 != s->tls.snarf) {
-				errno = VAL_EAGAIN;
-				ret = -1;
-				goto done;
-			}
-		} else {
-			switch (ret) {
-			case 0:
-				ret = 0;
-				goto done;
-			case GNUTLS_E_INTERRUPTED:
-			case GNUTLS_E_AGAIN:
-				cond = gnutls_record_get_direction(tls_socket_get_session(s))
-						? INPUT_EVENT_WX : INPUT_EVENT_RX;
-				errno = VAL_EAGAIN;
-				break;
-			case GNUTLS_E_PULL_ERROR:
-			case GNUTLS_E_PUSH_ERROR:
-				if (GNET_PROPERTY(tls_debug)) {
-					g_message("tls_writev() failed: "
-						"host=%s errno=\"%s\"",
-						host_addr_port_to_string(s->addr, s->port),
-						g_strerror(errno));
-				}
-				errno = EIO;
-				break;
-			default:
-				if (GNET_PROPERTY(tls_debug)) {
-					g_warning("tls_writev(): gnutls_record_send() failed: "
-						"host=%s error=\"%s\"",
-						host_addr_port_to_string(s->addr, s->port),
-						gnutls_strerror(ret));
-				}
-				errno = EIO;
-			}
-			ret = -1;
-			goto done;
-		}
-	}
+	done = 0;
+	ret = 0;
+	for (i = 0; i < iovcnt; i++) {
+		const size_t size = iov[i].iov_len;
 
-	ret = -2;	/* Shut the compiler: iovcnt could still be 0 */
-	written = 0;
-	for (i = 0; i < iovcnt; ++i) {
-		char *p;
-		size_t len;
-
-		p = iov[i].iov_base;
-		len = iov[i].iov_len;
-		g_assert(NULL != p && len > 0);
-		ret = gnutls_record_send(tls_socket_get_session(s), p, len);
-		if (ret < 0) {
-			switch (ret) {
-			case GNUTLS_E_INTERRUPTED:
-			case GNUTLS_E_AGAIN:
-				cond = gnutls_record_get_direction(tls_socket_get_session(s))
-						? INPUT_EVENT_WX : INPUT_EVENT_RX;
-				s->tls.snarf = len;
-				ret = written + len;
-				break;
-			case GNUTLS_E_PULL_ERROR:
-			case GNUTLS_E_PUSH_ERROR:
-				if (GNET_PROPERTY(tls_debug)) {
-					g_message("tls_writev() failed: "
-						"host=%s errno=\"%s\"",
-						host_addr_port_to_string(s->addr, s->port),
-						g_strerror(errno));
-				}
-				ret = -1;
-				break;
-			default:
-				if (GNET_PROPERTY(tls_debug)) {
-					g_warning("gnutls_record_send() failed: "
-						"host=%s error=\"%s\"",
-						host_addr_port_to_string(s->addr, s->port),
-						gnutls_strerror(ret));
-				}
-				errno = EIO;
-				ret = -1;
-			}
+		ret = tls_write(wio, iov[i].iov_base, size);
+		if ((ssize_t) -1 == ret)
 			break;
-		} else if (0 == ret) {
-			ret = written;
+		done += (size_t) ret;
+		if (size != (size_t) ret)
 			break;
-		} else {
-			written += ret;
-			ret = written;
-		}
 	}
-
-done:
-	if (s->gdk_tag && cond)
-		tls_socket_evt_change(s, cond);
-
-	g_assert((ssize_t) -1 == ret || ret >= 0);
-	return ret;
+	return done > 0 ? done : ret;
 }
 
 static ssize_t
 tls_readv(struct wrap_io *wio, struct iovec *iov, int iovcnt)
 {
-	inputevt_cond_t cond = 0;
 	struct gnutella_socket *s = wio->ctx;
-	size_t rcvd = 0;
-	ssize_t ret;
+	ssize_t ret, done;
 	int i;
 
 	g_assert(socket_uses_tls(s));
 	g_assert(iovcnt > 0);
 
-	if (s->wio.flush(&s->wio) < 0) {
-		if (!is_temporary_error(errno)) {
-			g_warning("tls_read: flush error: %s", g_strerror(errno));
-			return -1;
-		}
+	done = 0;
+	ret = 0;
+	for (i = 0; i < iovcnt; i++) {
+		const size_t size = iov[i].iov_len;
+
+		ret = tls_read(wio, iov[i].iov_base, size);
+		if ((ssize_t) -1 == ret)
+			break;
+		done += (size_t) ret;
+		if (size != (size_t) ret)
+			break;
 	}
 
-	ret = 0;	/* Shut the compiler: iovcnt could still be 0 */
-	for (i = 0; i < iovcnt; ++i) {
-		size_t len;
-		char *p;
-
-		p = iov[i].iov_base;
-		len = iov[i].iov_len;
-		g_assert(NULL != p && len > 0);
-		ret = gnutls_record_recv(tls_socket_get_session(s), p, len);
-		if (ret <= 0) {
-			break;
-		}
-		rcvd += ret;
-		if ((size_t) ret != len) {
-			break;
-		}
-	}
-
-	if (ret < 0) {
-		switch (ret) {
-		case GNUTLS_E_INTERRUPTED:
-		case GNUTLS_E_AGAIN:
-			cond = gnutls_record_get_direction(tls_socket_get_session(s))
-					? INPUT_EVENT_WX : INPUT_EVENT_RX;
-			if (rcvd > 0) {
-				ret = rcvd;
-			} else {
-				errno = VAL_EAGAIN;
-				ret = -1;
-			}
-			break;
-		case GNUTLS_E_PULL_ERROR:
-		case GNUTLS_E_PUSH_ERROR:
-			if (GNET_PROPERTY(tls_debug)) {
-				g_message("tls_readv(): socket_tls_readv() failed: "
-					"host=%s errno=\"%s\"",
-					host_addr_port_to_string(s->addr, s->port),
-					g_strerror(errno));
-			}
-			errno = EIO;
-			ret = -1;
-			break;
-		default:
-			if (GNET_PROPERTY(tls_debug)) {
-				g_warning("tls_readv(): gnutls_record_recv() failed: "
-					"host=%s error=\"%s\"",
-					host_addr_port_to_string(s->addr, s->port),
-					gnutls_strerror(ret));
-			}
-			errno = EIO;
-			ret = -1;
-		}
-	} else {
-		ret = rcvd;
-	}
-
-	if (s->gdk_tag && cond)
-		tls_socket_evt_change(s, cond);
-
-	g_assert((ssize_t) -1 == ret || ret >= 0);
-	return ret;
+	return done > 0 ? done : ret;
 }
 
 static ssize_t
@@ -883,29 +730,185 @@ tls_no_sendto(struct wrap_io *unused_wio, const gnet_host_t *unused_to,
 }
 
 void
-tls_wio_link(struct wrap_io *wio)
+tls_wio_link(struct gnutella_socket *s)
 {
-	g_assert(wio);	
-	wio->write = tls_write;
-	wio->read = tls_read;
-	wio->writev = tls_writev;
-	wio->readv = tls_readv;
-	wio->sendto = tls_no_sendto;
-	wio->flush = tls_flush;
+	socket_check(s);
+
+	s->wio.write = tls_write;
+	s->wio.read = tls_read;
+	s->wio.writev = tls_writev;
+	s->wio.readv = tls_readv;
+	s->wio.sendto = tls_no_sendto;
+	s->wio.flush = tls_flush;
+}
+
+void
+tls_bye(struct gnutella_socket *s)
+{
+	int ret;
+	
+	socket_check(s);
+	g_return_if_fail(s->tls.ctx);
+	g_return_if_fail(s->tls.ctx->session);
+
+	if ((SOCK_F_EOF | SOCK_F_SHUTDOWN) & s->flags)
+		return;
+
+	if (tls_flush(&s->wio) && GNET_PROPERTY(tls_debug)) {
+		g_warning("tls_bye: tls_flush() failed");
+	}
+
+	ret = gnutls_bye(s->tls.ctx->session,
+			SOCK_CONN_INCOMING != s->direction
+				? GNUTLS_SHUT_WR : GNUTLS_SHUT_RDWR);
+
+	if (ret < 0) {
+		switch (ret) {
+		case GNUTLS_E_INTERRUPTED:
+		case GNUTLS_E_AGAIN:
+			break;
+		case GNUTLS_E_PULL_ERROR:
+		case GNUTLS_E_PUSH_ERROR:
+			if (GNET_PROPERTY(tls_debug)) {
+				switch (errno) {
+				case EPIPE:
+				case ECONNRESET:
+					if (GNET_PROPERTY(tls_debug) < 2)
+						break;
+				default:
+					if (GNET_PROPERTY(tls_debug)) {
+						g_message("gnutls_bye() failed: host=%s errno=\"%s\"",
+							host_addr_port_to_string(s->addr, s->port),
+							g_strerror(errno));
+					}
+				}
+			}
+			break;
+		default:
+			if (GNET_PROPERTY(tls_debug)) {
+				g_warning("gnutls_bye() failed: host=%s error=\"%s\"",
+					host_addr_port_to_string(s->addr, s->port),
+					gnutls_strerror(ret));
+			}
+		}
+	}
 }
 
 const char *
 tls_version_string(void)
 {
 	static char buf[128];
+
 	if ('\0' == buf[0]) {
-		concat_strings(buf, sizeof buf, "GNU TLS ", gnutls_check_version(NULL),
-			" (compiled against ", LIBGNUTLS_VERSION, ")",
+		const char *current = gnutls_check_version(NULL);
+		int differ = strcmp(current, LIBGNUTLS_VERSION);
+
+		concat_strings(buf, sizeof buf, "GNU TLS ", current,
+			differ ? " (compiled against " : "",
+			differ ? LIBGNUTLS_VERSION : "",
+			differ ? ")" : "",
 			(void *) 0);
 	}
 	return buf;
 }
 
+gboolean
+tls_enabled(void)
+{
+	return TRUE;
+}
+
+static gnutls_x509_crt
+svn_release_notify_certificate(void)
+{
+	static const char certificate[] =
+"-----BEGIN CERTIFICATE-----\n"
+"MIIBKTCB1qADAgECAgEAMAsGCSqGSIb3DQEBBTAAMB4XDTA3MDgyNTA0MjIxMVoX\n"
+"DTA4MDgyNDA0MjIxNVowADBZMAsGCSqGSIb3DQEBAQNKADBHAkCpadMxWZWWzcV7\n"
+"Mu66wzBuQ8AkanGspm7ImdRKOlo55V3uBlSob9N/GFlzZ9kG6kS169wgdK2vNQwR\n"
+"5jOMeIMbAgMBAAGjQDA+MAwGA1UdEwEB/wQCMAAwDwYDVR0PAQH/BAUDAweAADAd\n"
+"BgNVHQ4EFgQU8pP/Zgh/K6N0zVHMEs2VIWZNjUIwCwYJKoZIhvcNAQEFA0EAO6ld\n"
+"1NFx0QRBCHE+BUaCX3tuRC0a7HRq8UEqhcKgW7Xk3nkGUNXTcSSo7wu+jpePUsw8\n"
+"njFhJCXeDIcR7jzNCA==\n"
+"-----END CERTIFICATE-----\n";
+	static gboolean initialized;
+	static gnutls_x509_crt cert;
+
+	if (!initialized) {
+		gnutls_datum cert_data;
+		int error;
+
+		initialized = TRUE;
+		error = gnutls_x509_crt_init(&cert);
+		if (error) {
+			g_warning("gnutls_x509_crt_init() failed: %s",
+					gnutls_strerror(error));
+			cert = NULL;
+			return NULL;
+		}
+
+		cert_data.data = (void *) certificate;
+		cert_data.size = CONST_STRLEN(certificate);
+		error = gnutls_x509_crt_import(cert, &cert_data, GNUTLS_X509_FMT_PEM);
+		if (error) {
+			g_warning("gnutls_x509_crt_import() failed: %s",
+					gnutls_strerror(error));
+			gnutls_x509_crt_deinit(cert);
+			cert = NULL;
+			return NULL;
+		}
+	}
+	return cert; 
+}
+
+gboolean
+svn_release_notification_can_verify(void)
+{
+	return NULL != svn_release_notify_certificate();
+}
+
+static gboolean
+verify_signature(gnutls_x509_crt cert,
+	const struct array *input, const struct array *signature)
+{
+	gnutls_datum data, sig;
+
+	g_return_val_if_fail(cert, FALSE);
+	g_return_val_if_fail(input, FALSE);
+	g_return_val_if_fail(signature, FALSE);
+
+	data.data = (void *) input->data;
+	data.size = input->size;
+
+	sig.data = (void *) signature->data;
+	sig.size = signature->size;
+
+	return 1 == gnutls_x509_crt_verify_data(cert, 0, &data, &sig);
+}
+
+/**
+ * Verifies "data" against "signature".
+ *
+ * @return TRUE if the signature matches.
+ */
+gboolean
+svn_release_notification_verify(guint32 revision, time_t date,
+	const struct array *signature)
+{
+	char rev[12], data[64];
+	struct array input;
+
+	uint32_to_string_buf(revision, rev, sizeof rev);
+	input.data = (void *) data;
+	input.size = concat_strings(data, sizeof data,
+					"r", rev,
+					"@", uint32_to_string(date),
+					(void *) 0);
+
+	return verify_signature(svn_release_notify_certificate(),
+				&input, signature);
+}
+
 #else	/* !HAS_GNUTLS*/
 
 enum tls_handshake_result
@@ -915,17 +918,32 @@ tls_handshake(struct gnutella_socket *s)
 	return TLS_HANDSHAKE_FINISHED;
 }
 
-tls_context_t 
-tls_init(const struct gnutella_socket *s)
+int
+tls_init(struct gnutella_socket *s)
 {
-	(void) s;
-	return NULL;
+	socket_check(s);
+	g_assert_not_reached();
+	return -1;
 }
 
 void
-tls_wio_link(struct wrap_io *wio)
+tls_free(struct gnutella_socket *s)
 {
-	(void) wio;
+	socket_check(s);
+	g_assert_not_reached();
+}
+
+void
+tls_bye(struct gnutella_socket *s)
+{
+	socket_check(s);
+	g_assert_not_reached();
+}
+
+void
+tls_wio_link(struct gnutella_socket *s)
+{
+	socket_check(s);
 	g_assert_not_reached();
 }
 
@@ -941,6 +959,28 @@ tls_version_string(void)
 	return NULL;
 }
 
+gboolean
+tls_enabled(void)
+{
+	return FALSE;
+}
+
+gboolean
+svn_release_notification_can_verify(void)
+{
+	return FALSE;
+}
+
+gboolean
+svn_release_notification_verify(guint32 revision, time_t date,
+	const struct array *signature)
+{
+	g_return_val_if_fail(signature, FALSE);
+	(void) revision;
+	(void) date;
+	return FALSE;
+}
+
 #endif	/* HAS_GNUTLS */
 
 /* vi: set ts=4 sw=4 cindent: */
