@@ -1,5 +1,5 @@
 /*
- * $Id: token.c 15162 2008-04-01 22:28:45Z cbiere $
+ * $Id: token.c 16301 2009-03-29 15:31:34Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: token.c 15162 2008-04-01 22:28:45Z cbiere $")
+RCSID("$Id: token.c 16301 2009-03-29 15:31:34Z rmanfredi $")
 
 #include "token.h"
 #include "clock.h"
@@ -59,30 +59,7 @@ RCSID("$Id: token.c 15162 2008-04-01 22:28:45Z cbiere $")
  * There can be up to 2^5 = 32 keys per version.
  */
 
-static const gchar *keys_096b[] = {
-	"bea7 69a5 a647 f605 46b0 d155 2ba6 cee7",
-	"68b0 2cf3 2c1a 8ae0 a72d f5c0 e77b bba8",
-	"856e 4221 1470 a903 193e 2cc9 79a5 5337",
-	"59c3 3f96 fbc0 0397 0356 6500 fc72 41b6",
-	"e0f3 9f6c 16d7 4231 cd00 e991 b511 db07",
-	"e765 cc0e 8672 692c cdc6 3b57 f178 cf59",
-	"3120 1d5a ffc2 4ad8 bd4b bb38 bf99 b026",
-	"8b39 85dd af31 86a5 2e7e 0b95 f030 482b",
-	"5107 a6b7 4013 3439 3dae b5b4 e679 a401",
-	"7d65 9e48 ee7e 7078 286b 29e9 e9be 296a",
-	"e82d 1335 53d0 28c1 3423 7b30 6358 de81",
-	"8b27 3698 03a2 6889 3bdd d095 34b6 0629",
-	"b178 7abb 38cd 1084 f861 f1b2 05ab 28bc",
-	"1253 e83d 6ee1 739e d7fe cb08 0527 3b3b",
-	"13cb 0ec4 7784 2bd3 728a 3cbb 7900 c25c",
-	"77da a447 ea85 ca52 4867 abae c992 aca3",
-	"232d 40d4 2d6f 473c 411a 2beb bb1c b72c",
-	"f62d be65 19a3 63c2 3714 e224 bf31 b565",
-	"34b8 c34b aebb 844e 8080 da67 036b 1fbb",
-	"e824 cbee 3b74 9c99 e808 ac6c 079b 1d16",
-};
-
-static const gchar *keys_096[] = {
+static const char *keys_096[] = {
 	"261c 78d6 fcc5 d96e 2649 061a 4534 29b5",
 	"2629 7de4 8edd 43eb 6c47 2b01 caf1 5e86",
 	"50c2 076a 5a15 5c0c 27fb eda0 381b 2eb7",
@@ -107,7 +84,7 @@ static const gchar *keys_096[] = {
 	"3084 a2bc f4ed 8b3c 2a2b 1834 cd8e 3f8b",
 };
 
-static const gchar *keys_096_1[] = {
+static const char *keys_096_1[] = {
 	"ac3e f7b1 af37 e22c ce69 f25f 8dd2 8e51",
 	"7c66 fddd f8d9 8bde 8c6c 072a 1935 2237",
 	"6aab e420 921d b32c 09e9 34b8 e403 525d",
@@ -132,7 +109,7 @@ static const gchar *keys_096_1[] = {
 	"9536 535d 0e1b e017 2f73 a0bc c01a dcf9",
 };
 
-static const gchar *keys_096_2[] = {
+static const char *keys_096_2[] = {
 	"fa04 15ab 5f1d 52cc 5a5d 865b 27b5 c5a5",
 	"7f0b 5987 86ec a100 85c5 d0fc f504 b58c",
 	"2b06 f3a7 05a8 2769 2679 1ab9 9ba0 2360",
@@ -157,7 +134,7 @@ static const gchar *keys_096_2[] = {
 	"e75d b41f 2332 309d c3ee 4dd7 060d fc08",
 };
 
-static const gchar *keys_096_3[] = {
+static const char *keys_096_3[] = {
 	"ca43 91c3 d080 c596 5ec2 9e17 5011 ebb6",
 	"4b53 6e49 a6a8 c045 c0c4 bfc7 8c54 4d9a",
 	"411b fc41 6643 ba3a 527a 52c9 4a72 3db0",
@@ -182,7 +159,7 @@ static const gchar *keys_096_3[] = {
 	"c994 a380 dde0 baff 8900 c9e5 d94c b9ff",
 };
 
-static const gchar *keys_096_4[] = {
+static const char *keys_096_4[] = {
 	"4313 0c48 3747 a2b5 580c f7a5 7adc d83f",
 	"94c7 eeca e5c1 c4c4 a09d 243c b846 6917",
 	"3188 446a 39da fca8 11ae ef30 8073 f61a",
@@ -207,7 +184,7 @@ static const gchar *keys_096_4[] = {
 	"a2d0 b172 becb 8516 8f17 b2f4 80e2 4488",
 };
 
-static const gchar *keys_096_5[] = {
+static const char *keys_096_5[] = {
 	"29af ab9b fc6b 58b7 9031 cf87 f7cc 3c63",
 	"5dee 3d3f efb1 ac8d 27f4 2fa9 b1c3 705b",
 	"117f 9530 1881 6432 ecf4 780e dbfc 45c9",
@@ -232,19 +209,47 @@ static const gchar *keys_096_5[] = {
 	"1f2f 4d01 aee4 8a3b 04c7 b1a7 7557 1d7e",
 };
 
+static const char *keys_096_6[] = {
+	"ca51 4809 04ec 2415 bb26 dd7d e927 3ef1",
+	"6a44 08d8 30bb c0a8 ab9b fe97 b4a3 41b8",
+	"c1bc 3d39 7579 2755 c7a7 0825 93e3 b42b",
+	"ab20 feaf 4268 48b1 b946 61d6 b9e4 5aea",
+	"da38 81e8 13e8 0b3b df48 2230 491c e973",
+	"d016 ea65 9110 2a9e cd78 d916 7b98 f4bb",
+	"d71d d56e 47ab 820d 3e2f 384e 7e6c b864",
+	"e43b 59b1 687f bda2 0b65 78ea bfd7 43b6",
+	"216f 2bf3 5f78 f9a2 ed42 495a 8649 ff3d",
+	"def4 46c0 c9be b07a 4166 1882 4a0b 032b",
+	"97dc d2cb b336 31c9 f50a 54c3 9249 37de",
+	"f226 7955 3c3d 7322 b354 5b39 26fd 5f5b",
+	"55ff bd4f 0626 9d57 44a1 7989 b717 7fc1",
+	"3f84 3b70 88bb 9cb0 f1af 71d7 653d 8a27",
+	"ff7f eea2 f128 5cf8 1d55 6dc7 88e4 96ae",
+	"bb77 eb02 35be 2856 42ac 5198 87ed 9e65",
+	"78e4 55ac c116 9a09 f010 7177 63fb 2556",
+	"20af 1d32 f5dc a674 bf0b d0eb 0ce7 ba27",
+	"741e 144c e7d5 6017 033c 8035 ba4c 271e",
+	"839f 7a68 30c0 8119 80b5 00ce 1f47 2a4e",
+	"49a2 5e0b 2c25 8fc9 b837 a30d 5d60 7c67",
+	"88e8 a3b2 a8cd 3bdf 9695 bb4b 4cce 6137",
+	"394d 4e34 c77e 4c94 17f9 030d df2e 2a2e",
+	"6fe4 cc5b 87db 9631 e69b a1dc 48da a41a",
+	"9a51 24dd 2024 b470 3131 2c2f f14c c289",
+	"3c03 6fac 146d f9f8 0ae4 fd2c 3ac9 594f",
+	"e25d edc1 5229 2984 fa4b d45f e79c fe84",
+	"3782 c665 d5d9 4055 d1c7 8aa2 9723 0b04",
+	"587d 2f6f a442 d49f 8de3 af20 a7c8 ac20",
+};
+
 /**
  * Describes the keys to use depending on the version.
  */
 struct tokkey {
 	version_t ver;		/**< Version number */
-	const gchar **keys;	/**< Keys to use */
+	const char **keys;	/**< Keys to use */
 	guint count;		/**< Amount of keys defined */
 } token_keys[] = {
 	/* Keep this array sorted by increasing timestamp */
-	{
-		{ 0, 96, 0, 'b', 0, 0, 1132614000 },		/* 2005-11-22 */
-		keys_096b, G_N_ELEMENTS(keys_096b),
-	},
 	{
 		{ 0, 96, 0, '\0', 0, 0, 1138057200 },		/* 2006-01-24 */
 		keys_096, G_N_ELEMENTS(keys_096),
@@ -269,13 +274,17 @@ struct tokkey {
 		{ 0, 96, 5, '\0', 0, 0, 1207083000 },		/* 2008-04-01 */
 		keys_096_5, G_N_ELEMENTS(keys_096_5),
 	},
+	{
+		{ 0, 96, 6, '\0', 0, 0, 1238338209 },		/* 2009-03-29 */
+		keys_096_6, G_N_ELEMENTS(keys_096_6),
+	},
 };
 
 /**
  * Token validation errors.
  */
 
-static const gchar *tok_errstr[] = {
+static const char *tok_errstr[] = {
 	"OK",							/**< TOK_OK */
 	"Bad length",					/**< TOK_BAD_LENGTH */
 	"Bad timestamp",				/**< TOK_BAD_STAMP */
@@ -297,12 +306,12 @@ static const gchar *tok_errstr[] = {
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *
+const char *
 tok_strerror(tok_error_t errnum)
 {
 	STATIC_ASSERT(G_N_ELEMENTS(tok_errstr) == TOK_MAX_ERROR);
 
-	if ((gint) errnum < 0 || errnum >= G_N_ELEMENTS(tok_errstr))
+	if ((int) errnum < 0 || errnum >= G_N_ELEMENTS(tok_errstr))
 		return "Invalid error code";
 
 	return tok_errstr[errnum];
@@ -355,7 +364,7 @@ find_latest(const version_t *rver)
  * @returns the key string and the index within the key array into `idx'
  * and the token key structure used in `tkused'.
  */
-static const gchar *
+static const char *
 random_key(time_t now, guint *idx, const struct tokkey **tkused)
 {
 	static gboolean warned = FALSE;
@@ -383,23 +392,23 @@ random_key(time_t now, guint *idx, const struct tokkey **tkused)
 /**
  * Generate new token for given version string.
  */
-static gchar *
-tok_generate(time_t now, const gchar *version)
+static char *
+tok_generate(time_t now, const char *version)
 {
-	gchar token[TOKEN_BASE64_SIZE + 1];
-	gchar digest[TOKEN_VERSION_SIZE];
-	gchar lvldigest[LEVEL_SIZE];
-	gchar lvlbase64[LEVEL_BASE64_SIZE + 1];
+	char token[TOKEN_BASE64_SIZE + 1];
+	char digest[TOKEN_VERSION_SIZE];
+	char lvldigest[LEVEL_SIZE];
+	char lvlbase64[LEVEL_BASE64_SIZE + 1];
 	const struct tokkey *tk;
 	guint idx;
-	const gchar *key;
+	const char *key;
 	SHA1Context ctx;
     struct sha1 sha1;
 	guint8 seed[3];
 	guint32 now32;
-	gint lvlsize;
-	gint klen;
-	gint i;
+	int lvlsize;
+	int klen;
+	int i;
 
 	/*
 	 * Compute token.
@@ -469,11 +478,11 @@ tok_generate(time_t now, const gchar *version)
  * It is not meant to be used for strict authentication management, since
  * the algorithm and the keys are exposed publicly.
  */
-gchar *
+char *
 tok_version(void)
 {
 	static time_t last_generated = 0;
-	static gchar *toklevel = NULL;
+	static char *toklevel = NULL;
 	time_t now = tm_time();
 
 	/*
@@ -484,14 +493,12 @@ tok_version(void)
 
 	g_assert(TOKEN_CLOCK_SKEW > 2 * TOKEN_LIFE);
 
-	if (now - last_generated < TOKEN_LIFE)
+	if (delta_time(now, last_generated) < TOKEN_LIFE)
 		return toklevel;
 
 	last_generated = now;
 
-	if (toklevel != NULL)
-		g_free(toklevel);
-
+	G_FREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_string);
 
 	return toklevel;
@@ -502,11 +509,11 @@ tok_version(void)
  *
  * @returns a pointer to static data.
  */
-gchar *
+char *
 tok_short_version(void)
 {
 	static time_t last_generated = 0;
-	static gchar *toklevel = NULL;
+	static char *toklevel = NULL;
 	time_t now = tm_time();
 
 	/*
@@ -517,14 +524,12 @@ tok_short_version(void)
 
 	g_assert(TOKEN_CLOCK_SKEW > 2 * TOKEN_LIFE);
 
-	if (now - last_generated < TOKEN_LIFE)
+	if (delta_time(now, last_generated) < TOKEN_LIFE)
 		return toklevel;
 
 	last_generated = now;
 
-	if (toklevel != NULL)
-		g_free(toklevel);
-
+	G_FREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_short_string);
 
 	return toklevel;
@@ -538,7 +543,7 @@ tok_short_version(void)
  */
 tok_error_t
 tok_version_valid(
-	const gchar *version, const gchar *tokenb64, gint len, host_addr_t addr)
+	const char *version, const char *tokenb64, int len, host_addr_t addr)
 {
 	time_t now = tm_time();
 	time_t stamp;
@@ -547,19 +552,19 @@ tok_version_valid(
 	const struct tokkey *rtk;
 	const struct tokkey *latest;
 	guint idx;
-	const gchar *key;
+	const char *key;
 	SHA1Context ctx;
-	gchar lvldigest[1024];
-	gchar token[TOKEN_VERSION_SIZE];
+	char lvldigest[1024];
+	char token[TOKEN_VERSION_SIZE];
 	struct sha1 digest;
 	version_t rver;
-	gchar *end;
-	gint toklen;
-	gint lvllen;
-	gint lvlsize;
-	gint klen;
+	char *end;
+	int toklen;
+	int lvllen;
+	int lvlsize;
+	int klen;
 	guint i;
-	gchar *c = (gchar *) &stamp32;
+	char *c = (char *) &stamp32;
 
 	end = strchr(tokenb64, ';');		/* After 25/02/2003 */
 	toklen = end ? (end - tokenb64) : len;
@@ -637,7 +642,7 @@ tok_version_valid(
 	lvllen = len - toklen - 2;				/* Forget about "; " */
 	end += 2;								/* Skip "; " */
 
-	if (lvllen >= (gint) sizeof(lvldigest) || lvllen <= 0)
+	if (lvllen >= (int) sizeof(lvldigest) || lvllen <= 0)
 		return TOK_BAD_LEVEL_LENGTH;
 
 	if (lvllen & 0x3)
