@@ -1,5 +1,5 @@
 /*
- * $Id: tx_dgram.c 12583 2006-12-06 05:42:15Z cbiere $
+ * $Id: tx_dgram.c 16250 2009-03-27 13:35:00Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_dgram.c 12583 2006-12-06 05:42:15Z cbiere $")
+RCSID("$Id: tx_dgram.c 16250 2009-03-27 13:35:00Z rmanfredi $")
 
 #include "bsched.h"
 #include "sockets.h"
@@ -46,8 +46,7 @@ RCSID("$Id: tx_dgram.c 12583 2006-12-06 05:42:15Z cbiere $")
 #include "hosts.h"
 #include "inet.h"
 
-#include "if/core/hosts.h"
-
+#include "lib/gnet_host.h"
 #include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
@@ -66,7 +65,7 @@ struct attr {
  * Invoked when the output file descriptor can accept more data.
  */
 static void
-is_writable(gpointer data, gint unused_source, inputevt_cond_t cond)
+is_writable(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	txdrv_t *tx = (txdrv_t *) data;
 
@@ -109,7 +108,7 @@ tx_dgram_init(txdrv_t *tx, gpointer args)
 
 	/*
 	 * Because we handle servicing of the upper layers explicitely within
-	 * the TX stack (i.e. upper layers detect that we were enable to comply
+	 * the TX stack (i.e. upper layers detect that we were unable to comply
 	 * with the whole write and enable us), there is no I/O callback attached
 	 * to the I/O source: we only create it to benefit from bandwidth limiting
 	 * through calls to bio_sendto().
@@ -140,7 +139,7 @@ tx_dgram_destroy(txdrv_t *tx)
 	wfree(attr, sizeof *attr);
 }
 
-static inline gint
+static inline int
 tx_dgram_write_error(txdrv_t *tx, const gnet_host_t *to, const char *func)
 {
 	if (is_temporary_error(errno) || ENOBUFS == errno)
