@@ -1,5 +1,5 @@
 /*
- * $Id: udp.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: udp.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: udp.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: udp.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "bogons.h"
 #include "bsched.h"
@@ -69,7 +69,7 @@ udp_is_valid_gnet(struct gnutella_socket *s, gboolean truncated)
 {
 	struct gnutella_node *n = node_udp_get_addr_port(s->addr, s->port);
 	gnutella_header_t *head;
-	const gchar *msg;
+	const char *msg;
 	guint16 size;				/**< Payload size, from the Gnutella message */
 
 	g_return_val_if_fail(n, FALSE);
@@ -144,6 +144,7 @@ udp_is_valid_gnet(struct gnutella_socket *s, gboolean truncated)
 	case GTA_MSG_PUSH_REQUEST:
 	case GTA_MSG_SEARCH_RESULTS:
 	case GTA_MSG_RUDP:
+	case GTA_MSG_DHT:
 		return TRUE;
 	case GTA_MSG_SEARCH:
 		msg = "Queries not yet processed from UDP";
@@ -176,6 +177,39 @@ log:
 	return FALSE;
 }
 
+/**
+ * Check whether we got a valid NTP reply.
+ */
+static gboolean
+is_ntp_reply(struct gnutella_socket *s)
+{
+	host_addr_t addr;
+	gboolean got_reply = FALSE;
+	
+	if (!host_addr_convert(s->addr, &addr, NET_TYPE_IPV4))
+		addr = s->addr;
+
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:
+		got_reply = 0x7f000001 == host_addr_ipv4(addr); /* 127.0.0.1:123 */
+		break;
+	case NET_TYPE_IPV6:
+		/* Only the loopback address (::1) qualifies as private */
+		got_reply = is_private_addr(addr); /* [::1]:123 */
+		break;
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		g_assert_not_reached();
+	}
+
+	if (got_reply) {
+		g_message("NTP detected at %s", host_addr_to_string(addr));
+		ntp_got_reply(s);
+	}
+
+	return got_reply;
+}
+
 /**
  * Notification from the socket layer that we got a new datagram.
  *
@@ -191,39 +225,14 @@ udp_received(struct gnutella_socket *s, gboolean truncated)
 	 * If reply comes from the NTP port, notify that they're running NTP.
 	 */
 
-	if (NTP_PORT == s->port) {
-		host_addr_t addr;
-		gboolean got_reply = FALSE;
-		
-		if (!host_addr_convert(s->addr, &addr, NET_TYPE_IPV4))
-			addr = s->addr;
-
-		switch (host_addr_net(addr)) {
-		case NET_TYPE_IPV4:
-			got_reply = 0x7f000001 == host_addr_ipv4(addr); /* 127.0.0.1:123 */
-			break;
-		case NET_TYPE_IPV6:
-			/* Only the loopback address (::1) qualifies as private */
-			got_reply = is_private_addr(addr); /* [::1]:123 */
-			break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			g_assert_not_reached();
-		}
-		if (got_reply) {
-			g_message("NTP detected at %s", host_addr_to_string(addr));
-			ntp_got_reply(s);
-			return;
-		}
-		/* FALL THROUGH -- reply did not come from localhost */
-	}
+	if (NTP_PORT == s->port && is_ntp_reply(s))
+		return;
 
 	/*
 	 * This must be regular Gnutella traffic then.
 	 */
 
 	inet_udp_got_incoming(s->addr);
-	bws_udp_count_read(s->pos);
 
 	/*
 	 * If we get traffic from a bogus IP (unroutable), warn, for now.
@@ -235,7 +244,7 @@ udp_received(struct gnutella_socket *s, gboolean truncated)
 		if (GNET_PROPERTY(udp_debug)) {
 			g_warning("UDP %sdatagram (%d byte%s) received from bogus IP %s",
 				truncated ? "truncated " : "",
-				(gint) s->pos, s->pos == 1 ? "" : "s",
+				(int) s->pos, s->pos == 1 ? "" : "s",
 				host_addr_to_string(s->addr));
 		}
 		gnet_stats_count_general(GNR_UDP_BOGUS_SOURCE_IP, 1);
@@ -261,7 +270,7 @@ udp_received(struct gnutella_socket *s, gboolean truncated)
  * forming a valid Gnutella message.
  */
 void
-udp_send_msg(const gnutella_node_t *n, gconstpointer buf, gint len)
+udp_send_msg(const gnutella_node_t *n, gconstpointer buf, int len)
 {
 	g_assert(NODE_IS_UDP(n));
 	g_return_if_fail(n->outq);
@@ -278,9 +287,14 @@ udp_send_msg(const gnutella_node_t *n, gconstpointer buf, gint len)
 void
 udp_send_mb(const gnutella_node_t *n, pmsg_t *mb)
 {
+	if (NULL == n || NULL == n->outq) {
+		pmsg_free(mb);
+		/* emit warnings */
+		g_return_if_fail(n);
+		g_return_if_fail(n->outq);
+		g_assert_not_reached();
+	}
 	g_assert(NODE_IS_UDP(n));
-	g_return_if_fail(n->outq);
-
 	mq_udp_node_putq(n->outq, mb, n);
 }
 
@@ -289,7 +303,7 @@ udp_send_mb(const gnutella_node_t *n, pmsg_t *mb)
  * specified MUID.
  */
 void
-udp_connect_back(const host_addr_t addr, guint16 port, const gchar *muid)
+udp_connect_back(const host_addr_t addr, guint16 port, const struct guid *muid)
 {
 	if (udp_send_ping(muid, addr, port, FALSE)) {
 		if (GNET_PROPERTY(udp_debug) > 19)
@@ -299,13 +313,13 @@ udp_connect_back(const host_addr_t addr, guint16 port, const gchar *muid)
 }
 
 struct udp_ping {
-	gchar muid[GUID_RAW_SIZE];	/* MUST be at offset zero */
-	time_t added;				/**< Timestamp of insertion */
+	struct guid muid;	/* MUST be at offset zero */
+	time_t added;		/**< Timestamp of insertion */
 };
 
 static const time_delta_t UDP_PING_TIMEOUT	    = 30;	/**< seconds */
 static const size_t		  UDP_PING_MAX 			= 1024;	/**< amount to track */
-static const gint 		  UDP_PING_PERIODIC_MS	= 5000;	/**< milliseconds */
+static const int 		  UDP_PING_PERIODIC_MS	= 5000;	/**< milliseconds */
 
 static hash_list_t *udp_pings;	/**< Tracks send/forwarded UDP Pings */
 static cevent_t *udp_ping_ev;	/**< Monitoring event */
@@ -361,7 +375,7 @@ udp_ping_timer(cqueue_t *cq, gpointer unused_udata)
 }
 
 static gboolean
-udp_ping_register(const gchar *muid)
+udp_ping_register(const struct guid *muid)
 {
 	static gboolean initialized;
 	struct udp_ping *ping;
@@ -386,19 +400,19 @@ udp_ping_register(const gchar *muid)
 	if (length >= UDP_PING_MAX) {
 		return FALSE;
 	} else if (length > (UDP_PING_MAX / 4) * 3) {
-		if ((random_raw() % UDP_PING_MAX) < length)
+		if ((random_u32() % UDP_PING_MAX) < length)
 			return FALSE;
 	}
 
 	ping = walloc(sizeof *ping);
-	memcpy(ping->muid, muid, GUID_RAW_SIZE);
+	ping->muid = *muid;
 	ping->added = tm_time();
 	hash_list_append(udp_pings, ping);
 	return TRUE;
 }
 
 gboolean
-udp_ping_is_registered(const gchar *muid)
+udp_ping_is_registered(const struct guid *muid)
 {
 	g_assert(muid);
 
@@ -418,7 +432,7 @@ udp_ping_is_registered(const gchar *muid)
  * Send a Gnutella ping to the specified host.
  */
 gboolean
-udp_send_ping(const gchar *muid, const host_addr_t addr, guint16 port,
+udp_send_ping(const struct guid *muid, const host_addr_t addr, guint16 port,
 	gboolean uhc_ping)
 {
 	struct gnutella_node *n = node_udp_get_addr_port(addr, port);
