@@ -1,5 +1,5 @@
 /*
- * $Id: udp.h 13901 2007-06-20 22:32:33Z cbiere $
+ * $Id: udp.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -46,15 +46,17 @@
 
 struct gnutella_socket;
 struct gnutella_node;
+struct guid;
 struct pmsg;
 
 void udp_received(struct gnutella_socket *s, gboolean truncated);
-void udp_connect_back(const host_addr_t addr, guint16 port, const gchar *muid);
-void udp_send_msg(const struct gnutella_node *n, gconstpointer buf, gint len);
-gboolean udp_send_ping(const gchar *muid, const host_addr_t addr, guint16 port,
-	gboolean uhc_ping);
+void udp_connect_back(const host_addr_t addr, guint16 port,
+	const struct guid *muid);
+void udp_send_msg(const struct gnutella_node *n, gconstpointer buf, int len);
+gboolean udp_send_ping(const struct guid *muid,
+	const host_addr_t addr, guint16 port, gboolean uhc_ping);
 void udp_send_mb(const struct gnutella_node *n, struct pmsg *mb);
-gboolean udp_ping_is_registered(const gchar *muid);
+gboolean udp_ping_is_registered(const struct guid *muid);
 void udp_close(void);
 
 #endif /* _core_udp_h_ */
