@@ -1,5 +1,5 @@
 /*
- * $Id: uhc.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: uhc.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -51,7 +51,7 @@ void uhc_get_hosts(void);
 gboolean uhc_is_waiting(void);
 
 void uhc_ipp_extract(
-	struct gnutella_node *n, const gchar *payload, gint paylen);
+	struct gnutella_node *n, const char *payload, int paylen);
 
 #endif /* _core_uhc_h_ */
 
