@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: upload_stats.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -45,9 +45,11 @@ void upload_stats_close(void);
 void upload_stats_file_aborted(const struct shared_file *sf, filesize_t done);
 void upload_stats_file_begin(const struct shared_file *sf);
 void upload_stats_file_complete(const struct shared_file *sf, filesize_t done);
+void upload_stats_file_requested(const struct shared_file *sf);
 void upload_stats_flush_if_dirty(void);
-void upload_stats_load_history(const gchar *ul_history_file_name);
+void upload_stats_load_history(const char *ul_history_file_name);
 void upload_stats_prune_nonexistent(void);
+void upload_stats_enforce_local_filename(const struct shared_file *sf);
 
 #endif /* _core_upload_stats_h_ */
 /* vi: set ts=4 sw=4 cindent: */
