@@ -1,5 +1,5 @@
 /*
- * $Id: verify.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: verify.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: verify.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: verify.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "file_object.h"
 #include "verify.h"
@@ -53,8 +53,6 @@ RCSID("$Id: verify.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
 #include "lib/override.h"	/* Must be the last header included */
 
 #define HASH_BUF_SIZE		(128 * 1024) /**< Size of the reading buffer */
-#define HASH_MS_PER_STEP	50	/**< Max. time to spent (in milliseconds) */
-#define HASH_RUNS_PER_STEP	64	/**< Upper limit; guard against bad clock */
 
 enum verify_magic { VERIFY_MAGIC = 0x2dc84379U };
 
@@ -198,6 +196,16 @@ verify_failure(struct verify *ctx)
 	ctx->status = VERIFY_INVALID;
 }
 
+static void
+verify_shutdown(struct verify *ctx)
+{
+	verify_check(ctx);
+
+	ctx->status = VERIFY_SHUTDOWN;
+	(void) ctx->callback(ctx, ctx->status, ctx->user_data);
+	ctx->status = VERIFY_INVALID;
+}
+
 static void
 verify_done(struct verify *ctx)
 {
@@ -260,7 +268,7 @@ verify_item_hash(gconstpointer key)
 		^ pointer_hash_func(ctx->user_data);
 }
 
-static gint
+static int
 verify_item_equal(gconstpointer p, gconstpointer q)
 {
 	const struct verify_file *a = p, *b = q;
@@ -305,6 +313,9 @@ verify_free(struct verify **ptr)
 			bg_task_cancel(ctx->task);
 			ctx->task = NULL;
 		}
+		if (VERIFY_INVALID != ctx->status) {
+			verify_shutdown(ctx);
+		}
 		if (ctx->files_to_hash) {
 			struct verify_file *item;
 
@@ -329,7 +340,19 @@ verify_context_free(void *data)
 	
 	verify_check(ctx);
 	/* If we're called, the task is being terminated */
+
+	/*
+	 * There's nothing to free here.  We're managing the overall context
+	 * ourselves, and this overall context is given as the task's context.
+	 * We just need to record the fact that the task has been terminated
+	 * here so verify_free() does not try to cancel it.
+	 */
+
 	ctx->task = NULL;
+
+	if (GNET_PROPERTY(verify_debug) > 1) {
+		g_message("destroying task for %s verification", verify_hash_name(ctx));
+	}
 }
 
 static void
@@ -354,15 +377,20 @@ verify_next_file(struct verify *ctx)
 			if (NULL == ctx->file) {
 				int fd;
 
-				fd = file_open(item->pathname, O_RDONLY);
+				fd = file_open(item->pathname, O_RDONLY, 0);
 				if (fd >= 0) {
 					ctx->file = file_object_new(fd, item->pathname, O_RDONLY);
 				}
 			}
 			if (NULL == ctx->file) {
-				g_warning("Failed to open \"%s\" for %s hashing: %s",
+				g_warning("failed to open \"%s\" for %s hashing: %s",
 					verify_hash_name(ctx), item->pathname, g_strerror(errno));
 			}
+		} else {
+			if (GNET_PROPERTY(verify_debug)) {
+				g_message("discarding request of %s digest for %s",
+					verify_hash_name(ctx), item->pathname);
+			}
 		}
 		verify_file_free(&item);
 
@@ -372,8 +400,8 @@ verify_next_file(struct verify *ctx)
 	}
 
 	if (ctx->file) {
-		if (GNET_PROPERTY(dbg) > 1) {
-			g_message("Verifying %s digest for %s",
+		if (GNET_PROPERTY(verify_debug)) {
+			g_message("verifying %s digest for %s",
 				verify_hash_name(ctx), file_object_get_pathname(ctx->file));
 		}
 		verify_hash_init(ctx);
@@ -393,7 +421,7 @@ verify_final(struct verify *ctx)
 	verify_check(ctx);
 
 	if (ctx->offset != ctx->end) {
-		g_warning("File shrunk? \"%s\"", file_object_get_pathname(ctx->file));
+		g_warning("file shrunk? \"%s\"", file_object_get_pathname(ctx->file));
 		verify_failure(ctx);
 	} else if (verify_hash_final(ctx)) {
 		g_warning("verify_hash_final() failed for \"%s\"",
@@ -453,33 +481,39 @@ static bgret_t
 verify_step_compute(struct bgtask *bt, void *data, int ticks)
 {
 	struct verify *ctx = data;
-	guint i = HASH_RUNS_PER_STEP;
-	tm_t t0;
+	int i = ticks;
+	int used = 0;		/* Amount used for CPU-intensive tasks */
+	int light = 0;		/* Amount used for system-intensive tasks */
 
 	verify_check(ctx);
-	(void) ticks;
-
-	bg_task_ticks_used(bt, 0);
-	tm_now_exact(&t0);
+	(void) bt;
 
 	while (i-- > 0) {
-		tm_t t1, elapsed;
-
 		if (NULL == ctx->file) {
 			verify_next_file(ctx);
 		}
 		if (ctx->file) {
 			verify_update(ctx);
+			used++;
+		} else {
+			light++;	/* Did not open file, still processed something */
 		}
 		if (NULL == ctx->file && 0 == hash_list_length(ctx->files_to_hash))
 			break;
-
-		tm_now_exact(&t1);
-		tm_elapsed(&elapsed, &t1, &t0);
-		if (tm2ms(&elapsed) > HASH_MS_PER_STEP)
-			break;
 	}
-	
+
+	/*
+	 * If we don't use all our ticks, we need to tell the scheduler
+	 * otherwise the average tick cost will be wrong and when we start
+	 * using all our ticks, the whole process could "freeze" during several
+	 * seconds, if not minutes!
+	 */
+
+	used += light / 10;			/* Arbitrary decimation factor */
+
+	if (used < ticks)
+		bg_task_ticks_used(bt, used);
+
 	if (ctx->file || hash_list_length(ctx->files_to_hash) > 0) {
 		return BGR_MORE;
 	} else {
@@ -495,6 +529,11 @@ verify_create_task(struct verify *ctx)
 	if (NULL == ctx->task) {
 		static const bgstep_cb_t step[] = { verify_step_compute };
 
+		if (GNET_PROPERTY(verify_debug) > 1) {
+			g_message("creating new task for %s verification",
+				verify_hash_name(ctx));
+		}
+
 		ctx->task = bg_task_create(verify_hash_name(ctx),
 							step, G_N_ELEMENTS(step),
 			  				ctx, verify_context_free,
@@ -537,6 +576,13 @@ verify_enqueue(struct verify *ctx, int high_priority,
 		}
 		inserted = TRUE;
 	}
+
+	if (GNET_PROPERTY(verify_debug)) {
+		g_message("%s %s digest verification for %s",
+			inserted ? "enqueued" : "already had queued",
+			verify_hash_name(ctx), pathname);
+	}
+
 	verify_create_task(ctx);
 	return inserted;
 }
