@@ -1,5 +1,5 @@
 /*
- * $Id: verify_tth.c 14141 2007-07-21 21:14:04Z cbiere $
+ * $Id: verify_tth.c 16042 2009-03-02 15:55:15Z rmanfredi $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -29,16 +29,13 @@
  *
  * Tigertree hash verification.
  *
- * This is not ready yet at all, do not try to use it yet. It is included
- * for compilation reasons only.
- *
  * @author Jeroen Asselman
  * @date 2003
  */
 
 #include "common.h"
 
-RCSID("$Id: verify_tth.c 14141 2007-07-21 21:14:04Z cbiere $")
+RCSID("$Id: verify_tth.c 16042 2009-03-02 15:55:15Z rmanfredi $")
 
 #include "downloads.h"
 #include "file_object.h"
@@ -154,11 +151,26 @@ request_tigertree_callback(const struct verify *ctx, enum verify_status status,
 	shared_file_check(sf);
 	switch (status) {
 	case VERIFY_START:
+		if (!(SHARE_F_INDEXED & shared_file_flags(sf))) {
+			/*
+			 * After a rescan, there might be files in the queue which are
+			 * no longer shared.
+			 */
+
+			if (GNET_PROPERTY(verify_debug) > 1) {
+				g_message("skipping TTH computation for %s: no longer shared",
+					shared_file_path(sf));
+			}
+			return FALSE;
+		}
 		if (
 			shared_file_tth(sf) &&
 			tth_cache_lookup(shared_file_tth(sf), shared_file_size(sf)) > 0
 		) {
-			if (GNET_PROPERTY(tigertree_debug) > 1) {
+			if (
+				GNET_PROPERTY(tigertree_debug) > 1 ||
+				GNET_PROPERTY(verify_debug) > 1
+			) {
 				g_message("TTH for %s is already cached (%s)",
 					shared_file_path(sf), tth_base32(shared_file_tth(sf)));
 			}
@@ -167,7 +179,7 @@ request_tigertree_callback(const struct verify *ctx, enum verify_status status,
 		gnet_prop_set_boolean_val(PROP_TTH_REBUILDING, TRUE);
 		return TRUE;
 	case VERIFY_PROGRESS:
-		return TRUE;
+		return 0 != (SHARE_F_INDEXED & shared_file_flags(sf));
 	case VERIFY_DONE:
 		{
 			const struct tth *tth = verify_tth_digest(ctx);
