@@ -1,5 +1,5 @@
 /*
- * $Id: version.c 13938 2007-06-23 04:52:29Z cbiere $
+ * $Id: version.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c 13938 2007-06-23 04:52:29Z cbiere $")
+RCSID("$Id: version.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "version.h"
 #include "token.h"
@@ -47,8 +47,8 @@ RCSID("$Id: version.c 13938 2007-06-23 04:52:29Z cbiere $")
 #include "if/core/main.h"
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
 #include "lib/getdate.h"
-#include "lib/misc.h"
 #include "lib/glib-missing.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
@@ -56,39 +56,18 @@ RCSID("$Id: version.c 13938 2007-06-23 04:52:29Z cbiere $")
 
 #define SECS_PER_DAY	86400
 
-const gchar *version_string = NULL;
-const gchar *version_short_string = NULL;
+const char *version_string = NULL;
+const char *version_short_string = NULL;
 
 static version_t our_version;
 static version_t last_rel_version;
 static version_t last_dev_version;
 static guint8 version_code;
 
-/**
- * Compare the given timestamp against the date 2007-03-22. We don't
- * care about the exact date here so there might be some offset of a few
- * hours with respect to the timezone or daylight saving.
- *
- * @param now A timestamp.
- * @return TRUE if 2007-03-22 is still in the future, FALSE otherwise.
- */
-static gboolean
-before_2007_03_22(time_t now)
-{
-	static gboolean initialized;
-	static time_t x;
-
-	if (!initialized) {
-		initialized = TRUE;
-		x = date2time("2007-03-22", tm_time());
-	}
-	return delta_time(now, x) < 0;
-}
-
 /**
  * Get version string.
  */
-const gchar *
+const char *
 version_get_string(void)
 {
 	return version_string;
@@ -107,7 +86,7 @@ version_get_code(void)
  * Dump original version string and decompiled form to stdout.
  */
 static void
-version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
+version_dump(const char *str, const version_t *ver, const char *cmptag)
 {
 	printf("VERSION%s \"%s\":\n"
 		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u build=%u\n",
@@ -119,11 +98,11 @@ version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
  * @return a user-friendly description of the version.
  * NB: returns pointer to static data.
  */
-const gchar *
+const char *
 version_str(const version_t *ver)
 {
-	static gchar str[80];
-	gint rw;
+	static char str[80];
+	int rw;
 
 	rw = gm_snprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
 
@@ -153,10 +132,10 @@ version_str(const version_t *ver)
  * and extract timestamp into `ver'.
  */
 static void
-version_stamp(const gchar *str, version_t *ver)
+version_stamp(const char *str, version_t *ver)
 {
-	static gchar stamp[256];
-	const gchar *p;
+	static char stamp[256];
+	const char *p;
 
 	ver->timestamp = 0;
 
@@ -172,7 +151,7 @@ version_stamp(const gchar *str, version_t *ver)
 
 	p = strchr(str, '(');
 	if (p) {
-		const gchar *end;
+		const char *end;
 
 		p++;
 		end = strchr(p, ';');
@@ -208,10 +187,10 @@ version_stamp(const gchar *str, version_t *ver)
  * were not facing a gtk-gnutella version, or if we did not recognize it.
  */
 static gboolean
-version_parse(const gchar *str, version_t *ver)
+version_parse(const char *str, version_t *ver)
 {
-	const gchar *v;
-	gint error;
+	const char *v;
+	int error;
 
 	/*
 	 * Modern version numbers are formatted like this:
@@ -246,7 +225,7 @@ version_parse(const gchar *str, version_t *ver)
 	 * where the '-3404' introduces the build number.
 	 */
 
-	if (NULL == (v = is_strprefix(str, "gtk-gnutella/")))
+	if (NULL == (v = is_strprefix(str, GTA_PRODUCT_NAME "/")))
 		return FALSE;
 
 	/*
@@ -301,25 +280,6 @@ version_parse(const gchar *str, version_t *ver)
 	} else
 		ver->build = 0;
 
-	/*
-	 * Legacy support: until 2007-03-22, we look at the build revision
-	 * number in the user-agent string if we did not get it so far.
-	 * We expect to find something like "(2006-02-22; r3456"
-	 *
-	 * Remove this code after 2007-03-22!
-	 */
-
-	if (0 == ver->build) {	/* Useless after 2007-03-22 normally */
-		const gchar *p;
-
-		p = strchr(str, '(');		/* ')' for vi */
-		if (p)
-			p = strchr(p, ';');		/* Move past date */
-
-		if (p && NULL != (p = is_strprefix(p, "; r")))
-			ver->build = (guint32) atoi(p);
-	}
-
 	if (GNET_PROPERTY(dbg) > 6)
 		version_dump(str, ver, "#");
 
@@ -330,7 +290,7 @@ version_parse(const gchar *str, version_t *ver)
  * Compare two tag chars, assuming version numbers are equal.
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-static gint
+static int
 version_tagcmp(guchar a, guchar b)
 {
 	if (a == b)
@@ -355,7 +315,7 @@ version_tagcmp(guchar a, guchar b)
  * Compare two gtk-gnutella versions, timestamp and build not withstanding.
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-gint
+int
 version_cmp(const version_t *a, const version_t *b)
 {
 	if (a->major == b->major) {
@@ -382,10 +342,10 @@ version_cmp(const version_t *a, const version_t *b)
  *
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-gint
+int
 version_build_cmp(const version_t *a, const version_t *b)
 {
-	gint cmp = version_cmp(a, b);
+	int cmp = version_cmp(a, b);
 	return 0 == cmp ? CMP(a->build, b->build) : cmp;
 }
 
@@ -395,7 +355,7 @@ version_build_cmp(const version_t *a, const version_t *b)
  * @returns OK if we were able to parse correctly.
  */
 gboolean
-version_fill(const gchar *version, version_t *vs)
+version_fill(const char *version, version_t *vs)
 {
 	if (!version_parse(version, vs))
 		return FALSE;
@@ -409,11 +369,11 @@ version_fill(const gchar *version, version_t *vs)
  * Invoked when a newer version is found.
  */
 static void
-version_new_found(const gchar *text, gboolean stable)
+version_new_found(const char *text, gboolean stable)
 {
-    static gchar last_stable[256] = "";
-    static gchar last_dev[256] = "";
-	gchar s[1024];
+    static char last_stable[256] = "";
+    static char last_dev[256] = "";
+	char s[1024];
 
     if (stable)
         utf8_strlcpy(last_stable, text, sizeof last_stable);
@@ -447,12 +407,12 @@ version_new_found(const gchar *text, gboolean stable)
  * looking as gtk-gnutella but which failed the token-based sanity checks.
  */
 gboolean
-version_check(const gchar *str, const gchar *token, const host_addr_t addr)
+version_check(const char *str, const char *token, const host_addr_t addr)
 {
 	version_t their_version;
 	version_t *target_version;
-	gint cmp;
-	const gchar *version;
+	int cmp;
+	const char *version;
 
 	if (!version_parse(str, &their_version))
 		return TRUE;			/* Not gtk-gnutella, or unparseable */
@@ -480,7 +440,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 	version_stamp(str, &their_version);
 
 	if (GNET_PROPERTY(dbg) > 6)
-		printf("VERSION time=%d\n", (gint) their_version.timestamp);
+		printf("VERSION time=%d\n", (int) their_version.timestamp);
 
 	/*
 	 * If version claims something older than TOKEN_START_DATE, then
@@ -614,16 +574,15 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
  *
  * @return A pointer to a static buffer holding the version string.
  */
-const gchar *
+const char *
 version_build_string(void)
 {
 	static gboolean initialized;
-	static gchar buf[128];
+	static char buf[128];
 
 	if (!initialized) {
-		time_t now = tm_time_exact();
-		const gchar *sysname = "Unknown";
-		const gchar *machine = NULL;
+		const char *sysname = "Unknown";
+		const char *machine = NULL;
 
 		initialized = TRUE;
 
@@ -640,28 +599,13 @@ version_build_string(void)
 		}
 #endif /* HAS_UNAME */
 
-		/*
-		 * Until 2007-03-22, we generate the version string using "r<build#>"
-		 * in the optional part.  After that date, the revision format is
-		 * expanded to include the build number right into the string.
-		 */
-
-		if (before_2007_03_22(now))				/* before 2007-03-22 */
-			gm_snprintf(buf, sizeof buf,
-					"gtk-gnutella/%s (%s; r%u; %s; %s%s%s)",
-					GTA_VERSION_NUMBER, GTA_RELEASE, main_get_build(),
-					GTA_INTERFACE,
-					sysname,
-					machine && machine[0] ? " " : "",
-					machine ? machine : "");
-		else								/* after 2007-03-22 */
-			gm_snprintf(buf, sizeof buf,
-					"gtk-gnutella/%s-%u (%s; %s; %s%s%s)",
-					GTA_VERSION_NUMBER, main_get_build(),
-					GTA_RELEASE, GTA_INTERFACE,
-					sysname,
-					machine && machine[0] ? " " : "",
-					machine ? machine : "");
+		gm_snprintf(buf, sizeof buf,
+			GTA_PRODUCT_NAME "/%s-%u (%s; %s; %s%s%s)",
+			GTA_VERSION_NUMBER, main_get_build(),
+			GTA_RELEASE, gtk_gnutella_interface(),
+			sysname,
+			machine && machine[0] ? " " : "",
+			machine ? machine : "");
 	}
 	return buf;
 }
@@ -689,16 +633,11 @@ version_init(void)
 	g_assert(our_version.timestamp != 0);
 
 	{
-		gchar buf[128];
+		char buf[128];
 
-		if (before_2007_03_22(now))				/* before 2007-03-22 */
-			gm_snprintf(buf, sizeof(buf),
-					"gtk-gnutella/%s (%s; r%u)",
-					GTA_VERSION_NUMBER, GTA_RELEASE, main_get_build());
-		else
-			gm_snprintf(buf, sizeof(buf),
-					"gtk-gnutella/%s-%u (%s)",
-					GTA_VERSION_NUMBER, main_get_build(), GTA_RELEASE);
+		gm_snprintf(buf, sizeof(buf),
+			GTA_PRODUCT_NAME "/%s-%u (%s)",
+			GTA_VERSION_NUMBER, main_get_build(), GTA_RELEASE);
 
 		version_short_string = atom_str_get(buf);
 	}
@@ -726,9 +665,7 @@ version_init(void)
 
 	if (
 		tok_is_ancient(now) ||
-		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN ||
-		(our_version.tag &&
-			delta_time(now, our_version.timestamp) > VERSION_UNSTABLE_WARN)
+		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN
 	) {
 		*deconstify_gboolean(&GNET_PROPERTY(ancient_version)) = TRUE;
 	}
@@ -769,18 +706,12 @@ version_ancient_warn(void)
 		return;
 	}
 
-	if (our_version.tag && elapsed > VERSION_UNSTABLE_WARN) {
-		g_warning("unstable version of gtk-gnutella is aging, please upgrade!");
-        gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
-		return;
-	}
-
 	/*
 	 * Check whether we're nearing ancient version status, to warn them
 	 * beforehand that the version will become old soon.
 	 */
 
-	lifetime = our_version.tag ? VERSION_UNSTABLE_WARN : VERSION_ANCIENT_WARN;
+	lifetime = VERSION_ANCIENT_WARN;
 	remain = delta_time(lifetime, elapsed);
 
 	g_assert(remain >= 0);		/* None of the checks above have fired */
@@ -813,31 +744,6 @@ version_ancient_warn(void)
 	}
 }
 
-/**
- * Check the timestamp in the GTKG version string and returns TRUE if it
- * is too old or could not be parsed, FALSE if OK.
- */
-gboolean
-version_is_too_old(const gchar *vendor)
-{
-	version_t ver;
-	time_t now = tm_time();
-	gint d;
-
-	version_stamp(vendor, &ver);		/* Fills ver->timestamp */
-	d = delta_time(now, ver.timestamp);
-	if (d > VERSION_ANCIENT_BAN)
-		return TRUE;
-
-	if (!version_parse(vendor, &ver))	/* Fills ver->tag */
-		return TRUE;					/* Unable to parse */
-
-	if (ver.tag && d > VERSION_UNSTABLE_BAN)
-		return TRUE;
-
-	return FALSE;
-}
-
 /**
  * Free version string.
  */
