@@ -1,5 +1,5 @@
 /*
- * $Id: c2ui.h 14372 2007-08-08 21:43:46Z cbiere $
+ * $Id: c2ui.h 15323 2008-05-02 20:04:24Z rmanfredi $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -69,8 +69,9 @@ void gcu_statusbar_warning(const gchar *message);
 void gcu_statusbar_message(const gchar *message);
 
 /* upload interface functions */
-void gcu_upload_stats_gui_add(const struct ul_stats *);
-void gcu_upload_stats_gui_update(const struct ul_stats *);
+void gcu_upload_stats_gui_add(struct ul_stats *);
+void gcu_upload_stats_gui_update(struct ul_stats *);
+void gcu_upload_stats_gui_update_name(struct ul_stats *);
 void gcu_upload_stats_gui_clear_all(void);
 void gcu_upload_stats_gui_freeze(void);
 void gcu_upload_stats_gui_thaw(void);
