@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h 14935 2007-09-19 13:57:30Z cbiere $
+ * $Id: downloads.h 16287 2009-03-28 21:56:51Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -28,6 +28,7 @@
 
 #include "lib/tm.h"				/* For tm_t */
 #include "lib/event.h"			/* For frequency_t */
+#include "lib/hashlist.h"
 #include "lib/list.h"
 #include "lib/slist.h"
 
@@ -73,8 +74,10 @@ struct vernum {
 	guint minor;
 };
 
+struct guid;
+
 struct dl_key {
-	const gchar *guid;			/**< GUID of server (atom) */
+	const struct guid *guid;	/**< GUID of server (atom) */
 	host_addr_t addr;			/**< IP address of server */
 	guint16 port;				/**< Port of server */
 };
@@ -83,16 +86,17 @@ enum dl_server_magic { DL_SERVER_MAGIC = 0x5e45e4ffU };
 
 struct dl_server {
 	enum dl_server_magic magic;	/**< Magic number */
-	gint refcnt;				/**< Reference count */
+	int refcnt;					/**< Reference count */
 	struct dl_key *key;			/**< Key properties */
 	list_t *list[DL_LIST_SZ];	/**< Download lists */
-	const gchar *vendor;		/**< Remote server vendor string (atom) */
-	const gchar *hostname;		/**< Remote hostname, if known (atom) */
-	GSList *proxies;		/**< Known push proxies (struct gnutella_host) */
+	const char *vendor;			/**< Remote server vendor string (atom) */
+	const char *hostname;		/**< Remote hostname, if known (atom) */
+	hash_list_t *proxies;		/**< Known push proxies (gnet_host_t) */
 	GHashTable *sha1_counts;
 	time_t retry_after;		/**< Time at which we may retry from this host */
 	time_t dns_lookup;		/**< Last DNS lookup for hostname */
 	time_t proxies_stamp;	/**< Time when list was last updated */
+	time_t last_connect;	/**< When we last connected to that server */
 	struct vernum parq_version; /**< Supported queueing version */
 	guint speed_avg;			/**< Average (EMA) upload speed, in bytes/sec */
 	guint32 attrs;
@@ -167,7 +171,7 @@ struct download {
     gnet_src_t src_handle;      /**< Handle */
 	gboolean src_handle_valid;	/**< TRUE if src_handle is initialized */
 
-	gchar error_str[256];		/**< Used to snprintf() error strings */
+	char error_str[256];		/**< Used to snprintf() error strings */
 	download_status_t status;   /**< Current status of the download */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 	rxdrv_t *rx;					/**< RX stack top */
@@ -179,7 +183,7 @@ struct download {
 
 	struct dl_file_info *file_info;
 	guint32 record_index;		/**< Index of the file on the Gnutella server */
-	const gchar *file_name;		/**< Name of the file on the Gnutella server */
+	const char *file_name;		/**< Name of the file on the Gnutella server */
 	filesize_t file_size;		/**< Total size of the file, in bytes */
 
 	filesize_t size;			/**< Total size of the next request, in bytes */
@@ -208,10 +212,10 @@ struct download {
 	guint32 header_read_eof;	/**< EOF errors with empty headers */
 	guint32 data_timeouts;		/**< # of timeouts after getting headers */
 
-	const gchar *remove_msg;
+	const char *remove_msg;
 
 	const struct sha1 *sha1;	/**< Known SHA1 (binary atom), NULL if none */
-	const gchar *uri;			/**< Uri if not dealing with regular gnutella
+	const char *uri;			/**< Uri if not dealing with regular gnutella
 								 **< file download */
 	time_t last_dmesh;			/**< Time when last download mesh was sent */
 
@@ -239,6 +243,10 @@ struct download {
  */
 
 enum {
+	DL_F_FROM_ERROR		= 1 << 27,	/**< Switched after error occurred */
+	DL_F_FROM_PLAIN		= 1 << 26,	/**< Switched from plain file download */
+	DL_F_SWITCHED		= 1 << 25,	/**< Scheduled after resource switching */
+	DL_F_MUST_IGNORE	= 1 << 24,	/**< Must ignore data when they come */
 	DL_F_FAKE_G2		= 1 << 23,	/**< Trying to fake G2, intuition only */
 	DL_F_TRIED_TLS		= 1 << 22,	/**< TLS connection was tried already */
 	DL_F_TRY_TLS		= 1 << 21,	/**< Try to initiate a TLS connection */
@@ -269,6 +277,7 @@ enum {
  * Server attributes.
  */
 enum {
+	DLS_A_DHT_PROX		= 1 << 10,	/**< Looking for push-proxies in DHT */
 	DLS_A_G2_ONLY		= 1 << 9,	/**< Server is "G2" only */
 	DLS_A_FOOBAR		= 1 << 8,	/**< Server is foobar */
 	DLS_A_REMOVED		= 1 << 7,	/**< Server marked for removal */
@@ -297,6 +306,7 @@ enum {
 
 #define download_filesize(d)	((d)->file_info->size)
 #define download_filedone(d)	((d)->file_info->done + (d)->file_info->buffered)
+#define download_fileremain(d)	(download_filesize(d) - download_filedone(d))
 #define download_buffered(d)	((d)->buffers == NULL ? 0 : (d)->buffers->held)
 
 /*
@@ -310,8 +320,12 @@ enum {
  */
 
 #define DOWNLOAD_IS_QUEUED(d) \
-	(	(d)->status == GTA_DL_QUEUED	\
-	||  (d)->status == GTA_DL_PASSIVE_QUEUED)
+	(  (d)->status == GTA_DL_QUEUED		\
+	|| (d)->status == GTA_DL_PASSIVE_QUEUED)
+
+#define DOWNLOAD_IS_PARQED(d) \
+	(  (d)->status == GTA_DL_PASSIVE_QUEUED	\
+	|| (d)->status == GTA_DL_ACTIVE_QUEUED	 )
 
 #define DOWNLOAD_IS_VERIFYING(d)		 \
 	(  (d)->status == GTA_DL_VERIFY_WAIT \
@@ -356,6 +370,13 @@ enum {
 	(	DOWNLOAD_IS_ACTIVE(d)			\
 	||	DOWNLOAD_IS_ESTABLISHING(d)		)
 
+#define DOWNLOAD_IS_SWITCHABLE(d)		\
+	(  (d)->status == GTA_DL_TIMEOUT_WAIT \
+	|| (d)->status == GTA_DL_QUEUED	\
+	|| (d)->status == GTA_DL_CONNECTING	\
+	|| (d)->status == GTA_DL_FALLBACK	\
+	|| (d)->status == GTA_DL_PUSH_SENT	)
+
 #define DOWNLOAD_IS_IN_PUSH_MODE(d) (d->push)
 
 gboolean download_has_blank_guid(const struct download *);
@@ -368,8 +389,8 @@ download_check(const struct download * const d)
 	g_assert(!d->always_push || !download_has_blank_guid(d));
 }
 
-const gchar *download_pathname(const struct download *);
-const gchar *download_basename(const struct download *);
+const char *download_pathname(const struct download *);
+const char *download_basename(const struct download *);
 
 /*
  * Public interface, visible only from the bridge.
@@ -381,30 +402,30 @@ const gchar *download_basename(const struct download *);
  *        actually needs to be in downloads.h and should be called from
  *        search.h and not from search_gui.h.
  */
-void download_index_changed(const host_addr_t, guint16, const gchar *,
+void download_index_changed(const host_addr_t, guint16, const struct guid *,
 		guint32, guint32);
 
-gboolean download_new(const gchar *filename,
-	const gchar *uri,
+gboolean download_new(const char *filename,
+	const char *uri,
 	filesize_t size,
 	const host_addr_t addr,
 	guint16 port,
-	const gchar *guid,
-	const gchar *hostname,
+	const struct guid *guid,
+	const char *hostname,
 	const struct sha1 *sha1,
 	const struct tth *tth,
 	time_t stamp,
     struct dl_file_info *fi,
 	const gnet_host_vec_t *proxies,
 	guint32 flags,
-	const gchar *parq_id);
+	const char *parq_id);
 
-void download_auto_new(const gchar *filename,
+void download_auto_new(const char *filename,
  	filesize_t size,
 	const host_addr_t addr,
 	guint16 port,
-	const gchar *guid,
-	const gchar *hostname,
+	const struct guid *guid,
+	const char *hostname,
 	const struct sha1 *sha1,
 	const struct tth *tth,
 	time_t stamp,
@@ -412,11 +433,22 @@ void download_auto_new(const gchar *filename,
 	gnet_host_vec_t *proxies,
 	guint32 flags);
 
-guint download_handle_magnet(const gchar *url);
-gchar *download_build_url(const struct download *);
-gint download_get_http_req_percent(const struct download *);
+void download_dht_auto_new(const char *filename,
+ 	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const struct guid *guid,
+	const struct sha1 *sha1,
+	const struct tth *tth,
+	time_t stamp,
+	struct dl_file_info *fi,
+	guint32 flags);
+
+guint download_handle_magnet(const char *url);
+char *download_build_url(const struct download *);
+int download_get_http_req_percent(const struct download *);
 void download_fallback_to_push(struct download *, gboolean, gboolean);
-gint download_remove_all_from_peer(const gchar *guid,
+int download_remove_all_from_peer(const struct guid *,
 		host_addr_t addr, guint16 port, gboolean unavailable);
 void download_remove_file(struct download *, gboolean reset);
 gboolean download_file_exists(const struct download *);
@@ -436,9 +468,9 @@ void download_gui_updates_freeze(void);
 void download_gui_updates_thaw(void);
 
 void download_clear_stopped(gboolean, gboolean, gboolean, gboolean, gboolean);
-const gchar *download_get_hostname(const struct download *);
-gdouble download_source_progress(const struct download *);
-gdouble download_total_progress(const struct download *);
+const char *download_get_hostname(const struct download *);
+double download_source_progress(const struct download *);
+double download_total_progress(const struct download *);
 gboolean download_something_to_clear(void);
 guint download_speed_avg(struct download *);
 
