@@ -1,5 +1,5 @@
 /*
- * $Id: main.h 14365 2007-08-08 05:05:08Z cbiere $
+ * $Id: main.h 16110 2009-03-10 15:27:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -32,7 +32,7 @@ guint32 main_get_build(void);
 
 #ifdef CORE_SOURCES
 
-void gtk_gnutella_exit(gint n);
+void gtk_gnutella_exit(int n);
 void gtk_gnutella_request_shutdown(void);
 gboolean debugging(guint t);
 const char *gtk_gnutella_interface(void);
