@@ -1,5 +1,5 @@
 /*
- * $Id: net_stats.h 14601 2007-08-20 19:26:41Z cbiere $
+ * $Id: net_stats.h 16177 2009-03-16 17:44:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -45,6 +45,7 @@ enum {
 	MSG_PUSH_REQUEST,
 	MSG_SEARCH,
 	MSG_SEARCH_RESULTS,
+	MSG_DHT,
 	MSG_TOTAL,     /**< always counted (for all the above types) */
 	
 	MSG_TYPE_COUNT /**< number of known message types */
@@ -86,6 +87,9 @@ typedef enum msg_drop_reason {
 	MSG_DROP_OWN_RESULT,
 	MSG_DROP_ANCIENT_QUERY,
 	MSG_DROP_BLANK_SERVENT_ID,
+	MSG_DROP_DHT_INVALID_TOKEN,
+	MSG_DROP_DHT_TOO_MANY_STORE,
+	MSG_DROP_DHT_UNPARSEABLE,
 	
 	MSG_DROP_REASON_COUNT /**< number of known reasons to drop a message */
 } msg_drop_reason_t;
@@ -111,9 +115,13 @@ typedef enum {
 	GNR_QUERY_UTF8,
 	GNR_QUERY_SHA1,
 	GNR_BROADCASTED_PUSHES,
-	GNR_PUSH_PROXY_RELAYED,
+	GNR_PUSH_PROXY_UDP_RELAYED,
+	GNR_PUSH_PROXY_TCP_RELAYED,
 	GNR_PUSH_PROXY_BROADCASTED,
+	GNR_PUSH_PROXY_ROUTE_NOT_PROXIED,
 	GNR_PUSH_PROXY_FAILED,
+	GNR_PUSH_RELAYED_VIA_LOCAL_ROUTE,
+	GNR_PUSH_RELAYED_VIA_TABLE_ROUTE,
 	GNR_LOCAL_DYN_QUERIES,
 	GNR_LEAF_DYN_QUERIES,
 	GNR_OOB_PROXIED_QUERIES,
@@ -127,13 +135,89 @@ typedef enum {
 	GNR_GTKG_REQUERIES,
 	GNR_QUERIES_WITH_GGEP_H,
 	GNR_GIV_CALLBACKS,
+	GNR_GIV_DISCARDED,
 	GNR_QUEUE_CALLBACKS,
+	GNR_QUEUE_DISCARDED,
 	GNR_UDP_BOGUS_SOURCE_IP,
 	GNR_UDP_ALIEN_MESSAGE,
 	GNR_UDP_UNPROCESSED_MESSAGE,
 	GNR_UDP_TX_COMPRESSED,
 	GNR_UDP_RX_COMPRESSED,
 	GNR_UDP_LARGER_HENCE_NOT_COMPRESSED,
+	GNR_CONSOLIDATED_SERVERS,
+	GNR_DUP_DOWNLOADS_IN_CONSOLIDATION,
+	GNR_DISCOVERED_SERVER_GUID,
+	GNR_CHANGED_SERVER_GUID,
+	GNR_GUID_COLLISIONS,
+	GNR_RECEIVED_KNOWN_FW_NODE_INFO,
+	GNR_ATTEMPTED_RESOURCE_SWITCHING,
+	GNR_ATTEMPTED_RESOURCE_SWITCHING_AFTER_ERROR,
+	GNR_SUCCESSFUL_RESOURCE_SWITCHING,
+	GNR_SUCCESSFUL_PLAIN_RESOURCE_SWITCHING,
+	GNR_SUCCESSFUL_RESOURCE_SWITCHING_AFTER_ERROR,
+	GNR_QUEUED_AFTER_SWITCHING,
+	GNR_SUNK_DATA,
+	GNR_IGNORED_DATA,
+	GNR_IGNORING_AFTER_MISMATCH,
+	GNR_IGNORING_TO_PRESERVE_CONNECTION,
+	GNR_IGNORING_DURING_AGGRESSIVE_SWARMING,
+	GNR_IGNORING_REFUSED,
+	GNR_CLIENT_RESOURCE_SWITCHING,
+	GNR_CLIENT_PLAIN_RESOURCE_SWITCHING,
+	GNR_CLIENT_FOLLOWUP_AFTER_ERROR,
+	GNR_PARQ_SLOT_RESOURCE_SWITCHING,
+	GNR_PARQ_RETRY_AFTER_VIOLATION,
+	GNR_PARQ_RETRY_AFTER_KICK_OUT,
+	GNR_PARQ_SLOT_LIMIT_OVERRIDES,
+	GNR_PARQ_QUICK_SLOTS_GRANTED,
+	GNR_PARQ_QUEUE_SENDING_ATTEMPTS,
+	GNR_PARQ_QUEUE_SENT,
+	GNR_PARQ_QUEUE_FOLLOW_UPS,
+	GNR_SEEDING_OF_ORPHAN,
+	GNR_DHT_ESTIMATED_SIZE,
+	GNR_DHT_KBALL_FURTHEST,
+	GNR_DHT_KBALL_CLOSEST,
+	GNR_DHT_ROUTING_BUCKETS,
+	GNR_DHT_ROUTING_LEAVES,
+	GNR_DHT_ROUTING_MAX_DEPTH,
+	GNR_DHT_ROUTING_GOOD_NODES,
+	GNR_DHT_ROUTING_STALE_NODES,
+	GNR_DHT_ROUTING_PENDING_NODES,
+	GNR_DHT_ROUTING_EVICTED_NODES,
+	GNR_DHT_ROUTING_EVICTED_FIREWALLED_NODES,
+	GNR_DHT_ROUTING_PROMOTED_PENDING_NODES,
+	GNR_DHT_COMPLETED_BUCKET_REFRESH,
+	GNR_DHT_DENIED_UNSPLITABLE_BUCKET_REFRESH,
+	GNR_DHT_BUCKET_ALIVE_CHECK,
+	GNR_DHT_ALIVE_PINGS_TO_GOOD_NODES,
+	GNR_DHT_ALIVE_PINGS_TO_STALE_NODES,
+	GNR_DHT_REJECTED_VALUE_ON_QUOTA,
+	GNR_DHT_REJECTED_VALUE_ON_CREATOR,
+	GNR_DHT_KEYS_HELD,
+	GNR_DHT_CACHED_KEYS_HELD,
+	GNR_DHT_VALUES_HELD,
+	GNR_DHT_FETCH_LOCAL_HITS,
+	GNR_DHT_FETCH_LOCAL_CACHED_HITS,
+	GNR_DHT_RETURNED_EXPANDED_VALUES,
+	GNR_DHT_RETURNED_SECONDARY_KEYS,
+	GNR_DHT_CLAIMED_SECONDARY_KEYS,
+	GNR_DHT_RETURNED_EXPANDED_CACHED_VALUES,
+	GNR_DHT_RETURNED_CACHED_SECONDARY_KEYS,
+	GNR_DHT_CLAIMED_CACHED_SECONDARY_KEYS,
+	GNR_DHT_PUBLISHED,
+	GNR_DHT_REMOVED,
+	GNR_DHT_STALE_REPLICATION,
+	GNR_DHT_REPLICATION,
+	GNR_DHT_REPUBLISH,
+	GNR_DHT_SECONDARY_KEY_FETCH,
+	GNR_DHT_DUP_VALUES,
+	GNR_DHT_KUID_COLLISIONS,
+	GNR_DHT_OWN_KUID_COLLISIONS,
+	GNR_DHT_ALT_LOC_LOOKUPS,
+	GNR_DHT_PUSH_PROXY_LOOKUPS,
+	GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS,
+	GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS,
+	GNR_DHT_SEEDING_OF_ORPHAN,
 	
 	GNR_TYPE_COUNT /* number of general stats */
 } gnr_stats_t;
@@ -189,7 +273,8 @@ void gnet_stats_get(gnet_stats_t *stats);
 void gnet_stats_tcp_get(gnet_stats_t *stats);
 void gnet_stats_udp_get(gnet_stats_t *stats);
 void gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *stats);
-const gchar *gnet_stats_drop_reason_to_string(msg_drop_reason_t reason);
+const char *gnet_stats_drop_reason_to_string(msg_drop_reason_t reason);
+const char *gnet_stats_general_to_string(gnr_stats_t type);
 
 #endif /* CORE_SOURCES */
 
