@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 13870 2007-06-18 03:27:25Z cbiere $
+ * $Id: nodes.h 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -28,9 +28,10 @@
 
 #include "common.h"
 
-#include "if/core/hosts.h"
+#include "if/core/guid.h"
 
 #include "lib/host_addr.h"
+#include "lib/gnet_host.h"
 #include "lib/vendors.h"
 
 struct node_id;
@@ -39,7 +40,7 @@ typedef const struct node_id *node_id_t;
 node_id_t node_id_ref(const node_id_t node_id);
 void node_id_unref(const node_id_t node_id);
 
-const gchar *node_id_to_string(const node_id_t node_id);
+const char *node_id_to_string(const node_id_t node_id);
 
 #define node_id_eq_func ((GCompareFunc) node_id_eq)
 gboolean node_id_eq(const node_id_t a, const node_id_t b);
@@ -59,7 +60,7 @@ enum rnode_mode {
  */
 typedef struct rnode_info {
 	/* General information always returned */
-	gchar vendor[4];			/**< Vendor code */
+	char vendor[4];			/**< Vendor code */
 	enum rnode_mode mode;		/**< Running configuration */
 	guint32 answer_flags;		/**< Flags for the "Node Info Reply" message */
 	guint32 op_flags;			/**< Operating flags */
@@ -95,7 +96,7 @@ typedef struct rnode_info {
 	guint64 cpu_sys;			/**< Total kernel CPU time used, in ms */
 	/* Information sent via optional GGEP blocks */
 	guint32 ggep_du;			/**< Daily average uptime, from "DU" */
-	const gchar *ggep_ua;		/**< User-Agent string (atom), from "UA" */
+	const char *ggep_ua;		/**< User-Agent string (atom), from "UA" */
 	host_addr_t ggep_ipv6;		/**< IPv6 address */
 } rnode_info_t;
 
@@ -212,6 +213,7 @@ typedef struct gnet_node_status {
 	guint32 gnet_files_count;	/**< Amount of files shared */
 	guint32 gnet_kbytes_count;	/**< Size of the library, in Kbytes */
 	gboolean gnet_info_known;	/**< Whether previous two values are known */
+	gboolean is_pseudo;			/**< TRUE if it's the pseudo UDP node */
 
 	guint32  sent;				/**< Number of sent packets */
 	guint32  received;			/**< Number of received packets */
@@ -225,10 +227,10 @@ typedef struct gnet_node_status {
 	guint32  n_spam;			/**< Number of messages rated as spam */
 	guint32  n_evil;			/**< Number of messages with evil filenames */
 
-    gint     squeue_sent;
-    gint     squeue_count;
-    gint     mqueue_count;
-    gint     mqueue_percent_used;
+    guint     squeue_sent;
+    guint     squeue_count;
+    guint     mqueue_count;
+    guint     mqueue_percent_used;
     gboolean in_tx_flow_control;
     gboolean in_tx_swift_control;
 
@@ -236,35 +238,35 @@ typedef struct gnet_node_status {
 	 * Traffic statistics -- RAM, 13/05/2002.
 	 */
 
-	gint32   tx_given;			/**< Bytes fed to the TX stack (from top) */
-	gint32   tx_deflated;		/**< Bytes deflated by the TX stack */
-	gint32   tx_written;		/**< Bytes written by the TX stack */
+	guint64   tx_given;			/**< Bytes fed to the TX stack (from top) */
+	guint64   tx_deflated;		/**< Bytes deflated by the TX stack */
+	guint64   tx_written;		/**< Bytes written by the TX stack */
     gboolean tx_compressed;     /**< Is TX traffic compressed */
-    gfloat   tx_compression_ratio; /**< TX compression ratio */
+    float   tx_compression_ratio; /**< TX compression ratio */
     guint32  tx_bps;			/**< TX traffic rate */
 
-	gint32   rx_given;			/**< Bytes fed to the RX stack (from bottom) */
-	gint32   rx_inflated;		/**< Bytes inflated by the RX stack */
-	gint32   rx_read;			/**< Bytes read from the RX stack */
+	guint64   rx_given;			/**< Bytes fed to the RX stack (from bottom) */
+	guint64   rx_inflated;		/**< Bytes inflated by the RX stack */
+	guint64   rx_read;			/**< Bytes read from the RX stack */
     gboolean rx_compressed;     /**< Is RX traffic compressed */
-    gfloat   rx_compression_ratio;/**< RX compression ratio */
-    gfloat   rx_bps;			/**< RX traffic rate */
+    float   rx_compression_ratio;/**< RX compression ratio */
+    float   rx_bps;			/**< RX traffic rate */
 
 	/*
 	 * Gnutella statistics -- RAM, 10/12/2003.
 	 */
 
 	gboolean has_qrp;		/**< Whether node is under QRP control */
-	gfloat qrp_efficiency;	/**< Queries matched / received on QRP control */
+	float qrp_efficiency;	/**< Queries matched / received on QRP control */
 	guint32 rx_queries;		/**< Total amount of queries received */
 	guint32 tx_queries;		/**< Total amount of queries sent */
 	guint32 rx_qhits;		/**< Total amount of hits received */
 	guint32 tx_qhits;		/**< Total amount of hits sent */
 
-	gint qrt_slots;			/**< Amount of slots in leaf's QRT */
-	gint qrt_generation;	/**< Generation number */
-	gint qrt_fill_ratio;	/**< % of filling */
-	gint qrt_pass_throw;	/**< Query limiter pass throw when table filled */
+	guint qrt_slots;			/**< Amount of slots in leaf's QRT */
+	guint qrt_generation;	/**< Generation number */
+	guint qrt_fill_ratio;	/**< % of filling */
+	guint qrt_pass_throw;	/**< Query limiter pass throw when table filled */
 
 	guint32  rt_avg;			/**< Average ping/pong roundtrip time */
 	guint32  rt_last;			/**< Last ping/pong roundtrip time */
@@ -272,20 +274,20 @@ typedef struct gnet_node_status {
 	guint32 tcp_rtt;			/**< RTT in ms over TCP */
 	guint32 udp_rtt;			/**< RTT in ms over UDP */
 
-    gint     shutdown_remain;   /**< Number of seconds before shutdown */
-    gchar    message[128];		/**< Additional information */
+    guint     shutdown_remain;	/**< Number of seconds before shutdown */
+    char    message[128];		/**< Additional information */
 } gnet_node_status_t;
 
 typedef struct gnet_node_info {
     node_id_t node_id;    	/**< Internal node ID */
 
-	gchar gnet_guid[GUID_RAW_SIZE];		/**< Seen on network (can be blank) */
+	struct guid gnet_guid;	/**< Seen on network (can be blank) */
 
-    gchar *error_str;       /**< To sprintf() error strings with vars */
-	const gchar *vendor;	/**< Vendor information (always UTF-8) */
+    char *error_str;		/**< To sprintf() error strings with vars */
+	const char *vendor;		/**< Vendor information (always UTF-8) */
 
-	gint proto_major;		/**< Protocol major number */
-	gint proto_minor;		/**< Protocol minor number */
+	int proto_major;		/**< Protocol major number */
+	int proto_minor;		/**< Protocol minor number */
 	vendor_code_t vcode;	/**< Vendor code (vcode.u32 == 0 when unknown) */
 	gboolean is_pseudo;		/**< TRUE if it's the pseudo UDP node */
 
@@ -358,7 +360,7 @@ typedef enum {
 	
 } gnet_node_state_t;
 
-#define GTA_NORMAL_TTL				4	/**< Regular TTL, for hops-flow */
+#define GTA_NORMAL_TTL			4		/**< Regular TTL, for hops-flow */
 
 /*
  * Nodes callback definitions
@@ -393,7 +395,7 @@ void node_remove_node_flags_changed_listener(node_flags_changed_listener_t);
  * Nodes public interface
  */
 void node_add(const host_addr_t addr, guint16, guint32 flags);
-void node_add_by_name(const gchar *host, guint16, guint32 flags);
+void node_add_by_name(const char *host, guint16, guint32 flags);
 void node_remove_by_id(const node_id_t node_id);
 void node_remove_nodes_by_id(const GSList *node_list);
 gboolean node_get_status(const node_id_t node_id, gnet_node_status_t *s);
@@ -402,8 +404,8 @@ void node_clear_info(gnet_node_info_t *info);
 void node_free_info(gnet_node_info_t *info);
 gboolean node_fill_flags(const node_id_t node_id, gnet_node_flags_t *flags);
 gboolean node_fill_info(const node_id_t node_id, gnet_node_info_t *info);
-const gchar *node_flags_to_string(const gnet_node_flags_t *flags);
-const gchar *node_peermode_to_string(node_peer_t m);
+const char *node_flags_to_string(const gnet_node_flags_t *flags);
+const char *node_peermode_to_string(node_peer_t m);
 
 
 #endif /* CORE_SOURCES */
