@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 15051 2007-12-07 18:52:43Z cbiere $
+ * $Id: search.h 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -29,6 +29,7 @@
 #include "common.h"
 
 #include "lib/misc.h"
+#include "lib/hashlist.h"
 #include "lib/vendors.h"
 #include "if/core/nodes.h"
 
@@ -48,59 +49,6 @@ enum {
 	SEARCH_F_LOCAL		= 1 << 4	/**< Search in local files */
 };
 
-typedef struct {
-	guint8 data[4 + 2];		/* IPv4 address (BE) + Port (LE) */
-} gnet_ipv4_host_t;
-
-typedef struct {
-	guint8 data[16 + 2];	/* IPv6 address + Port (LE) */
-} gnet_ipv6_host_t;
-
-/*
- * Host vectors held in query hits.
- */
-typedef struct gnet_host_vec {
-	gnet_ipv4_host_t *hvec_v4;	/**< Vector of alternate IPv4 locations */
-	gnet_ipv6_host_t *hvec_v6;	/**< Vector of alternate IPv6 locations */
-	guint8 n_ipv4;				/**< Amount of hosts in IPv4 vector */
-	guint8 n_ipv6;				/**< Amount of hosts in IPv6 vector */
-} gnet_host_vec_t;
-
-static inline gint
-gnet_host_vec_count(const gnet_host_vec_t *hvec)
-{
-	return (gint) hvec->n_ipv4 + hvec->n_ipv6; 
-}
-
-static inline gnet_host_t
-gnet_host_vec_get(const gnet_host_vec_t *hvec, guint i)
-{
-	gnet_host_t host;
-	host_addr_t addr;
-	guint16 port;
-
-	g_assert(i < (guint) gnet_host_vec_count(hvec));
-
-	if (i < hvec->n_ipv4) {
-		addr = host_addr_peek_ipv4(hvec->hvec_v4[i].data);
-		port = peek_le16(&hvec->hvec_v4[i].data[4]);
-	} else {
-		i -= hvec->n_ipv4;
-		addr = host_addr_peek_ipv6(hvec->hvec_v6[i].data);
-		port = peek_le16(&hvec->hvec_v6[i].data[16]);
-	}
-
-	gnet_host_set(&host, addr, port);
-	return host;
-}
-
-gnet_host_vec_t *gnet_host_vec_alloc(void);
-void gnet_host_vec_free(gnet_host_vec_t **vec_ptr);
-gnet_host_vec_t *gnet_host_vec_copy(const gnet_host_vec_t *);
-void gnet_host_vec_add(gnet_host_vec_t *, host_addr_t addr, guint16 port);
-gnet_host_vec_t *gnet_host_vec_create(gnet_host_t *, gint hcnt);
-gnet_host_vec_t *gnet_host_vec_from_list(const GSList *);
-
 /*
  * Result sets `status' flags.
  */
@@ -160,10 +108,10 @@ typedef struct gnet_results_set {
 	host_addr_t addr;
 	host_addr_t last_hop;		/**< IP of delivering node */
 
-	const gchar *guid;			/**< Servent's GUID (atom) */
-	const gchar *hostname;		/**< Optional: server's hostname */
-	const gchar *version;		/**< Version information (atom) */
-	const gchar *query;			/**< Optional: Original query string (atom) */
+	const struct guid *guid;	/**< Servent's GUID (atom) */
+	const char *hostname;		/**< Optional: server's hostname */
+	const char *version;		/**< Version information (atom) */
+	const char *query;			/**< Optional: Original query string (atom) */
 	gnet_host_vec_t *proxies;	/**< Optional: known push proxies */
 	GSList *records;
 
@@ -198,12 +146,12 @@ enum {
  * as identified by the parent results_set structure that contains this hit.
  */
 typedef struct gnet_record {
-	const gchar  *name;			/**< File name */
+	const char  *name;			/**< File name */
 	const struct sha1 *sha1;	/**< SHA1 URN (binary form, atom) */
 	const struct tth *tth;		/**< TTH URN (binary form, atom) */
-	const gchar  *tag;			/**< Optional tag data string (atom) */
-	const gchar  *xml;			/**< Optional XML data string (atom) */
-	const gchar  *path;			/**< Optional path (atom) */
+	const char  *tag;			/**< Optional tag data string (atom) */
+	const char  *xml;			/**< Optional XML data string (atom) */
+	const char  *path;			/**< Optional path (atom) */
 	gnet_host_vec_t *alt_locs;	/**< Optional: known alternate locations */
 	filesize_t size;			/**< Size of file, in bytes */
 	time_t  create_time;		/**< Create Time of file; zero if unknown */
@@ -224,7 +172,7 @@ typedef enum {
  */
 
 typedef void (*search_request_listener_t) (
-    query_type_t, const gchar *query, const host_addr_t addr, guint16);
+    query_type_t, const char *query, const host_addr_t addr, guint16);
 
 typedef void (*search_got_results_listener_t)
     (GSList *, const gnet_results_set_t *);
@@ -244,7 +192,7 @@ enum search_new_result {
 
 #ifdef CORE_SOURCES
 
-enum search_new_result search_new(gnet_search_t *ptr, const gchar *,
+enum search_new_result search_new(gnet_search_t *ptr, const char *,
 			time_t create_time, guint lifetime, guint32 timeout, flag_t flags);
 void search_close(gnet_search_t);
 
@@ -258,7 +206,7 @@ gboolean search_is_stopped(gnet_search_t sh);
 void search_reissue(gnet_search_t);
 void search_add_kept(gnet_search_t, guint32 kept);
 
-const gchar *search_query(gnet_search_t);
+const char *search_query(gnet_search_t);
 
 gboolean search_is_active(gnet_search_t);
 gboolean search_is_browse(gnet_search_t);
@@ -278,10 +226,10 @@ void search_free_alt_locs(gnet_record_t *);
 void search_update_items(gnet_search_t, guint32 items);
 
 gboolean search_browse(gnet_search_t,
-	const gchar *hostname, host_addr_t addr, guint16 port,
-	const gchar *guid, const gnet_host_vec_t *proxies, guint32 flags);
-gboolean search_locally(gnet_search_t sh, const gchar *query);
-guint search_handle_magnet(const gchar *url);
+	const char *hostname, host_addr_t addr, guint16 port,
+	const struct guid *guid, const gnet_host_vec_t *proxies, guint32 flags);
+gboolean search_locally(gnet_search_t sh, const char *query);
+guint search_handle_magnet(const char *url);
 
 void search_got_results_listener_add(search_got_results_listener_t);
 void search_got_results_listener_remove(search_got_results_listener_t);
