@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 12761 2007-01-16 16:07:30Z cbiere $
+ * $Id: search.h 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -34,9 +34,10 @@
 
 #ifdef GUI_SOURCES
 
-typedef struct search search_t;
+struct search;
 
-gboolean search_gui_new_search(const gchar *, flag_t flags, search_t **search);
+gboolean search_gui_new_search(const gchar *, flag_t flags, 
+			struct search **search);
 
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_search_h_ */
