@@ -1,5 +1,5 @@
 /*
- * $Id: adns.c 14954 2007-09-28 19:23:13Z bpringlemeir $
+ * $Id: adns.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -35,12 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: adns.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
+RCSID("$Id: adns.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #include "adns.h"
 #include "atoms.h"
+#include "debug.h"
 #include "inputevt.h"
-#include "misc.h"
+#include "ascii.h"
 #include "glib-missing.h"
 #include "tm.h"
 #include "walloc.h"
@@ -48,8 +49,6 @@ RCSID("$Id: adns.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
 
 #include "override.h"		/* Must be the last header included */
 
-static guint32 common_dbg = 0;	/**< @bug XXX need to init lib's props --RAM */
-
 /* private data types */
 
 struct adns_common {
@@ -64,16 +63,16 @@ struct adns_reverse_query {
 
 struct adns_query {
 	enum net_type net;
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 };
 
 struct adns_reply {
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 	host_addr_t addrs[10];
 };
 
 struct adns_reverse_reply {
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 	host_addr_t addr;
 };
 
@@ -95,13 +94,13 @@ struct adns_response {
 
 typedef struct adns_async_write {
 	struct adns_request req;	/**< The original ADNS request */
-	gchar *buf;					/**< Remaining data to write; walloc()ed */
+	char *buf;					/**< Remaining data to write; walloc()ed */
 	size_t pos;					/**< Read position */
 	size_t size;				/**< Size of the buffer */
 } adns_async_write_t;
 
 typedef struct adns_cache_entry {
-	const gchar *hostname;		/**< atom */
+	const char *hostname;		/**< atom */
 	time_t timestamp;
 	size_t n;				/**< Number of addr items */
 	guint id;
@@ -139,12 +138,12 @@ count_addrs(const host_addr_t *addrs, size_t m)
  */
 #define ADNS_CACHE_MAX_SIZE (1024)
 
-static const gchar adns_process_title[] = "DNS helper for gtk-gnutella";
+static const char adns_process_title[] = "DNS helper for gtk-gnutella";
 
 typedef struct adns_cache_struct {
 	GHashTable *ht;
 	guint pos;
-	gint timeout;
+	int timeout;
 	adns_cache_entry_t *entries[ADNS_CACHE_MAX_SIZE];
 } adns_cache_t;
 
@@ -152,7 +151,7 @@ static adns_cache_t *adns_cache = NULL;
 
 /* private variables */
 
-static gint adns_query_fd = -1;
+static int adns_query_fd = -1;
 static guint adns_query_event_id = 0;
 static guint adns_reply_event_id = 0;
 static gboolean is_helper = FALSE;		/**< Are we the DNS helper process? */
@@ -256,7 +255,7 @@ adns_cache_free(adns_cache_t *cache)
  */
 static void
 adns_cache_add(adns_cache_t *cache, time_t now,
-	const gchar *hostname, const host_addr_t *addrs, size_t n)
+	const char *hostname, const host_addr_t *addrs, size_t n)
 {
 	adns_cache_entry_t *entry;
 	size_t i;
@@ -307,7 +306,7 @@ adns_cache_add(adns_cache_t *cache, time_t now,
  */
 static size_t
 adns_cache_lookup(adns_cache_t *cache, time_t now,
-	const gchar *hostname, host_addr_t *addrs, size_t n)
+	const char *hostname, host_addr_t *addrs, size_t n)
 {
 	adns_cache_entry_t *entry;
 
@@ -352,7 +351,7 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
  * should only be used with a blocking `fd'.
  */
 static gboolean
-adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
+adns_do_transfer(int fd, gpointer buf, size_t len, gboolean do_write)
 {
 	ssize_t ret;
 	size_t n = len;
@@ -385,7 +384,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
 			return FALSE;
 		} else if (ret > 0) {
 			n -= ret;
-			buf = (gchar *) buf + ret;
+			buf = (char *) buf + ret;
 		}
 	}
 
@@ -398,7 +397,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
  * @return TRUE on success, FALSE if the operation failed
  */
 static gboolean
-adns_do_read(gint fd, gpointer buf, size_t len)
+adns_do_read(int fd, gpointer buf, size_t len)
 {
 	return adns_do_transfer(fd, buf, len, FALSE);
 }
@@ -409,7 +408,7 @@ adns_do_read(gint fd, gpointer buf, size_t len)
  * @return TRUE on success, FALSE if the operation failed
  */
 static gboolean
-adns_do_write(gint fd, gpointer buf, size_t len)
+adns_do_write(int fd, gpointer buf, size_t len)
 {
 	return adns_do_transfer(fd, buf, len, TRUE);
 }
@@ -430,7 +429,7 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 	if (req->common.reverse) {
 		const struct adns_reverse_query *query = &req->query.reverse;
 		struct adns_reverse_reply *reply = &ans->reply.reverse;
-		const gchar *host;
+		const char *host;
 
 
 		if (common_dbg > 1) {
@@ -478,7 +477,7 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
  * in case of non-recoverable error during read or write.
  */
 static void
-adns_helper(gint fd_in, gint fd_out)
+adns_helper(int fd_in, int fd_out)
 {
 	g_set_prgname(adns_process_title);
 	gm_setproctitle(g_get_prgname());
@@ -616,7 +615,7 @@ adns_reply_ready(const struct adns_response *ans)
  * the reply pipe will be closed and the callback will be lost.
  */
 static void
-adns_reply_callback(gpointer data, gint source, inputevt_cond_t condition)
+adns_reply_callback(gpointer data, int source, inputevt_cond_t condition)
 {
 	static struct adns_response ans;
 	static gpointer buf;
@@ -723,7 +722,7 @@ adns_async_write_free(adns_async_write_t *remain)
  * closed and the blocking adns_fallback() will be invoked.
  */
 static void
-adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
+adns_query_callback(gpointer data, int dest, inputevt_cond_t condition)
 {
 	adns_async_write_t *remain = data;
 
@@ -791,8 +790,8 @@ done:
 void
 adns_init(void)
 {
-	gint fd_query[2] = {-1, -1};
-	gint fd_reply[2] = {-1, -1};
+	int fd_query[2] = {-1, -1};
+	int fd_reply[2] = {-1, -1};
 	pid_t pid;
 
 	if (-1 == pipe(fd_query) || -1 == pipe(fd_reply)) {
@@ -874,7 +873,7 @@ prefork_failure:
 static gboolean
 adns_send_request(const struct adns_request *req)
 {
-	gchar buf[sizeof *req];
+	char buf[sizeof *req];
 	size_t size;
 	ssize_t written;
 
@@ -955,7 +954,7 @@ adns_send_request(const struct adns_request *req)
  * returned, adns_resolve() returns FALSE.
  */
 gboolean
-adns_resolve(const gchar *hostname, enum net_type net,
+adns_resolve(const char *hostname, enum net_type net,
 	adns_callback_t user_callback, gpointer user_data)
 {
 	struct adns_request req;
