@@ -1,5 +1,5 @@
 /*
- * $Id: aging.h 13502 2007-05-05 18:24:07Z cbiere $
+ * $Id: aging.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -37,7 +37,7 @@ typedef void (*aging_free_t)(gpointer value, gpointer udata);
 
 struct aging;
 
-struct aging *aging_make(gint delay, GHashFunc hash, GEqualFunc eq,
+struct aging *aging_make(int delay, GHashFunc hash, GEqualFunc eq,
 			aging_free_t kfree);
 
 void aging_destroy(struct aging *);
