@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.c 13540 2007-05-07 16:41:10Z rmanfredi $
+ * $Id: atoms.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -42,7 +42,7 @@
 
 #include "atoms.h"
 
-RCSID("$Id: atoms.c 13540 2007-05-07 16:41:10Z rmanfredi $")
+RCSID("$Id: atoms.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #if 0
 #define PROTECT_ATOMS
@@ -90,8 +90,8 @@ union mem_chunk {
   guint16  u16;
   guint32  u32;
   guint64  u64;
-  gfloat     f;
-  gdouble    d;
+  float     f;
+  double    d;
 };
 
 #define ATOM_TYPE_MASK	((size_t) 0x07)
@@ -111,7 +111,7 @@ typedef struct atom {
 #ifdef ATOMS_HAVE_MAGIC
 	atom_prot_magic_t magic;
 #endif /* PROTECT_ATOMS */
-	gint refcnt;			/**< Amount of references */
+	int refcnt;			/**< Amount of references */
 	/* Ensure `arena' is properly aligned */
 	union mem_chunk arena[1];			/* Start of user arena */
 } atom_t;
@@ -121,7 +121,7 @@ typedef struct atom {
 static inline atom_t *
 atom_from_arena(gconstpointer key)
 {
-	return (gpointer) ((gchar *) key - ARENA_OFFSET);
+	return (gpointer) ((char *) key - ARENA_OFFSET);
 }
 
 static inline void
@@ -277,7 +277,7 @@ mem_protect(gpointer ptr, size_t size)
 	size_t ps;
 
 	ps = compat_pagesize();
-	p = (gchar *) ptr - (uintptr_t) ptr % ps;
+	p = (char *) ptr - (uintptr_t) ptr % ps;
 	size = round_size(ps, size);
 
 	if (-1 == mprotect(p, size, PROT_READ))
@@ -296,7 +296,7 @@ mem_unprotect(gpointer ptr, size_t size)
 	size_t ps;
 
 	ps = compat_pagesize();
-	p = (gchar *) ptr - (uintptr_t) ptr % ps;
+	p = (char *) ptr - (uintptr_t) ptr % ps;
 	size = round_size(ps, size);
 
 	if (-1 == mprotect(p, size, PROT_READ | PROT_WRITE))
@@ -401,13 +401,13 @@ atom_dealloc(atom_t *a, size_t size)
 #endif /* PROTECT_ATOMS */
 
 typedef size_t (*len_func_t)(gconstpointer v);
-typedef const gchar *(*str_func_t)(gconstpointer v);
+typedef const char *(*str_func_t)(gconstpointer v);
 
 /**
  * Description of atom types.
  */
 typedef struct table_desc {
-	const gchar *type;			/**< Type of atoms */
+	const char *type;			/**< Type of atoms */
 	GHashTable *table;			/**< Table of atoms: "atom value" => 1 */
 	GHashFunc hash_func;		/**< Hashing function for atoms */
 	GCompareFunc eq_func;		/**< Atom equality function */
@@ -416,17 +416,19 @@ typedef struct table_desc {
 } table_desc_t;
 
 static size_t str_len(gconstpointer v);
-static const gchar *str_str(gconstpointer v);
+static const char *str_str(gconstpointer v);
 static size_t guid_len(gconstpointer v);
-static const gchar *guid_str(gconstpointer v);
+static const char *guid_str(gconstpointer v);
 static size_t sha1_len(gconstpointer v);
-static const gchar *sha1_str(gconstpointer v);
+static const char *sha1_str(gconstpointer v);
 static size_t tth_len(gconstpointer v);
-static const gchar *tth_str(gconstpointer v);
+static const char *tth_str(gconstpointer v);
 static size_t uint64_len(gconstpointer v);
-static const gchar *uint64_str(gconstpointer v);
+static const char *uint64_str(gconstpointer v);
 static size_t filesize_len(gconstpointer v);
-static const gchar *filesize_str(gconstpointer v);
+static const char *filesize_str(gconstpointer v);
+static size_t uint32_len(gconstpointer v);
+static const char *uint32_str(gconstpointer v);
 
 /**
  * The set of all atom types we know about.
@@ -439,6 +441,7 @@ static table_desc_t atoms[] = {
 	{ "uint64",	NULL, uint64_hash, uint64_eq,   uint64_len, uint64_str},/* 4 */
 	{ "filesize",
 		NULL, filesize_hash, filesize_eq, filesize_len, filesize_str},  /* 5 */
+	{ "uint32",	NULL, uint32_hash, uint32_eq,   uint32_len, uint32_str},/* 6 */
 };
 
 static GHashTable *ht_all_atoms;
@@ -449,16 +452,16 @@ static GHashTable *ht_all_atoms;
 static size_t 
 str_len(gconstpointer v)
 {
-	return strlen((const gchar *) v) + 1;
+	return strlen((const char *) v) + 1;
 }
 
 /**
  * @return printable form of a string, i.e. self.
  */
-static const gchar *
+static const char *
 str_str(gconstpointer v)
 {
-	return (const gchar *) v;
+	return (const char *) v;
 }
 
 /**
@@ -512,7 +515,7 @@ guid_hash(gconstpointer key)
 /**
  * Test two GUIDs for equality.
  */
-gint
+int
 guid_eq(gconstpointer a, gconstpointer b)
 {
 /* FIXME:	Disabled because of alignment problems
@@ -546,10 +549,11 @@ guid_len(gconstpointer unused_guid)
 /**
  * @return printable form of a GUID, as pointer to static data.
  */
-static const gchar *
+static const char *
 guid_str(gconstpointer v)
 {
-	return guid_hex_str((const gchar *) v);
+	const struct guid *guid = v;
+	return guid_hex_str(guid);
 }
 
 /**
@@ -570,7 +574,7 @@ sha1_hash(gconstpointer key)
  * @attention
  * NB: This routine is visible for the download mesh.
  */
-gint
+int
 sha1_eq(gconstpointer a, gconstpointer b)
 {
 	return a == b || 0 == memcmp(a, b, SHA1_RAW_SIZE);
@@ -589,7 +593,7 @@ sha1_len(gconstpointer unused_sha1)
 /**
  * @return printable form of a SHA1, as pointer to static data.
  */
-static const gchar *
+static const char *
 sha1_str(gconstpointer sha1)
 {
 	return sha1_base32(sha1);
@@ -610,7 +614,7 @@ tth_hash(gconstpointer key)
  * @attention
  * NB: This routine is visible for the download mesh.
  */
-gint
+int
 tth_eq(gconstpointer a, gconstpointer b)
 {
 	return a == b || 0 == memcmp(a, b, TTH_RAW_SIZE);
@@ -629,7 +633,7 @@ tth_len(gconstpointer unused_tth)
 /**
  * @return printable form of a TTH, as pointer to static data.
  */
-static const gchar *
+static const char *
 tth_str(gconstpointer tth)
 {
 	return tth_base32(tth);
@@ -655,13 +659,24 @@ filesize_len(gconstpointer unused_v)
 	return sizeof(filesize_t);
 }
 
+/**
+ * @return length of a 32-bit integer.
+ */
+static size_t
+uint32_len(gconstpointer unused_v)
+{
+	(void) unused_v;
+	return sizeof(guint32);
+}
+
+
 /**
  * @return printable form of a 64-bit integer, as pointer to static data.
  */
-static const gchar *
+static const char *
 uint64_str(gconstpointer v)
 {
-	static gchar buf[UINT64_DEC_BUFLEN];
+	static char buf[UINT64_DEC_BUFLEN];
 
 	uint64_to_string_buf(*(const guint64 *) v, buf, sizeof buf);
 	return buf;
@@ -672,7 +687,7 @@ uint64_str(gconstpointer v)
  *
  * @return whether both referenced 64-bit integers are equal.
  */
-gint
+int
 uint64_eq(gconstpointer a, gconstpointer b)
 {
 	return *(const guint64 *) a == *(const guint64 *) b;
@@ -693,10 +708,10 @@ uint64_hash(gconstpointer p)
 /**
  * @return printable form of a filesize_t, as pointer to static data.
  */
-static const gchar *
+static const char *
 filesize_str(gconstpointer v)
 {
-	static gchar buf[UINT64_DEC_BUFLEN];
+	static char buf[UINT64_DEC_BUFLEN];
 
 	uint64_to_string_buf(*(const filesize_t *) v, buf, sizeof buf);
 	return buf;
@@ -707,7 +722,7 @@ filesize_str(gconstpointer v)
  *
  * @return whether both referenced 64-bit integers are equal.
  */
-gint
+int
 filesize_eq(gconstpointer a, gconstpointer b)
 {
 	return *(const filesize_t *) a == *(const filesize_t *) b;
@@ -725,6 +740,40 @@ filesize_hash(gconstpointer p)
 	return v ^ (v >> 32);
 }
 
+/**
+ * Test two 32-bit integers for equality.
+ *
+ * @return whether both referenced 32-bit integers are equal.
+ */
+int
+uint32_eq(gconstpointer a, gconstpointer b)
+{
+	return *(const guint32 *) a == *(const guint32 *) b;
+}
+
+/**
+ * Calculate the 32-bit hash of a 32-bit integer
+ *
+ * @return the 32-bit hash value for the referenced 32-bit integer.
+ */
+guint
+uint32_hash(gconstpointer p)
+{
+	guint32 v = *(const guint32 *) p;
+	return v;
+}
+
+/**
+ * @return printable form of a 32-bit integer, as pointer to static data.
+ */
+static const char *
+uint32_str(gconstpointer v)
+{
+	static char buf[UINT32_DEC_BUFLEN];
+
+	uint32_to_string_buf(*(const guint32 *) v, buf, sizeof buf);
+	return buf;
+}
 
 /**
  * Initialize atom structures.
@@ -794,7 +843,7 @@ atom_get(enum atom_type type, gconstpointer key)
 #endif /* __GNUC__ */
 	
     g_assert(key != NULL);
-	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert((int) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
 
 	td = &atoms[type];		/* Where atoms of this type are held */
 
@@ -873,7 +922,7 @@ atom_free(enum atom_type type, gconstpointer key)
 	atom_t *a;
 
     g_assert(key != NULL);
-	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert((int) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
 
 	size = atom_is_registered(type, key);
 	g_assert(size > 0);
@@ -902,7 +951,7 @@ atom_free(enum atom_type type, gconstpointer key)
 
 /** Information about given spot. */
 struct spot {
-	gint count;			/**< Amount of allocation/free performed at spot */
+	int count;			/**< Amount of allocation/free performed at spot */
 };
 
 /**
@@ -911,11 +960,11 @@ struct spot {
  * @returns the atom's value.
  */
 gconstpointer
-atom_get_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
+atom_get_track(enum atom_type type, gconstpointer key, char *file, int line)
 {
 	gconstpointer atom;
 	atom_t *a;
-	gchar buf[512];
+	char buf[512];
 	gpointer k;
 	gpointer v;
 	struct spot *sp;
@@ -954,7 +1003,7 @@ tracking_free_kv(gpointer key, gpointer value, gpointer uu_user)
 {
 	(void) uu_user;
 
-	g_free(key);
+	G_FREE_NULL(key);
 	wfree(value, sizeof(struct spot));
 	return TRUE;
 }
@@ -975,10 +1024,10 @@ destroy_tracking_table(GHashTable *h)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
+atom_free_track(enum atom_type type, gconstpointer key, char *file, int line)
 {
 	atom_t *a;
-	gchar buf[512];
+	char buf[512];
 	gpointer k;
 	gpointer v;
 	struct spot *sp;
@@ -1016,16 +1065,16 @@ static void
 dump_tracking_entry(gpointer key, gpointer value, gpointer user)
 {
 	struct spot *sp = (struct spot *) value;
-	const gchar *what = (const gchar *) user;
+	const char *what = (const char *) user;
 
-	g_warning("%10d %s at \"%s\"", sp->count, what, (gchar *) key);
+	g_warning("%10d %s at \"%s\"", sp->count, what, (char *) key);
 }
 
 /**
  * Dump the values held in the tracking table `h'.
  */
 static void
-dump_tracking_table(gpointer atom, GHashTable *h, gchar *what)
+dump_tracking_table(gpointer atom, GHashTable *h, char *what)
 {
 	guint count = g_hash_table_size(h);
 
