@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.h 13946 2007-06-24 00:43:12Z cbiere $
+ * $Id: atoms.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -47,8 +47,9 @@ enum atom_type {
 	ATOM_GUID,		/**< GUIDs (binary, 16 bytes) */
 	ATOM_SHA1,		/**< SHA1 (binary, 20 bytes) */
 	ATOM_TTH,		/**< TTH (binary, 24 bytes) */
-	ATOM_UINT64,	/**< integers (binary, 8 bytes) */
+	ATOM_UINT64,	/**< unsigned 64-bit integers (binary, 8 bytes) */
 	ATOM_FILESIZE,	/**< filesize_t (binary) */
+	ATOM_UINT32,	/**< unsigned 32-bit integers (binary, 4 bytes) */
 
 	NUM_ATOM_TYPES
 };
@@ -79,6 +80,9 @@ void atom_free(enum atom_type type, gconstpointer key);
 #define atom_uint64_get(k)	atom_get_track(ATOM_UINT64, (k), _WHERE_, __LINE__)
 #define atom_uint64_free(k)	atom_free_track(ATOM_UINT64, (k), _WHERE_, __LINE__)
 
+#define atom_uint32_get(k)	atom_get_track(ATOM_UINT32, (k), _WHERE_, __LINE__)
+#define atom_uint32_free(k)	atom_free_track(ATOM_UINT32, (k), _WHERE_, __LINE__)
+
 #define atom_filesize_get(k) \
 	atom_get_track(ATOM_FILESIZE, (k), _WHERE_, __LINE__)
 #define atom_filesize_free(k) \
@@ -91,26 +95,26 @@ void atom_free(enum atom_type type, gconstpointer key);
 
 #else	/* !TRACK_ATOMS */
 
-static inline const gchar *
-atom_str_get(const gchar *k)
+static inline const char *
+atom_str_get(const char *k)
 {
 	return atom_get(ATOM_STRING, k);
 }
 
 static inline void
-atom_str_free(const gchar *k)
+atom_str_free(const char *k)
 {
 	return atom_free(ATOM_STRING, k);
 }
 
-static inline const gchar *
-atom_guid_get(const gchar *k)
+static inline const struct guid *
+atom_guid_get(const struct guid *k)
 {
 	return atom_get(ATOM_GUID, k);
 }
 
 static inline void
-atom_guid_free(const gchar *k)
+atom_guid_free(const struct guid *k)
 {
 	return atom_free(ATOM_GUID, k);
 }
@@ -163,6 +167,18 @@ atom_filesize_free(const filesize_t *k)
 	return atom_free(ATOM_FILESIZE, k);
 }
 
+static inline const guint32 *
+atom_uint32_get(const guint32 *k)
+{
+	return atom_get(ATOM_UINT32, k);
+}
+
+static inline void
+atom_uint32_free(const guint32 *k)
+{
+	return atom_free(ATOM_UINT32, k);
+}
+
 #endif	/* TRACK_ATOMS */
 
 /*
@@ -176,21 +192,23 @@ void atoms_close(void);
  * Hash functions and equality checks
  */
 guint filesize_hash(gconstpointer key);
-gint filesize_eq(gconstpointer a, gconstpointer b);
+int filesize_eq(gconstpointer a, gconstpointer b);
 guint sha1_hash(gconstpointer key);
-gint sha1_eq(gconstpointer a, gconstpointer b);
+int sha1_eq(gconstpointer a, gconstpointer b);
 guint tth_hash(gconstpointer key);
-gint tth_eq(gconstpointer a, gconstpointer b);
+int tth_eq(gconstpointer a, gconstpointer b);
 guint guid_hash(gconstpointer key);
-gint guid_eq(gconstpointer a, gconstpointer b);
+int guid_eq(gconstpointer a, gconstpointer b);
 guint uint64_hash(gconstpointer key);
-gint uint64_eq(gconstpointer a, gconstpointer b);
+int uint64_eq(gconstpointer a, gconstpointer b);
 guint binary_hash(const guchar *key, guint len);
+guint uint32_hash(gconstpointer key);
+int uint32_eq(gconstpointer a, gconstpointer b);
 
 #ifdef TRACK_ATOMS
 gconstpointer atom_get_track(enum atom_type, gconstpointer key,
-			gchar *file, gint line);
-void atom_free_track(enum atom_type, gconstpointer key, gchar *file, gint line);
+			char *file, int line);
+void atom_free_track(enum atom_type, gconstpointer key, char *file, int line);
 #endif
 
 
@@ -213,11 +231,12 @@ struct sha1;
 struct tth;
 
 GENERATE_ATOM_FREE_NULL(filesize, filesize_t *)
-GENERATE_ATOM_FREE_NULL(guid, gchar *)
+GENERATE_ATOM_FREE_NULL(guid, struct guid *)
 GENERATE_ATOM_FREE_NULL(sha1, struct sha1 *)
-GENERATE_ATOM_FREE_NULL(str, gchar *)
+GENERATE_ATOM_FREE_NULL(str, char *)
 GENERATE_ATOM_FREE_NULL(tth, struct tth *)
 GENERATE_ATOM_FREE_NULL(uint64, guint64 *)
+GENERATE_ATOM_FREE_NULL(uint32, guint32 *)
 #undef GENERATE_ATOM_FREE_NULL
 
 /**
@@ -236,11 +255,12 @@ atom_ ## name ## _change(const type *atom_ptr, const type value) \
 }
 
 GENERATE_ATOM_CHANGE(filesize, filesize_t *)
-GENERATE_ATOM_CHANGE(guid, gchar *)
+GENERATE_ATOM_CHANGE(guid, struct guid *)
 GENERATE_ATOM_CHANGE(sha1, struct sha1 *)
-GENERATE_ATOM_CHANGE(str, gchar *)
+GENERATE_ATOM_CHANGE(str, char *)
 GENERATE_ATOM_CHANGE(tth, struct tth *)
 GENERATE_ATOM_CHANGE(uint64, guint64 *)
+GENERATE_ATOM_CHANGE(uint32, guint32 *)
 #undef GENERATE_ATOM_CHANGE
 
 #endif	/* _atoms_h_ */
