@@ -1,5 +1,5 @@
 /*
- * $Id: bg.c 13773 2007-05-27 21:29:05Z rmanfredi $
+ * $Id: bg.c 16129 2009-03-12 06:47:48Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,9 +39,10 @@
 
 #include "common.h"
 
-RCSID("$Id: bg.c 13773 2007-05-27 21:29:05Z rmanfredi $")
+RCSID("$Id: bg.c 16129 2009-03-12 06:47:48Z rmanfredi $")
 
 #include "bg.h"
+#include "debug.h"
 #include "misc.h"
 #include "tm.h"
 #include "walloc.h"
@@ -57,8 +58,6 @@ enum bgtask_magic {
 #define MIN_LIFE		40000			/**< Min lifetime per task, in usecs */
 #define DELTA_FACTOR	4				/**< Max variations are 400% */
 
-static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
-
 /**
  * Internal representation of a user-defined task.
  *
@@ -75,27 +74,27 @@ static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
 struct bgtask {
 	enum bgtask_magic magic;/**< Magic number */
 	guint32 flags;			/**< Operating flags */
-	const gchar *name;		/**< Task name */
-	gint step;				/**< Current processing step */
-	gint seqno;				/**< Number of calls at same step */
+	const char *name;		/**< Task name */
+	int step;				/**< Current processing step */
+	int seqno;				/**< Number of calls at same step */
 	bgstep_cb_t *stepvec;	/**< Set of steps to run in sequence */
-	gint stepcnt;			/**< Amount of steps in the `stepvec' array */
+	int stepcnt;			/**< Amount of steps in the `stepvec' array */
 	gpointer ucontext;		/**< User context */
-	time_t ctime;			/**< Creation time */
-	gint wtime;				/**< Wall-clock run time sofar, in ms */
+	time_t created;			/**< Creation time */
+	int wtime;				/**< Wall-clock run time sofar, in ms */
 	bgclean_cb_t uctx_free;	/**< Free routine for context */
 	bgdone_cb_t done_cb;	/**< Called when done */
 	gpointer done_arg;		/**< "done" callback argument */
-	gint exitcode;			/**< Final "exit" code */
+	int exitcode;			/**< Final "exit" code */
 	bgsig_t signal;			/**< Last signal delivered */
 	GSList *signals;		/**< List of signals pending delivery */
 	jmp_buf env;			/**< Only valid when TASK_F_RUNNING */
 	tm_t start;				/**< Start time of scheduling "tick" */
-	gint ticks;				/**< Scheduling ticks for time slice */
-	gint ticks_used;		/**< Amount of ticks used by processing step */
-	gint prev_ticks;		/**< Ticks used when measuring `elapsed' below */
-	gint elapsed;			/**< Elapsed during last run, in usec */
-	gdouble tick_cost;		/**< Time in ms. spent by each tick */
+	int ticks;				/**< Scheduling ticks for time slice */
+	int ticks_used;		/**< Amount of ticks used by processing step */
+	int prev_ticks;		/**< Ticks used when measuring `elapsed' below */
+	int elapsed;			/**< Elapsed during last run, in usec */
+	double tick_cost;		/**< Time in ms. spent by each tick */
 	bgsig_cb_t sigh[BG_SIG_COUNT];	/**< Signal handlers */
 
 	/*
@@ -135,7 +134,7 @@ bg_task_check(const struct bgtask * const bt)
  * Access routines to internal fields.
  */
 
-gint
+int
 bg_task_seqno(const struct bgtask *bt)
 {
 	bg_task_check(bt);
@@ -149,7 +148,7 @@ bg_task_context(const struct bgtask *bt)
 	return bt->ucontext;
 }
 
-static gint runcount;
+static int runcount;
 static GSList *runq;
 static GSList *sleepq;
 static GSList *dead_tasks;
@@ -211,7 +210,7 @@ bg_sched_pick(void)
 static void
 bg_task_suspend(struct bgtask *bt)
 {
-	tm_t end, delta;
+	tm_t end;
 	time_delta_t elapsed;
 
 	bg_task_check(bt);
@@ -225,8 +224,7 @@ bg_task_suspend(struct bgtask *bt)
 	 */
 
 	tm_now_exact(&end);
-	tm_elapsed(&delta, &end, &bt->start);
-	elapsed = tm2us(&delta); 
+	elapsed = tm_elapsed_us(&end, &bt->start); 
 
 	/*
 	 * Compensate any clock adjustment by reusing the previous value we
@@ -253,12 +251,12 @@ bg_task_suspend(struct bgtask *bt)
 	 */
 
 	if (!(bt->flags & TASK_F_NOTICK)) {
-		gdouble new_cost =
+		double new_cost =
 			(4 * bt->tick_cost + (elapsed / bt->ticks_used)) / 5.0;
 
 		if (common_dbg > 4)
-			g_message("BGTASK \"%s\" total=%d msecs, elapsed=%lu, ticks=%d, "
-				"used=%d, tick_cost=%f usecs (was %f)",
+			g_message("BGTASK \"%s\" total=%d msecs, elapsed=%lu usecs, "
+				"ticks=%d, used=%d, tick_cost=%f usecs (was %f)",
 				bt->name, bt->wtime, (gulong)elapsed, bt->ticks, bt->ticks_used,
 				new_cost, bt->tick_cost);
 
@@ -370,16 +368,16 @@ bg_task_alloc(void)
  * @returns an opaque handle.
  */
 struct bgtask *
-bg_task_create(const gchar *name,	/**< Task name (for tracing) */
+bg_task_create(const char *name,	/**< Task name (for tracing) */
 	const bgstep_cb_t *steps,		/**< Work to perform (copied) */
-	gint stepcnt,					/**< Number of steps */
+	int stepcnt,					/**< Number of steps */
 	gpointer ucontext,				/**< User context */
 	bgclean_cb_t ucontext_free,		/**< Free routine for context */
 	bgdone_cb_t done_cb,			/**< Notification callback when done */
 	gpointer done_arg)				/**< Callback argument */
 {
 	struct bgtask *bt;
-	gint stepsize;
+	int stepsize;
 
 	g_assert(stepcnt > 0);
 	g_assert(steps);
@@ -423,9 +421,9 @@ bg_task_create(const gchar *name,	/**< Task name (for tracing) */
  */
 struct bgtask *
 bg_daemon_create(
-	const gchar *name,			/**< Task name (for tracing) */
+	const char *name,			/**< Task name (for tracing) */
 	const bgstep_cb_t *steps,	/**< Work to perform (copied) */
-	gint stepcnt,				/**< Number of steps */
+	int stepcnt,				/**< Number of steps */
 	gpointer ucontext,			/**< User context */
 	bgclean_cb_t ucontext_free,	/**< Free routine for context */
 	bgstart_cb_t start_cb,		/**< Starting working on an item */
@@ -434,7 +432,7 @@ bg_daemon_create(
 	bgnotify_cb_t notify)		/**< Start/Stop notify (optional) */
 {
 	struct bgtask *bt;
-	gint stepsize;
+	int stepsize;
 
 	g_assert(stepcnt > 0);
 	g_assert(steps);
@@ -489,8 +487,8 @@ static void
 bg_task_free(struct bgtask *bt)
 {
 	GSList *l;
-	gint stepsize;
-	gint count;
+	int stepsize;
+	int count;
 
 	g_assert(bt);
 	g_assert(BGTASK_DEAD_MAGIC == bt->magic);
@@ -613,7 +611,7 @@ bg_task_terminate(struct bgtask *bt)
  * We exit immediately, not returning to the user code.
  */
 void
-bg_task_exit(struct bgtask *bt, gint code)
+bg_task_exit(struct bgtask *bt, int code)
 {
 	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
@@ -651,7 +649,7 @@ bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
  *
  * @returns -1 if the task could not be signalled.
  */
-static gint
+static int
 bg_task_kill(struct bgtask *bt, bgsig_t sig)
 {
 	bgsig_cb_t sighandler;
@@ -807,7 +805,7 @@ bg_task_cancel(struct bgtask *bt)
  * all its ticks and it matters for the computation of the cost per tick.
  */
 void
-bg_task_ticks_used(struct bgtask *bt, gint used)
+bg_task_ticks_used(struct bgtask *bt, int used)
 {
 	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
@@ -902,9 +900,9 @@ void
 bg_sched_timer(gboolean overloaded)
 {
 	struct bgtask * volatile bt;
-	volatile gint remain = MAX_LIFE;
-	gint target;
-	volatile gint ticks;
+	volatile int remain = MAX_LIFE;
+	int target;
+	volatile int ticks;
 	bgret_t ret;
 
 	g_assert(current_task == NULL);
@@ -946,7 +944,10 @@ bg_sched_timer(gboolean overloaded)
 			g_assert(bt->prev_ticks >= 0);
 			g_assert(bt->prev_ticks <= INT_MAX / DELTA_FACTOR);
 
-			ticks = 1 + target / bt->tick_cost;
+			if (target < bt->tick_cost * (INT_MAX / DELTA_FACTOR - 1))
+				ticks = 1 + target / bt->tick_cost;
+			else
+				ticks = INT_MAX / DELTA_FACTOR;
 
 			if (bt->prev_ticks) {
 				if (ticks > bt->prev_ticks * DELTA_FACTOR) {
