@@ -1,5 +1,5 @@
 /*
- * $Id: endian.h 12848 2007-02-14 20:52:01Z cbiere $
+ * $Id: endian.h 16180 2009-03-16 23:11:00Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -33,17 +33,44 @@
  * or little-endian.
  */
 
-static inline ALWAYS_INLINE guchar
-peek_u8(gconstpointer p)
+static inline ALWAYS_INLINE unsigned char
+peek_u8(const void *p)
 {
-	const guchar *q = p;
+	const unsigned char *q = p;
 	return q[0] & 0xff;
 }
 
-static inline guint16
-peek_be16(gconstpointer p)
+static inline ALWAYS_INLINE guint16
+peek_u16(const void *p)
 {
-	const guchar *q = p;
+	const unsigned char *q = p;
+	guint16 v;
+	memcpy(&v, q, sizeof v);
+	return v;
+}
+
+static inline ALWAYS_INLINE guint32
+peek_u32(const void *p)
+{
+	const unsigned char *q = p;
+	guint32 v;
+	memcpy(&v, q, sizeof v);
+	return v;
+}
+
+static inline ALWAYS_INLINE guint64
+peek_u64(const void *p)
+{
+	const unsigned char *q = p;
+	guint64 v;
+	memcpy(&v, q, sizeof v);
+	return v;
+}
+
+static inline guint16
+peek_be16(const void *p)
+{
+	const unsigned char *q = p;
 	guint16 v;
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -55,9 +82,9 @@ peek_be16(gconstpointer p)
 }
 
 static inline guint32
-peek_be32(gconstpointer p)
+peek_be32(const void *p)
 {
-	const guchar *q = p;
+	const unsigned char *q = p;
 	guint32 v;
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -68,10 +95,24 @@ peek_be32(gconstpointer p)
 	return v;
 }
 
-static inline guint16
-peek_le16(gconstpointer p)
+static inline guint64
+peek_be64(const void *p)
 {
-	const guchar *q = p;
+	const unsigned char *q = p;
+	guint64 v;
+
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+	memcpy(&v, q, sizeof v);
+#else
+	v = ((guint64) peek_be32(q) << 32) | peek_be32(&q[sizeof v / 2]);
+#endif
+	return v;
+}
+
+static inline guint16
+peek_le16(const void *p)
+{
+	const unsigned char *q = p;
 	guint16 v;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -83,9 +124,9 @@ peek_le16(gconstpointer p)
 }
 
 static inline guint32
-peek_le32(gconstpointer p)
+peek_le32(const void *p)
 {
-	const guchar *q = p;
+	const unsigned char *q = p;
 	guint32 v;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -102,9 +143,9 @@ peek_le32(gconstpointer p)
  */
 
 static inline ALWAYS_INLINE gpointer
-poke_u8(gpointer p, guchar v)
+poke_u8(gpointer p, unsigned char v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 	*q = v & 0xff;
 	return &q[sizeof v];
 }
@@ -112,7 +153,7 @@ poke_u8(gpointer p, guchar v)
 static inline gpointer
 poke_be16(gpointer p, guint16 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -127,7 +168,7 @@ poke_be16(gpointer p, guint16 v)
 static inline gpointer
 poke_be32(gpointer p, guint32 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -142,7 +183,7 @@ poke_be32(gpointer p, guint32 v)
 static inline gpointer
 poke_be64(gpointer p, guint64 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -157,7 +198,7 @@ poke_be64(gpointer p, guint64 v)
 static inline gpointer
 poke_le16(gpointer p, guint16 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -172,7 +213,7 @@ poke_le16(gpointer p, guint16 v)
 static inline gpointer
 poke_le32(gpointer p, guint32 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -187,7 +228,7 @@ poke_le32(gpointer p, guint32 v)
 static inline gpointer
 poke_le64(gpointer p, guint64 v)
 {
-	guchar *q = p;
+	unsigned char *q = p;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
@@ -199,5 +240,61 @@ poke_le64(gpointer p, guint64 v)
 	return &q[sizeof v];
 }
 
+
+/*
+ * The list of architectures is taken from xdr_float.c of
+ * SUN's XDR implementation.
+ */
+#if \
+	defined(__alpha__) || \
+	defined(__arm__) || \
+	defined(__hppa__) || \
+	defined(__i386__) || \
+	defined(__ia64__) || \
+	defined(__m68k__) || \
+	defined(__mips__) || \
+	defined(__ns32k__) || \
+	defined(__powerpc__) || \
+	defined(__ppc__) || \
+	defined(__sh__) || \
+	defined(__sparc__) || \
+	defined(__x86_64__)
+#define FLOAT_USES_IEEE754
+#else
+#undef FLOAT_USES_IEEE754
+#error "This architecture may be unsupported. float must use IEEE 754."
+#endif
+
+#ifdef FLOAT_USES_IEEE754
+/*
+ * The following routines assume integer byte order is float byte order.
+ * Handles IEC 60559/IEEE 754 floating point single-precision (32-bit).
+ */
+
+static inline gpointer
+poke_float_be32(gpointer p, float v)
+{
+	guint32 tmp;
+
+	STATIC_ASSERT(sizeof(float) == 4);
+
+	memcpy(&tmp, &v, 4);
+	return poke_be32(p, tmp);
+}
+
+static inline float
+peek_float_be32(const void *p)
+{
+	guint32 tmp;
+	float v;
+
+	STATIC_ASSERT(sizeof(float) == 4);
+
+	tmp = peek_be32(p);
+	memcpy(&v, &tmp, 4);
+	return v;
+}
+#endif	/* FLOAT_USES_IEEE754 */
+
 #endif /* _endian_h_ */
 /* vi: set ts=4 sw=4 cindent: */
