@@ -1,5 +1,5 @@
 /*
- * $Id: eval.c,v 1.10 2005/10/01 17:13:43 cbiere Exp $
+ * $Id: eval.c 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,43 +35,46 @@
 
 #include "common.h"
 
-RCSID("$Id: eval.c,v 1.10 2005/10/01 17:13:43 cbiere Exp $");
+RCSID("$Id: eval.c 16103 2009-03-10 14:53:39Z rmanfredi $")
 
 #ifdef I_PWD
 #include <pwd.h>
 #endif
 
 #include "eval.h"
+#include "ascii.h"
 #include "atoms.h"
+#include "debug.h"
+#include "glib-missing.h"
 #include "misc.h"			/* For g_strlcpy() */
+
 #include "override.h"		/* Must be the last header included */
 
 #define MAX_STRING	1024	/**< Max length for substitution */
 
-static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
-
 static GHashTable *constants;
-static gchar *home;
+static const char *home;	/* string atom */
 
-static gchar *get_home(void);
-static gchar *get_variable(gchar *s, gchar **end);
+static const char *get_home(void);
+static const char *get_variable(const char *s, const char **end);
+static gboolean initialized;
 
 /**
  * Create a constant string, or reuse an existing one if possible.
  *
  * @returns a string atom.
  */
-static gchar *
-constant_make(gchar *s)
+static const char *
+constant_make(const char *s)
 {
-	gchar *v;
+	const char *v;
 
-	v = (gchar *) g_hash_table_lookup(constants, s);
+	v = g_hash_table_lookup(constants, s);
 	if (v != NULL)
 		return v;			/* Already exists */
 
 	v = atom_str_get(s);
-	g_hash_table_insert(constants, v, v);
+	gm_hash_table_insert_const(constants, v, v);
 
 	return v;
 }
@@ -82,8 +85,13 @@ constant_make(gchar *s)
 void
 eval_init(void)
 {
+	g_return_if_fail(!initialized);
+
 	constants = g_hash_table_new(g_str_hash, g_str_equal);
 	home = get_home();
+	g_assert(home);
+
+	initialized = TRUE;
 }
 
 static void
@@ -101,8 +109,10 @@ constants_free_kv(gpointer key,
 void
 eval_close(void)
 {
-	atom_str_free(home);
-
+	if (home) {
+		atom_str_free(home);
+		home = NULL;
+	}
 	g_hash_table_foreach(constants, constants_free_kv, NULL);
 	g_hash_table_destroy(constants);
 }
@@ -116,24 +126,24 @@ eval_close(void)
  *
  * @return the pointer right after the inserted value.
  */
-static gchar *
-insert_value(gchar *val, gchar *start, gint off,
-	gint len, gint maxlen)
+static char *
+insert_value(const char *val, char *start, size_t off,
+	size_t len, size_t maxlen)
 {
-	gint vlen = strlen(val);
+	size_t vlen = strlen(val);
 
 	g_assert(len <= maxlen);
-	g_assert(off >= 0 && off <= len);
+	g_assert(off <= len);
 
-	if (len + vlen > maxlen) {
+	if (vlen > maxlen - len) {
 		g_warning("ignoring variable substitution text \"%s\"", val);
 		return start;
 	}
 
-	memmove(start + vlen, start, len + 1 - off);
+	memmove(&start[vlen], start, len + 1 - off);
 	memmove(start, val, vlen);
 
-	return start + vlen;
+	return &start[vlen];
 }
 
 /**
@@ -149,63 +159,73 @@ insert_value(gchar *val, gchar *start, gint off,
  *
  * @return string atom, which is not meant to be freed until exit time.
  */
-gchar *
-eval_subst(const gchar *str)
+const char *
+eval_subst(const char *str)
 {
-	gchar buf[MAX_STRING];
-	gchar *p;
-	gchar *end = buf + sizeof(buf);
+	char buf[MAX_STRING];
+	char *end = &buf[sizeof(buf)];
+	char *p;
 	size_t len;
-	gchar c;
+	char c;
+
+	g_assert(initialized);
 
 	if (str == NULL)
 		return NULL;
 
-	len = g_strlcpy(buf, str, sizeof(buf));
-	if (len >= sizeof(buf)) {
-		g_warning("eval_subst: string too large for substitution (%d bytes)",
-			(int) len);
-		return constant_make((gchar *) str);
+	len = g_strlcpy(buf, str, sizeof buf);
+	if (len >= sizeof buf) {
+		g_warning("eval_subst: string too large for substitution (%lu bytes)",
+			(unsigned long) len);
+		return constant_make(str);
 	}
 
 
 	if (common_dbg > 3)
 		printf("eval_subst: on entry: \"%s\"\n", buf);
 
-	for (p =  buf, c = *p++; c; c = *p++) {
-		gchar *val = NULL;
-		gchar *start = p - 1;
+	for (p = buf, c = *p++; c; c = *p++) {
+		const char *val = NULL;
+		char *start = p - 1;
 
-		if (c == '~' && start == buf) {		/* Leading ~ only */
-			val = home;
-			memmove(start, start + 1, len - (start - buf));
-			len--;
+		switch (c) {
+		case '~':
+			if (start == buf && ('\0' == buf[1] || '/' == buf[1])) {
+				/* Leading ~ only */
+				val = home;
+				g_assert(val);
+				memmove(start, &start[1], len - (start - buf));
+				len--;
 
-			g_assert((ssize_t) len >= 0);
+				g_assert(size_is_non_negative(len));
+			}
+			break;
+		case '$':
+			{
+				const char *after;
 
-		} else if (c == '$') {
-			gchar *after;
+				val = get_variable(p, &after);
+				g_assert(val);
+				memmove(start, after, len + 1 - (after - buf));
+				len -= after - start;		/* Also removing leading '$' */
 
-			val = get_variable(p, &after);
-			memmove(start, after, len + 1 - (after - buf));
-			len -= after - start;		/* Also removing leading '$' */
-
-			g_assert((ssize_t) len >= 0);
+				g_assert(size_is_non_negative(len));
+			}
+			break;
 		}
 
-
 		if (val != NULL) {
-			gchar *next = insert_value(
-				val, start, start - buf, len, sizeof(buf) - 1);
-
+			char *next;
+			
+			next = insert_value(val, start, start - buf, len, sizeof buf - 1);
 			len += next - start;
 			p = next;
 
-			g_assert(len < sizeof(buf));
+			g_assert(len < sizeof buf);
 			g_assert(p < end);
 		}
 
-		g_assert(p <= (buf + len));
+		g_assert(p <= &buf[len]);
 	}
 
 	if (common_dbg > 3)
@@ -222,16 +242,36 @@ eval_subst(const gchar *str)
  *
  * @return string atom.
  */
-static gchar *
+static const char *
 get_home(void)
 {
 	const char *dir;
 
 	dir = getenv("HOME");
+
+	if (dir && !is_absolute_path(dir)) {
+		/* Ignore $HOME if it's empty or a relative path */
+		dir = NULL;
+	}
 	
+#if defined(HAS_GETLOGIN)
+	if (!dir) {
+		const char *name;
+		
+		name = getlogin();
+		if (name) {
+			static const struct passwd *pp;
+
+			pp = getpwnam(name);
+			if (pp)
+				dir = pp->pw_dir;
+		}
+	}
+#endif
+
 #if defined(HAS_GETUID)
 	if (!dir) {
-		static struct passwd *pp;
+		static const struct passwd *pp;
 		
 		pp = getpwuid(getuid());
 		if (pp)
@@ -256,13 +296,10 @@ get_home(void)
  * @return variable's value, or "" if not found and set `end' to the address
  * of the character right after the variable name.
  */
-static gchar *
-get_variable(gchar *s, gchar **end)
+static const char *
+get_variable(const char *s, const char **end)
 {
-	guchar *p = (guchar *) s;
-	guchar c;
-	gchar *name = s;
-	gchar *value;
+	const char *value, *p = s;
 	gboolean end_brace = FALSE;
 
 	/*
@@ -271,36 +308,37 @@ get_variable(gchar *s, gchar **end)
 
 	if (*p == '{') {
 		p++;
-		name++;
+		s++;
 		end_brace = TRUE;
 	}
 
-	while ((c = *p)) {
-		if (!isalnum(c) && c != '_')
-			break;
+	while (is_ascii_alnum(*p) || *p == '_') {
 		p++;
 	}
 
 	if (end_brace && *p == '}')
-		*end = (gchar *) (p + 1);
+		*end = &p[1];
 	else
-		*end = (gchar *) p;
+		*end = p;
 
 	/*
 	 * Get value from environment.
 	 */
 
-	c = *p;
-	*p = '\0';
-	value = getenv(name);
+	{
+		char *name;
 
-	if (value == NULL)
-		value = "";
+		name = g_strndup(s, p - s);
+		value = getenv(name);
 
-	if (common_dbg > 4)
-		printf("variable \"%s\" is \"%s\"\n", name, value);
+		if (value == NULL)
+			value = "";
 
-	*p = c;
+		if (common_dbg > 4)
+			printf("variable \"%s\" is \"%s\"\n", name, value);
+
+		G_FREE_NULL(name);
+	}
 
 	return value;
 }
