@@ -1,5 +1,5 @@
 /*
- * $Id: eval.h 12500 2006-11-26 10:53:21Z cbiere $
+ * $Id: eval.h 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -45,7 +45,7 @@
 void eval_init(void);
 void eval_close(void);
 
-const gchar *eval_subst(const gchar *str);
+const char *eval_subst(const char *str);
 
 #endif	/* _eval_h_ */
 
