@@ -1,5 +1,5 @@
 /*
- * $Id: file.c 14863 2007-09-09 20:02:59Z cbiere $
+ * $Id: file.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,19 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: file.c 14863 2007-09-09 20:02:59Z cbiere $")
+RCSID("$Id: file.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #include "file.h"
+#include "debug.h"
 #include "misc.h"
 #include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
-static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
-
-static const gchar orig_ext[] = "orig";
-static const gchar new_ext[] = "new";
-static const gchar instead_str[] = " instead";
-static const gchar empty_str[] = "";
+static const char orig_ext[] = "orig";
+static const char new_ext[] = "new";
+static const char instead_str[] = " instead";
+static const char empty_str[] = "";
 
 /**
  * In order to avoid having a dependency between file.c and ban.c,
@@ -88,14 +87,14 @@ file_register_fd_reclaimer(reclaim_fd_t callback)
  */
 static FILE *
 open_read(
-	const gchar *what, const file_path_t *fv, gint fvcnt, gboolean renaming,
-	gint *chosen)
+	const char *what, const file_path_t *fv, int fvcnt, gboolean renaming,
+	int *chosen)
 {
 	FILE *in;
-	gchar *path;
-	gchar *path_orig;
-	const gchar *instead = empty_str;
-	gint idx = 0;
+	char *path;
+	char *path_orig;
+	const char *instead = empty_str;
+	int idx = 0;
 
 	g_assert(fv != NULL);
 	g_assert(fvcnt >= 1);
@@ -148,7 +147,7 @@ open_read(
 
 	if (in == NULL && fvcnt > 1) {
 		const file_path_t *xfv;
-		gint xfvcnt;
+		int xfvcnt;
 
 		instead = instead_str;
 
@@ -194,7 +193,7 @@ out:
  * @returns opened FILE, or NULL if we were unable to open any.
  */
 FILE *
-file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
+file_config_open_read(const char *what, const file_path_t *fv, int fvcnt)
 {
 	return open_read(what, fv, fvcnt, TRUE, NULL);
 }
@@ -211,7 +210,7 @@ file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
  */
 FILE *
 file_config_open_read_norename(
-	const gchar *what, const file_path_t *fv, gint fvcnt)
+	const char *what, const file_path_t *fv, int fvcnt)
 {
 	return open_read(what, fv, fvcnt, FALSE, NULL);
 }
@@ -222,7 +221,7 @@ file_config_open_read_norename(
  */
 FILE *
 file_config_open_read_norename_chosen(
-	const gchar *what, const file_path_t *fv, gint fvcnt, gint *chosen)
+	const char *what, const file_path_t *fv, int fvcnt, int *chosen)
 {
 	return open_read(what, fv, fvcnt, FALSE, chosen);
 }
@@ -235,7 +234,7 @@ file_config_open_read_norename_chosen(
  * @returns opened FILE if success, NULL on error.
  */
 static FILE *
-file_config_open(const gchar *what, const file_path_t *fv)
+file_config_open(const char *what, const file_path_t *fv)
 {
 	FILE *out = NULL;
 	char *path;
@@ -257,7 +256,7 @@ file_config_open(const gchar *what, const file_path_t *fv)
  * Open configuration file for writing.
  */
 FILE *
-file_config_open_write(const gchar *what, const file_path_t *fv)
+file_config_open_write(const char *what, const file_path_t *fv)
 {
     return file_config_open(what, fv);
 }
@@ -304,14 +303,14 @@ failed:
  * Emit the configuration preamble.
  */
 void
-file_config_preamble(FILE *out, const gchar *what)
+file_config_preamble(FILE *out, const char *what)
 {
 	time_t now = tm_time();
 
 	g_assert(out);
 
 	fputs("# THIS FILE IS AUTOMATICALLY GENERATED -- DO NOT EDIT\n", out);
-	fprintf(out, "#\n# %s saved on %s#\n\n", what, ctime(&now));
+	fprintf(out, "#\n# %s saved on %s\n#\n\n", what, timestamp_to_string(now));
 }
 
 /**
@@ -334,11 +333,11 @@ file_path_set(file_path_t *fp, const char *dir, const char *name)
  * Errors are logged as a warning, unless `missing' is true, in which
  * case no error is logged for ENOENT.
  */
-static gint
-do_open(const gchar *path, gint flags, gint mode, gboolean missing)
+static int
+do_open(const char *path, int flags, int mode, gboolean missing)
 {
-	const gchar *what;
-	gint fd;
+	const char *what;
+	int fd;
 
 	if (!is_absolute_path(path)) {
 		errno = EPERM;
@@ -408,10 +407,10 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
  * Open file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
  */
-gint
-file_open(const gchar *path, gint flags)
+int
+file_open(const char *path, int flags, int mode)
 {
-	return do_open(path, flags, 0, FALSE);
+	return do_open(path, flags, mode, FALSE);
 }
 
 /**
@@ -419,8 +418,8 @@ file_open(const gchar *path, gint flags)
  * Errors are logged as a warning, unless the file is missing, in which
  * case nothing is logged.
  */
-gint
-file_open_missing(const gchar *path, gint flags)
+int
+file_open_missing(const char *path, int flags)
 {
 	return do_open(path, flags, 0, TRUE);
 }
@@ -429,8 +428,8 @@ file_open_missing(const gchar *path, gint flags)
  * Create file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
  */
-gint
-file_create(const gchar *path, gint flags, gint mode)
+int
+file_create(const char *path, int flags, int mode)
 {
 	return do_open(path, flags | O_CREAT, mode, FALSE);
 }
@@ -440,8 +439,8 @@ file_create(const gchar *path, gint flags, gint mode)
  * Errors are logged as a warning, unless the error is ENOENT which means
  * the directory does not exist.
  */
-gint
-file_create_missing(const gchar *path, gint flags, gint mode)
+int
+file_create_missing(const char *path, int flags, int mode)
 {
 	return do_open(path, flags | O_CREAT, mode, TRUE);
 }
@@ -452,11 +451,11 @@ file_create_missing(const gchar *path, gint flags, gint mode)
  * is TRUE.
  */
 static FILE *
-do_fopen(const gchar *path, const gchar *mode, gboolean missing)
+do_fopen(const char *path, const char *mode, gboolean missing)
 {
-	gchar m;
+	char m;
 	FILE *f;
-	const gchar *what;
+	const char *what;
 
 	if (!is_absolute_path(path)) {
 		errno = EPERM;
@@ -504,7 +503,7 @@ do_fopen(const gchar *path, const gchar *mode, gboolean missing)
  * Errors are logged as a warning.
  */
 FILE *
-file_fopen(const gchar *path, const gchar *mode)
+file_fopen(const char *path, const char *mode)
 {
 	return do_fopen(path, mode, FALSE);
 }
@@ -515,15 +514,15 @@ file_fopen(const gchar *path, const gchar *mode)
  * case nothing is logged.
  */
 FILE *
-file_fopen_missing(const gchar *path, const gchar *mode)
+file_fopen_missing(const char *path, const char *mode)
 {
 	return do_fopen(path, mode, TRUE);
 }
 
 void
-file_set_nonblocking(gint fd)
+file_set_nonblocking(int fd)
 {
-	gint ret, flags;
+	int ret, flags;
 
 	ret = fcntl(fd, F_GETFL, 0);
 	flags = ret | VAL_O_NONBLOCK;
@@ -531,4 +530,21 @@ file_set_nonblocking(gint fd)
 		fcntl(fd, F_SETFL, flags);
 }
 
+/**
+ * Closes the file and sets the descriptor to -1. Does nothing if
+ * the descriptor is already -1.
+ * @param fd_ptr Must point to a non-negative file descriptor or -1.
+ */
+void
+file_close(int *fd_ptr)
+{
+	g_assert(fd_ptr);
+	g_assert(*fd_ptr >= -1);
+
+	if (-1 != *fd_ptr) {
+		close(*fd_ptr);
+		*fd_ptr = -1;
+	}
+}
+
 /* vi: set ts=4: */
