@@ -1,5 +1,5 @@
 /*
- * $Id: getline.c 14336 2007-08-06 15:12:47Z cbiere $
+ * $Id: getline.c 16273 2009-03-28 17:21:01Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: getline.c 14336 2007-08-06 15:12:47Z cbiere $")
+RCSID("$Id: getline.c 16273 2009-03-28 17:21:01Z rmanfredi $")
 
 #include "getline.h"
 #include "walloc.h"
@@ -119,7 +119,7 @@ getline_reset(getline_t *o)
  * The trailing "\r\n" or "\n" is stripped from the accumulated line.
  */
 getline_result_t
-getline_read(getline_t *o, const gchar *data, size_t len, size_t *used)
+getline_read(getline_t *o, const char *data, size_t len, size_t *used)
 {
 	getline_result_t result = READ_MORE;
 	size_t used_bytes, needed, missing;
@@ -148,7 +148,7 @@ getline_read(getline_t *o, const gchar *data, size_t len, size_t *used)
 	 */
 
 	for (used_bytes = 0; used_bytes < len; /* NOTHING */) {
-		gchar c;
+		char c;
 
 		if (o->pos >= (o->size - 1))			/* Leave room for final NUL */
 			return READ_OVERFLOW;
@@ -182,8 +182,8 @@ getline_read(getline_t *o, const gchar *data, size_t len, size_t *used)
  * @return a C string (NUL-terminated) corresponding to the line we currently
  * have in the buffer.
  */
-const gchar *
-getline_str(getline_t *o)
+const char *
+getline_str(const getline_t *o)
 {
 	g_assert(o->pos < o->size);
 
@@ -195,7 +195,7 @@ getline_str(getline_t *o)
  * @return the length of the currently accumulated line.
  */
 size_t
-getline_length(getline_t *o)
+getline_length(const getline_t *o)
 {
 	return o->pos;
 }
@@ -204,7 +204,7 @@ getline_length(getline_t *o)
  * Copy source into dest.
  */
 void
-getline_copy(getline_t *source, getline_t *dest)
+getline_copy(const getline_t *source, getline_t *dest)
 {
 	g_assert(source);
 	g_assert(dest);
