@@ -1,5 +1,5 @@
 /*
- * $Id: getline.h 14336 2007-08-06 15:12:47Z cbiere $
+ * $Id: getline.h 16273 2009-03-28 17:21:01Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -57,7 +57,7 @@ typedef enum getline_result {
 typedef struct getline {
 	size_t maxlen;				/**< Maximum authorized length */
 	size_t size;				/**< Current allocated size for `line' */
-	gchar *line;				/**< Accumulator, NUL terminated when done */
+	char *line;				/**< Accumulator, NUL terminated when done */
 	size_t pos;					/**< Next writing position in line[] */
 } getline_t;
 
@@ -71,10 +71,10 @@ getline_t *getline_make(size_t maxsize);
 void getline_free(getline_t *);
 void getline_reset(getline_t *);
 getline_result_t getline_read(getline_t *,
-					const gchar *data, size_t len, size_t *used);
-const gchar *getline_str(getline_t *);
-size_t getline_length(getline_t *);
-void getline_copy(getline_t *source, getline_t *dest);
+					const char *data, size_t len, size_t *used);
+const char *getline_str(const getline_t *);
+size_t getline_length(const getline_t *);
+void getline_copy(const getline_t *source, getline_t *dest);
 void getline_set_maxlen(getline_t *, size_t);
 
 #endif	/* _getline_h_ */
