@@ -1,5 +1,5 @@
 /*
- * $Id: glib-missing.c 13984 2007-06-29 13:31:06Z cbiere $
+ * $Id: glib-missing.c 16247 2009-03-26 23:03:44Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -41,9 +41,10 @@
 
 #include "common.h"
 
-RCSID("$Id: glib-missing.c 13984 2007-06-29 13:31:06Z cbiere $")
+RCSID("$Id: glib-missing.c 16247 2009-03-26 23:03:44Z rmanfredi $")
 
 #include "glib-missing.h"
+#include "ascii.h"
 #include "iovec.h"
 #include "misc.h"
 #include "utf8.h"
@@ -298,6 +299,7 @@ gm_list_insert_after(GList *list, GList *lnk, gpointer data)
 #ifdef USE_GLIB1
 #undef g_list_delete_link		/* Remaped under -DTRACK_MALLOC */
 #undef g_slist_delete_link
+#undef g_list_insert_before
 GList *
 g_list_delete_link(GList *l, GList *lnk)
 {
@@ -318,6 +320,27 @@ g_slist_delete_link(GSList *sl, GSList *lnk)
 	return head;
 }
 
+GList *
+g_list_insert_before(GList *l, GList *lk, gpointer data)
+{
+	GList *new;
+
+	if (lk == NULL)
+		return g_list_append(l, data);
+
+	new = g_list_alloc();
+	new->data = data;
+
+	new->next = lk;
+	new->prev = lk->prev;
+
+	if (lk->prev)
+		lk->prev->next = new;
+
+	lk->prev = new;
+
+	return lk == l ? new : l;
+}
 #endif /* USE_GLIB1 */
 #endif	/* !TRACK_MALLOC */
 
@@ -343,10 +366,10 @@ g_hash_table_replace(GHashTable *ht, gpointer key, gpointer value)
  * @return The length of the resulting string.
  */
 static inline size_t
-buf_vprintf(gchar *dst, size_t size, const gchar *fmt, va_list args)
+buf_vprintf(char *dst, size_t size, const char *fmt, va_list args)
 #ifdef	HAS_VSNPRINTF
 {
-	gint retval;	/* printf()-functions really return int, not size_t */
+	int retval;	/* printf()-functions really return int, not size_t */
 	
 	g_assert(size > 0);	
 
@@ -365,7 +388,7 @@ buf_vprintf(gchar *dst, size_t size, const gchar *fmt, va_list args)
 }
 #else	/* !HAS_VSNPRINTF */
 {
-	gchar *buf;
+	char *buf;
 	size_t len;
   
 	g_assert(size > 0);	
@@ -394,13 +417,13 @@ buf_vprintf(gchar *dst, size_t size, const gchar *fmt, va_list args)
  * @return The length of the resulting string.
  */
 size_t
-gm_vsnprintf(gchar *dst, size_t size, const gchar *fmt, va_list args)
+gm_vsnprintf(char *dst, size_t size, const char *fmt, va_list args)
 {
 	size_t len;
 
 	g_return_val_if_fail(dst != NULL, 0);
 	g_return_val_if_fail(fmt != NULL, 0);
-	g_return_val_if_fail((ssize_t) size > 0, 0);
+	g_return_val_if_fail(size_is_positive(size), 0);
 	g_return_val_if_fail(size <= (size_t) INT_MAX, 0);
 
 	len = buf_vprintf(dst, size, fmt, args);
@@ -427,14 +450,14 @@ gm_vsnprintf(gchar *dst, size_t size, const gchar *fmt, va_list args)
  * @return The length of the resulting string.
  */
 size_t
-gm_snprintf(gchar *dst, size_t size, const gchar *fmt, ...)
+gm_snprintf(char *dst, size_t size, const char *fmt, ...)
 {
 	va_list args;
 	size_t len;
 
 	g_return_val_if_fail(dst != NULL, 0);
 	g_return_val_if_fail(fmt != NULL, 0);
-	g_return_val_if_fail((ssize_t) size > 0, 0);
+	g_return_val_if_fail(size_is_positive(size), 0);
 	g_return_val_if_fail(size <= (size_t) INT_MAX, 0);
 
 	va_start(args, fmt);
@@ -446,15 +469,15 @@ gm_snprintf(gchar *dst, size_t size, const gchar *fmt, ...)
 	return len;
 }
 
-static gint orig_argc;
-static gchar **orig_argv;
-static gchar **orig_env;
+static int orig_argc;
+static char **orig_argv;
+static char **orig_env;
 
 /**
  * Save the original main() arguments.
  */
 void
-gm_savemain(gint argc, gchar **argv, gchar **env)
+gm_savemain(int argc, char **argv, char **env)
 {
 	orig_argc = argc;
 	orig_argv = argv;
@@ -462,7 +485,7 @@ gm_savemain(gint argc, gchar **argv, gchar **env)
 }
 
 static inline size_t
-str_vec_count(gchar *strv[])
+str_vec_count(char *strv[])
 {
 	size_t i = 0;
 
@@ -481,7 +504,7 @@ str_vec_count(gchar *strv[])
  * @param env_ptr The original ``env'' variable.
  */
 static struct iovec
-gm_setproctitle_init(gint argc, gchar *argv[], gchar *env_ptr[])
+gm_setproctitle_init(int argc, char *argv[], char *env_ptr[])
 {
 	size_t env_count, n;
 	struct iovec *iov;
@@ -513,7 +536,7 @@ gm_setproctitle_init(gint argc, gchar *argv[], gchar *env_ptr[])
 	 * Scrap references to the arguments.
 	 */
 	{
-		gint i;
+		int i;
 
 		for (i = 1; i < argc; i++)
 			argv[i] = NULL;
@@ -528,7 +551,7 @@ gm_setproctitle_init(gint argc, gchar *argv[], gchar *env_ptr[])
  * Change the process title as seen by "ps".
  */
 void
-gm_setproctitle(const gchar *title)
+gm_setproctitle(const char *title)
 #if defined(HAS_SETPROCTITLE)
 {
 	setproctitle("%s", title);
@@ -554,7 +577,7 @@ gm_setproctitle(const gchar *title)
 /**
  * Return the process title as seen by "ps"
  */
-const gchar *
+const char *
 gm_getproctitle(void)
 {
 	return orig_argv[0];
@@ -568,9 +591,9 @@ gm_getproctitle(void)
  * contain embedded nuls and need not be nul-terminated.
  */
 GString *
-g_string_append_len(GString *gs, const gchar *val, gssize len)
+g_string_append_len(GString *gs, const char *val, gssize len)
 {
-	const gchar *p = val;
+	const char *p = val;
 
 	while (len--)
 		g_string_append_c(gs, *p++);
@@ -591,12 +614,12 @@ g_string_append_len(GString *gs, const gchar *val, gssize len)
  * @returns a newly allocated string or ``filename'' if it was a valid filename
  *		    already.
  */
-gchar *
-gm_sanitize_filename(const gchar *filename,
+char *
+gm_sanitize_filename(const char *filename,
 		gboolean no_spaces, gboolean no_evil)
 {
-	const gchar *s;
-	gchar *q;
+	const char *s;
+	char *q;
 
 	g_assert(filename);
 
@@ -612,17 +635,17 @@ gm_sanitize_filename(const gchar *filename,
 
 	/* Replace shell meta characters and likely problematic characters */
 	{
-		static const gchar evil[] = "$&*\\`:;()'\"<>?|~\177";
+		static const char evil[] = "$&*\\`:;()'\"<>?|~\177";
 		size_t i;
 		guchar c;
 		
-		for (i = 0; '\0' != (c = s[i]); ++i) {
+		for (i = 0; '\0' != (c = s[i]); i++) {
 			if (
 				c < 32
 				|| is_ascii_cntrl(c)
 				|| G_DIR_SEPARATOR == c
 				|| '/' == c 
-				|| (0 == i && '.' == c)
+				|| (0 == i && ('.' == c || '-' == c))
 				|| (no_spaces && is_ascii_space(c))
 				|| (no_evil && NULL != strchr(evil, c))
 		   ) {
@@ -636,6 +659,90 @@ gm_sanitize_filename(const gchar *filename,
 	return q ? q : deconstify_gchar(s);
 }
 
+/**
+ * Make filename prettier, by removing leading "_", making sure the filename
+ * does not start with "-" or ".", and stripping consecutive "_" or "_" that
+ * surround a punctuation character.
+ *
+ * Finally, ensure the filename is not completely empty, as this is
+ * awkward to manipulate from a shell.
+ *
+ * @param filename	the filename to beautify
+ *
+ * @returns a newly allocated string holding the beautified filename, even if
+ * it is a mere copy of the original.
+ */
+char *
+gm_beautify_filename(const char *filename)
+{
+	const char *s;
+	char *q;
+	guchar c;
+	size_t len;
+	size_t j = 0;
+	static const char punct[] = "_-+=.,<>{}[]";	/* 1st MUST be '_' */
+	static const char strip[] = "_-.";
+	static const char empty[] = "{empty}";
+
+	g_assert(filename);
+
+	s = filename;
+	len = strlen(filename);
+	q = g_malloc(len + 1);		/* Trailing NUL */
+
+	while ((c = *s++)) {
+		guchar d;
+
+		/* Beautified filename cannot start with stripped characters */
+		if (j == 0) {
+			if (NULL == strchr(strip, c))
+				q[j++] = c;
+			continue;
+		}
+
+		g_assert(j > 0);
+
+		d = q[j - 1];		/* Last char we've kept in beautified name */
+
+		/* A "_" followed by a punctuation character, strip the "_" */
+		if (d == '_' && NULL != strchr(punct, c)) {
+			q[j - 1] = c;
+			continue;
+		}
+
+		/* A punctuation character followed by "_", ignore that "_" */
+		if (NULL != strchr(&punct[1], d) && c == '_')
+			continue;
+
+		q[j++] = c;
+	}
+
+	g_assert(j <= len);
+	q[j] = '\0';
+
+	/* Ensure we have no empty name */
+	if (j == 0) {
+		G_FREE_NULL(q);
+		return g_strdup(empty);
+	}
+
+	/*
+	 * If there was an extension following stripped chars (e.g. "_.ext"),
+	 * then the filename kept will become "ext" (we assume a valid extension
+	 * cannot contain "escaped" chars).  In which case we will prepend the
+	 * string "{empty}." to it.
+	 */
+
+	if (NULL == strchr(q, '.') && j < len && '.' == filename[len - j]) {
+		char *r = g_strconcat(empty, ".", q, (void *) 0);
+		G_FREE_NULL(q);
+
+		return r;
+	}
+
+	return q;
+}
+
 /**
  * Frees the GString context but keeps the string data itself and returns
  * it. With Gtk+ 2.x g_string_free(gs, FALSE) would do the job but the
@@ -643,10 +750,10 @@ gm_sanitize_filename(const gchar *filename,
  *
  * @return The string data.
  */
-gchar *
+char *
 gm_string_finalize(GString *gs)
 {
-	gchar *s;
+	char *s;
 
 	g_return_val_if_fail(gs, NULL);
 	g_return_val_if_fail(gs->str, NULL);
@@ -728,4 +835,81 @@ gm_hash_table_foreach_key(GHashTable *ht, GFunc func, gpointer user_data)
 	g_hash_table_foreach(ht, gm_hash_table_foreach_keys_helper, &hp);
 }
 
+#ifdef USE_GLIB1
+/*
+ * glib1 is missing g_list_sort_with_data().
+ *
+ * The following (adapted) code is:
+ *
+ * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
+ *
+ * and was taken out of the glist.c file from glib 2.16.
+ */
+
+/**
+ * Internal merging sort for g_list_sort_with_data().
+ */
+static GList *
+g_list_sort_merge(
+	GList *l1, GList *l2, GCompareDataFunc compare_func, gpointer user_data)
+{
+	GList list, *l, *lprev;
+
+	l = &list; 
+	lprev = NULL;
+
+	while (l1 && l2) {
+		int cmp = (*compare_func)(l1->data, l2->data, user_data);
+
+		if (cmp <= 0) {
+			l->next = l1;
+			l1 = l1->next;
+		} else {
+			l->next = l2;
+			l2 = l2->next;
+		}
+		l = l->next;
+		l->prev = lprev; 
+		lprev = l;
+	}
+	l->next = l1 ? l1 : l2;
+	l->next->prev = l;
+
+	return list.next;
+}
+
+/**
+ * Like g_list_sort(), but the comparison function takes a user data argument.
+ *
+ * @return the new head of list
+ */
+GList *
+g_list_sort_with_data(
+	GList *list, GCompareDataFunc compare_func, gpointer user_data)
+{
+	GList *l1, *l2;
+  
+	if (!list) 
+		return NULL;
+	if (!list->next) 
+		return list;
+  
+	l1 = list; 
+	l2 = list->next;
+
+	while ((l2 = l2->next)) {
+		if (NULL == (l2 = l2->next)) 
+			break;
+		l1 = l1->next;
+	}
+	l2 = l1->next; 
+	l1->next = NULL; 
+
+	return g_list_sort_merge(
+		g_list_sort_with_data(list, compare_func, user_data),
+		g_list_sort_with_data(l2, compare_func, user_data),
+		compare_func, user_data);
+}
+#endif	/* USE_GLIB1 */
+
 /* vi: set ts=4 sw=4 cindent: */
