@@ -1,5 +1,5 @@
 /*
- * $Id: glib-missing.h 13955 2007-06-24 03:25:03Z cbiere $
+ * $Id: glib-missing.h 16247 2009-03-26 23:03:44Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -70,25 +70,35 @@ GList *gm_list_insert_after(GList *list, GList *lnk, gpointer data);
 #ifdef USE_GLIB1
 GList *g_list_delete_link(GList *l, GList *lnk);
 GSList *g_slist_delete_link(GSList *sl, GSList *lnk);
-GString *g_string_append_len(GString *gs, const gchar *val, gssize len);
+GList *g_list_insert_before(GList *l, GList *lk, gpointer data);
+GString *g_string_append_len(GString *gs, const char *val, gssize len);
 
 void g_hash_table_replace(GHashTable *ht, gpointer key, gpointer value);
 
 void g_mem_set_vtable(GMemVTable *vtable);
 gboolean g_mem_is_system_malloc(void);
+
+typedef int (*GCompareDataFunc)
+	(gconstpointer a, gconstpointer b, gpointer user_data);
+
+GList *g_list_sort_with_data(
+	GList *l, GCompareDataFunc cmp, gpointer user_data);
+
+#define g_string_printf		g_string_sprintf
 #endif
 
-gchar *gm_string_finalize(GString *gs);
+char *gm_string_finalize(GString *gs);
 
-size_t gm_vsnprintf(gchar *str, size_t n, gchar const *fmt, va_list args);
-size_t gm_snprintf(gchar *str, size_t n,
-	gchar const *fmt, ...) G_GNUC_PRINTF (3, 4);
+size_t gm_vsnprintf(char *str, size_t n, char const *fmt, va_list args);
+size_t gm_snprintf(char *str, size_t n,
+	char const *fmt, ...) G_GNUC_PRINTF (3, 4);
 
-void gm_savemain(gint argc, gchar **argv, gchar **env);
-const gchar *gm_getproctitle(void);
-void gm_setproctitle(const gchar *title);
-gchar *gm_sanitize_filename(const gchar *filename,
+void gm_savemain(int argc, char **argv, char **env);
+const char *gm_getproctitle(void);
+void gm_setproctitle(const char *title);
+char *gm_sanitize_filename(const char *filename,
 	gboolean no_spaces, gboolean no_evil);
+char *gm_beautify_filename(const char *filename);
 
 static inline void
 gm_hash_table_insert_const(GHashTable *ht,
