@@ -1,5 +1,5 @@
 /*
- * $Id: hashlist.c 14954 2007-09-28 19:23:13Z bpringlemeir $
+ * $Id: hashlist.c 16252 2009-03-27 14:45:04Z rmanfredi $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -42,7 +42,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hashlist.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
+RCSID("$Id: hashlist.c 16252 2009-03-27 14:45:04Z rmanfredi $")
 
 #include "hashlist.h"
 #include "misc.h"
@@ -61,10 +61,10 @@ typedef enum {
 
 struct hash_list {
 	hash_list_magic_t magic;
-	gint refcount;
+	int refcount;
 	GHashTable *ht;
 	GList *head, *tail;
-	gint len;
+	int len;
 	guint stamp;
 };
 
@@ -272,7 +272,6 @@ hash_list_prepend(hash_list_t *hl, gconstpointer key)
 void
 hash_list_insert_sorted(hash_list_t *hl, gconstpointer key, GCompareFunc func)
 {
-	struct hash_list_item *item;
 	GList *iter;
 
 	hash_list_check(hl);
@@ -281,34 +280,26 @@ hash_list_insert_sorted(hash_list_t *hl, gconstpointer key, GCompareFunc func)
 	g_assert(NULL == g_hash_table_lookup(hl->ht, key));
 
 	for (iter = hl->head; iter; iter = g_list_next(iter)) {
-		item = iter->data;
+		struct hash_list_item *item = iter->data;
 		if (func(key, item->orig_key) <= 0)
 			break;
 	}
-	
-	item = walloc(sizeof *item);
-	item->orig_key = key;
-	item->list = g_list_alloc();
-	item->list->data = item;
-	item->list->prev = g_list_previous(iter);
-	item->list->next = iter;
-	if (item->list->prev) {
-		item->list->prev->next = item->list;
+
+	if (NULL == iter) {
+		hash_list_append(hl, key);
+	} else {
+		struct hash_list_item *item;
+
+		item = walloc(sizeof *item);
+		item->orig_key = key;
+
+		/* Inserting ``item'' before ``iter'' */
+
+		hl->head = g_list_insert_before(hl->head, iter, item);
+		item->list = g_list_previous(iter);
+
+		hash_list_insert_item(hl, item);
 	}
-
-	if (hl->head == iter) {
-		hl->head = item->list;
-	}
-	if (!hl->tail) {
-		hl->tail = item->list;
-	}
-
-	g_hash_table_insert(hl->ht, deconstify_gpointer(key), item);
-
-	hl->len++;
-	hl->stamp++;
-
-	hash_list_regression(hl);
 }
 
 static gpointer
@@ -335,7 +326,7 @@ hash_list_remove_item(hash_list_t *hl, struct hash_list_item *item)
 
 /**
  * Remove `data' from the list.
- * @return The data that associated with the given key.
+ * @return The data that was associated with the given key.
  */
 gpointer
 hash_list_remove(hash_list_t *hl, gconstpointer key)
@@ -349,6 +340,32 @@ hash_list_remove(hash_list_t *hl, gconstpointer key)
 	return item ? hash_list_remove_item(hl, item) : NULL;
 }
 
+/**
+ * Remove head item from the list.
+ * @return The data that was stored there.
+ */
+gpointer
+hash_list_remove_head(hash_list_t *hl)
+{
+	if (NULL == hl->head)
+		return NULL;
+
+	return hash_list_remove_item(hl, hl->head->data);
+}
+
+/**
+ * Remove tail item from the list.
+ * @return The data that was stored there.
+ */
+gpointer
+hash_list_remove_tail(hash_list_t *hl)
+{
+	if (NULL == hl->tail)
+		return NULL;
+
+	return hash_list_remove_item(hl, hl->tail->data);
+}
+
 gpointer
 hash_list_shift(hash_list_t *hl)
 {
@@ -491,6 +508,30 @@ hash_list_length(const hash_list_t *hl)
 	return hl->len;
 }
 
+/**
+ * Extract the list of items so that the caller can iterate at will over
+ * it as sort it.  The caller must dispose of that list via g_list_free().
+ * The underlying data is not copied so it must NOT be freed.
+ *
+ * @returns a shallow copy of the underlying list.
+ */
+GList *
+hash_list_list(hash_list_t *hl)
+{
+	GList *l = NULL;
+	GList *lc = NULL;
+
+	hash_list_check(hl);
+
+	for (l = hl->tail; l; l = g_list_previous(l)) {
+		struct hash_list_item *item = l->data;
+
+		lc = g_list_prepend(lc, deconstify_gpointer(item->orig_key));
+	}
+
+	return lc;
+}
+
 /**
  * Get an iterator on the list, positionned before first item.
  * Get items with hash_list_iter_next().
@@ -598,18 +639,16 @@ hash_list_iter_previous(hash_list_iter_t *iter)
 	}
 }
 
-#if 0 /* UNUSED */
 /**
  * Checks whether there is a previous item in the iterator.
  */
 gboolean
-hash_list_has_iter_previous(const hash_list_iter_t *iter)
+hash_list_iter_has_previous(const hash_list_iter_t *iter)
 {
 	hash_list_iter_check(iter);
 
 	return NULL != iter->prev;
 }
-#endif /* UNUSED */
 
 /**
  * Release the iterator once we're done with it.
@@ -632,6 +671,8 @@ hash_list_iter_release(hash_list_iter_t **iter_ptr)
 
 /**
  * Check whether hashlist contains the `data'.
+ * If `orig_key_ptr' is not NULL and the key exists, a pointer to
+ * the stored key is written to it.
  */
 gboolean
 hash_list_contains(hash_list_t *hl, gconstpointer key,
@@ -650,6 +691,21 @@ hash_list_contains(hash_list_t *hl, gconstpointer key,
 
 /**
  * Get the next item after a given key.
+ *
+ * This is more costly than taking an iterator and traversing the structure,
+ * but it is safe to use when the processing of each item can remove the item
+ * from the traversed structure.
+ *
+ * Here's template code demonstrating usage:
+ *
+ *		gpointer next = hash_list_head(hl);
+ *		while (next) {
+ *			struct <item> *item = next;
+ *			next = hash_list_next(hl, next);
+ *			<process item, can be safely removed from hl>
+ *		}
+ *
+ * @return pointer to next item, NULL if we reached the end of the list.
  */
 gpointer
 hash_list_next(hash_list_t *hl, gconstpointer key)
