@@ -1,5 +1,5 @@
 /*
- * $Id: hashlist.h 14345 2007-08-07 07:48:54Z rmanfredi $
+ * $Id: hashlist.h 15410 2008-07-11 05:11:57Z rmanfredi $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -35,6 +35,8 @@ typedef struct hash_list hash_list_t;
 hash_list_t *hash_list_new(GHashFunc hash_func, GEqualFunc eq_func);
 void hash_list_free(hash_list_t **hl_ptr);
 gpointer hash_list_remove(hash_list_t *hl, gconstpointer key);
+gpointer hash_list_remove_head(hash_list_t *hl);
+gpointer hash_list_remove_tail(hash_list_t *hl);
 gpointer hash_list_shift(hash_list_t *hl);
 void hash_list_append(hash_list_t *hl, gconstpointer key);
 void hash_list_prepend(hash_list_t *hl, gconstpointer key);
@@ -47,11 +49,13 @@ gpointer hash_list_tail(const hash_list_t *hl);
 gpointer hash_list_next(hash_list_t *hl, gconstpointer key);
 gpointer hash_list_previous(hash_list_t *hl, gconstpointer key);
 guint hash_list_length(const hash_list_t *hl);
+GList *hash_list_list(hash_list_t *hl);
 
 hash_list_iter_t *hash_list_iterator(hash_list_t *hl);
 hash_list_iter_t *hash_list_iterator_tail(hash_list_t *hl);
 void hash_list_iter_release(hash_list_iter_t **iter_ptr);
 gboolean hash_list_iter_has_next(const hash_list_iter_t *i);
+gboolean hash_list_iter_has_previous(const hash_list_iter_t *iter);
 gpointer hash_list_iter_next(hash_list_iter_t *i);
 gpointer hash_list_iter_previous(hash_list_iter_t *i);
 gpointer hash_list_iter_follower(hash_list_iter_t *i);
