@@ -1,5 +1,5 @@
 /*
- * $Id: header.h 12500 2006-11-26 10:53:21Z cbiere $
+ * $Id: header.h 16275 2009-03-28 17:40:31Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -42,10 +42,9 @@
  * Header parsing and holding data structures.
  */
 
-struct header;
 typedef struct header header_t;
 
-gint header_num_lines(const header_t *h);
+int header_num_lines(const header_t *h);
 
 /*
  * Error codes.
@@ -75,23 +74,25 @@ gint header_num_lines(const header_t *h);
 header_t *header_make(void);
 void header_free(header_t *o);
 void header_reset(header_t *o);
-gint header_append(header_t *o, const gchar *text, gint len);
-void header_dump(const header_t *o, FILE *out);
-const gchar *header_strerror(guint errnum);
-gchar *header_get(const header_t *o, const gchar *field);
-gchar *header_getdup(const header_t *o, const gchar *field);
+int header_append(header_t *o, const char *text, int len);
+void header_dump(FILE *out, const header_t *o, const char *trailer);
+const char *header_strerror(guint errnum);
+char *header_get(const header_t *o, const char *field);
+char *header_getdup(const header_t *o, const char *field);
 
-gpointer header_fmt_make(const gchar *field, const gchar *separator,
-	gint len_hint);
-void header_fmt_free(gpointer o);
-void header_fmt_set_line_length(gpointer o, gint maxlen);
-gboolean header_fmt_value_fits(gpointer o, gint len, gint maxlen);
-void header_fmt_append(gpointer o, const gchar *str, const gchar *separator);
-void header_fmt_append_value(gpointer o, const gchar *str);
-gint header_fmt_length(gpointer o);
-void header_fmt_end(gpointer o);
-gchar *header_fmt_string(gpointer o);
-gchar *header_fmt_to_string(gpointer o);
+typedef struct header_fmt header_fmt_t;
+
+header_fmt_t *header_fmt_make(const char *field, const char *separator,
+	int len_hint, size_t max_size);
+void header_fmt_free(header_fmt_t *hf);
+void header_fmt_set_line_length(header_fmt_t *hf, int maxlen);
+gboolean header_fmt_value_fits(const header_fmt_t *hf, size_t len);
+gboolean header_fmt_append(header_fmt_t *hf, const char *str, const char *sep);
+gboolean header_fmt_append_value(header_fmt_t *hf, const char *str);
+int header_fmt_length(const header_fmt_t *hf);
+void header_fmt_end(header_fmt_t *hf);
+char *header_fmt_string(const header_fmt_t *hf);
+char *header_fmt_to_string(const header_fmt_t *hf);
 
 #endif	/* _header_h_ */
 
