@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.c 14954 2007-09-28 19:23:13Z bpringlemeir $
+ * $Id: host_addr.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: host_addr.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
+RCSID("$Id: host_addr.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #ifdef I_NET_IF
 #include <net/if.h>		/* For IFF_* flags */
@@ -50,6 +50,7 @@ RCSID("$Id: host_addr.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
 #include <netdb.h>				/* For gethostbyname() */
 #endif /* I_NETDB */
 
+#include "ascii.h"
 #include "atoms.h"				/* For binary_hash */
 #include "host_addr.h"
 #include "misc.h"
@@ -356,7 +357,7 @@ host_addr_convert(const host_addr_t from, host_addr_t *to,
  * @return The length of the resulting string assuming ``size'' is sufficient.
  */
 size_t
-host_addr_to_string_buf(const host_addr_t ha, gchar *dst, size_t size)
+host_addr_to_string_buf(const host_addr_t ha, char *dst, size_t size)
 {
 	switch (host_addr_net(ha)) {
 	case NET_TYPE_IPV4:
@@ -380,10 +381,10 @@ host_addr_to_string_buf(const host_addr_t ha, gchar *dst, size_t size)
  * @return a pointer to a static buffer holding a NUL-terminated string
  *         representing the given host address.
  */
-const gchar *
+const char *
 host_addr_to_string(const host_addr_t ha)
 {
-	static gchar buf[HOST_ADDR_BUFLEN];
+	static char buf[HOST_ADDR_BUFLEN];
 	size_t n;
 
 	n = host_addr_to_string_buf(ha, buf, sizeof buf);
@@ -405,10 +406,10 @@ host_addr_to_string(const host_addr_t ha)
  */
 size_t
 host_addr_port_to_string_buf(const host_addr_t ha, guint16 port,
-		gchar *dst, size_t size)
+		char *dst, size_t size)
 {
-	gchar host_buf[HOST_ADDR_BUFLEN];
-	gchar port_buf[UINT32_DEC_BUFLEN];
+	char host_buf[HOST_ADDR_BUFLEN];
+	char port_buf[UINT32_DEC_BUFLEN];
 	size_t n;
 
 	host_addr_to_string_buf(ha, host_buf, sizeof host_buf);
@@ -438,10 +439,10 @@ host_addr_port_to_string_buf(const host_addr_t ha, guint16 port,
  * @return a pointer to a static buffer holding a NUL-terminated string
  *         representing the given host address and port.
  */
-const gchar *
+const char *
 host_addr_port_to_string(const host_addr_t ha, guint16 port)
 {
-	static gchar buf[HOST_ADDR_PORT_BUFLEN];
+	static char buf[HOST_ADDR_PORT_BUFLEN];
 	size_t n;
 
 	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
@@ -449,10 +450,10 @@ host_addr_port_to_string(const host_addr_t ha, guint16 port)
 	return buf;
 }
 
-const gchar *
+const char *
 host_addr_port_to_string2(const host_addr_t ha, guint16 port)
 {
-	static gchar buf[HOST_ADDR_PORT_BUFLEN];
+	static char buf[HOST_ADDR_PORT_BUFLEN];
 	size_t n;
 
 	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
@@ -460,6 +461,64 @@ host_addr_port_to_string2(const host_addr_t ha, guint16 port)
 	return buf;
 }
 
+/**
+ * Prints the ``port'' followed by host address ``ha'' to ``dst''. The string
+ * written to ``dst'' is always NUL-terminated unless ``size'' is zero. If
+ * ``size'' is too small, the string will be truncated.
+ *
+ * @param dst the destination buffer; may be NULL iff ``size'' is zero.
+ * @param port the port number.
+ * @param ha the host address.
+ * @param size the size of ``dst'' in bytes.
+ *
+ * @return The length of the resulting string assuming ``size'' is sufficient.
+ */
+size_t
+host_port_addr_to_string_buf(guint16 port, const host_addr_t ha,
+	char *dst, size_t size)
+{
+	char port_buf[UINT32_DEC_BUFLEN];
+	char host_buf[HOST_ADDR_BUFLEN];
+	size_t n;
+
+	uint32_to_string_buf(port, port_buf, sizeof port_buf);
+	host_addr_to_string_buf(ha, host_buf, sizeof host_buf);
+
+	switch (host_addr_net(ha)) {
+	case NET_TYPE_IPV6:
+		n = concat_strings(dst, size, port_buf, ":[", host_buf, "]",
+			(void *) 0);
+		break;
+	case NET_TYPE_IPV4:
+		n = concat_strings(dst, size, port_buf, ":", host_buf, (void *) 0);
+		break;
+	default:
+		n = g_strlcpy(dst, host_buf, size);
+	}
+
+	return n;
+}
+
+/**
+ * Prints the host address ``ha'' followed by ``port'' to a static buffer. 
+ *
+ * @param ha the host address.
+ * @param port the port number.
+ *
+ * @return a pointer to a static buffer holding a NUL-terminated string
+ *         representing the given host address and port.
+ */
+const char *
+port_host_addr_to_string(guint16 port, const host_addr_t ha)
+{
+	static char buf[HOST_ADDR_PORT_BUFLEN];
+	size_t n;
+
+	n = host_port_addr_to_string_buf(port, ha, buf, sizeof buf);
+	g_assert(n < sizeof buf);
+	return buf;
+}
+
 const char *
 host_port_to_string(const char *hostname, host_addr_t addr, guint16 port)
 {
@@ -488,7 +547,7 @@ host_port_to_string(const char *hostname, host_addr_t addr, guint16 port)
  * @return Returns TRUE on success; otherwise FALSE.
  */
 gboolean
-string_to_host_addr(const char *s, const gchar **endptr, host_addr_t *addr_ptr)
+string_to_host_addr(const char *s, const char **endptr, host_addr_t *addr_ptr)
 {
 	guint32 ip;
 
@@ -531,9 +590,9 @@ string_to_host_addr(const char *s, const gchar **endptr, host_addr_t *addr_ptr)
  *         hostname.
  */
 gboolean
-string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
+string_to_host_or_addr(const char *s, const char **endptr, host_addr_t *ha)
 {
-	const gchar *ep;
+	const char *ep;
 	size_t len;
 	host_addr_t addr;
 
@@ -573,24 +632,34 @@ string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
 	return len > 0 && len <= MAX_HOSTLEN;
 }
 
+/**
+ * Parse "IP:port" string to retrieve the host address and port.
+ *
+ * @param str		the string to parse
+ * @param endptr	if not NULL, it will point the first character after
+ * 					the parsed elements.
+ * @param addr_ptr	where the parsed address is written
+ * @param port_ptr	where the parsed port is written
+ *
+ * @return TRUE if we parsed the string correctly, FALSE if it did not
+ * look like a valid "IP:port" string.
+ */
 gboolean
-string_to_host_addr_port(const gchar *str, const gchar **endptr,
+string_to_host_addr_port(const char *str, const char **endptr,
 	host_addr_t *addr_ptr, guint16 *port_ptr)
 {
-	const gchar *ep;
+	const char *ep;
 	host_addr_t addr;
 	gboolean ret;
 	guint16 port;
 
 	ret = string_to_host_or_addr(str, &ep, &addr);
 	if (ret && ':' == *ep && is_host_addr(addr)) {
-		guint32 u;
-		gint error;
+		int error;
 
 		ep++;
-		u = parse_uint32(ep, &ep, 10, &error);
-		port = error || u > 65535 ? 0 : u;
-		ret = 0 != port;
+		port = parse_uint16(ep, &ep, 10, &error);
+		ret = 0 == error && 0 != port;
 	} else {
 		ret = FALSE;
 		port = 0;
@@ -602,11 +671,53 @@ string_to_host_addr_port(const gchar *str, const gchar **endptr,
 		*port_ptr = port;
 	if (endptr)
 		*endptr = ep;
+
+	return ret;
+}
+
+/**
+ * Parse "port:IP" string to retrieve the host address and port.
+ *
+ * @param str		the string to parse
+ * @param endptr	if not NULL, it will point the first character after
+ * 					the parsed elements.
+ * @param port_ptr	where the parsed port is written
+ * @param addr_ptr	where the parsed address is written
+ *
+ * @return TRUE if we parsed the string correctly, FALSE if it did not
+ * look like a valid "port:IP" string.
+ */
+gboolean
+string_to_port_host_addr(const char *str, const char **endptr,
+	guint16 *port_ptr, host_addr_t *addr_ptr)
+{
+	const char *ep;
+	host_addr_t addr;
+	gboolean ret;
+	guint16 port;
+	int error;
+
+	port = parse_uint16(str, &ep, 10, &error);
+	if (!error && ':' == *ep) {
+		ret = string_to_host_or_addr(ep, &ep, &addr);
+		ret = ret && is_host_addr(addr);
+	} else {
+		addr = zero_host_addr;
+		ret = FALSE;
+	}
+
+	if (addr_ptr)
+		*addr_ptr = addr;
+	if (port_ptr)
+		*port_ptr = port;
+	if (endptr)
+		*endptr = ep;
+
 	return ret;
 }
 
 static void
-gethostbyname_error(const gchar *host)
+gethostbyname_error(const char *host)
 {
 #if defined(HAS_HSTRERROR)
 		g_warning("cannot resolve \"%s\": %s", host, hstrerror(h_errno));
@@ -695,7 +806,7 @@ socket_addr_init(socket_addr_t *sa_ptr, enum net_type net)
  * @return		On success, the hostname is returned. Otherwise, NULL is
  *				returned. The resulting string points to a static buffer.
  */
-const gchar *
+const char *
 host_addr_to_name(host_addr_t addr)
 {
 	socket_addr_t sa;
@@ -709,13 +820,13 @@ host_addr_to_name(host_addr_t addr)
 
 #ifdef HAS_GETNAMEINFO
 	{
-		static gchar host[1025];
-		gint error;
+		static char host[1025];
+		int error;
 
 		error = getnameinfo(socket_addr_get_const_sockaddr(&sa),
 					socket_addr_get_len(&sa), host, sizeof host, NULL, 0, 0);
 		if (error) {
-			gchar buf[HOST_ADDR_BUFLEN];
+			char buf[HOST_ADDR_BUFLEN];
 
 			host_addr_to_string_buf(addr, buf, sizeof buf);
 			g_message("getnameinfo() failed for \"%s\": %s",
@@ -728,7 +839,7 @@ host_addr_to_name(host_addr_t addr)
 	{
 		const struct hostent *he;
 		socklen_t len = 0;
-		const gchar *ptr = NULL;
+		const char *ptr = NULL;
 
 		switch (host_addr_net(addr)) {
 		case NET_TYPE_IPV4:
@@ -750,7 +861,7 @@ host_addr_to_name(host_addr_t addr)
 
 		he = gethostbyaddr(ptr, len, socket_addr_get_family(&sa));
 		if (!he) {
-			gchar buf[HOST_ADDR_BUFLEN];
+			char buf[HOST_ADDR_BUFLEN];
 
 			host_addr_to_string_buf(addr, buf, sizeof buf);
 			gethostbyname_error(buf);
@@ -762,14 +873,14 @@ host_addr_to_name(host_addr_t addr)
 }
 
 static GSList *
-resolve_hostname(const gchar *host, enum net_type net)
+resolve_hostname(const char *host, enum net_type net)
 #if defined(HAS_GETADDRINFO)
 {
 	static const struct addrinfo zero_hints;
 	struct addrinfo hints, *ai, *ai0 = NULL;
 	GHashTable *ht;
 	GSList *sl_addr;
-	gint error;
+	int error;
 
 	g_assert(host);
 	
@@ -835,7 +946,7 @@ resolve_hostname(const gchar *host, enum net_type net)
 	const struct hostent *he;
 	GHashTable *ht;
 	GSList *sl_addr;
-	gint af;
+	int af;
 	size_t i;
 
 	g_assert(host);
@@ -923,9 +1034,9 @@ resolve_hostname(const gchar *host, enum net_type net)
  *         On failure, NULL is returned.
  */
 GSList *
-name_to_host_addr(const gchar *host, enum net_type net)
+name_to_host_addr(const char *host, enum net_type net)
 {
-	const gchar *endptr;
+	const char *endptr;
 	host_addr_t addr;
 	
 	g_assert(host);
@@ -969,7 +1080,7 @@ host_addr_free_list(GSList **sl_ptr)
  * and return it.
  */
 host_addr_t
-name_to_single_host_addr(const gchar *host, enum net_type net)
+name_to_single_host_addr(const char *host, enum net_type net)
 {
 	GSList *sl_addr;
 	host_addr_t addr;
@@ -981,7 +1092,7 @@ name_to_single_host_addr(const gchar *host, enum net_type net)
 		size_t i, len;
 
 		len = g_slist_length(sl_addr);
-		i = len > 1 ? (random_raw() % len) : 0;
+		i = len > 1 ? (random_u32() % len) : 0;
 
 		for (sl = sl_addr; NULL != sl; sl = g_slist_next(sl)) {
 			const host_addr_t *addr_ptr = sl->data;
