@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.h 14520 2007-08-17 22:30:06Z cbiere $
+ * $Id: host_addr.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -55,7 +55,7 @@ enum net_type {
 	NET_TYPE_IPV6	= 6
 };
 
-static inline gint
+static inline int
 net_type_to_pf(enum net_type net)
 {
 	switch (net) {
@@ -73,7 +73,7 @@ net_type_to_pf(enum net_type net)
 	return PF_UNSPEC;
 }
 
-static inline gint
+static inline int
 net_type_to_af(enum net_type net)
 {
 	switch (net) {
@@ -122,9 +122,13 @@ static const host_addr_t ipv4_unspecified = {	/* 0.0.0.0/32 */
 	{ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
 };
 
-static const host_addr_t ipv4_loopback = {	/* 0.0.0.0/32 */
+static const host_addr_t ipv4_loopback = {	/* 127.0.0.1/32 */
 	NET_TYPE_IPV4,
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 	{ { 0x7f, 0x00, 0x00, 0x01, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
+#else
+	{ { 0x01, 0x00, 0x00, 0x7f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
+#endif
 };
 
 static const host_addr_t ipv6_unspecified = {	/* ::/128 */
@@ -191,7 +195,7 @@ host_addr_initialized(const host_addr_t ha)
 	return FALSE;
 }
 
-static inline const gchar *
+static inline const char *
 net_type_to_string(enum net_type net)
 {
 	switch (net) {
@@ -278,10 +282,10 @@ host_addr_equal(const host_addr_t a, const host_addr_t b)
 	return FALSE;
 }
 
-static inline gint
+static inline int
 host_addr_cmp(host_addr_t a, host_addr_t b)
 {
-	gint r;
+	int r;
 
 	r = CMP(a.net, b.net);
 	if (0 != r) {
@@ -330,7 +334,7 @@ host_addr_matches(const host_addr_t a, const host_addr_t b, guint8 bits)
 
 	case NET_TYPE_IPV6:
 		{
-			gint i;
+			int i;
 
 			bits = MIN(128, bits);
 			for (i = 0; bits >= 8; i++, bits -= 8) {
@@ -489,7 +493,7 @@ socket_addr_get_sockaddr(socket_addr_t *addr)
 	return (struct sockaddr *) socket_addr_get_const_sockaddr(addr);
 }
 
-static inline gint
+static inline int
 socket_addr_get_family(const socket_addr_t *addr)
 {
 	g_assert(addr != NULL);
@@ -525,25 +529,32 @@ host_addr_is_ipv4_mapped(const host_addr_t addr)
 		host_addr_matches(addr, ipv6_ipv4_mapped, 96);
 }
 
-const gchar *host_addr_to_string(const host_addr_t addr);
-size_t host_addr_to_string_buf(const host_addr_t addr, gchar *, size_t);
-gboolean string_to_host_addr(const gchar *s, const gchar **endptr, host_addr_t *addr_ptr);
-const gchar *host_addr_port_to_string(const host_addr_t addr, guint16 port);
-const gchar *host_addr_port_to_string2(const host_addr_t addr, guint16 port);
+const char *host_addr_to_string(const host_addr_t addr);
+size_t host_addr_to_string_buf(const host_addr_t addr, char *, size_t);
+gboolean string_to_host_addr(const char *s, const char **endptr,
+	host_addr_t *addr_ptr);
+const char *host_addr_port_to_string(const host_addr_t addr, guint16 port);
+const char *host_addr_port_to_string2(const host_addr_t addr, guint16 port);
 size_t host_addr_port_to_string_buf(const host_addr_t addr,
-				guint16 port, gchar *, size_t);
-gboolean string_to_host_addr_port(const gchar *str, const gchar **endptr,
+				guint16 port, char *, size_t);
+gboolean string_to_host_addr_port(const char *str, const char **endptr,
 	host_addr_t *addr_ptr, guint16 *port_ptr);
-const gchar *host_port_to_string(const gchar *hostname,
+gboolean
+string_to_port_host_addr(const char *str, const char **endptr,
+	guint16 *port_ptr, host_addr_t *addr_ptr);
+const char *host_port_to_string(const char *hostname,
 				host_addr_t addr, guint16 port);
+const char *port_host_addr_to_string(guint16 port, const host_addr_t ha);
+size_t host_port_addr_to_string_buf(guint16 port, const host_addr_t ha,
+	char *dst, size_t size);
 
-GSList *name_to_host_addr(const gchar *host, enum net_type net);
+GSList *name_to_host_addr(const char *host, enum net_type net);
 void host_addr_free_list(GSList **sl_ptr);
 
-host_addr_t name_to_single_host_addr(const gchar *host, enum net_type net);
+host_addr_t name_to_single_host_addr(const char *host, enum net_type net);
 
-const gchar *host_addr_to_name(const host_addr_t addr);
-gboolean string_to_host_or_addr(const char *s, const gchar **endptr,
+const char *host_addr_to_name(const host_addr_t addr);
+gboolean string_to_host_or_addr(const char *s, const char **endptr,
 		host_addr_t *ha);
 
 GSList *host_addr_get_interface_addrs(enum net_type net);
