@@ -1,5 +1,5 @@
 /*
- * $Id: inputevt.h 13371 2007-04-19 23:30:54Z cbiere $
+ * $Id: inputevt.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2002, ko (ko-@wanadoo.fr)
  * Copyright (c) 2005, Christian Biere
@@ -61,7 +61,7 @@ typedef enum {
  */
 typedef void (*inputevt_handler_t) (
 	gpointer data,
-	gint source,
+	int source,
 	inputevt_cond_t condition
 );
 
@@ -75,10 +75,10 @@ void inputevt_close(void);
 /**
  * This emulates the GDK input interface.
  */
-guint inputevt_add(gint source, inputevt_cond_t condition,
+guint inputevt_add(int source, inputevt_cond_t condition,
 	inputevt_handler_t handler, gpointer data);
 
-const gchar *inputevt_cond_to_string(inputevt_cond_t cond);
+const char *inputevt_cond_to_string(inputevt_cond_t cond);
 size_t inputevt_data_available(void);
 void inputevt_remove(guint id);
 
