@@ -1,5 +1,5 @@
 /*
- * $Id: iprange.c 14984 2007-10-24 22:30:35Z cbiere $
+ * $Id: iprange.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: iprange.c 14984 2007-10-24 22:30:35Z cbiere $")
+RCSID("$Id: iprange.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #include "host_addr.h"
 #include "iprange.h"
@@ -79,7 +79,7 @@ iprange_db_check(const struct iprange_db * const idb)
 /**
  * Error code stings.
  */
-static const gchar *iprange_errstr[] = {
+static const char *iprange_errstr[] = {
 	"OK",									/**< IPR_ERR_OK */
 	"Incorrect network prefix",				/**< IPR_ERR_BAD_PREFIX */
 	"CIDR range clash",						/**< IPR_ERR_RANGE_CLASH */
@@ -91,12 +91,12 @@ static const gchar *iprange_errstr[] = {
 /**
  * @return human-readable error string for given error code.
  */
-const gchar *
+const char *
 iprange_strerror(iprange_err_t errnum)
 {
 	STATIC_ASSERT(IPR_ERROR_COUNT == G_N_ELEMENTS(iprange_errstr));
 
-	if ((gint) errnum < 0 || errnum >= G_N_ELEMENTS(iprange_errstr))
+	if ((int) errnum < 0 || errnum >= G_N_ELEMENTS(iprange_errstr))
 		return "Invalid error code";
 
 	return iprange_errstr[errnum];
