@@ -1,5 +1,5 @@
 /*
- * $Id: iso3166.h 13870 2007-06-18 03:27:25Z cbiere $
+ * $Id: iso3166.h 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -30,11 +30,11 @@
 
 #define ISO3166_INVALID ((guint16)-1)
 
-guint16 iso3166_encode_cc(const gchar *cc);
+guint16 iso3166_encode_cc(const char *cc);
 void iso3166_init(void);
 void iso3166_close(void);
-const gchar *iso3166_country_name(guint16 code);
-const gchar *iso3166_country_cc(guint16 code);
+const char *iso3166_country_name(guint16 code);
+const char *iso3166_country_cc(guint16 code);
 
 #endif /* _lib_iso3166_h_ */
 /* vi: set ts=4 sw=4 cindent: */
