@@ -1,5 +1,5 @@
 /*
- * $Id: magnet.c 14954 2007-09-28 19:23:13Z bpringlemeir $
+ * $Id: magnet.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -37,18 +37,23 @@
 
 #include "common.h"
 
-RCSID("$Id: magnet.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
+RCSID("$Id: magnet.c 16242 2009-03-24 19:04:20Z cbiere $")
 
-#include "lib/atoms.h"
-#include "lib/glib-missing.h"
-#include "lib/magnet.h"
-#include "lib/tm.h"
-#include "lib/url.h"
-#include "lib/urn.h"
-#include "lib/utf8.h"
-#include "lib/walloc.h"
+#include "ascii.h"
+#include "atoms.h"
+#include "glib-missing.h"
+#include "gnet_host.h"
+#include "magnet.h"
+#include "sequence.h"
+#include "tm.h"
+#include "url.h"
+#include "urn.h"
+#include "utf8.h"
+#include "walloc.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "if/core/guid.h"
+
+#include "override.h"		/* Must be the last header included */
 
 /*
  * Private prototypes;
@@ -67,7 +72,7 @@ enum magnet_key {
 	MAGNET_KEY_EXACT_SOURCE,		/* eXact Source */
 	MAGNET_KEY_EXACT_TOPIC,			/* eXact Topic */
 	MAGNET_KEY_PARQ_ID,				/* PARQ ID */
-	
+
 	NUM_MAGNET_KEYS
 };
 
@@ -75,14 +80,17 @@ static const struct {
 	const char * const key;
 	const enum magnet_key id;
 } magnet_keys[] = {
+	/* Must be sorted alphabetically */
 	{ "",			MAGNET_KEY_NONE },
 	{ "as",			MAGNET_KEY_ALTERNATE_SOURCE },
 	{ "dn",			MAGNET_KEY_DISPLAY_NAME },
 	{ "kt",			MAGNET_KEY_KEYWORD_TOPIC },
+	{ "x.parq-id",	MAGNET_KEY_PARQ_ID },
 	{ "xl",			MAGNET_KEY_EXACT_LENGTH },
 	{ "xs",			MAGNET_KEY_EXACT_SOURCE },
 	{ "xt",			MAGNET_KEY_EXACT_TOPIC },
-	{ "x.parq-id",	MAGNET_KEY_PARQ_ID },
+
+	/* Above line left blank for "!}sort" under vi */
 };
 
 /*
@@ -90,33 +98,45 @@ static const struct {
  */
 
 static void
-clear_error_str(const gchar ***error_str)
+clear_error_str(const char ***error_str)
 {
 	if (NULL == *error_str) {
-		static const gchar *error_dummy;
+		static const char *error_dummy;
 		*error_str = &error_dummy;
 	}
 	**error_str = NULL;
 }
 
-static enum magnet_key
-magnet_key_get(const gchar *s)
+static void
+free_proxies_list(GSList *sl)
 {
-	guint i;
+	g_slist_foreach(sl, gnet_host_free_item, NULL);
+	g_slist_free(sl);
+}
 
+static enum magnet_key
+magnet_key_get(const char *s)
+{
 	STATIC_ASSERT(G_N_ELEMENTS(magnet_keys) == NUM_MAGNET_KEYS);
 	g_assert(s);
+
+#define GET_KEY(i) (magnet_keys[(i)].key)
+#define FOUND(i) G_STMT_START { \
+	return magnet_keys[(i)].id;	\
+	/* NOTREACHED */ \
+} G_STMT_END
+
+	BINARY_SEARCH(const char *, s, G_N_ELEMENTS(magnet_keys), ascii_strcasecmp,
+		GET_KEY, FOUND);
 	
-	for (i = 0; i < G_N_ELEMENTS(magnet_keys); i++) {
-		if (0 == ascii_strcasecmp(magnet_keys[i].key, s))
-			return magnet_keys[i].id;
-	}
+#undef FOUND
+#undef GET_KEY
 
 	return MAGNET_KEY_NONE;
 }
 
 static void
-plus_to_space(gchar *s)
+plus_to_space(char *s)
 {
 	while (s) {
 		s = strchr(s, '+');
@@ -127,53 +147,20 @@ plus_to_space(gchar *s)
 }
 
 static struct magnet_source *
-magnet_parse_location(const gchar *uri, const gchar **error_str)
+magnet_parse_path(const char *path, const char **error_str)
 {
 	static const struct magnet_source zero_ms;
 	struct magnet_source ms;
-	const gchar *p, *endptr, *host, *host_end;
+	const char *p, *endptr;
 
 	clear_error_str(&error_str);
-	g_return_val_if_fail(uri, NULL);
+	g_return_val_if_fail(path, NULL);
 
 	ms = zero_ms;
-	p = uri;
-
-	if (!string_to_host_or_addr(p, &endptr, &ms.addr)) {
-		*error_str = "Expected host part";
-		return NULL;
-	}
-
-	if (is_host_addr(ms.addr)) {
-		host = NULL;
-		host_end = NULL;
-	} else {
-		host = p;
-		host_end = endptr;
-	}
-	p += endptr - p;
-
-	if (':' == *p) {
-		const gchar *ep2;
-		gint error;
-		guint16 u;
-
-		p++;
-		u = parse_uint16(p, &ep2, 10, &error);
-		if (error) {
-			*error_str = "TCP port is out of range";
-			/* Skip this parameter */
-			return NULL;
-		}
-
-		ms.port = u;
-		p += ep2 - p;
-	} else {
-		ms.port = 80;
-	}
+	p = path;
 
 	if ('/' != *p) {
-		*error_str = "Expected port followed by '/'";
+		*error_str = "Expected path starting with '/'";
 		/* Skip this parameter */
 		return NULL;
 	}
@@ -193,19 +180,236 @@ magnet_parse_location(const gchar *uri, const gchar **error_str)
 		ms.path = atom_str_get(p);
 	}
 
-	if (host) {
-		gchar *h = g_strndup(host, host_end - host);
-		ms.hostname = atom_str_get(h);
-		G_FREE_NULL(h);
-	}
-
 	return wcopy(&ms, sizeof ms);
 }
 
-static struct magnet_source *
-magnet_parse_http_source(const gchar *uri, const gchar **error_str)
+static const char *
+magnet_parse_host_port(const char *hostport,
+	host_addr_t *addr, guint16 *port, const char **host, const char **host_end,
+	const char **error_str)
 {
-	const gchar *p;
+	const char *p;
+	const char *endptr;
+
+	clear_error_str(&error_str);
+	g_return_val_if_fail(hostport, NULL);
+
+	p = hostport;
+
+	if (!string_to_host_or_addr(p, &endptr, addr)) {
+		*error_str = "Expected host part";
+		return NULL;
+	}
+
+	if (is_host_addr(*addr)) {
+		if (host)     *host = NULL;
+		if (host_end) *host_end = NULL;
+	} else {
+		if (host)     *host = p;
+		if (host_end) *host_end = endptr;
+	}
+	p += endptr - p;
+
+	if (':' == *p) {
+		const char *ep2;
+		int error;
+		guint16 u;
+
+		p++;
+		u = parse_uint16(p, &ep2, 10, &error);
+		if (error) {
+			*error_str = "TCP port is out of range";
+			/* Skip this parameter */
+			return NULL;
+		}
+
+		*port = u;
+		p += ep2 - p;
+	} else {
+		*port = 80;
+	}
+
+	return p;
+}
+
+static struct magnet_source *
+magnet_parse_location(const char *uri, const char **error_str)
+{
+	struct magnet_source *ms;
+	const char *p, *host, *host_end;
+	host_addr_t addr;
+	guint16 port;
+
+	clear_error_str(&error_str);
+	g_return_val_if_fail(uri, NULL);
+
+	p = uri;
+
+	p = magnet_parse_host_port(uri, &addr, &port, &host, &host_end, error_str);
+	if (NULL == p)
+		return NULL;
+
+	ms = magnet_parse_path(p, error_str);
+	if (NULL == ms)
+		return NULL;
+
+	if (host) {
+		char *h = g_strndup(host, host_end - host);
+		ms->hostname = atom_str_get(h);
+		G_FREE_NULL(h);
+	}
+
+	ms->addr = addr;
+	ms->port = port;
+
+	return ms;
+}
+
+static gboolean
+magnet_parse_legacy_addr_list(const char *proxies,
+	const char **endptr, GSList **list, const char **error_str)
+{
+	GSList *sl = NULL;
+	const char *p;
+
+	clear_error_str(&error_str);
+	g_return_val_if_fail(proxies, FALSE);
+
+	p = proxies;
+
+	if (*p != '{') {
+		*endptr = p;
+		*error_str = "Expected push-proxies list";
+		return FALSE;
+	}
+
+	p++;
+
+	while (*p != '}') {
+		host_addr_t addr;
+		guint16 port;
+		gnet_host_t *host;
+
+		p = magnet_parse_host_port(p, &addr, &port, NULL, NULL, error_str);
+		if (NULL == p)
+			goto cleanup;
+
+		host = gnet_host_new(addr, port);
+		sl = g_slist_prepend(sl, host);		/* Will reverse list below */
+
+		if (*p == ',')
+			p++;
+	}
+
+	sl = g_slist_reverse(sl);		/* Keep order of listed push-proxies */
+	p++;
+	*endptr = p;
+
+	*list = sl;
+	return TRUE;
+
+cleanup:
+	free_proxies_list(sl);
+	return FALSE;
+}
+
+static gboolean
+magnet_parse_addr_list(const char *proxies,
+	const char **endptr, GSList **list, const char **error_str)
+{
+	GSList *sl = NULL;
+	const char *p;
+
+	clear_error_str(&error_str);
+	g_return_val_if_fail(proxies, FALSE);
+
+	p = proxies;
+
+	while (*p == ':') {
+		host_addr_t addr;
+		guint16 port;
+		gnet_host_t *host;
+
+		p = magnet_parse_host_port(p+1, &addr, &port, NULL, NULL, error_str);
+		if (NULL == p)
+			goto cleanup;
+
+		host = gnet_host_new(addr, port);
+		sl = g_slist_prepend(sl, host);		/* Will reverse list below */
+	}
+
+	sl = g_slist_reverse(sl);		/* Keep order of listed push-proxies */
+	*endptr = p;
+
+	*list = sl;
+	return TRUE;
+
+cleanup:
+	free_proxies_list(sl);
+	return FALSE;
+}
+
+static struct magnet_source *
+magnet_parse_proxy_location(const char *uri, const char **error_str)
+{
+	struct magnet_source *ms;
+	const char *p, *endptr = NULL;
+	GSList *sl = NULL;
+
+	clear_error_str(&error_str);
+	g_return_val_if_fail(uri, NULL);
+
+	p = uri;
+
+	/*
+	 * If starts with a '/', then we have no push-proxy following the GUID.
+	 */
+
+	if (*p == '/') {
+		endptr = p;
+		goto path;
+	}
+
+	/*
+	 * Legacy push:// support: push-proxy address given in a set.
+	 * We have a list of push-proxies: {addr1:port,addr2:port}.
+	 * The list may be empty if we have no known push proxies.
+	 *
+	 * FIXME: remove this in 0.96.7.	--RAM, 2009-03-22
+	 */
+
+	if (p[0] == ':' && p[1] == '{') {
+		if (!magnet_parse_legacy_addr_list(uri+1, &endptr, &sl, error_str))
+			return NULL;
+		goto path;
+	}
+
+	if (*p != ':') {
+		*error_str = "Expected / or : following the GUID";
+		return NULL;
+	}
+
+	if (!magnet_parse_addr_list(uri, &endptr, &sl, error_str))
+		return NULL;
+
+	/* FALL THROUGH */
+
+path:
+	ms = magnet_parse_path(endptr, error_str);
+	if (NULL == ms) {
+		free_proxies_list(sl);
+		return NULL;
+	}
+
+	ms->proxies = sl;		/* Perfectly OK to be NULL */
+
+	return ms;
+}
+
+static struct magnet_source *
+magnet_parse_http_source(const char *uri, const char **error_str)
+{
+	const char *p;
 
 	clear_error_str(&error_str);
 	g_return_val_if_fail(uri, NULL);
@@ -217,11 +421,11 @@ magnet_parse_http_source(const gchar *uri, const gchar **error_str)
 }
 
 static struct magnet_source *
-magnet_parse_push_source(const gchar *uri, const gchar **error_str)
+magnet_parse_push_source(const char *uri, const char **error_str)
 {
 	struct magnet_source *ms;
-	const gchar *p, *endptr;
-	gchar guid[GUID_RAW_SIZE];
+	const char *p, *endptr;
+	struct guid guid;
 
 	clear_error_str(&error_str);
 	g_return_val_if_fail(uri, NULL);
@@ -229,26 +433,28 @@ magnet_parse_push_source(const gchar *uri, const gchar **error_str)
 	p = is_strprefix(uri, "push://");
 	g_return_val_if_fail(p, NULL);
 
-	endptr = strchr(p, ':');
+	endptr = strchr(p, ':');		/* First push-proxy host */
+	if (NULL == endptr || GUID_HEX_SIZE != (endptr - p))
+		endptr = strchr(p, '/');	/* No push-proxy host */
+
 	if (
 		NULL == endptr ||
 		GUID_HEX_SIZE != (endptr - p) ||
-		!hex_to_guid(p, guid)
+		!hex_to_guid(p, &guid)
 	) {
 		*error_str = "Bad GUID in push source";
 		return NULL;
 	}
 
-	p = &endptr[1];
-	ms = magnet_parse_location(p, error_str);
+	ms = magnet_parse_proxy_location(endptr, error_str);
 	if (ms) {
-		ms->guid = atom_guid_get(guid);
+		ms->guid = atom_guid_get(&guid);
 	}
 	return ms;
 }
 
 struct magnet_source *
-magnet_parse_exact_source(const gchar *uri, const gchar **error_str)
+magnet_parse_exact_source(const char *uri, const char **error_str)
 {
 	clear_error_str(&error_str);
 	g_return_val_if_fail(uri, NULL);
@@ -270,7 +476,7 @@ magnet_parse_exact_source(const gchar *uri, const gchar **error_str)
 
 static void
 magnet_handle_key(struct magnet_resource *res,
-	const gchar *name, const gchar *value)
+	const char *name, const char *value)
 {
 	char *to_free = NULL;
 
@@ -292,7 +498,7 @@ magnet_handle_key(struct magnet_resource *res,
 			to_free = result;
 		}
 		value = result;
-		g_message("Assuming MAGNET URI key \"%s\" is %s encoded",
+		g_message("assuming MAGNET URI key \"%s\" is %s encoded",
 			name, encoding);
 	}
 
@@ -307,8 +513,9 @@ magnet_handle_key(struct magnet_resource *res,
 	case MAGNET_KEY_EXACT_SOURCE:
 		{
 			struct magnet_source *ms;
+			const char *error;
 
-			ms = magnet_parse_exact_source(value, NULL);
+			ms = magnet_parse_exact_source(value, &error);
 			if (ms) {
 				if (!res->sha1 && ms->sha1) {
 					res->sha1 = atom_sha1_get(ms->sha1);
@@ -318,13 +525,17 @@ magnet_handle_key(struct magnet_resource *res,
 				} else {
 					magnet_source_free(&ms);
 				}
+			} else {
+				g_message("could not parse source \"%s\" in MAGNET URI: %s",
+					value, NULL_STRING(error));
 			}
 		}
 		break;
 
 	case MAGNET_KEY_EXACT_TOPIC:
 		if (!magnet_set_exact_topic(res, value)) {
-			g_message("MAGNET URI contained unsupported exact topic.");
+			g_message("MAGNET URI contained unsupported exact topic \"%s\"",
+				value);
 		}
 		break;
 
@@ -334,7 +545,7 @@ magnet_handle_key(struct magnet_resource *res,
 
 	case MAGNET_KEY_EXACT_LENGTH:
 		{
-			gint error;
+			int error;
 			guint64 u;
 
 			u = parse_uint64(value, NULL, 10, &error);
@@ -349,7 +560,7 @@ magnet_handle_key(struct magnet_resource *res,
 		break;
 
 	case MAGNET_KEY_NONE:
-		g_message("Unhandled parameter in MAGNET URI \"%s\"", name);
+		g_message("unhandled parameter in MAGNET URI: \"%s\"", name);
 		break;
 
 	case NUM_MAGNET_KEYS:
@@ -360,11 +571,11 @@ magnet_handle_key(struct magnet_resource *res,
 }
 
 struct magnet_resource * 
-magnet_parse(const gchar *url, const gchar **error_str)
+magnet_parse(const char *url, const char **error_str)
 {
 	static const struct magnet_resource zero_resource;
 	struct magnet_resource res;
-	const gchar *p, *next;
+	const char *p, *next;
 
 	res = zero_resource;
 	clear_error_str(&error_str);
@@ -383,8 +594,8 @@ magnet_parse(const gchar *url, const gchar **error_str)
 
 	for (/* NOTHING */; p && '\0' != p[0]; p = next) {
 		enum magnet_key key;
-		const gchar *endptr;
-		gchar name[16]; /* Large enough to hold longest key we know */
+		const char *endptr;
+		char name[16]; /* Large enough to hold longest key we know */
 
 		name[0] = '\0';
 		endptr = strchr(p, '=');
@@ -392,7 +603,7 @@ magnet_parse(const gchar *url, const gchar **error_str)
 			size_t name_len;
 
 			name_len = endptr - p;
-			g_assert((ssize_t) name_len > 0);
+			g_assert(size_is_positive(name_len));
 
 			if (name_len < sizeof name) {  /* Ignore overlong key */
 				strncat(name, p, name_len);
@@ -407,9 +618,9 @@ magnet_parse(const gchar *url, const gchar **error_str)
 
 		key = magnet_key_get(name);
 		if (MAGNET_KEY_NONE == key) {
-			g_message("Skipping unknown key in MAGNET URI (%s)", name);
+			g_message("skipping unknown key \"%s\" in MAGNET URI", name);
 		} else {
-			gchar *value;
+			char *value;
 			size_t value_len;
 
 			value_len = endptr - p;
@@ -419,7 +630,7 @@ magnet_parse(const gchar *url, const gchar **error_str)
 			if (url_unescape(value, TRUE)) {
 				magnet_handle_key(&res, name, value);
 			} else {
-				g_message("Invalidly encoded value in MAGNET URI");
+				g_message("badly encoded value in MAGNET URI: \"%s\"", value);
 			}
 			G_FREE_NULL(value);
 		}
@@ -448,6 +659,10 @@ magnet_source_free(struct magnet_source **ms_ptr)
 		atom_sha1_free_null(&ms->sha1);
 		atom_tth_free_null(&ms->tth);
 		atom_guid_free_null(&ms->guid);
+		if (ms->proxies) {
+			free_proxies_list(ms->proxies);
+			ms->proxies = NULL;
+		}
 		wfree(ms, sizeof *ms);
 		*ms_ptr = NULL;
 	}
@@ -474,7 +689,7 @@ magnet_resource_free(struct magnet_resource **res_ptr)
 		res->sources = NULL;
 
 		for (sl = res->searches; sl != NULL; sl = g_slist_next(sl)) {
-			const gchar *s = sl->data;
+			const char *s = sl->data;
 			atom_str_free_null(&s);
 		}
 		g_slist_free(res->searches);
@@ -509,7 +724,7 @@ magnet_add_source(struct magnet_resource *res, struct magnet_source *s)
 }
 
 void
-magnet_add_source_by_url(struct magnet_resource *res, const gchar *url)
+magnet_add_source_by_url(struct magnet_resource *res, const char *url)
 {
 	struct magnet_source *s;
 
@@ -523,7 +738,7 @@ magnet_add_source_by_url(struct magnet_resource *res, const gchar *url)
 
 void
 magnet_add_sha1_source(struct magnet_resource *res, const struct sha1 *sha1,
-	const host_addr_t addr, const guint16 port, const gchar *guid)
+	const host_addr_t addr, const guint16 port, const struct guid *guid)
 {
 	struct magnet_source *s;
 
@@ -545,7 +760,7 @@ magnet_add_sha1_source(struct magnet_resource *res, const struct sha1 *sha1,
 }
 
 void
-magnet_add_search(struct magnet_resource *res, const gchar *search)
+magnet_add_search(struct magnet_resource *res, const char *search)
 {
 	g_return_if_fail(res);
 	g_return_if_fail(search);
@@ -583,7 +798,7 @@ magnet_set_tth(struct magnet_resource *res, const struct tth *tth)
 
 
 gboolean
-magnet_set_exact_topic(struct magnet_resource *res, const gchar *topic)
+magnet_set_exact_topic(struct magnet_resource *res, const char *topic)
 {
 	struct sha1 sha1;
 	struct tth tth;
@@ -612,9 +827,9 @@ magnet_set_exact_topic(struct magnet_resource *res, const gchar *topic)
 }
 
 void
-magnet_set_display_name(struct magnet_resource *res, const gchar *name)
+magnet_set_display_name(struct magnet_resource *res, const char *name)
 {
-	const gchar *atom;
+	const char *atom;
 
 	g_return_if_fail(res);
 	g_return_if_fail(name);
@@ -632,7 +847,7 @@ magnet_set_filesize(struct magnet_resource *res, filesize_t size)
 
 static inline void
 magnet_append_item(GString **gs_ptr, gboolean escape_value,
-	const gchar *key, const gchar *value)
+	const char *key, const char *value)
 {
 	GString *gs;
 
@@ -652,7 +867,7 @@ magnet_append_item(GString **gs_ptr, gboolean escape_value,
 	gs = g_string_append_c(gs, '=');
 
 	if (escape_value) {
-		gchar *escaped;
+		char *escaped;
 
 		escaped = url_escape_query(value);
 		gs = g_string_append(gs, escaped);
@@ -666,30 +881,82 @@ magnet_append_item(GString **gs_ptr, gboolean escape_value,
 	*gs_ptr = gs;
 }
 
-static gchar *
-magnet_source_to_string(struct magnet_source *s)
+static char *
+proxy_sequence_to_string(const sequence_t *s)
 {
-	gchar *url;
+	GString *gs;
+
+	gs = g_string_new(NULL);
+
+	if (!sequence_is_empty(s)) {
+		sequence_iter_t *iter;
+
+		iter = sequence_forward_iterator(s);
+		while (sequence_iter_has_next(iter)) {
+			gnet_host_t *host = sequence_iter_next(iter);
+			gs = g_string_append_c(gs, ':');
+			g_string_append(gs, gnet_host_to_string(host));
+		}
+		sequence_iterator_release(&iter);
+	}
+
+	return gm_string_finalize(gs);
+}
+
+static char *
+proxies_to_string(GSList *proxies)
+{
+	sequence_t seq;
+
+	return proxy_sequence_to_string(sequence_fill_from_gslist(&seq, proxies));
+}
+
+/**
+ * Create the string representation of the push-proxies, for inclusion
+ * in the push:// URL.
+ *
+ * @return An empty string (""), if the list is empty or the address NULL;
+ * otherwise a colon-separated list of IP:port, beginning with a colon. The
+ * string is newly allocated.
+ */
+char *
+magnet_proxies_to_string(hash_list_t *proxies)
+{
+	sequence_t seq;
+
+	if (NULL == proxies)
+		return g_strdup("");
+	
+	return proxy_sequence_to_string(
+		sequence_fill_from_hash_list(&seq, proxies));
+}
+
+/**
+ * Convert magnet source to a string representation.
+ *
+ * @return A newly allocated string.
+ */
+char *
+magnet_source_to_string(const struct magnet_source *s)
+{
+	char *url;
 
 	g_return_val_if_fail(s, NULL);
 
 	if (s->url) {
 		url = g_strdup(s->url);
 	} else {
-		const gchar *host, *prefix;
-		gchar prefix_buf[256];
-		gchar port_buf[16];
-
-		g_return_val_if_fail(0 != s->port, NULL);
-		g_return_val_if_fail(s->hostname || is_host_addr(s->addr), NULL);
-		g_return_val_if_fail(s->path || s->sha1, NULL);
+		char *proxies = NULL;
+		const char *host, *prefix;
+		char prefix_buf[256];
+		char port_buf[16];
 
 		if (s->guid) {
-			gchar guid_buf[GUID_HEX_SIZE + 1];
+			char guid_buf[GUID_HEX_SIZE + 1];
 			
 			guid_to_string_buf(s->guid, guid_buf, sizeof guid_buf);
 			concat_strings(prefix_buf, sizeof prefix_buf,
-				"push://", guid_buf, ":", (void *) 0);
+				"push://", guid_buf, (void *) 0);
 			prefix = prefix_buf;
 		} else {
 			prefix = "http://";
@@ -702,24 +969,34 @@ magnet_source_to_string(struct magnet_source *s)
 				gm_snprintf(port_buf, sizeof port_buf, ":%u",
 					(unsigned) s->port);
 			}
+		} else if (s->guid) {
+			host = proxies = proxies_to_string(s->proxies);
 		} else {
 			host = host_addr_port_to_string(s->addr, s->port);
 		}
 		if (s->path) {
 			url = g_strconcat(prefix, host, port_buf, s->path, (void *) 0);
-		} else {
-			g_assert(s->sha1);
+		} else if (s->sha1) {
 			url = g_strconcat(prefix, host, port_buf,
 					"/uri-res/N2R?", bitprint_to_urn_string(s->sha1, s->tth),
 					(void *) 0);
+		} else {
+			url = g_strconcat(prefix, host, port_buf, "/", (void *) 0);
 		}
+
+		G_FREE_NULL(proxies);
 	}
 
 	return url;
 }
 
-gchar *
-magnet_to_string(struct magnet_resource *res)
+/**
+ * Create a string representation of the magnet resource.
+ *
+ * @return A newly allocated string.
+ */
+char *
+magnet_to_string(const struct magnet_resource *res)
 {
 	GString *gs;
 	GSList *sl;
@@ -731,7 +1008,7 @@ magnet_to_string(struct magnet_resource *res)
 		magnet_append_item(&gs, TRUE, "dn", res->display_name);
 	}
 	if (0 != res->size) {
-		gchar buf[UINT64_DEC_BUFLEN];
+		char buf[UINT64_DEC_BUFLEN];
 
 		uint64_to_string_buf(res->size, buf, sizeof buf);
 		magnet_append_item(&gs, FALSE, "xl", buf);
@@ -745,7 +1022,7 @@ magnet_to_string(struct magnet_resource *res)
 	}
 
 	for (sl = res->sources; NULL != sl; sl = g_slist_next(sl)) {
-		gchar *url;
+		char *url;
 
 		url = magnet_source_to_string(sl->data);
 		magnet_append_item(&gs, TRUE, "xs", url);
@@ -765,9 +1042,9 @@ magnet_to_string(struct magnet_resource *res)
  * valid for a certain source.
  */
 void
-magnet_set_parq_id(struct magnet_resource *res, const gchar *parq_id)
+magnet_set_parq_id(struct magnet_resource *res, const char *parq_id)
 {
-	const gchar *atom;
+	const char *atom;
 
 	g_return_if_fail(res);
 	g_return_if_fail(parq_id);
