@@ -1,5 +1,5 @@
 /*
- * $Id: misc.h 15055 2007-12-09 01:31:13Z cbiere $
+ * $Id: misc.h 16281 2009-03-28 20:03:21Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -52,6 +52,9 @@
 
 #include "common.h"
 
+#include "compat_sleep_ms.h"
+#include "fs_free_space.h"
+#include "tm.h"
 #include "vmm.h"
 
 #define SIZE_FIELD_MAX 64		/**< Max size of sprintf-ed size quantity */
@@ -60,7 +63,7 @@
 #define GUID_BASE32_SIZE	26	/**< base32 GUID representation */
 
 typedef struct short_string {
-	gchar str[SIZE_FIELD_MAX];
+	char str[SIZE_FIELD_MAX];
 } short_string_t;
 
 /**
@@ -69,11 +72,11 @@ typedef struct short_string {
 #ifndef USE_GLIB2
 
 #ifndef HAS_STRLCPY
-size_t strlcpy(gchar *dst, const gchar *src, size_t dst_size);
+size_t strlcpy(char *dst, const char *src, size_t dst_size);
 #endif /* HAS_STRLCPY */
 
 #ifndef HAS_STRLCAT
-size_t strlcat(gchar *dst, const gchar *src, size_t dst_size);
+size_t strlcat(char *dst, const char *src, size_t dst_size);
 #endif /* HAS_STRLCAT */
 
 #define g_string_printf g_string_sprintf
@@ -81,13 +84,10 @@ size_t strlcat(gchar *dst, const gchar *src, size_t dst_size);
 #define g_strlcat strlcat
 #endif
 
-size_t concat_strings(gchar *dst, size_t size,
-	const gchar *s, ...) G_GNUC_NULL_TERMINATED;
-size_t w_concat_strings(gchar **dst,
-	const gchar *first, ...) G_GNUC_NULL_TERMINATED;
-
-gint ascii_strcasecmp(const gchar *s1, const gchar *s2);
-gint ascii_strncasecmp(const gchar *s1, const gchar *s2, size_t len);
+size_t concat_strings(char *dst, size_t size,
+	const char *s, ...) G_GNUC_NULL_TERMINATED;
+size_t w_concat_strings(char **dst,
+	const char *first, ...) G_GNUC_NULL_TERMINATED;
 
 /**
  * Converts an integer to a single hexadecimal ASCII digit. The are no checks,
@@ -103,207 +103,19 @@ hex_digit(guchar x)
 	return hex_alphabet_lower[x & 0xf]; 
 }
 
-/**
- * Converts a hexadecimal char (0-9, A-F, a-f) to an integer.
- *
- * @param c the character to convert.
- * @return 0..15 for valid hexadecimal ASCII characters, -1 otherwise.
- */
-static inline gint
-hex2int_inline(guchar c)
-{
-	extern const gint8 *hex2int_tab;
-	return hex2int_tab[c];
-}
-
-/**
- * Converts a decimal char (0-9) to an integer.
- *
- * @param c the character to convert.
- * @return 0..9 for valid decimal ASCII characters, -1 otherwise.
- */
-static inline gint
-dec2int_inline(guchar c)
-{
-	extern const gint8 *dec2int_tab;
-	return dec2int_tab[c];
-}
-
-/**
- * Converts an alphanumeric char (0-9, A-Z, a-z) to an integer.
- *
- * @param c the character to convert.
- * @return 0..9 for valid alphanumeric ASCII characters, -1 otherwise.
- */
-static inline gint
-alnum2int_inline(guchar c)
-{
-	extern const gint8 *alnum2int_tab;
-	return alnum2int_tab[c];
-}
-
-/**
- * ctype-like functions that allow only ASCII characters whereas the locale
- * would allow others. The parameter doesn't have to be casted to (unsigned
- * char) because these functions return false for everything out of [0..127].
- *
- * GLib 2.x has similar macros/functions but defines only a subset.
- */
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_blank(gint c)
-{
-	return c == 32 || c == 9;	/* space, tab */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_cntrl(gint c)
-{
-	return (c >= 0 && c <= 31) || c == 127;
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_digit(gint c)
-{
-	return c >= 48 && c <= 57;	/* 0-9 */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_xdigit(gint c)
-{
-	return -1 != hex2int_inline(c) && !(c & ~0x7f);
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_upper(gint c)
-{
-	return c >= 65 && c <= 90;		/* A-Z */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_lower(gint c)
-{
-	return c >= 97 && c <= 122;		/* a-z */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_alpha(gint c)
-{
-	return is_ascii_upper(c) || is_ascii_lower(c);	/* A-Z, a-z */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_alnum(gint c)
-{
-	return -1 != alnum2int_inline(c) && !(c & ~0x7f); /* A-Z, a-z, 0-9 */
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_space(gint c)
-{
-	return c == 32 || (c >= 9 && c <= 13);
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_graph(gint c)
-{
-	return c >= 33 && c <= 126;
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_print(gint c)
-{
-	return is_ascii_graph(c) || c == 32;
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
-is_ascii_punct(gint c)
-{
-	return c >= 33 && c <= 126 && !is_ascii_alnum(c);
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gint
-ascii_toupper(gint c)
-{
-	return is_ascii_lower(c) ? c - 32 : c;
-}
-
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gint
-ascii_tolower(gint c)
-{
-	return is_ascii_upper(c) ? c + 32 : c;
-}
-
 #if !GLIB_CHECK_VERSION(2,4,0)
-static inline WARN_UNUSED_RESULT const gchar *
-g_strip_context(const gchar *id, const gchar *val)
+static inline WARN_UNUSED_RESULT const char *
+g_strip_context(const char *id, const char *val)
 {
-	const gchar *s;
+	const char *s;
 
 	s = id != val ? NULL : strchr(id, '|');
 	return s ? ++s : val;
 }
 #endif /* GLib < 2.4.0 */
 
-/**
- * Skips over all ASCII space characters starting at ``s''.
- *
- * @return a pointer to the first non-space character starting from s.
- */
-static inline WARN_UNUSED_RESULT gchar *
-skip_ascii_spaces(const gchar *s)
-{
-	while (is_ascii_space(*s))
-		s++;
-
-	return deconstify_gchar(s);
-}
-
-/**
- * Skips over all characters which are not ASCII spaces starting at ``s''.
- *
- * @return a pointer to the first space or NUL character starting from s.
- */
-static inline WARN_UNUSED_RESULT gchar *
-skip_ascii_non_spaces(const gchar *s)
-{
-	while ('\0' != *s && !is_ascii_space(*s))
-		s++;
-
-	return deconstify_gchar(s);
-}
-
-/**
- * Skips over all characters which are ASCII alphanumerical characters
- * starting at ``s''.
- *
- * @return a pointer to the first space or NUL character starting from s.
- */
-static inline WARN_UNUSED_RESULT gchar *
-skip_ascii_alnum(const gchar *s)
-{
-	while (is_ascii_alnum(*s))
-		s++;
-
-	return deconstify_gchar(s);
-}
-
-/**
- * Skips over all ASCII blank characters starting at ``s''.
- *
- * @return A pointer to the first non-blank character starting from s.
- */
-static inline WARN_UNUSED_RESULT gchar *
-skip_ascii_blanks(const gchar *s)
-{
-	while (is_ascii_blank(*s))
-		s++;
-
-	return deconstify_gchar(s);
-}
-
-static inline WARN_UNUSED_RESULT gchar *
-skip_dir_separators(const gchar *s)
+static inline WARN_UNUSED_RESULT char *
+skip_dir_separators(const char *s)
 {
 	while ('/' == s[0] || G_DIR_SEPARATOR == s[0])
 		s++;
@@ -345,120 +157,59 @@ typedef guint16 flag_t;
 #define HOST_ADDR_BUFLEN	(MAX(IPV4_ADDR_BUFLEN, IPV6_ADDR_BUFLEN))
 #define HOST_ADDR_PORT_BUFLEN	(HOST_ADDR_BUFLEN + sizeof ":[65535]")
 
-gboolean parse_ipv6_addr(const gchar *s, uint8_t *dst, const gchar **endptr);
-const gchar *ipv6_to_string(const guint8 *ipv6);
-size_t ipv6_to_string_buf(const guint8 *ipv6, gchar *dst, size_t size);
+gboolean parse_ipv6_addr(const char *s, uint8_t *dst, const char **endptr);
+const char *ipv6_to_string(const guint8 *ipv6);
+size_t ipv6_to_string_buf(const guint8 *ipv6, char *dst, size_t size);
 
 /*
  * Network related string routines
  */
-guint32  string_to_ip(const gchar *);
-gboolean string_to_ip_strict(const gchar *s, guint32 *addr, const gchar **ep);
-gboolean string_to_ip_and_mask(const gchar *str, guint32 *ip, guint32 *netmask);
-gboolean string_to_ip_port(const gchar *str, guint32 *ip, guint16 *port);
-const gchar *ip_to_string(guint32);
-size_t ipv4_to_string_buf(guint32 ip, gchar *buf, size_t size);
-const gchar *hostname_port_to_string(const gchar *hostname, guint16 port);
-const gchar *local_hostname(void);
+guint32  string_to_ip(const char *);
+gboolean string_to_ip_strict(const char *s, guint32 *addr, const char **ep);
+gboolean string_to_ip_and_mask(const char *str, guint32 *ip, guint32 *netmask);
+gboolean string_to_ip_port(const char *str, guint32 *ip, guint16 *port);
+const char *ip_to_string(guint32);
+size_t ipv4_to_string_buf(guint32 ip, char *buf, size_t size);
+const char *hostname_port_to_string(const char *hostname, guint16 port);
+const char *local_hostname(void);
 #define port_is_valid(port) (port != 0)
 
 /*
  * Date string conversions
  */
-const gchar *timestamp_to_string(time_t date);
-const gchar *timestamp_utc_to_string(time_t date);
-const gchar *timestamp_rfc822_to_string(time_t date);
-const gchar *timestamp_rfc822_to_string2(time_t date);
-const gchar *timestamp_rfc1123_to_string(time_t date);
+const char *timestamp_to_string(time_t date);
+const char *timestamp_utc_to_string(time_t date);
+const char *timestamp_rfc822_to_string(time_t date);
+const char *timestamp_rfc822_to_string2(time_t date);
+const char *timestamp_rfc1123_to_string(time_t date);
 
-size_t timestamp_to_string_buf(time_t date, gchar *dst, size_t size);
-size_t timestamp_utc_to_string_buf(time_t date, gchar *dst, size_t size);
-size_t time_locale_to_string_buf(time_t date, gchar *dst, size_t size);
+size_t timestamp_to_string_buf(time_t date, char *dst, size_t size);
+size_t timestamp_utc_to_string_buf(time_t date, char *dst, size_t size);
+size_t time_locale_to_string_buf(time_t date, char *dst, size_t size);
 
 short_string_t timestamp_get_string(time_t date);
 
-/*
- * We use the direct difference of time_t values instead of difftime()
- * for performance. Just in case there is any system which requires difftime()
- * e.g. if time_t is BCD-encoded, define USE_DIFFTIME.
- */
-#if defined(USE_DIFFTIME)
-typedef gint64 time_delta_t;
-
-static inline time_delta_t
-delta_time(time_t t1, time_t t0)
-{
-	return difftime(t1, t0);
-}
-#else	/* !USE_DIFFTIME */
-typedef time_t time_delta_t;
-
-static inline time_delta_t
-delta_time(time_t t1, time_t t0)
-{
-	return t1 - t0;
-}
-
-static inline void
-time_t_check(void)
-{
-	/* If time_t is not a signed integer type, we cannot calculate properly
-	 * with the raw values. Define USE_DIFFTIME, if this check fails.*/
-	STATIC_ASSERT((time_t) -1 < 0);
-}
-#endif /* USE_DIFFTIME*/
-
-/**
- * Advances the given timestamp by delta using saturation arithmetic.
- * @param t the timestamp to advance.
- * @param delta the amount of seconds to advance.
- * @return the advanced timestamp or TIME_T_MAX.
- */
-static inline time_t
-time_advance(time_t t, gulong delta)
-{
-	/* Using time_t for delta and TIME_T_MAX instead of INT_MAX
-	 * would be cleaner but give a confusing interface. Jumping 136
-	 * years in time should be enough for everyone. Most systems
-	 * don't allow us to advance a time_t beyond 2038 anyway.
-	 */
-
-	do {
-		glong d;
-
-		d = MIN(delta, (gulong) LONG_MAX);
-		if (d >= TIME_T_MAX - t) {
-			t = TIME_T_MAX;
-			break;
-		}
-		t += d;
-		delta -= d;
-	} while (delta > 0);
-
-	return t;
-}
-
 /*
  * Time string conversions
  */
-const gchar *short_time(time_delta_t s);
-const gchar *short_time_ascii(time_delta_t t);
-const gchar *short_uptime(time_delta_t s);
-const gchar *compact_time(time_delta_t t);
+const char *short_time(time_delta_t s);
+const char *short_time_ascii(time_delta_t t);
+const char *short_uptime(time_delta_t s);
+const char *compact_time(time_delta_t t);
 
 /*
  * Size string conversions
  */
-const gchar *short_size(guint64 size, gboolean metric);
-const gchar *short_html_size(guint64 size, gboolean metric);
-const gchar *short_kb_size(guint64 size, gboolean metric);
-const gchar *short_rate(guint64 rate, gboolean metric);
-const gchar *compact_size(guint64 size, gboolean metric);
-const gchar *compact_rate(guint64 rate, gboolean metric);
-const gchar *compact_kb_size(guint32 size, gboolean metric);
-const gchar *nice_size(guint64 size, gboolean metric);
-gchar *short_value(gchar *buf, size_t size, guint64 v, gboolean metric);
-gchar *compact_value(gchar *buf, size_t size, guint64 v, gboolean metric);
+const char *short_size(guint64 size, gboolean metric);
+const char *short_html_size(guint64 size, gboolean metric);
+const char *short_kb_size(guint64 size, gboolean metric);
+const char *short_rate(guint64 rate, gboolean metric);
+const char *compact_size(guint64 size, gboolean metric);
+const char *compact_rate(guint64 rate, gboolean metric);
+const char *compact_kb_size(guint32 size, gboolean metric);
+const char *nice_size(guint64 size, gboolean metric);
+char *short_value(char *buf, size_t size, guint64 v, gboolean metric);
+char *compact_value(char *buf, size_t size, guint64 v, gboolean metric);
 
 short_string_t short_rate_get_string(guint64 rate, gboolean metric);
 
@@ -505,26 +256,34 @@ const char *bitprint_to_urn_string(const struct sha1 *, const struct tth *);
 /*
  * GUID<->hex string conversion
  */
-const gchar *guid_hex_str(const gchar *guid);
-gboolean hex_to_guid(const gchar *hexguid, gchar *guid);
-size_t guid_to_string_buf(const gchar *guid, gchar *dst, size_t size);
-const gchar *guid_to_string(const gchar *guid);
+struct guid;
+
+const char *guid_hex_str(const struct guid *);
+gboolean hex_to_guid(const char *, struct guid *);
+size_t guid_to_string_buf(const struct guid *, char *, size_t);
+const char *guid_to_string(const struct guid *);
 
 /*
  * GUID<->base32 string conversion
  */
-gchar *guid_base32_str(const gchar *guid);
-gchar *base32_to_guid(const gchar *base32);
+const char *guid_base32_str(const struct guid *);
+const struct guid *base32_to_guid(const char *);
+
+/*
+ * Generic binary to hexadecimal conversion.
+ */
+size_t bin_to_hex_buf(const void *data, size_t len, char *dst, size_t size);
 
 /*
  * Tests
  */
 gboolean is_absolute_path(const char *pathname);
-gboolean is_directory(const gchar *pathname);
-gboolean is_regular(const gchar *pathname);
-gboolean is_symlink(const gchar *pathname);
-gboolean file_exists(const gchar *pathname);
-gboolean file_does_not_exist(const gchar *pathname);
+gboolean is_directory(const char *pathname);
+gboolean is_regular(const char *pathname);
+gboolean is_symlink(const char *pathname);
+int is_same_file(const char *, const char *);
+gboolean file_exists(const char *pathname);
+gboolean file_does_not_exist(const char *pathname);
 guint32 next_pow2(guint32 n);
 
 #define IS_POWER_OF_2(x) ((x) && 0 == ((x) & ((x) - 1)))
@@ -551,34 +310,35 @@ is_pow2(guint32 value)
  */
 void random_init(void);
 guint32 random_value(guint32 max) WARN_UNUSED_RESULT;
-guint32 random_raw(void) WARN_UNUSED_RESULT;
-void guid_random_fill(gchar *xuid);
+guint32 random_u32(void) WARN_UNUSED_RESULT;
+void random_bytes(void *dst, size_t size);
+void guid_random_fill(struct guid *);
 
 /*
  * Stuff
  */
 void misc_init(void);
-size_t str_chomp(gchar *str, size_t len);
-gint hex2int(guchar c);
-gboolean is_printable(const gchar *buf, gint len);
-void dump_hex(FILE *, const gchar *, gconstpointer, gint);
-void locale_strlower(gchar *, const gchar *);
-void ascii_strlower(gchar *dst, const gchar *src);
-gint strcmp_delimit(const gchar *a, const gchar *b, const gchar *delimit);
-gint ascii_strcasecmp_delimit(const gchar *a, const gchar *b,
-		const gchar *delimit);
-size_t filename_shrink(const gchar *filename, gchar *buf, size_t size);
-char *unique_filename(const gchar *path, const gchar *file, const gchar *ext,
-		gboolean (*name_is_uniq)(const gchar *pathname));
-gchar *hex_escape(const gchar *name, gboolean strict);
-gchar *control_escape(const gchar *s);
-const gchar *lazy_string_to_printf_escape(const gchar *src);
-gint highest_bit_set(guint32 n) G_GNUC_CONST;
-gfloat force_range(gfloat value, gfloat min, gfloat max);
-gchar *absolute_pathname(const gchar *file);
-gchar *make_pathname(const gchar *dir, const gchar *file);
-gchar *short_filename(gchar *fullname);
-gchar *data_hex_str(const gchar *data, size_t len);
+size_t str_chomp(char *str, size_t len);
+int hex2int(guchar c);
+gboolean is_printable(const char *buf, int len);
+void dump_hex(FILE *, const char *, gconstpointer, int);
+void dump_string(FILE *out, const char *str, size_t len, const char *trailer);
+gboolean is_printable_iso8859_string(const char *s);
+void locale_strlower(char *, const char *);
+size_t filename_shrink(const char *filename, char *buf, size_t size);
+char *unique_filename(const char *path, const char *file, const char *ext,
+		gboolean (*name_is_uniq)(const char *pathname));
+char *hex_escape(const char *name, gboolean strict);
+char *control_escape(const char *s);
+const char *lazy_string_to_printf_escape(const char *src);
+int highest_bit_set(guint32 n) G_GNUC_CONST;
+size_t common_leading_bits(
+	gconstpointer k1, size_t k1bits, gconstpointer k2, size_t k2bits);
+float force_range(float value, float min, float max);
+char *absolute_pathname(const char *file);
+char *make_pathname(const char *dir, const char *file);
+char *short_filename(char *fullname);
+char *data_hex_str(const char *data, size_t len);
 
 #if defined(S_IROTH) && defined(S_IXOTH)
 /* 0755 */
@@ -588,51 +348,52 @@ gchar *data_hex_str(const gchar *data, size_t len);
 #define DEFAULT_DIRECTORY_MODE (S_IRWXU | S_IRGRP | S_IXGRP)
 #endif /* S_IROTH && S_IXOTH */
 
-gint create_directory(const gchar *dir, mode_t mode);
-gint compat_mkdir(const gchar *path, mode_t mode);
-gboolean filepath_exists(const gchar *dir, const gchar *file);
-const gchar * filepath_basename(const gchar *pathname);
-gchar * filepath_directory(const gchar *pathname);
-guint16 parse_uint16(const gchar *, gchar const **, guint, gint *)
+int create_directory(const char *dir, mode_t mode);
+int compat_mkdir(const char *path, mode_t mode);
+gboolean filepath_exists(const char *dir, const char *file);
+const char * filepath_basename(const char *pathname);
+char * filepath_directory(const char *pathname);
+guint16 parse_uint16(const char *, char const **, guint, int *)
 	NON_NULL_PARAM((1, 4));
-guint32 parse_uint32(const gchar *, gchar const **, guint, gint *)
+guint32 parse_uint32(const char *, char const **, guint, int *)
 	NON_NULL_PARAM((1, 4));
-guint64 parse_uint64(const gchar *, gchar const **, guint, gint *)
+guint64 parse_uint64(const char *, char const **, guint, int *)
 	NON_NULL_PARAM((1, 4));
-size_t uint32_to_string_buf(guint32 v, gchar *dst, size_t size);
-size_t uint64_to_string_buf(guint64 v, gchar *dst, size_t size);
-size_t off_t_to_string_buf(off_t v, gchar *dst, size_t size);
-size_t time_t_to_string_buf(time_t v, gchar *dst, size_t size);
-const gchar *uint32_to_string(guint32 v);
-const gchar *uint64_to_string(guint64 v);
-const gchar *uint64_to_string2(guint64 v);
-const gchar *off_t_to_string(off_t v);
-const gchar *time_t_to_string(time_t v);
-const gchar *filesize_to_string(filesize_t v);
-gint parse_major_minor(const gchar *src, gchar const **endptr,
+size_t int32_to_string_buf(gint32 v, char *dst, size_t size);
+size_t uint32_to_string_buf(guint32 v, char *dst, size_t size);
+size_t uint64_to_string_buf(guint64 v, char *dst, size_t size);
+size_t off_t_to_string_buf(off_t v, char *dst, size_t size);
+size_t time_t_to_string_buf(time_t v, char *dst, size_t size);
+const char *uint32_to_string(guint32 v);
+const char *uint64_to_string(guint64 v);
+const char *uint64_to_string2(guint64 v);
+const char *off_t_to_string(off_t v);
+const char *time_t_to_string(time_t v);
+const char *filesize_to_string(filesize_t v);
+const char *filesize_to_string2(filesize_t v);
+int parse_major_minor(const char *src, char const **endptr,
 	guint *major, guint *minor);
-gchar *is_strprefix(const gchar *s, const gchar *prefix) WARN_UNUSED_RESULT;
-gchar *is_strcaseprefix(const gchar *s, const gchar *prefix) WARN_UNUSED_RESULT;
-size_t html_escape(const gchar *src, gchar *dst, size_t dst_size);
-guint32 html_decode_entity(const gchar *src, const gchar **endptr);
-gint canonize_path(gchar *dst, const gchar *path);
+char *is_strprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
+char *is_strcaseprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
+size_t html_escape(const char *src, char *dst, size_t dst_size);
+guint32 html_decode_entity(const char *src, const char **endptr);
+int canonize_path(char *dst, const char *path);
 guint compat_max_fd(void);
 void close_file_descriptors(const int first_fd);
 int reserve_standard_file_descriptors(void);
 gboolean compat_is_superuser(void);
 int compat_daemonize(const char *directory);
-void set_close_on_exec(gint fd);
+void set_close_on_exec(int fd);
 void compat_fadvise_sequential(int fd, off_t offset, off_t size);
 void *compat_memmem(const void *data, size_t data_size,
 		const void *pattern, size_t pattern_size);
 
 int get_non_stdio_fd(int fd);
 
-typedef void (*signal_handler_t)(gint signo);
-signal_handler_t set_signal(gint signo, signal_handler_t handler);
+typedef void (*signal_handler_t)(int signo);
+signal_handler_t set_signal(int signo, signal_handler_t handler);
 
-gchar *ascii_strcasestr(const gchar *haystack, const gchar *needle);
-gchar *normalize_dir_separators(const gchar *s);
+char *normalize_dir_separators(const char *s);
 size_t memcmp_diff(const void *a, const void *b, size_t n);
 guint32 cpu_noise(void);
 
@@ -643,50 +404,115 @@ pointer_hash_func(const void *p)
 	return (((guint64) 0x4F1BBCDCUL * v) >> 32) ^ v;
 }
 
-static inline guint
-str_case_hash_func(gconstpointer key)
-{
-	const guchar *s = key;
-	gulong c, hash = 0;
-	
-	while ((c = ascii_tolower(*s++))) {
-		hash ^= (hash << 8) | c;
-	}
-	return hash ^ (((guint64) 1048573 * hash) >> 32);
-}
-
-static inline gint
-str_case_eq_func(gconstpointer a, gconstpointer b)
-{
-	return 0 == ascii_strcasecmp(a, b);
-}
-
 /**
  * Determines the length of a NUL-terminated string looking only at the first
  * "src_size" bytes. If src[0..size] contains no NUL byte, "src_size" is
  * returned. Otherwise, the returned value is identical to strlen(str). Thus,
  * it is safe to pass a possibly non-terminated buffer.
- * 
+ *
+ * @param src An initialized buffer.
+ * @param src_size The size of src in number of bytes. IF AND ONLY IF,
+ *        src is NUL-terminated, src_size may exceed the actual buffer length.
  * @return The number of bytes in "src" before the first found NUL or src_size
  *		   if there is no NUL.
  */
 static inline size_t
-clamp_strlen(const gchar *src, size_t src_size)
+clamp_strlen(const char *src, size_t src_size)
 {
-	const gchar *p;
-	
-	p = memchr(src, '\0', src_size);
-	return (p ? p : &src[src_size]) - src;
+	const char *p, *endptr = &src[src_size];
+
+	/* @NOTE: memchr() is intentionally NOT used because 'src_size' is allowed
+	 *        to exceed the size of the memory object 'src'.
+	 */
+	for (p = src; p != endptr && '\0' != *p; p++)
+		continue;
+
+	return p - src;
 }
 
-static inline const gchar *
-NULL_STRING(const gchar *s)
+/**
+ * Copies at most MIN(dst_size, src_len) bytes from "src" to "dst".
+ *
+ * @param dst the destination buffer.
+ * @param dst_size the size of dst in number of bytes. 
+ * @param src the source buffer. 
+ * @param src_len the length of src in number of bytes.
+ *
+ * @return The number of copied bytes.
+ */
+static inline size_t
+clamp_memcpy(char *dst, size_t dst_size, const char *src, size_t src_len)
+{
+	size_t n;
+
+	n = MIN(dst_size, src_len);
+	memcpy(dst, src, n);
+	return n;
+}
+
+/**
+ * Copies at most MIN(dst_size - 1, src_len) characters from the buffer "src"
+ * to the buffer "dst", ensuring the resulting string in "dst" is
+ * NUL-terminated and truncating it if necessary. If "src_len" is (size_t)-1,
+ * "src" must be NUL-terminated, otherwise the first "src_len" bytes of "src"
+ * must be initialized but a terminating NUL is not necessary.
+ *
+ * @NOTE: The 'dst' buffer is NOT padded with NUL-bytes.
+ *
+ * @param dst the destination buffer.
+ * @param dst_size the size of dst in number of bytes. 
+ * @param src a NUL-terminated string or at an initialized buffer of least
+ *        "src_len" bytes.
+ * @param src_len the length of src in number of bytes to copy at maximum. May
+ *        be (size_t)-1 if "src" is NUL-terminated.
+ *
+ * @return The length of the resulting string in number of bytes.
+ */
+static inline size_t
+clamp_strncpy(char *dst, size_t dst_size, const char *src, size_t src_len)
+{
+	if (dst_size-- > 0) {
+		size_t n;
+
+		if ((size_t) -1 == src_len) {
+			src_len = clamp_strlen(src, dst_size);
+		}
+		n = clamp_memcpy(dst, dst_size, src, src_len);
+		dst[n] = '\0';
+		return n;
+	} else {
+		return 0;
+	}
+}
+
+/**
+ * Copies at most "dst_size - 1" characters from the NUL-terminated string
+ * "src" to the buffer "dst", ensuring the resulting string in "dst" is
+ * NUL-terminated and truncating it if necessary.
+ *
+ * @NOTE: The 'dst' buffer is NOT padded with NUL-bytes.
+ *
+ * @param dst the destination buffer.
+ * @param dst_size the size of dst in number of bytes. 
+ * @param src a NUL-terminated string.
+ *
+ * @return The length of the resulting string in number of bytes.
+ */
+static inline size_t
+clamp_strcpy(char *dst, size_t dst_size, const char *src)
+{
+	return clamp_strncpy(dst, dst_size, src, (size_t) -1);
+}
+
+
+static inline const char *
+NULL_STRING(const char *s)
 {
 	return NULL != s ? s : "(null)";
 }
 
-static inline const gchar *
-EMPTY_STRING(const gchar *s)
+static inline const char *
+EMPTY_STRING(const char *s)
 {
 	return NULL != s ? s : "";
 }
@@ -768,7 +594,7 @@ round_size(size_t align, size_t n)
 struct stat;
 
 static inline gboolean
-is_temporary_error(gint error)
+is_temporary_error(int error)
 {
   switch (error) {
   case EAGAIN:
@@ -782,12 +608,80 @@ is_temporary_error(gint error)
 }
 
 /* Wrapper around lseek() to handle filesize -> off_t conversion. */
-gint seek_to_filepos(gint fd, filesize_t pos);
+int seek_to_filepos(int fd, filesize_t pos);
+filesize_t get_random_file_offset(const filesize_t size);
 
 guint filesize_per_100(filesize_t size, filesize_t part);
 guint filesize_per_1000(filesize_t size, filesize_t part);
 guint filesize_per_10000(filesize_t size, filesize_t part);
 
+/*
+ * NOTE: ssize_t is NOT the signed variant of size_t and casting values blindly
+ * to ssize_t may cause integer overflows.  Larger values, especially SIZE_MAX
+ * (size_t)-1 may be the result of errors or wrap arounds during calculations.
+ * Therefore in places where memory objects larger than half of the address
+ * space are unreasonable, the following two functions are useful to check for
+ * such conditions.
+ */
+
+/*
+ * Check whether a signed representation of size would be non-negative.
+ * @return TRUE if size is equal to zero or larger and smaller than
+ *         SIZE_MAX / 2.
+ */
+static inline gboolean
+size_is_non_negative(size_t size)
+{
+	return size <= SIZE_MAX / 2;
+}
+
+/**
+ * Check whether a signed representation of size would be positive.
+ * @return TRUE if size is larger than zero and smaller than SIZE_MAX / 2.
+ */
+static inline gboolean
+size_is_positive(size_t size)
+{
+	return size_is_non_negative(size - 1);
+}
+
+/*
+ * Calculate the sum of a and b but saturate towards SIZE_MAX.
+ * @return SIZE_MAX if a + b > SIZE_MAX, otherwise a + b.
+ */
+static inline size_t
+size_saturate_add(size_t a, size_t b)
+{
+	size_t ret = a + b;
+	if (G_UNLIKELY(ret < a))
+		return SIZE_MAX;
+	return ret;
+}
+
+/*
+ * Calculate the product of a and b but saturate towards SIZE_MAX.
+ * @return SIZE_MAX if a * b > SIZE_MAX, otherwise a * b.
+ */
+static inline size_t
+size_saturate_mult(size_t a, size_t b)
+{
+	if (G_UNLIKELY(0 != a && SIZE_MAX / a < b))
+		return SIZE_MAX;
+	return a * b;
+}
+
+/*
+ * Calculate the difference between a and b but saturate towards zero.
+ * @return zero if a < b, otherwise a - b.
+ */
+static inline size_t
+size_saturate_sub(size_t a, size_t b)
+{
+	if (G_UNLIKELY(a < b))
+		return 0;
+	return a - b;
+}
+
 /*
  * CIDR split of IP range.
  */
@@ -815,7 +709,7 @@ G_STMT_START { \
 	size_t bs_index, bs_j = 0, bs_k; \
 	for (bs_k = (bs_size); bs_k != 0; bs_k >>= 1) { \
 		bs_type bs_item; \
-		gint bs_cmp_result; \
+		int bs_cmp_result; \
 \
 		bs_index = bs_j + (bs_k >> 1); \
 		bs_item = bs_get_key(bs_index); \
