@@ -1,5 +1,5 @@
 /*
- * $Id: pattern.c,v 1.7 2005/09/10 10:10:36 daichik Exp $
+ * $Id: pattern.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pattern.c,v 1.7 2005/09/10 10:10:36 daichik Exp $");
+RCSID("$Id: pattern.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #include "misc.h"
 #include "pattern.h"
@@ -83,19 +83,14 @@ pattern_close(void)
  * @return a compiled pattern structure.
  */
 cpattern_t *
-pattern_compile(const gchar *pattern)
+pattern_compile(const char *pattern)
 {
 	cpattern_t *p = zalloc(pat_zone);
 	size_t plen, i, *pd = p->delta;
 	const guchar *c;
 
-	plen = strlen(pattern);
-	{
-		gchar *copy = g_malloc(1 + plen);
-		memcpy(copy, pattern, 1 + plen);
-		p->pattern = copy;
-	}
-	p->len = plen;
+	p->pattern = g_strdup(pattern);
+	p->len = plen = strlen(p->pattern);
 	p->duped = TRUE;
 
 	plen++;			/* Avoid increasing within the loop */
@@ -120,7 +115,7 @@ pattern_compile(const gchar *pattern)
  * NB: There is no pattern_free_fast(), just call zfree() on the result.
  */
 cpattern_t *
-pattern_compile_fast(const gchar *pattern, size_t plen)
+pattern_compile_fast(const char *pattern, size_t plen)
 {
 	cpattern_t *p = zalloc(pat_zone);
 	size_t i, *pd = p->delta;
@@ -152,7 +147,7 @@ pattern_free(cpattern_t *cpat)
 {
 	if (cpat->duped) {
 		g_free(deconstify_gchar(cpat->pattern));
-		cpat->pattern = NULL;
+		cpat->pattern = NULL; /* Don't use G_FREE_NULL b/c of lvalue cast */
 	}
 	zfree(pat_zone, cpat);
 }
@@ -164,19 +159,19 @@ pattern_free(cpattern_t *cpat)
  *
  * @return pointer to beginning of matching substring, NULL if not found.
  */
-const gchar *
+const char *
 pattern_qsearch(
 	cpattern_t *cpat,		/**< Compiled pattern */
-	const gchar *text,		/**< Text we're scanning */
+	const char *text,		/**< Text we're scanning */
 	size_t tlen,			/**< Text length, 0 = compute strlen(text) */
 	size_t toffset,			/**< Offset within text for search start */
 	qsearch_mode_t word)	/**< Beginning/whole word matching? */
 {
-	const gchar *p;			/* Pointer within string pattern */
-	const gchar *t;			/* Pointer within text */
-	const gchar *tp;		/* Initial local search text pointer */
-	const gchar *start;		/* Start of matching */
-	const gchar *end;		/* End of text (first byte after physical end) */
+	const char *p;			/* Pointer within string pattern */
+	const char *t;			/* Pointer within text */
+	const char *tp;		/* Initial local search text pointer */
+	const char *start;		/* Start of matching */
+	const char *end;		/* End of text (first byte after physical end) */
 	size_t i;				/* Position within pattern string */
 	size_t plen;
 
