@@ -1,5 +1,5 @@
 /*
- * $Id: prop.c 14690 2007-08-25 06:03:16Z cbiere $
+ * $Id: prop.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,22 +25,26 @@
 
 #include "common.h"
 
-RCSID("$Id: prop.c 14690 2007-08-25 06:03:16Z cbiere $")
+RCSID("$Id: prop.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
+#include "ascii.h"
 #include "getdate.h"
 #include "prop.h"
+#include "debug.h"
 #include "file.h"
 #include "misc.h"
+#include "sha1.h"
 #include "glib-missing.h"
 #include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
+#define PROP_FILE_ID	"_id"
+
 #define debug track_props
 static guint32 track_props = 0;	/**< XXX need to init lib's props--RAM */
-static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
 
 const struct {
-	const gchar *name; 
+	const char *name; 
 } prop_type_str[] = {
 	{ "boolean" 	},
 	{ "guint32" 	},
@@ -64,14 +68,14 @@ G_STMT_START {														\
 	}																\
 } G_STMT_END
 
-typedef gint (* prop_parse_func_t)(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i);
+typedef int (* prop_parse_func_t)(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i);
 
-static gint
-prop_parse_guint64(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+static int
+prop_parse_guint64(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i)
 {
-	gint error;
+	int error;
 	guint64 u;
 	
 	u = parse_uint64(str, endptr, 10, &error);
@@ -85,12 +89,12 @@ prop_parse_guint64(const gchar *name,
 	return error;
 }
 
-static gint
-prop_parse_timestamp(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+static int
+prop_parse_timestamp(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i)
 {
-	const gchar *ep;
-	gint error = 0;
+	const char *ep;
+	int error = 0;
 	guint64 u;
 	time_t t;
 
@@ -100,7 +104,8 @@ prop_parse_timestamp(const gchar *name,
 			"str=\"%s\": \"%s\"", name, str, g_strerror(error));
 	}
 	if (*ep != '-') {
-		t = u;	/* For backwards-compatibility accept raw numeric timestamps */
+		t = MIN(u + (time_t) 0, TIME_T_MAX + (guint64) 0);
+		/* For backwards-compatibility accept raw numeric timestamps */
 	} else {
 		t = date2time(str, tm_time());
 		if ((time_t)-1 == t) {
@@ -120,11 +125,11 @@ prop_parse_timestamp(const gchar *name,
 }
 
 
-static gint
-prop_parse_guint32(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+static int
+prop_parse_guint32(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i)
 {
-	gint error;
+	int error;
 	guint32 u;
 	
 	u = parse_uint32(str, endptr, 10, &error);
@@ -138,13 +143,13 @@ prop_parse_guint32(const gchar *name,
 	return error;
 }
 
-static gint
-prop_parse_ip(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+static int
+prop_parse_ip(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i)
 {
 	host_addr_t addr;
-	gint error;
-	const gchar *ep;
+	int error;
+	const char *ep;
 	
 	g_assert(name);
 	g_assert(str);
@@ -172,12 +177,12 @@ prop_parse_ip(const gchar *name,
 /**
  * Parse comma delimited boolean vector (TRUE/FALSE list).
  */
-static gint
-prop_parse_boolean(const gchar *name,
-	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+static int
+prop_parse_boolean(const char *name,
+	const char *str, const char **endptr, gpointer vec, size_t i)
 {
 	static const struct {
-		const gchar *s;
+		const char *s;
 		const gboolean v;
 	} tab[] = {
 		{ "0",		FALSE },
@@ -186,9 +191,9 @@ prop_parse_boolean(const gchar *name,
 		{ "TRUE",	TRUE },
 	};
 	gboolean b = FALSE;
-	const gchar *p = NULL;
+	const char *p = NULL;
 	guint j;
-	gint error = 0;
+	int error = 0;
 	
 	g_assert(name);
 	g_assert(str);
@@ -221,18 +226,18 @@ prop_parse_boolean(const gchar *name,
  * Parse prop vector.
  */
 static void
-prop_parse_vector(const gchar *name, const gchar *str,
+prop_parse_vector(const char *name, const char *str,
 	size_t size, gpointer vec, prop_parse_func_t parser)
 {
-	const gchar *p = str;
+	const char *p = str;
 	size_t i;
 
 	g_assert(str != NULL);
 	g_assert(vec != NULL);
 
 	for (i = 0; i < size && p; i++) {
-		const gchar *endptr;
-		gint error;
+		const char *endptr;
+		int error;
 
 		p = skip_ascii_spaces(p);
 		if ('\0' == *p)
@@ -258,14 +263,14 @@ prop_parse_vector(const gchar *name, const gchar *str,
 }
 
 static void
-prop_parse_guint64_vector(const gchar *name, const gchar *str,
+prop_parse_guint64_vector(const char *name, const char *str,
 	size_t size, guint64 *vec)
 {
 	prop_parse_vector(name, str, size, vec, prop_parse_guint64);
 }
 
 static void
-prop_parse_timestamp_vector(const gchar *name, const gchar *str,
+prop_parse_timestamp_vector(const char *name, const char *str,
 	size_t size, time_t *vec)
 {
 	prop_parse_vector(name, str, size, vec, prop_parse_timestamp);
@@ -273,21 +278,21 @@ prop_parse_timestamp_vector(const gchar *name, const gchar *str,
 
 
 static void
-prop_parse_guint32_vector(const gchar *name, const gchar *str,
+prop_parse_guint32_vector(const char *name, const char *str,
 	size_t size, guint32 *vec)
 {
 	prop_parse_vector(name, str, size, vec, prop_parse_guint32);
 }
 
 static void
-prop_parse_ip_vector(const gchar *name, const gchar *str,
+prop_parse_ip_vector(const char *name, const char *str,
 	size_t size, host_addr_t *vec)
 {
 	prop_parse_vector(name, str, size, vec, prop_parse_ip);
 }
 
 static void
-prop_parse_boolean_vector(const gchar *name, const gchar *str,
+prop_parse_boolean_vector(const char *name, const char *str,
 	size_t size, gboolean *vec)
 {
 	prop_parse_vector(name, str, size, vec, prop_parse_boolean);
@@ -295,12 +300,12 @@ prop_parse_boolean_vector(const gchar *name, const gchar *str,
 
 
 /**
- * Parse a hex string into a gchar array.
+ * Parse a hex string into a char array.
  *
  * @return TRUE if the data was fully parsed. FALSE on failure.
  */
 static gboolean 
-prop_parse_storage(const gchar *name, const gchar *str, size_t size, gchar *t)
+prop_parse_storage(const char *name, const char *str, size_t size, char *t)
 {
 	size_t i;
 
@@ -312,7 +317,7 @@ prop_parse_storage(const gchar *name, const gchar *str, size_t size, gchar *t)
 	}
 
 	for (i = 0; i < size; i++) {
-		gchar h, l;
+		char h, l;
 
 		h = str[i * 2];
 		l = str[i * 2 + 1];
@@ -421,9 +426,9 @@ prop_get_def(prop_set_t *ps, property_t p)
 		break;
 
 	case PROP_TYPE_STRING:
-		buf->data.string.def	= g_new(gchar*, 1);
+		buf->data.string.def	= g_new(char*, 1);
 		*buf->data.string.def   = g_strdup(*PROP(ps,p).data.string.def);
-		buf->data.string.value  = g_new(gchar*, 1);
+		buf->data.string.value  = g_new(char*, 1);
 		*buf->data.string.value = g_strdup(*PROP(ps,p).data.string.value);
 		break;
 
@@ -674,7 +679,7 @@ prop_set_guint64(prop_set_t *ps, property_t prop, const guint64 *src,
 			) {
 				*PROP(ps,prop).data.guint64.value = *src;
 			} else {
-				gchar buf[64];
+				char buf[64];
 				guint64 newval = *src;
 
 				if (newval > PROP(ps,prop).data.guint64.max)
@@ -928,7 +933,7 @@ prop_set_timestamp(prop_set_t *ps, property_t prop, const time_t *src,
 			) {
 				*PROP(ps,prop).data.timestamp.value = *src;
 			} else {
-				gchar buf[64];
+				char buf[64];
 				time_t newval = *src;
 
 				if (newval > PROP(ps,prop).data.timestamp.max)
@@ -1082,7 +1087,7 @@ prop_get_ip(prop_set_t *ps, property_t prop, host_addr_t *t,
 
 
 void
-prop_set_storage(prop_set_t *ps, property_t prop, const gchar *src,
+prop_set_storage(prop_set_t *ps, property_t prop, const char *src,
 	size_t length)
 {
 	gboolean differ = FALSE;
@@ -1111,15 +1116,15 @@ prop_set_storage(prop_set_t *ps, property_t prop, const gchar *src,
 	if (debug >= 5) {
 		printf("updated property [%s] (binary)\n", PROP(ps,prop).name);
 		dump_hex(stderr, PROP(ps,prop).name,
-			(const gchar *) PROP(ps,prop).data.storage.value,
+			(const char *) PROP(ps,prop).data.storage.value,
 			PROP(ps,prop).vector_size);
 	}
 
 	prop_emit_prop_changed(ps, prop);
 }
 
-gchar *
-prop_get_storage(prop_set_t *ps, property_t prop, gchar *t, size_t length)
+char *
+prop_get_storage(prop_set_t *ps, property_t prop, char *t, size_t length)
 {
 	gpointer target;
 
@@ -1142,10 +1147,27 @@ prop_get_storage(prop_set_t *ps, property_t prop, gchar *t, size_t length)
 	return target;
 }
 
-void
-prop_set_string(prop_set_t *ps, property_t prop, const gchar *val)
+const char *
+prop_get_storage_const(prop_set_t *ps, property_t prop)
 {
-	gchar *old;
+	g_assert(ps != NULL);
+
+	if (!prop_in_range(ps, prop))
+		g_error("prop_get_storage: unknown property %d", prop);
+	if (PROP(ps,prop).type != PROP_TYPE_STORAGE)
+		g_error("Type mismatch getting value for [%s] of type"
+			" %s when %s was expected",
+			PROP(ps,prop).name,
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_STORAGE].name);
+
+	return PROP(ps,prop).data.storage.value;
+}
+
+void
+prop_set_string(prop_set_t *ps, property_t prop, const char *val)
+{
+	char *old;
 	gboolean differ = FALSE;
 
 	g_assert(ps != NULL);
@@ -1203,11 +1225,11 @@ prop_set_string(prop_set_t *ps, property_t prop, const gchar *val)
  * returned. This memory must be free'ed later. The size parameter has
  * no effect in this case.
  */
-gchar *
-prop_get_string(prop_set_t *ps, property_t prop, gchar *t, size_t size)
+char *
+prop_get_string(prop_set_t *ps, property_t prop, char *t, size_t size)
 {
-	gchar *target;
-	gchar *s;
+	char *target;
+	char *s;
 
 	g_assert(ps != NULL);
 	g_assert(NULL == t || size > 0);
@@ -1251,7 +1273,7 @@ prop_get_string(prop_set_t *ps, property_t prop, gchar *t, size_t size)
  *         to copy the returned string as it points to a "static const"
  *		   buffer.
  */
-const gchar *
+const char *
 prop_name(prop_set_t *ps, property_t prop)
 {
 	return PROP(ps,prop).name;
@@ -1264,7 +1286,7 @@ prop_name(prop_set_t *ps, property_t prop)
  *         to copy the returned string as it points to a "static const"
  *		   buffer.
  */
-const gchar *
+const char *
 prop_description(prop_set_t *ps, property_t prop)
 {
 	return PROP(ps,prop).desc;
@@ -1277,7 +1299,7 @@ prop_description(prop_set_t *ps, property_t prop)
  *         to copy the returned string as it points to a "static const"
  *		   buffer.
  */
-const gchar *
+const char *
 prop_type_to_string(prop_set_t *ps, property_t prop)
 {
 	g_assert(PROP(ps,prop).type < NUM_PROP_TYPES);
@@ -1294,10 +1316,10 @@ prop_is_saved(prop_set_t *ps, property_t prop)
 /**
  * Fetches the value of property as a string.
  */
-const gchar *
+const char *
 prop_to_string(prop_set_t *ps, property_t prop)
 {
-	static gchar s[4096];
+	static char s[4096];
 
 	g_assert(ps != NULL);
 
@@ -1310,7 +1332,7 @@ prop_to_string(prop_set_t *ps, property_t prop)
 			guint32 val;
 
 			prop_get_guint32(ps, prop, &val, 0, 1);
-			gm_snprintf(s, sizeof(s), "%u", val);
+			uint32_to_string_buf(val, s, sizeof s);
 		}
 		break;
 	case PROP_TYPE_GUINT64:
@@ -1330,12 +1352,7 @@ prop_to_string(prop_set_t *ps, property_t prop)
 		}
 		break;
 	case PROP_TYPE_STRING:
-		{
-			gchar *buf = prop_get_string(ps, prop, NULL, 0);
-
-			g_strlcpy(s, buf ? buf : "", sizeof s);
-			G_FREE_NULL(buf);
-		}
+		prop_get_string(ps, prop, s, sizeof s);
 		break;
 	case PROP_TYPE_IP:
 		{
@@ -1375,8 +1392,10 @@ prop_to_string(prop_set_t *ps, property_t prop)
 		}
 		break;
 	case PROP_TYPE_STORAGE:
-		g_strlcpy(s, guid_hex_str(prop_get_storage(ps, prop, NULL,
-			PROP(ps,prop).vector_size)), sizeof s);
+		{
+			bin_to_hex_buf(prop_get_storage_const(ps, prop),
+				PROP(ps,prop).vector_size, s, sizeof s);
+		}
 		break;
 	default:
 		s[0] = '\0';
@@ -1390,10 +1409,10 @@ prop_to_string(prop_set_t *ps, property_t prop)
 /**
  * Fetches the default value of property as a string.
  */
-const gchar *
+const char *
 prop_default_to_string(prop_set_t *ps, property_t prop)
 {
-	static gchar s[4096];
+	static char s[4096];
 	const prop_def_t *p = &PROP(ps, prop);
 	
 	switch (p->type) {
@@ -1435,8 +1454,10 @@ prop_default_to_string(prop_set_t *ps, property_t prop)
 		}
 		break;
 	case PROP_TYPE_STORAGE:
-		g_strlcpy(s, guid_hex_str(prop_get_storage(ps, prop, NULL,
-			PROP(ps,prop).vector_size)), sizeof s);
+		{
+			bin_to_hex_buf(prop_get_storage_const(ps, prop),
+				PROP(ps,prop).vector_size, s, sizeof s);
+		}
 		break;
 	default:
 		s[0] = '\0';
@@ -1451,10 +1472,10 @@ prop_default_to_string(prop_set_t *ps, property_t prop)
 /**
  * @return "TRUE" or "FALSE" depending on the given boolean value.
  */
-static const gchar *
+static const char *
 config_boolean(gboolean b)
 {
-	static const gchar b_true[] = "TRUE", b_false[] = "FALSE";
+	static const char b_true[] = "TRUE", b_false[] = "FALSE";
 	return b ? b_true : b_false;
 }
 
@@ -1466,10 +1487,10 @@ config_boolean(gboolean b)
  *
  * @return A newly allocated string holding the formatted comment.
  */
-static gchar *
-config_comment(const gchar *s)
+static char *
+config_comment(const char *s)
 {
-	const gchar *word;
+	const char *word;
 	size_t line_len;
 	GString *out;
 
@@ -1481,7 +1502,7 @@ config_comment(const gchar *s)
 
 	while ('\0' != word[0]) {
 		static const size_t max_len = 72;
-		const gchar *endptr;
+		const char *endptr;
 		size_t word_len;
 
 		endptr = skip_ascii_non_spaces(word);
@@ -1506,13 +1527,35 @@ config_comment(const gchar *s)
 	return gm_string_finalize(out);
 }
 
+/**
+ * Generate a unique token representative of the file on the filesystem,
+ * based on the device ID and inode number.
+ */
+static const char *
+unique_file_token(const struct stat *st)
+{
+	static char buf[SHA1_BASE16_SIZE + 1];		/* Hexadecimal format */
+	SHA1Context ctx;
+	struct sha1 digest;
+
+	SHA1Reset(&ctx);
+	SHA1Input(&ctx, &st->st_dev, sizeof st->st_dev);
+	SHA1Input(&ctx, &st->st_ino, sizeof st->st_ino);
+	SHA1Result(&ctx, &digest);
+
+	bin_to_hex_buf(digest.data, sizeof digest.data, buf, sizeof buf);
+	buf[SHA1_BASE16_SIZE] = '\0';
+
+	return buf;
+}
+
 /**
  * Like prop_save_to_file(), but only perform when dirty, i.e. when at least
  * one persisted property changed since the last time we saved.
  */
 void
-prop_save_to_file_if_dirty(prop_set_t *ps, const gchar *dir,
-	const gchar *filename)
+prop_save_to_file_if_dirty(prop_set_t *ps, const char *dir,
+	const char *filename)
 {
 	if (!ps->dirty)
 		return;
@@ -1528,12 +1571,12 @@ prop_save_to_file_if_dirty(prop_set_t *ps, const gchar *dir,
  * saving.
  */
 void
-prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
+prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 {
 	FILE *config;
 	struct stat sb;
-	gchar *newfile;
-	gchar *pathname;
+	char *newfile;
+	char *pathname;
 	guint n;
 
 	g_assert(filename != NULL);
@@ -1555,7 +1598,7 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		 */
 
 		if (ps->mtime && delta_time(sb.st_mtime, ps->mtime) > 0) {
-			gchar *old = g_strconcat(pathname, ".old", (void *) 0);
+			char *old = g_strconcat(pathname, ".old", (void *) 0);
 			g_warning("config file \"%s\" changed whilst I was running",
 				pathname);
 			if (-1 == rename(pathname, old))
@@ -1589,7 +1632,7 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 			GTA_RELEASE, GTA_WEBSITE);
 
 	{
-		gchar *comment = config_comment(ps->desc);
+		char *comment = config_comment(ps->desc);
 
 		fprintf(config,
 			"#\n# Description of contents\n"
@@ -1600,21 +1643,21 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 
 	for (n = 0; n < ps->size; n++) {
 		prop_def_t *p = &ps->props[n];
-		gchar **vbuf;
+		char **vbuf;
 		guint i;
-		gchar sbuf[1024];
-		gchar *val = NULL;
+		char sbuf[1024];
+		char *val = NULL;
 		gboolean quotes = FALSE;
 		gboolean defaultvalue = TRUE;
 
 		if (p->save == FALSE)
 			continue;
 
-		vbuf = g_new(gchar*, p->vector_size+1);
+		vbuf = g_new(char*, p->vector_size+1);
 		vbuf[0] = NULL;
 
 		{
-			gchar *comment = config_comment(p->desc);
+			char *comment = config_comment(p->desc);
 
 			fprintf(config, "%s\n", comment);
 			G_FREE_NULL(comment);
@@ -1706,19 +1749,11 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 			break;
 		case PROP_TYPE_STORAGE:
 			{
-				const guchar *data = p->data.storage.value;
+				size_t hex_size = (p->vector_size * 2) + 1;
 
-				val = g_malloc((p->vector_size * 2) + 1);
-
-				for (i = 0; i < p->vector_size; i++) {
-					static const char hex_alphabet_lower[] = "0123456789abcdef";
-					gint c = data[i];
-
-					val[i * 2] = hex_alphabet_lower[c >> 4];
-					val[i * 2 + 1] = hex_alphabet_lower[c & 0x0f];
-				}
-
-				val[i * 2] = '\0';
+				val = g_malloc(hex_size);
+				bin_to_hex_buf(p->data.storage.value, p->vector_size,
+					val, hex_size);
 				quotes = TRUE;
 
 				/* No default values for storage type properties. */
@@ -1738,6 +1773,17 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		g_strfreev(vbuf);
 	}
 
+	/*
+	 * Write a unique token identifying this file, kept accross rename()
+	 * but not if the file is copied.
+	 */
+
+	if (-1 != fstat(fileno(config), &sb)) {
+		const char *id = unique_file_token(&sb);
+		fprintf(config, "# File ID (internal)\n");
+		fprintf(config, "%s = \"%s\"\n\n", PROP_FILE_ID, id);
+	}
+
 	fprintf(config, "### End of configuration file ###\n");
 
 	/*
@@ -1762,7 +1808,7 @@ end:
  * Called by prop_load_from_file to actually set the properties.
  */
 void
-prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
+prop_set_from_string(prop_set_t *ps, property_t prop, const char *val,
 	gboolean saved_only)
 {
 	prop_def_t *p;
@@ -1842,8 +1888,8 @@ prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
 		break;
 	case PROP_TYPE_STORAGE:
 		{
-			gchar s[1024];
-			gchar *d, *buf;
+			char s[1024];
+			char *d, *buf;
 
 			if (p->vector_size > sizeof s) {
 				d = g_malloc(p->vector_size);
@@ -1869,41 +1915,57 @@ prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
  * Called by prop_load_from_file to actually set the properties.
  */
 static void
-load_helper(prop_set_t *ps, property_t prop, const gchar *val)
+load_helper(prop_set_t *ps, property_t prop, const char *val)
 {
 	prop_set_from_string(ps, prop, val, TRUE);
 }
 
-void
-prop_load_from_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
+/**
+ * Load properties from file.
+ *
+ * @param ps		the property set associated with the file
+ * @param dir		directory where file lies
+ * @param filename	basename of file to load properties from
+ *
+ * @return FALSE if we have reasons to believe that the property file was
+ * not generated for this instance of gtk-gnutella but copied from another
+ * instance, TRUE otherwise.
+ */
+gboolean
+prop_load_from_file(prop_set_t *ps, const char *dir, const char *filename)
 {
 	static const char fmt[] = "Bad line %u in config file, ignored";
-	static gchar prop_tmp[4096];
+	static char prop_tmp[4096];
 	FILE *config;
-	gchar *path;
+	char *path;
 	guint n = 1;
 	struct stat buf;
 	gboolean truncated = FALSE;
+	gboolean good_id = FALSE;
+	const char *file_id;
 
 	g_assert(dir != NULL);
 	g_assert(filename != NULL);
 	g_assert(ps != NULL);
 
 	if (!is_directory(dir))
-		return;
+		return TRUE;
 
 	path = make_pathname(dir, filename);
 	config = file_fopen(path, "r");
 	if (!config) {
 		G_FREE_NULL(path);
-		return;
+		return TRUE;
 	}
 
-	if (-1 == fstat(fileno(config), &buf))
+	if (-1 == fstat(fileno(config), &buf)) {
 		g_warning("could open but not fstat \"%s\" (fd #%d): %s",
 			path, fileno(config), g_strerror(errno));
-	else
+		file_id = "";
+	} else {
 		ps->mtime = buf.st_mtime;
+		file_id = unique_file_token(&buf);
+	}
 
 	G_FREE_NULL(path);
 
@@ -1924,9 +1986,9 @@ prop_load_from_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 	 *
 	 */
 	while (fgets(prop_tmp, sizeof prop_tmp, config)) {
-		property_t i;
-		gchar *s, *k, *v;
-		gint c;
+		char *s, *k, *v;
+		int c;
+		property_t prop;
 
 		s = strchr(prop_tmp, '\n');
 		if (!s) {
@@ -1947,7 +2009,9 @@ prop_load_from_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		/* Skip leading blanks */
 		s = skip_ascii_blanks(s);
 		c = (guchar) *s;
-		if (!is_ascii_alpha(c))	/* <keyword> must start with a letter */
+
+		/* <keyword> starts with _ or letter  */
+		if (!is_ascii_alpha(c) && c != '_')
 			continue;
 
 		/* Here starts the <keyword> */
@@ -2000,22 +2064,31 @@ prop_load_from_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		if (common_dbg > 5)
 			g_message("k=\"%s\", v=\"%s\"", k, v);
 
-		for (i = 0; i < ps->size; i++)
-			if (!ascii_strcasecmp(k, (ps->props[i]).name)) {
-				load_helper(ps, i + ps->offset, v);
-				break;
-			}
-
-		if (i >= ps->size)
-			g_warning("config file, line %u: unknown keyword '%s', ignored",
-				n, k);
+		prop = prop_get_by_name(ps, k);
+		if (NO_PROP != prop) {
+			load_helper(ps, prop, v);
+		} else if (0 == strcmp(k, PROP_FILE_ID)) {
+			if (0 == strcmp(file_id, v))
+				good_id = TRUE;
+		} else {
+			g_warning("\"%s%c%s\", line %u: unknown property '%s' -- ignored",
+				dir, G_DIR_SEPARATOR, filename, n, k);
+		}
 	}
 
 	fclose(config);
+
+	return good_id;
 }
 
+/**
+ * Maps a property name to a numeric property ID.
+ * @param ps A valid property context.
+ * @parma name A string to look up.
+ * @return The property ID or NO_PROP if the given name maps to none.
+ */
 property_t
-prop_get_by_name(prop_set_t *ps, const gchar *name)
+prop_get_by_name(prop_set_t *ps, const char *name)
 {
 	g_assert(ps != NULL);
 
@@ -2023,12 +2096,12 @@ prop_get_by_name(prop_set_t *ps, const gchar *name)
 }
 
 GSList *
-prop_get_by_regex(prop_set_t *ps, const gchar *pattern, gint *error)
+prop_get_by_regex(prop_set_t *ps, const char *pattern, int *error)
 {
 	GSList *sl = NULL;
 	size_t i;
 	regex_t re;
-	gint ret;
+	int ret;
 
 	g_assert(NULL != ps);
 	g_assert(NULL != pattern);
