@@ -1,5 +1,5 @@
 /*
- * $Id: slist.c 13882 2007-06-19 00:05:04Z cbiere $
+ * $Id: slist.c 16284 2009-03-28 21:11:40Z cbiere $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -40,7 +40,7 @@
 
 #include "common.h"
 
-RCSID("$Id: slist.c 13882 2007-06-19 00:05:04Z cbiere $")
+RCSID("$Id: slist.c 16284 2009-03-28 21:11:40Z cbiere $")
 
 #include "slist.h"
 #include "misc.h"
@@ -58,10 +58,10 @@ typedef enum {
 
 struct slist {
 	slist_magic_t magic;
-	gint refcount;
+	int refcount;
 	GSList *head;
 	GSList *tail;
-	gint length;
+	int length;
 	guint stamp;
 };
 
@@ -267,7 +267,11 @@ slist_remove_item(slist_t *slist, GSList *prev, GSList *item)
 	if (item == slist->tail) {
 		slist->tail = prev;
 	}
-	IGNORE_RESULT(g_slist_delete_link(prev ? prev : item, item));
+	/* @note: Return value is only assigned to prev because
+	 *        g_slist_delete_link is incorrectly tagged to
+	 *        cause a GCC compiler warning otherwise.
+	 */
+	item = g_slist_delete_link(prev ? prev : item, item);
 
 	slist->length--;
 	slist->stamp++;
@@ -575,4 +579,29 @@ slist_foreach(const slist_t *slist, GFunc func, gpointer user_data)
 	slist_regression(slist);
 }
 
+static inline void
+freecb_wrapper(gpointer data, slist_destroy_cb freecb)
+{
+	(*freecb)(data);
+}
+
+/**
+ * Dispose of all the items remaining in the list, applying the supplied free
+ * callback on all the items, then freeing the slist_t container.
+ */
+void
+slist_free_all(slist_t **slist_ptr, slist_destroy_cb freecb)
+{
+	g_assert(slist_ptr);
+	g_assert(freecb);
+
+	if (*slist_ptr) {
+		slist_t *slist = *slist_ptr;
+
+		slist_check(slist);
+		G_SLIST_FOREACH_WITH_DATA(slist->head, freecb_wrapper, freecb);
+		slist_free(slist_ptr);
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
