@@ -1,5 +1,5 @@
 /*
- * $Id: socket.c 13616 2007-05-12 22:07:17Z cbiere $
+ * $Id: socket.c 16108 2009-03-10 15:19:52Z rmanfredi $
  * 
  * Copyright (c) 2005, Jeroen Asselman
  *
@@ -25,7 +25,7 @@
 
 #include "common.h"
 
-RCSID("$Id: socket.c 13616 2007-05-12 22:07:17Z cbiere $")
+RCSID("$Id: socket.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #include "lib/socket.h"
 #include "lib/file.h"
@@ -36,7 +36,7 @@ RCSID("$Id: socket.c 13616 2007-05-12 22:07:17Z cbiere $")
  *  Sets a socket to non-blocking behaviour
  */
 void 
-socket_set_nonblocking(gint fd)
+socket_set_nonblocking(int fd)
 #ifdef MINGW32
 {
 	gulong nonblock = 1;
