@@ -1,5 +1,5 @@
 /*
- * $Id: stats.c 12789 2007-01-30 21:33:15Z cbiere $
+ * $Id: stats.c 16188 2009-03-17 17:42:15Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,13 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: stats.c 12789 2007-01-30 21:33:15Z cbiere $")
+RCSID("$Id: stats.c 16188 2009-03-17 17:42:15Z rmanfredi $")
 
 #ifdef I_MATH
 #include <math.h>
 #endif	/* I_MATH */
 
 #include "stats.h"
+#include "glib-missing.h"	/* For g_slist_delete_link() */
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -49,10 +50,11 @@ RCSID("$Id: stats.c 12789 2007-01-30 21:33:15Z cbiere $")
  * A one-dimension container (x).
  */
 struct statx {
-	GSList *data;			/**< Data points (value = gdouble *) */
-	gint n;					/**< Amount of points */
-	gdouble sx;				/**< Sx: sum of all points */
-	gdouble sx2;			/**< Sx2: sum of the square of all points */
+	GSList *data;			/**< Data points (value = double *) */
+	int n;					/**< Amount of points */
+	double sx;				/**< Sx: sum of all points */
+	double sx2;				/**< Sx2: sum of the square of all points */
+	gboolean no_data;		/**< Do not keep data, it is managed externally */
 };
 
 typedef enum op {
@@ -63,24 +65,35 @@ typedef enum op {
 /**
  * Create one-dimension container.
  */
-gpointer
+statx_t *
 statx_make(void)
 {
-	struct statx *sx;
+	statx_t *sx;
 
 	sx = walloc0(sizeof(*sx));
 	return sx;
 }
 
+/**
+ * Create one-dimension set of statistics with no data management.
+ */
+statx_t *
+statx_make_nodata(void)
+{
+	statx_t *sx;
+
+	sx = walloc0(sizeof(*sx));
+	sx->no_data = TRUE;
+	return sx;
+}
+
 /**
  * Destroy one-dimension container.
  */
 void
-statx_free(gpointer ox)
+statx_free(statx_t *sx)
 {
-	struct statx *sx = (struct statx *) ox;
-
-	statx_clear(ox);
+	statx_clear(sx);
 	wfree(sx, sizeof(*sx));
 }
 
@@ -88,13 +101,12 @@ statx_free(gpointer ox)
  * Clear container.
  */
 void
-statx_clear(gpointer ox)
+statx_clear(statx_t *sx)
 {
-	struct statx *sx = (struct statx *) ox;
 	GSList *l;
 
 	for (l = sx->data; l; l = g_slist_next(l)) {
-		gdouble *vp = (gdouble *) l->data;
+		double *vp = (double *) l->data;
 		wfree(vp, sizeof(*vp));
 	}
 	g_slist_free(sx->data);
@@ -113,35 +125,37 @@ statx_clear(gpointer ox)
  * @param op the operation: STATS_OP_ADD or STATS_OP_REMOVE
  */
 static void
-statx_opx(struct statx *sx, gdouble val, stats_op_t op)
+statx_opx(statx_t *sx, double val, stats_op_t op)
 {
 	g_assert(op == STATS_OP_ADD || sx->n > 0);
-	g_assert(op == STATS_OP_ADD || sx->data != NULL);
+	g_assert(op == STATS_OP_ADD || sx->data != NULL || sx->no_data);
 
-	if (op == STATS_OP_REMOVE) {
-		GSList *l;
+	if (!sx->no_data) {
+		if (op == STATS_OP_REMOVE) {
+			GSList *l;
 
-		/*
-		 * If value is removed, it must belong to the data set.
-		 */
+			/*
+			 * If value is removed, it must belong to the data set.
+			 */
 
-		for (l = sx->data; l; l = g_slist_next(l)) {
-			gdouble *vp = (gdouble *) l->data;
+			for (l = sx->data; l; l = g_slist_next(l)) {
+				double *vp = (double *) l->data;
 
-			if (*vp == val) {
-				sx->data = g_slist_remove(sx->data, vp);
-				wfree(vp, sizeof(*vp));
-				break;
+				if (*vp == val) {
+					sx->data = g_slist_remove(sx->data, vp);
+					wfree(vp, sizeof(*vp));
+					break;
+				}
 			}
+
+			g_assert(l != NULL);		/* Found it */
+		} else {
+			double *vp;
+
+			vp = walloc(sizeof(*vp));
+			*vp = val;
+			sx->data = g_slist_prepend(sx->data, vp);
 		}
-
-		g_assert(l != NULL);		/* Found it */
-	} else {
-		gdouble *vp;
-
-		vp = walloc(sizeof(*vp));
-		*vp = val;
-		sx->data = g_slist_prepend(sx->data, vp);
 	}
 
 	sx->n += op;
@@ -153,38 +167,83 @@ statx_opx(struct statx *sx, gdouble val, stats_op_t op)
  * Add data point to container.
  */
 void
-statx_add(gpointer ox, gdouble val)
+statx_add(statx_t *sx, double val)
 {
-	statx_opx(ox, val, STATS_OP_ADD);
+	statx_opx(sx, val, STATS_OP_ADD);
 }
 
 /**
  * Remove data point from container.
  */
 void
-statx_remove(gpointer ox, gdouble val)
+statx_remove(statx_t *sx, double val)
 {
-	statx_opx(ox, val, STATS_OP_REMOVE);
+	statx_opx(sx, val, STATS_OP_REMOVE);
+}
+
+/**
+ * Remove oldest data point from container.
+ */
+void
+statx_remove_oldest(statx_t *sx)
+{
+	GSList *l;
+	double val = 0;
+
+	g_assert(!sx->no_data);
+	g_assert(sx->n >= 0);
+	g_assert((sx->n > 0) ^ (NULL == sx->data));
+
+	if (sx->n < 1)
+		return;
+
+	/*
+	 * Since we prepend new items to the list (for speed), we need to find
+	 * the next to last item to delete the final item.
+	 */
+
+	for (l = sx->data; l; l = g_slist_next(l)) {
+		GSList *next = g_slist_next(l);
+		if (next == NULL) {
+			/* Only one item in list, `l' points to it */
+			double *vp = (double *) l->data;
+			val = *vp;
+			wfree(vp, sizeof(*vp));
+			g_slist_free(sx->data);
+			sx->data = NULL;
+			break;
+		} else if (NULL == g_slist_next(next)) {
+			/* The item after `l' is the last item of the list */
+			double *vp = (double *) next->data;
+			val = *vp;
+			wfree(vp, sizeof(*vp));
+			next = g_slist_delete_link(l, next);
+			break;
+		}
+	}
+
+	sx->n--;
+	sx->sx -= val;
+	sx->sx2 -= val * val;
+
+	g_assert((sx->n > 0) ^ (NULL == sx->data));
 }
 
 /**
  * @return amount of data points.
  */
-gint
-statx_n(gpointer ox)
+int
+statx_n(const statx_t *sx)
 {
-	struct statx *sx = ox;
 	return sx->n;
 }
 
 /**
  * @return average of data points.
  */
-gdouble
-statx_avg(gpointer ox)
+double
+statx_avg(const statx_t *sx)
 {
-	struct statx *sx = ox;
-
 	g_assert(sx->n > 0);
 
 	return sx->sx / sx->n;
@@ -193,21 +252,19 @@ statx_avg(gpointer ox)
 /**
  * @return the standard deviation of the data points.
  */
-gdouble
-statx_sdev(gpointer ox)
+double
+statx_sdev(const statx_t *sx)
 {
-	return sqrt(statx_var(ox));
+	return sqrt(statx_var(sx));
 }
 
 /**
  * @return the variance of the data points.
  */
-gdouble
-statx_var(gpointer ox)
+double
+statx_var(const statx_t *sx)
 {
-	struct statx *sx = ox;
-
-	g_assert(sx->n > 0);
+	g_assert(sx->n > 1);
 
 	return (sx->sx2 - (sx->sx * sx->sx) / sx->n) / (sx->n - 1);
 }
@@ -215,20 +272,20 @@ statx_var(gpointer ox)
 /**
  * @return an array of datapoints which can be freed when done.
  */
-gdouble *
-statx_data(gpointer ox)
+double *
+statx_data(const statx_t *sx)
 {
-	struct statx *sx = ox;
-	gdouble *array;
-	gint i;
+	double *array;
+	int i;
 	GSList *l;
 
+	g_assert(!sx->no_data);
 	g_assert(sx->n > 0);
 
-	array = g_malloc(sizeof(gdouble) * sx->n);
+	array = g_malloc(sizeof(double) * sx->n);
 
 	for (i = 0, l = sx->data; i < sx->n && l; l = g_slist_next(l), i++) {
-		gdouble *vp = (gdouble *) l->data;
+		double *vp = (double *) l->data;
 		array[i] = *vp;
 	}
 
