@@ -1,5 +1,5 @@
 /*
- * $Id: tigertree.c 13627 2007-05-13 04:17:17Z cbiere $
+ * $Id: tigertree.c 15229 2008-04-20 18:03:03Z rmanfredi $
  *
  * Copyright (c) 2003-2004, Jeroen Asselman
  * Copyright (c) 2005, Martijn van Oosterhout <kleptog@svana.org>
@@ -80,7 +80,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tigertree.c 13627 2007-05-13 04:17:17Z cbiere $")
+RCSID("$Id: tigertree.c 15229 2008-04-20 18:03:03Z rmanfredi $")
 
 #include "base32.h"
 #include "endian.h"
@@ -88,7 +88,8 @@ RCSID("$Id: tigertree.c 13627 2007-05-13 04:17:17Z cbiere $")
 #include "tigertree.h"
 #include "override.h"		/* Must be the last header included */
 
-/* size of input to each non-leaf hash-tree node, not counting node 0x01 prefix */
+/* size of input to each non-leaf hash-tree node, not counting the node
+ * 0x01 prefix */
 #define TTH_NODESIZE	(TIGERSIZE * 2)
 
 /* default size of interim values stack, in TIGERSIZE
@@ -124,6 +125,23 @@ tt_block_count(filesize_t filesize)
 	return (filesize / TTH_BLOCKSIZE) + ((filesize % TTH_BLOCKSIZE) ? 1 : 0);
 }
 
+/**
+ * Compute the depth of a tree given the number of leaf blocks.
+ */
+unsigned
+tt_depth(size_t leaves)
+{
+	unsigned depth = 0;
+	size_t n = leaves;
+
+	while (n > 1) {
+		n = (n + 1) / 2;
+		depth++;
+	}
+
+	return depth;
+}
+
 unsigned
 tt_full_depth(filesize_t filesize)
 {
