@@ -1,5 +1,5 @@
 /*
- * $Id: urn.h 8551 2005-07-07 12:59:23Z daichik $
+ * $Id: urn.h 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -36,15 +36,19 @@
 #ifndef _urn_h_
 #define _urn_h_
 
-#include <glib.h>
+#include "common.h" 
 
 /*
  * Public interface.
  */
 
-gboolean urn_get_http_sha1(const gchar *buf, gchar *retval);
-gboolean urn_get_sha1(const gchar *buf, gchar *digest);
-gboolean urn_get_sha1_no_prefix(const gchar *buf, gchar *digest);
+gboolean parse_base32_sha1(const char *buf, size_t size, struct sha1 *sha1);
+
+gboolean urn_get_sha1(const char *buf, struct sha1 *sha1);
+gboolean urn_get_sha1_no_prefix(const char *buf, struct sha1 *sha1);
+gboolean urn_get_bitprint(const char *buf, size_t size,
+	struct sha1 *sha1, struct tth *tth);
+gboolean urn_get_tth(const char *buf, size_t size, struct tth *tth);
 
 #endif	/* _urn_h_ */
 
