@@ -1,5 +1,5 @@
 /*
- * $Id: wordvec.h 12500 2006-11-26 10:53:21Z cbiere $
+ * $Id: wordvec.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -43,15 +43,15 @@
  */
 
 typedef struct {				/**< Query word vector */
-	gchar *word;				/**< The word to search */
-	gint len;					/**< The word's length */
+	char *word;				/**< The word to search */
+	int len;					/**< The word's length */
 	guint amount;				/**< Amount of expected occurrences */
 } word_vec_t;
 
 void word_vec_init(void);
 void word_vec_close(void);
 
-guint word_vec_make(const gchar *query, word_vec_t **wovec);
+guint word_vec_make(const char *query, word_vec_t **wovec);
 void word_vec_free(word_vec_t *wovec, guint n);
 
 #endif	/* _wordvec_h_ */
