@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 14979 2007-10-16 19:17:13Z rmanfredi $
+ * $Id: main.c 16232 2009-03-22 08:12:19Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -47,11 +47,12 @@
 #include "core/dmesh.h"
 #include "core/downloads.h"
 #include "core/dq.h"
+#include "core/dump.h"
 #include "core/extensions.h"
 #include "core/features.h"
 #include "core/fileinfo.h"
 #include "core/file_object.h"
-#include "core/g2_cache.h"
+#include "core/gdht.h"
 #include "core/geo_ip.h"
 #include "core/gmsg.h"
 #include "core/gnet_stats.h"
@@ -64,6 +65,7 @@
 #include "core/http.h"
 #include "core/ignore.h"
 #include "core/inet.h"
+#include "core/ipp_cache.h"
 #include "core/local_shell.h"
 #include "core/move.h"
 #include "core/nodes.h"
@@ -80,7 +82,6 @@
 #include "core/sockets.h"
 #include "core/spam.h"
 #include "core/sq.h"
-#include "core/tls_cache.h"
 #include "core/tls_common.h"
 #include "core/topless.h"
 #include "core/tsync.h"
@@ -93,22 +94,27 @@
 #include "core/version.h"
 #include "core/vmsg.h"
 #include "core/whitelist.h"
-#include "dht/kuid.h"
-#include "dht/routing.h"
-#include "dht/rpc.h"
+#include "if/dht/kmsg.h"
+#include "if/dht/dht.h"
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/bg.h"
 #include "lib/cq.h"
 #include "lib/crash.h"
 #include "lib/crc.h"
+#include "lib/debug.h"
 #include "lib/dbus_util.h"
 #include "lib/eval.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
 #include "lib/iso3166.h"
+#include "lib/map.h"
+#include "lib/mime_type.h"
+#include "lib/patricia.h"
 #include "lib/pattern.h"
 #include "lib/socket.h"
+#include "lib/strtok.h"
+#include "lib/tea.h"
 #include "lib/tiger.h"
 #include "lib/tigertree.h"
 #include "lib/tm.h"
@@ -134,7 +140,7 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: main.c 14979 2007-10-16 19:17:13Z rmanfredi $")
+RCSID("$Id: main.c 16232 2009-03-22 08:12:19Z rmanfredi $")
 
 #define SLOW_UPDATE_PERIOD		20	/**< Update period for `main_slow_update' */
 #define EXIT_GRACE				30	/**< Seconds to wait before exiting */
@@ -317,7 +323,6 @@ log_cpu_usage(tm_t *since_time, gdouble *prev_user, gdouble *prev_sys)
 	gdouble sys;
 	gdouble total;
 	tm_t cur_time;
-	tm_t elapsed_time;
 	gdouble elapsed;
 
 	tm_now_exact(&cur_time);
@@ -335,8 +340,7 @@ log_cpu_usage(tm_t *since_time, gdouble *prev_user, gdouble *prev_sys)
 		total -= v;
 	}
 
-	tm_elapsed(&elapsed_time, &cur_time, since_time);
-	elapsed = tm2f(&elapsed_time);
+	elapsed = tm_elapsed_f(&cur_time, since_time);
 	*since_time = cur_time;
 
 	g_message("average CPU used: %.3f%% over %.2f secs",
@@ -379,6 +383,7 @@ gtk_gnutella_exit(gint n)
 #define DO(fn) 	do { exit_step = #fn; fn(); } while (0)
 
 	DO(shell_close);
+	DO(file_info_store_if_dirty);
 	DO(file_info_close_pre);
 	DO(node_bye_all);
 	DO(upload_close);	/* Done before upload_stats_close() for stats update */
@@ -391,6 +396,8 @@ gtk_gnutella_exit(gint n)
 	DO(http_close);
 	DO(uhc_close);
 	DO(move_close);
+	DO(dht_close);
+	DO(ipp_cache_save_all);
 
 	/*
 	 * When coming from atexit(), there is a sense of urgency.
@@ -399,26 +406,30 @@ gtk_gnutella_exit(gint n)
 	 */
 
 	DO(settings_save_if_dirty);
-	if (!running_topless) {
-		DO(settings_gui_save_if_dirty);
-	}
 
 	safe_to_exit = TRUE;	/* Will immediately exit if re-entered */
 
+	if (debugging(0) || signal_received || shutdown_requested)
+		g_message("context files and settings closed properly");
+
 	if (from_atexit)
 		return;
 
 #undef DO
 
 	if (!running_topless) {
+		settings_gui_save_if_dirty();
 		main_gui_shutdown();
+
+		if (debugging(0) || signal_received || shutdown_requested)
+			g_message("GUI shutdown completed");
 	}
 
-    hcache_shutdown();		/* Save host caches to disk */
+	cq_halt();				/* No more callbacks, with everything shutdown */
+	hcache_shutdown();		/* Save host caches to disk */
 	settings_shutdown();
 	oob_shutdown();			/* No longer deliver outstanding OOB hits */
 	socket_shutdown();
-	search_shutdown();
 	bsched_shutdown();
 
 	if (!running_topless) {
@@ -444,6 +455,11 @@ gtk_gnutella_exit(gint n)
 	if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA)
 		exit_grace *= 2;
 
+	if (debugging(0) || signal_received || shutdown_requested) {
+		g_message("waiting at most %s for BYE messages",
+			short_time(exit_grace));
+	}
+
 	while (node_bye_pending()) {
 		time_t now = time(NULL);
 		time_delta_t d;
@@ -457,13 +473,17 @@ gtk_gnutella_exit(gint n)
 		if (!running_topless) {
 			main_gui_shutdown_tick(exit_grace - d);
 		}
-		sleep(1);
+		compat_sleep_ms(50);
 	}
 
+	if (debugging(0) || signal_received || shutdown_requested)
+		g_message("running final shutdown sequence...");
+
+	search_shutdown();		/* Disable now, since we can get queries above */
+
 	bitzi_close();
-	dht_rpc_close();
-	dht_route_close();
 	ntp_close();
+	gdht_close();
 	sq_close();
 	dh_close();
 	dq_close();
@@ -504,8 +524,8 @@ gtk_gnutella_exit(gint n)
 	atom_str_free_null(&start_rfc822_date);
 	adns_close();
 	dbus_util_close();  /* After adns_close() to avoid strange crashes */
-	tls_cache_close();
-	g2_cache_close();
+	ipp_cache_close();
+	dump_close();
 	file_object_close();
 	settings_close();	/* Must come after hcache_close() */
 	atoms_close();
@@ -559,17 +579,22 @@ slow_main_timer(time_t now)
 		break;
 	case 2:
 		upload_stats_flush_if_dirty();
+		dht_update_size_estimate();
 		break;
 	case 3:
 		file_info_store_if_dirty();
+		file_info_slow_timer();
 		break;
 	case 4:
 		hcache_store_if_dirty(HOST_ULTRA);
 		break;
+	case 5:
+		dht_route_store_if_dirty();
+		break;
 	default:
 		g_assert_not_reached();
 	}
-	i = (i + 1) % 5;
+	i = (i + 1) % 6;
 
 	download_store_if_dirty();		/* Important, so always attempt it */
 	settings_save_if_dirty();		/* Nice to have, and file is small */
@@ -580,6 +605,7 @@ slow_main_timer(time_t now)
 	rx_collect();					/* Idem for freed RX stacks */
 	prune_page_cache();
 
+	download_slow_timer(now);
 	node_slow_timer(now);
 	ignore_timer(now);
 }
@@ -598,7 +624,6 @@ check_cpu_usage(void)
 	static gint load_avg = 0;		/* 100 * cpu% for integer arithmetic */
 	static gint avg = 0;			/* cpu% */
 	tm_t cur_tm;
-	tm_t elapsed_tm;
 	gint load = 0;
 	gdouble cpu;
 	gdouble elapsed;
@@ -611,9 +636,8 @@ check_cpu_usage(void)
 
 	tm_now_exact(&cur_tm);
 	cpu = tm_cputime(NULL, NULL);
-	tm_elapsed(&elapsed_tm, &cur_tm, &last_tm);
 
-	elapsed = tm2f(&elapsed_tm);
+	elapsed = tm_elapsed_f(&cur_tm, &last_tm);
 	elapsed = MAX(elapsed, 0.000001);	/* Prevent division by zero */
 	cpu_percent = 100.0 * (cpu - last_cpu) / elapsed;
 	cpu_percent = MIN(cpu_percent, 100.0);
@@ -621,6 +645,11 @@ check_cpu_usage(void)
 	coverage = callout_queue_coverage(ticks);
 	coverage = MAX(coverage, 0.001);	/* Prevent division by zero */
 
+	if (GNET_PROPERTY(cq_debug)) {
+		g_message("CQ: callout queue holds %d items and processed %d ticks",
+			cq_count(callout_queue), cq_ticks(callout_queue));
+	}
+
 	/*
 	 * Correct the percentage of CPU that would have been actually used
 	 * if we had had 100% of the CPU scheduling time.  We use the callout
@@ -712,34 +741,33 @@ main_timer(gpointer p)
     hcache_timer(now);
 	node_timer(now);				/* Node timeouts */
 	http_timer(now);				/* HTTP request timeouts */
-	if (!exiting) {
-		shell_timer(now);
-		download_timer(now);  	    /* Download timeouts */
-		parq_upload_timer(now);		/* PARQ upload timeouts/removal */
-		upload_timer(now);			/* Upload timeouts */
-		file_info_timer();          /* Notify about changes */
-		hsep_timer(now);			/* HSEP notify message timer */
-		pproxy_timer(now);			/* Push-proxy requests */
-		dh_timer(now);				/* Monitoring of query hits */
-	}
 	socket_timer(now);				/* Expire inactive sockets */
 	pcache_possibly_expired(now);	/* Expire pong cache */
 
+	if (exiting)
+		return TRUE;
+
+	shell_timer(now);
+	download_timer(now);  	    /* Download timeouts */
+	parq_upload_timer(now);		/* PARQ upload timeouts/removal */
+	upload_timer(now);			/* Upload timeouts */
+	file_info_timer();          /* Notify about changes */
+	hsep_timer(now);			/* HSEP notify message timer */
+	pproxy_timer(now);			/* Push-proxy requests */
+	dh_timer(now);				/* Monitoring of query hits */
+
 	/*
 	 * GUI update
 	 */
 
-	if (!exiting) {
+	if (!running_topless) {
+		main_gui_timer(now);
+	}
 
-		if (!running_topless) {
-			main_gui_timer(now);
-		}
-
-		/* Update for things that change slowly */
-		if (main_slow_update++ > SLOW_UPDATE_PERIOD) {
-			main_slow_update = 0;
-			slow_main_timer(now);
-		}
+	/* Update for things that change slowly */
+	if (main_slow_update++ > SLOW_UPDATE_PERIOD) {
+		main_slow_update = 0;
+		slow_main_timer(now);
 	}
 
 	/*
@@ -758,9 +786,7 @@ static gboolean
 scan_files_once(gpointer p)
 {
 	(void) p;
-	guc_allow_rescan_dir(FALSE);
 	share_scan();
-	guc_allow_rescan_dir(TRUE);
 
 	return FALSE;
 }
@@ -906,7 +932,7 @@ static struct {
 	OPTION(ping,			NONE, "Check whether gtk-gnutella is running."),
 	OPTION(shell,			NONE, "Access the local shell interface."),
 #ifdef USE_TOPLESS
-	OPTION(topless,			NONE, NULL),	/* accept by hide */
+	OPTION(topless,			NONE, NULL),	/* accept but hide */
 #else
 	OPTION(topless,			NONE, "Disable the graphical user-interface."),
 #endif	/* USE_TOPLESS */
@@ -1181,7 +1207,9 @@ handle_arguments(int argc, char **argv)
 #endif	/* USE_TOPLESS */
 	if (options[main_arg_version].used) {
 		printf("%s\n", version_build_string());
-		
+#ifndef OFFICIAL_BUILD
+		printf("(unofficial build, accessing \"%s\")\n", PACKAGE_SOURCE_DIR);
+#endif
 		printf("GLib %u.%u.%u",
 			glib_major_version, glib_minor_version, glib_micro_version);
 		if (
@@ -1241,7 +1269,9 @@ int
 main(int argc, char **argv)
 {
 	if (compat_is_superuser()) {
-		fprintf(stderr, "Never ever run this as root!\n");
+		fprintf(stderr, "Never ever run this as root! You may use:\n\n");
+		fprintf(stderr, "    su - username -c 'gtk-gnutella --daemonize'\n\n");
+		fprintf(stderr, "where 'username' stands for a regular user name.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -1312,6 +1342,9 @@ main(int argc, char **argv)
 	tiger_check();
 	tt_check();
 	random_init();
+	tea_test();
+	patricia_test();
+	strtok_test();
 	locale_init();
 	adns_init();
 	file_object_init();
@@ -1321,6 +1354,7 @@ main(int argc, char **argv)
 	iso3166_init();
 	dbus_util_init();
 	vendor_init();
+	mime_type_init();
 
 	if (!running_topless) {
 		main_gui_early_init(argc, argv, options[main_arg_no_xshm].used);
@@ -1332,10 +1366,12 @@ main(int argc, char **argv)
 	watcher_init();
 	hcache_init();			/* before settings_init() */
 	bsched_early_init();	/* before settings_init() */
+	ipp_cache_init();		/* before settings_init() */
 	settings_init();
+	set_library_debug(GNET_PROPERTY(lib_debug));
+	map_test();				/* after settings_init() */
+	ipp_cache_load_all();	/* after settings_init() */
 	tls_global_init();
-	tls_cache_init();
-	g2_cache_init();
 	hostiles_init();
 	spam_init();
 	bogons_init();
@@ -1346,13 +1382,15 @@ main(int argc, char **argv)
 	verify_tth_init();
 	move_init();
 	ignore_init();
-	file_info_init();
 	pattern_init();
 	word_vec_init();
+
+	file_info_init();
 	host_init();
 	pmsg_init();
 	gmsg_init();
 	bsched_init();
+	dump_init();
 	node_init();
     hcache_retrieve_all();	/* after settings_init() and node_init() */
 	routing_init();
@@ -1374,11 +1412,11 @@ main(int argc, char **argv)
 	dh_init();
 	bitzi_init();
 	sq_init();
+	gdht_init();
 	file_info_init_post();
 
-	kuid_init();			/* DHT */
-	dht_route_init();
-	dht_rpc_init();
+	kmsg_init();			/* DHT */
+	dht_init();
 
 	if (!running_topless) {
 		main_gui_init();
@@ -1411,6 +1449,8 @@ main(int argc, char **argv)
 	(void) tm_time_exact();
 	bsched_enable_all();
 	version_ancient_warn();
+	if (dht_enabled())
+		dht_attempt_bootstrap();
 
 	if (running_topless) {
 		topless_main_run();
