@@ -1,5 +1,5 @@
 /*
- * $Id: shell.c 14438 2007-08-13 10:31:55Z rmanfredi $
+ * $Id: shell.c 16242 2009-03-24 19:04:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "common.h"
 
-RCSID("$Id: shell.c 14438 2007-08-13 10:31:55Z rmanfredi $")
+RCSID("$Id: shell.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #include "shell.h"
 #include "cmd.h"
@@ -34,14 +34,14 @@ RCSID("$Id: shell.c 14438 2007-08-13 10:31:55Z rmanfredi $")
 #include "core/sockets.h"
 #include "core/version.h"
 
-#include "if/bridge/ui2c.h"
 #include "if/gnet_property_priv.h"
 
+#include "lib/ascii.h"
 #include "lib/file.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/inputevt.h"
-#include "lib/misc.h"
+#include "lib/pmsg.h"
 #include "lib/sha1.h"
 #include "lib/slist.h"
 #include "lib/tm.h"
@@ -52,14 +52,6 @@ RCSID("$Id: shell.c 14438 2007-08-13 10:31:55Z rmanfredi $")
 
 static GSList *sl_shells;
 
-/*
- * guc_share_scan() causes dispatching of I/O events, so we must not call
- * it whilst in an event callback (all commands are) because if the shell
- * connection dies, the shell context will no longer be valid. Therefore,
- * we just record the request and call guc_share_scan() from shell_timer().
- */
-static gboolean library_rescan_requested;
-
 enum shell_magic {
 	SHELL_MAGIC = 0xb3f3e711U
 };
@@ -68,7 +60,7 @@ struct gnutella_shell {
 	enum shell_magic	magic;
 	struct gnutella_socket *socket;
 	slist_t *output;
-	gchar *msg;   			/**< Additional information to reply code */
+	char *msg;   			/**< Additional information to reply code */
 	time_t last_update; 	/**< Last update (needed for timeout) */
 	guint64 line_count;		/**< Number of input lines after HELO */
 	gboolean shutdown;  	/**< In shutdown mode? */
@@ -154,6 +146,15 @@ shell_destroy(struct gnutella_shell *sh)
 	shell_free(sh);
 }
 
+void
+shell_set_msg(struct gnutella_shell *sh, const char *text)
+{
+	shell_check(sh);
+
+	G_FREE_NULL(sh->msg);
+	sh->msg = g_strdup(text);
+}
+
 static void
 shell_write_msg(struct gnutella_shell *sh)
 {
@@ -162,17 +163,10 @@ shell_write_msg(struct gnutella_shell *sh)
 	if (sh->msg) {
 		shell_write(sh, " ");
 		shell_write(sh, sh->msg);
-		G_FREE_NULL(sh->msg);
+		shell_set_msg(sh, NULL);
 	}
 }
 
-void
-shell_set_msg(struct gnutella_shell *sh, const char *text)
-{
-	shell_check(sh);
-	sh->msg = g_strdup(text);
-}
-
 static void
 shell_write_welcome(struct gnutella_shell *sh)
 {
@@ -201,14 +195,6 @@ shell_toggle_interactive(struct gnutella_shell *sh)
 	return sh->interactive;
 }
 
-gboolean
-shell_request_library_rescan(void)
-{
-	gboolean previous = library_rescan_requested;
-	library_rescan_requested = TRUE;
-	return previous;
-}
-
 /**
  * @returns a pointer to the end of the first token within s. If
  * s only consists of a single token, it returns a pointer to the
@@ -285,7 +271,7 @@ shell_unescape(char *s)
 /**
  * @return the next token from s starting from position pos. Make sure
  * that pos is 0 or something sensible when calling this the first time!.
- * The returned string needs to be g_free-ed when no longer needed.
+ * The returned string is newly allocated.
  */
 static char *
 shell_get_token(const char *s, int *pos)
@@ -298,7 +284,7 @@ shell_get_token(const char *s, int *pos)
 	g_assert(-1 == *pos || *pos >= 0);
 
 	if (*pos >= 0) {
-		start = &s[*pos];
+		start = skip_ascii_spaces(&s[*pos]);
 		if (*start == '\0') {
 			*pos = -1;
 		}
@@ -312,11 +298,14 @@ shell_get_token(const char *s, int *pos)
 	end = shell_token_end(start);
 
 	/* update position before removing quotes */
-	*pos = *end == '\0' ? -1 : end - s + 1;
+	switch (*end) {
+	case '\0':	*pos = -1; break;
+	default:	*pos = (end - s) + 1; break;
+	}
 
 	/* don't return enclosing quotes */
 	if (*start == '"' && *end == '"')
-		start ++;
+		start++;
 
 	retval = g_strndup(start, end - start);
 	shell_unescape(retval);
@@ -423,11 +412,18 @@ shell_exec(struct gnutella_shell *sh, const char *line)
 		handler = shell_cmd_get_handler(argv[0]);
 		if (handler) {
 			reply_code = (*handler)(sh, argc, argv);
-			if (REPLY_ERROR == reply_code && !sh->msg) {
-				shell_set_msg(sh, _("Malformed command"));
-			}
-			if (REPLY_READY == reply_code && !sh->msg) {
-				shell_set_msg(sh, _("OK"));
+			if (NULL == sh->msg) {
+				switch (reply_code) {
+				case REPLY_ERROR:
+					shell_set_msg(sh, _("Malformed command"));
+					break;
+				case REPLY_READY:
+					shell_set_msg(sh, _("OK"));
+					break;
+				case REPLY_NONE:
+				case REPLY_BYE:
+					break;
+				}
 			}
 		} else {
 			shell_set_msg(sh, _("Unknown command"));
@@ -692,11 +688,8 @@ shell_auth_cookie(void)
 	if (!initialized) {
 		SHA1Context ctx;
 		guint32 noise[64];
-		size_t i;
 
-		for (i = 0; i < G_N_ELEMENTS(noise); i++) {
-			noise[i] = random_raw();
-		}
+		random_bytes(noise, sizeof noise);
 		SHA1Reset(&ctx);
 		SHA1Input(&ctx, &noise, sizeof noise);
 		SHA1Result(&ctx, &cookie);
@@ -713,12 +706,12 @@ shell_auth_cookie(void)
  * @return TRUE if the connection is allowed.
  */
 static gboolean
-shell_auth(const gchar *str)
+shell_auth(const char *str)
 {
 	const struct sha1 *cookie;
-	gchar *tok_helo, *tok_cookie;
+	char *tok_helo, *tok_cookie;
 	gboolean ok = FALSE;
-	gint pos = 0;
+	int pos = 0;
 
 	tok_helo = shell_get_token(str, &pos);
 	tok_cookie = shell_get_token(str, &pos);
@@ -862,11 +855,6 @@ shell_timer(time_t now)
 		}
 		g_slist_free(to_remove);
 	}
-
-	if (library_rescan_requested) {
-		library_rescan_requested = FALSE;
-		guc_share_scan();	/* This can take several seconds */
-	}
 }
 
 void
