@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 14457 2007-08-14 04:23:12Z cbiere $
+ * $Id: bitzi.c 15779 2008-09-02 04:18:52Z cbiere $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -35,7 +35,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: bitzi.c 14457 2007-08-14 04:23:12Z cbiere $")
+RCSID("$Id: bitzi.c 15779 2008-09-02 04:18:52Z cbiere $")
 
 #include "gtk/search.h"
 #include "gtk/misc.h"			/* gui_record_sha1_eq() */
@@ -52,8 +52,9 @@ RCSID("$Id: bitzi.c 14457 2007-08-14 04:23:12Z cbiere $")
  * It assumes the enum's are in order.
  */
 
-static const gchar * const bitzi_fj_table[] = {
+static const char * const bitzi_fj_table[] = {
 	N_("Bitzi|Unknown"),				/**< UNKNOWN */
+	N_("Bitzi|Unrated"),				/**< UNRATED */
 	N_("Bitzi|Bitzi lookup failure"),	/**< FAILURE */
 	N_("Bitzi|Filesize mismatch"),		/**< WRONG_FILESIZE */
 	N_("Bitzi|Dangerous/Misleading"),	/**< DANGEROUS_MISLEADING */
@@ -67,7 +68,7 @@ static const gchar * const bitzi_fj_table[] = {
 	N_("Bitzi|Best Version"),			/**< BEST_VERSION*/
 };
 
-const gchar *
+const char *
 bitzi_fj_to_string(bitzi_fj_t fj)
 {
 	STATIC_ASSERT(NUM_BITZI_FJ == G_N_ELEMENTS(bitzi_fj_table));
@@ -98,7 +99,7 @@ bitzi_gui_update(const bitzi_data_t *bitzi_data)
 	search_gui_metadata_update(bitzi_data);
 }
 
-gchar *
+char *
 bitzi_gui_get_metadata(const bitzi_data_t *data)
 {
 	g_assert(data != NULL);
