@@ -1,5 +1,5 @@
 /*
- * $Id: downloads_common.c 14769 2007-09-06 03:23:37Z cbiere $
+ * $Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: downloads_common.c 14769 2007-09-06 03:23:37Z cbiere $")
+RCSID("$Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $")
 
 #include "gtk/columns.h"
 #include "gtk/downloads.h"
@@ -69,6 +69,7 @@ struct fileinfo_data {
 	filesize_t uploaded;
 
 	time_t	   created;
+	time_t	   modified;
 
 	gnet_fi_t handle;
 
@@ -101,6 +102,8 @@ static gboolean update_download_clear_needed;
 static gboolean download_gui_visible;
 
 static regex_t *filter_regex;
+static gboolean filter_regex_invert;
+static gboolean filter_regex_case_sensitive;
 
 /**
  * Remember that we need to check for cleared downloads at the next
@@ -182,21 +185,25 @@ fi_gui_set_details(const struct fileinfo_data *file)
 {
     gnet_fi_info_t *info;
 
+	g_return_if_fail(file);
 	fi_gui_clear_details();
 
     info = guc_fi_get_info(file->handle);
 	g_return_if_fail(info);
 
-	fi_gui_append_detail(_("Filename"), file->filename);
-	fi_gui_append_detail(_("Size"), nice_size(file->size, show_metric_units()));
-	fi_gui_append_detail(_("Created"), timestamp_to_string(file->created));
+	fi_gui_append_detail(FI_GUI_DETAIL_FILENAME, _("Filename"),
+		file->filename);
+	fi_gui_append_detail(FI_GUI_DETAIL_SIZE, _("Size"),
+		nice_size(file->size, show_metric_units()));
+	fi_gui_append_detail(FI_GUI_DETAIL_CREATED, _("Created"),
+		timestamp_to_string(file->created));
+	fi_gui_append_detail(FI_GUI_DETAIL_MODIFIED, _("Modified"),
+		file->modified ? timestamp_to_string(file->modified) : "-");
 
-	fi_gui_append_detail(_("SHA-1"),
-		info->sha1
-			? sha1_to_urn_string(info->sha1)
-			: _("Not available"));
+	fi_gui_append_detail(FI_GUI_DETAIL_SHA1, _("SHA-1"),
+		info->sha1 ? sha1_to_urn_string(info->sha1) : _("Not available"));
 
-	fi_gui_append_detail(_("Bitprint"),
+	fi_gui_append_detail(FI_GUI_DETAIL_BITPRINT, _("Bitprint"),
 		info->sha1 && info->tth
 			? bitprint_to_urn_string(info->sha1, info->tth)
 			: _("Not available"));
@@ -205,18 +212,22 @@ fi_gui_set_details(const struct fileinfo_data *file)
 		char buf[1024];
 
 		gm_snprintf(buf, sizeof buf,
-			_("%lu leaf hashes providing a granularity of %s"),
+			_("leaf hashes: %lu, depth: %u, granularity: %s"),
 			(unsigned long) info->tth_num_leaves,
+			info->tth_depth,
 			nice_size(info->tth_slice_size, show_metric_units()));
-   		fi_gui_append_detail(_("Tigertree"), buf);
+   		fi_gui_append_detail(FI_GUI_DETAIL_TIGERTREE, _("Tigertree"), buf);
 	} else {
-   		fi_gui_append_detail(_("Tigertree"), _("Not available"));
+   		fi_gui_append_detail(FI_GUI_DETAIL_TIGERTREE, _("Tigertree"),
+			_("Not available"));
 	}
 
 	if (info->sha1) {
-		fi_gui_append_detail(_("External metadata"), NULL);	/* Separator */
-		fi_gui_append_detail(_("Bitzi URL"), url_for_bitzi_lookup(info->sha1));
-		fi_gui_append_detail(_("ShareMonkey URL"),
+		fi_gui_append_detail(FI_GUI_DETAIL_UNSPECIFIED, _("External metadata"),
+			NULL);	/* Separator */
+		fi_gui_append_detail(FI_GUI_DETAIL_BITZI, _("Bitzi URL"),
+			url_for_bitzi_lookup(info->sha1));
+		fi_gui_append_detail(FI_GUI_DETAIL_SHAREMONKEY, _("ShareMonkey URL"),
 			url_for_sharemonkey_lookup(info->sha1, info->filename, info->size));
 	}
 
@@ -345,6 +356,8 @@ downloads_gui_status_string(const struct download *d)
 				}
 
 				status = tmpstr;
+			} else if (d->server->attrs & DLS_A_DHT_PROX) {
+				status = _("Searching for new push-proxies");
 			} else {
 				switch (d->status) {
 				case GTA_DL_PUSH_SENT:
@@ -383,19 +396,23 @@ downloads_gui_status_string(const struct download *d)
 		break;
 
 	case GTA_DL_COMPLETED:
-		if (d->last_update != d->start_date) {
-			time_delta_t t = delta_time(d->last_update, d->start_date);
-			
-			rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s) %s",
-				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
-				short_rate((d->range_end - d->skip + d->overlap_size) / t,
-					show_metric_units()),
-				short_time(t));
+		if (NULL != d->remove_msg) {
+			status = d->remove_msg;
 		} else {
+			if (d->last_update != d->start_date) {
+				time_delta_t t = delta_time(d->last_update, d->start_date);
+				
+				rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s) %s",
+					FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
+					short_rate((d->range_end - d->skip + d->overlap_size) / t,
+						show_metric_units()),
+					short_time(t));
+			} else {
 			rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s (< 1s)",
 				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"));
+			}
+			status = tmpstr;
 		}
-		status = tmpstr;
 		break;
 
 	case GTA_DL_VERIFY_WAIT:
@@ -1130,7 +1147,7 @@ fi_gui_file_visible(const struct fileinfo_data *file)
 		return 0 == file->recv_count
 			&& (file->actively_queued || file->passively_queued);
 	case nb_downloads_page_finished:
-		return file->finished && !file->seeding;
+		return file->finished || file->complete;
 	case nb_downloads_page_seeding:
 		return file->seeding;
 	case nb_downloads_page_paused:
@@ -1153,8 +1170,9 @@ fi_gui_file_update_matched(struct fileinfo_data *file)
 {
 	g_return_if_fail(file);
 
-	file->matched = !filter_regex
-		|| 0 == regexec(filter_regex, file->filename, 0, NULL, 0);
+	file->matched = !filter_regex ||
+		((0 != filter_regex_invert) ^
+		 (0 == regexec(filter_regex, file->filename, 0, NULL, 0)));
 }
 
 /**
@@ -1171,8 +1189,9 @@ fi_gui_file_set_filename(struct fileinfo_data *file)
     info = guc_fi_get_info(file->handle);
     g_return_if_fail(info);
 
-	file->filename = atom_str_get(lazy_filename_to_ui_string(info->filename));
-	file->created = info->ctime;
+	atom_str_change(&file->filename,
+		lazy_filename_to_ui_string(info->filename));
+	file->created = info->created;
 	guc_fi_free_info(info);
 	fi_gui_file_update_matched(file);
 }
@@ -1195,6 +1214,7 @@ fi_gui_file_fill_status(struct fileinfo_data *file)
 	file->uploaded = status.uploaded;
 	file->size = status.size;
 	file->done = status.done;
+	file->modified = status.modified;
 	file->progress = file->size ? filesize_per_10000(file->size, file->done) : 0;
 
 	file->paused = 0 != status.paused;
@@ -1268,6 +1288,17 @@ fi_gui_file_get_file_url_at_cursor(GtkWidget *unused_widget)
 	return file ? fi_gui_file_get_file_url(file) : NULL;
 }
 
+static struct fileinfo_data *
+fi_gui_file_by_handle(gnet_fi_t handle)
+{
+	struct fileinfo_data *file;
+
+	file = g_hash_table_lookup(fi_handles, uint_to_pointer(handle));
+	g_return_val_if_fail(file, NULL);
+	g_assert(handle == file->handle);
+	return file;
+}
+
 static void
 fi_gui_fi_added(gnet_fi_t handle)
 {
@@ -1310,15 +1341,14 @@ fi_gui_fi_removed(gnet_fi_t handle)
 {
 	struct fileinfo_data *file;
 	void *key;
-	
-	key = uint_to_pointer(handle);
-	file = g_hash_table_lookup(fi_handles, key);
+
+	file = fi_gui_file_by_handle(handle);
 	g_return_if_fail(file);
-	g_return_if_fail(handle == file->handle);
 
 	if (last_shown_valid && handle == last_shown) {
 		fi_gui_clear_info();
 	}
+	key = uint_to_pointer(handle);
 	g_hash_table_remove(fi_handles, key);
 	g_hash_table_remove(fi_updates, key);
 	g_assert(NULL == file->sources);
@@ -1345,17 +1375,13 @@ fi_gui_set_aliases(struct fileinfo_data *file)
 static struct fileinfo_data *
 fi_gui_source_get_file(const struct download *d)
 {
-	struct fileinfo_data *file;
 	const fileinfo_t *fi;
 
 	download_check(d);
 	fi = d->file_info;
 	g_return_val_if_fail(fi, NULL);
 
-	file = g_hash_table_lookup(fi_handles, uint_to_pointer(fi->fi_handle));
-	g_return_val_if_fail(file, NULL);
-	g_assert(fi->fi_handle == file->handle);
-	return file;
+	return fi_gui_file_by_handle(fi->fi_handle);
 }
 
 /**
@@ -1467,6 +1493,14 @@ fi_gui_show_info(struct fileinfo_data *file)
 	vp_draw_fi_progress(last_shown_valid, last_shown);
 }
 
+static void
+fi_gui_files_details_update(void)
+{
+	if (last_shown_valid) {
+		fi_gui_set_details(fi_gui_file_by_handle(last_shown));
+	}
+}
+
 void
 fi_gui_files_cursor_update(void)
 {
@@ -1487,7 +1521,7 @@ fi_gui_file_update(gnet_fi_t handle)
 {
 	struct fileinfo_data *file;
 
-	file = g_hash_table_lookup(fi_handles, uint_to_pointer(handle));
+	file = fi_gui_file_by_handle(handle);
 	g_return_if_fail(file);
 
 	fi_gui_file_fill_status(file);
@@ -1498,6 +1532,29 @@ fi_gui_file_update(gnet_fi_t handle)
 	}
 }
 
+gboolean
+fi_gui_rename(const char *filename)
+{
+	g_return_val_if_fail(filename, FALSE);
+	g_return_val_if_fail(last_shown_valid, FALSE);
+
+	return guc_fi_rename(last_shown, filename);
+}
+
+static void
+fi_gui_fi_info_changed(gnet_fi_t handle)
+{
+	struct fileinfo_data *file;
+
+	file = fi_gui_file_by_handle(handle);
+	g_return_if_fail(file);
+
+	fi_gui_file_set_filename(file);
+	if (last_shown_valid && handle == last_shown) {
+		fi_gui_show_info(file);
+	}
+}
+
 static void
 fi_gui_fi_status_changed(gnet_fi_t handle)
 {
@@ -1676,6 +1733,9 @@ fileinfo_data_cmp(const struct fileinfo_data *a, const struct fileinfo_data *b,
 	case c_fi_created:
 		ret = delta_time(a->created, b->created);
 		break;
+	case c_fi_modified:
+		ret = delta_time(a->modified, b->modified);
+		break;
 	case c_fi_num:
 		g_assert_not_reached();
 	}
@@ -1778,6 +1838,9 @@ fi_gui_file_column_text(const struct fileinfo_data *file, int column)
 	case c_fi_created:
 		text = timestamp_to_string(file->created);
 		break;
+	case c_fi_modified:
+		text = file->modified ? timestamp_to_string(file->modified) : "-";
+		break;
 	case c_fi_num:
 		g_assert_not_reached();
 	}
@@ -1827,6 +1890,7 @@ fi_gui_files_column_title(int column)
 	case c_fi_uploaded:	return _("Uploaded");
 	case c_fi_sources:	return _("Sources");
 	case c_fi_created:	return _("Created");
+	case c_fi_modified:	return _("Modified");
 	case c_fi_status:	return _("Status");
 	case c_fi_num:		break;
 	}
@@ -1849,6 +1913,7 @@ fi_gui_files_column_justify_right(int column)
 	case c_fi_uploaded:	return TRUE;
 	case c_fi_sources:	return FALSE;
 	case c_fi_created:	return FALSE;
+	case c_fi_modified:	return FALSE;
 	case c_fi_status:	return FALSE;
 	case c_fi_num:		break;
 	}
@@ -1881,6 +1946,7 @@ notebook_downloads_init_page(GtkNotebook *notebook, int page_num)
 	gtk_widget_show_all(GTK_WIDGET(container));
 
 	downloads_gui_update_popup_downloads();
+	fi_gui_files_details_update();
 }
 
 static void
@@ -1891,6 +1957,7 @@ on_visibility_change(gboolean visible)
 	download_gui_visible = visible;
 	if (visible) {
 		fi_gui_update_display();
+		fi_gui_files_details_update();
 		fi_gui_files_thaw();
 	} else {
 		fi_gui_files_freeze();
@@ -1966,6 +2033,7 @@ notebook_downloads_init(void)
 
 struct select_by_regex {
 	regex_t expr;
+	gboolean invert;
 	unsigned matches, total_nodes;
 };
 
@@ -1994,15 +2062,17 @@ fi_gui_select_by_regex_helper(struct fileinfo_data *file, void *user_data)
 	ctx->total_nodes++;
 
 	ret = regexec(&ctx->expr, fi_gui_file_get_filename(file), 0, NULL, 0);
-	if (0 == ret) {
-		ctx->matches++;
-		fi_gui_file_select(file);
-	} else if (REG_NOMATCH != ret) {
+	if (0 != ret && REG_NOMATCH != ret) {
 		fi_gui_regex_error(&ctx->expr, ret);
 		fi_gui_files_unselect_all();
 		ctx->matches = 0;
 		return TRUE;	/* stop */
 	}
+
+	if ((0 == ret) ^ (0 != ctx->invert)) {
+		ctx->matches++;
+		fi_gui_file_select(file);
+	}
 	return FALSE;
 }
 
@@ -2010,6 +2080,7 @@ static void
 fi_gui_select_by_regex(const char *regex)
 {
 	struct select_by_regex ctx;
+	gboolean case_sensitive;
     int ret, flags;
 
 	ctx.matches = 0;
@@ -2019,8 +2090,15 @@ fi_gui_select_by_regex(const char *regex)
 	if (NULL == regex || '\0' == regex[0])
 		return;
 
+	ctx.invert = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(gui_main_window_lookup(
+				"checkbutton_downloads_select_regex_invert")));
+	case_sensitive = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(gui_main_window_lookup(
+				"checkbutton_downloads_select_regex_case")));
+
 	flags = REG_EXTENDED | REG_NOSUB;
-   	flags |= GUI_PROPERTY(download_select_regex_case) ? 0 : REG_ICASE;
+   	flags |= case_sensitive ? 0 : REG_ICASE;
     ret = regcomp(&ctx.expr, regex, flags);
    	if (ret) {
 		fi_gui_regex_error(&ctx.expr, ret);
@@ -2056,6 +2134,25 @@ on_entry_downloads_select_regex_activate(GtkEditable *editable,
 	G_FREE_NULL(regex);
 }
 
+void
+on_checkbutton_downloads_select_regex_case_toggled(
+	GtkToggleButton *unused_button, void *user_data)
+{
+	(void) unused_button;
+	on_entry_downloads_select_regex_activate(
+		GTK_EDITABLE(gui_main_window_lookup("entry_downloads_select_regex")),
+		user_data);
+}
+
+void
+on_checkbutton_downloads_select_regex_invert_toggled(
+	GtkToggleButton *unused_button, void *user_data)
+{
+	(void) unused_button;
+	on_entry_downloads_select_regex_activate(
+		GTK_EDITABLE(gui_main_window_lookup("entry_downloads_select_regex")),
+		user_data);
+}
 
 static void
 fi_handles_filter(void *key, void *value, void *unused_udata)
@@ -2095,16 +2192,11 @@ fi_gui_filter_by_regex(const char *expr)
 	g_return_if_fail(fi_handles);
 
 	filter_regex_clear();
-	if (
-		expr &&
-	   	0 != strcmp(expr, "") &&
-	   	0 != strcmp(expr, ".") &&
-	   	0 != strcmp(expr, ".*")
-	) {
+	if (expr && 0 != strcmp(expr, "")) {
 		int ret, flags;
-		
+	
 		flags = REG_EXTENDED | REG_NOSUB;
-   		flags |= GUI_PROPERTY(download_filter_regex_case) ? 0 : REG_ICASE;
+		flags |= filter_regex_case_sensitive ? 0 : REG_ICASE;
 		filter_regex = g_malloc(sizeof *filter_regex);
     	ret = regcomp(filter_regex, expr, flags);
 		if (ret) {
@@ -2133,6 +2225,32 @@ on_entry_downloads_filter_regex_activate(GtkEditable *editable,
 	G_FREE_NULL(regex);
 }
 
+void
+on_checkbutton_downloads_filter_regex_case_toggled(
+	GtkToggleButton *unused_button, void *user_data)
+{
+	(void) unused_button;
+	filter_regex_case_sensitive = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(gui_main_window_lookup(
+				"checkbutton_downloads_filter_regex_case")));
+	on_entry_downloads_filter_regex_activate(
+		GTK_EDITABLE(gui_main_window_lookup("entry_downloads_filter_regex")),
+		user_data);
+}
+
+void
+on_checkbutton_downloads_filter_regex_invert_toggled(
+	GtkToggleButton *unused_button, void *user_data)
+{
+	(void) unused_button;
+	filter_regex_invert = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(gui_main_window_lookup(
+				"checkbutton_downloads_filter_regex_invert")));
+	on_entry_downloads_filter_regex_activate(
+		GTK_EDITABLE(gui_main_window_lookup("entry_downloads_filter_regex")),
+		user_data);
+}
+
 void
 fi_gui_common_init(void)
 {
@@ -2148,6 +2266,8 @@ fi_gui_common_init(void)
 		FREQ_SECS, 0);
     guc_fi_add_listener(fi_gui_fi_status_changed_transient,
 		EV_FI_STATUS_CHANGED_TRANSIENT, FREQ_SECS, 0);
+    guc_fi_add_listener(fi_gui_fi_info_changed, EV_FI_INFO_CHANGED,
+		FREQ_SECS, 0);
 
     guc_src_add_listener(fi_gui_src_added, EV_SRC_ADDED, FREQ_SECS, 0);
     guc_src_add_listener(fi_gui_src_removed, EV_SRC_REMOVED, FREQ_SECS, 0);
