@@ -1,5 +1,5 @@
 /*
- * $Id: filter_core.c 15039 2007-12-01 19:09:17Z cbiere $
+ * $Id: filter_core.c 15936 2009-01-27 21:45:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: filter_core.c 15039 2007-12-01 19:09:17Z cbiere $")
+RCSID("$Id: filter_core.c 15936 2009-01-27 21:45:12Z cbiere $")
 
 #include "gtk/filter.h"
 #include "gtk/filter_core.h"
@@ -2535,11 +2535,11 @@ filter_init_presets(void)
 		const char *regex;
 		filesize_t minsize;
 	} tab[] = {
-		{ N_("<Archive>"),	  "[.](bz2|gz|zip|rar|iso)$", 0 },
-		{ N_("<Audio>"), 	  "[.](mp3|m4a|ogg|flac)$", 1000000 },
+		{ N_("<Archive>"),	  "[.](bz2|gz|zip|rar|iso|7z)$", 0 },
+		{ N_("<Audio>"), 	  "[.](mp3|m4a|ogg|oga|flac)$", 1000000 },
 		{ N_("<Image>"), 	  "[.](bmp|gif|jpg|jpeg|png|psd|tif|tiff)$", 0 },
-		{ N_("<Literature>"), "[.](pdf|doc|lit|djvu|ps)$", 10000 },
-		{ N_("<Video>"), 	  "[.](avi|mpg|mp4|mpeg|mkv|ogm)$", 10000000 },
+		{ N_("<Literature>"), "[.](pdf|doc|lit|djvu|ps|txt)$", 10000 },
+		{ N_("<Video>"), 	  "[.](avi|mpg|mp4|mpeg|mkv|ogm|ogv)$", 10000000 },
 	};
 	unsigned i;
 
