@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.c 14681 2007-08-24 16:13:04Z cbiere $
+ * $Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $
  *
  * Copyright (c) 2003, Richard Eckart
  *
@@ -35,9 +35,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c 14681 2007-08-24 16:13:04Z cbiere $")
-
-#include "downloads_cb.h"
+RCSID("$Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $")
 
 #include "gtk/columns.h"
 #include "gtk/downloads_common.h"
@@ -51,6 +49,12 @@ RCSID("$Id: fileinfo.c 14681 2007-08-24 16:13:04Z cbiere $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
+/*
+ * Define this macro to enable consistency checks for the row cache.
+ * This will affect performance negatively O(1) -> O(n).
+ */
+#undef FILEINFO_C_ROW_CACHE_REGRESSION
+
 static GtkCList *clist_download_files;
 static GtkCList *clist_download_sources;
 static GtkCList *clist_download_aliases;
@@ -85,7 +89,7 @@ get_fileinfo_data(int row)
 	g_assert(file);
 	g_assert(row == fileinfo_data_get_row(file));
 
-#if 1
+#ifdef FILEINFO_C_ROW_CACHE_REGRESSION
 	{
 		struct fileinfo_data *x;
 
@@ -95,7 +99,7 @@ get_fileinfo_data(int row)
 		x = gtk_clist_get_row_data(clist_download_files, row);
    		g_assert(x == file);
 	}
-#endif
+#endif	/* FILEINFO_C_ROW_CACHE_REGRESSION */
 
 	return file;
 }
@@ -255,7 +259,7 @@ get_source(int row)
 	d = g_hash_table_lookup(source_rows, int_to_pointer(row));
 	g_assert(pointer_to_int(g_hash_table_lookup(fi_sources, d)) == row);
 
-#if 1
+#ifdef FILEINFO_C_ROW_CACHE_REGRESSION
 	{
 		struct download *x;
 
@@ -265,7 +269,7 @@ get_source(int row)
 		x = gtk_clist_get_row_data(clist_download_sources, row);
    		g_assert(x == d);
 	}
-#endif
+#endif	/* FILEINFO_C_ROW_CACHE_REGRESSION */
 
 	download_check(d);
 	return d;
@@ -719,6 +723,7 @@ fi_gui_files_foreach(fi_gui_files_foreach_cb func, void *user_data)
 void
 fi_gui_files_freeze(void)
 {
+	gtk_clist_freeze(clist_download_details);
 	gtk_clist_freeze(clist_download_files);
 	gtk_clist_freeze(clist_download_sources);
 }
@@ -726,6 +731,7 @@ fi_gui_files_freeze(void)
 void
 fi_gui_files_thaw(void)
 {
+	gtk_clist_thaw(clist_download_details);
 	gtk_clist_thaw(clist_download_files);
 	gtk_clist_thaw(clist_download_sources);
 }
@@ -740,11 +746,13 @@ fi_gui_clear_details(void)
 }
 
 void
-fi_gui_append_detail(const gchar *name, const gchar *value)
+fi_gui_append_detail(const enum fi_gui_detail unused_id,
+	const gchar *name, const gchar *value)
 {
 	GtkCList *clist = clist_download_details;
  	const gchar *titles[2];
 
+	(void) unused_id;
 	g_return_if_fail(clist);
 	titles[0] = name;
 	titles[1] = EMPTY_STRING(value);
