@@ -1,5 +1,5 @@
 /*
- * $Id: search.c 15037 2007-12-01 14:51:07Z cbiere $
+ * $Id: search.c 15941 2009-01-31 23:29:27Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -37,14 +37,13 @@
  */
 #include "gtk/gui.h"
 
-RCSID("$Id: search.c 15037 2007-12-01 14:51:07Z cbiere $")
+RCSID("$Id: search.c 15941 2009-01-31 23:29:27Z cbiere $")
 
 #include "search_cb.h"
 
 #include "gtk/bitzi.h"
 #include "gtk/columns.h"
 #include "gtk/drag.h"
-#include "gtk/drop.h"
 #include "gtk/misc.h"
 #include "gtk/notebooks.h"
 #include "gtk/search_common.h"
@@ -55,16 +54,18 @@ RCSID("$Id: search.c 15037 2007-12-01 14:51:07Z cbiere $")
 #include "if/bridge/ui2c.h"
 #include "if/core/sockets.h"
 
+#include "lib/ascii.h"
 #include "lib/atoms.h"
 #include "lib/base32.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
-#include "lib/misc.h"
+#include "lib/mime_type.h"
 #include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/urn.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 static gchar tmpstr[4096];
@@ -320,6 +321,17 @@ search_gui_compare_records(gint sort_col,
             result = strcmp(EMPTY_STRING(r1->ext), EMPTY_STRING(r2->ext));
             break;
 
+		
+        case c_sr_mime:
+			{
+				enum mime_type mt1, mt2;
+
+				mt1 = mime_type_from_extension(r1->ext);
+				mt2 = mime_type_from_extension(r2->ext);
+				result = CMP(mt1, mt2);
+			}
+			break;
+
         case c_sr_charset:
             result = strcmp(EMPTY_STRING(r1->charset),
 						EMPTY_STRING(r2->charset));
@@ -1007,6 +1019,9 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 	 		case c_sr_ext:
 				text = rc->ext;
 				break;
+			case c_sr_mime:
+				text = mime_type_to_string(mime_type_from_extension(rc->ext));
+				break;
 			case c_sr_charset:
 				if (!(ST_LOCAL & rs->status))
 					text = rc->charset;
@@ -1821,9 +1836,12 @@ search_gui_set_search_list_color(const struct search *search, int row)
 	GdkColor *fg, *bg;
 
 	style = gtk_widget_get_style(GTK_WIDGET(clist_search()));
-	if (search_gui_is_enabled(search)) {
+	if (search->unseen_items > 0) {
 		fg = &style->fg[GTK_STATE_ACTIVE];
 		bg = &style->bg[GTK_STATE_ACTIVE];
+	} else if (search_gui_is_enabled(search)) {
+		fg = NULL;
+		bg = NULL;
 	} else {
 		fg = &style->fg[GTK_STATE_INSENSITIVE];
 		bg = &style->bg[GTK_STATE_INSENSITIVE];
@@ -1909,9 +1927,18 @@ search_gui_metadata_update(const bitzi_data_t *data)
     	GtkCTreeNode *parent;
 
 		parent = find_parent_with_sha1(search, data->sha1);
-		if (parent)
+		if (parent) {
+			gui_record_t *grc;
+			record_t *record;
+
 			gtk_ctree_node_set_text(ctree, parent,
 					c_sr_meta, text ? text : _("Not in database"));
+			grc = gtk_ctree_node_get_row_data(ctree, parent);
+			record = grc->shared_record;
+			if (search_gui_item_is_inspected(record)) {
+				search_gui_set_bitzi_metadata(record);
+			}
+		}
 	}
 
 	G_FREE_NULL(text);
@@ -1948,7 +1975,7 @@ search_gui_queue_bitzi_by_sha1(const record_t *rec)
 	}
 
 	/* and then send the query... */
-	guc_query_bitzi_by_sha1(rec->sha1, rec->size);
+	guc_query_bitzi_by_sha1(rec->sha1, rec->size, TRUE);
 }
 
 record_t *
@@ -2054,4 +2081,12 @@ search_gui_flush_queues(void)
 		/* TODO: Implement this */
 }
 
+unsigned
+search_gui_queue_length(const struct search *search)
+{
+	g_return_val_if_fail(search, 0);
+	/* TODO: Implement this */
+	return 0;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
