@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.c 14452 2007-08-13 15:10:27Z cbiere $
+ * $Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $
  *
  * Copyright (c) 2003, Richard Eckart
  *
@@ -35,9 +35,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c 14452 2007-08-13 15:10:27Z cbiere $")
-
-#include "downloads_cb.h"
+RCSID("$Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $")
 
 #include "gtk/columns.h"
 #include "gtk/downloads_common.h"
@@ -62,26 +60,26 @@ static GtkListStore *store_aliases;
 static GtkListStore *store_files;
 static GtkListStore *store_sources;
 
-#ifdef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+#if GTK_CHECK_VERSION(2,6,0)
 static GtkSortType files_sort_order;
 static int files_sort_column;
 static int files_sort_depth;
-#endif	/* GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID */
+#endif	/* Gtk+ => 2.6.0 */
 
 static void
 fi_gui_files_sort_reset(void)
 {
-#ifdef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+#if GTK_CHECK_VERSION(2,6,0)
 	files_sort_column = GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID;
 	files_sort_order = GTK_SORT_ASCENDING;
 	files_sort_depth = 0;
-#endif	/* GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID */
+#endif	/* Gtk+ => 2.6.0 */
 }
 
 static void
 fi_gui_files_sort_save(void)
 {
-#ifdef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+#if GTK_CHECK_VERSION(2,6,0)
 	if (0 == files_sort_depth++) {
 		GtkTreeSortable *sortable;
 		GtkSortType order;
@@ -95,15 +93,15 @@ fi_gui_files_sort_save(void)
 					GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
 		}
 	}
-#endif /* GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID */
+#endif	/* Gtk+ => 2.6.0 */
 }
 
 static void
 fi_gui_files_sort_restore(void)
 {
-#ifdef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+#if GTK_CHECK_VERSION(2,6,0)
 	g_return_if_fail(files_sort_depth > 0);
-	files_sort_depth--
+	files_sort_depth--;
 
 	if (0 == files_sort_depth) {
 		if (GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID != files_sort_column) {
@@ -111,7 +109,7 @@ fi_gui_files_sort_restore(void)
 					files_sort_column, files_sort_order);
 		}
 	}
-#endif /* GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID */
+#endif	/* Gtk+ => 2.6.0 */
 }
 
 static inline void
@@ -389,7 +387,7 @@ download_details_get_text(GtkWidget *widget)
 		GValue value;
 
 		value = zero_value;
-		gtk_tree_model_get_value(model, &iter, 1, &value);
+		gtk_tree_model_get_value(model, &iter, 2, &value);
 		return g_strdup(g_value_get_string(&value));
 	} else {
 		return NULL;
@@ -530,6 +528,33 @@ fi_gui_get_alias(GtkWidget *widget)
 	}
 }
 
+static void
+on_cell_edited(GtkCellRendererText *unused_renderer, const char *path_str,
+	const char *text, gpointer data)
+{
+	GtkTreeView *tv = data;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	unsigned int id;
+
+	(void) unused_renderer;
+
+	g_return_if_fail(NULL != tv);
+	model = gtk_tree_view_get_model(tv);
+	g_return_if_fail(NULL != model);
+
+	path = gtk_tree_path_new_from_string(path_str);
+	gtk_tree_model_get_iter(model, &iter, path);
+
+	gtk_tree_model_get(model, &iter, 0, &id, (-1));
+	if (FI_GUI_DETAIL_FILENAME == id) {
+		fi_gui_rename(text);
+	}
+
+	gtk_tree_path_free(path);
+}
+
 static void
 fi_gui_details_treeview_init(void)
 {
@@ -538,6 +563,7 @@ fi_gui_details_treeview_init(void)
 		gfloat xalign;
 		gboolean editable;
 	} tab[] = {
+		{ "ID",		1.0, FALSE },
 		{ "Item",	1.0, FALSE },
 		{ "Value",	0.0, TRUE },
 	};
@@ -549,8 +575,8 @@ fi_gui_details_treeview_init(void)
 	g_return_if_fail(tv);
 	treeview_download_details = tv;
 
-	model = GTK_TREE_MODEL(
-		gtk_list_store_new(G_N_ELEMENTS(tab), G_TYPE_STRING, G_TYPE_STRING));
+	model = GTK_TREE_MODEL(gtk_list_store_new(G_N_ELEMENTS(tab),
+				G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING));
 
 	gtk_tree_view_set_model(tv, model);
 	g_object_unref(model);
@@ -563,12 +589,14 @@ fi_gui_details_treeview_init(void)
 		g_object_set(G_OBJECT(renderer),
 			"editable", tab[i].editable,
 			(void *) 0);
+		gui_signal_connect(renderer, "edited", on_cell_edited, tv);
 		column = gtk_tree_view_column_new_with_attributes(tab[i].title,
 					renderer, "text", i, (void *) 0);
 		g_object_set(column,
+			"visible",	i > 0 ? TRUE : FALSE,
 			"min-width", 1,
 			"resizable", TRUE,
-			"sizing", (0 == i)
+			"sizing", 1 == i
 						? GTK_TREE_VIEW_COLUMN_AUTOSIZE
 						: GTK_TREE_VIEW_COLUMN_FIXED,
 			(void *) 0);
@@ -614,6 +642,7 @@ treeview_download_files_init(void)
 			c_fi_progress == i ? gtk_cell_renderer_progress_new() : NULL,
 			render_files);
 	}
+
 	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(tv),
 		GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_headers_visible(tv, TRUE);
@@ -852,6 +881,7 @@ void
 fi_gui_files_freeze(void)
 {
 
+	g_object_freeze_notify(G_OBJECT(treeview_download_details));
 	g_object_freeze_notify(G_OBJECT(treeview_download_files));
 	g_object_freeze_notify(G_OBJECT(treeview_download_sources));
 	g_object_freeze_notify(G_OBJECT(store_files));
@@ -867,6 +897,7 @@ fi_gui_files_thaw(void)
 
 	g_object_thaw_notify(G_OBJECT(store_files));
 	g_object_thaw_notify(G_OBJECT(store_sources));
+	g_object_thaw_notify(G_OBJECT(treeview_download_details));
 	g_object_thaw_notify(G_OBJECT(treeview_download_files));
 	g_object_thaw_notify(G_OBJECT(treeview_download_sources));
 }
@@ -901,7 +932,8 @@ fi_gui_clear_details(void)
 }
 
 void
-fi_gui_append_detail(const gchar *title, const gchar *value)
+fi_gui_append_detail(const enum fi_gui_detail id,
+	const gchar *title, const gchar *value)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -910,7 +942,7 @@ fi_gui_append_detail(const gchar *title, const gchar *value)
     model = gtk_tree_view_get_model(treeview_download_details);
 
 	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, title, 1, value, (-1));
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, id, 1, title, 2, value, (-1));
 }
 
 /* vi: set ts=4 sw=4 cindent: */
