@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $
+ * $Id: gnet_stats.c 16076 2009-03-07 18:24:15Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $")
+RCSID("$Id: gnet_stats.c 16076 2009-03-07 18:24:15Z rmanfredi $")
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
@@ -156,7 +156,11 @@ general_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint type)
 {
 	if (stats->general[type] == 0)
 		g_strlcpy(dst, "-", size);
-	else if (type == GNR_QUERY_COMPACT_SIZE)
+	else if (
+		type == GNR_QUERY_COMPACT_SIZE ||
+		type == GNR_IGNORED_DATA ||
+		type == GNR_SUNK_DATA
+	)
 		g_strlcpy(dst,
 			compact_size(stats->general[type], show_metric_units()), size);
 	else
