@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c 14457 2007-08-14 04:23:12Z cbiere $
+ * $Id: nodes.c 15814 2008-09-07 03:09:56Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: nodes.c 14457 2007-08-14 04:23:12Z cbiere $")
+RCSID("$Id: nodes.c 15814 2008-09-07 03:09:56Z cbiere $")
 
 #include "gtk/misc.h"
 #include "gtk/nodes_common.h"
@@ -896,7 +896,7 @@ nodes_gui_browse_selected_helper(GtkTreeModel *model,
 	info = guc_node_get_info(data->node_id);
 	if (!info->is_pseudo) {
 		search_gui_new_browse_host(NULL, info->gnet_addr, info->gnet_port,
-			info->gnet_guid, NULL, 0);
+			&info->gnet_guid, NULL, 0);
 	}
 	guc_node_free_info(info);
 }
