@@ -1,5 +1,5 @@
 /*
- * $Id: search.c 15038 2007-12-01 15:11:17Z cbiere $
+ * $Id: search.c 15941 2009-01-31 23:29:27Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -53,9 +53,10 @@
 #include "if/core/sockets.h"
 
 #include "lib/atoms.h"
-#include "lib/misc.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
+#include "lib/mime_type.h"
+#include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/utf8.h"
@@ -63,7 +64,7 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: search.c 15038 2007-12-01 15:11:17Z cbiere $")
+RCSID("$Id: search.c 15941 2009-01-31 23:29:27Z cbiere $")
 
 static GtkTreeView *tree_view_search;
 
@@ -213,6 +214,9 @@ cell_renderer(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	case c_sr_ext:
 		text = data->record->ext;
 		break;
+	case c_sr_mime:
+		text = mime_type_to_string(mime_type_from_extension(data->record->ext));
+		break;
 	case c_sr_meta:
 		text = data->meta;
 		break;
@@ -353,7 +357,6 @@ add_column(
 		"reorderable", FALSE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		"expand", (gboolean) (0 == id),
 		(void *) 0);
 	
     gtk_tree_view_append_column(tv, column);
@@ -494,19 +497,30 @@ search_gui_clear_search(search_t *search)
 	g_assert(0 == g_hash_table_size(search->parents));
 }
 
+static gboolean
+search_gui_is_sorted(struct search *search)
+{
+	GtkTreeModel *model;
+	GtkTreeSortable *sortable;
+
+	g_return_val_if_fail(search, FALSE);
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(search->tree));
+	sortable = GTK_TREE_SORTABLE(model);
+	return gtk_tree_sortable_get_sort_column_id(sortable, NULL, NULL);
+}
+
 static void
 search_gui_disable_sort(struct search *search)
 {
 	if (search && search->sort) {
-#ifdef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+#if GTK_CHECK_VERSION(2,6,0)
 		GtkTreeModel *model;
 		GtkTreeSortable *sortable;
 
-		g_return_if_fail(search);
-
 		model = gtk_tree_view_get_model(GTK_TREE_VIEW(search->tree));
 		sortable = GTK_TREE_SORTABLE(model);
-		if (gtk_tree_sortable_get_sort_column_id(sortable, NULL)) {
+		if (gtk_tree_sortable_get_sort_column_id(sortable, NULL, NULL)) {
 			gtk_tree_sortable_set_sort_column_id(sortable,
 				GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_DESCENDING);
 		}
@@ -721,6 +735,16 @@ search_gui_cmp_ext(const struct result_data *a, const struct result_data *b)
 	return search_gui_cmp_strings(a->record->ext, b->record->ext);
 }
 
+static int
+search_gui_cmp_mime(const struct result_data *a, const struct result_data *b)
+{
+	enum mime_type mt_a, mt_b;
+
+	mt_a = mime_type_from_extension(a->record->ext);
+	mt_b = mime_type_from_extension(b->record->ext);
+	return CMP(mt_a, mt_b);
+}
+
 static int
 search_gui_cmp_meta(const struct result_data *a, const struct result_data *b)
 {
@@ -811,6 +835,7 @@ search_gui_cmp(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIter *iter2,
 	switch (column) {
 	case c_sr_filename: ret = search_gui_cmp_filename(a, b); break;
 	case c_sr_ext:		ret = search_gui_cmp_ext(a, b); break;
+	case c_sr_mime:		ret = search_gui_cmp_mime(a, b); break;
 	case c_sr_meta:		ret = search_gui_cmp_meta(a, b); break;
 	case c_sr_vendor:	ret = search_gui_cmp_vendor(a, b); break;
 	case c_sr_info:		ret = search_gui_cmp_info(a, b); break;
@@ -1362,7 +1387,10 @@ search_gui_update_list_label(const struct search *search)
 		return;
 
 	style = gtk_widget_get_style(GTK_WIDGET(tv));
-	if (search_gui_is_enabled(search)) {
+	if (search->unseen_items > 0) {
+		fg = &style->fg[GTK_STATE_ACTIVE];
+		bg = &style->bg[GTK_STATE_ACTIVE];
+	} else if (search_gui_is_enabled(search)) {
 		fg = NULL;
 		bg = NULL;
 	} else {
@@ -1459,7 +1487,7 @@ search_gui_request_bitzi_data_helper(gpointer key,
 	g_return_if_fail(rd->record->sha1);
 
 	atom_str_change(&rd->meta, _("Query queued..."));
-	guc_query_bitzi_by_sha1(rd->record->sha1, rd->record->size);
+	guc_query_bitzi_by_sha1(rd->record->sha1, rd->record->size, TRUE);
 }
 
 static void
@@ -1545,6 +1573,9 @@ search_gui_metadata_update(const bitzi_data_t *data)
 			GtkTreeView *tv = GTK_TREE_VIEW(search->tree);
 			atom_str_change(&rd->meta, text ? text : _("Not in database"));
 			search_gui_data_changed(gtk_tree_view_get_model(tv), rd);
+			if (search_gui_item_is_inspected(rd->record)) {
+				search_gui_set_bitzi_metadata(rd->record);
+			}
 		}
 	}
 
@@ -1732,7 +1763,7 @@ search_gui_flush_queue_data(search_t *search, GtkTreeModel *model,
 	 * cache. If so lets update the GUI to reflect this.
 	 */
 	if (NULL != rc->sha1 && guc_bitzi_has_cached_ticket(rc->sha1)) {
-		guc_query_bitzi_by_sha1(rc->sha1, rc->size);
+		guc_query_bitzi_by_sha1(rc->sha1, rc->size, FALSE);
 	}
 }
 
@@ -1745,14 +1776,15 @@ search_gui_flush_queue(search_t *search)
 	if (slist_length(search->queue) > 0) {
 		GtkTreeModel *model;
 		slist_iter_t *iter;
-		guint n = 0;
+		guint max_items;
 
 		search_gui_start_massive_update(search);
 
 		model = gtk_tree_view_get_model(GTK_TREE_VIEW(search->tree));
+		max_items = search_gui_is_sorted(search) ? 100 : 500;
 
 		iter = slist_iter_on_head(search->queue);
-		while (slist_iter_has_item(iter) && n++ < 100) {
+		while (slist_iter_has_item(iter) && max_items-- > 0) {
 			struct result_data *data;
 
 			data = slist_iter_current(iter);
@@ -1776,5 +1808,14 @@ search_gui_flush_queues(void)
 	}
 }
 
+unsigned
+search_gui_queue_length(const struct search *search)
+{
+	g_return_val_if_fail(search, 0);
+	g_return_val_if_fail(search->queue, 0);
+	
+	return slist_length(search->queue);
+}
+
 /* -*- mode: cc-mode; tab-width:4; -*- */
 /* vi: set ts=4 sw=4 cindent: */
