@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c 14524 2007-08-17 23:40:34Z cbiere $
+ * $Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -53,7 +53,7 @@
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: uploads.c 14524 2007-08-17 23:40:34Z cbiere $")
+RCSID("$Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $")
 
 static gboolean uploads_remove_lock;
 static gboolean uploads_shutting_down;
@@ -73,7 +73,7 @@ static void uploads_gui_add_upload(gnet_upload_info_t *u);
 static const char * const column_titles[UPLOADS_GUI_VISIBLE_COLUMNS] = {
 	N_("Filename"),
 	N_("Host"),
-	N_("Loc"),
+	N_("Country"),
 	N_("Size"),
 	N_("Range"),
 	N_("User-Agent"),
