@@ -1,5 +1,5 @@
 /*
- * $Id: gui.h 14001 2007-07-01 14:16:15Z cbiere $
+ * $Id: gui.h 14425 2007-08-12 17:05:22Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -30,55 +30,45 @@
 
 #ifdef USE_TOPLESS
 
-/* Diverse dummy definitions */
 #define settings_gui_save_if_dirty()
+#define settings_gui_shutdown()
 
 #define main_gui_early_init(argc, argv, disable_xshm)
 #define main_gui_init()
+#define main_gui_run(x)
+#define main_gui_exit(x)
 #define main_gui_timer(x)
-#define main_gui_update_coords()
 #define main_gui_shutdown()
 #define main_gui_shutdown_tick(remain)
-#define settings_gui_shutdown()
-
-#define search_gui_store_searches()
-
-#define icon_timer()
-
-static inline void
-main_gui_run(const gchar *unused_geometry_spec)
-{
-	GMainLoop *ml;
-
-	(void) unused_geometry_spec;
-
-#if defined(USE_GLIB1)
-	ml = g_main_new(FALSE);
-	g_main_run(ml);
-#elif defined(USE_GLIB2)
-	ml = g_main_loop_new(NULL, FALSE);
-	g_main_loop_run(ml);
-#endif /* GLIB */
-}
 
 #else	/* !USE_TOPLESS */
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+
+#include "gtk-missing.h"
 
 #ifdef USE_GTK1
+#include "gtk1/interface-glade.h"
 #include "gtk1/support-glade.h"
 #endif
+
 #ifdef USE_GTK2
+#include "gtk2/interface-glade.h"
 #include "gtk2/support-glade.h"
+
+#if !GTK_CHECK_VERSION(2,5,0)
+#include "pbarcellrenderer.h"
+#endif
+
+/* Common padding values for GtkCellRenderer */
+#define GUI_CELL_RENDERER_XPAD ((guint) 4U)
+#define GUI_CELL_RENDERER_YPAD ((guint) 0U)
+
 #endif
 
 #include "main.h"
 
-#ifdef USE_GTK1
-#define g_ascii_strcasecmp g_strcasecmp
-#define gdk_drawable_get_size gdk_window_get_size
-#endif
-
 /* GUI signal functions */
 #ifdef USE_GTK1
 #define gui_signal_connect(widget, name, func, data) \
@@ -116,11 +106,6 @@ main_gui_run(const gchar *unused_geometry_spec)
 	
 #endif	/* Gtk+ 2.0 */
 
-
-/* Common padding values for GtkCellRenderer */
-#define GUI_CELL_RENDERER_XPAD ((guint) 4U)
-#define GUI_CELL_RENDERER_YPAD ((guint) 0U)
-
 /**
  * Sorting constants.
  */
