@@ -1,5 +1,5 @@
 /*
- * $Id: html_view.c 14425 2007-08-12 17:05:22Z cbiere $
+ * $Id: html_view.c 15781 2008-09-02 16:30:18Z rmanfredi $
  *
  * Copyright (c) 2007 Christian Biere
  *
@@ -35,11 +35,11 @@
 
 #include "gui.h"
 
-RCSID("$Id: html_view.c 14425 2007-08-12 17:05:22Z cbiere $")
+RCSID("$Id: html_view.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
+#include "lib/ascii.h"
 #include "lib/glib-missing.h"
 #include "lib/html.h"
-#include "lib/misc.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
