@@ -1,5 +1,5 @@
 /*
- * $Id: search_xml.c 14457 2007-08-14 04:23:12Z cbiere $
+ * $Id: search_xml.c 15781 2008-09-02 16:30:18Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: search_xml.c 14457 2007-08-14 04:23:12Z cbiere $")
+RCSID("$Id: search_xml.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -49,6 +49,7 @@ RCSID("$Id: search_xml.c 14457 2007-08-14 04:23:12Z cbiere $")
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/ascii.h"
 #include "lib/getdate.h"
 #include "lib/glib-missing.h"
 #include "lib/tm.h"
@@ -416,8 +417,7 @@ search_store_xml(void)
      */
     root = xmlNewDocNode(doc, NULL, string_to_xmlChar("Searches"), NULL);
     xmlDocSetRootElement(doc, root);
-	/* Discard the newline of the ctime string */
-    xml_prop_printf(root, "Time", "%24.24s", ctime(&now));
+    xml_prop_printf(root, "Time", "%s", timestamp_to_string(now));
     xml_prop_printf(root, "Version", "%s", GTA_VERSION_NUMBER);
 
     /*
