@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_common.c 14617 2007-08-21 17:46:37Z rmanfredi $
+ * $Id: uploads_common.c 16176 2009-03-16 17:42:36Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_common.c 14617 2007-08-21 17:46:37Z rmanfredi $")
+RCSID("$Id: uploads_common.c 16176 2009-03-16 17:42:36Z rmanfredi $")
 
 #include "uploads.h"			/* For upload_row_data_t */
 #include "uploads_common.h"
@@ -65,7 +65,6 @@ uploads_gui_progress(const gnet_upload_status_t *u,
     case GTA_UL_HEADERS:
     case GTA_UL_EXPECTING:
     case GTA_UL_WAITING:
-    case GTA_UL_PFSP_WAITING:
     case GTA_UL_ABORTED:
 	case GTA_UL_QUEUED:
     case GTA_UL_QUEUE:
@@ -111,14 +110,19 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 		{
 			time_delta_t d = delta_time(data->last_update, data->start_date);
 	        filesize_t requested = data->range_end - data->range_start + 1;
+			size_t rw;
 
-			gm_snprintf(tmpstr, sizeof(tmpstr),
-				"%s (%s) %s%s #%u", _("Completed"),
+			rw = gm_snprintf(tmpstr, sizeof(tmpstr),
+				"%s (%s) %s %s#%u", _("Completed"),
 				d > 0 ? short_rate(requested / d, show_metric_units())
 						: _("< 1s"),
 				d > 0 ? short_time(d) : "",
-				u->parq_quick ? " (quick)" : "",
+				u->parq_quick ? _("(quick) ") : "",
 				u->reqnum);
+
+			if (u->error_count)
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					_(" E=%u"), u->error_count);
 		}
         break;
 
@@ -130,15 +134,20 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 			time_t now = tm_time();
 			gboolean stalled = delta_time(now, data->last_update) > IO_STALLED;
 			gchar pbuf[32];
+			size_t rw;
 
 			gm_snprintf(pbuf, sizeof pbuf, "%5.02f%% ", p * 100.0);
-			gm_snprintf(tmpstr, sizeof tmpstr, _("%s(%s) TR: %s %s #%u"),
+			rw = gm_snprintf(tmpstr, sizeof tmpstr, _("%s(%s) TR: %s %s#%u"),
 				p > 1.0 ? pbuf : "",
 				stalled ? _("stalled")
 					: short_rate(u->bps, show_metric_units()),
 				short_time(tr),
-				u->parq_quick ? " (quick)" : "",
+				u->parq_quick ? _("(quick) ") : "",
 				u->reqnum);
+
+			if (u->error_count)
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					_(" E=%u"), u->error_count);
 		}
 		break;
 
@@ -146,14 +155,19 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
         return _("Waiting for headers...");
 
     case GTA_UL_EXPECTING:
-        return _("Waiting for further request...");
+		if (u->error_count)
+			gm_snprintf(tmpstr, sizeof(tmpstr),
+				_("%s %s#%u E=%u"), _("Waiting for further request..."),
+				u->parq_quick ? _("(quick) ") : "", u->reqnum, u->error_count);
+		else
+			gm_snprintf(tmpstr, sizeof(tmpstr),
+				"%s %s#%u", _("Waiting for further request..."),
+				u->parq_quick ? _("(quick) ") : "", u->reqnum);
+		break;
 
     case GTA_UL_WAITING:
         return _("Reading follow-up request...");
 
-    case GTA_UL_PFSP_WAITING:
-        return _("Unavailable range, waiting retry...");
-
     case GTA_UL_ABORTED:
         return _("Transmission aborted");
 
@@ -166,6 +180,7 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 			gboolean queued;
 			guint available = 0;
 			gchar tbuf[64];
+			size_t rw;
 
 			gnet_prop_get_guint32_val(PROP_MAX_UPLOADS, &max_up);
 			gnet_prop_get_guint32_val(PROP_UL_RUNNING, &cur_up);
@@ -190,7 +205,7 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 				tbuf[0] = '\0';
 			}
 
-			gm_snprintf(tmpstr, sizeof tmpstr,
+			rw = gm_snprintf(tmpstr, sizeof tmpstr,
 						_("%s [%d] (slot %d/%d)%s %s %s"),
 						u->parq_frozen ? _("Frozen") :
 						queued ? _("Queued") : _("Waiting"),
@@ -201,6 +216,9 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 						_("lifetime:"),
 						short_time(u->parq_lifetime));
 
+			if (u->error_count)
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					_(" E=%u"), u->error_count);
 		}
 		break;
 
@@ -251,7 +269,6 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 	case GTA_UL_QUEUED:
 	case GTA_UL_QUEUE:
 	case GTA_UL_QUEUE_WAITING:
-	case GTA_UL_PFSP_WAITING:
 		break;
 	}
 
